local SuitablePreparedFood =
{
    "frogglebunwich",
	"fishsticks",
	"honeynuggets",
	"honeyham",
	"kabobs",
	"baconeggs",
	"meatballs",
	"bonestew",
	"perogies",
	"turkeydinner",
	"fishtacos",
	"unagi",
	"hotchili",
	"guacamole",
	"californiaroll",
	"seafoodgumbo",
	"bisque",
	"ceviche",
	"jellyopop",
	"lobsterbisque",
	"lobsterdinner",
	"sharkfinsoup",
	"surfnturf",
	"tropicalbouillabaisse",
	"caviar",
	"snakebonesoup",
	"feijoada",
	"steamedhamsandwich",
	"gummy_cake",
}

local GoldValueForPreparedFood =
{
    frogglebunwich = 1,
	fishsticks = 1,
	honeynuggets = 2,
	honeyham = 3,
	kabobs = 1,
	baconeggs = 4,
	meatballs = 1,
	bonestew = 3,
	perogies = 1,
	turkeydinner = 3,
	fishtacos = 1,
	unagi = 12,
	hotchili = 1,
	guacamole = 2,
	californiaroll = 3,
	seafoodgumbo = 4,
	bisque = 4,
	ceviche = 4,
	jellyopop = 3,
	lobsterbisque = 7,
	lobsterdinner = 10,
	sharkfinsoup = 1,
	surfnturf = 6,
	tropicalbouillabaisse = 6,
	caviar = 6,
	snakebonesoup = 5,
	feijoada = 5,
	steamedhamsandwich = 5,
	gummy_cake = 1,
}

--[[local DubloonValueForPreparedFood =
{

}]]

local function ChangeOrAddTradableGoldValue(inst)
    if inst.components.tradable == nil then
	    inst:AddComponent("tradable")
	end
	inst.components.tradable.goldvalue = GoldValueForPreparedFood[inst.prefab] or 0
	--inst.components.tradable.dubloonvalue = DubloonValueForPreparedFood[inst.prefab] or 0
end

for _,PreparedFood in pairs(SuitablePreparedFood) do
    AddPrefabPostInit(PreparedFood, ChangeOrAddTradableGoldValue)
end

local function IsMangroveAtPoint(x, y, z)
    return GetGroundTypeAtPosition(Vector3(x, y, z)) == GROUND.MANGROVE or 
		GetGroundTypeAtPosition(Vector3(x, y, z)) == GROUND.MANGROVE_SHORE
end

local function IsImpassablePoint(x, y, z)   
    return GetGroundTypeAtPosition(Vector3(x, y, z)) == GROUND.IMPASSABLE
end

local function IsEdgeAtPoint(x, y, z, radius)
    local radius = radius or 1

    if GetWorld().Map:GetTileAtPoint(x, y, z) ~= GROUND.IMPASSABLE then
        return false
    end
	
    for i = -radius, radius, 1 do
        if GetWorld().Map:GetTileAtPoint(x - radius, y, z + i) ~= GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(x + radius, y, z + i) ~= GROUND.IMPASSABLE then
            return true
        end
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if GetWorld().Map:GetTileAtPoint(x + i, y, z - radius) ~= GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(x + i, y, z + radius) ~= GROUND.IMPASSABLE then
            return true
        end
    end
	
    return false

end

AddPrefabPostInit("dug_grass", function(inst)
    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    if inst.components.deployable ~= nil then
		    local _ondeploy = inst.components.deployable.ondeploy
		    inst.components.deployable.ondeploy = function(inst, pt, deployer)
			    if IsMangroveAtPoint(pt.x, pt.y, pt.z) then
                    local grass_water = SpawnPrefab("grass_water")
				    grass_water.Transform:SetPosition(pt.x, pt.y, pt.z)
					inst.components.stackable:Get():Remove()
			        if deployer ~= nil and deployer.SoundEmitter ~= nil then
				        deployer.SoundEmitter:PlaySound("dontstarve/common/plant")
		        	end
					if grass_water.components.pickable ~= nil then
					    grass_water.components.pickable:MakeEmpty()
					end
				else
				    if _ondeploy then
					    _ondeploy(inst, pt, deployer)
					end
				end
			end
		end	
	end
end)

AddPrefabPostInit("grass_water", function(inst)
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.DIG)
	inst.components.workable:SetOnFinishCallback(function(inst, worker) 
		if inst.components.pickable ~= nil and inst.components.pickable:CanBePicked() then
			inst.components.lootdropper:SpawnLootPrefab("cutgrass")
		end
		if inst.components.pickable ~= nil and not inst.components.pickable.withered then
			inst.components.lootdropper:SpawnLootPrefab("dug_grass")
		else
			inst.components.lootdropper:SpawnLootPrefab("cutgrass")
		end
		inst:Remove()
	end)
	inst.components.workable:SetWorkLeft(1)	
	MakeMediumBurnable(inst)
	MakeSmallPropagator(inst)
end)

AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.DIG, function(inst) 
    if inst.prefab == "wilba" and inst.were then
        return "dig"
    elseif not inst.sg:HasStateTag("predig") then
        if inst.sg:HasStateTag("digging") then
            return "dig"
        else
            return "dig_start"
        end
    end    
end))

local function DoMountSound(inst, mount, sound)
    if mount ~= nil and mount.sounds ~= nil then
        inst.SoundEmitter:PlaySound(mount.sounds[sound])
    end
end

AddStategraphState("wilson", 
    State{
        name = "dojostleaction",
        tags = { "doing", "busy" },

        onenter = function(inst)
		    --Note: vanilla DS does not have "atk_pre" anim unless riding
            local buffaction = inst:GetBufferedAction()
            local target = buffaction ~= nil and buffaction.target or nil
            local equip = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
            inst.components.locomotor:Stop() 
			if buffaction ~= nil 
		    	and buffaction.action ~= nil 
			    and (buffaction.action == ACTIONS.SHADOWEMBERCAST or buffaction.action == ACTIONS.SHADOWFUELCAST) 
				and buffaction.pos ~= nil 
				and buffaction.pos.x ~= nil 
				and buffaction.pos.y ~= nil 
				and buffaction.pos.z ~= nil 
			then
			    inst:ForceFacePoint(buffaction.pos.x, buffaction.pos.y, buffaction.pos.z)
			end 
			
			if buffaction ~= nil 
		    	and buffaction.action ~= nil 
			    and buffaction.action == ACTIONS.SHADOW_TOSS
			then 
			    local toss_pos = nil 
			    if target ~= nil then 
				    toss_pos = target:GetPosition() 
				elseif buffaction.pos ~= nil then 
				    toss_pos = buffaction.pos
				end 
				
				if toss_pos ~= nil then 
				    inst:ForceFacePoint(toss_pos.x, toss_pos.y, toss_pos.z)
				end
			
			end
			
            local cooldown
            if inst.components.rider:IsRiding() then
                inst.AnimState:PlayAnimation("atk_pre")
                inst.AnimState:PushAnimation("atk", false)
                DoMountSound(inst, inst.components.rider:GetMount(), "angry")
                cooldown = 16 * FRAMES
            elseif equip ~= nil and equip:HasTag("whip") then
                inst.AnimState:PlayAnimation("whip_pre")
                inst.AnimState:PushAnimation("whip", false)
                inst.sg.statemem.iswhip = true
                inst.SoundEmitter:PlaySound("dontstarve/common/whip_large")
                cooldown = 17 * FRAMES
			elseif equip ~= nil and equip:HasTag("pocketwatch") then
				inst.AnimState:PlayAnimation("pocketwatch_atk_pre" )
				inst.AnimState:PushAnimation("pocketwatch_atk", false)
				inst.sg.statemem.ispocketwatch = true
				cooldown = 19 * FRAMES
                if equip:HasTag("shadow_item") then
                    inst.SoundEmitter:PlaySound("wanda2/characters/wanda/watch/weapon/pre_shadow")
					inst.AnimState:Show("pocketwatch_weapon_fx")
					inst.sg.statemem.ispocketwatch_fueled = true
                else
                    inst.SoundEmitter:PlaySound("wanda2/characters/wanda/watch/weapon/pre")
					inst.AnimState:Hide("pocketwatch_weapon_fx")
                end
            elseif equip ~= nil and equip:HasTag("jab") then
                inst.AnimState:PlayAnimation("spearjab_pre")
                inst.AnimState:PushAnimation("spearjab", false)
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_whoosh")
                cooldown = 21 * FRAMES
            elseif equip ~= nil and equip.components.weapon ~= nil and not equip:HasTag("punch") then
                inst.AnimState:PlayAnimation("atk")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
                cooldown = 13 * FRAMES
            elseif equip ~= nil and (equip:HasTag("light") or equip:HasTag("nopunch")) then
                inst.AnimState:PlayAnimation("atk")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
                cooldown = 13 * FRAMES
            elseif inst:HasTag("beaver") then
                inst.sg.statemem.isbeaver = true
                inst.AnimState:PlayAnimation("atk")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_whoosh")
                cooldown = 13 * FRAMES
            else
                inst.AnimState:PlayAnimation("punch")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_whoosh")
                cooldown = 24 * FRAMES
            end

            if target ~= nil and target:IsValid() then
                inst:FacePoint(target:GetPosition())
            end

            inst.sg.statemem.action = buffaction
            inst.sg:SetTimeout(cooldown)
        end,

        timeline =
        {
			--beaver: frame 6 action
			--whip: frame 10 action
			--other: frame 8 action
            TimeEvent(6 * FRAMES, function(inst)
                if inst.sg.statemem.isbeaver then
					inst.sg:RemoveStateTag("busy")
                    inst:PerformBufferedAction()
                end
            end),
            TimeEvent(8 * FRAMES, function(inst)
				if not (inst.sg.statemem.iswhip or
						inst.sg.statemem.ispocketwatch or
						inst.sg.statemem.isbeaver) then
					inst.sg:RemoveStateTag("busy")
                    inst:PerformBufferedAction()
                end
            end),
            TimeEvent(10 * FRAMES, function(inst)
                if inst.sg.statemem.iswhip or inst.sg.statemem.ispocketwatch then
					inst.sg:RemoveStateTag("busy")
                    inst:PerformBufferedAction()
                end
            end),
            TimeEvent(17*FRAMES, function(inst)
				if inst.sg.statemem.ispocketwatch then
                    inst.SoundEmitter:PlaySound(inst.sg.statemem.ispocketwatch_fueled and "wanda2/characters/wanda/watch/weapon/pst_shadow" or "wanda2/characters/wanda/watch/weapon/pst")
                end
            end),
        },

        ontimeout = function(inst)
            --anim pst should still be playing
            inst.sg:GoToState("idle", true)
        end,

        events =
        {
            EventHandler("equip", function(inst) inst.sg:GoToState("idle") end),
            EventHandler("unequip", function(inst) inst.sg:GoToState("idle") end),
        },

        onexit = function(inst)
            if inst.bufferedaction == inst.sg.statemem.action and
            (inst.components.playercontroller == nil or inst.components.playercontroller.lastheldaction ~= inst.bufferedaction) then
                inst:ClearBufferedAction()
            end
        end,
    }
) 

AddStategraphState("wilsonboating", 
    State{
        name = "dojostleaction",
        tags = { "doing", "busy" },
        onenter = function(inst)
            local buffaction = inst:GetBufferedAction()
            local target = buffaction ~= nil and buffaction.target or nil
            local equip = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
            inst.components.locomotor:Stop() 
 			if buffaction ~= nil 
		    	and buffaction.action ~= nil 
			    and (buffaction.action == ACTIONS.SHADOWEMBERCAST or buffaction.action == ACTIONS.SHADOWFUELCAST) 
				and buffaction.pos ~= nil 
				and buffaction.pos.x ~= nil 
				and buffaction.pos.y ~= nil 
				and buffaction.pos.z ~= nil 
			then
			    inst:ForceFacePoint(buffaction.pos.x, buffaction.pos.y, buffaction.pos.z)
			end
            local cooldown
            if equip ~= nil and equip:HasTag("whip") then
                inst.AnimState:PlayAnimation("whip_pre")
                inst.AnimState:PushAnimation("whip", false)
                inst.sg.statemem.iswhip = true
                inst.SoundEmitter:PlaySound("dontstarve/common/whip_large")
                cooldown = 17 * FRAMES
			elseif equip ~= nil and equip:HasTag("pocketwatch") then
				inst.AnimState:PlayAnimation("pocketwatch_atk_pre" )
				inst.AnimState:PushAnimation("pocketwatch_atk", false)
				inst.sg.statemem.ispocketwatch = true
				cooldown = 19 * FRAMES
                if equip:HasTag("shadow_item") then
                    inst.SoundEmitter:PlaySound("wanda2/characters/wanda/watch/weapon/pre_shadow")
					inst.AnimState:Show("pocketwatch_weapon_fx")
					inst.sg.statemem.ispocketwatch_fueled = true
                else
                    inst.SoundEmitter:PlaySound("wanda2/characters/wanda/watch/weapon/pre")
					inst.AnimState:Hide("pocketwatch_weapon_fx")
                end
            elseif equip ~= nil and equip:HasTag("jab") then
                inst.AnimState:PlayAnimation("spearjab_pre")
                inst.AnimState:PushAnimation("spearjab", false)
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_whoosh")
                cooldown = 21 * FRAMES
            elseif equip ~= nil and equip.components.weapon ~= nil and not equip:HasTag("punch") then
                inst.AnimState:PlayAnimation("atk")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
                cooldown = 13 * FRAMES
            elseif equip ~= nil and (equip:HasTag("light") or equip:HasTag("nopunch")) then
                inst.AnimState:PlayAnimation("atk")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
                cooldown = 13 * FRAMES
            elseif inst:HasTag("beaver") then
                inst.sg.statemem.isbeaver = true
                inst.AnimState:PlayAnimation("atk")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_whoosh")
                cooldown = 13 * FRAMES
            else
                inst.AnimState:PlayAnimation("punch")
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_whoosh")
                cooldown = 24 * FRAMES
            end

            if target ~= nil and target:IsValid() then
                inst:FacePoint(target:GetPosition())
            end

            inst.sg.statemem.action = buffaction
            inst.sg:SetTimeout(cooldown)
        end,

        timeline =
        {
			--beaver: frame 6 action
			--whip: frame 10 action
			--other: frame 8 action
            TimeEvent(6 * FRAMES, function(inst)
                if inst.sg.statemem.isbeaver then
					inst.sg:RemoveStateTag("busy")
                    inst:PerformBufferedAction()
                end
            end),
            TimeEvent(8 * FRAMES, function(inst)
				if not (inst.sg.statemem.iswhip or
						inst.sg.statemem.ispocketwatch or
						inst.sg.statemem.isbeaver) then
					inst.sg:RemoveStateTag("busy")
                    inst:PerformBufferedAction()
                end
            end),
            TimeEvent(10 * FRAMES, function(inst)
                if inst.sg.statemem.iswhip or inst.sg.statemem.ispocketwatch then
					inst.sg:RemoveStateTag("busy")
                    inst:PerformBufferedAction()
                end
            end),
            TimeEvent(17*FRAMES, function(inst)
				if inst.sg.statemem.ispocketwatch then
                    inst.SoundEmitter:PlaySound(inst.sg.statemem.ispocketwatch_fueled and "wanda2/characters/wanda/watch/weapon/pst_shadow" or "wanda2/characters/wanda/watch/weapon/pst")
                end
            end),
        },

        ontimeout = function(inst)
            --anim pst should still be playing
            inst.sg:GoToState("idle", true)
        end,

        events =
        {
            EventHandler("equip", function(inst) inst.sg:GoToState("idle") end),
            EventHandler("unequip", function(inst) inst.sg:GoToState("idle") end),
        },

        onexit = function(inst)
            if inst.bufferedaction == inst.sg.statemem.action and
            (inst.components.playercontroller == nil or inst.components.playercontroller.lastheldaction ~= inst.bufferedaction) then
                inst:ClearBufferedAction()
            end
        end,
    }
)



AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.PICK, function(inst, action) 
    if action.target.components.pickable then
	    if action.target.components.pickable.jostlepick then
	        return "dojostleaction"  
        elseif action.target.components.pickable.quickpick then
            return "doshortaction"
        else
            return "dolongaction"
        end
    end
end))

if IsDLCEnabled(PORKLAND_DLC) then
    AddStategraphPostInit("wilson", function(sg)
        Hooks.sg.handler.Action(sg, "FERTILIZE", function(inst, action, _fn)
            if inst:HasTag("healonfertilize") 
			    and action.invobject 
				and action.invobject:HasTag("heal_fertilize") 
				and not action.target then
                return "fertilize"
            end
            return _fn(inst, action)
        end)	
	end)
    AddStategraphPostInit("wilsonboating", function(sg)
        if not KnownModIndex:IsModEnabled("workshop-2767977231") then
            --[[for i, state in ipairs({"fertilize", "fertilize_short"}) do
                sg.states[state] = deepcopy(SGwilson.states[state])
                AddTagToState(sg, state, "boating")
            end]]
            --Hooks.sg.state.ToggleBoat(sg, "fertilize")
            --Hooks.sg.state.ToggleBoat(sg, "fertilize_short")
			--tonumber(GetWorld().meta.build_version) >= 540233
			Hooks.sg.handler.Action(sg, "FERTILIZE", function(inst, action, _fn)
                if inst:HasTag("healonfertilize") 
			        and action.invobject 
				    and not action.target then
                    return action.invobject:HasTag("heal_fertilize") and "fertilize" or "fertilize_short"
                end
                return _fn(inst, action)
            end)
		else
		    Hooks.sg.handler.Action(sg, "FERTILIZE", function(inst, action, _fn)
                if inst:HasTag("healonfertilize") 
			        and action.invobject 
				    and not action.target then
                    return action.invobject:HasTag("heal_fertilize") and "fertilize" or "fertilize_short"
				else
				    return "doshortaction"
                end
            end)	
        end
	end)	
end

local action_dig_start = State{ name = "dig_start",
        tags = {"predig", "working"},
        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("shovel_pre")
			local vehicle = inst.components.driver.vehicle
            if vehicle then vehicle:Show() end
        end,
        
        events=
        {
            EventHandler("unequip", function(inst) 
			    inst.sg:GoToState("idle") 
				local vehicle = inst.components.driver.vehicle
                if vehicle then vehicle:Hide() end					
			end ),
            EventHandler("animover", function(inst) inst.sg:GoToState("dig") end),
        },
    }

local action_dig = State{
        name = "dig",
        tags = {"predig", "digging", "working"},
        onenter = function(inst)
            if inst.prefab == "wilba" and inst.were then
                inst.SoundEmitter:PlaySound("dontstarve_DLC003/characters/werewilba/bark")
                if inst.atk_2 == true then
                    inst.AnimState:PlayAnimation("atk_2_werewilba")
                    inst.atk_2 = nil
                else
                    inst.AnimState:PlayAnimation("atk_werewilba")
                    inst.atk_2 = true
                end
                inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_whoosh",nil,.5)
            else
                inst.AnimState:PlayAnimation("shovel_loop")
            end

			inst.sg.statemem.action = inst:GetBufferedAction()
			
			local vehicle = inst.components.driver.vehicle
            if vehicle then vehicle:Show() end
        end,

        timeline=
        {
            TimeEvent(15*FRAMES, function(inst) 
--[[                if inst.sg.statemem.action and inst.sg.statemem.action.target then
					local fx = SpawnPrefab("shovel_dirt")
					fx.Transform:SetPosition( inst.sg.statemem.action.target.Transform:GetWorldPosition() )
				end
--]]                
                inst:PerformBufferedAction() 
                inst.sg:RemoveStateTag("predig") 
                inst.SoundEmitter:PlaySound("dontstarve/wilson/dig")
                
            end),
            
            TimeEvent(35*FRAMES, function(inst)
				if (TheInput:IsMouseDown(MOUSEBUTTON_RIGHT) or
				   TheInput:IsControlPressed(CONTROL_ACTION)  or TheInput:IsControlPressed(CONTROL_CONTROLLER_ACTION)) and 
					inst.sg.statemem.action and 
					inst.sg.statemem.action.target and 
					inst.sg.statemem.action.target:IsActionValid(inst.sg.statemem.action.action, true) and
					inst.sg.statemem.action.target.components.workable then
						inst:ClearBufferedAction()
						inst:PushBufferedAction(inst.sg.statemem.action)
				end
            end),
            
        },
        
        events=
        {
            EventHandler("unequip", function(inst) 
			    inst.sg:GoToState("idle") 
				local vehicle = inst.components.driver.vehicle
                if vehicle then vehicle:Hide() end					
			end ),
            EventHandler("animover", function(inst)
                if inst.prefab ~= "wilba" or not inst.were then
                    inst.AnimState:PlayAnimation("shovel_pst") 
                end

                inst.sg:GoToState("idle", true)
				inst:DoTaskInTime(12*FRAMES, function()
				    local vehicle = inst.components.driver.vehicle
                    if vehicle then vehicle:Hide() end
			    end)
            end ),
            
        },        
    }

AddStategraphState("wilsonboating", action_dig_start)
AddStategraphState("wilsonboating", action_dig)

--We can just add it in sg
--AddStategraphPostInit("wilsonboating", function(sg)
    --for i, state in ipairs({"dig_start", "dig"}) do
    --    AddTagToState(sg, state, "boating") 
    --end	
	--Hooks.sg.state.ToggleBoat(sg, "dig_start")  
	--Hooks.sg.state.ToggleBoat(sg, "dig")
--end)

if IsDLCEnabled(PORKLAND_DLC) then
    local creditcard_recipe = Recipe("creditcard", {Ingredient("oinc", 100)}, RECIPETABS.CITY, TECH.LOST)
	
    local bugzapper_recipe = Recipe("bug_zapper", {Ingredient("feather_thunder", 1), Ingredient("transistor", 2), Ingredient("nitre", 2)}, RECIPETABS.SURVIVAL, TECH.LOST)

end

if IsDLCEnabled(PORKLAND_DLC) then
    local function GetUpvalue(fn, name)
        local i = 1
        while debug.getupvalue(fn, i) and debug.getupvalue(fn, i) ~= name do
            i = i + 1
        end
        local name, value = debug.getupvalue(fn, i)
        return value, i
    end

    local ShopInterior = require("components/shopinterior")

    local SHOP_TYPES = GetUpvalue(ShopInterior.MakeShop, "SHOPTYPES")
	
    if TUNING.ADDITIONAL_PLANTS == true then
        table.insert(SHOP_TYPES["pig_shop_florist"], {"dug_bananabush", "oinc", 2})
	    table.insert(SHOP_TYPES["pig_shop_florist"], {"dug_monkeytail", "oinc", 2})
	    table.insert(SHOP_TYPES["pig_shop_florist"], {"dug_berrybush_juicy", "oinc", 2})
	end
	if TUNING.NEW_YEAR_ITEMS == true then
        table.insert(SHOP_TYPES["pig_shop_general_fiesta"], {"redlantern", "oinc", 10})
	    table.insert(SHOP_TYPES["pig_shop_general_fiesta"], {"miniboatlantern", "oinc", 10})	
	end
	table.insert(SHOP_TYPES["pig_shop_antiquities"], {"tentaclespots", "oinc", 3})
	table.insert(SHOP_TYPES["pig_shop_antiquities"], {"sand", "oinc", 1})
	table.insert(SHOP_TYPES["pig_shop_antiquities"], {"beefalowool", "oinc", 1})
	
	table.insert(SHOP_TYPES["pig_shop_tinker"], {"creditcard_blueprint", "oinc", 500})
	
end

AddPrefabPostInit("thunderbird", function(inst)
    if inst.components.lootdropper ~= nil then
	    inst.components.lootdropper:AddChanceLoot("bug_zapper_blueprint", 0.1)
	end
end)

AddPlayerPostInit(function(inst) 
    if TUNING.ADDITIONAL_PLANTS == true then
	    local dug_bananabush_recipe = Recipe("dug_bananabush", {Ingredient("cave_banana", 2), Ingredient("poop", 2)}, RECIPETABS.REFINE, TECH.SCIENCE_TWO)

	    local dug_monkeytail_recipe = Recipe("dug_monkeytail", {Ingredient("cutreeds", 2), Ingredient("poop", 2)}, RECIPETABS.REFINE, TECH.SCIENCE_TWO)

	    local dug_berrybush_juicy_recipe = Recipe("dug_berrybush_juicy", {Ingredient("berries", 4), Ingredient("poop", 2)}, RECIPETABS.REFINE, TECH.SCIENCE_TWO)		
		
	end
	
end)

AddIngredientValues({"berries_juicy"}, {fruit=0.5}, true)

AddComponentPostInit("playeractionpicker", function(self)
    local _GetRightClickActions = self.GetRightClickActions
    function self:GetRightClickActions(target_ent, position, leftaction)
	    local actions = nil
        local useitem = self.inst.components.inventory:GetActiveItem()
		local equipitem = self.inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
		if useitem and useitem:IsValid() and useitem:HasTag("allow_action_on_impassable") then
            if target_ent == self.inst then
                actions = self:GetInventoryActions(useitem, true)
            end
            if not actions then
                if target_ent then                
                    actions = self:GetUseItemActions(target_ent, useitem, true)
                end
                if not actions and position then
                    actions = self:GetPointActions(position, useitem, true)
                end
            end		
		end
		if not target_ent and position and equipitem and equipitem:IsValid() and equipitem:HasTag("allow_action_on_impassable") then
            if not actions then
                actions = self:GetPointActions(position, equipitem, true)
            end		
		end
		if not actions then
		    actions = _GetRightClickActions(self, target_ent, position, leftaction)
		end
	    return actions or {}
	end
end)

local function AllowActionOnEdge(inst)
    inst:AddTag("allow_action_on_impassable")
	if inst.components.deployable ~= nil then
	    local _oldtest = inst.components.deployable.test
		inst.components.deployable.test = function(inst, pt)
		    if IsEdgeAtPoint(pt.x, pt.y, pt.z, 1) then
			    return true  --Should there check blocker too?
			else
				if _oldtest then
				    return _oldtest(inst, pt)
			    end				
			end
		end
	end
end

AddPrefabPostInit("fence_item", AllowActionOnEdge)
AddPrefabPostInit("fence_gate_item", AllowActionOnEdge)
AddPrefabPostInit("wall_hay_item", AllowActionOnEdge)
AddPrefabPostInit("wall_wood_item", AllowActionOnEdge)
AddPrefabPostInit("wall_stone_item", AllowActionOnEdge)
AddPrefabPostInit("wall_ruins_item", AllowActionOnEdge)
AddPrefabPostInit("wall_limestone_item", AllowActionOnEdge)
AddPrefabPostInit("hedge_block_item", AllowActionOnEdge)
AddPrefabPostInit("hedge_cone_item", AllowActionOnEdge)
AddPrefabPostInit("hedge_layered_item", AllowActionOnEdge)

if not IsDLCEnabled(PORKLAND_DLC) and not IsDLCEnabled(CAPY_DLC) then
    ACTIONS.RUMMAGE.priority = 1
	ACTIONS.PICKUP.priority = 2
end

if IsDLCEnabled(PORKLAND_DLC) then
    ACTIONS.FERTILIZE.priority = 1
	local FERTILIZE_fn = ACTIONS.FERTILIZE.fn
	ACTIONS.FERTILIZE.fn = function(act)
	    if act.target == nil and act.doer:HasTag("healonfertilize") and act.invobject ~= nil then
		    if act.doer.components.health ~= nil then
		        act.doer.components.health:DoDelta(act.invobject.components.fertilizer.planthealingvalue or 2, false, "fertilize")
				if act.invobject.prefab == "spoiled_food" or act.invobject.prefab == "glommerfuel" then
				    act.doer.components.health:RegenFromFood(1, 1, 4, "health")
				elseif act.invobject.prefab == "rottenegg" then
				    act.doer.components.health:RegenFromFood(1, 1, 6, "health")
				elseif act.invobject.prefab == "compostwrap" or act.invobject.prefab == "fertilizer_fig" then
				    act.doer.components.health:RegenFromFood(1, 1, 32, "health")
				end
		        if act.invobject.components.stackable ~= nil and act.invobject.components.stackable.stacksize > 1 then
		            act.invobject.components.stackable:Get():Remove()
		        else
		    	    if act.invobject.components.finiteuses ~= nil then
		    		    act.invobject.components.finiteuses:Use(2)
		    	    else
		        	    act.invobject:Remove()
		    	    end
		        end	    
		        return true
		    end
        end			
		return FERTILIZE_fn(act)   
    end	
end

local function ForPlantingHealing(inst)
    if IsDLCEnabled(PORKLAND_DLC) and inst.components.fertilizer ~= nil then
        inst.components.fertilizer.planthealing = true
		inst.components.fertilizer.planthealingvalue = 0
    end
end

local function ForPlantingHealingTwo(inst)
    if IsDLCEnabled(PORKLAND_DLC) and inst.components.fertilizer ~= nil then
        inst.components.fertilizer.planthealing = true
		inst.components.fertilizer.planthealingvalue = 2
    end
end

local function ForPlantingHealingThree(inst)
    if IsDLCEnabled(PORKLAND_DLC) and inst.components.fertilizer ~= nil then
	    inst.components.fertilizer.planthealing = true
		inst.components.fertilizer.planthealingvalue = 3
    end
end

AddPrefabPostInit("spoiled_food", ForPlantingHealing)
AddPrefabPostInit("rottenegg", ForPlantingHealing)
AddPrefabPostInit("guano", ForPlantingHealingThree)
AddPrefabPostInit("fertilizer", ForPlantingHealingThree)
AddPrefabPostInit("fertilizer_fig", ForPlantingHealingTwo)
AddPrefabPostInit("glommerfuel", ForPlantingHealingTwo)

AddPrefabPostInit("compostwrap", function(inst)
    inst:RemoveComponent("healer")
	
	inst:AddComponent("fertilizer")
    inst.components.fertilizer.fertilizervalue = TUNING.POOP_FERTILIZE*6
    inst.components.fertilizer.soil_cycles = TUNING.POOP_SOILCYCLES*2
    inst.components.fertilizer.withered_cycles = TUNING.POOP_WITHEREDCYCLES*2
    inst.components.fertilizer.planthealing = true
	inst.components.fertilizer.planthealingvalue = 8
	
    inst:AddComponent("fuel")
    inst.components.fuel.fuelvalue = TUNING.LARGE_FUEL
    inst.components.fuel:SetOnTakenFn(function(inst, taker)
	    SpawnPrefab("poopcloud").Transform:SetPosition(taker.Transform:GetWorldPosition())
	end)

    inst:AddComponent("smotherer")	
	
	MakeSmallBurnable(inst, TUNING.MED_BURNTIME)
    inst.components.burnable:SetOnIgniteFn(function(inst)
	    DefaultBurnFn(inst)
        if inst.flies ~= nil then
            inst.flies:Remove()
            inst.flies = nil
        end
	end)
    MakeSmallPropagator(inst)	
	
	inst.flies = inst:SpawnChild("flies")
	
    inst:ListenForEvent("enterlimbo", function(inst)
        if inst.flies ~= nil then
            inst.flies:Remove()
            inst.flies = nil
        end	
	end)
	
    inst:ListenForEvent("exitlimbo", function(inst)
        if inst.flies == nil then
            inst.flies = inst:SpawnChild("flies")
        end	
	end)

    inst.components.inventoryitem:SetOnDroppedFn(function(inst)
        if inst.flies == nil then
            inst.flies = inst:SpawnChild("flies")
        end		
	end)
	
    inst.components.inventoryitem:SetOnPickupFn(function(inst)
        if inst.flies ~= nil then
            inst.flies:Remove()
            inst.flies = nil
        end		
	end)
	
    inst.components.inventoryitem:SetOnPutInInventoryFn(function(inst)
        if inst.flies ~= nil then
            inst.flies:Remove()
            inst.flies = nil
        end		
	end)	
end)

--AddPrefabPostInit("armor_bramble", function(inst)
 --   inst:AddComponent("fuel")
--    inst.components.fuel.fuelvalue = TUNING.LARGE_FUEL

  --  MakeSmallBurnable(inst, TUNING.SMALL_BURNTIME)
  --  MakeSmallPropagator(inst)
--[[	
    if TUNING.BUFF_BRAMBLE_ARMOR == true then	
	    if inst.components.armor ~= nil then
	        inst.components.armor:InitCondition(750, 0.8)
		end
	end
]]	
--end)

local STORE_fn = ACTIONS.STORE.fn
ACTIONS.STORE.fn = function(act)
    if act.target:HasTag("magician_hat") then
        if act.target.components.container and act.invobject.components.inventoryitem and act.doer.components.inventory then
            if not act.target.components.container:CanTakeItemInSlot(act.invobject) then
			    return false, "NOTALLOWED"
            end
		    local item = act.invobject.components.inventoryitem:RemoveFromOwner(act.target.components.container.acceptsstacks)
            if item then
				act.target.components.container:Open(act.doer)	
                if not act.target.components.container:GiveItem(item,nil,nil,false) then
                    if TheInput:ControllerAttached() then
				        act.doer.components.inventory:GiveItem(item)
                    else
                        act.doer.components.inventory:GiveActiveItem(item)
                    end
				    return false
                end
			    return true  	
		    end
		end
    end
    return STORE_fn(act)
end

if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
    AddPlayerPostInit(function(inst)
	    GetAllRecipes(true)
	    local pumpkin_lantern_recipe = GetRecipe("pumpkin_lantern")
	    if pumpkin_lantern_recipe then
	        pumpkin_lantern_recipe.level = TECH.SCIENCE_TWO
	    else
	        Recipe("pumpkin_lantern", {Ingredient("pumpkin", 1), Ingredient("fireflies", 1)}, RECIPETABS.LIGHT, TECH.SCIENCE_TWO, RECIPE_GAME_TYPE.COMMON)
	    end
	    local armorgrass_recipe = GetRecipe("armorgrass")
	    if armorgrass_recipe then
	        armorgrass_recipe.level = TECH.NONE
	    else
			Recipe("armorgrass", {Ingredient("cutgrass", 10), Ingredient("twigs", 2)}, RECIPETABS.WAR, TECH.NONE, RECIPE_GAME_TYPE.COMMON)
	    end
		
	end)
end

ACTIONS.GIVE.priority = 2

if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
    AddStategraphPostInit("wilsonboating", function(sg)
        Hooks.sg.state.onenter.Post(sg, "row_start", function(inst, arg)
		    if inst.prefab == "charlie" then
			    --Code from Island Adventures--
                if not inst:HasTag("mime") then 
	                inst.AnimState:OverrideSymbol("paddle", "swap_paddle", "paddle") 
		        end
                inst.AnimState:OverrideSymbol("wake_paddle", "swap_paddle", "wake_paddle")
				--Code from Island Adventures--
            end					
        end)
        Hooks.sg.handler.Event(sg, "sailunequipped", function(inst, data, _old)
		    if inst.prefab == "charlie" then
			    --Code from Island Adventures--
                if not inst:HasTag("mime") then 
	                inst.AnimState:OverrideSymbol("paddle", "swap_paddle", "paddle") 
		        end
                inst.AnimState:OverrideSymbol("wake_paddle", "swap_paddle", "wake_paddle")
				--Code from Island Adventures--
            end
            _old(inst, data)			
        end)		
    end)
end

AddPrefabPostInit("torch", function(inst)
    if not (GetPlayer() and GetPlayer():HasTag("charlie")) then
        return
    end
	
    inst:AddComponent("trader")
    inst.components.trader:SetAcceptTest(function(inst, item) 
	    return item:HasTag("darkitem")
	end)
	inst.components.trader.onaccept = function(inst, giver, item)
	    if item:HasTag("darkitem") then
		    local replacement = SpawnPrefab("torch_shadow")
		    if replacement ~= nil then
			    if inst:IsInLimbo() 
				    and replacement.components.inventoryitem ~= nil 
					and giver ~= nil
					and giver.components.inventory ~= nil then
					giver.components.inventory:GiveItem(replacement, nil, Vector3(TheSim:GetScreenPos(giver.Transform:GetWorldPosition())))
					if inst.components.inventoryitem ~= nil then
					    local owner = inst.components.inventoryitem.owner or nil
						if owner ~= nil then
						    if owner.components.inventory ~= nil then
					            owner.components.inventory:RemoveItem(inst)
							elseif owner.components.container ~= nil then
							    owner.components.container:RemoveItem(inst)
							else
							    inst:Remove()
							end
						else
						    inst:Remove()
					    end
					else
					    inst:Remove()
					end					
				else
	                SpawnPrefab("statue_transition").Transform:SetPosition(inst.Transform:GetWorldPosition())
                    SpawnPrefab("statue_transition_2").Transform:SetPosition(inst.Transform:GetWorldPosition())
				    replacement.Transform:SetPosition(inst.Transform:GetWorldPosition())
					if inst.GetIsOnWater ~= nil and inst:GetIsOnWater() then
				        if replacement.components.floatable ~= nil then
						    replacement.components.floatable:OnHitWater(true)
						elseif replacement.components.inventoryitem ~= nil then
					        replacement.components.inventoryitem:OnHitWater()
					    end
					end
					inst:Remove()				
				end		
			end
		end
	end
end)

local GIVE_strfn = ACTIONS.GIVE.strfn
ACTIONS.GIVE.strfn = function(act)
    local targ = act.target or act.invobject
    if targ.prefab == "torch" then
		return "TORCH"
	elseif targ:HasTag("creditcard") then
	    return "CREDITCARD"
	end
    return GIVE_strfn(act)
end

AddClassPostConstruct("widgets/itemtile", function(self)
    local UIAnim = require "widgets/uianim"
    if self.item:HasTag("magician_hat") then
        self.magiciantoolframe = self:AddChild(UIAnim())
        self.magiciantoolframe:GetAnimState():SetBank("inventory_fx_shadow")
        self.magiciantoolframe:GetAnimState():SetBuild("inventory_fx_shadow")
        self.magiciantoolframe:SetScale(0.25, 0.25, 0.25)
        self.magiciantoolframe:GetAnimState():PlayAnimation("idle", true)
        self.magiciantoolframe:SetClickable(false)
    end
    if self.item:HasTag("possess_with_dragon") then
        self.dragonfx = self:AddChild(UIAnim())
        self.dragonfx:GetAnimState():SetBank("inventory_fx_dragon")
        self.dragonfx:GetAnimState():SetBuild("inventory_fx_dragon")
        self.dragonfx:GetAnimState():PlayAnimation("idle")
        self.dragonfx:SetClickable(false)
    end	
end)

local function FIX_FIRE_FARM(inst)
    local _onignite_fn = inst.event_listeners and inst.event_listeners["onignite"] and inst.event_listeners["onignite"][inst][1]
    if _onignite_fn then
        inst.event_listeners["onignite"][inst][1] = function(inst)
            if inst.components.spawner and inst.components.spawner:IsOccupied() then
                _onignite_fn(inst)
            end
        end
    end
end

AddPrefabPostInit("pighouse", FIX_FIRE_FARM)
AddPrefabPostInit("wildborehouse", FIX_FIRE_FARM)

local function SmallFoodForCharlie(inst)
    inst:AddTag("smallfood_for_charlie")
end

AddPrefabPostInit("smallmeat", SmallFoodForCharlie)
AddPrefabPostInit("cookedsmallmeat", SmallFoodForCharlie)
AddPrefabPostInit("seeds", SmallFoodForCharlie)
AddPrefabPostInit("seeds_cooked", SmallFoodForCharlie)
AddPrefabPostInit("carrot_seeds", SmallFoodForCharlie)
AddPrefabPostInit("corn_seeds", SmallFoodForCharlie)
AddPrefabPostInit("radish_seeds", SmallFoodForCharlie)
AddPrefabPostInit("asparagus_seeds", SmallFoodForCharlie)
AddPrefabPostInit("aloe_seeds", SmallFoodForCharlie)
AddPrefabPostInit("pumpkin_seeds", SmallFoodForCharlie)
AddPrefabPostInit("eggplant_seeds", SmallFoodForCharlie)
AddPrefabPostInit("durian_seeds", SmallFoodForCharlie)
AddPrefabPostInit("pomegranate_seeds", SmallFoodForCharlie)
AddPrefabPostInit("dragonfruit_seeds", SmallFoodForCharlie)
AddPrefabPostInit("watermelon_seeds", SmallFoodForCharlie)
AddPrefabPostInit("sweet_potato_seeds", SmallFoodForCharlie)
AddPrefabPostInit("berries", SmallFoodForCharlie)
AddPrefabPostInit("berries_cooked", SmallFoodForCharlie)
AddPrefabPostInit("honeynuggets", SmallFoodForCharlie)
AddPrefabPostInit("kabobs", SmallFoodForCharlie)
AddPrefabPostInit("trailmix", SmallFoodForCharlie)
AddPrefabPostInit("californiaroll", SmallFoodForCharlie)
AddPrefabPostInit("jellyopop", SmallFoodForCharlie)
AddPrefabPostInit("bananapop", SmallFoodForCharlie)

--Put in other place:
--AddPrefabPostInit("berries_juicy", SmallFoodForCharlie)
--AddPrefabPostInit("berries_juicy_cooked", SmallFoodForCharlie)

local function Food_Undebuff_Shadow(inst)
    inst:AddTag("food_undebuff_shadow")
end

AddPrefabPostInit("monstermeat", Food_Undebuff_Shadow)
AddPrefabPostInit("cookedmonstermeat", Food_Undebuff_Shadow)
AddPrefabPostInit("red_cap", Food_Undebuff_Shadow)
AddPrefabPostInit("monsterlasagna", Food_Undebuff_Shadow)
AddPrefabPostInit("durian", Food_Undebuff_Shadow)
AddPrefabPostInit("monstermeat_dried", Food_Undebuff_Shadow)
AddPrefabPostInit("blue_cap_cooked", Food_Undebuff_Shadow)
AddPrefabPostInit("powcake", Food_Undebuff_Shadow)
AddPrefabPostInit("taffy", Food_Undebuff_Shadow)
AddPrefabPostInit("green_cap_cooked", Food_Undebuff_Shadow)
AddPrefabPostInit("spoiled_food", Food_Undebuff_Shadow)
AddPrefabPostInit("rottenegg", Food_Undebuff_Shadow)

AddPrefabPostInit("lilypad_placer", function(inst)
    if inst.components.placer ~= nil then
	    inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
	    inst.AnimState:SetLayer(LAYER_WORLD_BACKGROUND)
	    inst.AnimState:SetSortOrder(3)
	    inst.components.placer.placeTestFn = function(inst, pt)
	        local CanPlant = false
	        local puddles = TheSim:FindEntities(pt.x, pt.y, pt.z, 1, {"puddle"}, {"puddle_disappear"})
	        if #puddles > 0 then
		        local pt2 = Vector3(puddles[1].Transform:GetWorldPosition())
		        local blockers = TheSim:FindEntities(pt2.x, pt2.y, pt2.z, 2.5, nil, {"NOCLICK", "placer", "player"})
		        if #blockers <= 0 then
			        CanPlant = true
			        inst.Transform:SetPosition(puddles[1].Transform:GetWorldPosition())			
		        end
	        end
	        return CanPlant	
		end
	end
end)

local action_ten_sec = State 
{
    name = "dotensecaction",
    onenter = function(inst)
        inst.sg:GoToState("dolongaction", 10)
    end,
}

AddStategraphState("wilson", action_ten_sec)

if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
    AddStategraphState("wilsonboating", action_ten_sec)
end

local eyeofinsight_see = 
{
    id = "EYEOFINSIGHT_SEE",
    str = STRINGS.ACTIONS.EYEOFINSIGHT_SEE,
    fn = function(act)  --act.target, act.invobject, act.doer
        if act.doer ~= nil 
		    and act.invobject ~= nil 
		    and act.doer:HasTag("charlie")
			and not act.doer.IsInInsight
		then
            act.doer.IsInInsight = true
			if act.doer.components.transformable ~= nil then
			    act.doer.components.transformable:EnterInsightState(true)
			elseif act.doer.EnterInsightState ~= nil then
			    act.doer:EnterInsightState()
			end
			if act.invobject.SoundEmitter ~= nil then
			    act.invobject.SoundEmitter:PlaySound("dontstarve/HUD/get_gold")    
			end
			act.invobject:Remove()
		    return true
		end
    end,
    distance = 1,
    rmb = true,
    instant = false,
    mount_enabled = true,
    priority = 30,
}

AddAction(eyeofinsight_see)

local draw_oinc = 
{
    id = "DRAW_OINC",
    str = STRINGS.ACTIONS.DRAW_OINC,
    fn = function(act)  --act.target, act.invobject, act.doer
        if act.invobject ~= nil 
			and act.invobject:HasTag("creditcard")
			and act.invobject.oinc_amount_inside ~= nil
			and act.invobject.oinc_amount_inside > 0
		then
            if act.invobject.OnRemoveEntity ~= nil then
			    act.invobject:OnRemoveEntity()
				act.invobject.oinc_amount_inside = 0
			    return true
			end
		else
		    if act.doer ~= nil and act.doer.components.talker ~= nil then
			    if TUNING.LANGUAGE == true then
				    act.doer.components.talker:Say("There is not any oincs in this credit card.")
				elseif TUNING.LANGUAGE == false then
				    act.doer.components.talker:Say("这张信用卡里没有任何呼噜币。")
				end	
			    return true
			end
		end
    end,
    distance = 1,
    rmb = true,
    instant = false,
    mount_enabled = true,
    priority = 30,
}

AddAction(draw_oinc)

local blacken = 
{
    id = "BLACKEN",
    str = STRINGS.ACTIONS.BLACKEN,
    fn = function(act)  --act.target, act.invobject, act.doer
        if act.invobject ~= nil 
			and act.invobject:HasTag("blackener")
			and act.target ~= nil
			and act.target:HasTag("blackenable")
			and act.target.White ~= nil
			and act.target.OnBlacken ~= nil
		then
		    act.target:OnBlacken()
			if act.invobject.components.stackable then
			    act.invobject.components.stackable:Get():Remove()
			elseif act.invobject.components.finiteuses then
			    act.invobject.components.finiteuses:Use()
			else
			    act.invobject:Remove()
			end
		    return true
		end
    end,
    distance = 1,
    rmb = true,
    instant = false,
    mount_enabled = false,
    priority = 30,
}

AddAction(blacken)

local shadow_upgrade = 
{
    id = "SHADOW_UPGRADE",
    str = STRINGS.ACTIONS.SHADOW_UPGRADE,
    fn = function(act)  --act.target, act.invobject, act.doer
        local doer = act.doer
        local invobject = act.invobject
		local target = act.target
        if invobject ~= nil 
			and invobject:HasTag("shadowupgrader")
			and target ~= nil
			and target:HasTag("shadowupgradeable")
			and target.components.magicturnable ~= nil
		then
            return target.components.magicturnable:Upgrade(doer, invobject)
		end
    end,
    distance = 1,
    rmb = true,
    instant = false,
    mount_enabled = true,
    priority = 30,
}

AddAction(shadow_upgrade)

local SCYTHE = 
{
    id = "SCYTHE",
    str = STRINGS.ACTIONS.SCYTHE,
    fn = function(act)  --act.target, act.invobject, act.doer
        if act.invobject ~= nil and act.invobject.DoScythe then
            act.invobject:DoScythe(act.target, act.doer)
            return true
        end
		return false
    end,
    distance = 1.8,
    rmb = true,
    instant = false,
    mount_enabled = false,
    priority = 1,
}

AddAction(SCYTHE)

local SHADOW_FORGE = 
{
    id = "SHADOW_FORGE",
    str = STRINGS.ACTIONS.SHADOW_FORGE,
    fn = function(act)  --act.target, act.invobject, act.doer
        local target = act.target
		local doer = act.doer
		if doer ~= nil 
		    and target ~= nil 
		    and doer:HasTag("charlie")
			and target.FindSafeLocationToMove ~= nil
		then
            if target.SpawnShadowFX ~= nil then
			    target:SpawnShadowFX()
			end
			
			if not TheSim:FindFirstEntityWithTag("shadow_knowledge_classic") then
			    if doer.components.talker ~= nil then
	                if TUNING.LANGUAGE == true then
		                doer.components.talker:Say("Forged successfully, new classic of shadow knowledge!")
		            elseif TUNING.LANGUAGE == false then
		                doer.components.talker:Say("锻造成功，新的暗影知识典籍！")
		            end								
			    end
			
			    local shadow_knowledge_classic = SpawnPrefab("shadow_knowledge_classic")			
			    if doer.components.inventory ~= nil then
                    doer.components.inventory:GiveItem(shadow_knowledge_classic, nil, Vector3(TheSim:GetScreenPos(target.Transform:GetWorldPosition())))
                else
				    shadow_knowledge_classic.Transform:SetPosition(doer.Transform:GetWorldPosition())
				    SpawnPrefab("small_puff").Transform:SetPosition(doer.Transform:GetWorldPosition())			
			    end		
				
				if doer.page_read ~= nil then
				    shadow_knowledge_classic.page = doer.page_read
				end

            else
			
			    if math.random() < 0.1 and target.components.lootdropper ~= nil then
                    local x, y, z = target.Transform:GetWorldPosition()
	                target.components.lootdropper:FlingItem(SpawnPrefab("horrorfuel"), Vector3(x, math.random()*3, z))	 				
				end
			
			    if doer.components.talker ~= nil then
	                if TUNING.LANGUAGE == true then
		                doer.components.talker:Say("Ah, I already have a classic!")
		            elseif TUNING.LANGUAGE == false then
		                doer.components.talker:Say("啊，我已经有一本典籍了！")
		            end								
			    end			
			end

		    local x, y, z = target:FindSafeLocationToMove()
		    target.Transform:SetPosition(x, y, z)
			
		    return true
		
		end
    end,
    distance = 1,
    rmb = true,
    instant = false,
	mount_enabled = false,
    priority = 10,
}

AddAction(SHADOW_FORGE)

local SHADOW_READ =
{
    id = "SHADOW_READ",
    str = STRINGS.ACTIONS.SHADOW_READ,
    fn = function(act)  --act.target, act.invobject, act.doer
        local target = act.target or act.invobject
		local doer = act.doer
		if target ~= nil
		    and doer ~= nil
			and target.components.shadowbook ~= nil
			and doer.components.shadowreader ~= nil
		then
		    return doer.components.shadowreader:Read(target)
		end
    end,
    distance = 0.1,
    rmb = true,
    instant = false,
	mount_enabled = true,
    priority = 10,
}

AddAction(SHADOW_READ)

local SHADOW_REPAIR =
{
    id = "SHADOW_REPAIR",
    str = STRINGS.ACTIONS.SHADOW_REPAIR,
    fn = function(act)  --act.target, act.invobject, act.doer
        local invobject = act.invobject
		local target = act.target
		local doer = act.doer
		if invobject ~= nil
		    and target ~= nil
			and invobject.components.shadowrepairer ~= nil
			and target.components.shadowrepairable ~= nil
		then
		    return target.components.shadowrepairable:Repair(doer, invobject)
		end
    end,
    --distance = 0,
    rmb = true,
    instant = false,
	mount_enabled = true,
    priority = 10,
}

AddAction(SHADOW_REPAIR)

local LEVEL_TALK =
{
    id = "LEVELTALK",
    str = STRINGS.ACTIONS.LEVELTALK,
    fn = function(act)  --act.target, act.invobject, act.doer
	    if act.target ~= nil and act.target.components.interactivelevel ~= nil then 
		    act.target.components.interactivelevel:DoActivate(act.doer)
		    return true
	    end
    end,
    distance = 0.5,
    rmb = true,
    instant = false,
	mount_enabled = true,
    priority = 12,
}

AddAction(LEVEL_TALK)

AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.EYEOFINSIGHT_SEE, "dolongaction"))
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.DRAW_OINC, "give"))
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.BLACKEN, "dolongaction"))
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.SHADOW_UPGRADE, "dolongaction"))
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.SCYTHE, "chop_start"))

AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.SHADOW_FORGE, "dotensecaction"))
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.SHADOW_READ, "book"))

AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.SHADOW_REPAIR, "dolongaction"))
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.LEVELTALK, "give"))

if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.EYEOFINSIGHT_SEE, "dolongaction"))
	AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.DRAW_OINC, "give"))
	AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.BLACKEN, "dolongaction"))
	AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.SHADOW_UPGRADE, "dolongaction"))
	AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.SCYTHE, "chop_start"))
	
	AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.SHADOW_FORGE, "dotensecaction"))	
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.SHADOW_READ, "book"))	
	
	AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.SHADOW_REPAIR, "dolongaction"))	
	
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.LEVELTALK, "give"))
end

AddPrefabPostInit("ancient_altar", function(inst)
    if GetPlayer() and GetPlayer().prefab == "charlie" then
	    local PopupDialogScreen = require("screens/bigpopupdialog")
        inst:AddComponent("interactivelevel")
		inst.components.interactivelevel.OnActivate = function(inst, doer)
		    SetPause(true)
			
			local current_level = 1
			local experience_held = 0
			local experience_now = 0
			local experience_require = 0

			if doer ~= nil then
			    if doer.components.charlierefreshmodlevel ~= nil then
			        current_level = doer.components.charlierefreshmodlevel:GetLevel()
					experience_now = doer.components.charlierefreshmodlevel:GetExp()
					if current_level >= doer.components.charlierefreshmodlevel.max_level then
						experience_now = 0
					    experience_require = 0
					else
					    experience_require = doer.components.charlierefreshmodlevel:GetLevelData()["CHARLIE_LEVEL_REQUIREMENTS_DATA"]["charlie_exp"][current_level+1]
					end
				end
				if doer.components.inventory ~= nil then
				    experience_held = doer.components.inventory:Count("charlie_exp")
				end	
			end
			
			local CHARLIE_LEVEL_BODY = string.format("Charlie current level: %d\nCharlie experience held: %d\nCharlie experience progress bar: %d / %d", current_level, experience_held, experience_now, experience_require)
			local UPGRADE_FAIL = "Ah, I don't have any Shadow Embers to add."
			local UPGRADE_ADD_EXP = "I have consumed some Shadow Embers, but it is still not enough for me to level up."
			local UPGRADE_SUCCESS = "Upgrade successful! Now I have stronger strength!"
			local ALREADY_MAX_LEVEL = "I have reached the peak and don't need to upgrade anymore."
			if TUNING.LANGUAGE == false then
			    CHARLIE_LEVEL_BODY = string.format("查理当前等级：%d\n查理所持有的经验：%d\n查理经验进度条：%d / %d", current_level, experience_held, experience_now, experience_require)
				UPGRADE_FAIL = "啊，我没有任何暗影余烬来添加。"
				UPGRADE_ADD_EXP = "我已经消耗了一些暗影余烬，但还是不足以让我升级。"
				UPGRADE_SUCCESS = "升级成功！现在我拥有更强的力量！"
				ALREADY_MAX_LEVEL = "我已经达到巅峰，不需要再去升级了。"
			end
			
            local function Cancel()
                TheFrontEnd:PopScreen()
                SetPause(false)
		        inst.components.interactivelevel.inactive = true
            end		

	        local function TryToUpgrade()
                TheFrontEnd:PopScreen()
                SetPause(false)
				if doer ~= nil and doer.components.talker ~= nil then
				    if current_level >= (doer.components.charlierefreshmodlevel ~= nil and doer.components.charlierefreshmodlevel.max_level or 10) then
				        inst.components.interactivelevel.inactive = true     
						doer.components.talker:Say(ALREADY_MAX_LEVEL)
						return
				    end
				    if experience_held <= 0 and (experience_now < experience_require) then
					    doer.components.talker:Say(UPGRADE_FAIL)
					else
					    if doer.components.inventory ~= nil then 
						    doer.components.inventory:ConsumeByName("charlie_exp", experience_held)
						end
					    if doer.components.charlierefreshmodlevel ~= nil then
						    doer.components.charlierefreshmodlevel:AddExp(experience_held)				
							if current_level == doer.components.charlierefreshmodlevel:GetLevel() then
						        doer.components.talker:Say(UPGRADE_ADD_EXP)
					    	else
						        doer.components.talker:Say(UPGRADE_SUCCESS)
					    	end
						end
					end	
				end
		        inst.components.interactivelevel.inactive = true       
            end		
 
            TheFrontEnd:PushScreen(PopupDialogScreen(
            STRINGS.CHARLIE_LEVEL_TITLE, CHARLIE_LEVEL_BODY,
            {
                {text = STRINGS.CHARLIE_LEVEL_TRY_TO_UPGRADE, cb = TryToUpgrade},
                {text = STRINGS.CHARLIE_LEVEL_CLOSE, cb = Cancel}
            }
            ))
 
		end
	end
end)

AddComponentPostInit("shopper", function(self)
    local _GetMoney = self.GetMoney
	local _PayMoney = self.PayMoney
	
	function self:GetMoney(inventory)
	    if _GetMoney then
		    local money = _GetMoney(self, inventory)
		    local credit_card = inventory:FindItems(
                function(item)
                    return item:HasTag("creditcard")
                end)
	        for k,v in pairs(credit_card) do
			    if v ~= nil and v.oinc_amount_inside ~= nil then
			        money = money + v.oinc_amount_inside
				end
			end
			return money
		end
	end
	
	function self:PayMoney(inventory, cost)
	    if self.inst.components.builder ~= nil 
		    and self.inst.components.builder.freebuildmode then
		    return
		else
		    local credit_card_to_pay = inventory:FindItem(
                function(item)
                    return item:HasTag("creditcard") 
                end)	
		    if credit_card_to_pay ~= nil then
			    if credit_card_to_pay.oinc_amount_inside >= cost then
			        credit_card_to_pay.oinc_amount_inside = math.max(0, credit_card_to_pay.oinc_amount_inside - cost)
				else           		
					local debt = cost - credit_card_to_pay.oinc_amount_inside
					
					credit_card_to_pay.oinc_amount_inside = 0
					
	                local oincused = 0
	                local oinc10used = 0
	                local oinc100used = 0
	                local oincgained = 0
	                local oinc10gained = 0							
					local hasoincs,oincamount = inventory:Has("oinc", 0, true)
	                local hasoinc10s,oinc10amount = inventory:Has("oinc10", 0, true)
	                local hasoinc100s,oinc100amount = inventory:Has("oinc100", 0, true)

					local credit_cards_to_pay = inventory:FindItems(
                        function(item)
                            return item:HasTag("creditcard") 
							    and item.oinc_amount_inside ~= nil
								and item.oinc_amount_inside >= debt
                        end)	
					
					local lower_credit_cards_to_pay = inventory:FindItems(
                        function(item)
                            return item:HasTag("creditcard") 
							    and item.oinc_amount_inside ~= nil
								and item.oinc_amount_inside < debt
								and item.oinc_amount_inside > 0
                        end)					
					
					if #credit_cards_to_pay > 0 then
					    local card = credit_cards_to_pay[1]
                        card.oinc_amount_inside = math.max(0, card.oinc_amount_inside - debt)	
                        debt = 0					
					elseif #lower_credit_cards_to_pay > 0 then
					    for k,v in pairs(lower_credit_cards_to_pay) do
						    if v ~= nil and v.oinc_amount_inside ~= nil then
							    debt = math.max(0, debt - v.oinc_amount_inside)	
							    v.oinc_amount_inside = 0
							end
							if debt == 0 then
							    break
							end
						end										
					end

		            while debt > 0 do
			            while debt > 0 and oincamount > 0 do
				            oincamount = oincamount - 1
				            debt = debt - 1
				            oincused = oincused + 1
		            	end		
		            	if debt > 0 then
				            if oinc10amount > 0 then
					            oinc10amount = oinc10amount - 1
					            oinc10used = oinc10used + 1
					            for i=1,10 do
						            oincamount = oincamount + 1
					             	oincgained = oincgained + 1
					            end
		               		elseif oinc100amount > 0 then
					            oinc100amount = oinc100amount - 1
					            oinc100used = oinc100used + 1
					            for i=1,10 do
					              	oinc10amount = oinc10amount + 1
					            	oinc10gained = oinc10gained + 1
					            end
				            end
			            end
		            end

		            local oincresult = oincgained - oincused
		            if oincresult > 0 then
			            for i=1,oincresult do
				            local coin = SpawnPrefab("oinc")
				            inventory:GiveItem( coin )	
			            end
		            end

		            if oincresult < 0 then
			            for i=1,math.abs(oincresult) do
				            inventory:ConsumeByName("oinc", 1, true)
			            end
		            end

		            local oinc10result = oinc10gained - oinc10used
		            if oinc10result > 0 then
			            for i=1,oinc10result do
				            local coin = SpawnPrefab("oinc10")
				            inventory:GiveItem( coin )	
		               	end
		            end

		            if oinc10result < 0 then
			            for i=1,math.abs(oinc10result) do
				            inventory:ConsumeByName("oinc10", 1, true)
			            end
		            end

		            local oinc100result = 0 - oinc100used 
		            if oinc100result < 0 then
			            for i=1,math.abs(oinc100result) do
				            inventory:ConsumeByName("oinc100", 1, true)
			            end
		            end

				end
			else	
	            if _PayMoney then
		            _PayMoney(self, inventory, cost)
		        end			
			end
		end	
	end

end)

local function TagofAreaPoison(inst)
    inst:AddTag("areapoisoner")
end

AddPrefabPostInit("poisonhole", TagofAreaPoison)
AddPrefabPostInit("gascloud", TagofAreaPoison)

local function IsPointCloseToLavaOrPoisonousArea(pt, radius)

    for i = -radius, radius, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x - radius, pt.y, pt.z + i) == GROUND.VOLCANO_LAVA 
		    or GetWorld().Map:GetTileAtPoint(pt.x + radius, pt.y, pt.z + i) == GROUND.VOLCANO_LAVA 
		then
            return true
        end
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z - radius) == GROUND.VOLCANO_LAVA
		    or GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z + radius) == GROUND.VOLCANO_LAVA 
		then
            return true
        end
    end

    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, radius, {"areapoisoner"}) ) do
        if v:GetDistanceSqToPoint(pt) < radius * radius then
            return true
        end
    end
	
	if IsDLCEnabled(PORKLAND_DLC) then
	    for i = -radius, radius, 1 do
            if GetWorld().Map:GetTileAtPoint(pt.x - radius, pt.y, pt.z + i) == GROUND.GASJUNGLE 
		        or GetWorld().Map:GetTileAtPoint(pt.x + radius, pt.y, pt.z + i) == GROUND.GASJUNGLE 
		    then
                return true
            end
        end
	
        for i = -(radius - 1), radius - 1, 1 do
            if GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z - radius) == GROUND.GASJUNGLE
		        or GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z + radius) == GROUND.GASJUNGLE 
		    then
                return true
            end
        end
    end	
	
	return false

end

local function IsPointCloseToLava(pt, radius)
    for i = -radius, radius, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x - radius, pt.y, pt.z + i) == GROUND.VOLCANO_LAVA 
		    or GetWorld().Map:GetTileAtPoint(pt.x + radius, pt.y, pt.z + i) == GROUND.VOLCANO_LAVA 
		then
            return true
        end
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z - radius) == GROUND.VOLCANO_LAVA
		    or GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z + radius) == GROUND.VOLCANO_LAVA 
		then
            return true
        end
    end
	return false
end

AddComponentPostInit("blinkstaff", function(self)
    local _CanBlinkToPoint = self.CanBlinkToPoint
    function self:CanBlinkToPoint(pt)
	    if _CanBlinkToPoint then
		    if GetPlayer().GetIsOnWater ~= nil and IsPointCloseToLava(pt, 1) then
			    return false
			end
		    return _CanBlinkToPoint(self, pt)
		end
	end   
end)

--[[
local function ForestWorldAquaticBuildingsTest(pt, recipe)
    local px, py, pz = GetPlayer().Transform:GetWorldPosition()
    local player_on_land = GetVisualTileType(px, py, pz) ~= GROUND.IMPASSABLE	
	local x, y, z = pt:Get()
	local onWater = GetVisualTileType(pt.x, pt.y, pt.z) == GROUND.IMPASSABLE
	
	if player_on_land then
    	local minBuffer = 2
    	local testTile = GetVisualTileType(x + minBuffer, y, z)
		onWater = testTile == GROUND.IMPASSABLE and onWater
		testTile = GetVisualTileType(x - minBuffer, y, z)
        onWater = testTile == GROUND.IMPASSABLE and onWater
		testTile = GetVisualTileType(x, y, z + minBuffer)
        onWater = testTile == GROUND.IMPASSABLE and onWater		
		testTile = GetVisualTileType(x, y, z - minBuffer)
        onWater = testTile == GROUND.IMPASSABLE and onWater
		
    	if not onWater then
			return false
		end	
	
	else
      
    	local minBuffer = 0.5
		if recipe.name == "ballphinhouse" then
			minBuffer = 100
		end
		
		local tooClose = false 
    	local testTile = GetVisualTileType(x + minBuffer, y, z)
		tooClose = testTile ~= GROUND.IMPASSABLE or tooClose
		testTile = GetVisualTileType(x - minBuffer, y, z)
        tooClose = testTile ~= GROUND.IMPASSABLE or tooClose
		testTile = GetVisualTileType(x, y, z + minBuffer)
        tooClose = testTile ~= GROUND.IMPASSABLE or tooClose		
		testTile = GetVisualTileType(x, y, z - minBuffer)
        tooClose = testTile ~= GROUND.IMPASSABLE or 	tooClose	

        testTile = GetVisualTileType(x + minBuffer, y, z + minBuffer)
		tooClose = testTile ~= GROUND.IMPASSABLE or tooClose
		testTile = GetVisualTileType(x - minBuffer, y, z - minBuffer)
        tooClose = testTile ~= GROUND.IMPASSABLE or tooClose
		testTile = GetVisualTileType(x - minBuffer, y, z + minBuffer)
        tooClose = testTile ~= GROUND.IMPASSABLE or tooClose		
		testTile = GetVisualTileType(x + minBuffer, y, z - minBuffer)
        tooClose = testTile ~= GROUND.IMPASSABLE or tooClose		
		
		return onWater and not tooClose
	
	end

    local min_rad = recipe.min_spacing or 3.2
	if min_rad ~= 0 then
		local ents = TheSim:FindEntities(pt.x,pt.y,pt.z, 6, nil, {'player', 'fx', 'NOBLOCK'}) -- or we could include a flag to the search?
		for k, v in pairs(ents) do
			if v ~= self.inst and (not v.components.placer) and v.entity:IsVisible() and not (v.components.inventoryitem and v.components.inventoryitem.owner ) then		
				--if recipe.name == "treasurechest" and v.prefab == "pond" then
				--	min_rad = min_rad + 1
				--end
				local dsq = distsq(Vector3(v.Transform:GetWorldPosition()), pt)
				if dsq < min_rad*min_rad then
					return false
				end
			end
		end
	end	
	
	return false
end
]]

local function IsPointNearNonLilyPadPuddle(pt, radius)
    local puddle = TheSim:FindEntities(pt.x, pt.y, pt.z, radius, {"puddle"}, {"puddle_disappear"})
    local lilypad = TheSim:FindEntities(pt.x, pt.y, pt.z, radius, {"LilyPadForPuddle"})	
    if #puddle > 0 and #lilypad <= 0 then
	    return true
	else
	    return false
	end
end

local function IsPointNearNonLilyPadDeepPuddle(pt, radius)
    local deeppuddle = TheSim:FindEntities(pt.x, pt.y, pt.z, radius, {"deeppuddle"}, {"puddle_disappear"})
    local lilypad = TheSim:FindEntities(pt.x, pt.y, pt.z, radius, {"LilyPadForPuddle"})	
    if #deeppuddle > 0 and #lilypad <= 0 then
	    return true
	else
	    return false
	end
end

local function IsPointNearLilyPad(pt, radius)
    local lilypad = TheSim:FindEntities(pt.x, pt.y, pt.z, radius, {"LilyPadForPuddle"})	
    if #lilypad > 0 then
	    return true
	else
	    return false
	end
end

local function IsPointNearWaterObjects(pt, min_spacing)
    if min_spacing == 0 then
	    return false
	else
		local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 6, nil, {"player", "FX", "NOBLOCK"})
		for k, v in pairs(ents) do
			if (not v.components.placer) and v.entity:IsVisible() and not (v.components.inventoryitem and v.components.inventoryitem.owner) then
				local dsq = distsq(Vector3(v.Transform:GetWorldPosition()), pt)
				if dsq < min_spacing*min_spacing then
					return true
				else
				    return false
				end
			end
		end
	end
end

if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
    AddComponentPostInit("builder", function(self)
	    local _CanBuildAtPoint = self.CanBuildAtPoint
		function self:CanBuildAtPoint(pt, recipe)
		    if _CanBuildAtPoint then
			    if not recipe.aquatic then
				    return self.inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, recipe.name and recipe.name == "telebase" and 4 or 2) 
					    and not IsPointCloseToLavaOrPoisonousArea(pt, recipe.name and recipe.name == "telebase" and 3 or 1)
						and not IsPointNearNonLilyPadPuddle(pt, recipe.name and recipe.name == "telebase" and 4.5 or 2.5)
					    and _CanBuildAtPoint(self, pt, recipe)
				else
			        return (IsPointNearNonLilyPadPuddle(pt, 2.5) and not IsPointNearWaterObjects(pt, recipe.min_spacing or 3.2)) or _CanBuildAtPoint(self, pt, recipe)
				end
			end
		end
	end)
end

if TheInput and TheInput.AddPickCondition ~= nil then
    TheInput:AddPickCondition("PrioritizeAncientHangingLamp", function(ent)
	    return ent:HasTag("ancient_hanging_lamp") and not ent:HasTag("on_ground_now")
	end,
	10
	)
end

AddPrefabPostInit("dug_grass", function(inst)
    if SaveGameIndex:IsModePorkland() then
	    inst.AnimState:SetBuild("grassgreen_build")
		inst.components.inventoryitem.imagename = "dug_grass_green"
	end
end)

AddPrefabPostInit("grass_tall", function(inst)
    if SaveGameIndex:IsModeShipwrecked() or SaveGameIndex:IsModePorkland() then
        inst.MiniMapEntity:SetIcon("tall_grass_map_green.tex")
	else
	    inst.MiniMapEntity:SetIcon("tall_grass_map_yellow.tex")		
		inst.AnimState:SetMultColour(255/255, 200/255, 175/255, 1)
	end
end)

--[[Code available for reference, hide water effect of lotus on land
AddPrefabPostInit("lotus", function(inst)
    inst.AnimState:OverrideSymbol("water_ripple", "lotus", "")
	inst.AnimState:OverrideSymbol("water_shadow", "lotus", "")
end)
]]

AddPrefabPostInit("quackendrill", function(inst)
    inst:AddTag("eyeturret")  --For action string
	inst:AddTag("no_float_fx_spawn_on_puddle")
end)

AddPrefabPostInit("tar_extractor", function(inst)
	inst:AddTag("no_float_fx_spawn_on_puddle")
end)

AddPrefabPostInit("hacking_tall_grass_fx", function(inst)
    if not SaveGameIndex:IsModeShipwrecked() and not SaveGameIndex:IsModePorkland() then
		inst.AnimState:SetMultColour(1, 1, 1, 1)
    end
end)

if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
    AddComponentPostInit("pollinator", function(self)
	    local _CheckFlowerDensity = self.CheckFlowerDensity
		function self:CheckFlowerDensity()
            if _CheckFlowerDensity then
			    local pt = self.inst:GetPosition()
                if not self.inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 2) or IsPointCloseToLavaOrPoisonousArea(pt, 1) then
				    return false
				end
			    return _CheckFlowerDensity(self)
			end
		end
	end)
end

if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
    AddComponentPostInit("deployable", function(self)
	    local _CanDeploy = self.CanDeploy
		local _Deploy = self.Deploy
		
		function self:CanDeploy(pt, atdeploy)
            if _CanDeploy then
			    local range = 2.5
			    if self.inst.prefab == "lilypad_seed" then
				    range = 1
				end
			
			    if (self.inst.prefab == "quackendrill" or self.inst.prefab == "seaweed_stalk") 			
				    and IsPointNearNonLilyPadDeepPuddle(pt, 2.5) 			
				then
	                local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 4, nil, {"NOBLOCK", "player", "FX"}) 
		            local min_spacing = self.min_spacing or 2
	                for k, v in pairs(ents) do
			            if v ~= self.inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				            if distsq( Vector3(v.Transform:GetWorldPosition()), pt) < min_spacing*min_spacing then
					            return false
				            end
			            end
		            end	
		            return true
				end
			    if (self.inst.prefab == "lilypad_seed" 
				        or self.inst.prefab == "lotus_seed" 
						or self.inst.prefab == "brainysprout_seed" 
						or self.inst.prefab == "nubbin"
						or self.inst.prefab == "wall_enforcedlimestone_item"
						or self.inst.prefab == "mangrovetree_seed") 			
				    and IsPointNearNonLilyPadPuddle(pt, range) 			
				then
	                local ents2 = TheSim:FindEntities(pt.x, pt.y, pt.z, 4, nil, {"NOBLOCK", "player", "FX"}) 
		            local min_spacing2 = self.min_spacing or 2
	                for i, j in pairs(ents2) do
			            if j ~= self.inst and j:IsValid() and j.entity:IsVisible() and not j.components.placer and j.parent == nil then
				            if distsq( Vector3(j.Transform:GetWorldPosition()), pt) < min_spacing2*min_spacing2 then
					            return false
				            end
			            end
		            end	
		            return true
				end						
				if self.inst.prefab ~= "quackendrill" 
				    and self.inst.prefab ~= "seaweed_stalk"
	                and self.inst.prefab ~= "lilypad_seed" 
				    and self.inst.prefab ~= "lotus_seed" 
				    and self.inst.prefab ~= "brainysprout_seed" 
				    and self.inst.prefab ~= "nubbin"
					and self.inst.prefab ~= "wall_enforcedlimestone_item"
				    and self.inst.prefab ~= "mangrovetree_seed"			
				    and IsPointNearLilyPad(pt, 2.5) 	
				then
	                local ents3 = TheSim:FindEntities(pt.x, pt.y, pt.z, 4, nil, {"NOBLOCK", "player", "FX"}) 
		            local min_spacing3 = self.min_spacing or 2
	                for i2, j2 in pairs(ents3) do
			            if j2 ~= self.inst and j2:IsValid() and j2.entity:IsVisible() and not j2.components.placer and j2.parent == nil then
				            if distsq( Vector3(j2.Transform:GetWorldPosition()), pt) < min_spacing3*min_spacing3 then
					            return false
				            end
			            end
		            end	
		            return true				
				end
	            if IsPointCloseToLava(pt, 1) then 
                    return self.inst:HasTag("suitable_for_lava") and _CanDeploy(self, pt, atdeploy)
                elseif IsPointNearNonLilyPadPuddle(pt, 2.5) and not self.inst:HasTag("suitable_for_puddle_all") then
				    return self.inst:HasTag("suitable_for_puddle") and _CanDeploy(self, pt, atdeploy)
				end							
			    return _CanDeploy(self, pt, atdeploy)
			end
		end
		
		function self:Deploy(pt, deployer, rot)
		    if _Deploy then
			    local range = 2.5
			    if self.inst.prefab == "lilypad_seed" then
				    range = 1
				end
				
			    if (self.inst.prefab == "quackendrill" or self.inst.prefab == "seaweed_stalk") 			
				    and IsPointNearNonLilyPadDeepPuddle(pt, 2.5) 			
				then
		            if self.ondeploy then
	                    self.ondeploy(self.inst, pt, deployer, rot)
		            end
                    if self.inst:HasTag("plant") and deployer:HasTag("plantkin") then
                        if deployer.growplantfn then
                            deployer.growplantfn(deployer)
                        end
                    end
                    return true
				end
			    if (self.inst.prefab == "lilypad_seed" 
				        or self.inst.prefab == "lotus_seed" 
						or self.inst.prefab == "brainysprout_seed" 
						or self.inst.prefab == "nubbin"
						or self.inst.prefab == "wall_enforcedlimestone_item"
						or self.inst.prefab == "mangrovetree_seed") 			
				    and IsPointNearNonLilyPadPuddle(pt, range) 			
				then
		            if self.ondeploy then
	                    self.ondeploy(self.inst, pt, deployer, rot)
		            end
                    if self.inst:HasTag("plant") and deployer:HasTag("plantkin") then
                        if deployer.growplantfn then
                            deployer.growplantfn(deployer)
                        end
                    end
	                return true
				end				
				if self.inst.prefab ~= "quackendrill" 
				    and self.inst.prefab ~= "seaweed_stalk"
	                and self.inst.prefab ~= "lilypad_seed" 
				    and self.inst.prefab ~= "lotus_seed" 
				    and self.inst.prefab ~= "brainysprout_seed" 
				    and self.inst.prefab ~= "nubbin"
					and self.inst.prefab ~= "wall_enforcedlimestone_item"
				    and self.inst.prefab ~= "mangrovetree_seed"			
				    and IsPointNearLilyPad(pt, 2.5) 	
				then
		            if self.ondeploy then
	                    self.ondeploy(self.inst, pt, deployer, rot)
		            end
                    if self.inst:HasTag("plant") and deployer:HasTag("plantkin") then
                        if deployer.growplantfn then
                            deployer.growplantfn(deployer)
                        end
                    end
		            return true				
				end		
	            if IsPointCloseToLava(pt, 1) then 
                    return self.inst:HasTag("suitable_for_lava") and _Deploy(self, pt, atdeploy, rot)
                elseif IsPointNearNonLilyPadPuddle(pt, 2.5) and not self.inst:HasTag("suitable_for_puddle_all") then
				    return self.inst:HasTag("suitable_for_puddle") and _Deploy(self, pt, atdeploy, rot)
				end				
			    return _Deploy(self, pt, deployer, rot)
			end
		end
	end)
end

local function default_deploy_test(inst, pt, atdeploy)
	if not IsImpassablePoint(pt.x, pt.y, pt.z) then
        if not atdeploy then
    		local MouseCharacter = TheInput:GetWorldEntityUnderMouse()
    		if MouseCharacter and not MouseCharacter:HasTag("player") then
    			return false
    		end
        end
	    local ents = TheSim:FindEntities(pt.x,pt.y,pt.z, 4, nil, {'NOBLOCK', 'player', 'FX'})
		local min_spacing = inst.components.deployable.min_spacing or 2

	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt) < min_spacing*min_spacing then
					return false
				end
			end
		end
		return true
	end
	return false
end

local function DONOTDEPLOYTOONEARWATER(inst) --Need CAPY_DLC or PORKLAND_DLC
    if inst.components.deployable ~= nil then
	    local _DeployTest = inst.components.deployable.test
		if _DeployTest then
		    inst.components.deployable.test = function(inst, pt, atdeploy)
                if inst.prefab ~= "dug_grass" then
			        if not inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 2) then
				        return false
				    end				
				    return not IsImpassablePoint(pt.x, pt.y, pt.z) and _DeployTest(inst, pt, atdeploy)
				else
				    if IsMangroveAtPoint(pt.x, pt.y, pt.z) then
					    return default_deploy_test(inst, pt, atdeploy)
					end
			        if not inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 2) then
				        return false
				    end				
				    return not IsImpassablePoint(pt.x, pt.y, pt.z) and _DeployTest(inst, pt, atdeploy)									
				end
			end	
		else
		    inst.components.deployable.test = function(inst, pt, atdeploy)			
				return inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 2) and default_deploy_test(inst, pt, atdeploy)
			end		     
		end
	end
end

local function DONOTDEPLOYTOONEARWATER_WALL(inst) --Need CAPY_DLC or PORKLAND_DLC
    if inst.components.deployable ~= nil then
	    local _DeployTest = inst.components.deployable.test
		if _DeployTest then
		    inst.components.deployable.test = function(inst, pt, atdeploy)
			    if not inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 1) then
				    return false
				end				
				return _DeployTest(inst, pt, atdeploy)
			end	
		else
		    inst.components.deployable.test = function(inst, pt, atdeploy)			
				return inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 1) and default_deploy_test(inst, pt, atdeploy)
			end		     
		end
	end
end

if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
    AddPrefabPostInit("acorn", DONOTDEPLOYTOONEARWATER)
    AddPrefabPostInit("butterfly", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("coconut", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("waterdrop", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("wall_stone_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("wall_wood_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("wall_hay_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("wall_ruins_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("wall_limestone_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("trap_bramble", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("trap_teeth", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("teatree_nut", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("tar", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("spidereggsack", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("seeds", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("sandbagsmall_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("sandbag_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("portablecookpot_item", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("dug_berrybush", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("dug_berrybush2", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("dug_sapling", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("dug_grass", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("dug_marsh_bush", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("dug_bambootree", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("dug_bush_vine", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("dug_coffeebush", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("dug_elephantcactus", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("dug_nettle", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("pinecone", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("minisign_item", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("minisign_drawn", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("lureplantbulb", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("jungletreeseed", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("burr", DONOTDEPLOYTOONEARWATER)
	AddPrefabPostInit("hedge_block_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("hedge_cone_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("hedge_layered_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("fence_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("fence_gate_item", DONOTDEPLOYTOONEARWATER_WALL)
	AddPrefabPostInit("clawpalmtree_sapling", DONOTDEPLOYTOONEARWATER)	
end

AddPrefabPostInit("statueharp_rose_placer", function(inst)
    if inst.components.placer ~= nil then
	    inst.AnimState:OverrideSymbol("swap_statue", "statue_small_harp_build", "swap_statue")
		inst.AnimState:AddOverrideBuild("statue_small_harp_vine_build")
	end
end)

AddPrefabPostInit("statuemaxwell_rose_placer", function(inst)
    if inst.components.placer ~= nil then
		inst.AnimState:AddOverrideBuild("statue_maxwell_vine_build")
	end
end)

local telebase_parts =
{
    { part = "gemsocket", x = -1.6, z = -1.6 },
    { part = "gemsocket", x =  2.7, z = -0.8 },
    { part = "gemsocket", x = -0.8, z =  2.7 },
}

AddPrefabPostInit("telebase_placer", function(inst)
    if inst.components.placer ~= nil then
        local rot = 45 * math.pi / 180
        local sin_rot = math.sin(rot)
        local cos_rot = math.cos(rot)
        for i, v in ipairs(telebase_parts) do
            local part = SpawnPrefab(v.part)
            part.Transform:SetPosition(v.x * cos_rot - v.z * sin_rot, 0, v.z * cos_rot + v.x * sin_rot)
            part.entity:SetParent(inst.entity)
            part.entity:SetCanSleep(false)
            part:AddTag("NOCLICK")
			part:AddTag("NOBLOCK")
            part:AddTag("placer")
            inst.components.placer:LinkEntity(part)
        end
	end
end)

AddComponentPostInit("workable", function(self)
    local _SetWorkAction = self.SetWorkAction
	function self:SetWorkAction(act)
        if _SetWorkAction then
		    _SetWorkAction(self, act)
		end
		if act ~= nil then
		    if act == ACTIONS.DIG then
		        self.inst:AddTag("diggable")
			else
			    self.inst:RemoveTag("diggable")
			end
		end
	end
	
	function self:OnRemoveFromEntity()
	    self.inst:RemoveTag("diggable")
	end
end)

AddPrefabPostInit("statuemaxwell", function(inst)
    inst.AnimState:SetBuild("statue_maxwell_build")
end)

--[[
local function DoVariantCheck_Berrybush(inst)
    inst:DoTaskInTime(0.1, function()
	    if not inst.HasCheckedVariant then
		    inst.HasCheckedVariant = true
			if math.random() <= 0.5 then
			    SpawnPrefab("berrybush_juicy").Transform:SetPosition(inst.Transform:GetWorldPosition())
				inst:Remove()
			end
		end
	end)
	
	local _OnLoad = inst.OnLoad
	
	inst.OnLoad = function(inst, data)
	    if data == nil then
		    return
		end
		if _OnLoad then
		    _OnLoad(inst, data)
		end
		
		inst.HasCheckedVariant = true
	end
end

AddPrefabPostInit("berrybush", DoVariantCheck_Berrybush)
AddPrefabPostInit("berrybush_snake", DoVariantCheck_Berrybush)
AddPrefabPostInit("berrybush2", DoVariantCheck_Berrybush)
AddPrefabPostInit("berrybush2_snake", DoVariantCheck_Berrybush)

AddPrefabPostInit("reeds", DoVariantCheck_Berrybush)
AddPrefabPostInit("cave_banana_tree", DoVariantCheck_Berrybush)
]]

AddComponentPostInit("drivable", function(self)
    local _OnUpdate = self.OnUpdate
    function self:OnUpdate(dt)
	    if _OnUpdate then
		    local x, y, z = self.inst.Transform:GetWorldPosition()
		    if (GetWorld() and GetWorld().prefab == "forest" 
			    and GetVisualTileType(x, y, z) == GROUND.IMPASSABLE)
				or IsPointNearNonLilyPadPuddle(self.inst:GetPosition(), 2)
			then
			    return
			else
		        _OnUpdate(self, dt)
			end
		end
	end
end)

--Fixed the bug that critters in the "my pets" mod cannot change skin through reskin tool. Also fixed the on-water effect and flying critters' physics collision.
local critter_lamb_skins =
{	
	{
		skin = 	"sheepington_build",
		xml = 	"images/inventoryimages/critter_lamb_builder.xml",
		tex =	"critter_lamb_builder.tex",
		desc = 	"Classic", 
		cndesc = "经典",  
	},
	{
		skin = "lamb_winter",
		xml = 	"images/inventoryimages/lamb_winter.xml",
		tex =	"lamb_winter.tex",
		desc = 	"Snowflake Ewelet", 
		cndesc = "雪花小钢羊",
	},
}

AddPrefabPostInit("critter_lamb", function(inst)
    if inst.components.dsskins ~= nil then
	    inst.components.dsskins.skins = critter_lamb_skins
	end
    if inst.components.trader ~= nil then
	    inst:RemoveComponent("trader")
	end	
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        if inst.components.tiletracker == nil then
		    inst:AddComponent("tiletracker")
			inst.components.tiletracker:Start()
			inst.components.tiletracker:SetOnWaterChangeFn(function(inst, onwater)
			    if onwater then
				    if inst.float_fx == nil then
		                inst.float_fx = SpawnPrefab("float_fx_front")
		                inst.float_fx.entity:SetParent(inst.entity)
		                inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
                        inst.float_fx.Transform:SetPosition(0, 0.1, 0)	
                        inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)						
					end
					if inst.float_fx2 == nil then
		                inst.float_fx2 = SpawnPrefab("float_fx_back")
		                inst.float_fx2.entity:SetParent(inst.entity)
		                inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
                        inst.float_fx2.Transform:SetPosition(0, 0.1, 0)	
                        inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)						
					end
				else
	                if inst.float_fx ~= nil then
		                inst.float_fx:Remove()
		                inst.float_fx = nil
	                end
	                if inst.float_fx2 ~= nil then
		                inst.float_fx2:Remove()
		                inst.float_fx2 = nil
	                end					
				end
			end)
		end
	end
end)

local critter_puppy_skins =
{	
	{
		skin = 	"pupington_build",
		xml = 	"images/inventoryimages/critter_puppy_builder.xml",
		tex =	"critter_puppy_builder.tex",
		desc = 	"Classic",
		cndesc = "经典",
	},
	{
		skin = 	"puppy_winter",
		xml = 	"images/inventoryimages/puppy_winter.xml",
		tex =	"puppy_winter.tex",
		desc = 	"Arctic Vargling", 
		cndesc = "北极小座狼",
	},
}

AddPrefabPostInit("critter_puppy", function(inst)
    if inst.components.dsskins ~= nil then
	    inst.components.dsskins.skins = critter_puppy_skins
	end
    if inst.components.trader ~= nil then
	    inst:RemoveComponent("trader")
	end		
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        if inst.components.tiletracker == nil then
		    inst:AddComponent("tiletracker")
			inst.components.tiletracker:Start()
			inst.components.tiletracker:SetOnWaterChangeFn(function(inst, onwater)
			    if onwater then
				    if inst.float_fx == nil then
		                inst.float_fx = SpawnPrefab("float_fx_front")
		                inst.float_fx.entity:SetParent(inst.entity)
		                inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)
                        inst.float_fx.Transform:SetPosition(0, 0.1, 0)	
                        inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)						
					end
					if inst.float_fx2 == nil then
		                inst.float_fx2 = SpawnPrefab("float_fx_back")
		                inst.float_fx2.entity:SetParent(inst.entity)
		                inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)	
                        inst.float_fx2.Transform:SetPosition(0, 0.1, 0)	
                        inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)						
					end
				else
	                if inst.float_fx ~= nil then
		                inst.float_fx:Remove()
		                inst.float_fx = nil
	                end
	                if inst.float_fx2 ~= nil then
		                inst.float_fx2:Remove()
		                inst.float_fx2 = nil
	                end					
				end
			end)
		end
	end
end)

local critter_kitten_skins =
{	
	{
		skin = 	"kittington_build",
		xml = 	"images/inventoryimages/critter_kitten_builder.xml",
		tex =	"critter_kitten_builder.tex",
		desc = 	"Classic", 
		cndesc = "经典",
	},
	{
		skin = 	"kitten_winter",
		xml = 	"images/inventoryimages/kitten_winter.xml",
		tex =	"kitten_winter.tex",
		desc = 	"Snowkit", 
		cndesc = "雪猫",
	},
}

AddPrefabPostInit("critter_kitten", function(inst)
    if inst.components.dsskins ~= nil then
	    inst.components.dsskins.skins = critter_kitten_skins
	end
    if inst.components.trader ~= nil then
	    inst:RemoveComponent("trader")
	end		
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        if inst.components.tiletracker == nil then
		    inst:AddComponent("tiletracker")
			inst.components.tiletracker:Start()
			inst.components.tiletracker:SetOnWaterChangeFn(function(inst, onwater)
			    if onwater then
				    if inst.float_fx == nil then
		                inst.float_fx = SpawnPrefab("float_fx_front")
		                inst.float_fx.entity:SetParent(inst.entity)
		                inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
                        inst.float_fx.Transform:SetPosition(0, 0.1, 0)
                        inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)						
					end
					if inst.float_fx2 == nil then
		                inst.float_fx2 = SpawnPrefab("float_fx_back")
		                inst.float_fx2.entity:SetParent(inst.entity)
		                inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)	
                        inst.float_fx2.Transform:SetPosition(0, 0.1, 0)	
                        inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)						
					end
				else
	                if inst.float_fx ~= nil then
		                inst.float_fx:Remove()
		                inst.float_fx = nil
	                end
	                if inst.float_fx2 ~= nil then
		                inst.float_fx2:Remove()
		                inst.float_fx2 = nil
	                end					
				end
			end)
		end
	end	
end)

local critter_dragonling_skins = 
{
	{
		skin = 	"dragonling_build",
		xml = 	"images/inventoryimages/critter_dragonling_builder.xml",
		tex =	"critter_dragonling_builder.tex",
		desc = 	"Classic", 
		cndesc = "经典",
	},
	{
		skin = 	"dragonling_winter",
		xml = 	"images/inventoryimages/dragonling_winter.xml",
		tex =	"dragonling_winter.tex",
		desc = 	"Polar Broodling", 
		cndesc = "极地蜂群卫士",
	},
    {
		skin = 	"dragonling_bat",
		xml = 	"images/inventoryimages/dragonling_bat.xml",
		tex =	"dragonling_bat.tex",
		desc = 	"Vampire Batling", 
		cndesc = "吸血小蝙蝠",
	},	
}

AddPrefabPostInit("critter_dragonling", function(inst)
    if inst.components.dsskins ~= nil then
	    inst.components.dsskins.skins = critter_dragonling_skins
	end
    if inst.components.trader ~= nil then
	    inst:RemoveComponent("trader")
	end		
	if inst.Physics ~= nil then
	    inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
	    inst.Physics:ClearCollisionMask()
		if IsDLCEnabled(PORKLAND_DLC) then
		    inst.Physics:CollidesWith(GetWorldCollision())
			inst.Physics:CollidesWith(COLLISION.INTWALL)
		elseif GetWorld() and GetWorld().prefab == "shipwrecked" then
		    inst.Physics:CollidesWith(COLLISION.GROUND)
		else
		    inst.Physics:CollidesWith(COLLISION.WORLD)
		end
	    inst.Physics:CollidesWith(COLLISION.FLYERS)
        inst.Physics:CollidesWith(COLLISION.CHARACTERS)
	    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	        inst.Physics:CollidesWith(COLLISION.WAVES)		
	    end
	end		
    if inst.components.sleeper ~= nil then
		inst:ListenForEvent("gotosleep", function(inst) 
			if inst.GetIsOnWater ~= nil and inst:GetIsOnWater() then
				if inst.float_fx == nil then
		            inst.float_fx = SpawnPrefab("float_fx_front")
		            inst.float_fx.entity:SetParent(inst.entity)
		            inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
                    inst.float_fx.Transform:SetPosition(0, 0.1, 0)
                    inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)						
				end
				if inst.float_fx2 == nil then
		            inst.float_fx2 = SpawnPrefab("float_fx_back")
		            inst.float_fx2.entity:SetParent(inst.entity)
		            inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)	
                    inst.float_fx2.Transform:SetPosition(0, 0.1, 0)	
                    inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)						
				end				
			end
		end)
		inst:ListenForEvent("onwakeup", function(inst) 
	        if inst.float_fx ~= nil then
		        inst.float_fx:Remove()
		        inst.float_fx = nil
	        end
	        if inst.float_fx2 ~= nil then
		        inst.float_fx2:Remove()
		        inst.float_fx2 = nil
	        end			
		end)
	end	
end)

local critter_glomling_skins =
{
	{
    	skin = 	"glomling_build",
		xml = 	"images/inventoryimages/critter_glomling_builder.xml",
		tex =	"critter_glomling_builder.tex",
		desc = 	"Classic", 
		cndesc = "经典",
	},
	{
		skin = 	"glomling_winter",
		xml = 	"images/inventoryimages/glomling_winter.xml",
		tex =	"glomling_winter.tex",
		desc = 	"Snowy Glomglom", 
		cndesc = "洁白无瑕的小格罗姆",
	},
}

AddPrefabPostInit("critter_glomling", function(inst)
    if inst.components.dsskins ~= nil then
	    inst.components.dsskins.skins = critter_glomling_skins
	end
    if inst.components.trader ~= nil then
	    inst:RemoveComponent("trader")
	end		
	if inst.Physics ~= nil then
	    inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
	    inst.Physics:ClearCollisionMask()
		if IsDLCEnabled(PORKLAND_DLC) then
		    inst.Physics:CollidesWith(GetWorldCollision())
			inst.Physics:CollidesWith(COLLISION.INTWALL)
		elseif GetWorld() and GetWorld().prefab == "shipwrecked" then
		    inst.Physics:CollidesWith(COLLISION.GROUND)
		else
		    inst.Physics:CollidesWith(COLLISION.WORLD)
		end
	    inst.Physics:CollidesWith(COLLISION.FLYERS)
        inst.Physics:CollidesWith(COLLISION.CHARACTERS)
	    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	        inst.Physics:CollidesWith(COLLISION.WAVES)		
	    end
	end
    if inst.components.sleeper ~= nil then
		inst:ListenForEvent("gotosleep", function(inst) 
			if inst.GetIsOnWater ~= nil and inst:GetIsOnWater() then
				if inst.float_fx == nil then
		            inst.float_fx = SpawnPrefab("float_fx_front")
		            inst.float_fx.entity:SetParent(inst.entity)
		            inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
                    inst.float_fx.Transform:SetPosition(0, 0.1, 0)
                    inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)						
				end
				if inst.float_fx2 == nil then
		            inst.float_fx2 = SpawnPrefab("float_fx_back")
		            inst.float_fx2.entity:SetParent(inst.entity)
		            inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)	
                    inst.float_fx2.Transform:SetPosition(0, 0.1, 0)	
                    inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)						
				end				
			end
		end)
		inst:ListenForEvent("onwakeup", function(inst) 
	        if inst.float_fx ~= nil then
		        inst.float_fx:Remove()
		        inst.float_fx = nil
	        end
	        if inst.float_fx2 ~= nil then
		        inst.float_fx2:Remove()
		        inst.float_fx2 = nil
	        end			
		end)
	end		
end)

local critter_perdling_skins =
{
	{
		skin = 	"perdling_build",
		xml = 	"images/inventoryimages/critter_perdling_builder.xml",
		tex =	"critter_perdling_builder.tex",
		desc = 	"Classic", 
		cndesc = "经典",
	},
	{
		skin = 	"perdling_winter",
		xml = 	"images/inventoryimages/perdling_winter.xml",
		tex =	"perdling_winter.tex",
		desc = 	"Penglet", 
		cndesc = "小企鹅",
	},	
	{
		skin = 	"perdling_rooster",
		xml = 	"images/inventoryimages/perdling_rooster.xml",
		tex =	"perdling_rooster.tex",
		desc = 	"Rooslet", 
		cndesc = "公鸡",
	},
}

AddPrefabPostInit("critter_perdling", function(inst)
    if inst.components.dsskins ~= nil then
	    inst.components.dsskins.skins = critter_perdling_skins
	end
    if inst.components.trader ~= nil then
	    inst:RemoveComponent("trader")
	end		
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        if inst.components.tiletracker == nil then
		    inst:AddComponent("tiletracker")
			inst.components.tiletracker:Start()
			inst.components.tiletracker:SetOnWaterChangeFn(function(inst, onwater)
			    if onwater then
				    if inst.float_fx == nil then
		                inst.float_fx = SpawnPrefab("float_fx_front")
		                inst.float_fx.entity:SetParent(inst.entity)
		                inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
                        inst.float_fx.Transform:SetPosition(0, 0.1, 0)
                        inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)						
					end
					if inst.float_fx2 == nil then
		                inst.float_fx2 = SpawnPrefab("float_fx_back")
		                inst.float_fx2.entity:SetParent(inst.entity)
		                inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)	
                        inst.float_fx2.Transform:SetPosition(0, 0.1, 0)	
                        inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)						
					end
				else
	                if inst.float_fx ~= nil then
		                inst.float_fx:Remove()
		                inst.float_fx = nil
	                end
	                if inst.float_fx2 ~= nil then
		                inst.float_fx2:Remove()
		                inst.float_fx2 = nil
	                end					
				end
			end)
		end
	end	
end)

AddComponentPostInit("edible", function(self)
    local _CollectUseActions = self.CollectUseActions
	
	function self:CollectUseActions(doer, target, actions, right)
	    if target ~= nil 
		    and target:HasTag("critter") 
			and target.components.eater ~= nil
			and target.components.eater:CanEat(self.inst)
			and ACTIONS.FEED ~= nil
			and not right
		then
		    table.insert(actions, ACTIONS.FEED)
		    return
		end
	    if _CollectUseActions then
		    _CollectUseActions(self, doer, target, actions, right)
		end	
	end

end)

AddPrefabPostInit("hatpropeller", function(inst)
	inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
    inst.persists = false
end)

AddPrefabPostInit("log", function(inst)
    if IsDLCEnabled(PORKLAND_DLC) and SaveGameIndex:IsModePorkland() then
	    inst.shelfart = "log_rainforest"
	end
end)

local function MakeInstShadowUpgradeable(inst)
    inst:AddTag("shadowupgradeable")
	inst:AddComponent("magicturnable")
end

AddPrefabPostInit("cane", MakeInstShadowUpgradeable)
AddPrefabPostInit("walkingstick", MakeInstShadowUpgradeable)
AddPrefabPostInit("telestaff", MakeInstShadowUpgradeable)
AddPrefabPostInit("nightsword", MakeInstShadowUpgradeable)
AddPrefabPostInit("armor_sanity", MakeInstShadowUpgradeable)
AddPrefabPostInit("umbrella", MakeInstShadowUpgradeable)
AddPrefabPostInit("grass_umbrella", MakeInstShadowUpgradeable)
AddPrefabPostInit("double_umbrellahat", MakeInstShadowUpgradeable)
AddPrefabPostInit("eyebrellahat", MakeInstShadowUpgradeable)
AddPrefabPostInit("armorwood", MakeInstShadowUpgradeable)
AddPrefabPostInit("footballhat", MakeInstShadowUpgradeable)
AddPrefabPostInit("pickaxe", MakeInstShadowUpgradeable)

AddPrefabPostInit("sunken_boat", function(inst)
    inst:AddTag("sunken_boat")
end)

AddPrefabPostInit("deflated_balloon_basket", function(inst)
    inst:AddTag("deflated_balloon_basket")
end)

AddPrefabPostInit("ancient_remnant", function(inst)
    inst:AddTag("fountain_altar_sacrifice")
end)

AddPrefabPostInit("elephantcactus", function(inst)
    inst:AddTag("fire_proof")
	inst:AddTag("cannot_be_reaped")
end)

local function OnHitWater_Hedge(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)
	end	    
end

local function OnHitLand_Hedge(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	    
end

local function FixFloatAnim(inst)
	if inst.GetIsOnWater ~= nil then
	    inst:RemoveComponent("floatable")
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater_Hedge)
	    inst.components.floatable:SetOnHitLandFn(OnHitLand_Hedge)
	end  
	if inst.components.inventoryitem ~= nil then
	    inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand_Hedge)
	end
end

AddPrefabPostInit("hedge_block_item", FixFloatAnim)
AddPrefabPostInit("hedge_cone_item", FixFloatAnim)
AddPrefabPostInit("hedge_layered_item", FixFloatAnim)

if IsDLCEnabled(PORKLAND_DLC) then

    local ACTIONS_RUMMAGE_fn = ACTIONS.RUMMAGE.fn
	
	ACTIONS.RUMMAGE.fn = function(act)
	    local targ = act.target or act.invobject
	    if targ ~= nil and (targ:HasTag("turfhat") or targ:HasTag("possess_with_dragon")) then
	        if act.doer and act.doer.HUD and targ.components.container then
		        if targ.components.container:IsOpen() then
			        targ.components.container:Close(act.doer)
			        act.doer:PushEvent("closecontainer", {container=targ})
		        else
			        act.doer:PushEvent("opencontainer", {container=targ})
			        targ.components.container:Open(act.doer)
		        end
		        return true
         	end		
		else
		    return ACTIONS_RUMMAGE_fn(act)
		end
	end

end

if not IsDLCEnabled(PORKLAND_DLC) then

    AddComponentPostInit("reticule", function(self)
	
        function self:SetValidateFn(fn)
	        self.validatefn = fn
        end

        function self:NeedsValidation()
	        return self.validatefn ~= nil
        end

        function self:Validate()
	        if self.validatefn then
		        return self.validatefn(self.inst)
	        end
	        return false
        end
	
	end)
	
    AddComponentPostInit("playercontroller", function(self)
	
	    local _OnEquip = self.OnEquip
		local _OnUnequip = self.OnUnequip
		local _OnUpdate = self.OnUpdate
		local _DoControllerAltAction = self.DoControllerAltAction
	
	    function self:UpdateControllerReticule()
	
	        local boatitem = self.inst.components.driver and self.inst.components.driver.vehicle and self.inst.components.driver.vehicle.components.container and self.inst.components.driver.vehicle.components.container:GetItemInBoatSlot(BOATEQUIPSLOTS.BOAT_LAMP)
	        local equipped_item = self.inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)

	        if not equipped_item or not equipped_item.components.reticule then
		        equipped_item = boatitem
	        end

	        local newreticule = equipped_item ~= nil and equipped_item.components.reticule or self.inst.components.reticule
	        if newreticule ~= self.reticule then
	            if self.reticule ~= nil then
	                self.reticule:DestroyReticule()
	                self.reticule = nil
	            end

	            if newreticule ~= nil and newreticule.reticule == nil then
	    	        if (newreticule:NeedsValidation() and newreticule:Validate()) or not newreticule:NeedsValidation() then
	    		        self.reticule = newreticule
	    		        newreticule:CreateReticule()
	                end
	            end
	        elseif self.reticule ~= nil and self.reticule:NeedsValidation() and not self.reticule:Validate() then
		        self.reticule:DestroyReticule()
		        self.reticule = nil
	        end
	
		end
		
		function self:OnEquip(data)
		
		    if self.inst.components.reticule ~= nil then

	            local controller_mode = TheInput:ControllerAttached()

	            if data.eslot == EQUIPSLOTS.HANDS or ( IsDLCEnabled(CAPY_DLC) and data.eslot == BOATEQUIPSLOTS.BOAT_LAMP ) then

                    local newreticule = data.item.components.reticule or self.inst.components.reticule
                    if newreticule ~= self.reticule then
                        if self.reticule ~= nil then
                            self.reticule:DestroyReticule()
                            self.reticule = nil
                        end
                        self.reticule = newreticule
                        if newreticule ~= nil and newreticule.reticule == nil and (newreticule.mouseenabled or controller_mode) then
                            newreticule:CreateReticule()
                        end
                    end
					
                end
			
			elseif _OnEquip then
			    _OnEquip(self, data)
			end
			
		end
			
		function self:OnUnequip(data)
		    
			if self.inst.components.reticule ~= nil then

                if data.eslot == EQUIPSLOTS.HANDS or ( IsDLCEnabled(CAPY_DLC) and data.eslot == BOATEQUIPSLOTS.BOAT_LAMP ) then
                    if self.reticule ~= nil then
                        self.reticule:DestroyReticule()
                        self.reticule = self.inst.components.reticule
                        if self.reticule ~= nil and self.reticule.reticule == nil and (self.reticule.mouseenabled or TheInput:ControllerAttached()) then
                            self.reticule:CreateReticule()
                        end
                    end
                end
			
			elseif _OnUnequip then
			    _OnUnequip(self, data)
			end
			
		end

--Does not need this, see modmain:
--[[		
		function self:OnUpdate(dt)
		
		    if _OnUpdate then
			    _OnUpdate(self, dt)
			end
		    if TheInput:ControllerAttached() then
			    self:UpdateControllerReticule()
			end
			
		end		
]]

        function self:GetGroundUseSpecialAction(position, right)

            position = position or
                (self.reticule ~= nil and self.reticule.targetpos) or
                (self.terraformer ~= nil and self.terraformer:GetPosition()) or
                (self.placer ~= nil and self.placer:GetPosition()) or
                (self.deployplacer ~= nil and self.deployplacer:GetPosition()) or
                self.inst:GetPosition()

                local tile = GetWorld().Map:GetTileAtPoint(position.x, position.y, position.z)
                local passable = tile ~= GROUND.IMPASSABLE

            return CanEntitySeePoint(self.inst, position:Get())
                and passable
                and self.inst.components.playeractionpicker:GetPointSpecialActions(position, nil, right)[1]
                or nil

        end
		
		function self:DoControllerAltAction()
		
            if _DoControllerAltAction then
			    _DoControllerAltAction(self)		
			end
			
			local r = self:GetGroundUseSpecialAction(nil, true)
			if r then
		        self:DoAction(r)
		        return			
			end
			
		end
	
	end)

end

local function IsPickableOrHackable(inst)
    if inst ~= nil then
	    if inst.components.pickable ~= nil and inst.components.pickable.canbepicked then
		    return true
		end		
		if inst.components.hackable ~= nil and inst.components.hackable.canbehacked then
		    return true
		end
	end
    return false
end

AddComponentPostInit("playercontroller", function(self)

    local _GetActionButtonAction = self.GetActionButtonAction
	
	function self:GetActionButtonAction()
	
	    local tool = self.inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
		local rad = self.directwalking and 3 or 6
	    local pickup = FindEntity(self.inst, 
		    rad, 
			function(guy) 
			    return guy.prefab ~= "limpetrock" and IsPickableOrHackable(guy)
		    end, 
		    nil, 
		    {"FX", "NOCLICK", "flower", "flower_rainforest", "seashell", "ancient_fountain_of_knowledge", "tubertree", "cannot_be_reaped"}
		)	
	    if self:IsEnabled() 
		    and not (self.inst.sg:HasStateTag("working") or self.inst.sg:HasStateTag("doing")) 
			and pickup ~= nil 
			and tool ~= nil 
			and tool.components.shadowreaper ~= nil 
		    and (self.inst.components.rider == nil or not self.inst.components.rider:IsRiding())
		then
		    return BufferedAction(self.inst, pickup, ACTIONS.SCYTHE, tool)
		end
	
	    if _GetActionButtonAction then
		    return _GetActionButtonAction(self)
		end
		
	end

end)

AddPrefabPostInit("chester", function(inst)

    if not GetPlayer():HasTag("charlie") then
	    return
	end

    local _MorphChester = inst.MorphChester
	
	inst.SHADOW_ON_OPEN = function(inst)
	    if inst.ChesterState == "SHADOW" and GetWorld().components.magiccontainerinventory then 
		    GetWorld().components.magiccontainerinventory:empty(inst) 		
		end 		
	end
	
	inst.SHADOW_ON_CLOSE = function(inst)
	    if inst.ChesterState == "SHADOW" and GetWorld().components.magiccontainerinventory then 
	        GetWorld().components.magiccontainerinventory:fill(inst) 
	    end	
	end
	
	inst.MorphChester = function(inst)
	    if _MorphChester then
		    _MorphChester(inst)
		end
	    inst:DoTaskInTime(0, function()
		    if inst.ChesterState == "SHADOW" then
			    if inst.components.container ~= nil then
	                inst.components.container.widgetanimbank = "ui_portal_shadow_3x4"
	                inst.components.container.widgetanimbuild = "ui_portal_shadow_3x4"
                end		
			end
		end)
	end
	
	inst:DoTaskInTime(0, function()
		if inst.ChesterState == "SHADOW" then
			if inst.components.container ~= nil then
	            inst.components.container.widgetanimbank = "ui_portal_shadow_3x4"
	            inst.components.container.widgetanimbuild = "ui_portal_shadow_3x4"		
            end				
		end
	end)	
	
	if inst.components.container ~= nil then
		inst.components.container.itemtestfn = function(inst, item, slot)
			return not item.components.leader and not item:HasTag("magician_hat") and not item:HasTag("irreplaceable")
		end	
	end
	
    inst:ListenForEvent("onopen", inst.SHADOW_ON_OPEN)
    inst:ListenForEvent("onclose", inst.SHADOW_ON_CLOSE)		

end)

if not IsDLCEnabled(CAPY_DLC) and not IsDLCEnabled(PORKLAND_DLC) then

    AddComponentPostInit("locomotor", function(self)

        self.inst:AddTag("locomotor")
 
    end)
	
end

local function AddShadowDestroyable(inst)
    inst:AddTag("destroyable_via_shadow")
end

AddPrefabPostInit("rock_obsidian", AddShadowDestroyable)

AddComponentPostInit("placer", function(self)

    local _OnUpdatePlacer = self.OnUpdate
	
	function self:OnUpdate(dt)
	
	    if _OnUpdatePlacer then
		    _OnUpdatePlacer(self, dt)
		end
		
        if GetPlayer().components.playercontroller ~= nil
		    and GetPlayer().components.playercontroller.placer ~= nil
		then
		    return
		end
		
		if (TheInput:GetWorldEntityUnderMouse() ~= nil and not TheInput:GetWorldEntityUnderMouse():HasTag("OnFloor") and not TheInput:GetWorldEntityUnderMouse():HasTag("ignoremouseover")) or TheInput:GetHUDEntityUnderMouse() ~= nil then
            self.inst:Hide()
            for i, v in ipairs(self.linked) do
                v:Hide()
            end
		else

	        local pt = self.selected_pos or Input:GetWorldPosition()
            local tile = GROUND.GRASS
            if GetWorld() and GetWorld().Map then
                tile = GetWorld().Map:GetTileAtPoint(pt:Get())
            end
            local onground = true
			if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
                onground = not GetWorld().Map:IsWater(tile)
			end

		    if ( self.can_build or not self.hide_on_invalid )
			    and ( not self.hide_on_ground or onground )
			    and ( self.invobject == nil or not ( self.invobject.components.deployable ~= nil and self.invobject.components.deployable.onlydeploybyplantkin and not GetPlayer():HasTag("plantkin") ) )
			then
                self.inst:Show()
                for i, v in ipairs(self.linked) do
                    v:Show()
                end
            end				
		end
	
	end	

end)

AddPrefabPostInit("buttercup_placer", function(inst)
    inst.AnimState:OverrideSymbol("bc_bud", "barnacle_plant_colour_swaps", "yellow_bc_bud")
    inst.AnimState:OverrideSymbol("bc_face", "barnacle_plant_colour_swaps", "yellow_bc_face")
    inst.AnimState:OverrideSymbol("bc_flower_petal", "barnacle_plant_colour_swaps", "yellow_bc_flower_petal")
    inst.AnimState:Hide("bud1")
    inst.AnimState:Hide("bud2")
    inst.AnimState:Hide("bud3")
end)

local function FixSkinModFXBlockPlacement(inst)
    inst:AddTag("NOBLOCK")
end

AddPrefabPostInit("cane_victorian_fx", FixSkinModFXBlockPlacement)
AddPrefabPostInit("cane_ancient_fx", FixSkinModFXBlockPlacement)
AddPrefabPostInit("cane_candy_fx", FixSkinModFXBlockPlacement)
AddPrefabPostInit("cane_sharp_fx", FixSkinModFXBlockPlacement)

AddComponentPostInit("seasonmanager", function(self)
    local _CheckValidWildfireStarter = self.CheckValidWildfireStarter 
	local _OnUpdate = self.OnUpdate
	if _CheckValidWildfireStarter ~= nil then
        function self:CheckValidWildfireStarter(obj)
	        local x, y, z = obj.Transform:GetWorldPosition()
            local endothermicmachine = TheSim:FindEntities(x, y, z, 30, {"endothermicmachine"})
	        if #endothermicmachine > 0 then
			    return false
			elseif _CheckValidWildfireStarter ~= nil then
	            return _CheckValidWildfireStarter(self, obj)
			end
        end
	end
	if _OnUpdate ~= nil then
        function self:OnUpdate(dt)
            if self.current_temperature ~= nil 
			    and self.current_temperature >= 80
			    and not self.inst.AlreadyCheckedDryUpPuddle
			then
			    self.inst:PushEvent("checkdryuppuddle")
			    self.inst.AlreadyCheckedDryUpPuddle = true
			end
			if _OnUpdate ~= nil then
	            _OnUpdate(self, dt)
			end
        end	
	end
end)

--The following changes are for puddles

if not IsDLCEnabled(PORKLAND_DLC) then
    AddComponentPostInit("inventory", function(self)
	    function self:IsItemNameEquipped(item_name)
            for k,v in pairs(self.equipslots) do
                if v.prefab == item_name then
                    return true
                end
            end 
            return false		
		end
	end)
end

AddComponentPostInit("moisture", function(self)
    local _GetMoistureRate = self.GetMoistureRate 
	local _GetDryingRate = self.GetDryingRate
	if _GetMoistureRate ~= nil then
        function self:GetMoistureRate()
	        local x, y, z = self.inst.Transform:GetWorldPosition()
            local puddle = TheSim:FindEntities(x, y, z, 2.5, {"puddle"}, {"puddle_disappear"})
			local lilypad = TheSim:FindEntities(x, y, z, 2.5, {"LilyPadForPuddle"})
	        if #puddle > 0 and #lilypad <= 0 
			    and (self.inst.components.inventory == nil or not self.inst.components.inventory:IsItemNameEquipped("raincoat")) 
			then
			    return 1
			elseif _GetMoistureRate ~= nil then
	            return _GetMoistureRate(self)
			end
        end
	end
	
	if _GetDryingRate ~= nil then
	    function self:GetDryingRate()
	        local x2, y2, z2 = self.inst.Transform:GetWorldPosition()
            local puddle2 = TheSim:FindEntities(x2, y2, z2, 2.5, {"puddle"}, {"puddle_disappear"})
			local lilypad2 = TheSim:FindEntities(x2, y2, z2, 2.5, {"LilyPadForPuddle"})
	        if #puddle2 > 0 and #lilypad2 <= 0 
			    and (self.inst.components.inventory == nil or not self.inst.components.inventory:IsItemNameEquipped("raincoat")) 
			then
			    return 0
			elseif _GetDryingRate ~= nil then
	            return _GetDryingRate(self)
			end		
		end
	end
end)

AddComponentPostInit("inventoryitem", function(self)
    local _OnHitGround = self.OnHitGround
	if _OnHitGround ~= nil then
	    function self:OnHitGround(vely)
		    local x, y, z = self.inst.Transform:GetWorldPosition()
            local puddle = TheSim:FindEntities(x, y, z, 2.5, {"puddle"}, {"puddle_disappear"})
			local lilypad = TheSim:FindEntities(x, y, z, 2.5, {"LilyPadForPuddle"})			
			local OnPuddle = #puddle > 0 and #lilypad <= 0			
			if OnPuddle then
			    self.inst:RemoveTag("falling")
			    if self.onwater ~= OnPuddle then
					self.inst:PushEvent("onhitground", OnPuddle)
				    if self.OnHitWater ~= nil then
				        self:OnHitWater()
				    end
				end
				self.onwater = OnPuddle
                if not self.inst.Physics then
                    self.inst:StopUpdatingComponent(self)
                end				
            elseif _OnHitGround ~= nil then
			    _OnHitGround(self, vely)
			end
		end
    end
end)

AddComponentPostInit("floodable", function(self)
    local _OnUpdate = self.OnUpdate
	if _OnUpdate ~= nil then
	    function self:OnUpdate(dt)
		    local x, y, z = self.inst.Transform:GetWorldPosition()
            local potholepuddle = TheSim:FindEntities(x, y, z, 2.5, {"deeppuddle"}, {"puddle_disappear"})
			local checktagsuccess = false
			for k,v in pairs(potholepuddle) do
			    if v.MechsForPuddles ~= nil then
				    for i,j in pairs(v.MechsForPuddles) do
					    if self.inst.prefab == j then
						    checktagsuccess = true
						    break
						end
					end
				end
			end
			
			if #potholepuddle > 0 and checktagsuccess then
			    if not self.flooded then
                    self.flooded = true 
                    self.inst:AddTag("flooded")
                    if self.onStartFlooded then 
                        self.onStartFlooded(self.inst)
                    end				
				end
                if self.flooded then 
                    if self.floodEffect then 
                        self.timeToEffect = self.timeToEffect - dt
                        if self.timeToEffect <= 0 then 
                            local fx = SpawnPrefab(self.floodEffect)
                            if fx then 
                                local pt = self.inst:GetPosition()
                                fx.Transform:SetPosition(pt.x, pt.y, pt.z)
                            end 
                            if self.floodSound and self.inst.SoundEmitter then
                                self.inst.SoundEmitter:PlaySound(self.floodSound)
                            end
                            self.timeToEffect = self.fxPeriod
                        end 
                    end 
                end
			elseif _OnUpdate ~= nil then
			    _OnUpdate(self, dt)
			end
		end
    end
end)

AddComponentPostInit("tiletracker", function(self)
    local _OnUpdate = self.OnUpdate
    if _OnUpdate ~= nil then
	    function self:OnUpdate(dt)
		    local onpuddle = IsPointNearNonLilyPadPuddle(self.inst:GetPosition(), 2.5)
		    if onpuddle then
		        if self.onwaterchangefn or self.inst:HasTag("amphibious") then
			        local onwater = onpuddle
			        if onwater ~= self._onwater then
				        if self.onwaterchangefn then 
					        self.onwaterchangefn(self.inst, onwater)
				        end 
				        if self.inst:HasTag("amphibious") then 
					        if onwater then 
						        self.inst:AddTag("aquatic")
				        	else
						        self.inst:RemoveTag("aquatic")
					        end 
				        end 
			        end
			        self._onwater = onwater 
	         	end
			elseif _OnUpdate ~= nil then
			    if self._onwater then
				    if self.onwaterchangefn then 
				        self.onwaterchangefn(self.inst, false)
			        end 
				    if self.inst:HasTag("amphibious") then 
					    self.inst:RemoveTag("aquatic")
				    end 				
				    self._onwater = nil 
			    end
			    _OnUpdate(self, dt)
			end
		end
	end
end)

AddPrefabPostInit("buoy", function(inst)
    inst:AddTag("buoy")
end)


