
local function CanPlantAtPoint(inst)
    if inst ~= nil then
	    local x, y, z = inst.Transform:GetWorldPosition()
	    local tile = GetWorld().Map:GetTileAtPoint(x, y, z)
		if inst.IsPosSurroundedByLand ~= nil and not inst:IsPosSurroundedByLand(x, y, z, 2) then
		    return false
		end
        if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	        if tile == GROUND.ASH or tile == GROUND.VOLCANO 
		        or tile == GROUND.VOLCANO_ROCK or tile == GROUND.BRICK_GLOW
		    	or tile == GROUND.VOLCANO_LAVA or tile == GROUND.SNAKESKIN
		    then
		        return false
		    end
	    end
	    if IsDLCEnabled(PORKLAND_DLC) then
	        if tile == GROUND.FOUNDATION or tile == GROUND.COBBLEROAD
		        or tile == GROUND.GASJUNGLE or tile == GROUND.PIGRUINS
			    or tile == GROUND.PIGRUINS_NOCANOPY or tile == GROUND.BEARDRUG or tile == GROUND.INTERIOR
	    	then                                                               
		        return false
	    	end
    	end
        if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and tile ~= GROUND.ROCKY and tile ~= GROUND.ROAD
		    and tile ~= GROUND.UNDERROCK and tile ~= GROUND.WOODFLOOR and tile ~= GROUND.CARPET and tile ~= GROUND.CHECKER
            and tile < GROUND.UNDERGROUND
    	then
		    return true
	    end
	    return false		
	else
	    return false
	end
end

local function CanSpawnLandObjAtPoint(inst, excludecraftedturfs, excludeinteriors)
    if inst ~= nil then
	    local x, y, z = inst.Transform:GetWorldPosition()
	    local tile = GetWorld().Map:GetTileAtPoint(x, y, z)
		if inst.IsPosSurroundedByLand ~= nil and not inst:IsPosSurroundedByLand(x, y, z, 2) then
		    return false
		end
        if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	        if tile == GROUND.VOLCANO_LAVA then
		        return false
		    end
			if excludecraftedturfs and tile == GROUND.SNAKESKIN then
		        return false
			end
	    end
	    if IsDLCEnabled(PORKLAND_DLC) then
	        if excludeinteriors and tile == GROUND.INTERIOR	then                                                               
		        return false
	    	end
			if excludecraftedturfs 
			    and (tile == GROUND.BEARDRUG 
				    or tile == GROUND.FOUNDATION 
			    	or tile == GROUND.COBBLEROAD 
					or tile == GROUND.LAWN 
					or tile == GROUND.FIELDS)	
			then
			    return false
			end			
    	end
		if excludecraftedturfs then
		    if tile == GROUND.ROAD 
			    or tile == GROUND.WOODFLOOR 
				or tile == GROUND.CARPET 
				or tile == GROUND.CHECKER 		
			then
			    return false
			end
		end
        if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and tile < GROUND.UNDERGROUND then
		    return true
	    end
	    return false		
	else
	    return false
	end

end

local function IsDeployPointClear(inst, min_spacing)
    if inst ~= nil then
	    local x, y, z = inst.Transform:GetWorldPosition()	
	    local ents = TheSim:FindEntities(x, y, z, 4, nil, {"NOBLOCK", "FX", "player"}) 
        local minspacing = min_spacing or 2
	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and v:GetTimeAlive() >= 3 and not v.components.placer and v.parent == nil then
				if distsq(Vector3(v.Transform:GetWorldPosition()), Vector3(x, y, z)) < minspacing*minspacing then
					return false
				end
			end
		end		
		return true
	else
	    return false
	end
end

AddPrefabPostInit("smallbird", function(inst)
    if GetPlayer() and GetPlayer().prefab == "charlie" then
	    if inst.components.combat ~= nil and inst.components.lootdropper ~= nil then
		    inst.components.combat.onkilledbyother = function(inst, attacker)
			    if attacker ~= nil and attacker:HasTag("supercritterexpert") then
                    inst.components.lootdropper:SetLoot({"smallmeat", "feather_crow"})
				    local x, y, z = inst.Transform:GetWorldPosition()
			    	local tile = GetWorld().Map:GetTileAtPoint(x, y, z)
				    local IsOnRockTile = tile == GROUND.ROCKY or ((IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)) and tile == GROUND.MAGMAFIELD)
                    if IsOnRockTile and IsDeployPointClear(inst) then
			            SpawnPrefab("tallbirdnest").Transform:SetPosition(x, y, z)
		                SpawnPrefab("small_puff").Transform:SetPosition(x, y, z)
                    end						
                end			
			end
		end	
	end
end)

AddPrefabPostInit("catcoon", function(inst)
    if GetPlayer() and GetPlayer().prefab == "charlie" then
	    if inst.components.combat ~= nil and inst.components.lootdropper ~= nil then
		    inst.components.combat.onkilledbyother = function(inst, attacker)
			    if attacker ~= nil and attacker:HasTag("supercritterexpert") then
                    inst.components.lootdropper:SetLoot({"meat", "coontail"})
	                if CanSpawnLandObjAtPoint(inst) and IsDeployPointClear(inst, 3) then
			            SpawnPrefab("catcoonden").Transform:SetPosition(inst.Transform:GetWorldPosition())
		                SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
                    end	
                end							
			end
		end	
	end
end)

AddPrefabPostInit("crab", function(inst)
    if GetPlayer() and GetPlayer().prefab == "charlie" then
	    if inst.components.combat ~= nil and inst.components.lootdropper ~= nil then
		    inst.components.combat.onkilledbyother = function(inst, attacker)
			    if inst.iscrab and attacker ~= nil and attacker:HasTag("supercritterexpert") then
                    inst.components.lootdropper:SetLoot({"fish_raw_small", "fish_raw_small"})
                    if CanSpawnLandObjAtPoint(inst, false, true) and IsDeployPointClear(inst) then
			            SpawnPrefab("crabhole").Transform:SetPosition(inst.Transform:GetWorldPosition())
		                SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
                    end	
                end						
			end
		end	
	end
end)

AddPrefabPostInit("pog", function(inst)
    if GetPlayer() and GetPlayer().prefab == "charlie" then
	    if inst.components.combat ~= nil and inst.components.lootdropper ~= nil then
		    inst.components.combat.onkilledbyother = function(inst, attacker)
			    if attacker ~= nil and attacker:HasTag("supercritterexpert") then
                    inst.components.lootdropper:SetLoot({"smallmeat", "smallmeat", "clawpalmtree_sapling"})
                end					
			end
		end	
	end
end)


AddPrefabPostInit("perd", function(inst)
    if GetPlayer() and GetPlayer().prefab == "charlie" then
	    if inst.components.combat ~= nil and inst.components.lootdropper ~= nil then
		    inst.components.combat.onkilledbyother = function(inst, attacker)
			    if attacker ~= nil and attacker:HasTag("supercritterexpert") then
                    inst.components.lootdropper:SetLoot({"drumstick", "drumstick", "drumstick", "drumstick"})
					if CanPlantAtPoint(inst) and IsDeployPointClear(inst) then
					    if math.random() < 0.5 then
                            SpawnPrefab("berrybush").Transform:SetPosition(inst.Transform:GetWorldPosition())
					    else
					        SpawnPrefab("berrybush2").Transform:SetPosition(inst.Transform:GetWorldPosition())
				    	end
		                SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())		
                    end										
                end					
			end
		end	
	end
end)

AddStategraphState("wilson", 
    State{ 
        name = "masterdodge",
        tags = {"busy", "doing", "canrotate", "no_stun", "nomorph", "nodangle"},
	    onenter = function(inst)
		    inst:PerformBufferedAction()
		end, 
	}
)

AddStategraphState("wilsonboating", 
    State{ 
        name = "masterdodge",
        tags = {"busy", "doing", "canrotate", "no_stun", "nomorph", "nodangle"},
	    onenter = function(inst)
		    inst:PerformBufferedAction()
		end, 
	}
)

local action_twenty_sec_with_callback = State 
{
    name = "dotwentysecactionwithcallback",
	tags = {"doing", "busy"},
    onenter = function(inst)
        inst.sg:SetTimeout(20)
        inst.components.locomotor:Stop()
        inst.SoundEmitter:PlaySound("dontstarve/wilson/make_trap", "make")
        
        inst.AnimState:PlayAnimation("build_pre")
        inst.AnimState:PushAnimation("build_loop", true)        
    end,
	
    timeline =
    {
        TimeEvent(4 * FRAMES, function(inst)
            inst.sg:RemoveStateTag("busy")
        end),
		
        TimeEvent(1, function(inst)
		    local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil and buffaction.target ~= nil and buffaction.target.components.charliecoffeemaker ~= nil then
			    buffaction.target.components.charliecoffeemaker:TryToMakeCoffee(buffaction.doer)
			else
			    inst.sg:GoToState("idle")
			end
        end),
		
        TimeEvent(3, function(inst)
		    local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil and buffaction.target ~= nil and buffaction.target.components.charliecoffeemaker ~= nil then
			    buffaction.target.components.charliecoffeemaker:TryToMakeCoffee(buffaction.doer)
			else
			    inst.sg:GoToState("idle")
			end
        end),
		
        TimeEvent(5, function(inst)
		    local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil and buffaction.target ~= nil and buffaction.target.components.charliecoffeemaker ~= nil then
			    buffaction.target.components.charliecoffeemaker:TryToMakeCoffee(buffaction.doer)
			else
			    inst.sg:GoToState("idle")
			end
        end),
		
        TimeEvent(7, function(inst)
		    local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil and buffaction.target ~= nil and buffaction.target.components.charliecoffeemaker ~= nil then
			    buffaction.target.components.charliecoffeemaker:TryToMakeCoffee(buffaction.doer)
			else
			    inst.sg:GoToState("idle")
			end
        end),
		
        TimeEvent(9, function(inst)
		    local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil and buffaction.target ~= nil and buffaction.target.components.charliecoffeemaker ~= nil then
			    buffaction.target.components.charliecoffeemaker:TryToMakeCoffee(buffaction.doer)
			else
			    inst.sg:GoToState("idle")
			end
        end),
		
        TimeEvent(11, function(inst)
		    local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil and buffaction.target ~= nil and buffaction.target.components.charliecoffeemaker ~= nil then
			    buffaction.target.components.charliecoffeemaker:TryToMakeCoffee(buffaction.doer)
			else
			    inst.sg:GoToState("idle")
			end
        end),
		
        TimeEvent(13, function(inst)
		    local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil and buffaction.target ~= nil and buffaction.target.components.charliecoffeemaker ~= nil then
			    buffaction.target.components.charliecoffeemaker:TryToMakeCoffee(buffaction.doer)
			else
			    inst.sg:GoToState("idle")
			end
        end),
		
        TimeEvent(15, function(inst)
		    local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil and buffaction.target ~= nil and buffaction.target.components.charliecoffeemaker ~= nil then
			    buffaction.target.components.charliecoffeemaker:TryToMakeCoffee(buffaction.doer)
			else
			    inst.sg:GoToState("idle")
			end
        end),
		
        TimeEvent(17, function(inst)
		    local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil and buffaction.target ~= nil and buffaction.target.components.charliecoffeemaker ~= nil then
			    buffaction.target.components.charliecoffeemaker:TryToMakeCoffee(buffaction.doer)
			else
			    inst.sg:GoToState("idle")
			end
        end),
		
        TimeEvent(19, function(inst)
		    local buffaction = inst:GetBufferedAction()
            if buffaction ~= nil and buffaction.target ~= nil and buffaction.target.components.charliecoffeemaker ~= nil then
			    buffaction.target.components.charliecoffeemaker:TryToMakeCoffee(buffaction.doer)
			else
			    inst.sg:GoToState("idle")
			end
        end),
	},

    ontimeout = function(inst)
        inst.SoundEmitter:KillSound("make")
        inst.AnimState:PlayAnimation("build_pst")
        inst:PerformBufferedAction()
    end,

    events =
    {
        EventHandler("animqueueover", function(inst)
            if inst.AnimState:AnimDone() then
                inst.sg:GoToState("idle")
            end
        end),
    },
        
    onexit = function(inst)
        inst.SoundEmitter:KillSound("make")
    end,	
}


local action_valentinedayfish = State 
{
    name = "valentinedayfish",
	tags = {"doing", "busy"},
    onenter = function(inst)
	    if inst.components.hunger ~= nil and not inst.components.hunger:IsStarving() then
            inst.components.locomotor:Stop()
            inst.AnimState:Hide("RIPPLES")
            inst.AnimState:PlayAnimation("fishing_pre") --24
            inst.AnimState:PushAnimation("fishing_idle") --38
            inst.AnimState:PushAnimation("fishing_idle") 
            inst.AnimState:PushAnimation("bite_heavy_pre") --4
            inst.AnimState:PushAnimation("bite_heavy_loop") --13
            inst.AnimState:PushAnimation("bite_heavy_loop")
            inst.AnimState:PushAnimation("fish_catch", false) --13
		else
		    if inst.components.talker ~= nil then
			    inst.components.talker:Say(STRINGS.CHARLIE_ACTION_NEED_FOOD)
			end
		    inst.sg:GoToState("idle")
		end
    end,
	
    timeline = 
    {
        TimeEvent(13*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/common/fishingpole_cast") end),
            
        TimeEvent(24*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/common/fishingpole_baitsplash") end),

        TimeEvent(100*FRAMES, function(inst) 
            inst.SoundEmitter:PlaySound("dontstarve/common/fishingpole_fishinwater", "splash")
            inst.SoundEmitter:PlaySound("dontstarve/common/fishingpole_strain", "strain")
        end),
            
        TimeEvent(130*FRAMES, function(inst) 
            inst.SoundEmitter:KillSound("splash")
            inst.SoundEmitter:KillSound("strain")		
        end),

        TimeEvent(138*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/common/fishingpole_fishcaught") end),

        TimeEvent(143*FRAMES, function(inst) 
		    inst.sg:RemoveStateTag("busy") 
            inst:PerformBufferedAction()
		end),
    },

    events =
    {
        EventHandler("animqueueover", function(inst)
            inst.sg:GoToState("idle")
        end),
    },
	
}

AddStategraphState("wilson", action_twenty_sec_with_callback)
AddStategraphState("wilsonboating", action_twenty_sec_with_callback)

AddStategraphState("wilson", action_valentinedayfish)
AddStategraphState("wilsonboating", action_valentinedayfish)

local masterdodge = 
{
    id = "MASTERDODGE",
    str = STRINGS.ACTIONS.MASTERDODGE,
    fn = function(act)  --act.target, act.invobject, act.doer
	    local doer = act.doer
		doer.masterdodging = true 
		doer.masterdodging2 = true
		if doer.SoundEmitter ~= nil then
	        doer.SoundEmitter:PlaySound("dontstarve/movement/bodyfall_dirt", "bodyfall")
		end
        if doer.components.locomotor ~= nil then
		    doer.components.locomotor:Stop()
		end
		if doer.AnimState ~= nil then
            doer.AnimState:PlayAnimation("jump") 
			doer.AnimState:SetTime(0.25) 
		end
	    if doer.components.health ~= nil then
		    doer.components.health:SetInvincible(true)
	    end
		if doer.components.driver and doer.components.driver.vehicle and doer.components.driver.vehicle.components.boathealth then
		    doer.components.driver.vehicle.components.boathealth:SetInvincible(true)
		end	
	    if doer.components.driver ~= nil and doer.components.driver.vehicle then
		    doer.components.driver.vehicle:Show()
		end
	    doer:DoTaskInTime(0.25, function() 
		    doer.masterdodging = nil 
		    if doer.components.health ~= nil then
			    doer.components.health:SetInvincible(false)
		    end
			if doer.components.driver and doer.components.driver.vehicle and doer.components.driver.vehicle.components.boathealth then
		        doer.components.driver.vehicle.components.boathealth:SetInvincible(false)
		    end
			if doer.SoundEmitter ~= nil then
		        doer.SoundEmitter:PlaySound("dontstarve/movement/bodyfall_dirt", "bodyfall")
			end
	        if doer.components.driver ~= nil and doer.components.driver.vehicle then
		        doer.components.driver.vehicle:Hide()
		    end			
			if doer.sg ~= nil then
			    doer.sg:GoToState("idle") 
			end
		end)
		
		doer:DoTaskInTime(0.75, function() 
		    doer.masterdodging2 = nil
		end)
		
		return true	
    end,
    distance = 99,
    rmb = true,
    instant = false,
    mount_enabled = false,
    crosseswaterboundary = true, 	
    priority = 12,
}

AddAction(masterdodge) 

local shadowembercast = 
{
    id = "SHADOWEMBERCAST",
    str = STRINGS.ACTIONS.SHADOWEMBERCAST,
    fn = function(act)  --act.target, act.invobject, act.doer
	    local doer = act.doer
        if doer ~= nil 
		    and doer.emberguru 
		    and doer.components.inventory ~= nil 
		then
		    local item = doer.components.inventory:FindItem(function(item) 
		        if item ~= nil 
			        and item.prefab == "charlie_exp" 
				    and item.components.machine ~= nil
                    and item.components.machine:IsOn() 
		     	then
			        return true
		     	end
		        return false
		    end) 
			
			if item ~= nil then
			    if item.components.stackable ~= nil then
				    item.components.stackable:Get():Remove()
				else
				    item:Remove()
				end
			else
			    return false	
			end
            if doer.SoundEmitter ~= nil then
                doer.SoundEmitter:PlaySound("dontstarve/wilson/fireball_explo")
	        end 
			if IsPassableAtPoint(act.pos.x, act.pos.y, act.pos.z) and (doer.IsPosSurroundedByLand == nil or doer:IsPosSurroundedByLand(act.pos.x, act.pos.y, act.pos.z, 3)) then
			    local shadowfire = SpawnPrefab("fire") 
		    	shadowfire.Transform:SetPosition(act.pos.x, act.pos.y, act.pos.z)
                shadowfire.AnimState:SetMultColour(0, 0, 0, 0.6)
				shadowfire:AddTag("NOCLICK") 
			    shadowfire:AddTag("NOBLOCK") 
				shadowfire.components.firefx:SetLevel(4) 
				shadowfire.persists = false 
				shadowfire:DoTaskInTime(10, shadowfire.Remove) 
			end 
			for k,v in pairs(TheSim:FindEntities(act.pos.x, act.pos.y, act.pos.z, 5, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "shadow", "notarget", "player", "companion", "wall", "structure"})) do
			    if v ~= nil and v.prefab ~= nil and v.prefab ~= "" and v:IsValid() and v.entity:IsVisible() then
				    if v.components.freezable ~= nil then
					    v.components.freezable:Freeze(10) 
					end 		
					if doer.superemberguru and v.components.combat ~= nil and v.components.health ~= nil then
					    v.takenshadowfiredamagetask = v:DoPeriodicTask(1/30, function() 
						    if v.components.freezable ~= nil then 
						        v.components.freezable:Freeze(10) 
							end 
							if v.components.health ~= nil and not v.components.health:IsDead() then 
						        v.components.combat:GetAttacked(doer, 20) 
						    	v.components.health:SetVal(v.components.health.currenthealth - 30, doer.prefab) --shadow planar damage 
							elseif v.takenshadowfiredamagetask ~= nil then 
							    v.takenshadowfiredamagetask:Cancel()
							    v.takenshadowfiredamagetask = nil 							
							end 
						end) 
						v:DoTaskInTime(1.5, function()
						    if v.takenshadowfiredamagetask ~= nil then
							    v.takenshadowfiredamagetask:Cancel()
							    v.takenshadowfiredamagetask = nil 
							end
						    if v.components.freezable ~= nil then 
						        v.components.freezable:Freeze(8.5) 
							end							
						end)
					end
				end
			end 
			if doer:HasTag("shadow_form_on") 
		        and doer.shadowmanipulation 
			    and math.random() < (doer.supershadowmanipulation and 0.15 or 0.1) 
			then
		        if math.random() < 0.01 then
                    if doer.components.inventory ~= nil then
		                doer.components.inventory:GiveItem(SpawnPrefab("horrorfuel"), nil, act.pos) 
			    	else
				        SpawnPrefab("small_puff").Transform:SetPosition(doer.Transform:GetWorldPosition())
				        SpawnPrefab("horrorfuel").Transform:SetPosition(doer.Transform:GetWorldPosition())
		            end				
		    	else
                    if doer.components.inventory ~= nil then
		                doer.components.inventory:GiveItem(SpawnPrefab("nightmarefuel"), nil, act.pos) 
				    else
				        SpawnPrefab("small_puff").Transform:SetPosition(doer.Transform:GetWorldPosition())
				        SpawnPrefab("nightmarefuel").Transform:SetPosition(doer.Transform:GetWorldPosition())
		            end				
		    	end					
			end
			return true	
		else	
		   	return false	
		end		
    end,
    distance = 50,
    rmb = true,
    instant = false,
    mount_enabled = true,
    crosseswaterboundary = true, 	
    priority = 14,
}

AddAction(shadowembercast) 

local shadowfuelcast = 
{
    id = "SHADOWFUELCAST",
    str = STRINGS.ACTIONS.SHADOWFUELCAST,
    fn = function(act)  --act.target, act.invobject, act.doer
	    local doer = act.doer
        if doer ~= nil 
		    and doer.supershadowmanipulation 
            and doer:HasTag("shadow_form_on") 			
		    and doer.components.inventory ~= nil 
		then
		    local item = doer.components.inventory:FindItem(function(item) 
		        if item ~= nil 
			        and (item.prefab == "nightmarefuel" or item.prefab == "horrorfuel")
				    and item.components.machine ~= nil
                    and item.components.machine:IsOn() 
		     	then
			        return true
		     	end
		        return false
		    end) 
			
			if item == nil then 
			    return false
			end 
			
            if doer.SoundEmitter ~= nil then
                doer.SoundEmitter:PlaySound("dontstarve/wilson/fireball_explo")
	        end 
			
            local rot = 36 * math.pi / 180
            local sin_rot = math.sin(rot)
            local cos_rot = math.cos(rot)
	        local light_x1 = 0.8
	        local light_z1 = -2.7
		
	        local light_x2 = 2.7
	        local light_z2 = -0.8
		
	        local light_x3 = -0.8
	        local light_z3 = 2.7	
		
	        local light_x4 = -2.7
	        local light_z4 = 0.8 		
			
			local x = act.pos.x
			local y = act.pos.y
			local z = act.pos.z 
			
			local x2 = x + light_x1 * cos_rot - light_z1 * sin_rot
			local z2 = z + light_z1 * cos_rot + light_x1 * sin_rot
			local x3 = x + light_x2 * cos_rot - light_z2 * sin_rot
			local z3 = z + light_z2 * cos_rot + light_x2 * sin_rot
			local x4 = x + light_x3 * cos_rot - light_z3 * sin_rot
			local z4 = z + light_z3 * cos_rot + light_x3 * sin_rot
			local x5 = x + light_x4 * cos_rot - light_z4 * sin_rot
			local z5 = z + light_z4 * cos_rot + light_x4 * sin_rot
			
			if IsPassableAtPoint(x, 0, z) and (doer.IsPosSurroundedByLand == nil or doer:IsPosSurroundedByLand(x, 0, z, 3)) then 	
			    local shadowfire = item ~= nil and item.prefab == "horrorfuel" and SpawnPrefab("character_fire") or SpawnPrefab("fire") 
		    	shadowfire.Transform:SetPosition(x, 0, z)
                shadowfire.AnimState:SetMultColour(0, 0, 0, 0.6)
				shadowfire:AddTag("NOCLICK") 
			    shadowfire:AddTag("NOBLOCK") 
				shadowfire.components.firefx:SetLevel(item ~= nil and item.prefab == "horrorfuel" and 3 or 4, true) 
				shadowfire.persists = false 
				shadowfire:DoTaskInTime(10, shadowfire.Remove) 
			end 
			
			if IsPassableAtPoint(x2, 0, z2) and (doer.IsPosSurroundedByLand == nil or doer:IsPosSurroundedByLand(x2, 0, z2, 3)) then 	
			    local shadowfire2 = item ~= nil and item.prefab == "horrorfuel" and SpawnPrefab("character_fire") or SpawnPrefab("fire") 
		    	shadowfire2.Transform:SetPosition(x2, 0, z2)
                shadowfire2.AnimState:SetMultColour(0, 0, 0, 0.6)
				shadowfire2:AddTag("NOCLICK") 
			    shadowfire2:AddTag("NOBLOCK") 
				shadowfire2.components.firefx:SetLevel(item ~= nil and item.prefab == "horrorfuel" and 3 or 4, true) 
				shadowfire2.persists = false 
				shadowfire2:DoTaskInTime(10, shadowfire2.Remove) 
			end 

			if IsPassableAtPoint(x3, 0, z3) and (doer.IsPosSurroundedByLand == nil or doer:IsPosSurroundedByLand(x3, 0, z3, 3)) then 	
			    local shadowfire3 = item ~= nil and item.prefab == "horrorfuel" and SpawnPrefab("character_fire") or SpawnPrefab("fire") 
		    	shadowfire3.Transform:SetPosition(x3, 0, z3)
                shadowfire3.AnimState:SetMultColour(0, 0, 0, 0.6)
				shadowfire3:AddTag("NOCLICK") 
			    shadowfire3:AddTag("NOBLOCK") 
				shadowfire3.components.firefx:SetLevel(item ~= nil and item.prefab == "horrorfuel" and 3 or 4, true) 
				shadowfire3.persists = false 
				shadowfire3:DoTaskInTime(10, shadowfire3.Remove) 
			end 

			if IsPassableAtPoint(x4, 0, z4) and (doer.IsPosSurroundedByLand == nil or doer:IsPosSurroundedByLand(x4, 0, z4, 3)) then 	
			    local shadowfire4 = item ~= nil and item.prefab == "horrorfuel" and SpawnPrefab("character_fire") or SpawnPrefab("fire") 
		    	shadowfire4.Transform:SetPosition(x4, 0, z4)
                shadowfire4.AnimState:SetMultColour(0, 0, 0, 0.6)
				shadowfire4:AddTag("NOCLICK") 
			    shadowfire4:AddTag("NOBLOCK") 
				shadowfire4.components.firefx:SetLevel(item ~= nil and item.prefab == "horrorfuel" and 3 or 4, true) 
				shadowfire4.persists = false 
				shadowfire4:DoTaskInTime(10, shadowfire4.Remove) 
			end 

			if IsPassableAtPoint(x5, 0, z5) and (doer.IsPosSurroundedByLand == nil or doer:IsPosSurroundedByLand(x5, 0, z5, 3)) then 	
			    local shadowfire5 = item ~= nil and item.prefab == "horrorfuel" and SpawnPrefab("character_fire") or SpawnPrefab("fire") 
		    	shadowfire5.Transform:SetPosition(x5, 0, z5)
                shadowfire5.AnimState:SetMultColour(0, 0, 0, 0.6)
				shadowfire5:AddTag("NOCLICK") 
			    shadowfire5:AddTag("NOBLOCK") 
				shadowfire5.components.firefx:SetLevel(item ~= nil and item.prefab == "horrorfuel" and 3 or 4, true) 
				shadowfire5.persists = false 
				shadowfire5:DoTaskInTime(10, shadowfire5.Remove) 
			end 					
			
			for k,v in pairs(TheSim:FindEntities(x, y, z, 5, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "shadow", "notarget", "player", "companion", "wall", "structure"})) do
			    if v ~= nil and v.prefab ~= nil and v.prefab ~= "" and v:IsValid() and v.entity:IsVisible() then
					if v.components.combat ~= nil and v.components.health ~= nil then
					    v.components.combat:GetAttacked(doer, item ~= nil and item.prefab == "horrorfuel" and 400 or 150)
						v.components.health:SetVal(v.components.health.currenthealth - (item ~= nil and item.prefab == "horrorfuel" and 600 or 200), doer.prefab) --shadow planar damage
					end
				end
			end 
			
			if item ~= nil then
			    if item.components.stackable ~= nil then
				    item.components.stackable:Get():Remove()
				else
				    item:Remove()
				end 
			end			

			return true	
		else	
		   	return false	
		end		
    end,
    distance = 60,
    rmb = true,
    instant = false,
    mount_enabled = true,
    crosseswaterboundary = true, 	
    priority = 15,
}

AddAction(shadowfuelcast) 

local shadowtoss = 
{
    id = "SHADOW_TOSS",
    str = STRINGS.ACTIONS.SHADOW_TOSS,
	fn = function(act)
        if act.doer ~= nil and act.doer.components.inventory ~= nil then
		    local projectile = act.invobject
		    if projectile == nil then
			    projectile = act.doer.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
		    end		
		    if projectile ~= nil and projectile.components.complexprojectile ~= nil and act.doer:HasTag("shadow_form_on") then
			    projectile = act.doer.components.inventory:DropItem(projectile)
			    if projectile ~= nil then
                    local pos = nil
                    if act.target ~= nil then
                        pos = act.target:GetPosition()
                        projectile.components.complexprojectile.targetoffset = {x = 0, y = 1.5, z = 0}
                    else
                        pos = act.pos 
                    end
                    projectile.components.complexprojectile:Launch(pos, act.doer)
                    return true
                end
			elseif projectile ~= nil and projectile.components.shadowtorchspecific ~= nil and act.doer:HasTag("shadow_form_on") then
			    projectile = act.doer.components.inventory:DropItem(projectile)
			    if projectile ~= nil then
                    local pos = nil
                    if act.target ~= nil then
                        pos = act.target:GetPosition()
                        projectile.components.shadowtorchspecific.targetoffset = {x = 0, y = 1.5, z = 0}
                    else
                        pos = act.pos 
                    end
                    projectile.components.shadowtorchspecific:Launch(pos, act.doer)
                    return true
                end					
            end
		end 
	end, 
    distance = 75,
    rmb = true,
    instant = false,
    mount_enabled = true,
    crosseswaterboundary = true, 	
    priority = 15,	
}

AddAction(shadowtoss) 

local MAKECHARLIECOFFEE = 
{
    id = "MAKECHARLIECOFFEE",
    str = STRINGS.ACTIONS.MAKECHARLIECOFFEE,
	fn = function(act)
        if act.target ~= nil and act.target.components.charliecoffeemaker ~= nil then
		    act.target.components.charliecoffeemaker:MakeCoffee(act.doer)
		    return true
		end
	end, 
    distance = 0.5,
    rmb = true,
    instant = false,
    mount_enabled = true,
    crosseswaterboundary = true, 	
    priority = 15,	
}

AddAction(MAKECHARLIECOFFEE) 

local EXTRACTINGREDIENT = 
{
    id = "EXTRACTINGREDIENT",
    str = STRINGS.ACTIONS.EXTRACTINGREDIENT,
	fn = function(act)
        if act.target ~= nil and act.target.components.charliecoffeemaker ~= nil then
		    act.target.components.charliecoffeemaker:ExtractIngredient(act.doer)
		    return true
		end
	end, 
    distance = 0.5,
    rmb = true,
    instant = false,
    mount_enabled = true,
    crosseswaterboundary = true, 	
    priority = 15,	
}

AddAction(EXTRACTINGREDIENT) 

local VALENTINEDAYFISH = 
{
    id = "VALENTINEDAYFISH",
    str = STRINGS.ACTIONS.VALENTINEDAYFISH,
	fn = function(act)
        if act.invobject ~= nil and act.invobject.components.valentinedayfishingrod ~= nil then
		    act.invobject.components.valentinedayfishingrod:DropRewards(act.doer, act.pos)
		    return true
		end
	end, 
    distance = 3,
    rmb = true,
    instant = false,
    mount_enabled = false,
    crosseswaterboundary = true, 	
    priority = 12,	
}

AddAction(VALENTINEDAYFISH) 

local CHARLIECUTPIZZA = 
{
    id = "CHARLIECUTPIZZA",
    str = STRINGS.ACTIONS.CHARLIECUTPIZZA,
	fn = function(act)
        if act.target ~= nil and act.target.components.pizzacuttable ~= nil then
		    act.target.components.pizzacuttable:Cut(act.invobject, act.doer)
		    return true
		end
	end, 
    distance = 0.25,
    rmb = true,
    instant = false,
    mount_enabled = true,
    crosseswaterboundary = true, 	
    priority = 12,	
}

AddAction(CHARLIECUTPIZZA) 

AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.MASTERDODGE, "masterdodge")) 
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.SHADOWEMBERCAST, "dojostleaction")) 
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.SHADOWFUELCAST, "dojostleaction")) 
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.SHADOW_TOSS, "dojostleaction")) 
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.MAKECHARLIECOFFEE, "dotwentysecactionwithcallback")) 
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.EXTRACTINGREDIENT, "dolongaction")) 
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.VALENTINEDAYFISH, "valentinedayfish")) 
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.CHARLIECUTPIZZA, "dolongaction")) 

if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.MASTERDODGE, "masterdodge")) 
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.SHADOWEMBERCAST, "dojostleaction")) 
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.SHADOWFUELCAST, "dojostleaction")) 
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.SHADOW_TOSS, "dojostleaction"))	
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.MAKECHARLIECOFFEE, "dotwentysecactionwithcallback"))
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.EXTRACTINGREDIENT, "dolongaction"))
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.VALENTINEDAYFISH, "valentinedayfish")) 
    AddStategraphActionHandler("wilsonboating", ActionHandler(ACTIONS.CHARLIECUTPIZZA, "dolongaction"))
end

local function TransformToTripleScale(inst)
    inst.Transform:SetScale(3, 3, 3)
end

AddPrefabPostInit("charlie_dragon_house_placer", TransformToTripleScale)
AddPrefabPostInit("charlie_castle_placer", TransformToTripleScale)

AddPrefabPostInit("frog_poison", function(inst)
	if inst.components.combat ~= nil then
	    inst.components.combat.canbeattackedfn = function(inst, attacker)
		    return attacker ~= nil and not attacker:HasTag("ancient_herald")
		end
	end 
	inst:DoTaskInTime(0, function()
	    if inst:HasTag("aporkalypse_cleanup") then
		    inst:AddTag("fireimmune")
	        inst:AddTag("groundpoundimmune")			
		end
	end)
end)

AddPrefabPostInit("pugalisk", function(inst)
    --Fixed pugalisk cannot heal itself
	if inst.components.health ~= nil then
	    local _health_redirect = inst.components.health.redirect
	    inst.components.health.redirect = function(inst, amount, overtime, cause, ignore_invincible)
            if amount ~= nil and amount > 0 and cause == "regen" then
                inst.components.health:DoDelta(amount, overtime, cause, ignore_invincible, true)
            end		
		    if _health_redirect ~= nil then
			    _health_redirect(inst, amount, overtime, cause, ignore_invincible)
			end 
		end
	end
end)

AddPrefabPostInit("home_prototyper", function(inst)
    if inst.components.prototyper ~= nil and inst.components.prototyper.SetCanPrototypeTestFunction ~= nil then
	    inst.components.prototyper:SetCanPrototypeTestFunction(function()
		    local interiorSpawner = GetWorld().components.interiorspawner
			return interiorSpawner and interiorSpawner.current_interior 
			    and (interiorSpawner.current_interior.playerroom or (type(interiorSpawner.current_interior.dungeon_name) == "string" and interiorSpawner.current_interior.dungeon_name:sub(1, 14) == "charlie_castle"))
		end)
	end
end)

AddClassPostConstruct("widgets/crafttabs", function(self)
	if self.SetShouldShowTabFn ~= nil then 
	    self:SetShouldShowTabFn("HOME", function()
		    local interiorSpawner = GetWorld().components.interiorspawner
		    return interiorSpawner and interiorSpawner.current_interior 
			    and (interiorSpawner.current_interior.playerroom or (type(interiorSpawner.current_interior.dungeon_name) == "string" and interiorSpawner.current_interior.dungeon_name:sub(1, 14) == "charlie_castle"))
	    end)
	end
end)

AddPrefabPostInit("prop_door", function(inst)
	inst:DoTaskInTime(0, function()
	    if inst.components.door ~= nil 
		    and inst.door_data_bank ~= nil 
			and inst.door_data_bank == "palace_door" 
		    and inst.door_data_animstate ~= nil
			and inst.door_data_animstate == "south"
		then
		    local interior = GetInteriorSpawner():GetInteriorByName(inst.components.door.interior_name)
		    if interior and type(interior.dungeon_name) == "string" and interior.dungeon_name:sub(1, 14) == "charlie_castle" then		
		        local minimap = inst.entity:AddMiniMapEntity()
		        minimap:SetIcon("player_frontdoor.png")
		        minimap:SetIconOffset(4, 0)
			end
	    end
		if inst.door_data_bank ~= nil and inst.door_data_bank == "exitrope" then
		    local minimap = inst.entity:AddMiniMapEntity()
		    minimap:SetIcon("cave_open2.png") 
		end
	end)
end)

local function PreventPlacingOnCastle(inst) 
    --The original plan was to be able to expand the room, 
    --and the room volume was the same as the initial built-in rooms. 
    --Unfortunately, there were issues with the door position and detection, and no solution could be found.
    --Therefore, it had to be changed to this way.  :<
	--As compensation, the initial number of rooms in the castle increased from 4 to 9.
 	if inst.components.placer ~= nil then
	    local _house_door_placeTestFn = inst.components.placer.placeTestFn
		inst.components.placer.placeTestFn = function(inst, pt)
		    local interiorSpawner = GetWorld().components.interiorspawner
		    if _house_door_placeTestFn ~= nil and interiorSpawner and interiorSpawner.current_interior and interiorSpawner.current_interior.playerroom then
			    return _house_door_placeTestFn(inst, pt)
			else
			    return false
			end
		end
	end  
end

AddPrefabPostInit("wood_door_placer", PreventPlacingOnCastle)
AddPrefabPostInit("stone_door_placer", PreventPlacingOnCastle)
AddPrefabPostInit("organic_door_placer", PreventPlacingOnCastle)
AddPrefabPostInit("iron_door_placer", PreventPlacingOnCastle)
AddPrefabPostInit("pillar_door_placer", PreventPlacingOnCastle)
AddPrefabPostInit("curtain_door_placer", PreventPlacingOnCastle)
AddPrefabPostInit("round_door_placer", PreventPlacingOnCastle)
AddPrefabPostInit("plate_door_placer", PreventPlacingOnCastle)

if IsDLCEnabled(PORKLAND_DLC) then
    --Fixed warg summons hounds in interior
	AddComponentPostInit("hounded", function(self)
	    local _SummonHound = self.SummonHound
	    function self:SummonHound()
            if _SummonHound ~= nil and not TheCamera.interior then
			    _SummonHound(self)
			end 
        end
	end)
	--Lily Pad Seed Spawn 
	local function AddLilyPadSeedSpawnHandler(inst)
	    inst.IsLilyPadSeason = function(inst)
		    local TheSeasonManager = GetSeasonManager()
		    if TheSeasonManager ~= nil then
			    if GetWorld().prefab == "porkland" then
				    return TheSeasonManager:IsLushSeason()		
				end
				if GetWorld().prefab == "shipwrecked" then
				    return TheSeasonManager:IsGreenSeason() or TheSeasonManager:IsDrySeason()
				end
			    return TheSeasonManager:IsSpring() or TheSeasonManager:IsSummer()
			end
		    return false
		end
	
	    inst.DropLilyPadSeed = function(inst)
		    local x, y, z = inst.Transform:GetWorldPosition()
            local nearby_lilypad_seed = TheSim:FindEntities(x, y, z, 10, {"lilypad_seed"}, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO"})			
		    if #nearby_lilypad_seed == 0 and math.random() < 0.25 then
		        local lilypadseed = SpawnPrefab("lilypad_seed")		    
			    lilypadseed.Transform:SetPosition(x, y, z)
			    if lilypadseed.Physics ~= nil then
			        lilypadseed.Physics:SetVel(math.cos(math.random()*2*math.pi), GetRandomWithVariance(8, 4), math.sin(math.random()*2*math.pi))
		    	end 
		    	if lilypadseed.components.inventoryitem ~= nil then
			        lilypadseed.components.inventoryitem:OnLootDropped()
		    	end 
            end				
		end
		if TUNING.PUDDLE_SPAWN and TUNING.PUDDLE_SPAWN > 0 then
            inst:AddComponent("lilypadseedspawner")
		end
	end
	
	AddPrefabPostInit("pond", AddLilyPadSeedSpawnHandler)
	AddPrefabPostInit("pond_mos", AddLilyPadSeedSpawnHandler)
	AddPrefabPostInit("pond_cave", AddLilyPadSeedSpawnHandler)
	AddPrefabPostInit("tidalpool", AddLilyPadSeedSpawnHandler)
	AddPrefabPostInit("lilypad", AddLilyPadSeedSpawnHandler)
	
end

AddPrefabPostInit("krampus", function(inst)
    if GetPlayer() and GetPlayer().prefab == "charlie" then
	    inst:ListenForEvent("death", function(inst, data)
		    if data ~= nil 
			    and data.cause == "charlie" 
				and inst.components.lootdropper ~= nil 
				and GetCharlieTaskRose()
				and not TheSim:FindFirstEntityWithTag("task_gift")
				and math.random() < 0.05
			then
			    inst.components.lootdropper:SpawnLootPrefab("charlie_valentine_day_gift")
			end
		end)
	end
end)

local function AddValidForTaskRoseTag(inst) 
    inst:AddTag("valid_for_full_task_rose")
end

AddPrefabPostInit("goldnugget", AddValidForTaskRoseTag)
AddPrefabPostInit("dubloon", AddValidForTaskRoseTag)
AddPrefabPostInit("oinc", AddValidForTaskRoseTag)
AddPrefabPostInit("oinc10", AddValidForTaskRoseTag)
AddPrefabPostInit("oinc100", AddValidForTaskRoseTag)

local function AddWaterProofer(inst) 
	if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
		inst:AddComponent("waterproofer")
	    inst.components.waterproofer:SetEffectiveness(0)
	end
end

AddPrefabPostInit("obsidianfirepit", AddWaterProofer)