require("behaviours/faceentity")
require("behaviours/follow")
require("behaviours/wander")
require("behaviours/chaseandattack")

local START_FACE_DIST = 10
local KEEP_FACE_DIST = 15

local CharlieDragonBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

local function GetFaceTargetFn(inst)
    local target = GetClosestInstWithTag("player", inst, START_FACE_DIST)
    return target ~= nil and not target:HasTag("notarget") and target or nil
end

local function KeepFaceTargetFn(inst, target)
    return target ~= nil and not target:HasTag("notarget") and inst:IsNear(target, KEEP_FACE_DIST)
end

local function GetLeader(inst)
    return inst.components.follower.leader 
end

local function HasValidHome(inst)
    return inst.components.homeseeker and 
       inst.components.homeseeker.home and 
       inst.components.homeseeker.home:IsValid()
end

local function GetHomePos(inst)
    if GetLeader(inst) then
        return Point(inst.components.follower.leader.Transform:GetWorldPosition())
    end
    return HasValidHome(inst) and inst.components.homeseeker:GetHomePos()
end

function CharlieDragonBrain:OnStart()
    local root = PriorityNode(
    {
        ChaseAndAttack(self.inst, 12, 40),
		Follow(self.inst, GetLeader, 3, 8, 9),
        FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn),
		Wander(self.inst, GetHomePos, 15),
    }, 0.25)

    self.bt = BT(self.inst, root)
end

return CharlieDragonBrain
