local Absorblightchecker = Class(function(self, inst)
    self.inst = inst
	self.absorb_cooldown = 0

    self.inst:ListenForEvent("transformable_onshadowform", function() self:Enable(true) end)
    self.inst:ListenForEvent("transformable_onnormalform", function() self:Enable(false) end)
end)

local LIGHT_CANT_TAGS = {"FX", "INLIMBO", "DECOR", "INTERIOR_LIMBO", "player", "absorb_immune", "daylight", "shadow_item", "charlie_portal", "campfire", "wallsection", "fullwallsection", "furniture", "cornerpost", "centerlight", "swinglight", "lightrays"}
local DAMAGE_MUST_TAGS = {"hascombatcomponent"}
local DAMAGE_CANT_TAGS = {"FX", "NOCLICK", "INLIMBO", "DECOR", "INTERIOR_LIMBO", "invisible", "notarget", "shadow_item", "shadow_area_immune", "wall", "boat", "companion", "bramble", "elephantcactus"}
local LIGHT_MUST_TAG = {"lightsource"}
local FIRE_CANT_TAGS = {"INLIMBO", "DECOR", "INTERIOR_LIMBO", "player", "absorb_immune", "shadow_item", "charlie_portal"}
local FIRE_ONEOF_TAGS = {"fire", "smolder"}
local CAMPFIRE_MUST = {"campfire"}

function Absorblightchecker:Enable(enable)
	if enable then
		self.inst:StartUpdatingComponent(self)
	else
		self.inst:StopUpdatingComponent(self)
	end
end

function Absorblightchecker:CanAbsorbLight()
    if self.inst.components.health ~= nil and self.inst.components.health:IsDead() then
	    return false
	end

    if self.inst.shadowform ~= nil then 
	    return true 
	end
	
    if self.inst.components.transformable ~= nil and self.inst.components.transformable.shadowform then
	    if GetWorld():IsCave() or GetClock():IsNight() then 
		    return true
		end
		--Should we just return not self.inst.components.transformable.istakinglightdamage?
	    if self.inst.components.moisture ~= nil and self.inst.components.inventory ~= nil then
		    local TotalWaterProofness = self.inst.components.inventory:GetWaterproofness() + (self.inst.components.moisture.sheltered and self.inst.components.moisture.shelter_waterproofness or 0)
			local GetInventory = self.inst.components.inventory
			local IsFireProofItemEquipped = GetInventory ~= nil and GetInventory:EquipHasTag("fire_immune_item")
		    if TotalWaterProofness >= 1 or IsFireProofItemEquipped then
	            return true
			end
		end
		local HasHAM = IsDLCEnabled(PORKLAND_DLC)
		local IsInHAMWorld = HasHAM and SaveGameIndex:IsModePorkland()
		local IsFoggyWeather = IsInHAMWorld and GetSeasonManager() and GetSeasonManager():IsFoggy()
		local IsInAnyInterior = HasHAM and TheCamera.interior
		local BloodyMoonOn = IsInHAMWorld and GetClock() and GetClock():GetBloodMoon()
		if IsFoggyWeather or BloodyMoonOn then
		    if not IsInAnyInterior then
		        return true
			else		
	            if self.inst.components.moisture ~= nil and self.inst.components.inventory ~= nil then
		            local TotalWaterProofness = self.inst.components.inventory:GetWaterproofness() + (self.inst.components.moisture.sheltered and self.inst.components.moisture.shelter_waterproofness or 0)
			        local GetInventory = self.inst.components.inventory
			        local IsFireProofItemEquipped = GetInventory ~= nil and GetInventory:EquipHasTag("fire_immune_item")
		            if TotalWaterProofness >= 1 or IsFireProofItemEquipped then
	                    return true
				    else
				        return false
			        end
		        end
			end
		end
	end
	
	return false
end

function Absorblightchecker:AbsorbLight()
    local pos = Vector3(self.inst.Transform:GetWorldPosition())
    local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, TUNING.SHADOW_FORM_LIGHT_CHECK_RADIUS, LIGHT_MUST_TAG, LIGHT_CANT_TAGS)	
    for k,v in pairs(ents) do
	    if v.Light ~= nil and v.Light:IsEnabled() and self.inst:GetDistanceSqToInst(v) < (v.Light:GetCalculatedRadius() or TUNING.SHADOW_FORM_COMMON_LIGHT_RADIUS) then
            --Removing v or v.Light will cause some serious problems, so this is not eternal
			v.Light:Enable(false)       --Have to make sure if this is reversible but do not make it enable immediately either
			if v.components.machine ~= nil then  
				v.components.machine:TurnOff()   --For lantern
			end
			if self.inst.components.sanity ~= nil then
		        self.inst.components.sanity:DoDelta(TUNING.SHADOW_FORM_ABSORBLIGHT_SANITYGAIN)
			end
            if v.Transform ~= nil and v:IsValid() and self.inst:IsValid() then
	            SpawnPrefab("statue_transition").Transform:SetPosition(v.Transform:GetWorldPosition())
                SpawnPrefab("statue_transition_2").Transform:SetPosition(v.Transform:GetWorldPosition())
	            local x, y, z = v.Transform:GetWorldPosition()
	            local _ents = TheSim:FindEntities(x, y, z, TUNING.SHADOW_DAMAGE_RADIUS, DAMAGE_MUST_TAGS, DAMAGE_CANT_TAGS)
		        for i,ent in pairs(_ents) do
		            if ent ~= self.inst and ent ~= v then
			            if ent:IsValid() and ent.entity:IsVisible() and ent.components.combat ~= nil then
				            ent.components.combat:GetAttacked(self.inst, TUNING.SHADOW_AREA_DAMAGE) 						
						    if ent.Transform ~= nil and self.inst:IsValid() then
	                            SpawnPrefab("statue_transition").Transform:SetPosition(ent.Transform:GetWorldPosition())
                                SpawnPrefab("statue_transition_2").Transform:SetPosition(ent.Transform:GetWorldPosition())								
							end
				        end
		          	end
		        end	
            end
		end
    end
end

function Absorblightchecker:StealFire()
    local pos = Vector3(self.inst.Transform:GetWorldPosition())
    local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, TUNING.SHADOW_FORM_LIGHT_CHECK_RADIUS, nil, FIRE_CANT_TAGS, FIRE_ONEOF_TAGS)	
    for k,v in pairs(ents) do
		if v.components.burnable ~= nil then
		    if self.inst:GetDistanceSqToInst(v) < ((v:HasTag("campfire") and 2) or (v.components.burnable:GetLargestLightRadius() or TUNING.SHADOW_FORM_COMMON_LIGHT_RADIUS)) then
		        v.components.burnable:Extinguish()
				if self.inst.components.sanity ~= nil then
				    self.inst.components.sanity:DoDelta(TUNING.SHADOW_FORM_ABSORBLIGHT_SANITYGAIN)
				end
                if v.Transform ~= nil and v:IsValid() and self.inst:IsValid() then
	                SpawnPrefab("statue_transition").Transform:SetPosition(v.Transform:GetWorldPosition())
                    SpawnPrefab("statue_transition_2").Transform:SetPosition(v.Transform:GetWorldPosition())
	                local x, y, z = v.Transform:GetWorldPosition()
	                local _ents = TheSim:FindEntities(x, y, z, TUNING.SHADOW_DAMAGE_RADIUS, DAMAGE_MUST_TAGS, DAMAGE_CANT_TAGS)
		            for i,ent in pairs(_ents) do
		                if ent ~= self.inst and ent ~= v then
			                if ent:IsValid() and ent.entity:IsVisible() and ent.components.combat ~= nil then
				                ent.components.combat:GetAttacked(self.inst, TUNING.SHADOW_AREA_DAMAGE)
								if ent.Transform ~= nil and self.inst:IsValid() then
	                                SpawnPrefab("statue_transition").Transform:SetPosition(ent.Transform:GetWorldPosition())
                                    SpawnPrefab("statue_transition_2").Transform:SetPosition(ent.Transform:GetWorldPosition())								
								end
				            end
		          	    end
		            end
                end
		    end
		end
	end
end

function Absorblightchecker:IsNearAnyCampfire()
    local x, y, z = self.inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x, y, z, TUNING.SHADOW_FORM_LIGHT_CHECK_RADIUS*2, CAMPFIRE_MUST, FIRE_CANT_TAGS)	
    for k,v in pairs(ents) do
	    if v.components.burnable ~= nil and v.components.burnable:IsBurning() then
		    return true
		end
	end
	return false
end

function Absorblightchecker:OnUpdate(dt)
    if self.absorb_cooldown > 0 then 
        self.absorb_cooldown = self.absorb_cooldown - dt
		return
    end
	
    if self:CanAbsorbLight() then
        self.absorb_cooldown = 3 + (self:IsNearAnyCampfire() and 2 or 0)
	    self:AbsorbLight()
		self:StealFire()
    end

end

return Absorblightchecker