local function generatefreepositions(max)
	local pos_table = {}
	for num = 1, max do
		table.insert(pos_table, num)
	end
	return pos_table
end

local pos_modifier = 1.2

local CharlieShadowFlowerSpawner = Class(function(self, inst)
	self.inst = inst
	self.miniontype = "charlie_shadow_flower"
	self.maxminions = 27
	self.minionspawntime = {min = 5, max = 10}
	self.minions = {}
	self.numminions = 0
	self.distancemodifier = 11
	self.onspawnminionfn = nil
	self.onlostminionfn = nil
	self.onminionattacked = nil
	self.onminionattack = nil
	self.spawninprogress = false
	self.nextspawninfo = {}
	self.shouldspawn = true
	self.timeuntilspawn = nil
	self.minionpositions = nil
	--self.validtiletypes = {}
	self.addpositiondelay = 3
	
	self.freepositions = generatefreepositions(self.maxminions * pos_modifier)
end)

function CharlieShadowFlowerSpawner:RegenerateFreePositions()
	self.freepositions = generatefreepositions(self.maxminions * pos_modifier)
end

function CharlieShadowFlowerSpawner:GetDebugString()
	local str = string.format("Num Minions: %s, Spawn In Progress: %s,  Time For Spawn: %s, Should Spawn: %s",
		tostring(self.numminions), tostring(self.spawninprogress), tostring(self.nextspawninfo.time) or "NIL", tostring(self.shouldspawn))
	return str
end

function CharlieShadowFlowerSpawner:RemovePosition(num)	
	for k,v in pairs(self.freepositions) do
		if v == num then
			table.remove(self.freepositions, k)
		end
	end
	table.sort(self.freepositions)
end

function CharlieShadowFlowerSpawner:AddPosition(num)
	table.insert(self.freepositions, num)
	table.sort(self.freepositions)
end

local function MakeSaveable(table)
	local tosave = {}
	for k,v in pairs(table) do
		tosave[k] = {x = v.x, y = v.y, z = v.z}
	end
	return tosave
end

local function UnpackSave(table)
	local touse = {}
	for k,v in pairs(table) do
		touse[k] = Vector3(v.x, v.y, v.z)
	end
	return touse
end

function CharlieShadowFlowerSpawner:OnSave()
	local data = {}
	local guidtable = {}
	for k,v in pairs(self.minions) do
		if not data.minions then
			data.minions = {{GUID = v.GUID, NUMBER = v.minionnumber}}
		else
			table.insert(data.minions, {GUID = v.GUID, NUMBER = v.minionnumber})
		end
		table.insert(guidtable, v.GUID)
	end

--	data.freepositions = self.freepositions

	data.maxminions = self.maxminions

	if self.minionpositions ~= nil then
		data.minionpositions = MakeSaveable(self.minionpositions)
	end
	if self.spawninprogress then
		data.spawninprogress = self.spawninprogress
		self.timeuntilspawn = (self.nextspawninfo.start + self.nextspawninfo.time) - GetTime()
		if self.timeuntilspawn < 0 then self.timeuntilspawn = 1 end
		data.timeuntilspawn = self.timeuntilspawn
	end

	return data, guidtable
end

function CharlieShadowFlowerSpawner:OnLoad(data)		
	if data.maxminions then self.maxminions = data.maxminions end
	
	self.freepositions = generatefreepositions(self.maxminions * pos_modifier)
	
	if data.minionpositions then self.minionpositions = UnpackSave(data.minionpositions) end

	if data.spawninprogress then
		self:ResumeSpawn(data.timeuntilspawn)
	end
end

function CharlieShadowFlowerSpawner:LoadPostPass(newents, savedata)	

	if savedata.minions then
		for k,v in pairs(savedata.minions) do
			local minion = newents[v.GUID]
			if minion then
				minion = minion.entity
				minion.minionnumber = v.NUMBER
				self:TakeOwnership(minion)
				local pos = self:GetSpawnLocation(minion.minionnumber)
				if pos then
					minion.Transform:SetPosition(pos.x, pos.y, pos.z)
					self:RemovePosition(minion.minionnumber)
				end
			end
		end
	end
end

function CharlieShadowFlowerSpawner:TakeOwnership(minion)
	if self.onminionattacked then
		minion.attackedfn = function() self.onminionattacked(self.inst) end
		self.inst:ListenForEvent("attacked", minion.attackedfn, minion)
	end

	if self.onminionattack then
		minion.attackedotherfn = function() self.onminionattack(self.inst) end
		self.inst:ListenForEvent("onattackother", minion.attackedotherfn, minion)
	end

	minion.deathfn = function() minion:PushEvent("attacked") self:OnLostMinion(minion) end
	self.inst:ListenForEvent("death", minion.deathfn, minion)
	self.inst:ListenForEvent("onremove", minion.deathfn, minion)
	minion.minionlord = self.inst
	self.minions[minion] = minion
	self.numminions = self.numminions + 1
	self.inst:PushEvent("minionchange")

	if not minion.minionnumber then
		minion.minionnumber = self.freepositions[math.random(#self.freepositions)]	
	end
end

function CharlieShadowFlowerSpawner:OnLostMinion(minion)
	if minion then
		self.inst:DoTaskInTime(self.addpositiondelay, function() self:AddPosition(minion.minionnumber) end)

		if self.onminionattacked then
			self.inst:RemoveEventCallback("attacked", minion.attackedfn, minion)
		end
		if self.onminionattack then
			self.inst:RemoveEventCallback("onattackother", minion.attackedotherfn, minion)
		end

		self.inst:RemoveEventCallback("death", minion.deathfn, minion)
		self.inst:RemoveEventCallback("onremove", minion.deathfn, minion)

		self.minions[minion] = nil
		self.numminions = self.numminions - 1

		self.inst:PushEvent("minionchange")

		if not self:MaxedMinions() and self.shouldspawn then
			self:StartNextSpawn()
		end
	end
end

function CharlieShadowFlowerSpawner:MakeMinion()
	if self.miniontype and not self:MaxedMinions() then
		return SpawnPrefab(self.miniontype)
	end
end
--[[
function CharlieShadowFlowerSpawner:CheckTileCompatibility(tile)
	for k,v in pairs(self.validtiletypes) do
		if v == tile then
			return true
		end
	end
end
]]
function CharlieShadowFlowerSpawner:MakeSpawnLocations()
	local positions = {}
	for i = 1, 100 do
	   	local s = i/32.0--(num/2) -- 32.0
	   	local a = math.sqrt(s*512.0)
	   	local b = math.sqrt(s)
		table.insert(positions, Vector3(math.sin(a)*b, 0, math.cos(a)*b))
	end
	
	local useablepositions = {}
	local pt = Vector3(self.inst.Transform:GetWorldPosition())
	for k,v in pairs(positions) do
		local offset = Vector3(v.x * self.distancemodifier, 0, v.z * self.distancemodifier)
		local try_pos = offset + pt
		local tile = GetWorld().Map:GetTileAtPoint(try_pos.x, try_pos.y, try_pos.z)

		if tile ~= GROUND.IMPASSABLE 
		    and tile <= GROUND.UNDERGROUND 
		    and GetWorld().Pathfinder:IsClear(pt.x, pt.y, pt.z, try_pos.x, try_pos.y, try_pos.z, {ignorewalls = true}) 
			and	#TheSim:FindEntities(try_pos.x, try_pos.y, try_pos.z, 1, nil, {"FX", "NOCLICK", "DECOR", "INLIMBO"}) <= 0 
			--and ( self.inst.IsPosSurroundedByLand == nil or self.inst:IsPosSurroundedByLand(try_pos.x, try_pos.y, try_pos.z, 3) )
		then--Shadow Flower is able to float over water
			table.insert(useablepositions, try_pos)
			if #useablepositions >= (self.maxminions * pos_modifier) then 
				return useablepositions 
			end

		end
	end
	--if it couldn't find enough spots for minions.
	self.maxminions = #useablepositions
	self.freepositions = generatefreepositions(self.maxminions)
	if #useablepositions > 0 then
		return useablepositions
	else
		return nil
	end
end

function CharlieShadowFlowerSpawner:GetSpawnLocation(num)
	-- local pt = Vector3(self.inst.Transform:GetWorldPosition())
	if not self.minionpositions then
		return
	end
	
	local offset = self.minionpositions[num]
	-- return (pt.x + (offset.x * self.minionradius)), pt.y, (pt.z + (offset.z * self.minionradius))
	if offset then
		return Vector3(offset.x, offset.y, offset.z)
	end

end

function CharlieShadowFlowerSpawner:GetNextSpawnTime()
	return math.random(self.minionspawntime.min, self.minionspawntime.max)
end

function CharlieShadowFlowerSpawner:KillAllMinions()
	self.spawninprogress = false
	for k,v in pairs(self.minions) do
		self.inst:DoTaskInTime(math.random(),function() v.components.health:Kill() end)
	end
end

function CharlieShadowFlowerSpawner:SpawnNewMinion(force)


	if not self.minionpositions then
		self.minionpositions = self:MakeSpawnLocations()
	end

	if not self.minionpositions then
		self.failedFindingPositions = true
		return
	end

	if (force or self.shouldspawn) and not self:MaxedMinions() and #self.freepositions > 0 then
        self.spawninprogress = false

        local num = self.freepositions[math.random(#self.freepositions)]
        local pos = self:GetSpawnLocation(num)
        if pos ~= nil then
            local minion = self:MakeMinion()
            if minion ~= nil then
			    if minion.sg ~= nil then
                    minion.sg:GoToState("spawn")
				end
				minion:PushEvent("spawnfromsomething")
                minion.minionnumber = num
                self:TakeOwnership(minion)
                minion.Transform:SetPosition(pos:Get())
                self:RemovePosition(num)

                if self.onspawnminionfn ~= nil then
                    self.onspawnminionfn(self.inst, minion)
                end
            end
        elseif self.miniontype ~= nil and not self:MaxedMinions() then
            self.minionpositions = self:MakeSpawnLocations()
        end

        if self.shouldspawn and not self:MaxedMinions() then
            self:StartNextSpawn()
        end
    end
end


function CharlieShadowFlowerSpawner:MaxedMinions()
	return self.numminions >= self.maxminions
end

function CharlieShadowFlowerSpawner:SetSpawnInfo(time)
	self.nextspawninfo = {}
	self.nextspawninfo.start = GetTime()
	self.nextspawninfo.time = time
	return time
end

function CharlieShadowFlowerSpawner:StartNextSpawn()

	if not self.shouldspawn then
		return
	end

	if not self.spawninprogress and not self:MaxedMinions() then
		self.spawninprogress = true
		local time = self:SetSpawnInfo(self:GetNextSpawnTime())
		self.task = self.inst:DoTaskInTime(time, function() self:SpawnNewMinion() end)
	end
end

function CharlieShadowFlowerSpawner:ResumeSpawn(time)
	if time < 1 then time = 1 end
	self:SetSpawnInfo(time)
	self.task = self.inst:DoTaskInTime(time, function() self:SpawnNewMinion() end)
	self.spawninprogress = true
end

function CharlieShadowFlowerSpawner:LongUpdate(dt)

	local useuptime = function(time) 
		local iterations = 0
		while time > 0 do
			time = time - self:GetNextSpawnTime()
			iterations = iterations + 1
		end
		return iterations
	end

	if self.spawninprogress and self.shouldspawn then
		if self.task then
			self.task:Cancel()
			self.task = nil
		end

		local possiblespawns = useuptime(dt)

		-- we don't want to try to generate positions for every minion we spawn
		self.failedFindingPositions = nil
		for i = 1, possiblespawns do
			if self.task then
				self.task:Cancel()
				self.task = nil
			end
			if not self.failedFindingPositions then
				self:SpawnNewMinion()
			end
		end
		self.failedFindingPositions = nil
	end
end

function CharlieShadowFlowerSpawner:DespawnAll()
	self.spawninprogress = false
	for k,v in pairs(self.minions) do
	    if v ~= nil then
	        v.persists = false
		    v:DoTaskInTime(math.random(), function() 
			    v:Remove()
		    end)
		end
	end
end

function CharlieShadowFlowerSpawner:SpawnAll()
	for i = 1, self.maxminions do
		self.inst:DoTaskInTime(math.random(2,3) * math.random(), function()
			self:SpawnNewMinion(true)
		end)
	end
end

return CharlieShadowFlowerSpawner