local CharlieThroneSpawner = Class(function(self, inst)
	self.inst = inst
	self.enabled = false
	self.remaining_time = TUNING.TOTAL_DAY_TIME*5
    self.inst:DoTaskInTime(0, function()
        local player = GetPlayer()
		local world = GetWorld()
	    if TUNING.CHARLIE_THRONE_ENABLE == true 
	        and player ~= nil
			and player:HasTag("charlie")
		    and world ~= nil
			and world.components.charlienpcspawner ~= nil
            and not world.components.charlienpcspawner.enabled
            and world.components.shadowriftspawner ~= nil
			and not world.components.shadowriftspawner.enabled
	    then
	        self:Enable(true)
	    end
	end)
end)

function CharlieThroneSpawner:OnSave()
    return 
	{
		remaining_time = self.remaining_time,
	}
end

function CharlieThroneSpawner:OnLoad(data)
    if data ~= nil then
        self.remaining_time = data.remaining_time or TUNING.TOTAL_DAY_TIME*5
	end
end

function CharlieThroneSpawner:Enable(enable)
	if enable then
	    self.enabled = true
		self.inst:StartUpdatingComponent(self)
	else
	    self.enabled = false
		self.inst:StopUpdatingComponent(self)
	end
end

local BLOCKER_EXCLUDE_TAGS = {"FX", "INLIMBO"}
local function IsPointNearBlocker(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, BLOCKER_EXCLUDE_TAGS) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearPlayer(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, {"player"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearSomething(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"charlie_stage", "charlie_throne", "ruins_exit", "pugalisk_fountain"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearWallOrStructure(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"wall", "structure"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

function CharlieThroneSpawner:GetSpawnPoint()
	local centers = {}
    for i, node in ipairs( GetWorld().topology.nodes ) do
        if node.type ~= NODE_TYPE.Blank and node.type ~= NODE_TYPE.Blocker then
            local x = node.x
            local z = node.y
            local pos = Vector3(x, 0, z)
            local tilepos = Vector3( GetWorld().Map:GetTileCenterPoint(pos.x, pos.y, pos.z) )
            if IsPassableAtPoint(tilepos.x, tilepos.y, tilepos.z) 
			    and ( GetPlayer().IsPosSurroundedByLand == nil or GetPlayer():IsPosSurroundedByLand(tilepos.x, tilepos.y, tilepos.z, 3) )
				and not IsPointNearPlayer(tilepos, 150) 		
			    and not IsPointNearSomething(tilepos, 30) 	
				and not IsPointNearBlocker(tilepos, 5) 		
				and not IsPointNearWallOrStructure(tilepos, 10) 	
			then                                            
                table.insert(centers, {x = node.x, z = node.y})
            end
        end		
	end
    if #centers > 0 then
        local pos = centers[math.random(#centers)]
        return pos.x, 0, pos.z
    else
        return nil
    end
end

function CharlieThroneSpawner:SpawnThrone()
	local x, y, z = self:GetSpawnPoint()
	if x ~= nil and y ~= nil and z ~= nil then
        print("charlie throne appears in the world")
	    SpawnPrefab("charlie_throne").Transform:SetPosition(x, y, z)
	end
end

function CharlieThroneSpawner:LongUpdate(dt)
    if not self.enabled 
	    or TheSim:FindFirstEntityWithTag("charlie_throne")
		or TheSim:FindFirstEntityWithTag("charlie_npc")
		or TheSim:FindFirstEntityWithTag("shadow_rift")
	then
	    return
	end

    if self.remaining_time > 0 then
	    self.remaining_time = self.remaining_time - dt
	end
	
	if self.remaining_time <= 0 then
	    self.remaining_time = TUNING.TOTAL_DAY_TIME*5
	    self:SpawnThrone()
	end

end

function CharlieThroneSpawner:OnUpdate(dt)
    if not self.enabled 
	    or TheSim:FindFirstEntityWithTag("charlie_throne")
		or TheSim:FindFirstEntityWithTag("charlie_npc")
		or TheSim:FindFirstEntityWithTag("shadow_rift")
	then
	    return
	end

    if self.remaining_time > 0 then
	    self.remaining_time = self.remaining_time - dt
	end
	
	if self.remaining_time <= 0 then
	    self.remaining_time = TUNING.TOTAL_DAY_TIME*5
	    self:SpawnThrone()
	end

end

return CharlieThroneSpawner