local LilyPadSeedSpawner = Class(function(self, inst)
	self.inst = inst
	self.enabled = false
	self.remaining_time = 1440
	
   	self.inst:DoTaskInTime(0, function() self:CheckEnable() end)
	
    self.inst:ListenForEvent("seasonChange", function() self:CheckEnable() end, GetWorld())		
end)

function LilyPadSeedSpawner:OnSave()
    return 
	{
		remaining_time = self.remaining_time,
	}
end

function LilyPadSeedSpawner:OnLoad(data)
    if data ~= nil then
		self.remaining_time = data.remaining_time or 1440
	end
end

function LilyPadSeedSpawner:Enable(enable)
	if enable then
	    self.enabled = true
		self.inst:StartUpdatingComponent(self)
	else
	    self.enabled = false
		self.inst:StopUpdatingComponent(self)
	end
end

function LilyPadSeedSpawner:CheckEnable()
    if self.inst.IsLilyPadSeason ~= nil and self.inst:IsLilyPadSeason() then
	    self:Enable(true)
	else
	    self:Enable(false)
	end
end

function LilyPadSeedSpawner:SpawnLilyPadSeed()
    if IsDLCEnabled(PORKLAND_DLC) then
	    SpawnPrefab("lilypad_seed").Transform:SetPosition(self.inst.Transform:GetWorldPosition())
	end
end

function LilyPadSeedSpawner:DropLilyPadSeed()
    if self.inst.DropLilyPadSeed ~= nil then
	    self.inst:DropLilyPadSeed()
	else
	    self:SpawnLilyPadSeed()
	end
end

function LilyPadSeedSpawner:CanSpawnLilyPadSeed()
    return self.inst ~= nil and self.inst:IsValid() and self.enabled
end

function LilyPadSeedSpawner:OnUpdate(dt)
    if not self:CanSpawnLilyPadSeed() then
		self.inst:StopUpdatingComponent(self)
	    return
	end

    if self.remaining_time > 0 then
	    self.remaining_time = self.remaining_time - dt
	else
	    self.remaining_time = 1440
		self:DropLilyPadSeed()	
	end
end

function LilyPadSeedSpawner:LongUpdate(dt)
    self:OnUpdate(dt)
end

return LilyPadSeedSpawner