local Magicturnable = Class(function(self, inst)
    self.inst = inst
	self.remove_turner_on_used = true
	self.show_shadow_fx = true
end)

function Magicturnable:CanTurn(doer, turner)
    if doer == nil or turner == nil then
	    return false
	end
    if self.canturntest ~= nil then
	    return self.canturntest(self.inst, doer, turner)
	end
	if self.inst.prefab == "log" and (doer.components.hunger == nil or doer.components.hunger:GetPercent() < 0.5) then
	    return false
	end
    return self.inst:IsValid() and (self.inst:IsInLimbo() or self.inst.entity:IsVisible())
end

function Magicturnable:Turn(doer, turner)
    if self:CanTurn(doer, turner) then
	    if turner.components.magicturner ~= nil and turner.components.magicturner.convert_prefab ~= nil then
		    local replacement = SpawnPrefab(turner.components.magicturner.convert_prefab)
		    if replacement ~= nil then
			    if self.inst.components.perishable ~= nil and replacement.components.perishable ~= nil then
				    replacement.components.perishable:SetPercent(self.inst.components.perishable:GetPercent())
				end
				if self.inst.components.stackable ~= nil and replacement.components.stackable ~= nil then
				    replacement.components.stackable:SetStackSize(self.inst.components.stackable:StackSize())
				end
			    if self.inst.components.health ~= nil and replacement.components.health ~= nil then
				    replacement.components.health:SetPercent(self.inst.components.health:GetPercent())
				end				
				if self.inst.ApplyInheritedMoisture ~= nil then 
				    self.inst:ApplyInheritedMoisture(replacement)
				end
				if self.show_shadow_fx and not self.inst:IsInLimbo() then
	                SpawnPrefab("statue_transition").Transform:SetPosition(self.inst.Transform:GetWorldPosition())
                    SpawnPrefab("statue_transition_2").Transform:SetPosition(self.inst.Transform:GetWorldPosition())					
				end
				if self.onturned ~= nil then
				    self.onturned(self.inst, doer, turner)
				end
				if replacement.prefab == "livinglog" and doer.components.hunger ~= nil then
				    doer.components.hunger:SetPercent(0)
				end
				if self.inst:IsInLimbo() and replacement.components.inventoryitem ~= nil and doer.components.inventory ~= nil then
				    doer.components.inventory:GiveItem(replacement, nil, Vector3(TheSim:GetScreenPos(doer.Transform:GetWorldPosition())))
					self.inst:Remove()
				else
				    replacement.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
					if self.inst.GetIsOnWater ~= nil and self.inst:GetIsOnWater() then
				        if replacement.components.floatable ~= nil then
						    replacement.components.floatable:OnHitWater(true)
						elseif replacement.components.inventoryitem ~= nil then
					        replacement.components.inventoryitem:OnHitWater()
					    end
					end
					self.inst:Remove()
				end
				if self.remove_turner_on_used then
				    if turner.components.stackable ~= nil then
					    turner.components.stackable:Get():Remove()
					else
					    turner:Remove()
					end
				end
				return true
			end
		end
	end
end

local SHADOW_UPGRADE_ITEMS =
{
    cane = "cane_flower",
	walkingstick = "cane_flower",
    telestaff = "supertelestaff",
    nightsword = "purenightsword",
	armor_sanity = "armor_shadow_void",
	umbrella = "voidcloth_umbrella",
	grass_umbrella = "voidcloth_umbrella",
	double_umbrellahat = "voidcloth_umbrella",	
	eyebrellahat = "voidcloth_umbrella",		
	armorwood = "armor_voidcloth",
	footballhat = "voidclothhat",
	pickaxe = "voidcloth_scythe",	
}

function Magicturnable:Upgrade(doer, turner)
    --Don't want to create another new component, so putting the other-actions function here.
	if self:CanTurn(doer, turner) then
	    local prefab_to_spawn = SHADOW_UPGRADE_ITEMS[self.inst.prefab]
	    if prefab_to_spawn ~= nil then
		    local replacement = SpawnPrefab(prefab_to_spawn)
			if replacement ~= nil then
				if self.inst.ApplyInheritedMoisture ~= nil then 
				    self.inst:ApplyInheritedMoisture(replacement)
				end
				if self.show_shadow_fx and not self.inst:IsInLimbo() then
	                SpawnPrefab("statue_transition").Transform:SetPosition(self.inst.Transform:GetWorldPosition())
                    SpawnPrefab("statue_transition_2").Transform:SetPosition(self.inst.Transform:GetWorldPosition())					
				end
				if self.onupgraded ~= nil then
				    self.onupgraded(self.inst, doer, turner)
				end				
				if self.inst:IsInLimbo() and replacement.components.inventoryitem ~= nil and doer.components.inventory ~= nil then
				    doer.components.inventory:GiveItem(replacement, nil, Vector3(TheSim:GetScreenPos(doer.Transform:GetWorldPosition())))
			        self.inst:Remove()
				else
				    replacement.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
					if self.inst.GetIsOnWater ~= nil and self.inst:GetIsOnWater() then
				        if replacement.components.floatable ~= nil then
						    replacement.components.floatable:OnHitWater(true)
						elseif replacement.components.inventoryitem ~= nil then
					        replacement.components.inventoryitem:OnHitWater()
					    end
					end
					self.inst:Remove()
				end
				if self.remove_turner_on_used then
				    if turner.components.stackable ~= nil then
					    turner.components.stackable:Get():Remove()
					else
					    turner:Remove()
					end
				end
				return true
			end
		end		
	end
end

return Magicturnable
