local PeriodicFlowerSpawner = Class(function(self, inst)
	self.inst = inst
	self.enabled = false
	self.remaining_time = 240
end)

function PeriodicFlowerSpawner:OnSave()
    return 
	{
		enabled = self.enabled,
		remaining_time = self.remaining_time,
	}
end

function PeriodicFlowerSpawner:OnLoad(data)
    if data ~= nil then
	    if data.enabled then
		    self:Enable(true)
		else
		    self:Enable(false)
		end
		self.remaining_time = data.remaining_time or 240
	end
end

function PeriodicFlowerSpawner:Enable(enable)
	if enable then
	    self.enabled = true
		self.inst:StartUpdatingComponent(self)
	else
	    self.enabled = false
		self.inst:StopUpdatingComponent(self)
	end
end

function PeriodicFlowerSpawner:LongUpdate(dt)
    if not self.enabled then
		self.inst:StopUpdatingComponent(self)	
	    return
	end

    if self.remaining_time > 0 then
	    self.remaining_time = self.remaining_time - dt
	end
	
	if self.remaining_time <= 0 then
	    self.remaining_time = 240
	    if self.inst.SpawnFlowerNearby ~= nil then
		    self.inst:SpawnFlowerNearby()
		end
	end

end

function PeriodicFlowerSpawner:OnUpdate(dt)
    if not self.enabled then
		self.inst:StopUpdatingComponent(self)
	    return
	end

    if self.remaining_time > 0 then
	    self.remaining_time = self.remaining_time - dt
	end
	
	if self.remaining_time <= 0 then
	    self.remaining_time = 240
	    if self.inst.SpawnFlowerNearby ~= nil then
		    self.inst:SpawnFlowerNearby()
		end
	end

end

return PeriodicFlowerSpawner