local PuddleSpawner = Class(function(self, inst)
	self.inst = inst
	self.enabled = false
	self.remaining_time = 240
	
   	self.inst:DoTaskInTime(0, function() self:CheckEnable() end)
	
    self.inst:ListenForEvent("rainstart", function() self:CheckEnable() end)
    self.inst:ListenForEvent("rainstop", function() self:CheckEnable() end)	
    self.inst:ListenForEvent("seasonChange", function() self:CheckEnable() end)		
	self.inst:ListenForEvent("checkdryuppuddle", function() self:CheckEnable() end)		
end)

function PuddleSpawner:OnSave()
    return 
	{
		remaining_time = self.remaining_time,
		puddle_time_left = self.puddle_time_left,
	}
end

function PuddleSpawner:OnLoad(data)
    if data ~= nil then
		self.remaining_time = data.remaining_time or 240
		if data.puddle_time_left then
		    self.puddle_time_left = data.puddle_time_left
			self:StartDryUpTask()
		end
	end
end

function PuddleSpawner:Enable(enable)
	if enable then
	    self.enabled = true
		self.inst:StartUpdatingComponent(self)
	else
	    self.enabled = false
		self.inst:StopUpdatingComponent(self)
	end
end

function PuddleSpawner:IsRainySeason()
    local TheSeasonManager = GetSeasonManager()
	if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    if TheSeasonManager then
		    if TheSeasonManager:IsSpring() then
			    return true
			end
			if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)) 
			    and GetWorld().prefab == "shipwrecked" 
			    and TheSeasonManager:IsGreenSeason()
			then
			    return true
			end
			if IsDLCEnabled(PORKLAND_DLC)
			    and GetWorld().prefab == "porkland" 
			    and TheSeasonManager:IsHumidSeason()			
			then
			    return true
			end
		end
    elseif TheSeasonManager and TheSeasonManager:IsSummer() then
	    return true
	end
	return false
end

function PuddleSpawner:CheckEnable()
    local TheSeasonManager = GetSeasonManager()
    if TheSeasonManager and TheSeasonManager:IsRaining() and self:IsRainySeason() then
	    self:Enable(true)
	else
	    self:Enable(false)
	end
	if self:IsDrySeason() 
	    and TheSeasonManager 
		and not TheSeasonManager:IsRaining()	
		and TheSeasonManager:GetTemperature() 
		and TheSeasonManager:GetTemperature() >= 80 
	then
	    self:StartDryUpTask()
	else
	    self:StopDryUpTask()
	end
end

function PuddleSpawner:IsDrySeason()
    local TheSeasonManager = GetSeasonManager()
	if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    if TheSeasonManager then
		    if TheSeasonManager:IsSummer() then
			    return true
			end
			if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)) 
			    and GetWorld().prefab == "shipwrecked" 
			    and TheSeasonManager:IsDrySeason()
			then
			    return true
			end
			if IsDLCEnabled(PORKLAND_DLC)
			    and GetWorld().prefab == "porkland" 
			    and TheSeasonManager:IsLushSeason()			
			then
			    return true
			end
		end
    elseif TheSeasonManager and TheSeasonManager:IsWinter() then
	    return true
	end
	return false
end

function PuddleSpawner:DryUpPuddles(n)
    local number = type(n) == "number" and n or math.random(10, 15)
	local puddles_to_dry_up = {}
    for k, v in pairs(Ents) do
        if v:HasTag("puddle") 
		    and not v:HasTag("shadowpuddle") 
			and not v:HasTag("shadowacidpuddle") 
			and not v:HasTag("puddle_disappear") 
		then
		    local x, y, z = v.Transform:GetWorldPosition()
		    local nearby_base = TheSim:FindEntities(x, y, z, 30, nil, nil, {"wall", "structure"}) 
			if #nearby_base == 0 and not table.contains(puddles_to_dry_up, v) then
			    table.insert(puddles_to_dry_up, v)
			end
		end
		if #puddles_to_dry_up >= number then
		    break
		end
    end
	if #puddles_to_dry_up > 0 then
	    for _, puddle in pairs(puddles_to_dry_up) do
		    puddle.persists = false
			puddle:AddTag("puddle_disappear")
			puddle:PushEvent("dryup")
			if puddle.DestroyAnimation ~= nil then
			    puddle.AnimState:PlayAnimation(puddle.DestroyAnimation)
			else
			    puddle:DoTaskInTime(1, puddle.Remove)
			end
			puddle:ListenForEvent("animover", puddle.Remove)
		end
	end
end

function PuddleSpawner:PuddleTimeLeftReduce(time_to_reduce, time_left)
    local TheSeasonManager = GetSeasonManager()
    if (not self:IsDrySeason())
	    or (TheSeasonManager 
		and (TheSeasonManager:IsRaining()	
		or (TheSeasonManager:GetTemperature() 
		and TheSeasonManager:GetTemperature() < 80)))
	then
	    self:StopDryUpTask()
	    return
	end
    if self.puddle_time_left == nil then
	    self.puddle_time_left = time_left or math.random(12, 18)
	end
    if self.puddle_time_left > 0 then
	    local timetoreduce = time_to_reduce or 1
		self.puddle_time_left = self.puddle_time_left - timetoreduce
	else
	    self.puddle_time_left = time_left or math.random(12, 18)
	    self:DryUpPuddles()
	end 
end

function PuddleSpawner:StartDryUpTask(t, time_to_reduce, time_left)
    if self.inst.DryUpPuddleTask == nil then
	    local base_time = t or 12
	    if self.fixed_spawn_time == 120 then
		    base_time = base_time * 2
		elseif self.fixed_spawn_time == 60 then
		    base_time = base_time * 3
		elseif self.fixed_spawn_time == 480 then
            base_time = base_time * 0.5
		elseif self.fixed_spawn_time == 960 then
		    base_time = base_time * 0.25
		end
	    self.inst.DryUpPuddleTask = self.inst:DoPeriodicTask(base_time, function() self:PuddleTimeLeftReduce(time_to_reduce, time_left) end)
	end 
end

function PuddleSpawner:StopDryUpTask()
    if self.inst.DryUpPuddleTask ~= nil then
	    self.inst.DryUpPuddleTask:Cancel()
	    self.inst.DryUpPuddleTask = nil
		self.puddle_time_left = nil
		self.inst.AlreadyCheckedDryUpPuddle = nil
	end
end

function PuddleSpawner:SetSpawnTime(v)
    if type(v) == "number" then
	    if v > 0 then
	        self.fixed_spawn_time = v
			self.remaining_time = v
		else
		    self.neverspawn = true
		end
	end
end

function PuddleSpawner:GetPuddleSpawnPoint()
	local centers = {}
    for i, node in pairs(GetWorld().topology.nodes) do
        if node.type ~= NODE_TYPE.Blank and node.type ~= NODE_TYPE.Blocker and node.x ~= nil and node.y ~= nil then
            local x = node.x
            local z = node.y
            local pos = Vector3(x, 0, z)
            local tilepos = Vector3(GetWorld().Map:GetTileCenterPoint(pos.x, pos.y, pos.z))
			local tile = GetWorld().Map:GetTileAtPoint(tilepos.x, tilepos.y, tilepos.z)
			local blockers = TheSim:FindEntities(tilepos.x, tilepos.y, tilepos.z, 1, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem"})
			local nearby_puddles = TheSim:FindEntities(tilepos.x, tilepos.y, tilepos.z, 3, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"})
			local IsPassablePoint = false
			local NotInInterior = true
	        if IsDLCEnabled(PORKLAND_DLC) then
	            if tile == GROUND.INTERIOR then
		            NotInInterior = false
	            end
            end			
			if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and tile < GROUND.UNDERGROUND then
			    IsPassablePoint = true
			end
            if (self.inst.IsPosSurroundedByLand == nil or self.inst:IsPosSurroundedByLand(tilepos.x, tilepos.y, tilepos.z, 3)) 
			    and IsPassablePoint
			    and NotInInterior
				and #blockers == 0
				and #nearby_puddles == 0
			then                                            
                table.insert(centers, {x = node.x, z = node.y})
            end
        end		
	end
    if #centers > 0 then
        local pos = centers[math.random(#centers)]
        return pos.x, 0, pos.z
    end
end

function PuddleSpawner:SpawnPuddle(puddletype)
    if not self.neverspawn then
	    local x, y, z = self:GetPuddleSpawnPoint()
		if x ~= nil and y ~= nil and z ~= nil then
			if type(puddletype) == "string" then
			    local puddle = SpawnPrefab(puddletype)
			    puddle.Transform:SetPosition(x, y, z) 
				if puddle.StartingAnimation ~= nil and puddle.LoopingAnimation ~= nil then 
				    puddle.AnimState:PlayAnimation(puddle.StartingAnimation)
					puddle.AnimState:PushAnimation(puddle.LoopingAnimation, true)
				end 				
			elseif math.random() < 0.25 then
			    local puddle = SpawnPrefab("potholepuddle")
			    puddle.Transform:SetPosition(x, y, z) 
				if puddle.StartingAnimation ~= nil and puddle.LoopingAnimation ~= nil then 
				    puddle.AnimState:PlayAnimation(puddle.StartingAnimation)
					puddle.AnimState:PushAnimation(puddle.LoopingAnimation, true)
				end 			
			else
			    local puddle = SpawnPrefab("shallowpuddle")
			    puddle.Transform:SetPosition(x, y, z) 
				if puddle.StartingAnimation ~= nil and puddle.LoopingAnimation ~= nil then 
				    puddle.AnimState:PlayAnimation(puddle.StartingAnimation)
					puddle.AnimState:PushAnimation(puddle.LoopingAnimation, true)
				end 			
			end		
		end
	end
end

function PuddleSpawner:SummonPuddle(doer, pos, puddletype)
    if doer ~= nil and pos ~= nil and not self.neverspawn then
		local result_offset = FindValidPositionByFan(math.random()*2*math.pi, math.random()*30, 12, function(offset)
            local spawn_point = pos + offset
			if GetWorld() == nil or GetWorld().Map == nil then
			    return false
			end
			local tile = GetWorld().Map:GetTileAtPoint(spawn_point.x, spawn_point.y, spawn_point.z)
			local blockers = TheSim:FindEntities(spawn_point.x, spawn_point.y, spawn_point.z, 1, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem"})
			local nearby_puddles = TheSim:FindEntities(spawn_point.x, spawn_point.y, spawn_point.z, 3, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"})
			if #blockers > 0 or #nearby_puddles > 0 then
			    return false
			end
			if self.inst.IsPosSurroundedByLand ~= nil 
			    and not self.inst:IsPosSurroundedByLand(spawn_point.x, spawn_point.y, spawn_point.z, 3) then
			    return false
			end 
            if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and tile < GROUND.UNDERGROUND then
                return true
            end
            return false
        end)	
		if result_offset ~= nil then
		    local spawn_pt = pos + result_offset
			if type(puddletype) == "string" then
			    local puddle = SpawnPrefab(puddletype)
			    puddle.Transform:SetPosition(spawn_pt.x, spawn_pt.y, spawn_pt.z) 
				if puddle.StartingAnimation ~= nil and puddle.LoopingAnimation ~= nil then 
				    puddle.AnimState:PlayAnimation(puddle.StartingAnimation)
					puddle.AnimState:PushAnimation(puddle.LoopingAnimation, true)
				end 				
			elseif math.random() < 0.25 then
			    local puddle = SpawnPrefab("potholepuddle")
			    puddle.Transform:SetPosition(spawn_pt.x, spawn_pt.y, spawn_pt.z) 
				if puddle.StartingAnimation ~= nil and puddle.LoopingAnimation ~= nil then 
				    puddle.AnimState:PlayAnimation(puddle.StartingAnimation)
					puddle.AnimState:PushAnimation(puddle.LoopingAnimation, true)
				end 			
			else
			    local puddle = SpawnPrefab("shallowpuddle")
			    puddle.Transform:SetPosition(spawn_pt.x, spawn_pt.y, spawn_pt.z) 
				if puddle.StartingAnimation ~= nil and puddle.LoopingAnimation ~= nil then 
				    puddle.AnimState:PlayAnimation(puddle.StartingAnimation)
					puddle.AnimState:PushAnimation(puddle.LoopingAnimation, true)
				end 			
			end
		else
		    self:SpawnPuddle(puddletype)
		end 
	else
	    self:SpawnPuddle(puddletype)
	end
end

function PuddleSpawner:CanSpawnPuddle()
    local TheSeasonManager = GetSeasonManager()
    if self.neverspawn or self:IsDrySeason() then
	    return false
	elseif TheSeasonManager and TheSeasonManager:IsRaining() and self:IsRainySeason() then
        return self.enabled
	else
	    return false
	end
end

function PuddleSpawner:OnUpdate(dt)
    if not self:CanSpawnPuddle() then
		self.inst:StopUpdatingComponent(self)
	    return
	end

    if self.remaining_time > 0 then
	    self.remaining_time = self.remaining_time - dt
	else
	    self.remaining_time = self.fixed_spawn_time or 240
		self:SpawnPuddle()	
	end
end

function PuddleSpawner:LongUpdate(dt)
    self:OnUpdate(dt)
end

return PuddleSpawner