local PuddleSpreader = Class(function(self, inst)
	self.inst = inst
	self.spread_time_left = 500
	
    if type(TUNING.PUDDLE_SPAWN) == "number" then
	    if TUNING.PUDDLE_SPAWN <= 60 then
		    self.spread_time_left = 240
		elseif TUNING.PUDDLE_SPAWN <= 120 then
		    self.spread_time_left = 350
		elseif TUNING.PUDDLE_SPAWN <= 240 then
		    self.spread_time_left = 500
		elseif TUNING.PUDDLE_SPAWN <= 480 then
		    self.spread_time_left = 750
		elseif TUNING.PUDDLE_SPAWN <= 960 then
		    self.spread_time_left = 1000
		else
		    self.spread_time_left = 1250
		end
	end
	
	self.inst:DoTaskInTime(0, function() self:SetTimeLeftAndCheckEnable() end)
	
    self.inst:ListenForEvent("rainstart", function() 
        local TheSeasonManager = GetSeasonManager()
        if TheSeasonManager and TheSeasonManager:IsRaining() and self:IsRainySeason() then
	        self:Enable(true)
	    else
	        self:Enable(false)
	    end
	end, GetWorld())
	
    self.inst:ListenForEvent("rainstop", function() 
        local TheSeasonManager = GetSeasonManager()
        if TheSeasonManager and TheSeasonManager:IsRaining() and self:IsRainySeason() then
	        self:Enable(true)
	    else
	        self:Enable(false)
	    end
	end, GetWorld())	
	
    self.inst:ListenForEvent("seasonChange", function() 
        local TheSeasonManager = GetSeasonManager()
        if TheSeasonManager and TheSeasonManager:IsRaining() and self:IsRainySeason() then
	        self:Enable(true)
	    else
	        self:Enable(false)
	    end
	end, GetWorld())
	
end)

function PuddleSpreader:OnSave()
    return 
	{
		spread_time_left = self.spread_time_left,
	}
end

function PuddleSpreader:OnLoad(data)
    if data ~= nil then
		self.spread_time_left = data.spread_time_left or 500
	end
end

function PuddleSpreader:Enable(enable)
	if enable then
	    self.enabled = true
		self.inst:StartUpdatingComponent(self)
	else
	    self.enabled = false
		self.inst:StopUpdatingComponent(self)
	end
end

function PuddleSpreader:IsRainySeason()
    local TheSeasonManager = GetSeasonManager()
	if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    if TheSeasonManager then
		    if TheSeasonManager:IsSpring() then
			    return true
			end
			if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)) 
			    and GetWorld().prefab == "shipwrecked" 
			    and TheSeasonManager:IsGreenSeason()
			then
			    return true
			end
			if IsDLCEnabled(PORKLAND_DLC)
			    and GetWorld().prefab == "porkland" 
			    and TheSeasonManager:IsHumidSeason()			
			then
			    return true
			end
		end
    elseif TheSeasonManager and TheSeasonManager:IsSummer() then
	    return true
	end
	return false
end

function PuddleSpreader:SetTimeLeftAndCheckEnable()
    if type(TUNING.PUDDLE_SPAWN) == "number" then
	    if TUNING.PUDDLE_SPAWN <= 0 then
		    self.neverspread = true
		elseif TUNING.PUDDLE_SPAWN <= 60 then
		    self.fixed_spread_time = 240
		elseif TUNING.PUDDLE_SPAWN <= 120 then
		    self.fixed_spread_time = 350
		elseif TUNING.PUDDLE_SPAWN <= 240 then
		    self.fixed_spread_time = 500
		elseif TUNING.PUDDLE_SPAWN <= 480 then
		    self.fixed_spread_time = 750
		elseif TUNING.PUDDLE_SPAWN <= 960 then
		    self.fixed_spread_time = 1000
		else
		    self.fixed_spread_time = 1250
		end
	end
    local TheSeasonManager = GetSeasonManager()
    if TheSeasonManager and TheSeasonManager:IsRaining() and self:IsRainySeason() then
	    self:Enable(true)
	else
	    self:Enable(false)
	end
end

function PuddleSpreader:GetValidPosition(x, y, z)
    if x ~= nil and y ~= nil and z ~= nil then
	    local possible_position = {}
		local PUDDLE_SCALE = 2.25
		local position1_near_blocker = TheSim:FindEntities(x + PUDDLE_SCALE, y, z, 3, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem", "puddle", "butterpuddle", "shadow_acid_smoke"})
		local position1_near_puddle = TheSim:FindEntities(x + PUDDLE_SCALE, y, z, 0.5, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"})
	    if #position1_near_blocker == 0 and #position1_near_puddle == 0 then
		    table.insert(possible_position, {x = x + PUDDLE_SCALE, y = y, z = z})
		end 
		local position2_near_blocker = TheSim:FindEntities(x - PUDDLE_SCALE, y, z, 3, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem", "puddle", "butterpuddle", "shadow_acid_smoke"})
		local position2_near_puddle = TheSim:FindEntities(x - PUDDLE_SCALE, y, z, 0.5, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"})
	    if #position2_near_blocker == 0 and #position2_near_puddle == 0 then
		    table.insert(possible_position, {x = x - PUDDLE_SCALE, y = y, z = z})
		end 			
		local position3_near_blocker = TheSim:FindEntities(x + PUDDLE_SCALE, y, z + PUDDLE_SCALE, 3, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem", "puddle", "butterpuddle", "shadow_acid_smoke"})
		local position3_near_puddle = TheSim:FindEntities(x + PUDDLE_SCALE, y, z + PUDDLE_SCALE, 0.5, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"})
	    if #position3_near_blocker == 0 and #position3_near_puddle == 0 then
		    table.insert(possible_position, {x = x + PUDDLE_SCALE, y = y, z = z + PUDDLE_SCALE})
		end 		
		local position4_near_blocker = TheSim:FindEntities(x + PUDDLE_SCALE, y, z - PUDDLE_SCALE, 3, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem", "puddle", "butterpuddle", "shadow_acid_smoke"})
		local position4_near_puddle = TheSim:FindEntities(x + PUDDLE_SCALE, y, z - PUDDLE_SCALE, 0.5, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"})
	    if #position4_near_blocker == 0 and #position4_near_puddle == 0 then
		    table.insert(possible_position, {x = x + PUDDLE_SCALE, y = y, z = z - PUDDLE_SCALE})
		end 		
		local position5_near_blocker = TheSim:FindEntities(x - PUDDLE_SCALE, y, z - PUDDLE_SCALE, 3, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem", "puddle", "butterpuddle", "shadow_acid_smoke"})
		local position5_near_puddle = TheSim:FindEntities(x - PUDDLE_SCALE, y, z - PUDDLE_SCALE, 0.5, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"})
	    if #position5_near_blocker == 0 and #position5_near_puddle == 0 then
		    table.insert(possible_position, {x = x - PUDDLE_SCALE, y = y, z = z - PUDDLE_SCALE})
		end 		
		local position6_near_blocker = TheSim:FindEntities(x - PUDDLE_SCALE, y, z + PUDDLE_SCALE, 3, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem", "puddle", "butterpuddle", "shadow_acid_smoke"})
		local position6_near_puddle = TheSim:FindEntities(x - PUDDLE_SCALE, y, z + PUDDLE_SCALE, 0.5, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"})
	    if #position6_near_blocker == 0 and #position6_near_puddle == 0 then
		    table.insert(possible_position, {x = x - PUDDLE_SCALE, y = y, z = z + PUDDLE_SCALE})
		end 		
		local position7_near_blocker = TheSim:FindEntities(x, y, z + PUDDLE_SCALE, 3, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem", "puddle", "butterpuddle", "shadow_acid_smoke"})
		local position7_near_puddle = TheSim:FindEntities(x, y, z + PUDDLE_SCALE, 0.5, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"})
	    if #position7_near_blocker == 0 and #position7_near_puddle == 0 then
		    table.insert(possible_position, {x = x, y = y, z = z + PUDDLE_SCALE})
		end 
		local position8_near_blocker = TheSim:FindEntities(x, y, z - PUDDLE_SCALE, 3, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem", "puddle", "butterpuddle", "shadow_acid_smoke"})
		local position8_near_puddle = TheSim:FindEntities(x, y, z - PUDDLE_SCALE, 0.5, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"})
	    if #position8_near_blocker == 0 and #position8_near_puddle == 0 then
		    table.insert(possible_position, {x = x, y = y, z = z - PUDDLE_SCALE})
		end 
		
		if #possible_position > 0 then
		    local spread_position = possible_position[math.random(#possible_position)]
            return spread_position.x, spread_position.y, spread_position.z
        end			
	end
end

function PuddleSpreader:Spread(puddletype)
    local x, y, z = self.inst.Transform:GetWorldPosition()
	local spreadx, spready, spreadz = self:GetValidPosition(x, y, z)
    if spreadx ~= nil and spready ~= nil and spreadz ~= nil then
        local puddle_type_to_spawn = self.inst.prefab
	    if type(puddletype) == "string" then
		    puddle_type_to_spawn = puddletype
		end
		local puddle = SpawnPrefab(puddle_type_to_spawn)
		if puddle ~= nil then
		    puddle.Transform:SetPosition(spreadx, spready, spreadz)
			if puddle.StartingAnimation ~= nil and puddle.LoopingAnimation ~= nil then 
			    puddle.AnimState:PlayAnimation(puddle.StartingAnimation)
				puddle.AnimState:PushAnimation(puddle.LoopingAnimation, true)
			end 	
		end
	end 
end

function PuddleSpreader:OnUpdate(dt)
    local x, y, z = self.inst.Transform:GetWorldPosition()
	local spreadx, spready, spreadz = self:GetValidPosition(x, y, z)
    if spreadx ~= nil and self.enabled and not self.neverspread then
        if self.spread_time_left > 0 then
	        self.spread_time_left = self.spread_time_left - dt
	    else
	        self.spread_time_left = self.fixed_spread_time or 500
			if math.random() < 0.5 then
		        self:Spread()	
			end
	    end
	else
	    self.inst:StopUpdatingComponent(self)
	end
end

function PuddleSpreader:LongUpdate(dt)
    self:OnUpdate(dt)
end

return PuddleSpreader