local ShadowRiftSpawner = Class(function(self, inst)
	self.inst = inst
	self.enabled = false
	self.remaining_time = TUNING.TOTAL_DAY_TIME*5    --Charlie Shadow Rift
	
	--Don't want create any new components!
	self.remaining_time2 = TUNING.TOTAL_DAY_TIME*10	  --Shadow Forge
	self.remaining_time3 = TUNING.TOTAL_DAY_TIME*10   --Charlie Fountain
	self.remaining_time4 = TUNING.TOTAL_DAY_TIME*15   --Charlie Ancient Gateway
	
	self.remaining_time5 = TUNING.TOTAL_DAY_TIME      --Buttercup
	
end)

function ShadowRiftSpawner:OnSave()
    return 
	{
		enabled = self.enabled,
		remaining_time = self.remaining_time,
		remaining_time2 = self.remaining_time2,
		remaining_time3 = self.remaining_time3,
		remaining_time4 = self.remaining_time4,
		remaining_time5 = self.remaining_time5,
	}
end

function ShadowRiftSpawner:OnLoad(data)
    if data ~= nil then
	    if data.enabled then
		    self:Enable(true)
		else
		    self:Enable(false)
		end
		self.remaining_time = data.remaining_time or TUNING.TOTAL_DAY_TIME*5
		self.remaining_time2 = data.remaining_time2 or TUNING.TOTAL_DAY_TIME*10
		self.remaining_time3 = data.remaining_time3 or TUNING.TOTAL_DAY_TIME*10
		self.remaining_time4 = data.remaining_time4 or TUNING.TOTAL_DAY_TIME*15
		self.remaining_time5 = data.remaining_time5 or TUNING.TOTAL_DAY_TIME
	end
end

function ShadowRiftSpawner:Enable(enable)
	if enable then
	    self.enabled = true
		self.inst:StartUpdatingComponent(self)
	else
	    self.enabled = false
		self.inst:StopUpdatingComponent(self)
	end
end

local function IsPointNearBlocker(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, {"FX", "NOCLICK"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearSomething(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"player", "charlie_stage", "charlie_throne", "ruins_exit", "pugalisk_fountain"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearWallOrStructure(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"wall", "structure"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

function ShadowRiftSpawner:GetSpawnPoint()
	local centers = {}
    for i, node in ipairs( GetWorld().topology.nodes ) do
        if node.type ~= NODE_TYPE.Blank and node.type ~= NODE_TYPE.Blocker then
            local x = node.x
            local z = node.y
            local pos = Vector3(x, 0, z)
            local tilepos = Vector3( GetWorld().Map:GetTileCenterPoint(pos.x, pos.y, pos.z) )
            if IsPassableAtPoint(tilepos.x, tilepos.y, tilepos.z) 
			    and ( GetPlayer().IsPosSurroundedByLand == nil or GetPlayer():IsPosSurroundedByLand(tilepos.x, tilepos.y, tilepos.z, 3) )
			    and not IsPointNearSomething(tilepos, 30) 		
				and not IsPointNearWallOrStructure(tilepos, 10) 	
				and not IsPointNearBlocker(tilepos, 3)
			then                                            
                table.insert(centers, {x = node.x, z = node.y})
            end
        end		
	end
    if #centers > 0 then
        local pos = centers[math.random(#centers)]
        return pos.x, 0, pos.z
    else
        return nil
    end
end

function ShadowRiftSpawner:GetAncientGatewaySpawnPoint()
	local centers = {}
    for i, node in ipairs( GetWorld().topology.nodes ) do
        if node.type ~= NODE_TYPE.Blank and node.type ~= NODE_TYPE.Blocker then
            local x = node.x
            local z = node.y
            local pos = Vector3(x, 0, z)
            local tilepos = Vector3( GetWorld().Map:GetTileCenterPoint(pos.x, pos.y, pos.z) )
            if IsPassableAtPoint(tilepos.x, tilepos.y, tilepos.z) 
			    and ( GetPlayer().IsPosSurroundedByLand == nil or GetPlayer():IsPosSurroundedByLand(tilepos.x, tilepos.y, tilepos.z, 12) )
			    and not IsPointNearSomething(tilepos, 30) 		
				and not IsPointNearWallOrStructure(tilepos, 20) 	
				and not IsPointNearBlocker(tilepos, 12)
			then                                            
                table.insert(centers, {x = node.x, z = node.y})
            end
        end		
	end
    if #centers > 0 then
        local pos = centers[math.random(#centers)]
        return pos.x, 0, pos.z
    else
        return nil
    end
end

function ShadowRiftSpawner:GetButtercupSpawnPoint()
	local centers = {}
    for i, node in ipairs( GetWorld().topology.nodes ) do
        if node.type ~= NODE_TYPE.Blank and node.type ~= NODE_TYPE.Blocker then
            local x = node.x
            local z = node.y
            local pos = Vector3(x, 0, z)
            local tilepos = Vector3( GetWorld().Map:GetTileCenterPoint(pos.x, pos.y, pos.z) )
            if IsPassableAtPoint(tilepos.x, tilepos.y, tilepos.z) 
			    and ( GetPlayer().IsPosSurroundedByLand == nil or GetPlayer():IsPosSurroundedByLand(tilepos.x, tilepos.y, tilepos.z, 3) )
			    and not IsPointNearSomething(tilepos, 20) 		
				and not IsPointNearWallOrStructure(tilepos, 10) 	
				and not IsPointNearBlocker(tilepos, 5)
			then                                            
                table.insert(centers, {x = node.x, z = node.y})
            end
        end		
	end
    if #centers > 0 then
        local pos = centers[math.random(#centers)]
        return pos.x, 0, pos.z
    else
        return nil
    end
end

function ShadowRiftSpawner:SpawnRift()
	local x, y, z = self:GetSpawnPoint()
	if x ~= nil and y ~= nil and z ~= nil then
        print("charlie shadow rift appears in the world")
	    SpawnPrefab("charlie_shadow_rift").Transform:SetPosition(x, y, z)
	end
end

function ShadowRiftSpawner:SpawnShadowForge()
	local x, y, z = self:GetSpawnPoint()
	if x ~= nil and y ~= nil and z ~= nil then
        print("shadow forge appears in the world")
	    SpawnPrefab("shadow_forge").Transform:SetPosition(x, y, z)
	end
end

function ShadowRiftSpawner:SpawnFountain()
	local x, y, z = self:GetSpawnPoint()
	if x ~= nil and y ~= nil and z ~= nil then
        print("charlie fountain appears in the world")
	    SpawnPrefab("charlie_fountain").Transform:SetPosition(x, y, z)
	end
end

function ShadowRiftSpawner:SpawnAncientGateway()
	local x, y, z = self:GetAncientGatewaySpawnPoint()
	if x ~= nil and y ~= nil and z ~= nil then
        print("charlie ancient gateway appears in the world")
	    SpawnPrefab("charlie_ancient_gateway").Transform:SetPosition(x, y, z)
	end
end

function ShadowRiftSpawner:SpawnButtercup()
	local x, y, z = self:GetButtercupSpawnPoint()
	if x ~= nil and y ~= nil and z ~= nil then
        print("buttercup appears in the world")
	    SpawnPrefab("buttercup").Transform:SetPosition(x, y, z)
	end
end

function ShadowRiftSpawner:LongUpdate(dt)

    if not self.enabled then
	    return
	end

    if TheSim:FindFirstEntityWithTag("shadow_forge") then
        if self.remaining_time3 > 0 and not TheSim:FindFirstEntityWithTag("charlie_fountain") then
	        self.remaining_time3 = self.remaining_time3 - dt
	    end

        if self.remaining_time4 > 0 and not TheSim:FindFirstEntityWithTag("charlie_ancient_gateway") then
	        self.remaining_time4 = self.remaining_time4 - dt
	    end

        if self.remaining_time5 > 0 and c_countprefabs("buttercup", true) < 30 then
	        self.remaining_time5 = self.remaining_time5 - dt
	    end
	
	    if self.remaining_time3 <= 0 then
	        self.remaining_time3 = TUNING.TOTAL_DAY_TIME*10
			if not TheSim:FindFirstEntityWithTag("charlie_fountain") then
	            self:SpawnFountain()
			end
     	end		
		
	    if self.remaining_time4 <= 0 then
	        self.remaining_time4 = TUNING.TOTAL_DAY_TIME*15
			if not TheSim:FindFirstEntityWithTag("charlie_ancient_gateway") then
	            self:SpawnAncientGateway()
			end
     	end				

	    if self.remaining_time5 <= 0 then
	        self.remaining_time5 = TUNING.TOTAL_DAY_TIME
			if c_countprefabs("buttercup", true) < 30 then
	            self:SpawnButtercup()
			end
     	end	
		
	end

    if TheSim:FindFirstEntityWithTag("shadow_rift") then
        if self.remaining_time2 > 0 and not TheSim:FindFirstEntityWithTag("shadow_forge") then
	        self.remaining_time2 = self.remaining_time2 - dt
	    end
	
	    if self.remaining_time2 <= 0 then
	        self.remaining_time2 = TUNING.TOTAL_DAY_TIME*10
			if not TheSim:FindFirstEntityWithTag("shadow_forge") then
	            self:SpawnShadowForge()
			end
     	end		
	end

    if TheSim:FindFirstEntityWithTag("charlie_throne")
		or TheSim:FindFirstEntityWithTag("charlie_npc")
		or TheSim:FindFirstEntityWithTag("shadow_rift")
	then
	    return
	end

    if self.remaining_time > 0 then
	    self.remaining_time = self.remaining_time - dt
	end
	
	if self.remaining_time <= 0 then
	    self.remaining_time = TUNING.TOTAL_DAY_TIME*5
	    self:SpawnRift()
	end

end

function ShadowRiftSpawner:OnUpdate(dt)

    if not self.enabled then
	    return
	end

    if TheSim:FindFirstEntityWithTag("shadow_forge") then
        if self.remaining_time3 > 0 and not TheSim:FindFirstEntityWithTag("charlie_fountain") then
	        self.remaining_time3 = self.remaining_time3 - dt
	    end

        if self.remaining_time4 > 0 and not TheSim:FindFirstEntityWithTag("charlie_ancient_gateway") then
	        self.remaining_time4 = self.remaining_time4 - dt
	    end
		
        if self.remaining_time5 > 0 and c_countprefabs("buttercup", true) < 30 then
	        self.remaining_time5 = self.remaining_time5 - dt
	    end		
	
	    if self.remaining_time3 <= 0 then
	        self.remaining_time3 = TUNING.TOTAL_DAY_TIME*10
			if not TheSim:FindFirstEntityWithTag("charlie_fountain") then
	            self:SpawnFountain()
			end
     	end		
		
	    if self.remaining_time4 <= 0 then
	        self.remaining_time4 = TUNING.TOTAL_DAY_TIME*15
			if not TheSim:FindFirstEntityWithTag("charlie_ancient_gateway") then
	            self:SpawnAncientGateway()
			end
     	end				

	    if self.remaining_time5 <= 0 then
	        self.remaining_time5 = TUNING.TOTAL_DAY_TIME
			if c_countprefabs("buttercup", true) < 30 then
	            self:SpawnButtercup()
			end
     	end	
		
	end

    if TheSim:FindFirstEntityWithTag("shadow_rift") then
        if self.remaining_time2 > 0 and not TheSim:FindFirstEntityWithTag("shadow_forge") then
	        self.remaining_time2 = self.remaining_time2 - dt
	    end
	
	    if self.remaining_time2 <= 0 then
	        self.remaining_time2 = TUNING.TOTAL_DAY_TIME*10
			if not TheSim:FindFirstEntityWithTag("shadow_forge") then
	            self:SpawnShadowForge()
			end
     	end		
	end
	
    if TheSim:FindFirstEntityWithTag("charlie_throne")
		or TheSim:FindFirstEntityWithTag("charlie_npc")
		or TheSim:FindFirstEntityWithTag("shadow_rift")
	then
	    return
	end

    if self.remaining_time > 0 then
	    self.remaining_time = self.remaining_time - dt
	end
	
	if self.remaining_time <= 0 then
	    self.remaining_time = TUNING.TOTAL_DAY_TIME*5
	    self:SpawnRift()
	end

end

return ShadowRiftSpawner