require("prefabutil")

local item_assets =
{
    Asset("ANIM", "anim/archive_lockbox.zip"),
}

local assets =
{
    Asset("ANIM", "anim/archive_knowledge_dispensary.zip"),
    Asset("ANIM", "anim/archive_knowledge_dispensary_b.zip"),
    Asset("ANIM", "anim/archive_knowledge_dispensary_c.zip"),
}

local prefabs =
{
    "ancient_fountain_of_knowledge",
}

local function onhammered(inst)
    inst.components.lootdropper:DropLoot()
    local fx = SpawnPrefab("collapse_big")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
	fx.SoundEmitter:PlaySound("dontstarve/common/destroy_metal")
    inst:Remove()
end

local function onhit(inst)
    inst.AnimState:PlayAnimation("use_pre")
	if inst.components.pickable ~= nil and inst.components.pickable.canbepicked then
	    inst.AnimState:PushAnimation("use_loop", true)
    else
        inst.AnimState:PushAnimation("idle")
	end
end

local function onpickedfn(inst, picker)
    local x, y, z = inst.Transform:GetWorldPosition()
    for i=1, math.random(2,6) do
	    inst.components.lootdropper:FlingItem(SpawnPrefab("thulecite_pieces"), Vector3(x, 3, z))
	end
    inst.AnimState:PlayAnimation("use_post")
    inst.AnimState:PushAnimation("idle")
end

local function makefullfn(inst)
    inst.AnimState:PlayAnimation("use_loop", true)
	if inst:HasTag("fountain_obsidian") then
	    if inst.components.endothermicmachine == nil then
	        inst:AddComponent("endothermicmachine")
		end
		inst:AddTag("endothermicmachine")
	end
end

local function fn()
    local inst = CreateEntity()   

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()

    inst.MiniMapEntity:SetIcon("butter_machine.tex")  --This is not actually yellow, so being exchanged

    MakeObstaclePhysics(inst, 0.66)
	
    inst.AnimState:SetBank("knowledge_dispensary")
    inst.AnimState:SetBuild("archive_knowledge_dispensary_b")
    inst.AnimState:PlayAnimation("idle")	
	
	inst:ListenForEvent("animover", function()
        if inst.AnimState:IsCurrentAnimation("idle") then
            if math.random() < 1/30 then
                local rand = math.random(1,3)
                inst.AnimState:PlayAnimation("taunt"..rand)
                inst.AnimState:PushAnimation("idle")
            else
                inst.AnimState:PlayAnimation("idle")
            end
        end	
	end)
	
    inst:AddTag("structure")
	inst:AddTag("ancient_fountain_of_knowledge")

    inst:AddComponent("inspectable")

    inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"cutstone","cutstone","thulecite","thulecite","thulecite","thulecite","thulecite","thulecite"})

	inst:AddComponent("pickable")
	inst.components.pickable:SetUp("thulecite_pieces", TUNING.TOTAL_DAY_TIME*5)
	inst.components.pickable.picksound = "dontstarve/HUD/get_gold"
    inst.components.pickable.onpickedfn = onpickedfn
    inst.components.pickable.makefullfn = makefullfn		

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
    inst.components.workable:SetOnFinishCallback(onhammered)
    inst.components.workable:SetOnWorkCallback(onhit)
	
    return inst
end

local function ondeploy(inst, pt, deployer)
    local ancient_fountain_of_knowledge = SpawnPrefab("ancient_fountain_of_knowledge") 
	if ancient_fountain_of_knowledge then
        ancient_fountain_of_knowledge.Transform:SetPosition(pt.x, pt.y, pt.z)
		ancient_fountain_of_knowledge.SoundEmitter:PlaySound("dontstarve/common/place_structure_stone")
		ancient_fountain_of_knowledge.AnimState:PlayAnimation("use_loop", true)
		inst:Remove()		
	end
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function IsPointOutsideInteriorWall(pt, radius)
    if TheCamera.interior then
        local interiorSpawner = GetWorld().components.interiorspawner 
        if GetWorld() and interiorSpawner and interiorSpawner.current_interior then
            local originpt = interiorSpawner:getSpawnOrigin()
            local width = interiorSpawner.current_interior.width
            local depth = interiorSpawner.current_interior.depth        
            local dMax = originpt.x + depth/2
            local dMin = originpt.x - depth/2
            local wMax = originpt.z + width/2
            local wMin = originpt.z - width/2        
            local dist = radius or 0.5
            if pt.x < dMin+dist or pt.x > dMax-dist or pt.z < wMin+dist or pt.z > wMax-dist then
                return true
            end
        end
    end
    return false
end

local function deploytest(inst, pt)
	if inst.IsPosSurroundedByLand ~= nil then
	    if not inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 3) then
		    return false
		end
	end
	local radius = 2	
    for i = -radius, radius, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x - radius, pt.y, pt.z + i) == GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(pt.x + radius, pt.y, pt.z + i) == GROUND.IMPASSABLE then
            return false
        end
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z - radius) == GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z + radius) == GROUND.IMPASSABLE then
            return false
        end
    end			
	
	if not IsPointOutsideInteriorWall(pt, 1) then
	    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 6, nil, {"NOBLOCK", "player", "FX"})
		local min_spacing = inst.components.deployable.min_spacing or 2
	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt ) < min_spacing*min_spacing then
					return false
				end
			end
		end
		return true
	end
	return false
end

local function item_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("archive_lockbox")
    inst.AnimState:SetBuild("archive_lockbox")
    inst.AnimState:PlayAnimation("idle")

    inst:AddComponent("inspectable")

    inst:AddComponent("inventoryitem")

    inst:AddTag("eyeturret")  --For action string
	
    if inst.GetIsOnWater ~= nil then
        inst:AddComponent("floatable")
	    inst.components.floatable:SetOnHitWaterFn(OnHitWater)
    	inst.components.floatable:SetOnHitLandFn(OnHitLand)
	    inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
    end	

    inst:AddComponent("deployable")
    inst.components.deployable.ondeploy = ondeploy
	inst.components.deployable.test = deploytest
    inst.components.deployable.placer = "ancient_fountain_of_knowledge_placer"
    inst.components.deployable.deploydistance = 1.5	
	inst.components.deployable.min_spacing = 2
	
	return inst
end

local function OnRefuseButter(inst, giver, item)
    if giver ~= nil and giver.components.talker ~= nil then
	    if inst.components.pickable ~= nil and inst.components.pickable.canbepicked then
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("Food is already prepared, I need to take it out first before I can put any items in it.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say("食物已经制作好了，我要先拿出来才能放物品。")
		    end	
		elseif item ~= nil and (item.prefab == "butterfly" or item.prefab == "butterflywings") then
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("It's already full, can't make any more butter, wait for one more round.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say("它已经满了，不能再制作更多黄油，还是等一轮吧。")
		    end			    
        elseif item ~= nil and item.prefab == "butter" then
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("It's already full, can't make any more waffles, wait for one more round.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say("它已经满了，不能再制作更多华夫饼，还是等一轮吧。")
		    end			
		else
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("Wrong item placement.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say("错误的物品放置。")
		    end		
		end
	end
end

local function OnAcceptButter(inst, giver, item)
    if item ~= nil then
	    if item.prefab == "butterfly" or item.prefab == "butterflywings" then
		    inst.butter_to_make = math.min(inst.butter_to_make + 1, inst.max_food)
		elseif item.prefab == "butter" then
		    inst.waffles_to_make = math.min(inst.waffles_to_make + 1, inst.max_food)
		end
	    if giver ~= nil and giver.components.talker ~= nil then
		    if item.prefab == "butterfly" or item.prefab == "butterflywings" then
			    if inst.butter_to_make < inst.max_food then
	                if TUNING.LANGUAGE == true then
		                giver.components.talker:Say("Placed successfully, butter will be produced after a period of time.")
		            elseif TUNING.LANGUAGE == false then
		                giver.components.talker:Say("放置成功，过一段时间就能产出黄油了。")
		            end					
				else
	                if TUNING.LANGUAGE == true then
		                giver.components.talker:Say("Placed successfully, full capacity, butter will be produced after a period of time.")
		            elseif TUNING.LANGUAGE == false then
		                giver.components.talker:Say("放置成功，已满格，过一段时间就能产出黄油了。")
		            end				
				end
			elseif item.prefab == "butter" then
			    if inst.waffles_to_make < inst.max_food then
	                if TUNING.LANGUAGE == true then
		                giver.components.talker:Say("Placed successfully, waffles will be produced after a period of time.")
		            elseif TUNING.LANGUAGE == false then
		                giver.components.talker:Say("放置成功，过一段时间就能产出华夫饼了。")
		            end				
				else
	                if TUNING.LANGUAGE == true then
		                giver.components.talker:Say("Placed successfully, full capacity, waffles will be produced after a period of time.")
		            elseif TUNING.LANGUAGE == false then
		                giver.components.talker:Say("放置成功，已满格，过一段时间就能产出华夫饼了。")
		            end				
				end
			end	
		end
	    if inst.butter_to_make > 0 or inst.waffles_to_make > 0 then
            inst.components.pickable:Resume()
		end
	end
end

local function AcceptTestButter(inst, item, giver)
    if inst.components.pickable ~= nil and inst.components.pickable.canbepicked then
        return false
    end
    if item ~= nil then 
	    if item.prefab == "butterfly" or item.prefab == "butterflywings" then
		    return inst.butter_to_make < inst.max_food
		elseif item.prefab == "butter" then
		    return inst.waffles_to_make < inst.max_food
		end
    end
	return false
end

local function onpickedfn_butter(inst, picker)
    local x, y, z = inst.Transform:GetWorldPosition()
	
	if inst.butter_to_make > 0 then
	    if picker ~= nil and picker.components.inventory ~= nil then
		    local butter = SpawnPrefab("butter")
	        picker.components.inventory:GiveItem(butter, nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
			butter.components.stackable:SetStackSize(inst.butter_to_make)
		end	
		inst.butter_to_make = 0
	end
	
	if inst.waffles_to_make > 0 then
	    if picker ~= nil and picker.components.inventory ~= nil then
		    local waffles = SpawnPrefab("waffles")
	        picker.components.inventory:GiveItem(waffles, nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
			waffles.components.stackable:SetStackSize(inst.waffles_to_make)
		end		
		inst.waffles_to_make = 0
	end
	
	if math.random() < 0.01 then
	    if picker ~= nil and picker.components.inventory ~= nil then
	        picker.components.inventory:GiveItem(SpawnPrefab("pvz_sun"), nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
		else
		    inst.components.lootdropper:FlingItem(SpawnPrefab("pvz_sun"), Vector3(x, 3, z))
		end
	end
	
    inst.AnimState:PlayAnimation("use_post")
    inst.AnimState:PushAnimation("idle")
	
	inst.components.pickable:Pause()
end

local function OnSaveButter(inst, data)
    data.butter_to_make = inst.butter_to_make or 0
	data.waffles_to_make = inst.waffles_to_make or 0
end

local function OnLoadButter(inst, data)
    if data ~= nil then
	    inst.butter_to_make = data.butter_to_make or 0
		inst.waffles_to_make = data.waffles_to_make or 0
	end
end

local function butter_fn()
    local inst = CreateEntity()   

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()

    inst.MiniMapEntity:SetIcon("fountain_of_knowledge_a.tex")  --This is actually yellow, so being exchanged

    MakeObstaclePhysics(inst, 0.66)
	
    inst.AnimState:SetBank("knowledge_dispensary")
    inst.AnimState:SetBuild("archive_knowledge_dispensary")
    inst.AnimState:PlayAnimation("idle")	
	
	inst:ListenForEvent("animover", function()
        if inst.AnimState:IsCurrentAnimation("idle") then
            if math.random() < 1/30 then
                local rand = math.random(1,3)
                inst.AnimState:PlayAnimation("taunt"..rand)
                inst.AnimState:PushAnimation("idle")
            else
                inst.AnimState:PlayAnimation("idle")
            end
        end	
	end)
	
    inst:AddTag("structure")
	inst:AddTag("butter_machine")
	inst:AddTag("cannot_be_reaped") 
	
	inst.butter_to_make = 0
	inst.waffles_to_make = 0
	inst.max_food = 40

    inst:AddComponent("inspectable")

    inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"cutstone"}) 

	inst:AddComponent("pickable")
	inst.components.pickable:SetUp(nil, TUNING.TOTAL_DAY_TIME)
	inst.components.pickable.picksound = "dontstarve/common/butterfly_trap"
    inst.components.pickable.onpickedfn = onpickedfn_butter
    inst.components.pickable.makefullfn = makefullfn	
	
	inst:AddComponent("trader")
	inst.components.trader.acceptnontradable = true
	inst.components.trader.onrefuse = OnRefuseButter
	inst.components.trader.onaccept = OnAcceptButter
    inst.components.trader:SetAcceptTest(AcceptTestButter)	

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
    inst.components.workable:SetOnFinishCallback(onhammered)
    inst.components.workable:SetOnWorkCallback(onhit)
	
	inst.OnSave = OnSaveButter
	inst.OnLoad = OnLoadButter
	
    return inst
end

local function ondeploy_butter(inst, pt, deployer)
    local butter_machine = SpawnPrefab("butter_machine") 
	if butter_machine then
        butter_machine.Transform:SetPosition(pt.x, pt.y, pt.z)
		butter_machine.SoundEmitter:PlaySound("dontstarve/common/place_structure_stone")
		if butter_machine.components.pickable ~= nil then
		    butter_machine.components.pickable:Pause()
		    butter_machine.components.pickable:MakeEmpty()
		end
		inst:Remove()		
	end
end

local function butter_item_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("archive_lockbox")
    inst.AnimState:SetBuild("archive_lockbox")
    inst.AnimState:PlayAnimation("idle")

    inst:AddComponent("inspectable")

    inst:AddComponent("inventoryitem")

    inst:AddTag("eyeturret")  --For action string
	
    if inst.GetIsOnWater ~= nil then
        inst:AddComponent("floatable")
	    inst.components.floatable:SetOnHitWaterFn(OnHitWater)
    	inst.components.floatable:SetOnHitLandFn(OnHitLand)
	    inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
    end	

    inst:AddComponent("deployable")
    inst.components.deployable.ondeploy = ondeploy_butter
	inst.components.deployable.test = deploytest
    inst.components.deployable.placer = "butter_machine_placer"
    inst.components.deployable.deploydistance = 1.5	
	inst.components.deployable.min_spacing = 2
	
	return inst
end

local function onpickedfn_obsidian(inst, picker)
    local x, y, z = inst.Transform:GetWorldPosition()
    for i=1, math.random(2,3) do
	    inst.components.lootdropper:FlingItem(SpawnPrefab("obsidian"), Vector3(x, 3, z))
	end
	for i=1, math.random(5,8) do
        inst.components.lootdropper:FlingItem(SpawnPrefab("charcoal"), Vector3(x, 3, z))
	end
    inst.AnimState:PlayAnimation("use_post")
    inst.AnimState:PushAnimation("idle")
	inst:RemoveComponent("endothermicmachine")
	inst:RemoveTag("endothermicmachine")
end

local function GetHeatFn(inst, observer)
    if inst.components.pickable ~= nil and inst.components.pickable.canbepicked then
	    return -100
	else
	    return 0
	end
end

local function obsidian_fn()
    local inst = CreateEntity()   

    inst.entity:AddTransform()

    if inst.GetIsOnWater == nil then
	    inst:DoTaskInTime(0, inst.Remove)
	    return inst
	end

    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()

    inst.MiniMapEntity:SetIcon("fountain_obsidian.tex")

    MakeObstaclePhysics(inst, 0.66)
	
    inst.AnimState:SetBank("knowledge_dispensary")
    inst.AnimState:SetBuild("archive_knowledge_dispensary_c")
    inst.AnimState:PlayAnimation("idle")	
	
	inst:ListenForEvent("animover", function()
        if inst.AnimState:IsCurrentAnimation("idle") then
            if math.random() < 1/30 then
                local rand = math.random(1,3)
                inst.AnimState:PlayAnimation("taunt"..rand)
                inst.AnimState:PushAnimation("idle")
            else
                inst.AnimState:PlayAnimation("idle")
            end
        end	
	end)
	
    inst:AddTag("structure")
	inst:AddTag("fountain_obsidian") 
	inst:AddTag("cannot_be_reaped") 

    inst:AddComponent("inspectable")

    inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"obsidian"})

	inst:AddComponent("pickable")
	inst.components.pickable:SetUp("obsidian", TUNING.TOTAL_DAY_TIME*4)
	inst.components.pickable.picksound = "dontstarve_DLC002/common/obsidian_wetsizzles"
    inst.components.pickable.onpickedfn = onpickedfn_obsidian
    inst.components.pickable.makefullfn = makefullfn	

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
    inst.components.workable:SetOnFinishCallback(onhammered)
    inst.components.workable:SetOnWorkCallback(onhit)

    if inst.components.pickable ~= nil and inst.components.pickable.canbepicked then
	    if inst.components.endothermicmachine == nil then
            inst:AddComponent("endothermicmachine")
		end
		inst:AddTag("endothermicmachine")
	end
	
    inst:AddComponent("heater")
    inst.components.heater.heatfn = GetHeatFn
    inst.components.heater.iscooler = true	
	
    return inst
end

local function ondeploy_obsidian(inst, pt, deployer)
    local fountain_obsidian = SpawnPrefab("fountain_obsidian") 
	if fountain_obsidian then
        fountain_obsidian.Transform:SetPosition(pt.x, pt.y, pt.z)
		fountain_obsidian.SoundEmitter:PlaySound("dontstarve/common/place_structure_stone")
		fountain_obsidian.AnimState:PlayAnimation("use_loop", true)
		inst:Remove()		
	end
end

local function obsidian_item_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()

    if inst.GetIsOnWater == nil then
	    inst:DoTaskInTime(0, inst.Remove)
	    return inst
	end

    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("archive_lockbox")
    inst.AnimState:SetBuild("archive_lockbox")
    inst.AnimState:PlayAnimation("idle")

    inst:AddComponent("inspectable")

    inst:AddComponent("inventoryitem")

    inst:AddTag("eyeturret")  --For action string
	
    if inst.GetIsOnWater ~= nil then
        inst:AddComponent("floatable")
	    inst.components.floatable:SetOnHitWaterFn(OnHitWater)
    	inst.components.floatable:SetOnHitLandFn(OnHitLand)
	    inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
    end	

    inst:AddComponent("deployable")
    inst.components.deployable.ondeploy = ondeploy_obsidian
	inst.components.deployable.test = deploytest
    inst.components.deployable.placer = "fountain_obsidian_placer"
    inst.components.deployable.deploydistance = 1.5	
	inst.components.deployable.min_spacing = 2
	
	return inst
end

return Prefab("ancient_fountain_of_knowledge", fn, assets),
    MakePlacer("ancient_fountain_of_knowledge_placer", "knowledge_dispensary", "archive_knowledge_dispensary_b", "idle"),
	Prefab("ancient_fountain_of_knowledge_item", item_fn, item_assets, prefabs),
	Prefab("butter_machine", butter_fn, assets),
	Prefab("butter_machine_item", butter_item_fn, item_assets, prefabs),
	MakePlacer("butter_machine_placer", "knowledge_dispensary", "archive_knowledge_dispensary", "idle"),
	Prefab("fountain_obsidian", obsidian_fn, assets),
	Prefab("fountain_obsidian_item", obsidian_item_fn, item_assets, prefabs),
	MakePlacer("fountain_obsidian_placer", "knowledge_dispensary", "archive_knowledge_dispensary_c", "idle")