local assets =
{
    Asset("ANIM", "anim/hat_antlion.zip"),
	Asset("ANIM", "anim/ui_antlionhat_1x1.zip"),
}

local prefabs =
{
    "turf_smoke_fx",
	"gridplacer_turfhat",
}

local function IsEdgeAtPoint(x, y, z, radius)
    local radius = radius or 1

    if GetWorld().Map:GetTileAtPoint(x, y, z) ~= GROUND.IMPASSABLE then
        return false
    end
	
    for i = -radius, radius, 1 do
        if GetWorld().Map:GetTileAtPoint(x - radius, y, z + i) ~= GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(x + radius, y, z + i) ~= GROUND.IMPASSABLE then
            return true
        end
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if GetWorld().Map:GetTileAtPoint(x + i, y, z - radius) ~= GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(x + i, y, z + radius) ~= GROUND.IMPASSABLE then
            return true
        end
    end
	
    return false

end

local function IsPointTooHardOrPoisonous(x, y, z)
    if not IsDLCEnabled(PORKLAND_DLC) then
	    return false  --There are currently no too hard or toxic turfs in non-Hamlet
	end
	local tile = GetWorld().Map:GetTileAtPoint(x, y, z)
    return tile == GROUND.DEEPRAINFOREST or tile == GROUND.GASJUNGLE
end

local function Terraform(inst, owner)
    local item_tile = nil
    local item = inst.components.container:GetItemInSlot(1)
    if item and item.data and item.data.tile then
        item_tile = item.data.tile
    end
    local pt = inst:GetPosition()	
    local original_tile_type = GetWorld().Map:GetTileAtPoint(pt.x, pt.y, pt.z)
    if item_tile == original_tile_type then
        return
    end	
    if item_tile ~= nil and item.components.deployable and item.components.deployable:CanDeploy(pt) then
        item.components.deployable.ondeploy(item, pt, owner)
		SpawnPrefab("turf_smoke_fx").Transform:SetPosition(GetWorld().Map:GetTileCenterPoint(pt.x, pt.y, pt.z))
		inst.SoundEmitter:PlaySound("dontstarve/wilson/dig")
		local tile_type = GetWorld().Map:GetTileAtPoint(pt.x, pt.y, pt.z)
		if item_tile == tile_type then
		    inst.components.finiteuses:Use()
            return
		end
    end
    if inst.components.terraformer ~= nil 
	    and inst.components.terraformer:CanTerraformPoint(pt) 
		and not IsEdgeAtPoint(pt.x, pt.y, pt.z) 
	    and not IsPointTooHardOrPoisonous(pt.x, pt.y, pt.z)
	then
        inst.components.terraformer:Terraform(pt)
		SpawnPrefab("turf_smoke_fx").Transform:SetPosition(GetWorld().Map:GetTileCenterPoint(pt.x, pt.y, pt.z))
        inst.SoundEmitter:PlaySound("dontstarve/wilson/dig")
        inst.components.finiteuses:Use()
    end
	
end

local function onequip(inst, owner)
	owner.AnimState:OverrideSymbol("swap_hat", "hat_antlion", "swap_hat")
	owner.AnimState:Show("HAT")
	owner.AnimState:Show("HAIR_HAT")
	owner.AnimState:Hide("HAIR_NOHAT")
	owner.AnimState:Hide("HAIR")
	
	if owner:HasTag("player") then
		owner.AnimState:Hide("HEAD")
		owner.AnimState:Show("HEAD_HAIR")
		owner.AnimState:Hide("HAIRFRONT")
	end	

    if inst.components.autoterraformer ~= nil then
        inst.components.autoterraformer:StartTerraforming()
    end
	
	if owner.gridplacer == nil and owner:HasTag("player") then
	    owner.gridplacer = SpawnPrefab("gridplacer_turfhat")
        owner.gridplacer:SetPlayer(GetPlayer())
	end
	
    if inst.components.container ~= nil then
        inst.components.container:Open(owner)
    end
	if inst.digturftask == nil and owner.components.locomotor ~= nil then
	    inst.digturftask = inst:DoPeriodicTask(0.25, function() Terraform(inst, owner) end)
	end	
end

local function onunequip(inst, owner)
	owner.AnimState:Hide("HAT")
	owner.AnimState:Hide("HAIR_HAT")
	owner.AnimState:Show("HAIR_NOHAT")
	owner.AnimState:Show("HAIR")

	if owner:HasTag("player") then
		owner.AnimState:Show("HEAD")
		owner.AnimState:Hide("HEAD_HAIR")
		owner.AnimState:Show("HAIRFRONT")
	end

    if inst.components.autoterraformer ~= nil then
        inst.components.autoterraformer:StopTerraforming()
    end
	
	if owner.gridplacer ~= nil then
	    owner.gridplacer:Remove()
		owner.gridplacer = nil
	end
	
    if inst.components.container ~= nil then
        inst.components.container:Close()
    end	
	if inst.digturftask ~= nil then 
	    inst.digturftask:Cancel() 
		inst.digturftask = nil 
	end
end

local function TestItem_AntlionHat(inst, item, slot)
	return item:HasTag("groundtile")
end

local function onfinished(inst)
    inst.components.container:DropEverything(inst:GetPosition())
    inst:Remove()
end

local function onfinishterraforming(inst, x, y, z) 
    SpawnPrefab("turf_smoke_fx").Transform:SetPosition(GetWorld().Map:GetTileCenterPoint(x, y, z))
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local slotpos = {}

for y = 0, 0 do
	table.insert(slotpos, Vector3(0, -y*75 -10 ,0))
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("antlionhat")
    inst.AnimState:SetBuild("hat_antlion")
    inst.AnimState:PlayAnimation("anim")

    inst:AddTag("hat")
    inst:AddTag("turfhat")
	
    inst:AddComponent("inventoryitem")

	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		
	end

    inst:AddComponent("inspectable")

    inst:AddComponent("tradable")

    inst:AddComponent("equippable")
    inst.components.equippable.equipslot = EQUIPSLOTS.HEAD
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)

    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetOnFinished(onfinished)
    inst.components.finiteuses:SetMaxUses(TUNING.ANTLIONHAT_USES)
    inst.components.finiteuses:SetUses(TUNING.ANTLIONHAT_USES)

    inst:AddComponent("container")
    inst.components.container:SetNumSlots(#slotpos)
    inst.components.container.widgetslotpos = slotpos
	inst.components.container.canbeopened = true
	inst.components.container.itemtestfn = TestItem_AntlionHat
    inst.components.container.widgetpos = Vector3(-5,-200,0)
	inst.components.container.side_align_tip = 0
	inst.components.container.widgetanimbank = "ui_antlionhat_1x1"
	inst.components.container.widgetanimbuild = "ui_antlionhat_1x1"
    inst.components.container.excludefromcrafting =	true
	inst.components.container.type = "hand_inv"
 
    inst:AddComponent("terraformer")
	inst.components.terraformer.CollectPointActions = function() end 

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(TUNING.WATERPROOFNESS_SMALL)
    end
	
    return inst
end

return Prefab("antlionhat", fn, assets, prefabs) 