local assets =
{
	Asset("ANIM", "anim/armor_sanity_curve.zip"),
}

local prefabs = 
{

}

local function OnBlocked(owner) 
    if owner ~= nil and owner.SoundEmitter ~= nil then
        owner.SoundEmitter:PlaySound("dontstarve/wilson/hit_nightarmour") 
	end
end

local function GraduallyDisappearAndAppear(owner, dt) 
    if owner ~= nil and owner.TransparencyPercentage ~= nil and dt ~= nil then
	    if not owner.ShouldAppearNow then
	        owner.TransparencyPercentage = math.max(owner.TransparencyPercentage - dt, 0)
		else
		    owner.TransparencyPercentage = math.min(owner.TransparencyPercentage + dt, 1)
		end
	    if owner:HasTag("shadow_form_on") then
		    owner.AnimState:SetMultColour(TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT, owner.TransparencyPercentage)
		else
		    owner.AnimState:SetMultColour(1, 1, 1, owner.TransparencyPercentage)
		end 
		if owner.TransparencyPercentage <= 0 then
		    owner:AddTag("notarget") 
		    owner.ShouldAppearNow = true
		elseif owner.TransparencyPercentage >= 1 and owner.ShouldAppearNow then
		    if not owner.was_no_target then
		        owner:RemoveTag("notarget")
			end
		    owner.ShouldAppearNow = nil
		end
	end
end

local function onequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_body", "armor_sanity_curve", "swap_body")
    inst:ListenForEvent("blocked", OnBlocked, owner)
    if owner.components.health ~= nil then
        owner.components.health.fire_damage_scale = owner.components.health.fire_damage_scale - 1
    end
	owner.was_no_target = owner:HasTag("notarget") or nil
	owner.TransparencyPercentage = 1
	if not owner:HasTag("shadow_form_on") and owner.DynamicShadow ~= nil then
	    owner.DynamicShadow:Enable(false)
	end
	if owner.ArmorVoidTask == nil then
		owner.ArmorVoidTask = owner:DoPeriodicTask(1/30, GraduallyDisappearAndAppear, nil, 1/30)
	end	
end

local function onunequip(inst, owner) 
    owner.AnimState:ClearOverrideSymbol("swap_body")
    inst:RemoveEventCallback("blocked", OnBlocked, owner)
    if owner.components.health ~= nil then
        owner.components.health.fire_damage_scale = owner.components.health.fire_damage_scale + 1
    end	
	if owner.ArmorVoidTask ~= nil then
	    owner.ArmorVoidTask:Cancel()
		owner.ArmorVoidTask = nil
		owner.TransparencyPercentage = nil
		owner.ShouldAppearNow = nil
		if not owner.was_no_target then
		    owner:RemoveTag("notarget")
		end
		owner.was_no_target = nil
	end
	if owner:HasTag("shadow_form_on") then
	    if TUNING.SHADOW_CHARLIE_MULT ~= 0 then
	        owner.AnimState:SetMultColour(TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT)
		else
		    owner.AnimState:SetMultColour(TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT, 0.5)
		end
	else
	    owner.AnimState:SetMultColour(1, 1, 1, 1)
		if owner.DynamicShadow ~= nil then
		    owner.DynamicShadow:Enable(true)
		end
	end
end

local function OnRepaired(inst, doer, repair_item)
	if doer ~= nil and doer.SoundEmitter ~= nil then
	    doer.SoundEmitter:PlaySound("dontstarve/HUD/repair_clothing")
   	end	
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
		inst.float_fx.Transform:SetScale(0.5, 0.75, 0.75)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetScale(0.5, 0.75, 0.75)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function fn()
	local inst = CreateEntity()
    
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("armor_sanity_curve")
    inst.AnimState:SetBuild("armor_sanity_curve")
    inst.AnimState:PlayAnimation("anim")
    
    inst:AddTag("sanity")
	inst:AddTag("shadow_item")
	--inst:AddTag("fire_immune_item")  --Don't need this tag as this armor is 100% water proofness, allowing Shadow Charlie to take no damage from light.
    
    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.foleysound = "dontstarve/movement/foley/nightarmour"
	
    inst:AddComponent("armor")
    inst.components.armor:InitCondition(1500, 1)
    
    inst:AddComponent("equippable")
    inst.components.equippable.equipslot = EQUIPSLOTS.BODY
	inst.components.equippable.insulated = true  --from electricity, not temperature
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)
 
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(1)
	else
 	    inst:AddComponent("dapperness")
		inst.components.dapperness.mitigates_rain = true	
    end 

    inst:AddComponent("shadowrepairable")
	inst.components.shadowrepairable:SetRepairMaterial("voidcloth")
	inst.components.shadowrepairable:SetOnRepaired(OnRepaired)
	
	if inst.GetIsOnWater ~= nil then
	    MakeInventoryFloatable(inst, "idle_water", "anim")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
	    inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)   		
    end

    return inst
end

return Prefab("armor_shadow_void", fn, assets, prefabs) 