require "stategraphs/SGfriendlybee"

local assets =
{
	Asset("ANIM", "anim/bee.zip"),
	Asset("ANIM", "anim/bee_build.zip"),
	Asset("ANIM", "anim/bee_angry_build.zip"),
	Asset("SOUND", "sound/bee.fsb"),
}
    
local prefabs =
{
	"stinger",
	"honey",
}

local sounds = 
{
	takeoff = "dontstarve/bee/bee_takeoff",
	attack = "dontstarve/bee/bee_attack",
	buzz = "dontstarve/bee/bee_fly_LP",
	hit = "dontstarve/bee/bee_hurt",
	death = "dontstarve/bee/bee_death",
}

local brain = require("brains/friendlybeebrain")

local function RetargetFilterFn(inst)
    return function(guy)
        return
                (not guy.LightWatcher or guy.LightWatcher:IsInLight())
                and  guy.components.health
                and  not guy.components.health:IsDead()
                and  guy.components.combat
                and  inst.components.combat:CanTarget(guy)
                and  GetPlayer().components.combat.target == guy
    end
end

local CANT_TAGS = {"FX", "NOCLICK", "INLIMBO", "abigail"}

local function RetargetFn(inst)
    return FindEntity(inst, TUNING.PIG_TARGET_DIST, RetargetFilterFn(inst), nil, CANT_TAGS) or false
end

local function KeepTargetFn(inst, target)
    return (
	    inst.components.combat:CanTarget(target)
        and (not target.LightWatcher or target.LightWatcher:IsInLight())
        and not ( target.sg and target.sg:HasStateTag("transform") )
		) or false		
end

local function fn(Sim)
	local inst = CreateEntity()
	
    inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddLightWatcher()
	inst.entity:AddDynamicShadow()

    inst.AnimState:SetBank("bee")
	inst.AnimState:SetBuild("bee_build")
    inst.AnimState:PlayAnimation("idle")
    inst.AnimState:SetRayTestOnBB(true)
	
	inst.DynamicShadow:SetSize( 0.8, 0.5 )
    inst.Transform:SetFourFaced()
	inst.Transform:SetScale(1.1, 1.1, 1.1)

    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    MakePoisonableCharacter(inst)
	    MakeAmphibiousCharacterPhysics(inst, 1, 0.5)
	else
	    MakeCharacterPhysics(inst, 1, 0.5)
	end
	
	inst.Physics:SetCollisionGroup(COLLISION.FLYERS)
	inst.Physics:ClearCollisionMask()
	inst.Physics:CollidesWith(COLLISION.FLYERS)
	
	if IsDLCEnabled(PORKLAND_DLC) then
	    inst.Physics:CollidesWith(GetWorldCollision())
	    inst.Physics:CollidesWith(COLLISION.INTWALL)
	elseif IsDLCEnabled(CAPY_DLC) then
	    inst.Physics:CollidesWith(COLLISION.GROUND)
	else
	    inst.Physics:CollidesWith(COLLISION.WORLD)
	end

	inst:AddTag("bee")
	inst:AddTag("insect")
	inst:AddTag("smallcreature")
	inst:AddTag("cattoyairborne")
	inst:AddTag("flying")
	inst:AddTag("companion")

	inst:AddComponent("lootdropper")
	inst.components.lootdropper:AddRandomLoot("bundlewrap_blueprint", 1)
	inst.components.lootdropper:AddRandomLoot("honey", 4)
	inst.components.lootdropper:AddRandomLoot("stinger", 20)   
	inst.components.lootdropper.numrandomloot = 1
	
	inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
	inst.components.locomotor:EnableGroundSpeedMultiplier(false)
	inst.components.locomotor:SetTriggersCreep(false)
	
	inst:SetStateGraph("SGfriendlybee")
	
    MakeSmallBurnableCharacter(inst, "body", Vector3(0, -1, 1))
    MakeTinyFreezableCharacter(inst, "body", Vector3(0, -1, 1))	

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(TUNING.BEE_HEALTH*1.25)
	
    inst:AddComponent("combat")
    inst.components.combat.hiteffectsymbol = "body"
	inst.components.combat:SetDefaultDamage(TUNING.BEE_DAMAGE*1.5)
	inst.components.combat:SetAttackPeriod(TUNING.BEE_ATTACK_PERIOD)
	inst.components.combat:SetKeepTargetFunction(KeepTargetFn)
	inst.components.combat:SetRetargetFunction(2, RetargetFn)
	inst.components.combat:SetRange(1)
	inst.components.combat:SetTarget(nil)
	
	inst:AddComponent("pollinator")

    inst:AddComponent("follower")

    inst:RemoveEventCallback("attacked", function(inst, data)
	    if inst.components.follower.leader == data.attacker then
		    inst.components.follower:SetLeader(nil)
	    end	
	end)

    if inst.components.follower.SetFollowExitDestinations ~= nil then
        inst.components.follower:SetFollowExitDestinations({EXIT_DESTINATION.LAND, EXIT_DESTINATION.WATER})
    end

	if GetPlayer() and GetPlayer().components.leader ~= nil then
		GetPlayer().components.leader:AddFollower(inst)        
	end	
	
	inst.sounds = sounds
	
	inst:SetBrain(brain)
	
	return inst	
end

return Prefab("bee_friendly", fn, assets, prefabs)