local assets =
{
    Asset("ANIM", "anim/berries_juicy.zip"),
}

local prefabs =
{
    "spoiled_food",
}

local function OnHitWater(inst, size, vert_offset, x_scale, y_scale, z_scale)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		if size ~= nil then
		    inst.float_fx.AnimState:PlayAnimation("idle_front_"..size, true)
		end
		if vert_offset ~= nil then
		    inst.float_fx.Transform:SetPosition(0, vert_offset, 0)
		end
		if x_scale ~= nil then
		    inst.float_fx.Transform:SetScale(x_scale, y_scale or x_scale, z_scale or x_scale)
		end
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		if size ~= nil then
		    inst.float_fx2.AnimState:PlayAnimation("idle_back_"..size, true)
		end
		if vert_offset ~= nil then
		    inst.float_fx2.Transform:SetPosition(0, vert_offset, 0)
		end
		if x_scale ~= nil then
		    inst.float_fx2.Transform:SetScale(x_scale, y_scale or x_scale, z_scale or x_scale)
		end		
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function fn()
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
	
    MakeInventoryPhysics(inst)
	
    inst.AnimState:SetBank("berries_juicy")
    inst.AnimState:SetBuild("berries_juicy")
    inst.AnimState:PlayAnimation("idle")

    inst:AddTag("smallfood_for_charlie")

    inst:AddComponent("edible")
    inst.components.edible.healthvalue = TUNING.HEALING_TINY
    inst.components.edible.hungervalue = TUNING.CALORIES_SMALL
    inst.components.edible.sanityvalue = 0
    inst.components.edible.foodtype = "VEGGIE"
	inst.components.edible.foodstate = "RAW"

    inst:AddComponent("perishable")
    inst.components.perishable:SetPerishTime(TUNING.TOTAL_DAY_TIME*2)
    inst.components.perishable:StartPerishing()
    inst.components.perishable.onperishreplacement = "spoiled_food"
	
    inst:AddComponent("stackable")
    inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

    inst:AddComponent("inspectable")
    inst:AddComponent("inventoryitem")		

    MakeSmallBurnable(inst)
    MakeSmallPropagator(inst)

    inst:AddComponent("bait")

    inst:AddComponent("tradable")

    inst:AddComponent("cookable")
    inst.components.cookable.product = "berries_juicy_cooked"

	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    MakeBlowInHurricane(inst, TUNING.WINDBLOWN_SCALE_MIN.LIGHT, TUNING.WINDBLOWN_SCALE_MAX.LIGHT)
		
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(function(inst)
			OnHitWater(inst, "med", nil, 0.7)
		end)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
        inst:ListenForEvent("onputininventory", OnHitLand)	
	end	
	
    return inst
end

local function cooked_fn()
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
	
    MakeInventoryPhysics(inst)
	
    inst.AnimState:SetBank("berries_juicy")
    inst.AnimState:SetBuild("berries_juicy")
    inst.AnimState:PlayAnimation("cooked")

    inst:AddTag("smallfood_for_charlie")

    inst:AddComponent("edible")
    inst.components.edible.healthvalue = TUNING.HEALING_SMALL
    inst.components.edible.hungervalue = TUNING.CALORIES_MEDSMALL
    inst.components.edible.sanityvalue = 0
    inst.components.edible.foodtype = "VEGGIE"
	inst.components.edible.foodstate = "COOKED"

    inst:AddComponent("perishable")
    inst.components.perishable:SetPerishTime(TUNING.TOTAL_DAY_TIME)
    inst.components.perishable:StartPerishing()
    inst.components.perishable.onperishreplacement = "spoiled_food"
	
    inst:AddComponent("stackable")
    inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

    inst:AddComponent("inspectable")
    inst:AddComponent("inventoryitem")		

    MakeSmallBurnable(inst)
    MakeSmallPropagator(inst)

    inst:AddComponent("bait")

    inst:AddComponent("tradable")

	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(function(inst)
			OnHitWater(inst)
		end)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
        inst:ListenForEvent("onputininventory", OnHitLand)	
	end	
		
    return inst
end

return Prefab("berries_juicy", fn, assets, prefabs),
    Prefab("berries_juicy_cooked", cooked_fn, assets, prefabs)