local assets =
{
    Asset("ANIM", "anim/berrybush_juicy.zip"),
}

local prefabs =
{
    "dug_berrybush_juicy",
    "perd",
    "twigs",
--    "spoiled_food",
}

local function setberries(inst, pct)
    if inst._setberriesonanimover then
        inst._setberriesonanimover = nil
        inst:RemoveEventCallback("animover", setberries)
    end

    local berries =
        (pct == nil and "") or
        (pct >= .9 and "berriesmost") or
        (pct >= .33 and "berriesmore") or
        "berries"

    for i, v in ipairs({ "berries", "berriesmore", "berriesmost" }) do
        if v == berries then
            inst.AnimState:Show(v)
        else
            inst.AnimState:Hide(v)
        end
    end
end

local function setberriesonanimover(inst)
    if inst._setberriesonanimover then
        setberries(inst, nil)
    else
        inst._setberriesonanimover = true
        inst:ListenForEvent("animover", setberries)
    end
end

local function cancelsetberriesonanimover(inst)
    if inst._setberriesonanimover then
        setberries(inst, nil)
    end
end


local function ontransplantfn(inst)
    inst.AnimState:PlayAnimation("dead")
    setberries(inst, nil)
	inst.components.pickable:MakeBarren()
end

local function makeemptyfn(inst)
    if POPULATING then 
	    inst.AnimState:PushAnimation("idle")
	else
        inst.AnimState:PlayAnimation("dead_to_idle")
        inst.AnimState:PushAnimation("idle")
    end
    setberries(inst, nil)
end

local function pickanim(inst)
	if inst.components.pickable then
		if inst.components.pickable:CanBePicked() then
			local percent = 0
			if inst.components.pickable then
				percent = inst.components.pickable.cycles_left / inst.components.pickable.max_cycles
			end
			if percent >= .9 then
				return "berriesmost"
			elseif percent >= .33 then
				return "berriesmore"
			else
				return "berries"
			end
		else
			if inst.components.pickable:IsBarren() then
				return "idle_dead"
			else
				return "idle"
			end
		end
	end
	return "idle"
end

local function shake(inst)
    if inst.components.pickable ~= nil and
        not inst.components.pickable:CanBePicked() and
        inst.components.pickable:IsBarren() then
        inst.AnimState:PlayAnimation("shake_dead")
        inst.AnimState:PushAnimation("dead", false)
    else
        inst.AnimState:PlayAnimation("shake")
        inst.AnimState:PushAnimation("idle")
    end
    cancelsetberriesonanimover(inst)
end

local function spawnperd(inst)
    if inst:IsValid() then
		local perd = SpawnPrefab("perd")
		local spawnpos = Vector3(inst.Transform:GetWorldPosition())
		spawnpos = spawnpos + TheCamera:GetDownVec()
		perd.Transform:SetPosition(spawnpos:Get() )
		perd.sg:GoToState("appear")
		perd.components.homeseeker:SetHome(inst)
		shake(inst)
	end
end

local function onpickedfn(inst, picker)

    if inst.components.pickable ~= nil then
        if inst.components.pickable:IsBarren() then
            inst.AnimState:PlayAnimation("idle_to_dead")
            inst.AnimState:PushAnimation("dead", false)
            setberries(inst, nil)
        else
            inst.AnimState:PlayAnimation("picked")
            inst.AnimState:PushAnimation("idle")
            setberriesonanimover(inst)
        end
    end
	
	if inst.components.pickable ~= nil and inst.components.lootdropper ~= nil then
        local pt = inst:GetPosition()
        pt.y = pt.y + (inst.components.pickable.dropheight or 0)
        for i = 1, 3 do
            inst.components.lootdropper:SpawnLootPrefab("berries_juicy", pt)
        end	
	end
	
	if picker ~= nil 
	    and not GetWorld():HasTag("shipwrecked")
	    and not picker:HasTag("berrythief") 
	    and math.random() < TUNING.PERD_SPAWNCHANCE 
	then
	    inst:DoTaskInTime(3+math.random()*3, spawnperd)
	end
end

local function getregentimefn_juicy(inst)
	if inst.components.pickable then
		local num_cycles_passed = math.min(inst.components.pickable.max_cycles - inst.components.pickable.cycles_left, 0)
		return TUNING.BERRY_JUICY_REGROW_TIME + TUNING.BERRY_JUICY_REGROW_INCREASE*num_cycles_passed + math.random()*TUNING.BERRY_JUICY_REGROW_VARIANCE
	else
		return TUNING.BERRY_JUICY_REGROW_TIME
	end
end

local function onregenfn(inst)
	inst.AnimState:PlayAnimation(pickanim(inst))
end

local function makebarrenfn(inst)
    if (inst.components.pickable and inst.components.pickable.withered) or inst.AnimState:IsCurrentAnimation("idle") then
	    inst.AnimState:PlayAnimation("idle_to_dead")
        inst.AnimState:PushAnimation("dead", false)
    else
        inst.AnimState:PlayAnimation("dead")
    end
	cancelsetberriesonanimover(inst)
end

local function makefullfn(inst)
    local anim = "idle"
    local berries = nil
    if inst.components.pickable ~= nil then
        if inst.components.pickable:CanBePicked() then
            berries = inst.components.pickable.cycles_left ~= nil and inst.components.pickable.cycles_left / inst.components.pickable.max_cycles or 1
        elseif inst.components.pickable:IsBarren() then
            anim = "dead"
        end
    end
    if anim ~= "idle" then
        inst.AnimState:PlayAnimation(anim)
    elseif POPULATING then
        inst.AnimState:PlayAnimation("idle", true)
    else
        inst.AnimState:PlayAnimation("grow")
        inst.AnimState:PushAnimation("idle", true)
    end
    setberries(inst, berries)
end

local function onworked_juicy(inst, worker, workleft)
    --This is possible when beaver is gnaw-digging the bush,
    --and the expected behaviour should be same as jostling.
    if workleft > 0 and
        inst.components.lootdropper ~= nil and
        inst.components.pickable ~= nil and
        inst.components.pickable.droppicked and
        inst.components.pickable:CanBePicked() then
        inst.components.pickable:Pick(worker)
    end
end

local function dig_up_juicy(inst, worker)
    if inst.components.pickable ~= nil and inst.components.lootdropper ~= nil then
        local withered = inst.components.pickable.withered
        if withered or inst.components.pickable:IsBarren() then
            inst.components.lootdropper:SpawnLootPrefab("twigs")
            inst.components.lootdropper:SpawnLootPrefab("twigs")
        else
            if inst.components.pickable:CanBePicked() then
                local pt = inst:GetPosition()
                pt.y = pt.y + (inst.components.pickable.dropheight or 0)
                for i = 1, 3 do
                    inst.components.lootdropper:SpawnLootPrefab("berries_juicy", pt)
                end
            end
            inst.components.lootdropper:SpawnLootPrefab("dug_berrybush_juicy")
        end
    end
    inst:Remove()
end

local function fn()
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddMiniMapEntity()

    inst.MiniMapEntity:SetIcon("juicy_berry_bush.tex")	
	
	inst:AddTag("bush")
    inst:AddTag("plant")
	inst:AddTag("renewable")
 
	MakeObstaclePhysics(inst, 0.1)

    inst.AnimState:SetBank("berrybush_juicy")
    inst.AnimState:SetBuild("berrybush_juicy")
    inst.AnimState:PlayAnimation("idle", true)
	setberries(inst, 1)
	
    inst.AnimState:SetTime(math.random() * inst.AnimState:GetCurrentAnimationLength())
    inst.spawnsperd = true
	
	inst:AddComponent("lootdropper")
		
    inst:AddComponent("pickable")
    inst.components.pickable.picksound = "dontstarve/wilson/harvest_berries"
    inst.components.pickable.onpickedfn = onpickedfn
    inst.components.pickable.makeemptyfn = makeemptyfn
    inst.components.pickable.makebarrenfn = makebarrenfn
    inst.components.pickable.makefullfn = makefullfn
	inst.components.pickable.ontransplantfn = ontransplantfn
    inst.components.pickable:SetUp(nil, TUNING.BERRY_JUICY_REGROW_TIME)  --Set the product to nil for droppicked
    inst.components.pickable.getregentimefn = getregentimefn_juicy
    inst.components.pickable.max_cycles = TUNING.BERRYBUSH_JUICY_CYCLES + math.random(2)
    inst.components.pickable.cycles_left = inst.components.pickable.max_cycles
    inst.components.pickable.jostlepick = true   
    inst.components.pickable.droppicked = true    	--droppicked effect is in onpickedfn 
    inst.components.pickable.dropheight = 3.5
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    local variance = math.random() * 4 - 2
	    inst.makewitherabletask = inst:DoTaskInTime(TUNING.WITHER_BUFFER_TIME + variance, function(inst) inst.components.pickable:MakeWitherable() end)
    end
	
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    --MakePickableBlowInWindGust(inst, TUNING.GRASS_WINDBLOWN_SPEED, TUNING.GRASS_WINDBLOWN_FALL_CHANCE)
		--#TO DO: withered gust blow anim already in file, but not used at all
	    inst.onblownpstdone = function(inst)
            if inst.components.pickable and inst.components.pickable:CanBePicked() and inst.AnimState:IsCurrentAnimation("blown_pst_idle") then
                inst.AnimState:PlayAnimation("idle", true)
            end
            inst:RemoveEventCallback("animover", inst.onblownpstdone)
        end

        inst.ongustanimdone = function(inst)
        if inst.components.pickable and inst.components.pickable:CanBePicked() then
            if inst.components.blowinwindgust:IsGusting() then
                local anim = math.random(1,2)
                inst.AnimState:PlayAnimation("blown_loop_idle"..anim, false)
            else
                inst:DoTaskInTime(math.random()/2, function(inst)
                    inst:RemoveEventCallback("animover", inst.ongustanimdone)
                    inst.AnimState:PlayAnimation("blown_pst_idle", false)
                    -- changed this from a push animation to an animover listen event so that it can be interrupted if necessary, and that a check can be made at the end to know if it should go to idle at that time.
                    --inst.AnimState:PushAnimation("idle", true)
                    inst:ListenForEvent("animover", inst.onblownpstdone)
                end)
            end
        else
            inst:RemoveEventCallback("animover", inst.ongustanimdone)
            end
        end

        inst.onguststart = function(inst, windspeed)
            if inst.components.pickable and inst.components.pickable:CanBePicked() then
                inst:DoTaskInTime(math.random()/2, function(inst)
                    inst.AnimState:PlayAnimation("blown_pre_idle", false)
                    inst:ListenForEvent("animover", inst.ongustanimdone)
                end)
            end
        end

        inst.ongustpick = function(inst)
            if inst.components.pickable and inst.components.pickable:CanBePicked() then
                inst.components.pickable:MakeEmpty()
                local pt = inst:GetPosition()
                pt.y = pt.y + (inst.components.pickable.dropheight or 0)
                for i = 1, 3 do
				    inst.components.lootdropper:SpawnLootPrefab(inst.components.pickable.product, pt)
				end
            end
        end

        inst:AddComponent("blowinwindgust")
        inst.components.blowinwindgust:SetWindSpeedThreshold(TUNING.GRASS_WINDBLOWN_SPEED)
        inst.components.blowinwindgust:SetDestroyChance(TUNING.GRASS_WINDBLOWN_FALL_CHANCE)
        inst.components.blowinwindgust:SetGustStartFn(inst.onguststart)
        inst.components.blowinwindgust:SetDestroyFn(inst.ongustpick)
        inst.components.blowinwindgust:Start()
	
	end
	
    MakeLargeBurnable(inst)
    MakeMediumPropagator(inst)	
    
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.DIG)
    inst.components.workable:SetWorkLeft(1)
	inst.components.workable:SetOnWorkCallback(onworked_juicy)
	inst.components.workable:SetOnFinishCallback(dig_up_juicy)

    inst:AddComponent("inspectable")
	
    MakeSnowCovered(inst)
    MakeNoGrowInWinter(inst)

    return inst
end

return Prefab("berrybush_juicy", fn, assets, prefabs)