local assets =
{
    Asset("ANIM", "anim/bugzapper.zip"),
    Asset("ANIM", "anim/swap_bugzapper.zip"), 
}

local prefabs =
{
    "lightning_rod_fx",
	"sparks",
}

local function onequip(inst, owner) 
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
	owner.AnimState:OverrideSymbol("lantern_overlay", "swap_bugzapper", "lantern_overlay")
    if inst.components.fueled:IsEmpty() then
        owner.AnimState:OverrideSymbol("swap_object", "swap_bugzapper", "swap_lantern_off")
		owner.AnimState:Hide("LANTERN_OVERLAY")
		inst.Light:Enable(false)
    else
        owner.AnimState:OverrideSymbol("swap_object", "swap_bugzapper", "swap_lantern_on")
		owner.AnimState:Show("LANTERN_OVERLAY") 
		inst.Light:Enable(true)
		inst.components.fueled:StartConsuming()
		if inst.fx == nil then
	        inst.fx = SpawnPrefab("bug_zapper_fx")
            inst.fx.entity:AddFollower()
            inst.fx.Follower:FollowSymbol(owner.GUID, "lantern_overlay", 50, 0, 0)
	    end
    end	
end

local function onunequip(inst, owner) 	
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
    owner.AnimState:ClearOverrideSymbol("lantern_overlay")
	owner.AnimState:Hide("LANTERN_OVERLAY") 
	inst.Light:Enable(false)
	inst.components.fueled:StopConsuming()
	if inst.fx ~= nil then
	    inst.fx:Remove()
        inst.fx = nil 
	end	
end

local BUGS_MUST_TAGS = {"insect"}
local BUGS_CANT_TAGS = {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "DECOR", "invisible", "notarget", "player", "boat", "companion"}
local function onattack(inst, attacker, target)
    if inst.components.fueled and inst.components.fueled:IsEmpty() then
	    inst:Remove()
	    return
	end
	inst.SoundEmitter:PlaySound("dontstarve/common/lightningrod")	
	if target ~= nil and target:HasTag("insect") and target.components.health ~= nil then
	    target.components.health:Kill()
	end
	if target ~= nil and target.Transform ~= nil then
	    local x, y, z = target.Transform:GetWorldPosition()
		SpawnPrefab("lightning_rod_fx").Transform:SetPosition(x, y, z)
		SpawnPrefab("sparks").Transform:SetPosition(x, y, z)		
	    local ents = TheSim:FindEntities(x, y, z, 5, BUGS_MUST_TAGS, BUGS_CANT_TAGS)
		for k,v in pairs(ents) do
		    if v ~= inst and v ~= target and v ~= attacker then
			    if v:IsValid() and v.entity:IsVisible() and v.components.health ~= nil then
				    v.components.health:Kill()
					if v.Transform ~= nil then
					    SpawnPrefab("lightning_rod_fx").Transform:SetPosition(v.Transform:GetWorldPosition())
						SpawnPrefab("sparks").Transform:SetPosition(v.Transform:GetWorldPosition())
				    end
				end
			end
		end
	end
end

local function fn()
    local inst = CreateEntity()
    
    inst.entity:AddTransform()
	
	if not IsDLCEnabled(PORKLAND_DLC) then
	    inst:DoTaskInTime(0, inst.Remove)
		return inst
	end
	
    inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddLight()
	
    inst.Light:Enable(false)
    inst.Light:SetIntensity(0.75)
    inst.Light:SetColour(252/255, 251/255, 237/255)
    inst.Light:SetFalloff(0.6)
    inst.Light:SetRadius(4)

    MakeInventoryPhysics(inst)
    MakeInventoryFloatable(inst, "idle_on_water", "idle_on")

    inst.AnimState:SetBank("bugzapper")
    inst.AnimState:SetBuild("bugzapper")
    inst.AnimState:PlayAnimation("idle_on", true)
	
	inst:AddTag("bug_zapper")
	
	inst:AddComponent("inventoryitem")
	
    inst:AddComponent("inspectable")
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(TUNING.SPEAR_DAMAGE)
	inst.components.weapon:SetOnAttack(onattack)
    inst.components.weapon:SetElectric()

    inst:AddComponent("equippable")    
    inst.components.equippable:SetOnEquip(onequip)     
    inst.components.equippable:SetOnUnequip(onunequip)
    inst.components.equippable:SetOnPocket(function(owner) 
	    inst.Light:Enable(false) 
		inst.components.fueled:StopConsuming()
	end)  

    inst:AddComponent("fueled")
    inst.components.fueled:SetSectionCallback(
        function(section)
            if section == 0 then	
                if inst.components.inventoryitem and inst.components.inventoryitem:IsHeld() then
                    local owner = inst.components.inventoryitem.owner
                    --inst:Remove()                
                    if owner then
                        owner:PushEvent("nightstickranout", {nightstick = inst})
                    end
                end
                inst.Light:Enable(false)
				inst.AnimState:PlayAnimation("idle_off")
				inst.components.floatable:UpdateAnimations("idle_water", "idle_off")
                if inst.components.equippable:IsEquipped() then
                    inst.components.inventoryitem.owner.AnimState:OverrideSymbol("swap_object", "swap_bugzapper", "swap_lantern_off")
                    inst.components.inventoryitem.owner.AnimState:Hide("LANTERN_OVERLAY") 
                end	
				inst.components.weapon.stimuli = nil
				inst:AddTag("sharp")
	            if inst.fx ~= nil then
	                inst.fx:Remove()
                    inst.fx = nil 
	            end								
            end
        end)
    inst.components.fueled:InitializeFuelLevel(TUNING.NIGHTSTICK_FUEL*4)
    inst.components.fueled:SetDepletedFn(function(inst) 
	    --inst:Remove() 
	    inst.Light:Enable(false)
	    --inst.AnimState:PlayAnimation("idle_off")
        --inst.components.floatable:UpdateAnimations("idle_water", "idle_off")
        --if inst.components.equippable:IsEquipped() then
         --   inst.components.inventoryitem.owner.AnimState:OverrideSymbol("swap_object", "swap_bugzapper", "swap_lantern_off")
         --   inst.components.inventoryitem.owner.AnimState:Hide("LANTERN_OVERLAY") 
        --end
	end)
	
	inst:ListenForEvent("ondropped", function(inst)
		inst.components.fueled:StopConsuming()
	    inst.Light:Enable(false)
	end)
	
    inst:ListenForEvent("startrowing", function(inst, data) 
        if inst.components.equippable:IsEquipped() then
    	    if inst.fx ~= nil then
    		    inst.fx:Remove()
                inst.fx = nil 
		    end		    
	        inst.Light:Enable(false)
			inst.components.fueled:StopConsuming()
		end
	end, inst)  
	
    inst:ListenForEvent("stoprowing", function(inst, data) 
        if inst.components.equippable:IsEquipped() and not inst.components.fueled:IsEmpty() then
		    local owner = inst.components.equippable.equipper
		    if inst.fx == nil and owner ~= nil then
	            inst.fx = SpawnPrefab("bug_zapper_fx")
                inst.fx.entity:AddFollower()
                inst.fx.Follower:FollowSymbol(owner.GUID, "lantern_overlay", 50, 0, 0)
	        end	    
	        inst.Light:Enable(true)
			inst.components.fueled:StartConsuming()
		end
	end, inst)		
	
    return inst	
end

local function fx_fx()
    local inst = CreateEntity()
    
	if not IsDLCEnabled(PORKLAND_DLC) then
	    inst:DoTaskInTime(0, inst.Remove)
		return inst
	end
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    inst.AnimState:SetBank("FORCEEXPORT/lantern_overlay")
    inst.AnimState:SetBuild("bugzapper")
    inst.AnimState:PlayAnimation("idle", true)

	inst.SoundEmitter:PlaySound("dontstarve/characters/wx78/charged", "overcharge_sound")
	inst.SoundEmitter:SetParameter("overcharge_sound", "intensity", 0.1)
	
	inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
	inst:AddTag("FX")
	inst:AddTag("INTERIOR_LIMBO_IMMUNE")
	
	inst.persists = false
	
	return inst
end

return Prefab("bug_zapper", fn, assets, prefabs),
    Prefab("bug_zapper_fx", fx_fx, assets)
