--Buttercup from Plants vs. Zombies 2. Buttercup butters butter onto attacking mobs, stunning them with butter. Butter!

local buttercup_level_data = require("buttercup_level_data")

local buttercup_assets =
{ --Anim from DST, I cannot make Buttercup anim, so use this barnacle plant.
    Asset("ANIM", "anim/barnacle_plant.zip"),
    Asset("ANIM", "anim/barnacle_plant_colour_swaps.zip"),
}

local buttercup_projectile_assets = 
{ 
	Asset("ANIM", "anim/pvz2butter.zip"),
}

local butterpuddle_assets = 
{
	Asset("ANIM", "anim/honey_trail.zip"),
}

local plantfood_assets = 
{
	Asset("ANIM", "anim/jy_pvz_powerbean.zip"),
}

local sun_assets = 
{
    Asset("ANIM", "anim/jy_pvz_sun.zip"),
}

local buttercup_prefabs = 
{
    "butter",
}

local plantfood_prefabs = 
{

}

local sun_prefabs = 
{
    "buttercup",
}

require("prefabutil")

local brain = require("brains/buttercupbrain")

local function ButtercupKeepTarget(inst, target)
    return false
end

local function ButtercupOnRefuse(inst, giver, item)
    if giver ~= nil and giver.components.talker ~= nil then
	    if TUNING.LANGUAGE == true then
		    giver.components.talker:Say("It is too busy, or giving wrong item.")
        elseif TUNING.LANGUAGE == false then
		    giver.components.talker:Say("它太忙了，或是给了错误的物品。")
	    end	
	end
end

local function ButtercupOnAccept(inst, giver, item)
    if item ~= nil then
	    item:Remove()
	end
	inst.sg:GoToState("plantfood_attack")
end

local function ButtercupAcceptTest(inst, item, giver)
    if inst.sg ~= nil 
	    and (inst.sg:HasStateTag("busy") or inst.sg:HasStateTag("attack")) 
	then
	    return false
	end
	return item ~= nil and item:HasTag("plantfood")
end

local function on_attacked_butter_mobs(inst, data)
    if data ~= nil 
	    and data.attacker ~= nil 
		and not data.attacker:HasTag("flying") 
		and not data.attacker:HasTag("ghost") 
		and (inst.sg == nil or not inst.sg:HasStateTag("plantfoodattack"))
	then
        inst.components.combat:SuggestTarget(data.attacker)
	end
end

local function on_death_butter_mobs(inst, attacker)
    if attacker ~= nil 
	    and not attacker:HasTag("flying") 
		and not attacker:HasTag("ghost")
	then
		local butter = SpawnPrefab("buttercup_projectile")
        butter.Transform:SetPosition(inst.Transform:GetWorldPosition())
        butter.components.complexprojectile:Launch(attacker:GetPosition(), inst) 		
	end
end

local function on_death_by_fire(inst, data)
    if data ~= nil and data.cause == "fire" then
		local butter = SpawnPrefab("buttercup_projectile")
        butter.Transform:SetPosition(inst.Transform:GetWorldPosition())
        butter.components.complexprojectile:Launch(inst:GetPosition(), inst) 		
	end
end

local function on_remove(inst, data)
    if inst.components.inventory and inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
	    inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS):Remove()
	end
end

local function equip_ranged_weapon(inst)
    if inst.components.inventory and not inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
        local ranged_weapon = CreateEntity()
        ranged_weapon.persists = false

        ranged_weapon.entity:AddTransform()

        ranged_weapon:AddComponent("weapon")
        ranged_weapon.components.weapon:SetDamage(inst.components.combat.defaultdamage)
        ranged_weapon.components.weapon:SetRange(inst.components.combat.attackrange, inst.components.combat.attackrange + 4)
        ranged_weapon.components.weapon:SetProjectile("buttercup_projectile")

        ranged_weapon:AddComponent("inventoryitem")
        ranged_weapon.components.inventoryitem:SetOnDroppedFn(ranged_weapon.Remove)

        ranged_weapon:AddComponent("equippable")
        ranged_weapon:AddTag("nosteal")
		ranged_weapon:AddTag("cantdrop")
		ranged_weapon:AddTag("projectile")
        inst.components.inventory:Equip(ranged_weapon)
    end
end

local function SetNormalDamage(inst, v)
    if v ~= nil then
	    inst.NormalDamage = v
	end
end

local function SetPlantFoodDamage(inst, v)
    if v ~= nil then
	    inst.PlantFoodDamage = v
	end
end

local function buttercup_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	inst.entity:AddMiniMapEntity()
	
	MakeObstaclePhysics(inst, 0.5)

    inst.Transform:SetSixFaced()

    inst:AddTag("buttercup")
	inst:AddTag("veggie")
	inst:AddTag("plant_mob")
	inst:AddTag("noauradamage")
	inst:AddTag("groundpoundimmune")

    inst.AnimState:SetBank("barnacle_plant")
    inst.AnimState:SetBuild("barnacle_plant_colour_swaps")
    inst.AnimState:PlayAnimation("idle")
    inst.AnimState:OverrideSymbol("bc_bud", "barnacle_plant_colour_swaps", "yellow_bc_bud")
    inst.AnimState:OverrideSymbol("bc_face", "barnacle_plant_colour_swaps", "yellow_bc_face")
    inst.AnimState:OverrideSymbol("bc_flower_petal", "barnacle_plant_colour_swaps", "yellow_bc_flower_petal")
    inst.AnimState:Hide("bud1")
    inst.AnimState:Hide("bud2")
    inst.AnimState:Hide("bud3")
	
	inst.MiniMapEntity:SetIcon("buttercup.tex")

    inst:AddComponent("inspectable")

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(TUNING.BUTTERCUP_HEALTH)

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(TUNING.BUTTERCUP_DAMAGE)
    inst.components.combat:SetAttackPeriod(TUNING.BUTTERCUP_ATTACK_PERIOD)
    inst.components.combat:SetRange(TUNING.BUTTERCUP_ATTACK_DISTANCE)
    inst.components.combat:SetKeepTargetFunction(ButtercupKeepTarget)
	inst.components.combat.keeptargettimeout = 1
	inst.components.combat.onkilledbyother = on_death_butter_mobs
	
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"butter", "pvz_sun", "butter_machine_item"})
	
	inst:AddComponent("inventory")
	
	inst:AddComponent("trader")
	inst.components.trader.acceptnontradable = true
	inst.components.trader.onrefuse = ButtercupOnRefuse
	inst.components.trader.onaccept = ButtercupOnAccept
    inst.components.trader:SetAcceptTest(ButtercupAcceptTest)	
	
	inst:AddComponent("charlierefreshmodlevel")
	inst.components.charlierefreshmodlevel:SetLevelData(buttercup_level_data)
	inst.components.charlierefreshmodlevel:EnableMastery(true)
	inst.components.charlierefreshmodlevel:SetRequiredConsumables("seedpacket")	
	
	MakeMediumBurnableCharacter(inst, "body")
	
	inst:ListenForEvent("attacked", on_attacked_butter_mobs)
	inst:ListenForEvent("death", on_death_by_fire)
	inst:ListenForEvent("onremove", on_remove)
	
	inst.SetNormalDamage = SetNormalDamage
	inst.SetPlantFoodDamage = SetPlantFoodDamage
	inst.Stun = 8
	inst.StunPF = 12

	inst:DoTaskInTime(math.random()*2*FRAMES, equip_ranged_weapon)

    inst:SetStateGraph("SGbuttercup")
    inst:SetBrain(brain)

    return inst
end

local function projectile_keeptarget(inst)
    return false
end

local function set_thrown_physics(inst)
    inst.Physics:SetMass(1)
    inst.Physics:SetFriction(0)
    inst.Physics:SetDamping(0)
    inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
    inst.Physics:ClearCollisionMask()
    inst.Physics:CollidesWith(COLLISION.GROUND)
    inst.Physics:CollidesWith(COLLISION.OBSTACLES)
    inst.Physics:CollidesWith(COLLISION.ITEMS)
    inst.Physics:SetCapsule(0.2, 0.2)
end

local function onhit(inst, attacker, target)
    local x, y, z = inst.Transform:GetWorldPosition()
    local impactfx = SpawnPrefab("impact")
    if impactfx then
        impactfx.Transform:SetPosition(x, y, z)
		if attacker ~= nil then
            impactfx:FacePoint(attacker.Transform:GetWorldPosition())
		end
    end

    local entities = TheSim:FindEntities(x, y, z, 3, {"hascombatcomponent"}, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "flying", "ghost", "shadow", "buttercup", "notarget", "plant_mob"})
    for i, v in pairs(entities) do
        if v ~= nil and v:IsValid() and v.entity:IsVisible() and inst.components.combat:CanTarget(v) then

            if attacker ~= nil and v.components.combat ~= nil then
			    if attacker.PlantFoodDamage == nil then
                    v.components.combat:GetAttacked(attacker, (attacker.plant_food_attack and TUNING.BUTTERCUP_DAMAGE_PF) or (attacker.NormalDamage or TUNING.BUTTERCUP_DAMAGE), inst)
				else
				    v.components.combat:GetAttacked(attacker, (attacker.plant_food_attack and attacker.PlantFoodDamage) or (attacker.NormalDamage or TUNING.BUTTERCUP_DAMAGE), inst)
				end
            else
                inst.components.combat:DoAttack(v)
            end
			
			if v.components.freezable ~= nil then
			    if attacker ~= nil and attacker.Stun ~= nil and attacker.StunPF ~= nil then
				    v.components.freezable:Freeze(attacker.plant_food_attack and attacker.StunPF or attacker.Stun)
				else
			        v.components.freezable:Freeze(attacker ~= nil and attacker.plant_food_attack and TUNING.BUTTERCUP_STUN_DURATION_PF or TUNING.BUTTERCUP_STUN_DURATION)
				end
			end
			
        end
    end	
	
    local butterpuddles = TheSim:FindEntities(x, y, z, 2, {"butterpuddle"})
	local puddles = TheSim:FindEntities(x, y, z, 3, {"puddle"})
	if #butterpuddles <= 0 and #puddles <= 0 and inst:IsOnValidGround() then
		if inst.IsPosSurroundedByLand == nil or inst:IsPosSurroundedByLand(x, y, z, 3) then
			local butter_puddle = SpawnPrefab("butterpuddle")
			butter_puddle.Transform:SetPosition(x, y, z)
			if attacker ~= nil and attacker.plant_food_attack then
			    butter_puddle:SetLifeTime(150)
				butter_puddle.from_plant_food = true
				if attacker.PlantFoodDamage ~= nil then
				    butter_puddle.damage_pf = attacker.PlantFoodDamage
				end
			else
			    butter_puddle:SetLifeTime(120)
				if attacker.NormalDamage ~= nil then
				    butter_puddle.damage = attacker.NormalDamage 
				end
			end
		end		
	end	
	
	inst.SoundEmitter:PlaySound("dontstarve/common/butterfly_trap")	

    inst:Remove()
end

local function onthrown(inst)
    inst.AnimState:PlayAnimation("spin_loop", true)
end

local PROJECTILE_LAUNCH_OFFSET = Vector3(1.0, 8.0, 0)

local function buttercup_projectile_fn()
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddPhysics()
	
    set_thrown_physics(inst)
    inst.Physics:SetDontRemoveOnSleep(true)

    inst:AddTag("NOCLICK")
    inst:AddTag("notarget")

    inst.AnimState:SetBank("pvz2butter")
    inst.AnimState:SetBuild("pvz2butter")
    inst.AnimState:PlayAnimation("idle")
	
    inst:AddComponent("complexprojectile")
    inst.components.complexprojectile:SetHorizontalSpeed(18)
    inst.components.complexprojectile:SetLaunchOffset(PROJECTILE_LAUNCH_OFFSET)
	inst.components.complexprojectile:SetOnLaunch(onthrown) 
    inst.components.complexprojectile:SetOnHit(onhit)
	inst.components.complexprojectile.gravity = -5

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(TUNING.BUTTERCUP_DAMAGE)
    inst.components.combat:SetRange(TUNING.BUTTERCUP_ATTACK_DISTANCE)
    inst.components.combat:SetKeepTargetFunction(projectile_keeptarget)	

	inst.persists = false
	
    return inst
end

local function ButterPuddleDisappear(inst)
    inst.persists = false
    if inst.disappeartask then 
		inst.disappeartask:Cancel() 
		inst.disappeartask = nil 		
	end	
	inst.disappeartaskinfo = nil
	inst.AnimState:PlayAnimation("trail1_pst")
	inst:ListenForEvent("animover", inst.Remove)
end

local function InitButterPuddle(inst)
    if not inst.loaded then
	    inst.AnimState:PlayAnimation("trail1_pre")
	    inst.AnimState:PushAnimation("trail1")
    else
	    inst.AnimState:PlayAnimation("trail1")
	end
end

local function SetLifeTime(inst, lifetime)
    if lifetime ~= nil then
	    inst.disappeartask, inst.disappeartaskinfo = inst:ResumeTask(lifetime, ButterPuddleDisappear)
	end
end

local function ButterPuddleOnLongUpdate(inst, dt)
    if inst.disappeartask and inst.disappeartaskinfo then
        local new_time = math.max(inst:TimeRemainingInTask(inst.disappeartaskinfo) - dt, 0)

        inst.disappeartask:Cancel()
        inst.disappeartask = nil 
        inst.disappeartaskinfo = nil

        inst.disappeartask, inst.disappeartaskinfo = inst:ResumeTask(new_time, ButterPuddleDisappear)
    end
end

local function ButterPuddleOnSave(inst, data)
    if inst.disappeartaskinfo then
        data.disappeartask = inst:TimeRemainingInTask(inst.disappeartaskinfo)
    end
	data.damage = inst.damage or nil
	data.damage_pf = inst.damage_pf or nil
	data.stun = inst.stun or nil
	data.stun_pf = inst.stun_pf or nil
	data.from_plant_food = inst.from_plant_food or nil
end

local function ButterPuddleOnLoad(inst, data)
    inst.loaded = true
	if data ~= nil then 
        if data.disappeartask then
            if inst.disappeartask then 
			    inst.disappeartask:Cancel() 
				inst.disappeartask = nil 		
			end
            inst.disappeartaskinfo = nil
            inst.disappeartask, inst.disappeartaskinfo = inst:ResumeTask(data.disappeartask, ButterPuddleDisappear)
        end
        if data.from_plant_food then
            inst.from_plant_food = true
        end		
		if data.damage then
		    inst.damage = data.damage
		end
		if data.damage_pf then
		    inst.damage_pf = data.damage_pf
		end		
		if data.stun then
		    inst.stun = data.stun
		end	
		if data.stun_pf then
		    inst.stun_pf = data.stun_pf
		end	
	end
end

local function butterpuddle_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

	inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
    inst:AddTag("notarget")
	inst:AddTag("butterpuddle")
	
    inst.AnimState:SetBank("honey_trail")
    inst.AnimState:SetBuild("honey_trail")
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
    inst.AnimState:SetSortOrder(3)
	
	inst:DoTaskInTime(0, InitButterPuddle)

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(TUNING.BUTTERCUP_DAMAGE)
    inst.components.combat:SetRange(TUNING.BUTTERCUP_ATTACK_DISTANCE)
    inst.components.combat:SetKeepTargetFunction(projectile_keeptarget)
	
	inst:AddComponent("butternearbymobs")
	
	inst.SetLifeTime = SetLifeTime
	inst.ButterPuddleDisappear = ButterPuddleDisappear
    inst.OnLongUpdate = ButterPuddleOnLongUpdate
	inst.OnSave = ButterPuddleOnSave
	inst.OnLoad = ButterPuddleOnLoad
	
	return inst
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)	
		inst.float_fx.Transform:SetPosition(0, 0.25, 0)
		inst.float_fx.Transform:SetScale(0.75, 0.75, 0.75)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.Transform:SetPosition(0, 0.25, 0)
		inst.float_fx2.Transform:SetScale(0.75, 0.75, 0.75)
	end	
end

local function SunOnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)	
		inst.float_fx.Transform:SetPosition(0, -0.25, 0)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.Transform:SetPosition(0, -0.25, 0)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function pvz2_plantfood_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)

    inst:AddTag("plantfood")
	
    inst.AnimState:SetBank("jy_pvz_powerbean")
    inst.AnimState:SetBuild("jy_pvz_powerbean")
    inst.AnimState:PlayAnimation("idle")
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")

    inst:AddComponent("stackable")
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
	end
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end	
	
	return inst
end

local function ondeploy(inst, pt, deployer)
    local buttercup = SpawnPrefab("buttercup") 
    if buttercup then 
        buttercup.Transform:SetPosition(pt.x, pt.y, pt.z)
        buttercup.SoundEmitter:PlaySound("dontstarve/common/plant")
        inst:Remove()
    end 
end

local function IsPointNearButtercup(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, {"buttercup"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function deploytest(inst, pt)
	if inst.IsPosSurroundedByLand ~= nil then
	    if not inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 3) then
		    return false
		end
	end

	local radius = 2	
    for i = -radius, radius, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x - radius, pt.y, pt.z + i) == GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(pt.x + radius, pt.y, pt.z + i) == GROUND.IMPASSABLE then
            return false
        end
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z - radius) == GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z + radius) == GROUND.IMPASSABLE then
            return false
        end
    end			

	local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 6, nil, {"NOBLOCK", "player", "FX"})
	local min_spacing = 3
	for k, v in pairs(ents) do
		if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
			if distsq( Vector3(v.Transform:GetWorldPosition()), pt ) < min_spacing*min_spacing then
				return false
			end
		end
	end
	
	return not IsPointNearButtercup(pt, 5) and not TheCamera.interior	
end

local function pvz_sun_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)

    inst:AddTag("pvz_sun")
	
    inst.AnimState:SetBank("jy_pvz_sun")
    inst.AnimState:SetBuild("jy_pvz_sun")
    inst.AnimState:PlayAnimation("sun_move", true)
	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")

    inst:AddComponent("stackable")
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

    inst:AddComponent("deployable")
    inst.components.deployable.ondeploy = ondeploy
	inst.components.deployable.test = deploytest
    inst.components.deployable.placer = "buttercup_placer"
    inst.components.deployable.deploydistance = 1.5
	
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(SunOnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
	end

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end
	
	return inst
end

return Prefab("buttercup", buttercup_fn, buttercup_assets, buttercup_prefabs),
    Prefab("buttercup_projectile", buttercup_projectile_fn, buttercup_projectile_assets, buttercup_prefabs),
	Prefab("butterpuddle", butterpuddle_fn, butterpuddle_assets, buttercup_prefabs),
	Prefab("pvz2_plantfood", pvz2_plantfood_fn, plantfood_assets, plantfood_prefabs),
	Prefab("pvz_sun", pvz_sun_fn, sun_assets, sun_prefabs),
	MakePlacer("buttercup_placer", "barnacle_plant", "barnacle_plant_colour_swaps", "idle")  --Additional changes are in other places (modmain_additional.lua).