require "brains/friendlybutterflybrain"
require "stategraphs/SGfriendlybutterfly"

local assets =
{
	Asset("ANIM", "anim/butterfly_basic.zip"),
}
	
if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then	
    table.insert(assets, Asset("ANIM", "anim/butterfly_tropical_basic.zip"))
end
	
local prefabs =
{
	"butterflywings",
	"butter",
	"flower",
	"rose_inv",
	"petals",
	"small_puff",
}

local function TryToSpawn(inst) 
    if GetClock() and GetClock():IsDay() and inst:IsOnValidGround() then
	    local prefab_to_spawn = SpawnPrefab(math.random() < 0.01 and "rose_inv" or "petals")
		if prefab_to_spawn then
		    local x, y, z = inst.Transform:GetWorldPosition()
		    prefab_to_spawn.Transform:SetPosition(x, y, z)
			SpawnPrefab("small_puff").Transform:SetPosition(x, y, z)
			if prefab_to_spawn.GetIsOnWater ~= nil
			    and prefab_to_spawn:GetIsOnWater()
			then
			    if prefab_to_spawn.components.floatable ~= nil then
				    prefab_to_spawn.components.floatable:OnHitWater(true)
				elseif prefab_to_spawn.components.inventoryitem ~= nil then
			        prefab_to_spawn.components.inventoryitem:OnHitWater()
			    end			
			end
		end
	end
end

local function fn(Sim)
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddDynamicShadow()
	
	inst.DynamicShadow:SetSize( 0.8, 0.5 )
	
	inst.Transform:SetTwoFaced()

	inst.AnimState:SetBank("butterfly")
	inst.AnimState:PlayAnimation("idle")
	inst.AnimState:SetRayTestOnBB(true)
	inst.AnimState:SetMultColour(255/255, 200/255, 175/255, 0.75)
	
	inst:AddTag("butterfly")
	inst:AddTag("flying")
	inst:AddTag("insect")
	inst:AddTag("smallcreature")
	inst:AddTag("cattoyairborne")
	inst:AddTag("companion")

    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    MakePoisonableCharacter(inst)
	    MakeAmphibiousCharacterPhysics(inst, 1, 0.25)
	else
	    MakeCharacterPhysics(inst, 1, 0.25)
	end
	
	inst.Physics:SetCollisionGroup(COLLISION.FLYERS)
	inst.Physics:ClearCollisionMask()
	
	if IsDLCEnabled(PORKLAND_DLC) then
	    inst.Physics:CollidesWith(GetWorldCollision())
	    inst.Physics:CollidesWith(COLLISION.INTWALL)
	else
	    inst.Physics:CollidesWith(COLLISION.WORLD)
	end

	if SaveGameIndex:IsModeShipwrecked() then
		inst.AnimState:SetBuild("butterfly_tropical_basic")
		inst.shelfart = "butterfly_tropical"
	else
		inst.AnimState:SetBuild("butterfly_basic")
	end
	
	inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
	inst.components.locomotor:EnableGroundSpeedMultiplier(false)
	inst.components.locomotor:SetTriggersCreep(false)
	
	inst:SetStateGraph("SGfriendlybutterfly")
	
	inst:AddComponent("health")
	inst.components.health:SetMaxHealth(1)

	inst:AddComponent("pollinator")
	
	inst:AddComponent("combat")
	inst.components.combat.hiteffectsymbol = "butterfly_body"

	inst:AddComponent("knownlocations")

	MakeSmallBurnableCharacter(inst, "butterfly_body")
	MakeTinyFreezableCharacter(inst, "butterfly_body")
	
	inst:AddComponent("inspectable")

	inst:AddComponent("lootdropper")
	inst.components.lootdropper:AddRandomLoot("butter", 0.1)
	inst.components.lootdropper:AddRandomLoot("butterflywings", 5)   
	inst.components.lootdropper.numrandomloot = 1

	local brain = require "brains/friendlybutterflybrain"
	inst:SetBrain(brain)
	
    inst:AddComponent("follower")

    inst:RemoveEventCallback("attacked", function(inst, data)
	    if inst.components.follower.leader == data.attacker then
		    inst.components.follower:SetLeader(nil)
	    end	
	end)

    if inst.components.follower.SetFollowExitDestinations ~= nil then
        inst.components.follower:SetFollowExitDestinations({EXIT_DESTINATION.LAND, EXIT_DESTINATION.WATER})
    end
	
	inst:ListenForEvent("daytime", function() TryToSpawn(inst) end, GetWorld())

	if GetPlayer() and GetPlayer().components.leader ~= nil then
		GetPlayer().components.leader:AddFollower(inst)        
	end

	return inst
end

return Prefab("butterfly_friendly", fn, assets, prefabs)
