local MakePlayerCharacter = require("prefabs/player_common")
local charlie_level_data = require("charlie_level_data")
local extensions_fn = require("prefabs/charlie_extensions")

local assets = 
{
    Asset("ANIM", "anim/charlie.zip"),
	Asset("ANIM", "anim/charlie_gladiator.zip"),
	Asset("ANIM", "anim/charlie_goh.zip"),
	Asset("ANIM", "anim/charlie_hightide.zip"),
	Asset("ANIM", "anim/charlie_queen.zip"),
	Asset("ANIM", "anim/charlie_rose.zip"),
	Asset("ANIM", "anim/charlie_survivor.zip"),
	Asset("ANIM", "anim/charlie_shadow.zip"),
}

local prefabs = 
{
	"tophat",
	"rabbit",
	"magician_chest",
	"charlie_portal",
	"shadowcorrosionmachine",
	"rose_inv",
	"townportal",
	"shadow_rose_bouquet",
	"statueharp_rose",
	"statuemaxwell_rose",
	"rabbithole",
	"charlie_dragon_house",
	"charlie_castle",
}

local start_inv =
{
    "tophat",
	"rabbit",
	"rose_inv",
}

local KILL_INNOCENT_ENG = {"Oh, I am sorry!", "I don't mean it!", "No! I won't do that again!", "So sorry..."}
local KILL_INNOCENT_CN = {"哦，我很抱歉！", "我不是故意的！", "不！我不会再那样做！", "很抱歉……"}
local INNOCENT_DIE_ENG = {"Oh, poor little thing!", "That thing engulfs it!", "Is disaster coming?", "Final Destination, eh."}
local INNOCENT_DIE_CN = {"哦，可怜的小家伙！", "那东西吞没了它！", "灾难要降临了吗", "死神来了，呃。"}
local TAKEN_BY_SHADOW_ENG = {"I don't think it's over yet.", "NO! I won't let you take me again!", "Is that what you want, Maxy?", "I'm waiting for you here", "I won't let you wither away.", "You will gain more advanced knowledge."}
local TAKEN_BY_SHADOW_CN = {"我认为这还没结束。", "不！我不会让你再次带走我的！", "这是你想要的结果吗，麦斯？", "我在这里等着你……", "我不会让你衰弱的。", "你会获得更多高级知识。"}
local INHERIT_SHADOW_ENG = {"Looks like I'm leaving...", "That's not the way it is.", "The Constant is empty!", "This can't let me get rid of her.", "We can never go back to the past...", "Only a short while ago, how brilliant we are..."}
local INHERIT_SHADOW_CN = {"看来我要离开了……", "事情不应该是这样的。", "永恒之域空荡荡！", "这样不能让我摆脱她。", "我们永远回不到过去……", "曾几何时，我们是多么光辉……"}

local PHRASES_FORM_SHADOW_ENG = {"Now you see me...", "The darkness calls me...", "I missed you...", "It's Metheus's turn...", "Nightmare fuel is playing a role..."}
local PHRASES_FORM_NORMAL_ENG = {"Oh, goodness...", "I'll never get used to that...", "That's enough of that.", "Let me go!", "Did I just... lose control?"}
local PHRASES_FORM_SHADOW_CN = {"现在你可以看到我了……", "黑暗要召唤我了……", "我想你了……", "轮到米修斯了……", "梦魇燃料在发挥作用……"}
local PHRASES_FORM_NORMAL_CN = {"哦，天哪……", "我从不习惯那样……", "够了。", "放过我！", "我刚才……失控了吗？"}

local DAMAGE_MUST_TAGS = {"hascombatcomponent"}
local DAMAGE_CANT_TAGS = {"FX", "NOCLICK", "INLIMBO", "DECOR", "INTERIOR_LIMBO", "invisible", "notarget", "shadow_item", "shadow_area_immune", "wall", "boat", "companion", "bramble", "elephantcactus", "buttercup"}

local function OnTimerDone(inst, data)
    if data ~= nil then
	    if data.name ~= nil then
		    if data.name == "sanityloss_cooldown" then
			    if inst:HasTag("sanitylosscooldown") then
			        inst:RemoveTag("sanitylosscooldown")
			    end			
			elseif data.name == "despawn_protection" then			
	            if inst.children then
                    for k,child in pairs(inst.children) do
                        if k.prefab == "spawnprotectionbuff" then
						    if k.OnEnableProtectionFn then
							    k:OnEnableProtectionFn(inst, false)
							end
						end
                    end
                end
            end
		end
	end
end

local function UpdateVariableValues(inst)
    if GetWorld():IsRuins() then
        if inst.shadowform ~= nil then
            inst.components.combat.damagemultiplier = TUNING.CHARLIE_SHDAOW_RUINS_DAMAGE_MULT_VANILLA
	        inst.components.sanity.night_drain_mult = TUNING.SHADOW_CHARLIE_NIGHT_SANITY_MULT
	        inst.components.sanity.neg_aura_mult = TUNING.SHADOW_CHARLIE_NEG_SANITY_MULT
        else
            inst.components.combat.damagemultiplier = TUNING.CHARLIE_SHDAOW_DAMAGE_MULT_VANILLA
	        inst.components.sanity.night_drain_mult = TUNING.CHARLIE_NIGHT_SANITY_MULT_NIGHT_VANILLA
	        inst.components.sanity.neg_aura_mult = TUNING.CHARLIE_NEG_SANITY_MULT_NIGHT_VANILLA
	    end
	    inst.components.health.absorb = TUNING.CHARLIE_PROTECTION_NIGHT_VANILLA	
    elseif GetWorld():IsCave() then
        if inst.shadowform ~= nil then
            inst.components.combat.damagemultiplier = TUNING.CHARLIE_SHDAOW_CAVE_DAMAGE_MULT_VANILLA
	        inst.components.sanity.night_drain_mult = TUNING.SHADOW_CHARLIE_NIGHT_SANITY_MULT
	        inst.components.sanity.neg_aura_mult = TUNING.SHADOW_CHARLIE_NEG_SANITY_MULT
        else
            inst.components.combat.damagemultiplier = TUNING.CHARLIE_DAMAGE_MULT_NIGHT_VANILLA*1.25
	        inst.components.sanity.night_drain_mult = TUNING.CHARLIE_NIGHT_SANITY_MULT_NIGHT_VANILLA
	        inst.components.sanity.neg_aura_mult = TUNING.CHARLIE_NEG_SANITY_MULT_NIGHT_VANILLA
	    end
	    inst.components.health.absorb = TUNING.CHARLIE_PROTECTION_NIGHT_VANILLA
    else
	    if GetClock():IsDay() then
            inst.components.combat.damagemultiplier = TUNING.CHARLIE_DAMAGE_MULT_DAY_VANILLA
	        inst.components.sanity.night_drain_mult = TUNING.CHARLIE_NIGHT_SANITY_MULT_DAY_VANILLA
	        inst.components.sanity.neg_aura_mult = TUNING.CHARLIE_NEG_SANITY_MULT_DAY_VANILLA
            inst.components.health.absorb = TUNING.CHARLIE_PROTECTION_DAY_VANILLA	
        elseif GetClock():IsNight() then
		    if inst.shadowform ~= nil then
                inst.components.combat.damagemultiplier = TUNING.CHARLIE_SHDAOW_DAMAGE_MULT_VANILLA
	            inst.components.sanity.night_drain_mult = TUNING.SHADOW_CHARLIE_NIGHT_SANITY_MULT
	            inst.components.sanity.neg_aura_mult = TUNING.SHADOW_CHARLIE_NEG_SANITY_MULT					
			else
                inst.components.combat.damagemultiplier = TUNING.CHARLIE_DAMAGE_MULT_NIGHT_VANILLA
	            inst.components.sanity.night_drain_mult = TUNING.CHARLIE_NIGHT_SANITY_MULT_NIGHT_VANILLA
	            inst.components.sanity.neg_aura_mult = TUNING.CHARLIE_NEG_SANITY_MULT_NIGHT_VANILLA
			end
            inst.components.health.absorb = TUNING.CHARLIE_PROTECTION_NIGHT_VANILLA	
        elseif GetClock():IsDusk() then
		    if inst.shadowform ~= nil then
                inst.components.combat.damagemultiplier = TUNING.CHARLIE_SHDAOW_DAMAGE_MULT_VANILLA_DUSK
	            inst.components.sanity.night_drain_mult = TUNING.SHADOW_CHARLIE_NIGHT_SANITY_MULT
	            inst.components.sanity.neg_aura_mult = TUNING.SHADOW_CHARLIE_NEG_SANITY_MULT					
			else		
                inst.components.combat.damagemultiplier = TUNING.CHARLIE_DAMAGE_MULT_DUSK_VANILLA
 	            inst.components.sanity.night_drain_mult = TUNING.CHARLIE_NIGHT_SANITY_MULT_DUSK_VANILLA
	            inst.components.sanity.neg_aura_mult = TUNING.CHARLIE_NEG_SANITY_MULT_DUSK_VANILLA
            end			
            inst.components.health.absorb = TUNING.CHARLIE_PROTECTION_DUSK_VANILLA
		end
    end
end

local function OnDeath(inst)
    if IsDLCEnabled(PORKLAND_DLC) then
		inst:DoTaskInTime(0, function()
            local pt = Vector3(inst.Transform:GetWorldPosition())
	      	SpawnPrefab("vortex_cloak_fx").Transform:SetPosition(pt:Get())
	    end)
	end
    if (inst.components.transformable ~= nil and inst.components.transformable.shadowform) or (inst.shadowform ~= nil) then
	    if TUNING.LANGUAGE == true then
	        inst.components.talker:Say(INHERIT_SHADOW_ENG[math.random(#INHERIT_SHADOW_ENG)], nil, nil, nil, nil, true)
		elseif TUNING.LANGUAGE == false then
		    inst.components.talker:Say(INHERIT_SHADOW_CN[math.random(#INHERIT_SHADOW_CN)], nil, nil, nil, nil, true)
        end
    else
		if TUNING.LANGUAGE == true then
	        inst.components.talker:Say(TAKEN_BY_SHADOW_ENG[math.random(#TAKEN_BY_SHADOW_ENG)], nil, nil, nil, nil, true)
		elseif TUNING.LANGUAGE == false then
		    inst.components.talker:Say(TAKEN_BY_SHADOW_CN[math.random(#TAKEN_BY_SHADOW_CN)], nil, nil, nil, nil, true)
        end		
	end			
    if inst.components.transformable ~= nil then
		inst.components.transformable.ignoresanitychange = true
		inst:DoTaskInTime(3.1, function() inst.components.transformable.ignoresanitychange = false end)
		if inst.components.transformable.shadowform then
		    inst.components.transformable:SetUpNormalForm(true)  --Don't update health when killed
		end
		inst.components.sanity:SetPercent(1)  --This will not affect sanity value after resurrection.
	end
end

local function ResetDropShadowFuel(inst)
    inst.dropshadowfuelcooldown = nil 
    if inst.dropshadowfueltask then
	    inst.dropshadowfueltask:Cancel()
		inst.dropshadowfueltask = nil 
	end 
	if inst.dropshadowfueltaskinfo then
	    inst.dropshadowfueltaskinfo = nil 
	end
end

local function ResetSapwnShadowPuddle(inst)
    inst.spawnshadowpuddlecooldown = nil 
    if inst.shadowpuddlespawntask then
	    inst.shadowpuddlespawntask:Cancel()
		inst.shadowpuddlespawntask = nil 
	end 
	if inst.shadowpuddlespawntaskinfo then
	    inst.shadowpuddlespawntaskinfo = nil 
	end
end 

local function _IsPointCloseToImpassable(x, y, z, radius)
    local map = GetWorld().Map
    
    for i = -radius, radius, 1 do
        if map:GetTileAtPoint(x - radius, y, z + i) == GROUND.IMPASSABLE or map:GetTileAtPoint(x + radius, y, z + i) == GROUND.IMPASSABLE then
            return true
        end
    end

    for i = -(radius - 1), radius - 1, 1 do
        if map:GetTileAtPoint(x + i, y, z - radius) == GROUND.IMPASSABLE or map:GetTileAtPoint(x + i, y, z + radius) == GROUND.IMPASSABLE then
            return true
        end
    end

    return false
end 

local function CheckPeriodicShadowPuddleSpawn(inst)
    if inst:HasTag("shadow_form_on") and inst.supershadowmanipulation then
	    if not inst.spawnshadowpuddlecooldown then
		    local x, y, z = inst.Transform:GetWorldPosition() 
			local find_shadowpuddle = TheSim:FindEntities(x, y, z, 3, {"shadowpuddle"}) 
			local find_base = TheSim:FindEntities(x, y, z, 10, nil, nil, {"wall", "structure"}) 
			if not _IsPointCloseToImpassable(x, y, z, 3) and #find_shadowpuddle <= 0 and #find_base <= 0 and (inst.IsPosSurroundedByLand == nil or inst:IsPosSurroundedByLand(x, y, z, 7)) then
			    local shadowpuddle = SpawnPrefab("shadowpuddle")
			    shadowpuddle.Transform:SetPosition(x, y, z) 
				if shadowpuddle.StartingAnimation ~= nil and shadowpuddle.LoopingAnimation ~= nil then 
				    shadowpuddle.AnimState:PlayAnimation(shadowpuddle.StartingAnimation)
					shadowpuddle.AnimState:PushAnimation(shadowpuddle.LoopingAnimation, true)
				end 
		        inst.spawnshadowpuddlecooldown = true 
			    if not inst.shadowpuddlespawntask then 
		            inst.shadowpuddlespawntask, inst.shadowpuddlespawntaskinfo = inst:ResumeTask(240, ResetSapwnShadowPuddle)
                end 						
			end 
		end 
	end 
end

local function DropShadowFuelAsShadowManipulation(inst) 
	if math.random() < 0.1 then
        local horrorfuel = SpawnPrefab("horrorfuel")
		horrorfuel.Transform:SetPosition(inst.Transform:GetWorldPosition())
		horrorfuel.Physics:SetVel(math.random()*math.cos(math.random()*2*math.pi), GetRandomWithVariance(8, 4), math.random()*math.sin(math.random()*2*math.pi))
    	if horrorfuel.GetIsOnWater ~= nil and horrorfuel:GetIsOnWater()	then
		    horrorfuel:DoTaskInTime(1, function() 
			    if horrorfuel.components.inventoryitem == nil or not horrorfuel.components.inventoryitem:IsHeld() then 
		            if horrorfuel.components.floatable ~= nil then
	    	            horrorfuel.components.floatable:OnHitWater(true)
		        	elseif horrorfuel.components.inventoryitem ~= nil then
		                horrorfuel.components.inventoryitem:OnHitWater()
		            end		
				end 
            end)		
		end			
    else
        local nightmarefuel = SpawnPrefab("nightmarefuel") 
		nightmarefuel.Transform:SetPosition(inst.Transform:GetWorldPosition())
		nightmarefuel.Physics:SetVel(math.random()*math.cos(math.random()*2*math.pi), GetRandomWithVariance(8, 4), math.random()*math.sin(math.random()*2*math.pi))
    	if nightmarefuel.GetIsOnWater ~= nil and nightmarefuel:GetIsOnWater() then
		    nightmarefuel:DoTaskInTime(1, function() 
			    if nightmarefuel.components.inventoryitem == nil or not nightmarefuel.components.inventoryitem:IsHeld() then 
		            if nightmarefuel.components.floatable ~= nil then
	    	            nightmarefuel.components.floatable:OnHitWater(true)
		        	elseif nightmarefuel.components.inventoryitem ~= nil then
		                nightmarefuel.components.inventoryitem:OnHitWater()
		            end		
				end 
            end)				
		end				
	end		
	inst.dropshadowfuelcooldown = true
	if not inst.dropshadowfueltask then
	    inst.dropshadowfueltask, inst.dropshadowfueltaskinfo = inst:ResumeTask(120, ResetDropShadowFuel)
	end 
end

local function BecomeShadowForm(inst)     
    inst.shadowform = true
	UpdateVariableValues(inst)
	SpawnPrefab("statue_transition").entity:SetParent(inst.entity)
	SpawnPrefab("statue_transition_2").entity:SetParent(inst.entity)
	if TUNING.SHADOW_CHARLIE_MULT ~= 0 then
	    inst.AnimState:SetMultColour(TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT)
	else
	    inst.AnimState:SetMultColour(TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT, TUNING.SHADOW_CHARLIE_MULT, 0.5)
	end
	inst.components.talker.colour = Vector3(0.9, 0.29, 0.29, 1)
	inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * TUNING.CHARLIE_SHADOW_WALK_SPEED_MULT_DS
	inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * TUNING.CHARLIE_SHADOW_RUN_SPEED_MULT_DS
	inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE*TUNING.SHADOW_CHARLIE_HUNGER_RATE_MULT)
	inst.components.hunger:SetKillRate(TUNING.SHADOW_CHARLIE_HUNGER_KILL_RATE_MULT)
	inst.components.combat:SetAttackPeriod(TUNING.WILSON_ATTACK_PERIOD * TUNING.SHADOW_ATTACK_PERIOD_MODE)
	inst.Light:Enable(true)
	inst.DynamicShadow:Enable(false)
	inst.components.burnable:SetBurnTime(0)
	inst:AddTag("shadow_form_on")
	inst:AddTag("not_hit_stunned")
	inst.components.health:SetMaxHealth(TUNING.SHADOW_CHARLIE_HEALTH)
	inst.components.health:DoDelta(0, nil, nil, true)   --To update health hud
	inst.Physics:ClearCollisionMask()
    inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
    inst.Physics:CollidesWith(COLLISION.WORLD)
	inst.Physics:CollidesWith(COLLISION.SANITY)
    if inst.components.dsskins ~= nil then
		inst.components.dsskins:Changeskin(true)
    end
    if inst.components.absorblightchecker ~= nil then
		inst.components.absorblightchecker:Enable(true)
    end	
    if inst.components.physicsthingsshadowdestroyer ~= nil then
		inst.components.physicsthingsshadowdestroyer:Enable(true)
    end	
    if inst.components.autoshadowabsorber ~= nil then
		inst.components.autoshadowabsorber:CheckIfCanEnable()
    end		
    if inst:HasTag("lightsource") then       
        inst:RemoveTag("lightsource")    
    end	
	CheckPeriodicShadowPuddleSpawn(inst)
end

local function BecomeNormalForm(inst)     
    inst.shadowform = nil
    UpdateVariableValues(inst)
	SpawnPrefab("statue_transition").entity:SetParent(inst.entity)
	SpawnPrefab("statue_transition_2").entity:SetParent(inst.entity)
	inst.AnimState:SetMultColour(1, 1, 1, 1)
	inst.components.talker.colour = Vector3(255/255, 160/255, 203/255, 1) 
	inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * TUNING.CHARLIE_WALK_SPEED_MULT_DS
    inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * TUNING.CHARLIE_RUN_SPEED_MULT_DS
	inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE*TUNING.CHARLIE_HUNGER_RATE_MULT_VANILLA)
	inst.components.hunger:SetKillRate(TUNING.CHARLIE_HUNGER_KILL_RATE_MULT)
	inst.components.combat:SetAttackPeriod(TUNING.WILSON_ATTACK_PERIOD * TUNING.UNSHADOW_ATTACK_PERIOD_MODE)
	if not inst.IsInInsight then
	    inst.Light:Enable(false)
	end
	if inst.components.inventory ~= nil and inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then 
	    local limited_to_shadow_form_item = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS):HasTag("limited_to_shadow_form") or nil 
		if limited_to_shadow_form_item ~= nil then 
		    inst.components.inventory:DropItem(inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS), true, true)
		end 
	end 	
	inst.DynamicShadow:Enable(true)
	inst.components.burnable:SetBurnTime(TUNING.CHARLIE_BURN_TIME)
	inst:RemoveTag("shadow_form_on")
	inst:RemoveTag("not_hit_stunned")
	inst.components.health:SetMaxHealth(TUNING.NORMAL_CHARLIE_HEALTH)
	inst.components.health:DoDelta(0, nil, nil, true)  --To update health hud
	inst.Physics:ClearCollisionMask()
    inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
    inst.Physics:CollidesWith(COLLISION.WORLD)
    inst.Physics:CollidesWith(COLLISION.OBSTACLES)
    inst.Physics:CollidesWith(COLLISION.CHARACTERS)
    if inst.components.dsskins ~= nil then
		inst.components.dsskins:Changeskin()
    end	
    if inst.components.absorblightchecker ~= nil then
		inst.components.absorblightchecker:Enable(false)
    end	
    if inst.components.physicsthingsshadowdestroyer ~= nil then
		inst.components.physicsthingsshadowdestroyer:Enable(false)
    end	
    if inst.components.autoshadowabsorber ~= nil then
		inst.components.autoshadowabsorber:CheckIfCanEnable()
    end	
	if inst.shadowmanipulation and not inst.dropshadowfuelcooldown then
	    DropShadowFuelAsShadowManipulation(inst)
	end
	CheckPeriodicShadowPuddleSpawn(inst) 
end

local function OnEatSomething(inst, data)
	if data ~= nil and data.food ~= nil and data.food.prefab == "flowersalad" then
		if inst.components.sanity ~= nil then
			inst.components.sanity:DoDelta(TUNING.SANITY_SMALL)
		end
	end
end

--Leaved this for modder:
--[[
local function OnHealthDeltaChange(inst)
    if inst.components.health.currenthealth <= TUNING.CHARLIE_HEALTH_LOW then
        if (inst.components.transformable ~= nil and inst.components.transformable.shadowform) or (inst.shadowform ~= nil) then
			if TUNING.LANGUAGE == true then
	            inst.components.talker:Say(INHERIT_SHADOW_ENG[math.random(#INHERIT_SHADOW_ENG)], nil, nil, nil, nil, true)
		    elseif TUNING.LANGUAGE == false then
		     	inst.components.talker:Say(INHERIT_SHADOW_CN[math.random(#INHERIT_SHADOW_CN)], nil, nil, nil, nil, true)
            end
        else
			if TUNING.LANGUAGE == true then
	            inst.components.talker:Say(TAKEN_BY_SHADOW_ENG[math.random(#TAKEN_BY_SHADOW_ENG)], nil, nil, nil, nil, true)
		    elseif TUNING.LANGUAGE == false then
		        inst.components.talker:Say(TAKEN_BY_SHADOW_CN[math.random(#TAKEN_BY_SHADOW_CN)], nil, nil, nil, nil, true)
            end		
		end
	end
end
]]

local function IsEntValidToDropPlantFood(inst)
    return not (inst:HasTag("veggie") or 
                inst:HasTag("structure") or 
                inst:HasTag("wall") or 
                inst:HasTag("balloon") or 
                inst:HasTag("chess") or 
                inst:HasTag("shadow") or 
                inst:HasTag("shadowcreature") or 
				inst:HasTag("player") or 
				inst:HasTag("plant_mob") or 
				inst:HasTag("INLIMBO") or
				inst:HasTag("INTERIOR_LIMBO") or
				inst:HasTag("no_durability_loss_on_hit") or
                inst:HasTag("smashable"))
		and ( inst.prefab ~= "pugalisk_body" and inst.prefab ~= "pugalisk_tail" and inst.prefab ~= "pugalisk_segment" )
        and ( inst.components.combat ~= nil and inst.components.health ~= nil )
end

local function IsEntValidToDropExp(inst)
    return not (inst:HasTag("veggie") or 
                inst:HasTag("structure") or 
                inst:HasTag("wall") or 
                inst:HasTag("balloon") or 
                inst:HasTag("chess") or 
				inst:HasTag("player") or 
				inst:HasTag("INLIMBO") or
				inst:HasTag("INTERIOR_LIMBO") or
				inst:HasTag("no_durability_loss_on_hit") or
                inst:HasTag("smashable"))
		and ( inst.prefab ~= "pugalisk_body" and inst.prefab ~= "pugalisk_tail" and inst.prefab ~= "pugalisk_segment" )
        and ( inst.components.combat ~= nil and inst.components.health ~= nil )
end

local function OnKill(inst, data)
    if data == nil then
        return
    end
	if data.cause == inst.prefab and not inst:HasTag("critterexpert")
	    and not inst:HasTag("sanitylosscooldown")
	    and data.inst:HasTag("smallcreature")
        and not data.inst:HasTag("piko")
		and not data.inst:HasTag("bee")
		and not data.inst:HasTag("teenbird")
		and not data.inst:HasTag("catcoon")
		and not data.inst:HasTag("pog")
		and not data.inst:HasTag("hostile")
		and not data.inst:HasTag("killer")
		and not data.inst:HasTag("mosquito")
		and not data.inst:HasTag("penguin")
		and not data.inst:HasTag("cocoon")
		and not data.inst:HasTag("frog") then
        if inst.components.sanity ~= nil then
            inst.components.sanity:DoDelta(-TUNING.SANITY_TINY)
        end 
        if TUNING.LANGUAGE == true then
	      	inst.components.talker:Say(KILL_INNOCENT_ENG[math.random(#KILL_INNOCENT_ENG)])
		elseif TUNING.LANGUAGE == false then
			inst.components.talker:Say(KILL_INNOCENT_CN[math.random(#KILL_INNOCENT_CN)])
		end
		inst:AddTag("sanitylosscooldown")
		if not inst.components.timer then
            inst:AddComponent("timer")
		end
		inst.components.timer:StartTimer("sanityloss_cooldown", TUNING.CHARLIE_SANITYLOSS_COOLDOWNTIME)

	elseif data.cause ~= inst.prefab and not inst:HasTag("critterexpert") and not inst:HasTag("sanitylosscooldown")
	    and data.inst:HasTag("smallcreature") and inst:GetDistanceSqToInst(data.inst) < 8*8
		and not data.inst:HasTag("piko")
		and not data.inst:HasTag("bee")
		and not data.inst:HasTag("teenbird")
        and not data.inst:HasTag("catcoon")
        and not data.inst:HasTag("pog")		
		and not data.inst:HasTag("hostile")
		and not data.inst:HasTag("killer")
		and not data.inst:HasTag("mosquito")
		and not data.inst:HasTag("penguin")
		and not data.inst:HasTag("cocoon")
		and not data.inst:HasTag("frog") then
        if inst.components.sanity ~= nil then
            inst.components.sanity:DoDelta(-TUNING.SANITY_SUPERTINY*2)
        end
        if TUNING.LANGUAGE == true then
	      	inst.components.talker:Say(INNOCENT_DIE_ENG[math.random(#KILL_INNOCENT_ENG)])
		elseif TUNING.LANGUAGE == false then
			inst.components.talker:Say(INNOCENT_DIE_CN[math.random(#KILL_INNOCENT_CN)])
		end
		inst:AddTag("sanitylosscooldown")
		if not inst.components.timer then
            inst:AddComponent("timer")
		end
		inst.components.timer:StartTimer("sanityloss_cooldown", TUNING.CHARLIE_SANITYLOSS_COOLDOWNTIME)		
	end
	
	--"Buttercup" Update July 2023, now the mobs have chance to drop Plant Food:
	local chance_to_drop = 0.01
	if data.cause == inst.prefab then
	    chance_to_drop = 0.02
	end
	
	if data.inst ~= nil and data.inst.components.health ~= nil then
	    chance_to_drop = chance_to_drop + (data.inst.components.health.maxhealth / 10000)
	end
	
	if data.inst ~= nil and data.inst.components.combat ~= nil then
	    chance_to_drop = chance_to_drop + (data.inst.components.combat.defaultdamage / 1000)
	end	
	
	if data.inst ~= nil and data.inst.Transform ~= nil and IsEntValidToDropPlantFood(data.inst) then
	    local x, y, z = data.inst.Transform:GetWorldPosition()
	    if data.inst:HasTag("epic") then
		    for i=1, 8 do
			    if data.inst.components.lootdropper ~= nil then 
				    local pvz2_plantfood = SpawnPrefab("pvz2_plantfood")
                    data.inst.components.lootdropper:FlingItem(pvz2_plantfood, Vector3(x, math.random()*4, z)) 
					if pvz2_plantfood.GetIsOnWater ~= nil and pvz2_plantfood:GetIsOnWater() then 
					    pvz2_plantfood:DoTaskInTime(1, function()
                            if pvz2_plantfood.components.inventoryitem == nil or not pvz2_plantfood.components.inventoryitem:IsHeld() then
							    if pvz2_plantfood.components.floatable ~= nil then 
								    pvz2_plantfood.components.floatable:OnHitWater(true) 
								elseif pvz2_plantfood.components.inventoryitem ~= nil then 
								    pvz2_plantfood.components.inventoryitem:OnHitWater()  
								end 
							end 
					    end) 
					end 
				else
				    local pvz2_plantfood = SpawnPrefab("pvz2_plantfood")
			        pvz2_plantfood.Transform:SetPosition(x, math.random()*4, z)
					pvz2_plantfood.Physics:SetVel(math.random()*math.cos(math.random()*2*math.pi), GetRandomWithVariance(8, 4), math.random()*math.sin(math.random()*2*math.pi))
					if pvz2_plantfood.GetIsOnWater ~= nil and pvz2_plantfood:GetIsOnWater() then 
					    pvz2_plantfood:DoTaskInTime(1, function()
                            if pvz2_plantfood.components.inventoryitem == nil or not pvz2_plantfood.components.inventoryitem:IsHeld() then
							    if pvz2_plantfood.components.floatable ~= nil then 
								    pvz2_plantfood.components.floatable:OnHitWater(true) 
								elseif pvz2_plantfood.components.inventoryitem ~= nil then 
								    pvz2_plantfood.components.inventoryitem:OnHitWater()  
								end 
							end 
					    end) 
					end 					
				end
			end
		elseif math.random() < chance_to_drop then
			if data.inst.components.lootdropper ~= nil then 
			    local pvz2_plantfood = SpawnPrefab("pvz2_plantfood")
                data.inst.components.lootdropper:FlingItem(pvz2_plantfood, Vector3(x, math.random()*4, z)) 
				if pvz2_plantfood.GetIsOnWater ~= nil and pvz2_plantfood:GetIsOnWater() then 
				    pvz2_plantfood:DoTaskInTime(1, function()
                        if pvz2_plantfood.components.inventoryitem == nil or not pvz2_plantfood.components.inventoryitem:IsHeld() then
						    if pvz2_plantfood.components.floatable ~= nil then 
							    pvz2_plantfood.components.floatable:OnHitWater(true) 
							elseif pvz2_plantfood.components.inventoryitem ~= nil then 
							    pvz2_plantfood.components.inventoryitem:OnHitWater()  
							end 
						end 
				    end) 
				end 				
			else
			    local pvz2_plantfood = SpawnPrefab("pvz2_plantfood")
			    pvz2_plantfood.Transform:SetPosition(x, math.random()*4, z) 
				pvz2_plantfood.Physics:SetVel(math.random()*math.cos(math.random()*2*math.pi), GetRandomWithVariance(8, 4), math.random()*math.sin(math.random()*2*math.pi))
				if pvz2_plantfood.GetIsOnWater ~= nil and pvz2_plantfood:GetIsOnWater() then 
				    pvz2_plantfood:DoTaskInTime(1, function()
                        if pvz2_plantfood.components.inventoryitem == nil or not pvz2_plantfood.components.inventoryitem:IsHeld() then
						    if pvz2_plantfood.components.floatable ~= nil then 
							    pvz2_plantfood.components.floatable:OnHitWater(true) 
							elseif pvz2_plantfood.components.inventoryitem ~= nil then 
							    pvz2_plantfood.components.inventoryitem:OnHitWater()  
							end 
						end 
				    end) 
				end 								
			end			
		end	
	end
	
	--Drop Exp:
	local chance_to_drop_exp = 0
	if data.cause == inst.prefab and inst.components.inventory ~= nil and inst.components.inventory:EquipHasTag("shadow_torch") then
	    chance_to_drop_exp = 0.25		
        local shadowboxes = inst.superemberguru and inst.components.inventory:FindItems(function(item) 
		    if item ~= nil and item.prefab == "charlie_shadowbox" then
			    return true
			end
		    return false
		end) or {}
		if #shadowboxes > 0 then
		    for i=1, #shadowboxes do
			    chance_to_drop_exp = chance_to_drop_exp + 0.05 
				if chance_to_drop_exp >= 1 then
				    break
				end
			end
		end	
	end
	
	if data.inst ~= nil and data.inst.Transform ~= nil and IsEntValidToDropExp(data.inst) and chance_to_drop_exp > 0 then
	    local x2, y2, z2 = data.inst.Transform:GetWorldPosition()
	    if data.inst:HasTag("epic") then
		    for i=1, 8 do
			    if data.inst.components.lootdropper ~= nil then
				    local charlie_exp = SpawnPrefab("charlie_exp")
                    data.inst.components.lootdropper:FlingItem(charlie_exp, Vector3(x2, math.random()*4, z2))
					charlie_exp.AnimState:PlayAnimation("idle_pre")
				    charlie_exp.AnimState:PushAnimation("idle_loop", true)	
				else
			        local charlie_exp = SpawnPrefab("charlie_exp")
	                charlie_exp.Transform:SetPosition(x2, math.random()*4, z2)
				    charlie_exp.Physics:SetVel(math.random()*math.cos(math.random()*2*math.pi), GetRandomWithVariance(8, 4), math.random()*math.sin(math.random()*2*math.pi))
					charlie_exp.AnimState:PlayAnimation("idle_pre")
				    charlie_exp.AnimState:PushAnimation("idle_loop", true)	
				end
			end
		elseif math.random() < chance_to_drop_exp then
			if data.inst.components.lootdropper ~= nil then
			    local charlie_exp = SpawnPrefab("charlie_exp")
                data.inst.components.lootdropper:FlingItem(charlie_exp, Vector3(x2, math.random()*4, z2))
				charlie_exp.AnimState:PlayAnimation("idle_pre")
				charlie_exp.AnimState:PushAnimation("idle_loop", true)	
			else
			    local charlie_exp = SpawnPrefab("charlie_exp")
	            charlie_exp.Transform:SetPosition(x2, math.random()*4, z2)
				charlie_exp.Physics:SetVel(math.random()*math.cos(math.random()*2*math.pi), GetRandomWithVariance(8, 4), math.random()*math.sin(math.random()*2*math.pi))
				charlie_exp.AnimState:PlayAnimation("idle_pre")
				charlie_exp.AnimState:PushAnimation("idle_loop", true)	
			end			
		end	
	end	
	
	--Drop Dragon Feather:
	local chance_to_drop_dragon_feather = 0
	if data.inst ~= nil and data.cause == inst.prefab and IsEntValidToDropExp(data.inst) then
	    if data.inst.prefab == "robin" or data.inst.prefab == "parrot" or data.inst.prefab == "parrot_pirate" then
		    chance_to_drop_dragon_feather = 0.05
	    elseif data.inst.prefab == "firehound" or data.inst.prefab == "shark" or data.inst.prefab == "snake" then
		    chance_to_drop_dragon_feather = 0.1
		elseif data.inst.prefab == "dragoon" then
		    chance_to_drop_dragon_feather = 0.15
		elseif data.inst.prefab == "dragonfly" 
		    or data.inst.prefab == "tigershark" 
			or data.inst.prefab == "pugalisk" 
			or data.inst.prefab == "ancient_herald"
			or data.inst.prefab == "ancient_hulk" then
		    chance_to_drop_dragon_feather = 1
		end
	end
	
	if chance_to_drop_dragon_feather > 0 then
	    local x3, y3, z3 = data.inst.Transform:GetWorldPosition()
	    if data.inst:HasTag("epic") then
		    for i=1, 4 do
			    if data.inst.components.lootdropper ~= nil then
                    data.inst.components.lootdropper:SpawnLootPrefab("charlie_dragon_feather")
				end
			end
		elseif math.random() < chance_to_drop_dragon_feather then
			if data.inst.components.lootdropper ~= nil then
			    data.inst.components.lootdropper:SpawnLootPrefab("charlie_dragon_feather")
            end				
		end		
	end
	
end

local function OnKillinInv(inst, data)
    if data == nil then
        return
    end
    local victim = data.victim
    if victim:HasTag("smallcreature") and not inst:HasTag("critterexpert")
	    and not inst:HasTag("sanitylosscooldown")
        and not victim:HasTag("piko")
		and not victim:HasTag("bee")
		and not victim:HasTag("teenbird")
		and not victim:HasTag("catcoon")
		and not victim:HasTag("pog")
		and not victim:HasTag("hostile")
		and not victim:HasTag("killer")
		and not victim:HasTag("mosquito")
		and not victim:HasTag("penguin")
		and not victim:HasTag("cocoon")
		and not victim:HasTag("frog") then
        if inst.components.sanity ~= nil then
            inst.components.sanity:DoDelta(-TUNING.SANITY_TINY)
        end 	
        if TUNING.LANGUAGE == true then
	      	inst.components.talker:Say(KILL_INNOCENT_ENG[math.random(#KILL_INNOCENT_ENG)])
		elseif TUNING.LANGUAGE == false then
			inst.components.talker:Say(KILL_INNOCENT_CN[math.random(#KILL_INNOCENT_CN)])
		end
		inst:AddTag("sanitylosscooldown")
		if not inst.components.timer then
            inst:AddComponent("timer")
		end
		inst.components.timer:StartTimer("sanityloss_cooldown", TUNING.CHARLIE_SANITYLOSS_COOLDOWNTIME)
    end			
end

local function IsWaterPoint(x, y, z)
    if GetPlayer().GetIsOnWater == nil then
	    return false
	end
    return GetWorld().Map:IsWater(GetWorld().Map:GetTileAtPoint(x, y, z))
end

local function IsPointNearWater(x, y, z, r)
    local radius = r or 1.5

    for i = -radius, radius, 1 do
        if IsWaterPoint(x - radius, y, z + i) or IsWaterPoint(x + radius, y, z + i) then
            return true
        end	
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if IsWaterPoint(x + i, y, z - radius) or IsWaterPoint(x + i, y, z + radius) then
            return true
        end
    end
	
    return false
end

local function IsPointCloseToLava(pt, radius)
    if GetPlayer().GetIsOnWater == nil then
	    return false
	end

    for i = -radius, radius, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x - radius, pt.y, pt.z + i) == GROUND.VOLCANO_LAVA 
		    or GetWorld().Map:GetTileAtPoint(pt.x + radius, pt.y, pt.z + i) == GROUND.VOLCANO_LAVA 
		then
            return true
        end
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z - radius) == GROUND.VOLCANO_LAVA
		    or GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z + radius) == GROUND.VOLCANO_LAVA 
		then
            return true
        end
    end
	return false
end


local function IsPointOutsideInteriorWall(x, y, z)
    if TheCamera.interior then
        local interiorSpawner = GetWorld().components.interiorspawner 
        if GetWorld() and interiorSpawner and interiorSpawner.current_interior then
            local originpt = interiorSpawner:getSpawnOrigin()
            local width = interiorSpawner.current_interior.width
            local depth = interiorSpawner.current_interior.depth        
            local dMax = originpt.x + depth/2
            local dMin = originpt.x - depth/2
            local wMax = originpt.z + width/2
            local wMin = originpt.z - width/2        
            local dist = 0.5
            if x < dMin+dist or x > dMax-dist or z < wMin+dist or z > wMax-dist then
                return true
            end
        end
    end
    return false
end

local function OnAttacked(inst, data)
    if not inst:IsValid() then 
	    return
	end
    if inst.components.health ~= nil and inst.components.health:IsDead() then
	    return
	end
	
    if inst.components.inventory ~= nil
	    and inst.components.inventory:EquipHasTag("shadow_helmet") 
		and data ~= nil
		and data.attacker ~= nil
		and data.attacker.components.combat ~= nil
		and not data.attacker:HasTag("elephantcactus")
		and not data.attacker:HasTag("bramble")
		and not data.attacker:HasTag("buttercup")
	then
        SpawnPrefab("statue_transition").entity:SetParent(data.attacker.entity)
	    SpawnPrefab("statue_transition_2").entity:SetParent(data.attacker.entity)
	    data.attacker.components.combat:GetAttacked(inst, 100)
		if data.attacker.Transform ~= nil then
		    local x, y, z = data.attacker.Transform:GetWorldPosition()
			local ents = TheSim:FindEntities(x, y, z, 5, DAMAGE_MUST_TAGS, DAMAGE_CANT_TAGS)
			for k,v in pairs(ents) do
			    if v ~= nil 
				    and v ~= inst 
					and v ~= data.attacker 
				    and v:IsValid() 
					and v.entity:IsVisible() 
					and v.components.combat ~= nil
				then
                    SpawnPrefab("statue_transition").entity:SetParent(v.entity)
	                SpawnPrefab("statue_transition_2").entity:SetParent(v.entity)				
				    v.components.combat:GetAttacked(inst, 100)
				end
			end	
		end
	end	
	
    if (inst.components.transformable ~= nil and inst.components.transformable.shadowform) or (inst.shadowform ~= nil) then
	    local pt = inst:GetPosition()
	    local fx = SpawnPrefab("small_puff")
	    fx.Transform:SetPosition(pt.x, pt.y+0.1, pt.z)
		fx.Transform:SetScale(1.2, 1.2, 1.2)
	    inst:Hide()
		inst.components.health:SetInvincible(true)
		inst:DoTaskInTime(0.25, function() 
			inst.components.health:SetInvincible(false)	
			inst:Show()
			if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
                local pt = inst:GetPosition()
                local tiletype = GetGroundTypeAtPosition(pt)
                local tile = GetWorld().Map:GetTileAtPoint(pt.x,pt.y,pt.z)		
		        local OnWater = GetWorld().Map:IsWater(tile)
                if inst:HasTag("aquatic") then    --I leave this for some rare situations
                    local offset = FindWaterOffset(inst:GetPosition(), 2*math.pi*math.random(), 10, 12)
                    local pos = inst:GetPosition()
                    if offset then
                        pos = pos + offset
                        inst.Transform:SetPosition(pos:Get())
                    end
			    else
                    if GetWorld().Map then               
					    for k = 1, 4 do
						    local pos = Vector3(inst.Transform:GetWorldPosition())
					    	local offset = 10
					    	pos.x = pos.x + (math.random(2*offset)-offset)          
					     	pos.z = pos.z + (math.random(2*offset)-offset)
					    	local tile = GetWorld().Map:GetTileAtPoint(pos:Get())
						    local OnWater = GetWorld().Map:IsWater(tile)
						    if tile ~= GROUND.IMPASSABLE 
							    and not IsPointNearWater(pos:Get()) 
								and not IsPointOutsideInteriorWall(pos:Get())
								and not IsPointCloseToLava(pos, 1)
							then
						     	inst.Transform:SetPosition(pos:Get())
						    	break
						    end
					    end
                    end				
			    end		
	        else
                if GetWorld().Map then        
				    for k = 1, 4 do
				    	local pos = Vector3(inst.Transform:GetWorldPosition())
				    	local offset = 10
				    	pos.x = pos.x + (math.random(2*offset)-offset)          
				     	pos.z = pos.z + (math.random(2*offset)-offset)
				    	if GetWorld().Map:GetTileAtPoint(pos:Get()) ~= GROUND.IMPASSABLE then
					    	inst.Transform:SetPosition(pos:Get())
					    	break
				    	end
		    		end
                end            
		    end
	        local pt = inst:GetPosition()
	        local fx = SpawnPrefab("small_puff")
	        fx.Transform:SetPosition(pt.x, pt.y+0.1, pt.z)
			fx.Transform:SetScale(1.2, 1.2, 1.2)
		end)
	end
end

local function OnBoatAttacked(inst, data)
    if not inst:IsValid() then 
	    return
	end
    if inst.components.health ~= nil and inst.components.health:IsDead() then
	    return
	end

    if inst.components.inventory ~= nil
	    and inst.components.inventory:EquipHasTag("shadow_helmet") 
		and data ~= nil
		and data.attacker ~= nil
		and data.attacker.components.combat ~= nil
		and not data.attacker:HasTag("elephantcactus")
		and not data.attacker:HasTag("bramble")
		and not data.attacker:HasTag("buttercup")
	then
        SpawnPrefab("statue_transition").entity:SetParent(data.attacker.entity)
	    SpawnPrefab("statue_transition_2").entity:SetParent(data.attacker.entity)
	    data.attacker.components.combat:GetAttacked(inst, 100)
		if data.attacker.Transform ~= nil then
		    local x, y, z = data.attacker.Transform:GetWorldPosition()
			local ents = TheSim:FindEntities(x, y, z, 5, DAMAGE_MUST_TAGS, DAMAGE_CANT_TAGS)
			for k,v in pairs(ents) do
			    if v ~= nil 
				    and v ~= inst 
					and v ~= data.attacker 
				    and v:IsValid() 
					and v.entity:IsVisible() 
					and v.components.combat ~= nil
				then
                    SpawnPrefab("statue_transition").entity:SetParent(v.entity)
	                SpawnPrefab("statue_transition_2").entity:SetParent(v.entity)				
				    v.components.combat:GetAttacked(inst, 100)
				end
			end	
		end
	end

    if (inst.components.transformable ~= nil and inst.components.transformable.shadowform) or (inst.shadowform ~= nil) then
	    local pt = inst:GetPosition()
	    local fx = SpawnPrefab("small_puff")
	    fx.Transform:SetPosition(pt.x, pt.y+0.1, pt.z)
		fx.Transform:SetScale(1.2, 1.2, 1.2)
	    inst:Hide()
		inst.components.health:SetInvincible(true)
		if inst.components.driver and inst.components.driver.vehicle and inst.components.driver.vehicle.components.boathealth then
		    inst.components.driver.vehicle.components.boathealth:SetInvincible(true)
		end
		inst:DoTaskInTime(0.25, function() 
			inst.components.health:SetInvincible(false)
		    if inst.components.driver and inst.components.driver.vehicle and inst.components.driver.vehicle.components.boathealth then
		        inst.components.driver.vehicle.components.boathealth:SetInvincible(false)
		    end			
			inst:Show()
		    if IsDLCEnabled(PORKLAND_DLC) then
                if SaveGameIndex:IsModePorkland() then   
                    if GetWorld().Map then              
					    for k = 1, 4 do
					    	local pos = Vector3(inst.Transform:GetWorldPosition())
					    	local offset = 10
					    	pos.x = pos.x + (math.random(2*offset)-offset)          
					    	pos.z = pos.z + (math.random(2*offset)-offset)
					    	local tile = GetWorld().Map:GetTileAtPoint(pos:Get())
					    	local OnWater = GetWorld().Map:IsWater(tile)
					    	if tile ~= GROUND.IMPASSABLE and OnWater then
					     		inst.Transform:SetPosition(pos:Get())
					    		break
					    	end
					    end
                    end						
			    else
                    local offset = FindWaterOffset(inst:GetPosition(), 2*math.pi*math.random(), 10, 12)
                    local pos = inst:GetPosition()
                    if offset then
                        pos = pos + offset
                        inst.Transform:SetPosition(pos:Get())
                    end	
			    end
		    else
                local offset = FindWaterOffset(inst:GetPosition(), 2*math.pi*math.random(), 10, 12)
                local pos = inst:GetPosition()
                if offset then
                    pos = pos + offset
                    inst.Transform:SetPosition(pos:Get())
                end				
		    end
	        local pt = inst:GetPosition()
	        local fx = SpawnPrefab("small_puff")
	        fx.Transform:SetPosition(pt.x, pt.y+0.1, pt.z)
			fx.Transform:SetScale(1.2, 1.2, 1.2)
		end)
	end
end

local function OnBlocked(inst, data)
    if inst.components.inventory ~= nil
	    and inst.components.inventory:EquipHasTag("super_shadow_item") 
		and data ~= nil
		and data.attacker ~= nil
		and data.attacker.components.combat ~= nil
		and not data.attacker:HasTag("elephantcactus")
		and not data.attacker:HasTag("bramble")
		and not data.attacker:HasTag("buttercup")
	then
        SpawnPrefab("statue_transition").entity:SetParent(data.attacker.entity)
	    SpawnPrefab("statue_transition_2").entity:SetParent(data.attacker.entity)
	    data.attacker.components.combat:GetAttacked(inst, 34)
	end
	
    if inst.components.inventory ~= nil
	    and inst.components.inventory:EquipHasTag("shadow_helmet") 
		and data ~= nil
		and data.attacker ~= nil
		and data.attacker.components.combat ~= nil
		and not data.attacker:HasTag("elephantcactus")
		and not data.attacker:HasTag("bramble")
		and not data.attacker:HasTag("buttercup")
	then
        SpawnPrefab("statue_transition").entity:SetParent(data.attacker.entity)
	    SpawnPrefab("statue_transition_2").entity:SetParent(data.attacker.entity)
	    data.attacker.components.combat:GetAttacked(inst, 100)
		if data.attacker.Transform ~= nil then
		    local x, y, z = data.attacker.Transform:GetWorldPosition()
			local ents = TheSim:FindEntities(x, y, z, 5, DAMAGE_MUST_TAGS, DAMAGE_CANT_TAGS)
			for k,v in pairs(ents) do
			    if v ~= nil 
				    and v ~= inst 
					and v ~= data.attacker 
				    and v:IsValid() 
					and v.entity:IsVisible() 
					and v.components.combat ~= nil
				then
                    SpawnPrefab("statue_transition").entity:SetParent(v.entity)
	                SpawnPrefab("statue_transition_2").entity:SetParent(v.entity)				
				    v.components.combat:GetAttacked(inst, 100)
				end
			end	
		end
	end	
	
end

local function CanCharlieGainSanityFromLight(inst)
    if inst:HasTag("shadow_form_on") then
	    return false
	end
	return GetClock():IsNight() or GetClock():IsDusk() or GetWorld():IsCave()
end

local function VariableSanityFn(inst)
    local delta = 0

    local flower_amount = inst.components.inventory:Count("petals")
	    + inst.components.inventory:Count("petals_evil")
        + inst.components.inventory:Count("butterfly")
        + (inst.components.inventory:Count("cactus_flower") * 5)
	    + (inst.components.inventory:Count("tuber_bloom_crop") * 5)
	    + (inst.components.inventory:Count("bramble_bulb") * 10)
	    + inst.components.inventory:Count("lotus_flower")
	    + (inst.components.inventory:Count("abigail_flower") * 10)		
	    + (inst.components.inventory:Count("glommerflower") * 10)
		+ (inst.components.inventory:Count("shadow_rose_bouquet") * 10)
		+ (inst.components.inventory:Count("rose_inv") * 10)
    local fuel_amount = inst.components.inventory:Count("nightmarefuel") + (inst.components.inventory:Count("ancient_remnant") * 5)
    local shadowform = (inst.components.transformable ~= nil and inst.components.transformable.shadowform) or inst.shadowform ~= nil
    
    if not GetClock():IsDusk() and not GetClock():IsNight() and not GetWorld():IsCave() and not shadowform then
        if flower_amount >= 30 then
            delta = TUNING.DAPPERNESS_HUGE*0.625    
        elseif flower_amount >= 20 then
            delta = TUNING.DAPPERNESS_HUGE*0.4     
        elseif flower_amount >= 10 then
            delta = TUNING.DAPPERNESS_LARGE*0.9  
        elseif flower_amount >= 1 then
            delta = TUNING.DAPPERNESS_MED   
        end
	end
		
    if shadowform then
        if fuel_amount >= 40 then
            delta = -TUNING.DAPPERNESS_HUGE*1.125    
        elseif fuel_amount >= 20 then
            delta = -TUNING.DAPPERNESS_HUGE*0.75        
        elseif fuel_amount >= 10 then
            delta = -TUNING.DAPPERNESS_LARGE*1.15  
        elseif fuel_amount >= 1 then
            delta = -TUNING.DAPPERNESS_MED*1.3   
        end	
        if GetClock() and (GetClock():GetMoonPhase() == "full" and GetClock():IsNight()) and not (GetClock().GetBloodMoon ~= nil and GetClock():GetBloodMoon()) then
	        delta = TUNING.DAPPERNESS_HUGE*5      
        elseif GetClock() and GetClock().GetBloodMoon ~= nil and GetClock():GetBloodMoon() then
	        delta = -TUNING.DAPPERNESS_HUGE*5     
        end		
	end	
	
	if CanCharlieGainSanityFromLight(inst) then
	    local x, y, z = inst.Transform:GetWorldPosition()	
	    local max_rad = 10
	    local ents = TheSim:FindEntities(x, y, z, max_rad, {"lightsource"}, {"fire", "smolder", "FX", "NOCLICK", "INLIMBO", "DECOR", "INTERIOR_LIMBO"})
        for k,v in pairs(ents) do 
    	    if v.Light ~= nil and v.Light:IsEnabled() then
    		    local sz = TUNING.SANITYAURA_TINY
    		    local rad = v.Light:GetCalculatedRadius() or 1
    		    sz = sz * ( math.min(max_rad, rad) / max_rad )
			    local distsq = inst:GetDistanceSqToInst(v)
			    delta = delta + sz/math.max(1, distsq)
    	    end
        end	
	end
	
    return delta
end

local function EnterDark(inst, data)
    if not inst:IsValid() then 
	    return
	end

    if inst.components.health ~= nil and inst.components.health:IsDead() then
	    return
	end

	if inst.IsInInsight then
	    return
	end	
	
	if inst.components.transformable ~= nil then
        inst.components.transformable:IgnoreSanityToSetUpShadowForm()
    else
        BecomeShadowForm(inst)
    end
end

local function EnterLight(inst, data)
    if not inst:IsValid() then 
	    return
	end

    if inst.components.health ~= nil and inst.components.health:IsDead() then
	    return
	end

	if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then 
        --DATA is always nil, so I have to try another way, see the component normalformchecker
    else
        if data ~= nil and data.inst ~= GetPlayer() then
	        BecomeNormalForm(inst)
		end
    end	
end

local function OnTalk(inst)
    if (inst.components.transformable ~= nil and inst.components.transformable.shadowform) or (inst.shadowform ~= nil) then
		inst.SoundEmitter:PlaySound("dontstarve/charlie/warn")
    end
end

local function CanTeleportAtPoint(inst, pos)
    local TeleportPoint = GetWorld().Map:GetTileAtPoint(pos.x, pos.y, pos.z)
    if TeleportPoint == GROUND.IMPASSABLE 
	    or IsPointOutsideInteriorWall(pos.x, pos.y, pos.z) 
	    or IsPointCloseToLava(pos, 1)
	then 
	    return false 
	end
    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
		if inst:GetIsOnWater() or inst:HasTag("aquatic") then
            return GetWorld().Map:IsWater(TeleportPoint)
		else
		    return inst:IsPosSurroundedByLand(pos.x, pos.y, pos.z, 1.5)
		end
    else
        return true
    end
end

local function GetPointSpecialActions(inst, pos, useitem, right)

	if inst.supershadowmanipulation 
	    and not TheInput:ControllerAttached() --TO DO: FIXED CONTROLLER COMPATIBILITY
	    and inst:HasTag("shadow_form_on") 
	    and inst.components.inventory ~= nil 
		and inst.components.inventory:FindItem(function(item) 
		    if item ~= nil 
			    and (item.prefab == "nightmarefuel" or item.prefab == "horrorfuel")
				and item.components.machine ~= nil
                and item.components.machine:IsOn() 
			then
			    return true
			end
		    return false
		end) 
		and inst:IsValid() 
		and right 
	then
	    return { ACTIONS.SHADOWFUELCAST }		
	end	

	if inst.emberguru 
	    and not TheInput:ControllerAttached() 
	    and inst.components.inventory ~= nil 
		and inst.components.inventory:FindItem(function(item) 
		    if item ~= nil 
			    and item.prefab == "charlie_exp" 
				and item.components.machine ~= nil
                and item.components.machine:IsOn() 
			then
			    return true
			end
		    return false
		end) 
		and inst:IsValid() 
		and right 
	then
	    return { ACTIONS.SHADOWEMBERCAST }		
	end	
	
    local IsCurrentlyShadowForm = (inst.components.transformable ~= nil and inst.components.transformable.shadowform) or (inst.shadowform ~= nil)
    if IsCurrentlyShadowForm and inst:IsValid() then
        if right and useitem == nil then
            if CanTeleportAtPoint(inst, pos) then
			    return { ACTIONS.TELEPORT }
			end
        end
	end	
	
	local HasTargetOrIsTargeted = nil  

	if inst.superdodgemaster then 
	    if inst.components.combat ~= nil and inst.components.combat.target ~= nil then 
	        HasTargetOrIsTargeted = true 
	    end 
	    local x, y, z = inst.Transform:GetWorldPosition() 
    	for k,v in pairs(TheSim:FindEntities(x, y, z, 10, {"hascombatcomponent"}, {"FX", "NOCLICK", "INLIMBO", "DECOR", "INTERIOR_LIMBO", "invisible", "notarget"})) do 
	        if v ~= nil 
		        and v:IsValid() 
		    	and v.entity:IsVisible() 
		    	and v.components.combat ~= nil 
		        and v.components.combat.target ~= nil 
		    	and v.components.combat.target == inst 
		    then 
		        HasTargetOrIsTargeted = true 
		    	break 
	    	end 
    	end 
	end 
	
	if inst.superdodgemaster 
	    and not TheInput:ControllerAttached() 
		and HasTargetOrIsTargeted 
		and not inst.masterdodging2 
		and not IsCurrentlyShadowForm 
		and inst:IsValid() 
		and right 
	then  --In Shadow Form just to use teleport action to dodge damage
		return { ACTIONS.MASTERDODGE }			
	end 
	
    return {}
end

local function OnSave(inst, data)
    if IsDLCEnabled(PORKLAND_DLC) then
	    if inst.components.grogginess ~= nil then
	        if inst.components.grogginess.grog_amount ~= nil 
			    and inst.components.grogginess.grog_amount > 0 then
		        print("Save grogginess")
		        data.grogginess = inst.components.grogginess.grog_amount
		    end
	    end
	end	
	if inst:HasTag("sanitylosscooldown") then
	    print("Save tag of sanitylosscooldown")
	    data.sanitylosscooldown = true
	end	
	if inst.components.timer ~= nil then
	    if inst.components.timer:TimerExists("sanityloss_cooldown") then
		    print("Save sanityloss_cooldown timer time left")
		    data.timer_sanityloss_cooldown = inst.components.timer:GetTimeLeft("sanityloss_cooldown")
		end
	    if inst.components.timer:TimerExists("despawn_protection") then
		    print("Save despawn_protection timer time left")
		    data.timer_despawn_protection = inst.components.timer:GetTimeLeft("despawn_protection")
		end		
	end
	if inst.shadowform ~= nil then
	    data.shadowform = true
	end
    data.first_time_set_done_skin = inst:HasTag("first_time_set_done_skin") or nil
	data.first_time_set_done_portal = inst:HasTag("first_time_set_done_portal") or nil
	data.first_time_set_done_mechete = inst:HasTag("first_time_set_done_mechete") or nil
	data.IsInInsight = inst.IsInInsight or nil
	data.page_read = inst.page_read or nil
    if inst.insighttask and inst.insighttaskinfo then
        data.insighttasktimeleft = inst:TimeRemainingInTask(inst.insighttaskinfo)
    end	
	data.SanityGainFromPlantingFlower = inst.SanityGainFromPlantingFlower or nil
	data.flowercommunicator = inst:HasTag("flowercommunicator") or nil
	data.flowersupercommunicator = inst:HasTag("flowersupercommunicator") or nil
	data.bonusrabbitdoer = inst:HasTag("bonusrabbitdoer") or nil
	data.superbonusrabbitdoer = inst:HasTag("superbonusrabbitdoer") or nil
	data.critterexpert = inst:HasTag("critterexpert") or nil
	data.supercritterexpert = inst:HasTag("supercritterexpert") or nil 
	data.swiftwarrior = inst.swiftwarrior or nil
	data.superswiftwarrior = inst.superswiftwarrior or nil
	data.dodgemaster = inst.dodgemaster or nil
	data.superdodgemaster = inst.superdodgemaster or nil 
	data.thermologist = inst.thermologist or nil
	data.superthermologist = inst.superthermologist or nil
	data.emberguru = inst.emberguru or nil
	data.superemberguru = inst.superemberguru or nil  
	data.ShadowHealthGain = inst.ShadowHealthGain or nil
	data.ShadowHealthTime = inst.ShadowHealthTime or nil 
	data.syntheticobserver = inst.syntheticobserver or nil 
	data.supersyntheticobserver = inst.supersyntheticobserver or nil 
	data.shadowmanipulation = inst.shadowmanipulation or nil 
	data.supershadowmanipulation = inst.supershadowmanipulation or nil 
    if inst.dropshadowfueltask and inst.dropshadowfueltaskinfo then
        data.dropshadowfueltimeleft = inst:TimeRemainingInTask(inst.dropshadowfueltaskinfo)
    end		
    if inst.shadowpuddlespawntask and inst.shadowpuddlespawntaskinfo then
        data.shadowpuddletimeleft = inst:TimeRemainingInTask(inst.shadowpuddlespawntaskinfo)
    end			
    if inst.taskcoffeespawntask and inst.taskcoffeespawntaskinfo then
        data.spawntaskcoffeetimeleft = inst:TimeRemainingInTask(inst.taskcoffeespawntaskinfo)
    end			
	data.PatchedDragonHouseRecipe = inst.PatchedDragonHouseRecipe or nil 
	data.shadowacider = inst.shadowacider or nil
    data.supershadowacider = inst.supershadowacider or nil	
	data.acidshadowsmoker = inst.acidshadowsmoker or nil	
	data.superacidshadowsmoker = inst.superacidshadowsmoker or nil	
	data.FirstTimeValentineDayNote = inst.FirstTimeValentineDayNote or nil
	data.FirstTimeTaskCoffeeSpawn = inst.FirstTimeTaskCoffeeSpawn or nil
end

local function OnLoad(inst, data)
    if data == nil then 
	    return 
	end
    if IsDLCEnabled(PORKLAND_DLC) then    
	    if data.grogginess then
	        if inst.components.grogginess ~= nil then
		        if inst.components.grogginess.grog_amount ~= nil then
			        print("Add grogginess")
		            inst.components.grogginess:AddGrogginess(data.grogginess)
			    end
		    end
	    end
	end	
	if data.sanitylosscooldown then
	    if not inst:HasTag("sanitylosscooldown") then
		    print("Add tag of sanitylosscooldown")
		    inst:AddTag("sanitylosscooldown")
		end
	end	
	if data.timer_sanityloss_cooldown then
	    if inst.components.timer ~= nil then
		    if inst.components.timer:TimerExists("sanityloss_cooldown") then
		        print("Load sanityloss_cooldown timer time")
				inst.components.timer:SetTimeLeft("sanityloss_cooldown", data.timer_sanityloss_cooldown)			
		    else
			    print("Load missing sanityloss_cooldown timer time")
			    inst.components.timer:StartTimer("sanityloss_cooldown", data.timer_sanityloss_cooldown)
		    end
		end
	end
	if data.timer_despawn_protection then
	    if inst.components.timer ~= nil then
		    if inst.components.timer:TimerExists("despawn_protection") then
		        print("Load despawn_protection timer time")
				inst.components.timer:SetTimeLeft("despawn_protection", data.timer_despawn_protection)			
		    else
			    print("Load missing despawn_protection timer time")
			    inst.components.timer:StartTimer("despawn_protection", data.timer_despawn_protection)
		    end
 			local protection = SpawnPrefab("spawnprotectionbuff")
			inst:AddChild(protection)
			if protection.OnEnableProtectionFn ~= nil then
			    protection:OnEnableProtectionFn(inst, true)
			end          
		end
	end		
	if data.shadowform then
	    BecomeShadowForm(inst)
	end
    if data.first_time_set_done_skin then
	    inst:AddTag("first_time_set_done_skin")
	end
    if data.first_time_set_done_portal then
	    inst:AddTag("first_time_set_done_portal")
	end
    if data.first_time_set_done_mechete then
	    inst:AddTag("first_time_set_done_mechete")
	end
	if data.IsInInsight then
	    inst.IsInInsight = true
		inst.Light:Enable(true)
	end	
	if data.insighttasktimeleft then
	    inst:SetExitInsightStateTask(data.insighttasktimeleft)		
	end
    if data.page_read then
        inst.page_read = data.page_read
    end	
	if data.SanityGainFromPlantingFlower then
	    inst.SanityGainFromPlantingFlower = data.SanityGainFromPlantingFlower
	end
    if data.flowercommunicator then
	    inst:AddTag("flowercommunicator")
	end
    if data.flowersupercommunicator then
	    inst:AddTag("flowersupercommunicator")
	end	
    if data.bonusrabbitdoer then
	    inst:AddTag("bonusrabbitdoer")
	end
    if data.superbonusrabbitdoer then
	    inst:AddTag("superbonusrabbitdoer")
	end	
    if data.critterexpert then
	    inst:AddTag("critterexpert")
	end
    if data.supercritterexpert then
	    inst:AddTag("supercritterexpert")
		inst:RemoveTag("scarytoprey")
	end	
	if data.swiftwarrior then
	    inst.swiftwarrior = true
	end
	if data.superswiftwarrior then
	    inst.superswiftwarrior = true
	end 
	if data.dodgemaster then
	    inst.dodgemaster = true
	end
	if data.superdodgemaster then
	    inst.superdodgemaster = true
	end
	if data.thermologist then
	    inst.thermologist = true
	end
	if data.superthermologist then
	    inst.superthermologist = true
	end 
	if data.emberguru then
	    inst.emberguru = true
	end
	if data.superemberguru then
	    inst.superemberguru = true
	end	
	if data.ShadowHealthGain then
	    inst.ShadowHealthGain = data.ShadowHealthGain
	end
	if data.ShadowHealthTime then
	    inst.ShadowHealthTime = data.ShadowHealthTime
	end 
	if data.syntheticobserver then 
	    inst.syntheticobserver = true 
	end 
	if data.supersyntheticobserver then 
	    inst.supersyntheticobserver = true 
	end 
	if data.shadowmanipulation then
	    inst.shadowmanipulation = true 
	end
	if data.supershadowmanipulation then
	    inst.supershadowmanipulation = true 
	end	
	if data.dropshadowfueltimeleft then 
	    inst.dropshadowfuelcooldown = true 
        if inst.dropshadowfueltask then
	        inst.dropshadowfueltask:Cancel()
		    inst.dropshadowfueltask = nil 
	    end 
	    if inst.dropshadowfueltaskinfo then
	        inst.dropshadowfueltaskinfo = nil 
     	end
	    inst.dropshadowfueltask, inst.dropshadowfueltaskinfo = inst:ResumeTask(data.dropshadowfueltimeleft, ResetDropShadowFuel)
	end	
	if data.shadowpuddletimeleft then 
	    inst.spawnshadowpuddlecooldown = true 
        if inst.shadowpuddlespawntask then
	        inst.shadowpuddlespawntask:Cancel()
		    inst.shadowpuddlespawntask = nil 
	    end 
	    if inst.shadowpuddlespawntaskinfo then
	        inst.shadowpuddlespawntaskinfo = nil 
     	end
	    inst.shadowpuddlespawntask, inst.shadowpuddlespawntaskinfo = inst:ResumeTask(data.shadowpuddletimeleft, ResetSapwnShadowPuddle)
	end		
	if data.spawntaskcoffeetimeleft then
        if inst.taskcoffeespawntask then
	        inst.taskcoffeespawntask:Cancel()
		    inst.taskcoffeespawntask = nil 
	    end 
	    if inst.taskcoffeespawntaskinfo then
	        inst.taskcoffeespawntaskinfo = nil 
     	end
	    inst.taskcoffeespawntask, inst.taskcoffeespawntaskinfo = inst:ResumeTask(data.spawntaskcoffeetimeleft, inst.SpawnTaskCoffee)	
	end
	if data.PatchedDragonHouseRecipe then 
	    inst.PatchedDragonHouseRecipe = true
	end
	if data.shadowacider then 
	    inst.shadowacider = true
	end
	if data.supershadowacider then 
	    inst.supershadowacider = true
	end
	if data.acidshadowsmoker then 
	    inst.acidshadowsmoker = true
	end
	if data.superacidshadowsmoker then 
	    inst.superacidshadowsmoker = true
	end
	if data.FirstTimeValentineDayNote then
	    inst.FirstTimeValentineDayNote = true
	end
	if data.FirstTimeTaskCoffeeSpawn then
	    inst.FirstTimeTaskCoffeeSpawn = true
	end
end

local function UpdateCharlieLocomotor(inst)
    inst.hasspeedbuffonhittingother = nil
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    if inst:HasTag("shadow_form_on") then 
            if inst.components.locomotor ~= nil then 
                inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * TUNING.SHADOW_CHARLIE_WALK_SPEED_MULT_VANILLA  
                inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * TUNING.SHADOW_CHARLIE_RUN_SPEED_MULT_VANILLA   
            end		
		else
	        if inst.components.locomotor ~= nil then 
                inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * TUNING.CHARLIE_WALK_SPEED_MULT_VANILLA   
	            inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * TUNING.CHARLIE_RUN_SPEED_MULT_VANILLA	    		
            end			
		end
	else
	    if inst:HasTag("shadow_form_on") then 
            if inst.components.locomotor ~= nil then 
                inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * TUNING.CHARLIE_SHADOW_WALK_SPEED_MULT_DS 
	            inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * TUNING.CHARLIE_SHADOW_RUN_SPEED_MULT_DS    
            end		
		else 
	        if inst.components.locomotor ~= nil then 
                inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * TUNING.CHARLIE_WALK_SPEED_MULT_DS   
	            inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * TUNING.CHARLIE_RUN_SPEED_MULT_DS    		
            end			
		end	
	end
end

local function FixIronLordDisplay(inst)
	inst.AnimState:OverrideSymbol("torso", "living_suit_build", "torso")	
	inst.AnimState:OverrideSymbol("torso_pelvis", "living_suit_build", "torso_pelvis")
end

local function FixSkinDisplayAndSpeed(inst)
	local GetSkinName = (inst.components.dsskins ~= nil and inst.components.dsskins.skin) or nil
	if GetSkinName ~= nil then
	    inst.AnimState:SetBuild(GetSkinName)
	    inst.AnimState:OverrideSymbol("torso", GetSkinName, "torso")	
	    inst.AnimState:OverrideSymbol("torso_pelvis", GetSkinName, "torso_pelvis")
	end 
	UpdateCharlieLocomotor(inst)
end

local function SetSkinForFirstTime(inst)
	if not inst:HasTag("first_time_set_done_skin")
	    and inst.components.dsskins ~= nil 
	    and inst.AnimState ~= nil 
	    and inst.shadowform == nil 
		and (inst.components.transformable == nil or not inst.components.transformable.shadowform)
	then
	    local GetSkinName = TUNING.CHARLIE_STARTING_SKIN		
	    if GetSkinName ~= nil then
	        inst.AnimState:SetBuild(GetSkinName)
	        inst.AnimState:OverrideSymbol("torso", GetSkinName, "torso")	
	        inst.AnimState:OverrideSymbol("torso_pelvis", GetSkinName, "torso_pelvis")
	        inst.components.dsskins.skin = GetSkinName
        end
	    inst:AddTag("first_time_set_done_skin")
	end
end

local function StartingWithPortal(inst)
    if TUNING.STARTING_WITH_PORTAL == true	  
		and not inst:HasTag("first_time_set_done_portal")
	then
	    if not TheSim:FindFirstEntityWithTag("charlie_portal") then
	        SpawnPrefab("charlie_portal").Transform:SetPosition(inst.Transform:GetWorldPosition())
		end
	    inst:AddTag("first_time_set_done_portal")
	end
end

local function StartingSWworldWithMachete(inst)
    if TUNING.STARTING_WITH_MACHETE == true
	    and not inst:HasTag("first_time_set_done_mechete")
	    and GetWorld()
		and GetWorld():HasTag("shipwrecked")	
	then
        local pt = inst:GetPosition()
        local offset = FindGroundOffset(pt, math.random()*2*math.pi, 3+math.random()*5, 10, true, false)
        if offset then	    
		    local spawnpt = pt + offset
		    SpawnPrefab("machete").Transform:SetPosition(spawnpt.x, 0, spawnpt.z)
		end
		inst:AddTag("first_time_set_done_mechete")
	end
end

local function GetMultFromFood_HUNGER(inst, food, original_value)
	return food ~= nil
	    and food:HasTag("smallfood_for_charlie") 
        and not inst.inst:HasTag("shadow_form_on") --inst return inst.components.eater, not inst.
		and TUNING.CHARLIE_HUNGER_MULT_FOR_SMALLFOOD 
		or 1
end

local function GetMultFromFood_HEALTH(inst, food, original_value)
	return food ~= nil
	    and food:HasTag("food_undebuff_shadow") 
	    and inst.inst:HasTag("shadow_form_on")    --inst return inst.components.eater, not inst.
		and original_value ~= nil
		and original_value < 0
		and 0
	    or 1
end

--Following is written in SetAbsorptionModifiers instead:
--[[
local function GetMultFromFood_SANITY(inst, food, original_value)
	return ( inst.inst.shadowform ~= nil or 
	            (inst.inst.components.transformable ~= nil and inst.inst.components.transformable.shadowform) 
		    ) 
			and 0
	    or 1
end
]]

local function IsAboveGroundAtPoint(x, y, z) --Just a random name
    --local tile = GetWorld().Map:GetTileAtPoint(x, y, z)
	local radius = 3
	
    if GetPlayer().GetIsOnLandOutside ~= nil
	    and not GetPlayer():GetIsOnLandOutside(x, y, z) 
	then
	    return false
	end
	
	if GetPlayer().IsPosSurroundedByLand ~= nil 
	    and not GetPlayer():IsPosSurroundedByLand(x, y, z, 5)
	then
	    return false
	end
	
    for i = -radius, radius, 1 do
        if GetWorld().Map:GetTileAtPoint(x - radius, y, z + i) ~= GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(x + radius, y, z + i) ~= GROUND.IMPASSABLE then
            return true
        end
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if GetWorld().Map:GetTileAtPoint(x + i, y, z - radius) ~= GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(x + i, y, z + radius) ~= GROUND.IMPASSABLE then
            return true
        end
    end	
	
	return false
end

local function IsPointNearPlayer(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, {"player"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearSth(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"sunken_boat", "deflated_balloon_basket", "ruins_exit", "pugalisk_fountain"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function PatchCharlieStage(_)
    if TheSim:FindFirstEntityWithTag("charlie_stage") then
	    return
	end
    print("Patch Charlie Stage Start")
    if SaveGameIndex:GetCurrentMode() ~= "adventure" then
        if GetWorld().prefab == "forest" or GetWorld().prefab == "shipwrecked" or GetWorld().prefab == "porkland" then
            local charlie_stage = nil
            for i, node in ipairs( GetWorld().topology.nodes ) do
                if node.type ~= NODE_TYPE.Blank and node.type ~= NODE_TYPE.Blocker then
                    local x = node.cent[1]
                    local z = node.cent[2]
                    local pos = Vector3(x, 0, z)
                    local tilepos = Vector3( GetWorld().Map:GetTileCenterPoint(pos.x, pos.y, pos.z) )
                    if IsAboveGroundAtPoint(tilepos.x, tilepos.y, tilepos.z) 
					    and not IsPointNearPlayer(tilepos, 300) 
					    and not IsPointNearSth(tilepos, 30) 
					    and not (RoadManager ~= nil and RoadManager:IsOnRoad(tilepos.x, 0, tilepos.z))
					then                                            
                        charlie_stage = SpawnPrefab("charlie_stage_post")
                        charlie_stage.Transform:SetPosition(tilepos:Get())
                        print("Spawned the charlie_stage.")
                    end
                end
                if charlie_stage then
                    break
                end
            end		
			if not charlie_stage then
			    print("UH OH! We couldn't find a spot in the world for the charlie_stage!")
			end
		end
    else
        print("it is adventure, no charlie stage")
    end
end

local function OnExitInsightState(inst)
    if inst.IsInInsight then
	    inst.IsInInsight = nil
	end
	inst.Light:Enable(false)
	if inst.components.transformable then
	    inst.components.transformable:ExitInsightState()
	end	
	if inst.insighttask then 
		inst.insighttask:Cancel() 
		inst.insighttask = nil  
	end	
	if inst.insighttaskinfo then
	    inst.insighttaskinfo = nil 
	end	
end

local function SetExitInsightStateTask(inst, time)
	if inst.insighttask then 
		inst.insighttask:Cancel() 
		inst.insighttask = nil  
	end
	if inst.insighttaskinfo then
	    inst.insighttaskinfo = nil 
	end
	if not time then 
		time = TUNING.TOTAL_DAY_TIME*10
	end
	inst.insighttask, inst.insighttaskinfo = inst:ResumeTask(time, OnExitInsightState)  
end

local function Transformable_EnterInsightState(inst)
    inst:SetExitInsightStateTask()
end

local function EnterInsightState(inst)
    inst.Light:Enable(true)
	Transformable_EnterInsightState(inst)
	if inst.shadowform ~= nil then
	    BecomeNormalForm(inst)
	end
end

local function CheckIfCanUnlockThrone(inst)
    local world = GetWorld()
	local find_cave_regenerator = false
	if inst.components.inventory ~= nil then
	    find_cave_regenerator = inst.components.inventory:FindItem(
        function(item)
            return item.prefab == "cave_regenerator"
        end)
	end
	if world ~= nil and find_cave_regenerator then
	    world:PushEvent("unlockcharliethrone")
	end
end

local function OnLongUpdate(inst, dt)
    if inst.insighttask and inst.insighttaskinfo then
        local new_time = math.max(inst:TimeRemainingInTask(inst.insighttaskinfo) - dt, 0)

        inst.insighttask:Cancel()
        inst.insighttask = nil 
        inst.insighttaskinfo = nil

        inst.insighttask, inst.insighttaskinfo = inst:ResumeTask(new_time, OnExitInsightState)
    end
    if inst.dropshadowfueltask and inst.dropshadowfueltaskinfo then
        local new_time2 = math.max(inst:TimeRemainingInTask(inst.dropshadowfueltaskinfo) - dt, 0)

        inst.dropshadowfueltask:Cancel()
        inst.dropshadowfueltask = nil 
        inst.dropshadowfueltaskinfo = nil

        inst.dropshadowfueltask, inst.dropshadowfueltaskinfo = inst:ResumeTask(new_time2, ResetDropShadowFuel)
    end	
    if inst.shadowpuddlespawntask and inst.shadowpuddlespawntaskinfo then
        local new_time3 = math.max(inst:TimeRemainingInTask(inst.shadowpuddlespawntaskinfo) - dt, 0)

        inst.shadowpuddlespawntask:Cancel()
        inst.shadowpuddlespawntask = nil 
        inst.shadowpuddlespawntaskinfo = nil

        inst.shadowpuddlespawntask, inst.shadowpuddlespawntaskinfo = inst:ResumeTask(new_time3, ResetSapwnShadowPuddle)
    end		
    if inst.taskcoffeespawntask and inst.taskcoffeespawntaskinfo then
        local new_time4 = math.max(inst:TimeRemainingInTask(inst.taskcoffeespawntaskinfo) - dt, 0)

        inst.taskcoffeespawntask:Cancel()
        inst.taskcoffeespawntask = nil 
        inst.taskcoffeespawntaskinfo = nil

        inst.taskcoffeespawntask, inst.taskcoffeespawntaskinfo = inst:ResumeTask(new_time4, inst.SpawnTaskCoffee)
    end	
end

local function GetTeleportPoint()
	local pt = nil
	local rotation = GetPlayer().Transform:GetRotation()*DEGREES
	local pos = GetPlayer():GetPosition()
	for r = 30, 1, -1 do
        local numtries = 2*math.pi*r

		pt = FindWalkableOffset(pos, rotation, r, numtries)
		
        if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
            if GetPlayer():GetIsOnWater() or GetPlayer():HasTag("aquatic") then
        	    pt = FindWaterOffset(pos, rotation, r, numtries)
            else
			    pt = FindGroundOffset(pos, rotation, r, numtries)
            end
		end

		if pt then
			return pt + pos
		end
	end	
end

local function ValidateFn(inst)
    return inst:HasTag("shadow_form_on")
end

local function SetDamageMult(inst, v)
    if v ~= nil then
	    inst.damage_mult = v
	end
end

local function SetDamageMultShadow(inst, v)
    if v ~= nil then
	    inst.damage_mult_shadow = v
	end
end

local function SetUnarmedDamageShadow(inst, v)
    if v ~= nil then
	    inst.unarmed_damage_shadow = v
	end
end

local function CharlieOnHitOther(inst, target, damage, stimuli)
    if inst ~= nil then
	    if inst.swiftwarrior 
		    and (inst.superswiftwarrior or math.random() < 0.5)
		    and inst.components.locomotor ~= nil 
		    and target ~= nil 
			and target:IsValid() 
			and target.entity:IsVisible() 
			and IsEntValidToDropExp(target) 
            and inst.hasspeedbuffonhittingother == nil  --This is also checked in other places
		then		
		    inst.hasspeedbuffonhittingother = true
		    inst.components.locomotor.walkspeed = inst.components.locomotor.walkspeed + (inst.superswiftwarrior and 3 or 2) 
			inst.components.locomotor.runspeed = inst.components.locomotor.runspeed + (inst.superswiftwarrior and 3 or 2) 
			inst:DoTaskInTime(2, UpdateCharlieLocomotor)
		end
		
		local weapon = inst.components.inventory ~= nil and inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) or nil 
        if inst.thermologist then  
		    if weapon ~= nil and weapon.prefab == "torch" then
			    if target ~= nil 
				    and target.components.burnable ~= nil 
					and math.random() < 0.1*target.components.burnable.flammability 
				then  --bonus chance to ignite target
				    target.components.burnable:Ignite()
				end
			elseif weapon ~= nil and weapon.prefab == "icestaff" then
			    if target ~= nil and target.components.freezable ~= nil then  --bonus coldness
				    target.components.freezable:AddColdness(0.5) 
				end						
			end
		end 
		
		if inst.superthermologist and target ~= nil and IsEntValidToDropExp(target) then
		    if target ~= nil 
			    and target.components.burnable ~= nil 
				and target.components.health ~= nil  
				and math.random() < 0.1*target.components.burnable.flammability 
			then
			    target.components.health:DoFireDamage(20, inst, true)
			end
		    if target ~= nil and math.random() < 0.2 and target.components.freezable ~= nil then
			    target.components.freezable:AddColdness(1) 
			end		
            if math.random() < 0.05 then 
		        local POSSIBLE_ITEMS_TO_GET = 
	         	{
		            "torch", 
		        	"icestaff",   
		         	"blueamulet", 
		        	"firestaff", 
		        	"bluegem", 
		         	"redgem", 
		        	"heatrock", 
	        	}
				if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
				    table.insert(POSSIBLE_ITEMS_TO_GET, "ice")
				end 
                if inst.components.inventory ~= nil and target ~= nil then
		            inst.components.inventory:GiveItem(SpawnPrefab(GetRandomItem(POSSIBLE_ITEMS_TO_GET)), nil, Vector3(TheSim:GetScreenPos(target.Transform:GetWorldPosition()))) 
				else
				    SpawnPrefab("small_puff").Transform:SetPosition(inst.Transform:GetWorldPosition())
				    SpawnPrefab(GetRandomItem(POSSIBLE_ITEMS_TO_GET)).Transform:SetPosition(inst.Transform:GetWorldPosition())
		        end					
            end			
		end
		
		if inst:HasTag("shadow_form_on") 
		    and inst.syntheticobserver 
			and target ~= nil 
			and target:HasTag("shadow") 
			and inst.components.health ~= nil 
			and math.random() < (inst.supersyntheticobserver and 0.4 or 0.25)
		then
		    inst.components.health:DoDelta(inst.supersyntheticobserver and 30 or 20)  
		end 
		
		if inst:HasTag("shadow_form_on") 
		    and inst.shadowmanipulation 
			and target ~= nil 
			and target:HasTag("shadow") 
			and (inst.supershadowmanipulation or math.random() < 0.5)
		then 
		    if math.random() < 0.1 then
                if inst.components.inventory ~= nil then
		            inst.components.inventory:GiveItem(SpawnPrefab("horrorfuel"), nil, Vector3(TheSim:GetScreenPos(target.Transform:GetWorldPosition()))) 
				else
				    SpawnPrefab("small_puff").Transform:SetPosition(inst.Transform:GetWorldPosition())
				    SpawnPrefab("horrorfuel").Transform:SetPosition(inst.Transform:GetWorldPosition())
		        end				
			else
                if inst.components.inventory ~= nil then
		            inst.components.inventory:GiveItem(SpawnPrefab("nightmarefuel"), nil, Vector3(TheSim:GetScreenPos(target.Transform:GetWorldPosition()))) 
				else
				    SpawnPrefab("small_puff").Transform:SetPosition(inst.Transform:GetWorldPosition())
				    SpawnPrefab("nightmarefuel").Transform:SetPosition(inst.Transform:GetWorldPosition())
		        end				
			end
		end
		
		if inst:HasTag("shadow_form_on") 
		    and inst.superacidshadowsmoker 
			and target ~= nil 
			and IsEntValidToDropExp(target) 
			and math.random() < 0.1  
		then
		    local environment_to_spawn = math.random() < 0.5 and "shadowacidpuddle" or "shadow_acid_smoke"
		    local x, y, z = target.Transform:GetWorldPosition() 
			local find_puddle_or_smoke = TheSim:FindEntities(x, y, z, 3, nil, nil, {"puddle", "butterpuddle", "shadow_acid_smoke"}) 
			local find_base = TheSim:FindEntities(x, y, z, 10, nil, nil, {"wall", "structure"}) 

			if #find_puddle_or_smoke <= 0 
			    and #find_base <= 0 
			    and not _IsPointCloseToImpassable(x, y, z, 3)
				and (inst.IsPosSurroundedByLand == nil or inst:IsPosSurroundedByLand(x, y, z, 7))
			then
			    if environment_to_spawn == "shadowacidpuddle" then
			        local shadowacidpuddle = SpawnPrefab(environment_to_spawn)
			        shadowacidpuddle.Transform:SetPosition(x, y, z) 
				    if shadowacidpuddle.StartingAnimation ~= nil and shadowacidpuddle.LoopingAnimation ~= nil then 
				        shadowacidpuddle.AnimState:PlayAnimation(shadowacidpuddle.StartingAnimation)
					    shadowacidpuddle.AnimState:PushAnimation(shadowacidpuddle.LoopingAnimation, true)
				    end 
					if shadowacidpuddle.SetLifeTime ~= nil then
					    shadowacidpuddle:SetLifeTime(480)
					end						
					if math.random() < 0.1 then
					    shadowacidpuddle.superacid = true 
					end				
                else
			        local shadow_acid_smoke = SpawnPrefab(environment_to_spawn)
			        shadow_acid_smoke.Transform:SetPosition(x, y, z) 
					if shadow_acid_smoke.SetLifeTime ~= nil then
					    shadow_acid_smoke:SetLifeTime(480)
					end						
                end			
			end
		end
	
	end
end

local function CharlieGetAttacked(self, attacker, damage, weapon, stimuli)
    if self ~= nil 
	    and self.inst ~= nil 
		and self.inst.masterdodging 
		and self.inst.successfullydodge == nil 
	then
        self.inst.successfullydodge = true 
	end
    if self ~= nil 
	    and self.inst ~= nil 
		and self.inst.dodgemaster 
        and math.random() < ((self.inst.superdodgemaster and (0.35 + (self.inst.successfullydodge and 0.15 or 0))) or 0.25)
	then
	    if self.inst.masterdodging == nil then
	        self.inst.successfullydodge = nil 
		end
        return false
    elseif self ~= nil and self._CharlieGetAttacked ~= nil then 
	    if self.inst ~= nil and self.inst.masterdodging == nil then
		    self.inst.successfullydodge = nil
		end
	    return self._CharlieGetAttacked(self, attacker, damage, weapon, stimuli)
	end
end 

local function CharlieValentineDayNote(inst)
    local x, y, z = inst.Transform:GetWorldPosition()
    local nearby_valentine_day_rose = TheSim:FindEntities(x, y, z, 10, {"charlie_rose_valentine_day"}) 
    if #nearby_valentine_day_rose == 0 and inst.components.talker ~= nil then
	    inst.components.talker:Say(STRINGS.CHARLIE_VALENTINE_DAY_NOTE or "")
	end
	inst.FirstTimeValentineDayNote = true
end

local function PatchValentineDayRose(inst)
    if not GetCharlieTaskRose() then
        if GetWorld() ~= nil and GetWorld().Map ~= nil and GetWorld().topology ~= nil and GetWorld().topology.nodes ~= nil then
            local charlie_rose_valentine_day = nil
            for i, node in ipairs(GetWorld().topology.nodes) do
                if node.type ~= NODE_TYPE.Blank and node.type ~= NODE_TYPE.Blocker then
                    local x = node.cent[1]
                    local z = node.cent[2]
                    local pos = Vector3(x, 0, z)
                    local tilepos = Vector3(GetWorld().Map:GetTileCenterPoint(pos.x, pos.y, pos.z))
					local nearby_stage = TheSim:FindEntities(tilepos.x, tilepos.y, tilepos.z, 30, {"charlie_stage"}) 
					local nearby_blocker = TheSim:FindEntities(tilepos.x, tilepos.y, tilepos.z, 3, nil, {"FX"}) 
					local tile = GetWorld().Map:GetTileAtPoint(tilepos.x, tilepos.y, tilepos.z)
					local is_plantable_tile = false
                    if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and tile ~= GROUND.ROCKY and tile ~= GROUND.ROAD
		                and tile ~= GROUND.UNDERROCK and tile ~= GROUND.WOODFLOOR and tile ~= GROUND.CARPET and tile ~= GROUND.CHECKER
		                and tile < GROUND.UNDERGROUND
	                then
		                is_plantable_tile = true
	                end
                    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	                    if tile == GROUND.ASH or tile == GROUND.VOLCANO 
		                    or tile == GROUND.VOLCANO_ROCK or tile == GROUND.BRICK_GLOW
			                or tile == GROUND.VOLCANO_LAVA or tile == GROUND.SNAKESKIN
		                then
		                    is_plantable_tile = false
		                end
	                end					
	                if IsDLCEnabled(PORKLAND_DLC) then
	                    if tile == GROUND.FOUNDATION or tile == GROUND.COBBLEROAD
		                    or tile == GROUND.GASJUNGLE or tile == GROUND.PIGRUINS
			                or tile == GROUND.PIGRUINS_NOCANOPY or tile == GROUND.BEARDRUG or tile == GROUND.INTERIOR
		                then                                                               
		                    is_plantable_tile = false
		                end
	                end							
                    if IsAboveGroundAtPoint(tilepos.x, tilepos.y, tilepos.z) 
					    and not IsPointNearPlayer(tilepos, 300) 
					    and not IsPointNearSth(tilepos, 30) 
						and #nearby_stage == 0
						and #nearby_blocker == 0
						and not IsPointCloseToLava(tilepos, 5)
						and is_plantable_tile
					    and not (RoadManager ~= nil and RoadManager:IsOnRoad(tilepos.x, 0, tilepos.z))
			        then                                            
                        charlie_rose_valentine_day = SpawnPrefab("charlie_rose_2024")
                        charlie_rose_valentine_day.Transform:SetPosition(tilepos:Get())
                    end
                end
                if charlie_rose_valentine_day ~= nil then
                    break
                end
            end		
        end	
	end
	if not TheSim:FindFirstEntityWithTag("valentine_day_fishing_rod") then
	
	    local centers = {}
	    if GetWorld() and GetWorld().topology and GetWorld().topology.nodes and GetWorld().Map then
            for i, node in pairs(GetWorld().topology.nodes) do
			    if node ~= nil and node.x ~= nil and node.y ~= nil then
		            local tile = GetWorld().Map:GetTileAtPoint(node.x, 0, node.y)
		            local nearby_player = TheSim:FindEntities(node.x, 0, node.y, 200, {"player"}) 
				    local blockers = TheSim:FindEntities(node.x, 0, node.y, 2, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER", "isinventoryitem"})
			    	local nearby_boat = TheSim:FindEntities(node.x, 0, node.y, 100, {"sunken_boat"})
			        local NotInInterior = true
                    local IsImpassableTile = true
					local IsPassableTile = true
					if GetWorld().prefab == "forest" then
                        if tile ~= GROUND.IMPASSABLE then
		                    IsImpassableTile = false  --Fix its position later in other places
	                    end
					elseif tile == GROUND.IMPASSABLE or tile == GROUND.INVALID or tile >= GROUND.UNDERGROUND then
					    IsPassableTile = false
					end
			        if GetWorld().prefab ~= "forest" and inst.IsPosSurroundedByLand ~= nil and inst:IsPosSurroundedByLand(node.x, 0, node.y, 3) then
			            IsImpassableTile = false
		    	    end 					
			        if IsDLCEnabled(PORKLAND_DLC) then
				        if tile == GROUND.INTERIOR then
				            NotInInterior = false
				        end
		    	    end
                    if #nearby_player == 0 and #blockers == 0 and #nearby_boat == 0 and IsImpassableTile and IsPassableTile and NotInInterior then
                        table.insert(centers, {x = node.x, z = node.y})
                    end
				end
            end
	    end 
        if #centers > 0 then
		    local spawn_position = centers[math.random(#centers)]
            SpawnPrefab("valentine_day_fishing_rod").Transform:SetPosition(spawn_position.x, 0, spawn_position.z)
	    end
	
	end
end

local function Initialize(inst)
    SetSkinForFirstTime(inst)
    StartingWithPortal(inst)
    StartingSWworldWithMachete(inst)
    if inst:HasTag("lightsource") then       
        inst:RemoveTag("lightsource")    
    end		
    --Use tag so that it can be compatible with vanilla DS
    if inst:HasTag("burnable") then       
        inst:RemoveTag("burnable")    
    end
	if inst:HasTag("wildfirestarter") then       
        inst:RemoveTag("wildfirestarter")    
    end
	if not inst.PatchedDragonHouseRecipe then
	    if inst.components.builder ~= nil then
		    if inst:HasTag("supercritterexpert") then
		        inst.components.builder:UnlockRecipe("charlie_dragon_house")
			end
		    if inst.superthermologist then
		        inst.components.builder:UnlockRecipe("charlie_dragon_sword")
			end
		end 
	    inst.PatchedDragonHouseRecipe = true
	end 
	if os.time() >= 1709305537 and os.time() <= 1713196800 then
	    if not inst.FirstTimeValentineDayNote then
	        inst:DoTaskInTime(10, CharlieValentineDayNote)
		end
	elseif inst.FirstTimeValentineDayNote then
	    inst.FirstTimeValentineDayNote = nil
	end
	inst:DoTaskInTime(1.5, PatchValentineDayRose)
end

local fn = function(inst)
	inst.soundsname = "charlie"
	
	inst.MiniMapEntity:SetIcon("charlie.tex")

	inst:AddTag("charlie")
    inst:AddTag("grue_immune")
	inst:AddTag("flower_lover")
	inst:AddTag("sweetheart")
    inst:AddTag("dsskins")
	inst:AddTag("fear_of_darkness")
	inst:AddTag("strengthened_by_darkness")	
	inst:AddTag("vessel_of_Metheus")
	inst:AddTag("former_magician_assistant")
	inst:AddTag("prospective_throne_owner")
	inst:AddTag("transformable")
	inst:AddTag("hayfever_resistant")
	inst:AddTag("agile")

    inst.shadowform = nil    --Only for vanilla DS. For DLCs check 'transformable' component
	inst.IsShadowForm = inst.shadowform

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
       	inst:AddComponent("transformable")
    end

--Don't like this change, not seen well in non-Hamlet-compatible save, as well as OOC:
--[[	
	if inst.components.inventory ~= nil then
	    inst.components.inventory.maxslots = TUNING.CHARLIE_STARTING_SLOTS
	end
]]

	inst:AddComponent("dsskins")	
	inst:AddComponent("timer")

    inst:AddComponent("shadowreader")

	inst:AddComponent("magician")
	inst.components.magician:SetCommonRequiredTag("tricks_player")
	inst.components.magician:SetMakeDisappearTag("rabbit")
	inst.components.magician:SetEnchantRequiredTag("former_magician_assistant")
	inst.components.magician:SetEnchantTag("tophat")
	inst.components.magician:SetEnchantExcludeTag("magician_hat")
	inst.components.magician:SetMagicRabbitHoleCheckRadius(30)
	inst.components.magician:SetMagicRabbitCheckRadius(20)
	inst.components.magician:SetMagicFlowerCheckRadius(25)
	inst.components.magician:SetMagicPetalsCheckRadius(25)
	inst.components.magician:SetMakeDisappearHungerCost(10)
	inst.components.magician:SetMakeDisappearRabbitHoleSpawnChance(0.1)
	inst.components.magician:SetMakeDisappearGoldSpawnChance(0.25)
	inst.components.magician:SetMagicTricksSuccessChance(0.25)
	inst.components.magician:SetMagicTricksSuccessSanityGain(5)
	inst.components.magician:SetMagicTricksFailSanityLose(5)
	inst.components.magician:SetConjureFlowersSanityCost(10)
	inst.components.magician:SetConjureFlowersGrogginess(5)
	inst.components.magician:SetConjureFlowersChance(0.3)
	inst.components.magician:SetEnchantSanityCost(15)

	inst:AddComponent("charlierefreshmodlevel")
	inst.components.charlierefreshmodlevel:SetLevelData(charlie_level_data)
	inst.components.charlierefreshmodlevel:EnableMastery(true)
	inst.components.charlierefreshmodlevel:SetRequiredConsumables("charlie_exp") 

    inst:AddComponent("autoshadowabsorber")	
	
    inst.entity:AddLight()
	if not inst.IsInInsight then
        inst.Light:Enable(false)
	end
    inst.Light:SetRadius(9999)
    inst.Light:SetFalloff(0.5)   
    inst.Light:SetIntensity(0.6)   
    inst.Light:SetColour(245/255, 40/255, 0/255)   

    inst.SetExitInsightStateTask = SetExitInsightStateTask 
    inst.OnLongUpdate = OnLongUpdate 
	inst.SetDamageMult = SetDamageMult
	inst.SetDamageMultShadow = SetDamageMultShadow
	inst.SetUnarmedDamageShadow = SetUnarmedDamageShadow
	inst.DropShadowFuelAsShadowManipulation = DropShadowFuelAsShadowManipulation 
	inst.ResetDropShadowFuel = ResetDropShadowFuel 
	inst.CheckPeriodicShadowPuddleSpawn = CheckPeriodicShadowPuddleSpawn
	inst.ResetSapwnShadowPuddle = ResetSapwnShadowPuddle
	inst.UpdateCharlieLocomotor = UpdateCharlieLocomotor
	
	inst.SanityGainFromPlantingFlower = 10
	inst.ShadowHealthGain = 12
	inst.ShadowHealthTime = 20

    if not (IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)) then
       	inst.components.combat.damagemultiplier = TUNING.CHARLIE_DAMAGE_MULT_DAY_VANILLA
	    inst.components.sanity.night_drain_mult = TUNING.CHARLIE_NIGHT_SANITY_MULT_DAY_VANILLA
	    inst.components.sanity.neg_aura_mult = TUNING.CHARLIE_NEG_SANITY_MULT_DAY_VANILLA
        inst.components.health.absorb = TUNING.CHARLIE_PROTECTION_DAY_VANILLA
	    inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * TUNING.CHARLIE_WALK_SPEED_MULT_DS
	    inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * TUNING.CHARLIE_RUN_SPEED_MULT_DS
	    inst:DoTaskInTime(0, function() UpdateVariableValues(inst) end)	
	    inst:ListenForEvent("dusktime", function() UpdateVariableValues(inst) end, GetWorld())
        inst:ListenForEvent("daytime", function() UpdateVariableValues(inst) end, GetWorld())
        inst:ListenForEvent("nighttime", function() UpdateVariableValues(inst) end, GetWorld()) 
		
		inst:AddComponent("formchecker")
		
	    inst.BecomeNormalForm = BecomeNormalForm
	    inst.BecomeShadowForm = BecomeShadowForm
	    inst.UpdateVariableValues = UpdateVariableValues
		inst.EnterInsightState = EnterInsightState
	else
	    inst.components.locomotor.walkspeed = TUNING.WILSON_WALK_SPEED * TUNING.CHARLIE_WALK_SPEED_MULT_VANILLA
	    inst.components.locomotor.runspeed = TUNING.WILSON_RUN_SPEED * TUNING.CHARLIE_RUN_SPEED_MULT_VANILLA	
    end

    if inst.components.talker ~= nil then
        inst.components.talker.colour = Vector3(255/255, 160/255, 203/255, 1)
		if TUNING.CHARLIE_GRUE_VOICE == true then
		    inst.components.talker.ontalk = OnTalk
		end
	end

    if inst.components.health ~= nil then
        inst.components.health:SetMaxHealth(TUNING.NORMAL_CHARLIE_HEALTH)  
    end

	if inst.components.sanity ~= nil then
	    inst.components.sanity:SetMax(TUNING.CHARLIE_SANITY)
		inst.components.sanity.custom_rate_fn = VariableSanityFn
    end

    if inst.components.hunger ~= nil then
        inst.components.hunger:SetMax(TUNING.CHARLIE_HUNGER)
        inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE*TUNING.CHARLIE_HUNGER_RATE_MULT_VANILLA)
	end

    MakeMediumBurnableCharacter(inst, "torso")
    inst.components.burnable:SetBurnTime(TUNING.CHARLIE_BURN_TIME)

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then     
		inst.components.burnable.lightningimmune = true
		inst.components.burnable.no_wildfire = true
		
		inst:ListenForEvent("Transformable_EnterInsightState", Transformable_EnterInsightState)
    end

    if inst.components.combat ~= nil then
	    inst.components.combat:SetAttackPeriod(TUNING.WILSON_ATTACK_PERIOD*0.85)
		inst.components.combat.onhitotherfn = CharlieOnHitOther 
		inst.components.combat._CharlieGetAttacked = inst.components.combat.GetAttacked
		inst.components.combat.GetAttacked = CharlieGetAttacked
	end
	
    if inst.components.builder ~= nil then
       	inst.components.builder.magic_bonus = TUNING.CHARLIE_MAGIC_BONUS
    	inst.components.builder.ancient_bonus = TUNING.CHARLIE_ANCIENT_BONUS
	end
 
 	if inst.components.grue ~= nil then 
		inst.components.grue.Start = function() end 
	end
	
	if inst.components.playeractionpicker ~= nil then
        inst.components.playeractionpicker.pointspecialactionsfn = GetPointSpecialActions
	end
	
	--For Controller Use:
    inst:AddComponent("reticule")
    inst.components.reticule.targetfn = GetTeleportPoint
	inst.components.reticule.validatefn = ValidateFn
    inst.components.reticule.ease = true

    if inst.components.eater ~= nil then --TO DO: VANILLA AND ROG NEED TO CHECK FOLLOWING FUNCTIONS
	    inst.components.eater.gethungermultfn = GetMultFromFood_HUNGER
	    inst.components.eater.gethealthmultfn = GetMultFromFood_HEALTH
	    --inst.components.eater.getsanitymultfn = GetMultFromFood_SANITY  --written in SetAbsorptionModifiers instead
    end
		
	if IsDLCEnabled(PORKLAND_DLC) then
		inst:ListenForEvent("livingartifactoveron", FixIronLordDisplay)
		inst:ListenForEvent("livingartifactoveroff", FixSkinDisplayAndSpeed)
    end	

    inst:ListenForEvent("entity_death", function(world, data) OnKill(inst, data) end, GetWorld())
	inst:ListenForEvent("killed", function(inst, data) OnKillinInv(inst, data) end)
    inst:ListenForEvent("enterdark", function(inst, data) EnterDark(inst, data) end)
    inst:ListenForEvent("enterlight", function(inst, data) EnterLight(inst, data) end)
    inst:ListenForEvent("oneatsomething", OnEatSomething)
    inst:ListenForEvent("death", OnDeath)
	inst:ListenForEvent("attacked", OnAttacked)
	inst:ListenForEvent("timerdone", OnTimerDone)
	inst:ListenForEvent("blocked", OnBlocked)

    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:ListenForEvent("arrive", function() CheckIfCanUnlockThrone(inst) end, GetWorld())  --Sadly only SW and HAM having this
	else
	    inst:DoTaskInTime(1, CheckIfCanUnlockThrone)
	end
	
    --Adjusted to new components:
    inst:AddComponent("readyformagicchecker")
	inst:AddComponent("absorblightchecker")
	inst:AddComponent("normalformchecker")
	inst:AddComponent("physicsthingsshadowdestroyer")
	
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
		inst:ListenForEvent("boatattacked", OnBoatAttacked)
	end

--------------------------------------------------------------------------------------------------

	if TUNING.CHARLIE_HIDE_HATS == true then
		inst:ListenForEvent("equip", function()		
		    inst.AnimState:ClearOverrideSymbol("swap_hat")		
		    inst.AnimState:Show("hair")		
		    inst.AnimState:Show("head")  
		end)
	end
	
---------------------------------------------------------------------------------------------------

	GetAllRecipes(true)
	
    local grass_umbrella_recipe = GetRecipe("grass_umbrella")
    if grass_umbrella_recipe then 
		grass_umbrella_recipe.level = TECH.NONE
    else
        Recipe("grass_umbrella", {Ingredient("twigs", 4) ,Ingredient("cutgrass", 3), Ingredient("petals", 6)}, RECIPETABS.SURVIVAL, TECH.NONE, RECIPE_GAME_TYPE.COMMON)
    end
	
	local tophat_recipe = GetRecipe("tophat")
	if tophat_recipe then
	    tophat_recipe.level = TECH.NONE
	else
	    Recipe("tophat", {Ingredient("silk", 6)}, RECIPETABS.DRESS, TECH.NONE, RECIPE_GAME_TYPE.COMMON)
	end
	
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    local magician_chest_recipe = Recipe("magician_chest", {Ingredient("silk", 1), Ingredient("boards", 4), Ingredient("nightmarefuel", 9)}, RECIPETABS.MAGIC, {MAGIC = 5}, RECIPE_GAME_TYPE.COMMON, "magician_chest_placer", 1.75)
	    magician_chest_recipe.atlas = "images/magician_chest.xml"
	    magician_chest_recipe.image = "magician_chest.tex"
		magician_chest_recipe.sortkey = 1.03
		
	    local portal_recipe = Recipe("charlie_portal", {Ingredient("rose_inv", 12, "images/inventoryimages/rose_inv.xml"), Ingredient("nightmarefuel", 10), Ingredient("cutstone", 8)}, RECIPETABS.MAGIC, {MAGIC = 6}, RECIPE_GAME_TYPE.COMMON, "charlie_portal_placer", 5)
	    portal_recipe.atlas = "images/florid_postern.xml"
	    portal_recipe.image = "florid_postern.tex"
		portal_recipe.sortkey = 1.04
		
		local shadowcorrosionmachine_recipe = Recipe("shadowcorrosionmachine", {Ingredient("cutstone", 4), Ingredient("redgem", 2), Ingredient("purplegem", 2)}, RECIPETABS.MAGIC, {MAGIC = 6}, RECIPE_GAME_TYPE.COMMON, "shadowcorrosionmachine_placer")
		shadowcorrosionmachine_recipe.atlas = "images/shadowcorrosionmachine.xml"
		shadowcorrosionmachine_recipe.image = "shadowcorrosionmachine.tex"
		shadowcorrosionmachine_recipe.sortkey = 1.05

		local statueharp_rose_recipe = Recipe("statueharp_rose", {Ingredient("marble", 3), Ingredient("rose_inv", 4), Ingredient("nightmarefuel", 6)}, RECIPETABS.MAGIC, {MAGIC = 6}, RECIPE_GAME_TYPE.COMMON, "statueharp_rose_placer")
		statueharp_rose_recipe.atlas = "images/harp_statue.xml"
		statueharp_rose_recipe.image = "harp_statue.tex"
		statueharp_rose_recipe.sortkey = 1.06

		local statuemaxwell_rose_recipe = Recipe("statuemaxwell_rose", {Ingredient("marble", 3), Ingredient("rose_inv", 4), Ingredient("nightmarefuel", 6)}, RECIPETABS.MAGIC, {MAGIC = 6}, RECIPE_GAME_TYPE.COMMON, "statuemaxwell_rose_placer")
		statuemaxwell_rose_recipe.atlas = "images/maxwell_statue.xml"
		statuemaxwell_rose_recipe.image = "maxwell_statue.tex"
		statuemaxwell_rose_recipe.sortkey = 1.07
		
		local magic_carrot_recipe = Recipe("magic_carrot", {Ingredient("rabbit", 1)}, RECIPETABS.MAGIC, TECH.LOST, RECIPE_GAME_TYPE.COMMON, nil, nil, nil, 3)
		magic_carrot_recipe.atlas = "images/inventoryimages.xml"
		magic_carrot_recipe.image = "carrot.tex"
		magic_carrot_recipe.product = "carrot"
		magic_carrot_recipe.sortkey = 1.08
		
		local magic_smallmeat_recipe = Recipe("magic_smallmeat", {Ingredient("rabbit", 1)}, RECIPETABS.MAGIC, TECH.LOST, RECIPE_GAME_TYPE.COMMON, nil, nil, nil, 2)
		magic_smallmeat_recipe.atlas = "images/inventoryimages.xml"
		magic_smallmeat_recipe.image = "smallmeat.tex"
		magic_smallmeat_recipe.product = "smallmeat"
		magic_smallmeat_recipe.sortkey = 1.09		
		
		local magic_honey_recipe = Recipe("magic_honey", {Ingredient("bee", 1)}, RECIPETABS.MAGIC, TECH.LOST, RECIPE_GAME_TYPE.COMMON, nil, nil, nil, 4)
		magic_honey_recipe.atlas = "images/inventoryimages.xml"
		magic_honey_recipe.image = "honey.tex"
		magic_honey_recipe.product = "honey"
		magic_honey_recipe.sortkey = 1.1 
		
		local magic_rabbithole_recipe = Recipe("magic_rabbithole", {Ingredient("rabbit", 1), Ingredient("shovel", 1), Ingredient("nightmarefuel", 4)}, RECIPETABS.MAGIC, TECH.LOST, RECIPE_GAME_TYPE.COMMON, "rabbithole_placer")
		magic_rabbithole_recipe.atlas = "images/inventoryimages.xml"
		magic_rabbithole_recipe.image = "rabbit.tex"
		magic_rabbithole_recipe.product = "rabbithole"
		magic_rabbithole_recipe.sortkey = 1.14		
			
    else
	    local magician_chest_recipe = Recipe("magician_chest", {Ingredient("silk", 1), Ingredient("boards", 4), Ingredient("nightmarefuel", 9)}, RECIPETABS.MAGIC, {MAGIC = 5}, "magician_chest_placer", 1.75)
	    magician_chest_recipe.atlas = "images/magician_chest.xml"
	    magician_chest_recipe.image = "magician_chest.tex"
		magician_chest_recipe.sortkey = 1.03
		
	    local portal_recipe = Recipe("charlie_portal", {Ingredient("rose_inv", 12), Ingredient("nightmarefuel", 10), Ingredient("cutstone", 8)}, RECIPETABS.MAGIC, {MAGIC = 6}, "charlie_portal_placer", 5)
	    portal_recipe.atlas = "images/florid_postern.xml"
	    portal_recipe.image = "florid_postern.tex"
		portal_recipe.sortkey = 1.04
		
	    local shadowcorrosionmachine_recipe = Recipe("shadowcorrosionmachine", {Ingredient("cutstone", 4), Ingredient("redgem", 2), Ingredient("purplegem", 2)}, RECIPETABS.MAGIC, {MAGIC = 6}, "shadowcorrosionmachine_placer")
	    shadowcorrosionmachine_recipe.atlas = "images/shadowcorrosionmachine.xml"
	    shadowcorrosionmachine_recipe.image = "shadowcorrosionmachine.tex"
		shadowcorrosionmachine_recipe.sortkey = 1.05

		local statueharp_rose_recipe = Recipe("statueharp_rose", {Ingredient("marble", 3), Ingredient("rose_inv", 4), Ingredient("nightmarefuel", 6)}, RECIPETABS.MAGIC, {MAGIC = 6}, "statueharp_rose_placer")
		statueharp_rose_recipe.atlas = "images/harp_statue.xml"
		statueharp_rose_recipe.image = "harp_statue.tex"
		statueharp_rose_recipe.sortkey = 1.06

		local statuemaxwell_rose_recipe = Recipe("statuemaxwell_rose", {Ingredient("marble", 3), Ingredient("rose_inv", 4), Ingredient("nightmarefuel", 6)}, RECIPETABS.MAGIC, {MAGIC = 6}, "statuemaxwell_rose_placer")
		statuemaxwell_rose_recipe.atlas = "images/maxwell_statue.xml"
		statuemaxwell_rose_recipe.image = "maxwell_statue.tex"
		statuemaxwell_rose_recipe.sortkey = 1.07

		local magic_carrot_recipe = Recipe("magic_carrot", {Ingredient("rabbit", 1)}, RECIPETABS.MAGIC, TECH.LOST, nil, nil, nil, 3)
		magic_carrot_recipe.atlas = "images/inventoryimages.xml"
		magic_carrot_recipe.image = "carrot.tex"
		magic_carrot_recipe.product = "carrot"
		magic_carrot_recipe.sortkey = 1.08

		local magic_smallmeat_recipe = Recipe("magic_smallmeat", {Ingredient("rabbit", 1)}, RECIPETABS.MAGIC, TECH.LOST, nil, nil, nil, 2)
		magic_smallmeat_recipe.atlas = "images/inventoryimages.xml"
		magic_smallmeat_recipe.image = "smallmeat.tex"
		magic_smallmeat_recipe.product = "smallmeat"
		magic_smallmeat_recipe.sortkey = 1.09

		local magic_honey_recipe = Recipe("magic_honey", {Ingredient("bee", 1)}, RECIPETABS.MAGIC, TECH.LOST, nil, nil, nil, 4)
		magic_honey_recipe.atlas = "images/inventoryimages.xml"
		magic_honey_recipe.image = "honey.tex"
		magic_honey_recipe.product = "honey"
		magic_honey_recipe.sortkey = 1.1
		
		local magic_rabbithole_recipe = Recipe("magic_rabbithole", {Ingredient("rabbit", 1), Ingredient("shovel", 1), Ingredient("nightmarefuel", 4)}, RECIPETABS.MAGIC, TECH.LOST, "rabbithole_placer")
		magic_rabbithole_recipe.atlas = "images/inventoryimages.xml"
		magic_rabbithole_recipe.image = "rabbit.tex"
		magic_rabbithole_recipe.product = "rabbithole"
		magic_rabbithole_recipe.sortkey = 1.14			

	end

	local inv_rose_recipe = Recipe("rose_inv", {Ingredient("petals", 3), Ingredient("stinger", 1), Ingredient("boneshard", 1)}, RECIPETABS.REFINE, TECH.LOST, RECIPE_GAME_TYPE.COMMON)

	local shadow_rose_bouquet_recipe = Recipe("shadow_rose_bouquet", {Ingredient("rose_inv", 7), Ingredient("nightmarefuel", 1)}, RECIPETABS.MAGIC, TECH.NONE, RECIPE_GAME_TYPE.COMMON)
	shadow_rose_bouquet_recipe.sortkey = 1.01	

	local shadow_torch_recipe = Recipe("torch_shadow", {Ingredient("torch", 1), Ingredient("nightmarefuel", 1)}, RECIPETABS.MAGIC, TECH.NONE, RECIPE_GAME_TYPE.COMMON)
	shadow_torch_recipe.sortkey = 1	

    local shadowbox_recipe = Recipe("charlie_shadowbox", {Ingredient("nightmarefuel", 20), Ingredient("charlie_exp", 40), Ingredient("purplegem", 2)}, RECIPETABS.MAGIC, {MAGIC = 5}, RECIPE_GAME_TYPE.COMMON)
	shadowbox_recipe.sortkey = 1.02

	local magic_honeycomb_recipe = Recipe("magic_honeycomb", {Ingredient("bee", 2)}, RECIPETABS.MAGIC, TECH.LOST)
	magic_honeycomb_recipe.atlas = "images/inventoryimages.xml"
	magic_honeycomb_recipe.image = "honeycomb.tex"
	magic_honeycomb_recipe.product = "honeycomb"
	magic_honeycomb_recipe.sortkey = 1.11		
	
	local magic_tentaclespike_recipe = Recipe("magic_tentaclespike", {Ingredient("killerbee", 1), Ingredient("monstermeat", 2), Ingredient("tentaclespots", 1)}, RECIPETABS.MAGIC, TECH.LOST)
	magic_tentaclespike_recipe.atlas = "images/inventoryimages.xml"
	magic_tentaclespike_recipe.image = "tentaclespike.tex"
	magic_tentaclespike_recipe.product = "tentaclespike"
	magic_tentaclespike_recipe.sortkey = 1.12	
	
	local magic_butter_recipe = Recipe("magic_butter", {Ingredient("butterfly", 20)}, RECIPETABS.MAGIC, TECH.LOST)
	magic_butter_recipe.atlas = "images/inventoryimages.xml"
	magic_butter_recipe.image = "butter.tex"
	magic_butter_recipe.product = "butter"
	magic_butter_recipe.sortkey = 1.13	
	
	local acidbottle = Recipe("shadow_acid_bottle", {Ingredient("nightmarefuel", 4), Ingredient("nitre", 3), Ingredient("rocks", 6)}, RECIPETABS.MAGIC, TECH.LOST)
	acidbottle.sortkey = 1.14
	
	local superacidbottle = Recipe("shadow_acid_bottle_advanced", {Ingredient("horrorfuel", 4), Ingredient("nitre", 3), Ingredient("rocks", 6)}, RECIPETABS.MAGIC, TECH.LOST)
	superacidbottle.sortkey = 1.15
	
	if IsDLCEnabled(PORKLAND_DLC) then
	    local multifunctional_tool_recipe = Recipe("multifunctional_tool", {Ingredient("multitool_axe_pickaxe", 1), Ingredient("thulecite", 5)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
        
	    local antlion_hat_recipe = Recipe("antlionhat", {Ingredient("beefalowool", 5), Ingredient("pigskin", 3), Ingredient("orangegem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
	    	
		local town_portal_recipe = Recipe("townportal_item", {Ingredient("cutstone", 3), Ingredient("orangegem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
	    
		local ancient_hanging_lamp_recipe = Recipe("ancient_hanging_lamp_item", {Ingredient("nightmarefuel", 6), Ingredient("thulecite", 4), Ingredient("bluegem", 2)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
	
		local ancientfountainofknowledgeitem_recipe = Recipe("ancient_fountain_of_knowledge_item", {Ingredient("cutstone", 4), Ingredient("thulecite", 12), Ingredient("charlie_exp", 6)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
	
        local shadow_helmet_recipe = Recipe("shadow_helmet", {Ingredient("nightmarefuel", 5), Ingredient("thulecite_pieces", 10)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
	
        local star_recipe = Recipe("starstaff_shadow", {Ingredient("yellowstaff", 1), Ingredient("nightmarefuel", 100), Ingredient("redgem", 5)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
	
	    local savereseter_recipe = Recipe("save_reseter", {Ingredient("horrorfuel", 5), Ingredient("thulecite", 3), Ingredient("orangegem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
	
	    local savesaver_recipe = Recipe("save_saver", {Ingredient("horrorfuel", 1), Ingredient("thulecite", 1), Ingredient("redgem", 2)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
		
	    local eyeofinsight_recipe = Recipe("eyeofinsight", {Ingredient("deerclops_eyeball", 1), Ingredient("thulecite_pieces", 5), Ingredient("nightmarefuel", 6)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
	
	    local armordreadstone_recipe = Recipe("armordreadstone", {Ingredient("dreadstone", 6), Ingredient("horrorfuel", 4)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
	
	    local dreadstonehat_recipe = Recipe("dreadstonehat", {Ingredient("dreadstone", 4), Ingredient("horrorfuel", 4)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)

	    local dark_crown_recipe = Recipe("shadow_ruins_crown", {Ingredient("horrorfuel", 30), Ingredient("thulecite_pieces", 40), Ingredient("redgem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)

        local orangestaff_shadow_recipe = Recipe("orangestaff_shadow", {Ingredient("orangestaff", 1), Ingredient("horrorfuel", 20), Ingredient("orangegem", 5)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)

        local repair_recipe = Recipe("voidcloth_kit", {Ingredient("horrorfuel", 1), Ingredient("voidcloth", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.COMMON, nil, nil, true)
		
		local fountain_obsidian_item_recipe = Recipe("fountain_obsidian_item", {Ingredient("cutstone", 4), Ingredient("obsidian", 12), Ingredient("charlie_exp", 6)}, RECIPETABS.OBSIDIAN, {OBSIDIAN = 2}, {RECIPE_GAME_TYPE.VANILLA, RECIPE_GAME_TYPE.SHIPWRECKED, RECIPE_GAME_TYPE.PORKLAND}, nil, nil, true) 
		
	elseif IsDLCEnabled(CAPY_DLC) then
	    local multifunctional_tool_recipe =	Recipe("multifunctional_tool", {Ingredient("multitool_axe_pickaxe", 1), Ingredient("thulecite", 5)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
        
	    local antlion_hat_recipe = Recipe("antlionhat", {Ingredient("beefalowool", 5), Ingredient("pigskin", 3), Ingredient("orangegem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
		local town_portal_recipe = Recipe("townportal_item", {Ingredient("cutstone", 3), Ingredient("orangegem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
		local ancient_hanging_lamp_recipe = Recipe("ancient_hanging_lamp_item", {Ingredient("nightmarefuel", 6), Ingredient("thulecite", 4), Ingredient("bluegem", 2)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
		local ancientfountainofknowledgeitem_recipe = Recipe("ancient_fountain_of_knowledge_item", {Ingredient("cutstone", 4), Ingredient("thulecite", 12)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
        local shadow_helmet_recipe = Recipe("shadow_helmet", {Ingredient("nightmarefuel", 5), Ingredient("thulecite_pieces", 10)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
        local star_recipe = Recipe("starstaff_shadow", {Ingredient("yellowstaff", 1), Ingredient("nightmarefuel", 30), Ingredient("redgem", 5)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	   
	    local savereseter_recipe = Recipe("save_reseter", {Ingredient("horrorfuel", 5), Ingredient("thulecite", 3), Ingredient("orangegem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
	    local savesaver_recipe = Recipe("save_saver", {Ingredient("horrorfuel", 1), Ingredient("thulecite", 1), Ingredient("redgem", 2)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
	    local eyeofinsight_recipe = Recipe("eyeofinsight", {Ingredient("deerclops_eyeball", 1), Ingredient("thulecite_pieces", 5), Ingredient("nightmarefuel", 6)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
	    local armordreadstone_recipe = Recipe("armordreadstone", {Ingredient("dreadstone", 6), Ingredient("horrorfuel", 4)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
	    local dreadstonehat_recipe = Recipe("dreadstonehat", {Ingredient("dreadstone", 4), Ingredient("horrorfuel", 4)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
	    local dark_crown_recipe = Recipe("shadow_ruins_crown", {Ingredient("horrorfuel", 30), Ingredient("thulecite_pieces", 40), Ingredient("redgem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
	    
        local orangestaff_shadow_recipe = Recipe("orangestaff_shadow", {Ingredient("orangestaff", 1), Ingredient("horrorfuel", 20), Ingredient("orangegem", 5)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
        
        local repair_recipe = Recipe("voidcloth_kit", {Ingredient("horrorfuel", 1), Ingredient("voidcloth", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, RECIPE_GAME_TYPE.VANILLA, nil, nil, true)
        
		local fountain_obsidian_item_recipe = Recipe("fountain_obsidian_item", {Ingredient("cutstone", 4), Ingredient("obsidian", 12), Ingredient("charlie_exp", 6)}, RECIPETABS.OBSIDIAN, {OBSIDIAN = 2}, RECIPE_GAME_TYPE.SHIPWRECKED, nil, nil, true)
		
	else
	    local multifunctional_tool_recipe = Recipe("multifunctional_tool", {Ingredient("multitool_axe_pickaxe", 1), Ingredient("thulecite", 5)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, nil, nil, true)
        
	    local antlion_hat_recipe = Recipe("antlionhat", {Ingredient("beefalowool", 5), Ingredient("pigskin", 3), Ingredient("orangegem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, nil, nil, true)
	    
		local town_portal_recipe = Recipe("townportal_item", {Ingredient("cutstone", 3), Ingredient("orangegem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, nil, nil, true)
	   
		local ancient_hanging_lamp_recipe = Recipe("ancient_hanging_lamp_item", {Ingredient("nightmarefuel", 6), Ingredient("thulecite", 4), Ingredient("bluegem", 2)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, nil, nil, true)
	    
		local ancientfountainofknowledgeitem_recipe = Recipe("ancient_fountain_of_knowledge_item", {Ingredient("cutstone", 4), Ingredient("thulecite", 12)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, nil, nil, true)
	    
        local shadow_helmet_recipe = Recipe("shadow_helmet", {Ingredient("nightmarefuel", 5), Ingredient("thulecite_pieces", 10)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, nil, nil, true)
	    
        local star_recipe = Recipe("starstaff_shadow", {Ingredient("yellowstaff", 1), Ingredient("nightmarefuel", 30), Ingredient("redgem", 5)}, RECIPETABS.ANCIENT, {ANCIENT = 6}, nil, nil, true)
	    
	    local savereseter_recipe = Recipe("save_reseter", {Ingredient("horrorfuel", 5), Ingredient("thulecite", 3), Ingredient("orangegem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, nil, nil, true)
	    
	    local savesaver_recipe = Recipe("save_saver", {Ingredient("horrorfuel", 1), Ingredient("thulecite", 1), Ingredient("redgem", 2)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, nil, nil, true)
	    
	    local eyeofinsight_recipe = Recipe("eyeofinsight", {Ingredient("deerclops_eyeball", 1), Ingredient("thulecite_pieces", 5), Ingredient("nightmarefuel", 6)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, nil, nil, true)
	    
	    local armordreadstone_recipe = Recipe("armordreadstone", {Ingredient("dreadstone", 6), Ingredient("horrorfuel", 4)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, nil, nil, true)
	    
	    local dreadstonehat_recipe = Recipe("dreadstonehat", {Ingredient("dreadstone", 4), Ingredient("horrorfuel", 4)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, nil, nil, true)
	    
	    local dark_crown_recipe = Recipe("shadow_ruins_crown", {Ingredient("horrorfuel", 30), Ingredient("thulecite_pieces", 40), Ingredient("redgem", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, nil, nil, true)
	    
        local orangestaff_shadow_recipe = Recipe("orangestaff_shadow", {Ingredient("orangestaff", 1), Ingredient("horrorfuel", 20), Ingredient("orangegem", 5)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, nil, nil, true)
        
        local repair_recipe = Recipe("voidcloth_kit", {Ingredient("horrorfuel", 1), Ingredient("voidcloth", 1)}, RECIPETABS.ANCIENT, {ANCIENT = 8}, nil, nil, true)
       
	end
	
---------------------------------------------------------------------------------------------------

    inst:DoTaskInTime(0, Initialize)

    if GetWorld() then
	    GetWorld():DoTaskInTime(1, function(_)
	        PatchCharlieStage(_)		
		end)
	end
	
	extensions_fn(inst)

	inst.OnSave = OnSave
    inst.OnLoad = OnLoad
end
	
return MakePlayerCharacter("charlie", prefabs, assets, fn, start_inv)