local assets = 
{
    Asset("ANIM", "anim/atrium_gate.zip"),
    Asset("ANIM", "anim/atrium_gate_build.zip"),
    Asset("ANIM", "anim/atrium_floor.zip"),
}

local assets_fx = 
{
    Asset("ANIM", "anim/atrium_gate_overload_fx.zip"),
}

local prefabs = 
{

}

local REPAIR_GATE_ANIM_LENGTH = 114 * FRAMES

local CHARLIE_SPAWN_DELAY = 2
local CHARLIE_START_CAST_DELAY = 3
local CHARLIE_CAST_TIME = REPAIR_GATE_ANIM_LENGTH + 0.5 + (20 * FRAMES)

local START_REPAIRING_GATE_DELAY = CHARLIE_START_CAST_DELAY + (80 * FRAMES)
local REPAIR_GATE_DELAY = START_REPAIRING_GATE_DELAY + REPAIR_GATE_ANIM_LENGTH

local START_TWEENING_DELAY = REPAIR_GATE_ANIM_LENGTH * 0.95
local TWEEN_TO_BLACK_TIME = REPAIR_GATE_ANIM_LENGTH - START_TWEENING_DELAY
local REVERT_COLOUR_TIME = 3.5

local function GetInspectStatus(inst)
    return inst:HasTag("gateway_activated") and "ACTIVATED"
	    or nil
end

local function onpickedfn(inst, picker)
    local x, y, z = inst.Transform:GetWorldPosition()
    for i=1, math.random(10,20) do
	    inst.components.lootdropper:FlingItem(SpawnPrefab("thulecite"), Vector3(x, 3, z))
	end
    inst.AnimState:PlayAnimation("overload_pst")
    inst.AnimState:PushAnimation("idle")
end

local function makefullfn(inst)
    inst.AnimState:PlayAnimation("overload_loop", true)
end

local function TweenToNormalColour(inst)
    inst.components.colourtweener:StartTween({1, 1, 1, 1}, REVERT_COLOUR_TIME)
end

local function RevertToNormalColour(inst)
    inst:DoTaskInTime(0.2, TweenToNormalColour)
end

local function TweenToBlack(inst)
    inst.components.colourtweener:StartTween({0, 0, 0, 1}, TWEEN_TO_BLACK_TIME, RevertToNormalColour)
end

local function RoundPillarAngle(value)
    local _start = value > 0 and 45 or -315
    local _end   = value > 0 and 315 or -45

    for target = _start, _end, 90 do
        local difference = math.abs(value - target)
        if difference < 5 then
            return target
        end
    end

    return value
end

local function FindSceneCameraAngle(inst)
    local pillar_pos = inst:GetPosition()
	local gate_pos = inst:GetPosition()

    local angle = math.atan2(gate_pos.z - pillar_pos.z, pillar_pos.x - gate_pos.x) / DEGREES + 180

    angle = RoundPillarAngle(angle)

    local offset = ((angle == 45 or angle == 225) and -90) or 90

    return (angle + offset) % 360
end

local function FindCharlieSpawnPoint(inst)
    local pillar_pos = inst:GetPosition()
	local gate_pos = inst:GetPosition()
    local spawn_pos = (pillar_pos * 0.4) + (gate_pos * 0.6)
	local _cameraangle = FindSceneCameraAngle(inst)

    -- Moving the spawn point slightly ahead of the atrium gate.
    local angle = _cameraangle / RADIANS
    local mult = 3

    local offset = Vector3(math.cos(angle), 0, math.sin(angle)):Normalize() * mult
	
    return spawn_pos + offset
end

local function SpawnCharlieNPC(inst)
    local charlie_npc = SpawnPrefab("charlie_npc")
    local spawn_pos = FindCharlieSpawnPoint(inst)
	local gate_pos = inst:GetPosition()
    
	charlie_npc.persists = false
    charlie_npc.Transform:SetPosition(spawn_pos:Get())
    charlie_npc:ForceFacePoint(gate_pos:Get())
    charlie_npc:StartCastingWithDelay(CHARLIE_START_CAST_DELAY, CHARLIE_CAST_TIME)
end

local function StartRepairingGate(inst)
    inst.AnimState:PlayAnimation("fixing")

    inst.SoundEmitter:PlaySound("rifts2/charliecutscene/????") -- On fixing atrium gate sound (loop maybe?) @amanda ("fixing" anim).

    inst:DoTaskInTime(START_TWEENING_DELAY, TweenToBlack)
end

local function RepairGate(inst)
	inst.AnimState:SetBuild("atrium_gate_build")
	inst.AnimState:PlayAnimation("fixed")
	inst.MiniMapEntity:SetIcon("charlie_ancient_gateway_activated.tex")
	inst.SoundEmitter:PlaySound("rifts2/charliecutscene/????")
	inst.components.pickable.caninteractwith = true
    if inst._gatefx ~= nil then
        inst._gatefx:KillFX()
        inst._gatefx = nil
    end
    GetPlayer().HUD:Show()
	GetPlayer().components.playercontroller:Enable(true)
	GetPlayer().components.health:SetInvincible(false)	
end

local function ActivateGateway(inst)
    if inst._gatefx == nil then
        inst._gatefx = SpawnPrefab("charlie_ancient_gateway_fx")
        inst._gatefx.entity:SetParent(inst.entity)
        --table.insert(inst.highlightchildren, inst._gatefx)
    end
	GetPlayer().HUD:Hide()
    GetPlayer().components.playercontroller:Enable(false)
	GetPlayer().components.health:SetInvincible(true)
    inst:AddTag("gateway_activated")
    inst:DoTaskInTime(CHARLIE_SPAWN_DELAY, SpawnCharlieNPC)
	inst:DoTaskInTime(START_REPAIRING_GATE_DELAY, StartRepairingGate)
    inst:DoTaskInTime(REPAIR_GATE_DELAY, RepairGate)
end

--Strangely, have to do this to prevent Chinese disorderly code.
local CN_1 = STRINGS.CN_ANCIENT_GATEWAY_1
local CN_2 = STRINGS.CN_ANCIENT_GATEWAY_2
local CN_3 = STRINGS.CN_ANCIENT_GATEWAY_3
local CN_4 = STRINGS.CN_ANCIENT_GATEWAY_4

local function OnRefuse(inst, giver, item)
    if giver ~= nil and giver.components.talker ~= nil then
	    if inst:HasTag("gateway_activated") then
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("It is already complete, does not need to be repaired again.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say(CN_1)  --ѾˣҪٴ޸ˡ
		    end	
		else
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("Me in the future told me that the repair material is dark rocks.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say(CN_2) --δҸ޸ְɫʯͷ
		    end		
		end
	end
end

local function OnAccept(inst, giver, item)
    if not inst:HasTag("gateway_activated") then
	    inst.dreadstone_amount = math.min(inst.dreadstone_amount + 1, 5)
		if inst.dreadstone_amount == 5 then
		    ActivateGateway(inst)
            if giver ~= nil and giver.components.talker ~= nil then
	            if TUNING.LANGUAGE == true then
		            giver.components.talker:Say("It's changing!")
		        elseif TUNING.LANGUAGE == false then
		            giver.components.talker:Say(CN_3)  --ڷ仯
		        end		
	        end
		    return
		end
        if giver ~= nil and giver.components.talker ~= nil then
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("I need more of this to fully repair it.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say(CN_4) --Ҫֶ޸
		    end		
	    end
	end
end

local function AcceptTest(inst, item, giver)
    if inst:HasTag("gateway_activated") then
        return false
	end
    return item ~= nil and item:HasTag("dreadstone")
end

local function CreateFloor()
    local inst = CreateEntity()

    inst:AddTag("DECOR")
    inst:AddTag("NOCLICK")

    inst.persists = false

    inst.entity:AddTransform()
    inst.entity:AddAnimState()

    inst.AnimState:SetBank("atrium_floor")
    inst.AnimState:SetBuild("atrium_floor")
    inst.AnimState:PlayAnimation("idle_active")
    inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
    inst.AnimState:SetSortOrder(-3)

    return inst
end

local function checkidleanim(inst)
    if inst.AnimState:IsCurrentAnimation("idle") then
        inst.AnimState:PlayAnimation("cooldown")
        inst.AnimState:PushAnimation("idle")
    end
    inst:DoTaskInTime(math.random()*10 + 8, checkidleanim)
end

local function OnSave(inst, data)
    data.dreadstone_amount = inst.dreadstone_amount
    data.activated = inst:HasTag("gateway_activated") or nil
end

local function OnLoad(inst, data)
    if data ~= nil then    
 		inst.dreadstone_amount = data.dreadstone_amount or 0
		if data.activated then
		    inst:AddTag("gateway_activated")
	        inst.AnimState:SetBuild("atrium_gate_build")
	        inst.MiniMapEntity:SetIcon("charlie_ancient_gateway_activated.tex")	
			inst.components.pickable.caninteractwith = true
		end
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddLight()

    MakeObstaclePhysics(inst, 1)

    inst.Light:Enable(false)
    inst.Light:SetRadius(8.0)
    inst.Light:SetFalloff(.9)
    inst.Light:SetIntensity(0.65)
    inst.Light:SetColour(200 / 255, 140 / 255, 140 / 255)

    inst.AnimState:SetBank("atrium_gate")
    inst.AnimState:SetBuild("atrium_gate")
    inst.AnimState:PlayAnimation("idle")
	
    CreateFloor().entity:SetParent(inst.entity)	

    inst.MiniMapEntity:SetIcon("charlie_ancient_gateway.tex")

    inst:AddTag("charlie_ancient_gateway")
	inst:AddTag("shadow_destroy_immune")
	inst:AddTag("pugalisk_avoids")
	inst:AddTag("cannot_be_reaped")
	
    inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = GetInspectStatus
	
	--inst.highlightchildren = {}
	inst.dreadstone_amount = 0

	inst:AddComponent("pickable")
	inst.components.pickable:SetUp("cave_regenerator", TUNING.TOTAL_DAY_TIME*20)
	inst.components.pickable.picksound = "dontstarve/HUD/get_gold"
    inst.components.pickable.onpickedfn = onpickedfn
    inst.components.pickable.makefullfn = makefullfn
	inst.components.pickable.caninteractwith = false

	inst:AddComponent("trader")
	inst.components.trader.acceptnontradable = true
	inst.components.trader.onrefuse = OnRefuse
	inst.components.trader.onaccept = OnAccept
    inst.components.trader:SetAcceptTest(AcceptTest)

    inst:AddComponent("colourtweener")

    inst:AddComponent("lootdropper")
	
    inst:DoTaskInTime(math.random()*10 + 8, checkidleanim)	
	
    inst.OnSave = OnSave
    inst.OnLoad = OnLoad
	
	return inst
end

local function SetFX(inst, anim)
    if inst.killed then
        return
    elseif (inst.anim or "idle") ~= anim then
        if inst.anim ~= nil then
            inst.AnimState:PushAnimation(inst.anim.."_pst", false)
            inst.AnimState:PushAnimation(anim.."_pre", false)
        else
            inst.AnimState:PlayAnimation(anim.."_pre")
        end
        inst.AnimState:PushAnimation(anim.."_loop", true)

        if anim == "idle" then
            inst:RemoveTag("NOCLICK")
            inst:RemoveTag("DECOR")
            inst:AddTag("FX")
        else
            inst:RemoveTag("FX")
            inst:AddTag("DECOR")
            inst:AddTag("NOCLICK")
        end
    end
    inst.anim = anim
end

local function KillFX(inst)
    if not inst.killed then
        inst.killed = true
        inst.AnimState:PushAnimation((inst.anim or "idle").."_pst", false)
        inst:ListenForEvent("animqueueover", inst.Remove)
        inst:DoTaskInTime(4, inst.Remove)
    end
end

local function fn_fx()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddLight()

    inst.Light:Enable(false)
    inst.Light:SetColour(200 / 255, 140 / 255, 140 / 255)
    inst.Light:SetRadius(8.0)
    inst.Light:SetFalloff(.9)
    inst.Light:SetIntensity(0.65)

    inst.AnimState:SetBank("atrium_gate_overload_fx")
    inst.AnimState:SetBuild("atrium_gate_overload_fx")
    inst.AnimState:PlayAnimation("idle_pre")

    inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
    inst.AnimState:SetFinalOffset(1)

    inst:AddTag("FX")
    inst:AddTag("INTERIOR_LIMBO_IMMUNE")

    inst.anim = nil
    inst.AnimState:PushAnimation("idle_loop", true)

    inst.SetFX = SetFX
    inst.KillFX = KillFX

    inst.persists = false

    return inst
end


return Prefab("charlie_ancient_gateway", fn, assets, prefabs),
    Prefab("charlie_ancient_gateway_fx", fn_fx, assets_fx, prefabs)