require("prefabutil")

local flag_post_assets =
{
	Asset("ANIM", "anim/flag_post.zip"),
	Asset("ANIM", "anim/flag_post_wilson_build.zip"),
	Asset("ANIM", "anim/flag_post_royal_build.zip"),
	Asset("ANIM", "anim/flag_post_perdy_build.zip"),
	Asset("ANIM", "anim/flag_post_hand_build.zip"),	
	Asset("ANIM", "anim/flag_post_duster_build.zip"),		
}

local flag_post_prefabs = 
{
	"collapse_small",
}

local function OnHammered(inst, worker)
	if inst.components.burnable ~= nil and inst.components.burnable:IsBurning() then
		inst.components.burnable:Extinguish()
	end
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_metal")	
	inst:Remove()
end

local function OnHit(inst, worker)
	if not inst:HasTag("burnt") then
		inst.AnimState:PlayAnimation("hit")
		inst.AnimState:PushAnimation("idle", true)
	end
end

local function OnBuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("idle", true)
end

local function OnSave(inst, data)
	if inst.components.burnable ~= nil and inst.components.burnable:IsBurning() or inst:HasTag("burnt") then
		data.burnt = true
	end
end

local function OnLoad(inst, data)
	if data ~= nil and data.burnt and inst.components.burnable ~= nil then
		inst.components.burnable.onburnt(inst)
	end
end

local function common_flag_post_fn(build, is_hamlet_exclusive, is_charlie_exclusive)
    local inst = CreateEntity()

    if is_hamlet_exclusive then
	    if not IsDLCEnabled(PORKLAND_DLC) then
		    inst:DoTaskInTime(0, inst.Remove)
	        return inst
		end
	end

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
    MakeObstaclePhysics(inst, 0.25)   

	inst.AnimState:SetBank("flag_post")
	inst.AnimState:SetBuild(build)
	inst.AnimState:PlayAnimation("idle", true)

    inst:AddTag("structure")
	
	if is_charlie_exclusive then
	    inst:AddTag("charlie_exclusive")
		inst:AddTag("shadow_destroy_immune")
	end

	inst:AddComponent("inspectable")
	inst:AddComponent("lootdropper")
	
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(2)
	inst.components.workable:SetOnFinishCallback(OnHammered)
	inst.components.workable:SetOnWorkCallback(OnHit)

	MakeMediumBurnable(inst, nil, nil, true)
	MakeMediumPropagator(inst)
	
	inst:ListenForEvent("onbuilt", OnBuilt)

	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
	
    return inst
end

local function flag_post_wilson_fn()
    local inst = common_flag_post_fn("flag_post_wilson_build", true)
	
	return inst
end

local function flag_post_royal_fn()
    local inst = common_flag_post_fn("flag_post_royal_build", true)
	
	return inst
end

local function flag_post_perdy_fn()
    local inst = common_flag_post_fn("flag_post_perdy_build", true)
	
	return inst
end

local function flag_post_hand_fn()
    local inst = common_flag_post_fn("flag_post_hand_build", true, true)
	
	return inst
end

local function flag_post_duster_fn()
    local inst = common_flag_post_fn("flag_post_duster_build", true)
	
	return inst
end

return Prefab("flag_post_wilson", flag_post_wilson_fn, flag_post_assets, flag_post_prefabs),
    Prefab("flag_post_royal", flag_post_royal_fn, flag_post_assets, flag_post_prefabs),
	Prefab("flag_post_perdy", flag_post_perdy_fn, flag_post_assets, flag_post_prefabs),
	Prefab("flag_post_hand", flag_post_hand_fn, flag_post_assets, flag_post_prefabs),
	Prefab("flag_post_duster", flag_post_duster_fn, flag_post_assets, flag_post_prefabs),
	MakePlacer("flag_post_wilson_placer", "flag_post", "flag_post_wilson_build", "idle"),
    MakePlacer("flag_post_royal_placer", "flag_post", "flag_post_royal_build", "idle"),
	MakePlacer("flag_post_perdy_placer", "flag_post", "flag_post_perdy_build", "idle"),
	MakePlacer("flag_post_hand_placer", "flag_post", "flag_post_hand_build", "idle"),
	MakePlacer("flag_post_duster_placer", "flag_post", "flag_post_duster_build", "idle")