local assets = 
{
    Asset("ANIM", "anim/des_fsp_grand_final_fix.zip"),	
}

local prefabs = 
{

}

local function CanPlantAtPoint(x, y, z)
    local tile = GetWorld().Map:GetTileAtPoint(x, y, z)
    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    if tile == GROUND.ASH or tile == GROUND.VOLCANO 
		    or tile == GROUND.VOLCANO_ROCK or tile == GROUND.BRICK_GLOW
			or tile == GROUND.VOLCANO_LAVA or tile == GROUND.SNAKESKIN
		then
		    return false
		end
	end
	if IsDLCEnabled(PORKLAND_DLC) then
	    if tile == GROUND.FOUNDATION or tile == GROUND.COBBLEROAD
		    or tile == GROUND.GASJUNGLE or tile == GROUND.PIGRUINS
			or tile == GROUND.PIGRUINS_NOCANOPY or tile == GROUND.BEARDRUG or tile == GROUND.INTERIOR
		then                                                               
		    return false
		end
	end
    if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and tile ~= GROUND.ROCKY and tile ~= GROUND.ROAD
		and tile ~= GROUND.UNDERROCK and tile ~= GROUND.WOODFLOOR and tile ~= GROUND.CARPET and tile ~= GROUND.CHECKER
		and tile < GROUND.UNDERGROUND
	then
		return true
	end
	return false
end

local BLOCKER_EXCLUDE_TAGS = {"FX", "INLIMBO"}
local function IsPointNearBlocker(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, BLOCKER_EXCLUDE_TAGS) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearFlower(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"flower", "flower_rainforest"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearWallOrStructure(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"wall", "structure"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function SpawnFlowerNearby(inst)
    local world = GetWorld()
    local flower_to_spawn = "flower"
	if GetClock() and GetClock():GetMoonPhase() == "full" then
	    flower_to_spawn = "flower_evil"
	end
	if world ~= nil and world:HasTag("porkland") then
	    flower_to_spawn = "flower_rainforest"
	end	
	if math.random() < 0.01 then
	    flower_to_spawn = "flower_rose"
	end	
	if PrefabExists(flower_to_spawn) then
        local pt = inst:GetPosition()
	    local pos = FindWalkableOffset(pt, math.random()*2*math.pi, 5+math.random()*12, 30, true, false)
		if inst.GetIsOnLand ~= nil then
		    pos = FindGroundOffset(pt, math.random()*2*math.pi, 5+math.random()*12, 30, true, false)
		end
        if pos then	    
		    local spawnpt = pt + pos
			if CanPlantAtPoint(spawnpt.x, 0, spawnpt.z)
			    and ( inst.IsPosSurroundedByLand == nil or inst:IsPosSurroundedByLand(spawnpt.x, 0, spawnpt.z, 3) )
				and not IsPointNearBlocker(spawnpt, 3) 	
				and not IsPointNearFlower(spawnpt, 5)
			    and not IsPointNearWallOrStructure(spawnpt, 10) 	
                and not ( RoadManager ~= nil and RoadManager:IsOnRoad(spawnpt.x, 0, spawnpt.z) )				
			then
			    print("flower spawns from charlie_fountain")
				local fx = SpawnPrefab("small_puff")
			    local flower = SpawnPrefab(flower_to_spawn)
		        flower.Transform:SetPosition(spawnpt.x, 0, spawnpt.z)
				fx.Transform:SetPosition(spawnpt.x, 0, spawnpt.z)
			end
		end	
	end
end

local NUM_BUGS = 3
local BUG_OFFSET = 1.4
local BUG_RANDOM_RANGE = .5
local function SpawnFountainBugs(inst)
    local x, y, z = inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x, y, z, BUG_OFFSET + BUG_RANDOM_RANGE + 0.1)
    local bug_pts = {}
    for k,ent in ipairs(ents) do
        if ent.prefab == "fireflies" then
            table.insert(bug_pts, ent:GetPosition())
        end
    end

    local pos = inst:GetPosition()
    local offset = nil
    for i = #bug_pts + 1, NUM_BUGS do
        if i == 1 then
            offset = Vector3(BUG_OFFSET, 0, 0)
        elseif i == 2 then
            local dir = bug_pts[1] - pos
            local ca = math.cos(0.33*2*math.pi)
            local sa = math.sin(0.33*2*math.pi)
            offset = Vector3(ca*dir.x - sa*dir.y, 0, sa*dir.x + ca*dir.y):Normalize() * BUG_OFFSET
        elseif i == 3 then
            offset = Vector3(0,0,0)
            for _,pt in ipairs(bug_pts) do
                offset = offset + (pt - pos)
            end
            offset = offset:Normalize() * BUG_OFFSET
            offset.x = -offset.x
            offset.z = -offset.z
        end

        local bug = SpawnPrefab("fireflies")
        bug.Transform:SetPosition((pos + offset + Vector3(math.random()*BUG_RANDOM_RANGE, 0, math.random()*BUG_RANDOM_RANGE)):Get())
        table.insert(bug_pts, bug:GetPosition())
    end
end

local function reset(inst)
    inst:RemoveTag("charlie_fountain_dry")
	inst.AnimState:PlayAnimation("fsp_cycle", true)
	inst.components.periodicflowerspawner:Enable(true)
	inst.components.activatable.inactive = true
	inst.components.fishable:Unfreeze()
	SpawnFountainBugs(inst)
    if inst.resettask then
        inst.resettask:Cancel()
        inst.resettask = nil
    end
    if inst.resettaskinfo then
        inst.resettaskinfo = nil
    end	
end

local function GetInspectStatus(inst)
    return inst:HasTag("charlie_fountain_dry") and "DRY"
	    or nil
end

local function OnRefuse(inst, giver, item)
    if giver ~= nil and giver.components.talker ~= nil then
	    if not inst:HasTag("charlie_fountain_dry") then
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("It is now in its heyday, and does not require sacrifice.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say("它现在是全盛时期，不需要献祭。")
		    end	
		else
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("According to the instructions, the offerings should be cloth left over by the ancients.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say("根据指示，祭品应该是古人遗留下来的布。")
		    end		
		end
	end
end

local function OnAccept(inst, giver, item)
    reset(inst)
    if giver ~= nil 
	    and giver.components.poisonable ~= nil 
		and giver.components.poisonable.poisoned 
	then
        giver.components.poisonable:Cure(inst)
    end
    if giver ~= nil 
	    and giver.components.health ~= nil 
		and giver.components.health:GetPercent() < 1 
	then
        giver.components.health:DoDelta(500, false, inst.prefab)
    end	
	if giver ~= nil then
	    giver:PushEvent("celebrate") --Sadly only for SW or HAM enabled
	end
end

local function AcceptTest(inst, item, giver)
    if not inst:HasTag("charlie_fountain_dry") then
	    return false
	end
    return item ~= nil and item:HasTag("fountain_altar_sacrifice")
end

local function OnActivate(inst, doer)
    inst:AddTag("charlie_fountain_dry")
    inst.AnimState:PlayAnimation("fsp_dry", true)
	inst.components.periodicflowerspawner:Enable(false)
	inst.components.fishable:Freeze()
	if doer ~= nil then
	    local possible_items = 
		{ 
		    "rose_inv",
			"honeycomb",
			"dug_berrybush",
			"dug_berrybush2",
			"dug_grass",
			"dug_sapling",
			"dug_marsh_bush",
			"lureplantbulb",
			"pinecone",
			"gears",
			"marble",
			"goldnugget",
			"dug_berrybush_juicy",
			"dug_bananabush",
			"dug_monkeytail",
		}
		if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
		    table.insert(possible_items, "acorn")
		end		
		if GetWorld():HasTag("shipwrecked") then
		    table.insert(possible_items, "dug_bambootree")
			table.insert(possible_items, "dug_bush_vine")
			table.insert(possible_items, "seaweed_stalk")
			table.insert(possible_items, "dug_coffeebush")
			table.insert(possible_items, "dug_elephantcactus")
		end
		if GetWorld():HasTag("porkland") then
		    table.insert(possible_items, "dug_nettle")
			table.insert(possible_items, "burr")
			table.insert(possible_items, "teatree_nut")
			table.insert(possible_items, "bramble_bulb")
		end
	    local item = SpawnPrefab(GetRandomItem(possible_items))
	    if doer.components.inventory ~= nil then
		    doer.components.inventory:GiveItem(item, nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
		else
		    item.Transform:SetPosition(doer.Transform:GetWorldPosition())
		    SpawnPrefab("small_puff").Transform:SetPosition(doer.Transform:GetWorldPosition())		
		end
	end
    if not inst.resettask then
        inst.resettask, inst.resettaskinfo = inst:ResumeTask(TUNING.TOTAL_DAY_TIME*20, reset)
    end
end

local function OnLongUpdate(inst, dt)
    if inst.resettask and inst.resettaskinfo then
        local new_time = math.max(inst:TimeRemainingInTask(inst.resettaskinfo) - dt, 0)

        inst.resettask:Cancel()
        inst.resettask = nil 
        inst.resettaskinfo = nil

        inst.resettask, inst.resettaskinfo = inst:ResumeTask(new_time, reset)
    end
end

local function GetHeatFn(inst, observer)
    local season_manager = GetSeasonManager()
    if season_manager ~= nil 
	    and season_manager:GetCurrentTemperature() > 60 
	    and not inst:HasTag("charlie_fountain_dry")
	then
	    return -60
	end
	return 0
end

local function OnSave(inst, data)
    data.dry = inst:HasTag("charlie_fountain_dry") or nil
    if inst.resettaskinfo then
        data.resettask = inst:TimeRemainingInTask(inst.resettaskinfo)
    end
end

local function OnLoad(inst, data)
    if data ~= nil then    
	    if data.dry then
            inst:AddTag("charlie_fountain_dry")
		    inst.AnimState:PlayAnimation("fsp_dry", true)
		    inst.components.periodicflowerspawner:Enable(false)
		    inst.components.activatable.inactive = false
			inst.components.fishable:Freeze()
		end
        if data.resettask then
            if inst.resettask then 
			    inst.resettask:Cancel() 
				inst.resettask = nil 		
			end
            inst.resettaskinfo = nil
            inst.resettask, inst.resettaskinfo = inst:ResumeTask(data.resettask, reset)
        end 		
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()
	
	MakeObstaclePhysics(inst, 0.75)
	
	inst:AddTag("charlie_fountain")
	inst:AddTag("shadow_destroy_immune")
	inst:AddTag("pugalisk_avoids")
	inst:AddTag("fishable")
	
	inst.MiniMapEntity:SetIcon("charlie_fountain.tex")
	
    inst.AnimState:SetBank("des_fsp_grand_final_fix")
    inst.AnimState:SetBuild("des_fsp_grand_final_fix")
	inst.AnimState:PlayAnimation("fsp_cycle", true)
	
    inst:AddComponent("inspectable")
	inst.components.inspectable:RecordViews()
	inst.components.inspectable.getstatus = GetInspectStatus
	
	inst:AddComponent("periodicflowerspawner")
	inst.components.periodicflowerspawner:Enable(true)
	
	inst:AddComponent("trader")
	inst.components.trader.acceptnontradable = true
	inst.components.trader.onrefuse = OnRefuse
	inst.components.trader.onaccept = OnAccept
    inst.components.trader:SetAcceptTest(AcceptTest)

    inst:AddComponent("activatable")
    inst.components.activatable.OnActivate = OnActivate
    inst.components.activatable.inactive = true	

	inst:AddComponent("fishable")
	inst.components.fishable:SetRespawnTime(50)
	inst.components.fishable:AddFish("fish")
	inst.components.fishable:AddFish("eel")
	inst.components.fishable.maxfish = 15
	inst.components.fishable.fishleft = 15
	
    if GetWorld():HasTag("shipwrecked") then
        inst.components.fishable:AddFish("tropical_fish")
    end

    inst:AddComponent("heater")
    inst.components.heater.heatfn = GetHeatFn
    inst.components.heater.iscooler = true		
	
	inst.SpawnFlowerNearby = SpawnFlowerNearby
	inst.OnLongUpdate = OnLongUpdate
    inst.OnSave = OnSave
    inst.OnLoad = OnLoad
    
	return inst
end

return Prefab("charlie_fountain", fn, assets, prefabs)