
local assets =
{
    Asset("ANIM", "anim/charlie_lectern.zip"),
}

local prefabs =
{
    "papyrus",
}

local function Restore(inst)
    if not inst.HasBook then
        if inst.components.activatable ~= nil then
            inst.components.activatable.inactive = true
        end
		if inst.components.pickable ~= nil then
		    inst.components.pickable.caninteractwith = false
		end
	    inst.HasBook = true
		inst.AnimState:PlayAnimation("idle")
	    inst.AnimState:OverrideSymbol("book_pages", "charlie_lectern", "book_pages")
		inst.AnimState:OverrideSymbol("playbill", "charlie_lectern", "playbill")
		if inst.task then 
		    inst.task:Cancel() 
		    inst.task = nil  
	    end			
        if inst.taskinfo then
            inst.taskinfo = nil 
        end		
	end
end

local function SetTask(inst, time)
	if inst.task then 
		inst.task:Cancel() 
		inst.task = nil  
	end
    if inst.taskinfo then 
        inst.taskinfo = nil 
    end		
	if not time then 
		time = TUNING.TOTAL_DAY_TIME*5 + (math.random()*TUNING.TOTAL_DAY_TIME*3)
	end
	inst.task, inst.taskinfo = inst:ResumeTask( time, function() Restore(inst) end )  
end

local function flippage(inst, doer)
    local anim = (math.random() < 0.5 and "pageturn_extra") or "pageturn"
    inst.AnimState:PlayAnimation(anim, false)
    inst.AnimState:PushAnimation("switch", false)
	
    inst.SoundEmitter:PlaySound("stageplay_set/stage/lecturn_pageturn")
	
	inst.HasBook = false
	inst:SetTask()

	if inst.components.pickable ~= nil then
	    inst.components.pickable.caninteractwith = true
	end

	if doer == GetPlayer() then
	    GetPlayer().HUD:Hide()
	    GetPlayer().components.playercontroller:Enable(false)
	    GetPlayer().components.health:SetInvincible(true)
	end

	inst:DoTaskInTime(0.75, function()
	    if doer == GetPlayer() then
	        if TUNING.LANGUAGE == true then
		        GetPlayer().components.talker:Say("Obviously I can take a quick look at shorthand.")
		    elseif TUNING.LANGUAGE == false then
		        GetPlayer().components.talker:Say("显然我可以速看速记。")
		    end
		end
	end)	
	
	inst:DoTaskInTime(1.5, function()
	    inst.AnimState:OverrideSymbol("book_pages", "charlie_lectern", "")
		inst.AnimState:OverrideSymbol("playbill", "charlie_lectern", "")
	    if doer == GetPlayer() then
	        GetPlayer().HUD:Show()
		    GetPlayer().components.playercontroller:Enable(true)
			GetPlayer().components.health:SetInvincible(false)
		end
	end)
	
	if doer ~= nil then
        local x, _, z = inst.Transform:GetWorldPosition()
        local x_dist = math.random()        
	    local z_dist = math.random()        
		local subtheta = math.random() * 2 * math.pi
        local spawn_x, spawn_z
        spawn_x, spawn_z = x + math.cos(subtheta) * x_dist, z + math.sin(subtheta) * z_dist				
		inst.components.lootdropper:DropLoot(Vector3(spawn_x, 1, spawn_z))
	    SpawnPrefab("small_puff").Transform:SetPosition(spawn_x, 1, spawn_z)	
	end
end

local function checkidleanim(inst)
    if inst.AnimState:IsCurrentAnimation("idle") 
	    and inst.components.pickable ~= nil and not inst.components.pickable.hasbeenpicked
	then
        inst.AnimState:PlayAnimation("idle_vine_move_"..math.random(1,3))
        inst.AnimState:PushAnimation("idle")
    end
    inst:DoTaskInTime(math.random()*3 + 2, checkidleanim)
end

local function onpickedfn(inst, picker)
    inst.AnimState:OverrideSymbol("rose", "charlie_lectern", "")
end

local function onregenfn(inst)
    inst.AnimState:OverrideSymbol("rose", "charlie_lectern", "rose")
end

local function onsave(inst, data)
	data.HasBook = inst.HasBook
    if inst.task and inst.taskinfo then
        data.timeleft = inst:TimeRemainingInTask(inst.taskinfo)
    end	
end

local function onload(inst, data)
    if data and data.HasBook ~= nil then
        inst.HasBook = data.HasBook
		if inst.HasBook == true then
            --do nothing
		else
	        inst.AnimState:OverrideSymbol("book_pages", "charlie_lectern", "")
		    inst.AnimState:OverrideSymbol("playbill", "charlie_lectern", "")
		end
    end	
	if data and data.timeleft then
	    inst:SetTask(data.timeleft)		
        if inst.components.activatable ~= nil then
            inst.components.activatable.inactive = false
        end
	end 		
	if inst.HasBook then
        if inst.components.activatable ~= nil then
            inst.components.activatable.inactive = true
        end	
	end
    if inst.components.pickable ~= nil and inst.components.pickable.hasbeenpicked then
        inst.AnimState:OverrideSymbol("rose", "charlie_lectern", "")
	end	
end

local function OnLongUpdate(inst, dt)
    if inst.task and inst.taskinfo then
        local new_time = math.max(inst:TimeRemainingInTask(inst.taskinfo) - dt, 0)

        inst.task:Cancel()
        inst.task = nil 
        inst.taskinfo = nil

        inst.task, inst.taskinfo = inst:ResumeTask(new_time, function() Restore(inst) end)
    end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()

    MakeObstaclePhysics(inst, 0.2)
	
    inst:AddTag("charlie_lecturn")
    inst:AddTag("pugalisk_avoids")
	inst:AddTag("shadow_destroy_immune")

    inst.MiniMapEntity:SetIcon("charlie_playbill.tex")

    inst.AnimState:SetBank("charlie_lectern")
    inst.AnimState:SetBuild("charlie_lectern")
    inst.AnimState:PlayAnimation("idle")

    inst:AddComponent("inspectable")

    inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"papyrus"})

    MakeSnowCovered(inst)

    inst.checkidleanim = checkidleanim
	inst.HasBook = true
	inst.SetTask = SetTask

    inst:DoTaskInTime(math.random()*3 + 2, checkidleanim)
	
	if GetPlayer() and GetPlayer().prefab == "charlie" then
	    inst:AddComponent("activatable")
        inst.components.activatable.OnActivate = flippage
        inst.components.activatable.inactive = true
	    inst.components.activatable.standingaction = true

	    inst:AddComponent("pickable")
	    inst.components.pickable:SetUp("rose_inv", TUNING.TOTAL_DAY_TIME*15) 
	    inst.components.pickable.picksound = "dontstarve/wilson/pickup_reeds" 
        inst.components.pickable.onpickedfn = onpickedfn 
        inst.components.pickable.onregenfn = onregenfn 
	    inst.components.pickable.caninteractwith = false
	end

    inst.OnSave = onsave
    inst.OnLoad = onload
	inst.OnLongUpdate = OnLongUpdate

    return inst
end

return Prefab("charlie_lecturn", fn, assets, prefabs)    
