local assets =
{
    Asset("ANIM", "anim/charlie_basic.zip"),
}

local prefabs =
{
    
}
--[[
local KNOWS_CHARLIE_LOOKUP = {
    winona  = true,
    waxwell = true,
}
]]
local CAST_SOUND_NAME = "castloop"
local AMB_SOUND_NAME  = "amb"

--------------------------------------------------------------------------

local function EnableDynamicShadow(inst)
    inst.DynamicShadow:Enable(true)
end

local function DisableDynamicShadow(inst)
    inst.DynamicShadow:Enable(false)
end

local function EnableTrader(inst)
    inst.components.trader:Enable()
end

local function DisableTrader(inst)
    inst.components.trader:Disable()
end

local function PlayDespawnSound(inst)
    inst.SoundEmitter:PlaySound("dontstarve/ghost/ghost_howl") -- Despawn sound @amanda ("spawn_out" anim)
end

local function OnRemove(inst)
    inst.SoundEmitter:KillSound(AMB_SOUND_NAME)
    inst.SoundEmitter:KillSound(CAST_SOUND_NAME)

    -- Charliecutscene cmp save/load will handle this not running.
    --if inst.atrium ~= nil and inst.atrium.components.charliecutscene ~= nil then
    --    inst.atrium.components.charliecutscene:Finish()
    --end
end

--------------------------------------------------------------------------

local function StartCasting(inst, cast_time)
    inst.SoundEmitter:PlaySound("rifts2/charlie/casting_lp", CAST_SOUND_NAME) -- Cast sound loop @amanda ("cast_pre" + "cast_idle" anims)

    inst.AnimState:PlayAnimation("cast_pre")
    inst.AnimState:PushAnimation("cast_idle")
end

local function Despawn(inst)
    inst.SoundEmitter:KillSound(CAST_SOUND_NAME)

    inst.SoundEmitter:PlaySound("rifts2/charlie/casting_pst")

    inst.AnimState:PlayAnimation("cast_pst")
    inst.AnimState:PushAnimation("idle")
    inst.AnimState:PushAnimation("spawn_out")

    inst:DoTaskInTime((60 + 60) * FRAMES, PlayDespawnSound)
    inst:DoTaskInTime((60 + 60 + 45) * FRAMES, DisableDynamicShadow)
	DisableTrader(inst)

    local despawn_time = (60 + 60 + 100) * FRAMES -- Animation lengths.
    inst.removetask = inst:DoTaskInTime(despawn_time, inst.Remove)
end

local function StartCastingWithDelay(inst, delay, cast_time)
    inst:DoTaskInTime(delay, inst.StartCasting)
    inst:DoTaskInTime(delay + cast_time, inst.Despawn)
end

local function OnRefuse(inst, giver, item)
    if giver == GetPlayer() then
	    GetPlayer().HUD:Hide()
		GetPlayer().components.playercontroller:Enable(false)
		GetPlayer().components.health:SetInvincible(true)
	    if TUNING.LANGUAGE == true then
		    inst.components.talker:Say("Thanks, but what I need is a pure black thing.")
		elseif TUNING.LANGUAGE == false then
		    inst.components.talker:Say("谢谢，但是我需要的是一种黑色的纯粹东西。")
		end
		inst:DoTaskInTime(3.5, function()
	        if TUNING.LANGUAGE == true then
		        GetPlayer().components.talker:Say("Okay, I'll find it as soon as possible.")
		    elseif TUNING.LANGUAGE == false then
		        GetPlayer().components.talker:Say("好的，我会尽快去找的。")
		    end
		end)		
		inst:DoTaskInTime(4.5, function()
	        GetPlayer().HUD:Show()
		    GetPlayer().components.playercontroller:Enable(true)
			GetPlayer().components.health:SetInvincible(false)
		end)
	end
end

local REPAIR_GATE_ANIM_LENGTH = 114 * FRAMES
local CHARLIE_CAST_TIME = REPAIR_GATE_ANIM_LENGTH + 0.5 + (20 * FRAMES)

local function OnAccept(inst, giver, item)
    if giver == GetPlayer() then
        local world = GetWorld()
		
        if world ~= nil then
			if world.components.charlienpcspawner ~= nil then
			    world.components.charlienpcspawner:Enable(false)
			end
			if world.components.shadowriftspawner ~= nil then
			    world.components.shadowriftspawner:Enable(true)
			end			
		end		
		
		inst.persists = false
		
	    GetPlayer().HUD:Hide()
		GetPlayer().components.playercontroller:Enable(false)
		GetPlayer().components.health:SetInvincible(true)
		inst:StartCasting()
		inst:DoTaskInTime(CHARLIE_CAST_TIME, inst.Despawn)	
	    if TUNING.LANGUAGE == true then
		    inst.components.talker:Say("Thanks, my plan has made new progress.")
		elseif TUNING.LANGUAGE == false then
		    inst.components.talker:Say("谢谢，我的计划有了新的进展。")
		end		
		inst:DoTaskInTime(2.75, function()
	        if TUNING.LANGUAGE == true then
		        inst.components.talker:Say("More interesting places are waiting for you to explore on your own.")
		    elseif TUNING.LANGUAGE == false then
		        inst.components.talker:Say("更多有趣的地方就等待你自己去探索吧。")
		    end
		end)			
		GetPlayer():DoTaskInTime(7, function()
	        if TUNING.LANGUAGE == true then
		        GetPlayer().components.talker:Say("Okay, goodbye!")
		    elseif TUNING.LANGUAGE == false then
		        GetPlayer().components.talker:Say("好的，再见！")
		    end
		end)		
		GetPlayer():DoTaskInTime(8, function()
	        GetPlayer().HUD:Show()
		    GetPlayer().components.playercontroller:Enable(true)
			GetPlayer().components.health:SetInvincible(false)
            GetPlayer().components.inventory:GiveItem(SpawnPrefab("rose_inv"), nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
		end)	
	end
end

local function AcceptTest(inst, item)
    return item ~= nil and item.prefab ~= nil and item.prefab == "horrorfuel"
end

--------------------------------------------------------------------------
--[[
local function DisplayNameFn(inst)
    if ThePlayer and KNOWS_CHARLIE_LOOKUP[ThePlayer.prefab] then
        return STRINGS.NAMES[string.upper(inst.prefab)]
    else
        return STRINGS.NAMES[string.upper(inst.prefab.."_ALT")]
    end
end
]]
--------------------------------------------------------------------------

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()
	inst.entity:AddMiniMapEntity()

    MakeObstaclePhysics(inst, 0.5)
	
	inst.MiniMapEntity:SetIcon("charlie_npc.tex")

    inst.DynamicShadow:SetSize(3, 2)
    inst.DynamicShadow:Enable(false)

    inst.Transform:SetTwoFaced()

	inst:AddTag("shadow_destroy_immune")
    inst:AddTag("pugalisk_avoids")	
    inst:AddTag("charlie_npc")
	
    inst.AnimState:SetBank("charlie_basic")
    inst.AnimState:SetBuild("charlie_basic")
    inst.AnimState:PlayAnimation("spawn")
    inst.AnimState:PushAnimation("idle", true)

    --inst.displaynamefn = DisplayNameFn

    inst.StartCasting = StartCasting
    inst.Despawn = Despawn
    inst.StartCastingWithDelay = StartCastingWithDelay

    inst:AddComponent("inspectable")

    --inst.persists = false

    inst.OnRemoveEntity = OnRemove

    inst:DoTaskInTime(22*FRAMES, EnableDynamicShadow)
	inst:DoTaskInTime(25*FRAMES, EnableTrader)

    inst.SoundEmitter:PlaySound("dontstarve/ghost/ghost_howl") -- Spawn sound @amanda. Plays when spawned ("spawn" anim).
    inst.SoundEmitter:PlaySound("rifts2/charlie/charlie_amb", AMB_SOUND_NAME)
    inst.SoundEmitter:SetVolume(AMB_SOUND_NAME, 0.2)
	
	inst:AddComponent("talker")
    inst.components.talker.colour = Vector3(0.9, 0.29, 0.29, 1)
    inst.components.talker.offset = Vector3(0, -700, 0)
	
	inst:AddComponent("trader")
	inst.components.trader.acceptnontradable = true
	inst.components.trader.onrefuse = OnRefuse
	inst.components.trader.onaccept = OnAccept
    inst.components.trader:SetAcceptTest(AcceptTest)

    DisableTrader(inst)

    return inst
end

return Prefab("charlie_npc", fn, assets, prefabs)
