require("prefabutil")

local assets_stone =
{
    Asset("ANIM", "anim/portal_stone.zip"),
}

local assets_fx =
{
    Asset("ANIM", "anim/portal_stone.zip"),
}

local prefabs =
{

}

local function MakeActive(inst)
    if inst.sg ~= nil then
	    inst.sg:GoToState("spawn_pre")
	end
end

local function MakeUsed(inst)
    if inst.sg ~= nil then
	    inst.sg:GoToState("spawn_pre")
	end
end

local function DoResurrect(inst, dude)
    if inst.sg ~= nil then
	    inst.sg:GoToState("spawn_pre")
	end
    inst.components.colourtweener:StartTween({ 1, 1, 1, 1 }, 14 * FRAMES)
	inst:AddTag("busy")	

    if dude.components.poisonable and dude.components.poisonable:IsPoisoned() then 
		dude.components.poisonable:Cure()
	end 

	if GetWorld():IsCave() or GetClock():IsNight() then
	    inst.rememberlight = true
		inst.Light:Enable(true)
	end
	
    dude.Transform:SetPosition(inst.Transform:GetWorldPosition())
    dude:Hide()
    TheCamera:SetDistance(12)
	if dude.components.hunger then
	    dude.components.hunger:Pause()
	end

	if dude.components.driver then
		if dude.components.driver:GetIsDriving() then 
			dude.components.driver:OnDismount()
		end 
	end 
	
    scheduler:ExecuteInTime(3, function()
        dude:Show()
        if inst.sg ~= nil then
	        inst.sg:GoToState("spawn_pre")
    	end
        inst.components.colourtweener:StartTween({ 1, 1, 1, 1 }, 14 * FRAMES)
        
        if dude.components.hunger then
            dude.components.hunger:SetPercent(1)
			dude.components.hunger:Resume()
        end

        if dude.components.health then
            dude.components.health:Respawn(500)
        end
        
        if dude.components.sanity then
			dude.components.sanity:SetPercent(1)
        end

        if dude.components.moisture then
        	dude.components.moisture.moisture = 0
        end

        if dude.components.temperature then
        	dude.components.temperature:SetTemperature(35)
        end
        
        dude.sg:GoToState("wakeup")
        inst:RemoveTag("busy")

        if dude:HasTag("charlie") and not TheSim:FindFirstEntityWithTag("spawnprotectionbuff") then
		    if not dude.components.timer then
			    dude:AddComponent("timer")
			end
		    dude.components.timer:StartTimer("despawn_protection", 60)
			local protection = SpawnPrefab("spawnprotectionbuff")
			dude:AddChild(protection)
			if protection.OnEnableProtectionFn ~= nil then
			    protection:OnEnableProtectionFn(dude, true)
			end
		end
		
        dude:DoTaskInTime(3, function(inst) 
	        if dude.HUD then
	            dude.HUD:Show()
	        end
	        TheCamera:SetDefault()           
        end)
        
    end)
	
	inst:DoTaskInTime(0, function()
	    print("charlie_portal resurrector refresh")
        if inst.components.resurrector == nil then					
			inst:AddComponent("resurrector")  
		end
		inst.components.resurrector.used = nil
		inst.components.resurrector.active = true
        inst.components.resurrector:OnBuilt(nil)
	end)

	if IsDLCEnabled(PORKLAND_DLC) then      
	
	    dude:DoTaskInTime(0, function()
		    local snapcam = true

		    -- Do not transition to the same room.
		    if (TheCamera.interior and not inst:GetIsInInterior()) or inst.interior then
			    GetInteriorSpawner():PlayTransition(dude, nil, inst.interior, inst, true)
			    snapcam = false
		    end

		    dude.Transform:SetPosition(inst.Transform:GetWorldPosition())

		    if snapcam then
			    TheCamera:Snap()
			    TheFrontEnd:DoFadeIn(1)
		    end

		    if not TheCamera.interior then
			    TheCamera:SetDistance(12)	
		    end
	    end)		
		
	end

end

local function OnHit(inst, worker, workleft)
    if worker ~= nil and worker:HasTag("prospective_throne_owner") then
        if inst.sg ~= nil then
	        inst.sg:GoToState("spawn_pre")
	    end
	    if workleft <= 0 then
	        inst.components.lootdropper:DropLoot()
	        SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	        inst.SoundEmitter:PlaySound("dontstarve/common/destroy_stone")	
	        inst:Remove()		
		end
	else
	    inst.components.workable:SetWorkLeft(20)
	end
end

local function OnBuilt(inst)
    if inst.sg ~= nil then
	    inst.sg:GoToState("spawn_pre")
	end
end

local function OnRemove(inst)
    if SaveGameIndex then
        SaveGameIndex:DeregisterResurrector(inst)
    end
end

local function RemoveOnInvalidGround(inst)
    if not inst:IsOnValidGround() then
	    inst:Remove()
	end
	if inst.GetIsOnWater ~= nil and inst:GetIsOnWater() then
	    inst:Remove()
	end
end

local function OnNewDay(inst)
    inst:DoTaskInTime(0, function()
        if not GetWorld():IsCave() then
	        inst.rememberlight = nil
            inst.Light:Enable(false)
	    end
	end)
end

local function OnPick(inst, picker)
    inst.AnimState:OverrideSymbol("rose", "portal_stone", "")
end

local function OnRegen(inst)
    inst.AnimState:OverrideSymbol("rose", "portal_stone", "rose")
end

local function OnSave(inst, data)
    data.rememberlight = inst.rememberlight or nil
end

local function OnLoad(inst, data)
    if data ~= nil and data.rememberlight then
	    inst.rememberlight = true
		inst.Light:Enable(true)
	end
	if inst.components.pickable ~= nil and inst.components.pickable.hasbeenpicked then
        inst.AnimState:OverrideSymbol("rose", "portal_stone", "")
	end
end

local function MakePortal(name, bank, build, assets, prefabs)
    local function fn()
        local inst = CreateEntity()
		
        inst.entity:AddTransform()
        inst.entity:AddAnimState()
        inst.entity:AddSoundEmitter()
        inst.entity:AddMiniMapEntity()
        inst.entity:AddLight()

        if TheSim:FindFirstEntityWithTag("charlie_portal") and TheSim:FindFirstEntityWithTag("charlie_portal") ~= inst then
		    --print("charlie_portal can only exist one per world!")
		    inst:Remove()
			return inst
		else
 
            inst.AnimState:SetBank(bank)
            inst.AnimState:SetBuild(build)
            inst.AnimState:PlayAnimation("idle_loop", true)
		
            inst.MiniMapEntity:SetIcon("charlie_portal.tex")

            inst.Light:Enable(false)
            inst.Light:SetRadius(1)
            inst.Light:SetFalloff(0.3)   
            inst.Light:SetIntensity(0.75)   
            inst.Light:SetColour(180/255, 195/255, 150/255)       

            inst:AddTag("charlie_portal")
			inst:AddTag("pugalisk_avoids")

            inst:SetStateGraph("SGcharlieportal")

            inst:AddComponent("colourtweener")
			
            inst:AddComponent("lootdropper")
		
            inst:AddComponent("inspectable")
            inst.components.inspectable:RecordViews()

	        inst:AddComponent("resurrector")
            inst.components.resurrector:OnBuilt(nil)
		    inst.components.resurrector.active = true
	        inst.components.resurrector.doresurrect = DoResurrect
		
		    if GetPlayer():HasTag("prospective_throne_owner") then
		        inst:AddComponent("workable")
	            inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	            inst.components.workable:SetWorkLeft(20)
	            inst.components.workable:SetOnWorkCallback(OnHit)

	            inst:AddComponent("pickable")
	            inst.components.pickable:SetUp("rose_inv", TUNING.TOTAL_DAY_TIME*10) 
	            inst.components.pickable.picksound = "dontstarve/wilson/pickup_reeds" 
                inst.components.pickable.onpickedfn = OnPick 
                inst.components.pickable.onregenfn = OnRegen 
            end				

            inst:ListenForEvent("daytime", function() OnNewDay(inst) end, GetWorld())
            inst:ListenForEvent("onbuilt", OnBuilt)
		    inst:ListenForEvent("onremove", OnRemove)

            inst:DoTaskInTime(0, function()
			    --Don't want it to attract anything
                if inst:HasTag("lightsource") then       
                    inst:RemoveTag("lightsource")    
                end
				RemoveOnInvalidGround(inst)
            end)
			
	        inst.OnSave = OnSave
            inst.OnLoad = OnLoad			

            return inst	
		end

    end

    return Prefab(name, fn, assets, prefabs)
end

return MakePortal("charlie_portal", "portal_dst", "portal_stone", assets_stone, prefabs),
    MakePlacer("charlie_portal_placer", "portal_dst", "portal_stone", "idle_loop")