
local assets =
{
    Asset("ANIM", "anim/charlie_stage.zip"),
}

local postassets =
{
    Asset("ANIM", "anim/charlie_curtains.zip"),
}

local seatassets =
{
    Asset("ANIM", "anim/charlie_seat.zip"),
}

local prefabs =
{
    "charlie_lecturn",
    "charlie_seat",
    "charlie_stage",
    "charlie_stage_lip",
}

local function Vector3FromTheta(theta, radius)
    local radius = radius or 1
    return Vector3(radius * math.cos(theta), 0, -radius * math.sin(theta))
end

local function spawn_stage_prefab(inst, angle, dist, prefab)
    local stage_position = inst:GetPosition()

    local offset = FindWalkableOffset(stage_position, angle, dist, nil, false, true)
        or Vector3FromTheta(angle, dist)

    local pt = stage_position
	if offset then
	    pt = stage_position + offset
	end
	
    if offset and ( inst.IsPosSurroundedByLand == nil or inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 3) ) then
        local prefab_instance = SpawnPrefab(prefab)
        prefab_instance.Transform:SetPosition((stage_position + offset):Get())
	end

end

local function setup(inst)
    if not inst.loaded then
        local x, y, z = inst.Transform:GetWorldPosition()
        local lecturn = SpawnPrefab("charlie_lecturn")
        local dist = 1.41 * 7.25
        local angle = -math.pi/6
        local offset = Vector3(dist * math.cos( angle ), 0, -dist * math.sin( angle ))
        lecturn.Transform:SetPosition(x+offset.x,0,z+offset.z)

		local ents_to_destroy = TheSim:FindEntities(x, y, z, 15, nil, {"ground", "player", "irreplaceable", "invisible_collision_box", "INLIMBO", "INTERIOR_LIMBO", "FX", "DECOR", "NOCLICK"})
		for k,v in pairs(ents_to_destroy) do
		    if v ~= inst and v:IsValid() and v.entity:IsVisible() and v.prefab ~= nil
			    and v.prefab ~= "charlie_lecturn" and v.prefab ~= "charlie_stage_lip" and v.prefab ~= "deflated_balloon"
				and v.prefab ~= "charlie_stage" and v.prefab ~= "charlie_seat" and v.prefab ~= "deflated_balloon_basket"
				and v.parent == nil and v.components.placer == nil
				and v.prefab ~= "cave_entrance"
				and v.components.door == nil and (v.components.activatable == nil or v.prefab == "sandhill")
			then
			    v:Remove()
				print("charlie stage removed", v.prefab)
			end
		end	
		local ferns = TheSim:FindEntities(x, y, z, 15, {"fern_plant"})
		for i,j in pairs(ferns) do
		    if j.prefab == "deep_jungle_fern_noise_plant" then
			    j:Remove()
				print("charlie stage removed", j.prefab)
			end
		end
    end

    --inst.lip = SpawnPrefab("charlie_stage_lip")
    --inst:AddChild(inst.lip)

    inst.floor = SpawnPrefab("charlie_stage")
    inst:AddChild(inst.floor)

    local increment = (math.pi/7)
    spawn_stage_prefab(inst,  -math.pi/4 + (increment*0.5),  6, "charlie_seat")
    spawn_stage_prefab(inst,  -math.pi/4 + (increment*1.5),  6, "charlie_seat")
    spawn_stage_prefab(inst,  -math.pi/4 - (increment*1.5),  6, "charlie_seat")
    spawn_stage_prefab(inst,  -math.pi/4 - (increment*0.5),  6, "charlie_seat")

end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()

    inst.AnimState:SetBank("charlie_stage")
    inst.AnimState:SetBuild("charlie_Stage")
    inst.AnimState:PlayAnimation("idle")

    inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
    inst.AnimState:SetFinalOffset(1)
    inst.AnimState:SetSortOrder(3)

    inst:AddTag("stage")
    inst:AddTag("DECOR")
    inst:AddTag("NOCLICK")
    inst:AddTag("NOBLOCK")
	inst:AddTag("pugalisk_avoids")

    inst.persists = false

    return inst
end

local function lipfn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()

    inst.AnimState:SetBank("charlie_stage")
    inst.AnimState:SetBuild("charlie_stage")
    inst.AnimState:PlayAnimation("lip")
    inst.AnimState:SetFinalOffset(0)
    inst.AnimState:SetOrientation(ANIM_ORIENTATION.Default)
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
    inst.AnimState:SetSortOrder(3) 

    inst.Transform:SetRotation(90)	
    inst.Transform:SetScale(0.98,0.98,0.98)

    inst:AddTag("DECOR")
    inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
	inst:AddTag("pugalisk_avoids")

    inst.persists = false

    inst:Hide()
	
    return inst
end

local function SpawnThingsNearby(inst)
    local possible_mobs_to_spawn = { "hound", "spider", "spider_warrior", "frog", "mosquito", "merm", "pigman", "pigguard" }
	local possible_mobs_to_spawn_in_winter = { "hound", "icehound", "penguin", "rabbit" }
	local possible_mobs_to_spawn_in_summer = { "hound", "firehound", "houndfire", "spider", "spider_warrior", "frog", "mosquito", "merm", "pigman", "pigguard" }
    local sw_possible_mobs_to_spawn = { "crocodog", "spider", "spider_warrior", "merm", "wildbore", "snake", "snake_poison" }
	local sw_possible_mobs_to_spawn_in_wet_or_green_season = { "crocodog", "watercrocodog", "spider", "spider_warrior", "mosquito_poison", "merm", "wildbore", "snake", "snake_poison" }
	local sw_possible_mobs_to_spawn_in_dry_season = { "crocodog", "poisoncrocodog", "spider", "spider_warrior", "snake", "snake_poison", "dragoon" }
	local ham_possible_mobs_to_spawn = { "vampirebat", "mean_flytrap", "adult_flytrap", "snake_amphibious", "frog_poison", "mosquito", "hippopotamoose" }
    local things_to_spawn = GetRandomItem(possible_mobs_to_spawn)
	
	if GetWorld() then
		if GetWorld():HasTag("shipwrecked") then
            things_to_spawn = GetRandomItem(sw_possible_mobs_to_spawn)
        elseif GetWorld():HasTag("porkland") then
		    things_to_spawn = GetRandomItem(ham_possible_mobs_to_spawn)
		end		
	end
	local amount = math.random(2,6)
    local pt = inst:GetPosition()
    local range = 1 + math.random()*3
    local angle = math.random()*2*math.pi
	local offset = FindWalkableOffset(pt, angle, range, 10)
    if offset then
        local newpoint = pt+offset
        for i=1, amount do
            range = math.random()*2
            angle = math.random()*2*math.pi
            local suboffset = FindWalkableOffset(newpoint, angle, range, 10)
            local spawnpt = newpoint + suboffset
	
	        if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)) and not GetWorld():HasTag("porkland") then
	            if not GetWorld():HasTag("shipwrecked") and GetSeasonManager():IsWinter() then
		            things_to_spawn = GetRandomItem(possible_mobs_to_spawn_in_winter)
		        elseif not GetWorld():HasTag("shipwrecked") and GetSeasonManager():IsSummer() then
		            things_to_spawn = GetRandomItem(possible_mobs_to_spawn_in_summer)
		        elseif GetWorld():HasTag("shipwrecked") and (GetSeasonManager():IsWetSeason() or GetSeasonManager():IsGreenSeason()) then
		            things_to_spawn = GetRandomItem(sw_possible_mobs_to_spawn_in_wet_or_green_season)
                elseif GetWorld():HasTag("shipwrecked") and GetSeasonManager():IsDrySeason() then
		            things_to_spawn = GetRandomItem(sw_possible_mobs_to_spawn_in_dry_season)
		        end
	        elseif not GetWorld():HasTag("porkland") then
	            if GetSeasonManager():IsWinter() then
		            things_to_spawn = GetRandomItem(possible_mobs_to_spawn_in_winter)
		        elseif GetSeasonManager():IsSummer() then
		            things_to_spawn = GetRandomItem(possible_mobs_to_spawn_in_summer)
                end			
	        end			
			
            local to_spawn = SpawnPrefab(things_to_spawn)		
            to_spawn.Transform:SetPosition(spawnpt.x, 0, spawnpt.z)
			SpawnPrefab("small_puff").Transform:SetPosition(spawnpt.x, 0, spawnpt.z)
        end
	end
end

local function OpenStage(inst)
    if not inst.IsOpen then
        if inst.components.activatable ~= nil then
            inst.components.activatable.inactive = true
        end
		if inst.components.pickable ~= nil then
		    inst.components.pickable.caninteractwith = false
		end
	    inst.IsOpen = true
        if not inst:IsAsleep() and not inst:IsInLimbo() then
		    inst.AnimState:PlayAnimation("curtain_open")
	        inst.AnimState:PushAnimation("idle_open")
		else
	        inst.AnimState:PlayAnimation("idle_open")
	    end
		if inst.task then 
		    inst.task:Cancel() 
		    inst.task = nil  
	    end	
        if inst.taskinfo then
            inst.taskinfo = nil 
        end			
	end
end

local function SetOpenTask(inst, time)
	if inst.task then 
		inst.task:Cancel() 
		inst.task = nil  
	end
    if inst.taskinfo then 
        inst.taskinfo = nil 
    end		
	if not time then 
		time = TUNING.TOTAL_DAY_TIME*2 + (math.random()*TUNING.TOTAL_DAY_TIME)
	end
	inst.task, inst.taskinfo = inst:ResumeTask( time, function() OpenStage(inst) end )  
end

local function checkidleanim(inst)
    if inst.AnimState:IsCurrentAnimation("idle_closed") then
        inst.AnimState:PlayAnimation("idle_closed"..math.random(2,5))
        inst.AnimState:PushAnimation("idle_closed")	
	elseif inst.AnimState:IsCurrentAnimation("idle_open") then
	    local anims = {"idle_open2", "idle_open3", "idle_open4", "narrate", "narrator_off", "narrator_on", "stinger"}
        inst.AnimState:PlayAnimation(GetRandomItem(anims))
        inst.AnimState:PushAnimation("idle_open")	
    end
    inst:DoTaskInTime(math.random()*3 + 2, checkidleanim)
end

local function OnActivate(inst, doer)
 
	if doer == GetPlayer() then
	    GetPlayer().HUD:Hide()
	    GetPlayer().components.playercontroller:Enable(false)
	    GetPlayer().components.health:SetInvincible(true)
	end

	inst:DoTaskInTime(0.5, function()
	    if doer == GetPlayer() then
	        if TUNING.LANGUAGE == true then
		        GetPlayer().components.talker:Say("I am Charlie and I come from the W family. I am looking for Maxy.")
		    elseif TUNING.LANGUAGE == false then
		        GetPlayer().components.talker:Say("我是查理，我来自W家族。我正在寻找小麦。")
		    end
		end
	end)	
	
	inst:DoTaskInTime(3.5, function()
	    if doer == GetPlayer() then
	        if TUNING.LANGUAGE == true then
		        GetPlayer().components.talker:Say("I vaguely remember that accident... Maxy, what exactly did you do...")
		    elseif TUNING.LANGUAGE == false then
		        GetPlayer().components.talker:Say("我隐隐约约还记得那次事故......小麦，你到底做了什么......")
		    end
		end
	end)	

	inst:DoTaskInTime(6.5, function()
	    if doer == GetPlayer() then
	        if TUNING.LANGUAGE == true then
		        GetPlayer().components.talker:Say("And then... I woke up in this dark forest... The monsters here are so scary...")
		    elseif TUNING.LANGUAGE == false then
		        GetPlayer().components.talker:Say("然后......我在这黑暗森林里醒来了......这里的怪物好可怕......")
		    end
		end
	end)	
	
	inst:DoTaskInTime(9.5, function()
	    if doer == GetPlayer() then
	        if TUNING.LANGUAGE == true then
		        GetPlayer().components.talker:Say("Maxy... Where in heaven are you... You said...")
		    elseif TUNING.LANGUAGE == false then
		        GetPlayer().components.talker:Say("小麦......你究竟在哪里......你说过......")
		    end
		end
	end)	
	
	inst:DoTaskInTime(12, function()
	    if doer == GetPlayer() then
	        GetPlayer().HUD:Show()
		    GetPlayer().components.playercontroller:Enable(true)
			GetPlayer().components.health:SetInvincible(false)
		end
		
        if inst.IsOpen then
	        inst.IsOpen = false
            inst.AnimState:PlayAnimation("close")
            inst.AnimState:PushAnimation("idle_closed")
		    inst:SetOpenTask()
			if inst.components.pickable ~= nil then
			    inst.components.pickable.caninteractwith = true
			end
    	end	

        if math.random() >= 0.3 then
            local x, _, z = inst.Transform:GetWorldPosition()
            local x_dist = 2 + 3 * math.random()        
	        local z_dist = 1 + 4 * math.random()        
		    local subtheta = math.random() * 2 * math.pi
            local spawn_x, spawn_z
                spawn_x, spawn_z = x + math.cos(subtheta) * x_dist, z + math.sin(subtheta) * z_dist	
	        local BlockerNearby = TheSim:FindEntities(spawn_x, 0, spawn_z, 0.3, nil, {"NOCLICK", "FX", "player"})
	        if #BlockerNearby <= 0 then					
			    inst.components.lootdropper:DropLoot(Vector3(spawn_x, 3, spawn_z))
		        SpawnPrefab("small_puff").Transform:SetPosition(spawn_x, 3, spawn_z)
		    end
        else
	        SpawnThingsNearby(inst)	
	    end  		
	end)	
	
end

local function onpickedfn(inst, picker)
    inst.AnimState:OverrideSymbol("rose", "charlie_curtains", "")
end

local function onregenfn(inst)
    inst.AnimState:OverrideSymbol("rose", "charlie_curtains", "rose")
end

local function onsave(inst,data)
    data.loaded = true
	data.IsOpen = inst.IsOpen
    if inst.task and inst.taskinfo then
        data.timeleft = inst:TimeRemainingInTask(inst.taskinfo)
    end	
end

local function onload(inst,data)
    if data and data.loaded then
        inst.loaded = data.loaded
    end
    if data and data.IsOpen ~= nil then
        inst.IsOpen = data.IsOpen
		if inst.IsOpen == true then
		    inst.AnimState:PlayAnimation("idle_open")
		else
		    inst.AnimState:PlayAnimation("idle_closed")
		end
    end	
	if data and data.timeleft then
	    inst:SetOpenTask(data.timeleft)		
        if inst.components.activatable ~= nil then
            inst.components.activatable.inactive = false
        end
	end 		
	if inst.IsOpen then
        if inst.components.activatable ~= nil then
            inst.components.activatable.inactive = true
        end	
	end
    if inst.components.pickable ~= nil and inst.components.pickable.hasbeenpicked then
	    inst.AnimState:OverrideSymbol("rose", "charlie_curtains", "")
	end	
end

local function OnLongUpdate(inst, dt)
    if inst.task and inst.taskinfo then
        local new_time = math.max(inst:TimeRemainingInTask(inst.taskinfo) - dt, 0)

        inst.task:Cancel()
        inst.task = nil 
        inst.taskinfo = nil

        inst.task, inst.taskinfo = inst:ResumeTask(new_time, function() OpenStage(inst) end)
    end
end

local function postfn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()

    inst.AnimState:SetBank("charlie_curtains")
    inst.AnimState:SetBuild("charlie_curtains")
    inst.AnimState:PlayAnimation("idle_open")
	
	inst:AddTag("charlie_stage")
	inst:AddTag("pugalisk_avoids")

    inst.MiniMapEntity:SetIcon("charlie_stage.tex")

    inst:AddComponent("inspectable")

    inst.OnSave = onsave
    inst.OnLoad = onload
	inst.OnLongUpdate = OnLongUpdate

    inst:DoTaskInTime(0, setup)

    inst.checkidleanim = checkidleanim
	inst:DoTaskInTime(math.random()*3 + 2, checkidleanim)

    if GetPlayer() and GetPlayer().prefab == "charlie" then
	    inst:AddComponent("activatable")
        inst.components.activatable.OnActivate = OnActivate
        inst.components.activatable.inactive = true
	    inst.components.activatable.standingaction = true

	    inst:AddComponent("pickable")
	    inst.components.pickable:SetUp("rose_inv", TUNING.TOTAL_DAY_TIME*12) 
	    inst.components.pickable.picksound = "dontstarve/wilson/pickup_reeds" 
        inst.components.pickable.onpickedfn = onpickedfn 
        inst.components.pickable.onregenfn = onregenfn 
	    inst.components.pickable.caninteractwith = false
	end

	inst.IsOpen = true
	
	inst.SetOpenTask = SetOpenTask

    inst:AddComponent("lootdropper")
	inst.components.lootdropper.speed = 5  --TO DO: MAKE VANILLA AND ROG CHECK THIS
	inst.components.lootdropper:AddRandomLoot("thulecite", 1)
	inst.components.lootdropper:AddRandomLoot("thulecite_pieces", 1)
	inst.components.lootdropper:AddRandomLoot("orangegem", 2)
	inst.components.lootdropper:AddRandomLoot("yellowgem", 2)
	inst.components.lootdropper:AddRandomLoot("greengem", 2)
	inst.components.lootdropper:AddRandomLoot("bluegem", 4)
	inst.components.lootdropper:AddRandomLoot("redgem", 4)
	inst.components.lootdropper:AddRandomLoot("purplegem", 4)
	inst.components.lootdropper:AddRandomLoot("nightmarefuel", 5)
	inst.components.lootdropper:AddRandomLoot("dug_berrybush_juicy", 5)
	inst.components.lootdropper:AddRandomLoot("dug_bananabush", 5)
	inst.components.lootdropper:AddRandomLoot("dug_monkeytail", 5)
	inst.components.lootdropper:AddRandomLoot("goldnugget", 10)
	inst.components.lootdropper:AddRandomLoot("marble", 30)
    inst.components.lootdropper.numrandomloot = 1
	
	if GetWorld():HasTag("shipwrecked") then
	    inst.components.lootdropper:AddRandomLoot("obsidian", 1)
	end
	
	if GetWorld():HasTag("porkland") then
	    inst.components.lootdropper:AddRandomLoot("iron", 1)
	end
	
    return inst
end

local function seatfn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()

    inst:AddTag("DECOR")
    inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
	inst:AddTag("pugalisk_avoids")

    inst.AnimState:SetBank("charlie_seat")
    inst.AnimState:SetBuild("charlie_seat")
    inst.AnimState:PlayAnimation("test")
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
    inst.AnimState:SetFinalOffset(1)
    inst.AnimState:SetSortOrder(3)

    inst.persists = false

    return inst
end

local function collision_box_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()

    inst:AddTag("NOBLOCK")
	inst:AddTag("invisible_collision_box")
	inst:AddTag("pugalisk_avoids")
	
    MakeObstaclePhysics(inst, 0.5)

    return inst
end

return Prefab("charlie_stage", fn, assets, prefabs),
       Prefab("charlie_stage_lip", lipfn, assets, prefabs),
       Prefab("charlie_stage_post", postfn, postassets, prefabs),
       Prefab("charlie_seat", seatfn, seatassets, prefabs),
	   Prefab("charlie_door_collision_box", collision_box_fn, assets, prefabs)