local assets =
{
	Asset("ANIM", "anim/maxwell_throne.zip"),
}

local prefabs = 
{
    "horrorfuel",
}

local flower_assets =
{
	Asset("ANIM", "anim/abigail_flower_rework.zip"),
}

local flower_prefabs = 
{
    "horrorfuel",
}

local horrorfuel_assets =
{
	Asset("ANIM", "anim/horrorfuel.zip"),
}

local caneflower_assets = 
{
	Asset("ANIM", "anim/cane_flower.zip"),
	Asset("ANIM", "anim/swap_cane_flower.zip"),
	Asset("ANIM", "anim/swap_cane.zip"),
}

local caneflowerfx_assets = 
{
    Asset("ANIM", "anim/wormwood_plant_fx.zip"),
}

local supertelestaff_assets =
{
	Asset("ANIM", "anim/telestaff_shadowbeam.zip"),
	Asset("ANIM", "anim/swap_telestaff_shadowbeam.zip"),
	Asset("ANIM", "anim/swap_staffs.zip"),
}

local purenightsword_assets =
{
	Asset("ANIM", "anim/nightsword_lightsbane.zip"),
	Asset("ANIM", "anim/swap_nightsword_lightsbane.zip"),
	Asset("ANIM", "anim/swap_nightmaresword.zip"),
}

local rift_assets = 
{
	Asset("ANIM", "anim/shadowrift_portal.zip"),
}

local rainbowsword_assets = 
{
	Asset("ANIM", "anim/spear_terraprisma.zip"),
	Asset("ANIM", "anim/swap_spear_terraprisma.zip"),
	Asset("ANIM", "anim/swap_spear.zip"),
}

local horrorfuel_prefabs = 
{

}

local caneflower_prefabs = 
{
    "cane_flower_fx",
}

local caneflowerfx_prefabs = 
{

}

local supertelestaff_prefabs = 
{

}

local purenightsword_prefabs =
{

}

local rift_prefabs = 
{

}

local rainbowsword_prefabs =
{

}

local AMBIENT_SOUND_PATH = "rifts2/shadow_rift/shadowrift_portal_allstage"
local AMBIENT_SOUND_LOOP_NAME = "shadowrift_portal_ambience"
local AMBIENT_SOUND_PARAM_NAME = "stage"
local AMBIENT_SOUND_STAGE_TO_INTENSITY = {0.1, 0.4, 0.7}
local PHYSICS_SIZE_BY_STAGE = {1.2, 2.2, 3.2}

local SHAKE_PARAMS_BY_STAGE = {
    {0.5, .01, .1, 50 },
    {1.0, .03, .2, 100},
    {1.5, .06, .3, 200},
}

local SHADELINGS_BY_STAGE = {1, 2, 3}

local BLOCKER_EXCLUDE_TAGS = {"FX", "INLIMBO"}
local function IsPointNearBlocker(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, BLOCKER_EXCLUDE_TAGS) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearPlayer(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, {"player"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearSomething(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"charlie_stage", "charlie_throne", "ruins_exit", "pugalisk_fountain"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearWallOrStructure(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"wall", "structure"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function OnRepaired(inst, doer, repair_item)
	if doer ~= nil and doer.SoundEmitter ~= nil then
	    doer.SoundEmitter:PlaySound("dontstarve/HUD/repair_clothing")
	end
end

local function FindSafeLocationToMove(inst)
	local centers = {}
    for i, node in ipairs( GetWorld().topology.nodes ) do
        if node.type ~= NODE_TYPE.Blank and node.type ~= NODE_TYPE.Blocker then
            local x = node.x
            local z = node.y
            local pos = Vector3(x, 0, z)
            local tilepos = Vector3( GetWorld().Map:GetTileCenterPoint(pos.x, pos.y, pos.z) )
            if IsPassableAtPoint(tilepos.x, tilepos.y, tilepos.z) 
			    and ( GetPlayer().IsPosSurroundedByLand == nil or GetPlayer():IsPosSurroundedByLand(tilepos.x, tilepos.y, tilepos.z, 3) )
				and not IsPointNearPlayer(tilepos, 150) 		
			    and not IsPointNearSomething(tilepos, 30) 	
				and not IsPointNearBlocker(tilepos, 5) 		
				and not IsPointNearWallOrStructure(tilepos, 10) 		
			then                                            
                table.insert(centers, {x = node.x, z = node.y})
            end
        end		
	end
    if #centers > 0 then
        local pos = centers[math.random(#centers)]
        return pos.x, 0, pos.z
    else
        return inst.Transform:GetWorldPosition()
    end
end

local function SpawnShadowFX(inst)
    SpawnPrefab("statue_transition").Transform:SetPosition(inst.Transform:GetWorldPosition())
    SpawnPrefab("statue_transition_2").Transform:SetPosition(inst.Transform:GetWorldPosition())
end

local function SpawnCircleNightmarefuel(inst, radius)
    local radius = radius or 10
    local pt = inst:GetPosition()
    local theta = math.random() * 2 * math.pi
    local numitems = math.pi*radius

    for i = 1, numitems do
        local offset = Vector3(radius * math.cos( theta ), 0, -radius * math.sin( theta ))
        local wander_point = pt + offset
		if IsPassableAtPoint(wander_point.x, wander_point.y, wander_point.z) then
            local spawn = SpawnPrefab("nightmarefuel")
            spawn.Transform:SetPosition( wander_point.x, wander_point.y, wander_point.z ) 
            if spawn.GetIsOnWater ~= nil and spawn:GetIsOnWater() then
		        if spawn.components.floatable ~= nil then
			        spawn.components.floatable:OnHitWater(true)
			    elseif spawn.components.inventoryitem ~= nil then
		            spawn.components.inventoryitem:OnHitWater()
		        end
            end
		end
        theta = theta - (2 * math.pi / numitems)
    end
end

local function PlaySleepSound(inst)
    inst.SoundEmitter:PlaySound("dontstarve/common/throne/playerappear")
end

local function OnHit(inst, worker, workleft)
    local world = GetWorld()
    if inst.CanBeDestroyed == true and inst.White == nil then
	    if workleft <= 0 then
	        inst.components.lootdropper:DropLoot()        
	        inst.SoundEmitter:PlaySound("dontstarve/common/destroy_stone")	
			inst.AnimState:PlayAnimation("death")
			inst.persists = false
            if world ~= nil then
			    if world.components.charliethronespawner ~= nil then
			        world.components.charliethronespawner:Enable(false)
				end
			    if world.components.charlienpcspawner ~= nil then
			        world.components.charlienpcspawner:Enable(true)
				end
			end
	        inst:ListenForEvent("animover", function()
			    SpawnShadowFX(inst)
                inst:Remove()
            end)			
		end
	else
	    inst.components.workable:SetWorkLeft(10)
		local pos = Vector3( inst.Transform:GetWorldPosition() )
		if worker ~= nil and worker.components.inventory ~= nil then
		    pos = Vector3( worker.Transform:GetWorldPosition() )
			local tool = worker.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
			if tool ~= nil and not tool:HasTag("irreplaceable") then
			    tool:Remove()
			end
		end
		GetSeasonManager():DoLightningStrike(pos)
	end
end

local function onsleep(inst, sleeper)

    if inst.White ~= nil then
	    if sleeper.components.talker ~= nil then
		    if TUNING.LANGUAGE == true then
		        sleeper.components.talker:Say("I can only do that when it's black.")
			elseif TUNING.LANGUAGE == false then
			    sleeper.components.talker:Say("我只能在它黑的时候那样做。")
			end
		end
		inst.AnimState:SetAddColour(1, 1, 1, 1)
	    return
	end
	
    if sleeper:HasTag("shadow_form_on") then
	    if sleeper.components.talker ~= nil then
		    if TUNING.LANGUAGE == true then
		        sleeper.components.talker:Say("I need to maintain my normal form.")
			elseif TUNING.LANGUAGE == false then
			    sleeper.components.talker:Say("我需要维持在普通的形态中。")
			end
		end
	    return
	end	
	
	inst.White = true
	
	if sleeper.components.health then
	    sleeper.components.health:SetInvincible(true)
	end
	
	if sleeper.components.playercontroller then
	    sleeper.components.playercontroller:Enable(false)
    end
	
	GetPlayer().HUD:Hide()
	TheFrontEnd:Fade(false, 1)
	
	SpawnShadowFX(inst)
	PlaySleepSound(inst)

	inst:DoTaskInTime(1.2, function() 
		
		GetPlayer().HUD:Show()
		TheFrontEnd:Fade(true, 1) 
		
		if sleeper.components.sanity then
			sleeper.components.sanity:DoDelta(-50)
		end
		
		if sleeper.components.hunger then
			sleeper.components.hunger:DoDelta(150, false, true)
		end
		
		if sleeper.components.health then
			sleeper.components.health:DoDelta(225, false, "charlie_throne", true)
			sleeper.components.health:SetInvincible(false)
		end
		
        if sleeper.components.playercontroller then
		    sleeper.components.playercontroller:Enable(true)
		end
		
		if sleeper.sg then
		    sleeper.sg:GoToState("wakeup")	
		end
		
		SpawnShadowFX(inst)
		PlaySleepSound(inst)
		
		inst.AnimState:SetAddColour(1, 1, 1, 1)
		inst.Light:Enable(true)
		if math.random() < 0.05 then
		    SpawnCircleNightmarefuel(inst, math.random(5,12))
		end
		
		inst.components.charlieshadowflowerspawner:DespawnAll()
		inst.components.charlieshadowflowerspawner.minionpositions = nil
		inst.components.charlieshadowflowerspawner:SpawnAll()
		
	end)
end

local function startwhitelonfullmoon(inst)
    if GetClock() and GetClock():GetMoonPhase() == "full" and not inst.White then
        inst.White = true
	    inst.AnimState:SetAddColour(1, 1, 1, 1)
		inst.Light:Enable(true)
		if math.random() < 0.05 then
		    SpawnCircleNightmarefuel(inst, math.random(5,12))
		end
	end
end

local function Unlock(inst)
    if not inst.CanBeDestroyed then
        inst.CanBeDestroyed = true
	end
end

local SHADOW_FLOWER_MUST_TAGS = { "charlie_shadow_flower" }
local SHADOW_FLOWER_CANT_TAGS = { "INLIMBO" }
local function TryToMove(inst)
    local find_any_shadow_flower = FindEntity(
	    inst,
		100, 
		function(guy) 
            return guy.components.combat ~= nil 
			    and guy.components.health ~= nil 
				and not guy.components.health:IsDead()
	            and guy:IsValid()
		        and guy.entity:IsVisible()
        end,
        SHADOW_FLOWER_MUST_TAGS,
        SHADOW_FLOWER_CANT_TAGS
	)
	if not find_any_shadow_flower then
	    SpawnShadowFX(inst)
		local x, y, z = FindSafeLocationToMove(inst)
		inst.Transform:SetPosition(x, y, z)	
	end
end

local function OnBlacken(inst)
    inst.White = nil
	inst.AnimState:SetAddColour(0, 0, 0, 0)  
    inst.Light:Enable(false)	
end

local function OnRemove(inst)
    inst.components.charlieshadowflowerspawner:DespawnAll()
end

local function Highlight(self)
    if self ~= nil and self.inst ~= nil then
	    if self.inst.White ~= nil then
		    self.inst.AnimState:SetAddColour(1, 1, 1, 1)
		elseif TheSim:GetLightAtPoint(self.inst.Transform:GetWorldPosition()) > TUNING.DARK_CUTOFF then
            self.highlit = true
	        self.highlight_add_colour_red = 0.2
            self.highlight_add_colour_green = 0.2
            self.highlight_add_colour_blue = 0.2
            self:ApplyColour() 		
		end
	end
end

local function UnHighlight(self)
    if self ~= nil and self.inst ~= nil then
	    if self.inst.White ~= nil then
		    self.inst.AnimState:SetAddColour(1, 1, 1, 1)
		else
            self.highlit = nil
	        self.highlight_add_colour_red = nil
            self.highlight_add_colour_green = nil
            self.highlight_add_colour_blue = nil
	        self:ApplyColour() 		
		end
	end
end

local function ApplyColour(self)
    if self ~= nil and self.inst ~= nil then
	    if self.inst.White ~= nil then
		    self.inst.AnimState:SetAddColour(1, 1, 1, 1)
		else
	        self.inst.AnimState:SetAddColour((self.highlight_add_colour_red or 0), (self.highlight_add_colour_green or 0), (self.highlight_add_colour_blue or 0), 1)
		end
	end
end

local function GetStatus(inst)
    return (inst.White ~= nil
        and "WHITE")
		or (inst.CanBeDestroyed ~= nil and "DESTROYABLE")
        or nil
end

local function OnEntityWake(inst)
	if inst.White then
		inst.AnimState:SetAddColour(1, 1, 1, 1)
	end
end

local function OnSave(inst, data)
    data.White = inst.White or nil
	data.CanBeDestroyed = inst.CanBeDestroyed or nil
end

local function OnLoad(inst, data)
    if data ~= nil then
	    if data.White then
		    inst.White = true
			inst.AnimState:SetAddColour(1, 1, 1, 1)
			inst.Light:Enable(true)
		end
		if data.CanBeDestroyed then
		    inst.CanBeDestroyed = true
		end
	end
end

local function fn()
    local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddDynamicShadow()
	inst.entity:AddMiniMapEntity()
	inst.entity:AddLight()
	
	MakeObstaclePhysics(inst, 2)
	
	inst.DynamicShadow:SetSize(3, 2)
	
	inst.MiniMapEntity:SetIcon("nightmare_throne.tex")
	
    inst.AnimState:SetBank("throne")
    inst.AnimState:SetBuild("maxwell_throne")
    inst.AnimState:PlayAnimation("idle")	

    inst.Light:Enable(false)
    inst.Light:SetRadius(1)
    inst.Light:SetFalloff(0.3)   
    inst.Light:SetIntensity(0.75)   
    inst.Light:SetColour(180/255, 195/255, 150/255)  

	inst:AddTag("charlie_throne")
	inst:AddTag("pugalisk_avoids")
	inst:AddTag("shadow_destroy_immune")
	inst:AddTag("blackenable")
	
	inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = GetStatus
	
    inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"cutstone","cutstone","nightmarefuel","nightmarefuel","nightmarefuel","nightmarefuel","nightmarefuel","horrorfuel"})

    inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(10)
	inst.components.workable:SetOnWorkCallback(OnHit)
	
	if GetPlayer() and GetPlayer():HasTag("prospective_throne_owner") then
	    inst:AddComponent("sleepingbag")
	    inst.components.sleepingbag.onsleep = onsleep
		
		inst:ListenForEvent("nighttime", function() startwhitelonfullmoon(inst) end, GetWorld())
	end
	
	inst:ListenForEvent("unlockcharliethrone", function() Unlock(inst) end, GetWorld())
	inst:ListenForEvent("trytomovecharliethrone", function() TryToMove(inst) end, GetWorld())	
    inst:ListenForEvent("onremove", OnRemove)
	
    inst:AddComponent("charlieshadowflowerspawner")
    inst.components.charlieshadowflowerspawner.miniontype = "charlie_shadow_flower"
    inst.components.charlieshadowflowerspawner.distancemodifier = 5
    inst.components.charlieshadowflowerspawner.maxminions = 45
	inst.components.charlieshadowflowerspawner.shouldspawn = false
    inst.components.charlieshadowflowerspawner.addpositiondelay = 0

	inst:AddComponent("highlight")
	inst.components.highlight.Highlight = Highlight
	inst.components.highlight.UnHighlight = UnHighlight
	inst.components.highlight.ApplyColour = ApplyColour
	
	inst.OnEntityWake = OnEntityWake
    inst.OnBlacken = OnBlacken
    inst.OnSave = OnSave
    inst.OnLoad = OnLoad
	
	inst:DoTaskInTime(0, function()
        if inst:HasTag("lightsource") then       
            inst:RemoveTag("lightsource")    
        end
	    if inst.White then
		    inst.AnimState:SetAddColour(1, 1, 1, 1)
		end
	end)
	
	return inst
end

local RETARGET_MUST_TAGS = { "hascombatcomponent" }
local RETARGET_CANT_TAGS = { "INLIMBO", "plantkin", "shadow_form_on", "plant_mob", "charlie_throne" }
local RETARGET_ONEOF_TAGS = { "character", "monster", "animal", "prey" }
local function RetargetFn(inst)
    return FindEntity(
	    inst,
		2, 
		function(guy) 
            return guy.components.combat ~= nil 
			    and guy.components.health ~= nil 
				and not guy.components.health:IsDead()
        end,
        RETARGET_MUST_TAGS,
        RETARGET_CANT_TAGS,
        RETARGET_ONEOF_TAGS		
	)
end

local function ShouldKeepTarget(inst, target)
    return target ~= nil 
	    and target:IsValid()
		and target.entity:IsVisible()
		and target.components.health ~= nil
		and not target.components.health:IsDead()
		and target:GetDistanceSqToInst(inst) < 16
		and inst.components.combat:CanTarget(target)
	    or false
end

local function OnAttacked(inst, data)  
    inst.SoundEmitter:PlaySound("dontstarve/common/ghost_spawn")
end

local function OnDeath(inst, data)  
    if inst.attacktask then
	    inst.attacktask:Cancel()
	    inst.attacktask = nil
	end
    inst.AnimState:PlayAnimation("level3_pst")
	inst.AnimState:PushAnimation("level0_loop")
    RemovePhysicsColliders(inst)            
    inst.components.lootdropper:DropLoot(Vector3(inst.Transform:GetWorldPosition()))     
	GetWorld():PushEvent("trytomovecharliethrone")
end

local function SpawnFromSomething(inst, data)  
    inst.AnimState:PlayAnimation("level3_pre")
	inst.AnimState:PushAnimation("level3_loop", true)
end

local function OnNewTarget(inst, data)
    inst.components.combat:DoAttack()
end

local function SanityAura(inst, observer)
    return not observer:HasTag("shadow_form_on") and -TUNING.SANITYAURA_MED or 0
end

local function flower_fn()
    local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddLight()
	
	MakeObstaclePhysics(inst, 0.25)	

    inst.AnimState:SetBank("abigail_flower_rework")
    inst.AnimState:SetBuild("abigail_flower_rework")
    inst.AnimState:PlayAnimation("level3_loop", true)
	inst.AnimState:SetMultColour(0.15, 0.15, 0.15, 0.75)
	
    inst.Light:Enable(true)
    inst.Light:SetRadius(0.5)
    inst.Light:SetFalloff(0.5)   
    inst.Light:SetIntensity(0.6)   
    inst.Light:SetColour(28/255, 0/255, 128/255)  

    inst:DoTaskInTime(0, function()
	    --Don't want it to attract anything
        if inst:HasTag("lightsource") then       
            inst:RemoveTag("lightsource")    
        end
    end)

	inst:AddTag("charlie_shadow_flower")
	inst:AddTag("pugalisk_avoids")
	inst:AddTag("shadow_destroy_immune")
	inst:AddTag("scarytoprey")
    inst:AddTag("plant_mob")
    inst:AddTag("veggie")
	inst:AddTag("flying")
	inst:AddTag("notraptrigger")

    inst:AddComponent("inspectable")
	
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(100)
    inst.components.health.destroytime = 0.5  --TO DO: MAKE VANILLA (NO DLCS) CHECK THIS

    inst:AddComponent("combat")
    inst.components.combat:SetAttackPeriod(1.25)
    inst.components.combat:SetRange(2)
    inst.components.combat:SetRetargetFunction(0.2, RetargetFn)
    inst.components.combat:SetKeepTargetFunction(ShouldKeepTarget)
    inst.components.combat:SetDefaultDamage(34)

    inst:AddComponent("lootdropper")
	inst.components.lootdropper:AddChanceLoot("nightmarefuel", 0.01)

	inst:AddComponent("sanityaura")
	inst.components.sanityaura.aurafn = SanityAura
	
	inst:ListenForEvent("attacked", OnAttacked)
	inst:ListenForEvent("death", OnDeath)
	inst:ListenForEvent("newcombattarget", OnNewTarget)
	inst:ListenForEvent("spawnfromsomething", SpawnFromSomething)
	
	inst.attacktask = inst:DoPeriodicTask(1.25, function()  --A ugly way as I don't want to create sg or brain...
	    inst.components.combat:DoAttack()	
	end)
	
	return inst
end

local function CalcTargetLightOverride(player)
	if player ~= nil then
		local sanity = player.components.sanity
		if sanity ~= nil then
			local k = sanity:GetPercent()
			if k < 0.6 then
				k = 1 - k / 0.6
				return k * k
			end
		end
	end
	return 0
end

local function UpdateLightOverride(inst, instant)
    local _player = GetPlayer()
	inst.targetlight = CalcTargetLightOverride(_player)
	inst.currentlight = instant and inst.targetlight or inst.targetlight * .1 + inst.currentlight * .9
	inst.AnimState:SetLightOverride(inst.currentlight)
end

local function OnEntityWake(inst)
	if not inst.task then
		inst.task = inst:DoPeriodicTask(1, UpdateLightOverride, math.random())
		UpdateLightOverride(inst, true)
	end
end

local function OnEntitySleep(inst)
	if inst.task then
		inst.task:Cancel()
		inst.task = nil
	end
end

local function CreateCore()
	local inst = CreateEntity()

	inst.entity:AddTransform()
	inst.entity:AddAnimState()

	inst:AddTag("FX")
	inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
	inst:AddTag("INTERIOR_LIMBO_IMMUNE")	

	inst.AnimState:SetBank("horrorfuel")
	inst.AnimState:SetBuild("horrorfuel")
	inst.AnimState:PlayAnimation("middle_loop", true)
	inst.AnimState:SetFinalOffset(1)

	inst.persists = false
	inst.currentlight = 0
	inst.targetlight = 0
	inst.OnEntityWake = OnEntityWake
	inst.OnEntitySleep = OnEntitySleep
	inst.OnRemoveEntity = OnEntitySleep

	return inst
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function OnlyForShadow(inst) 
    return (GetPlayer().components.transformable ~= nil and GetPlayer().components.transformable.shadowform) 
	    or (GetPlayer().shadowform ~= nil)
end

local CORROSION_MUST_ONEOF_TAGS = {"lightsource", "fire", "smolder"}
local CORROSION_CANT_TAGS = {"FX", "INLIMBO", "DECOR", "INTERIOR_LIMBO", "player", "absorb_immune", "wallsection", "fullwallsection", "furniture", "cornerpost", "centerlight", "swinglight", "lightrays", "anthill_inside", "shop_entrance", "ruins_entrance"}
local function Corrosion(inst) 
    GetPlayer().components.hunger:DoDelta(150)
    if GetPlayer().components.sanity.current == 0 then
		GetPlayer().components.inventory:DropEverything()
    else
	    GetPlayer().components.health:DoDelta(225)
	    GetPlayer().components.sanity:DoDelta(-15)
	end

    local pos = Vector3(GetPlayer().Transform:GetWorldPosition())
    local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, 50, nil, CORROSION_CANT_TAGS, CORROSION_MUST_ONEOF_TAGS)	
    for k,v in pairs(ents) do
	    if v.Light ~= nil and v.Light:IsEnabled() and v.components.burnable == nil then
		    print("Light Corrosion")
            --Removing v or v.Light will cause some serious problems, so this is not eternal
			v.Light:Enable(false)       --Have to make sure if this is reversible but do not make it enable immediately either
			if v.components.machine ~= nil then
			    v.components.machine:TurnOff()  --For lantern
			end
            if v.Transform ~= nil and v:IsValid() and GetPlayer():IsValid() then
	            SpawnPrefab("statue_transition").Transform:SetPosition(v.Transform:GetWorldPosition())
                SpawnPrefab("statue_transition_2").Transform:SetPosition(v.Transform:GetWorldPosition())
            end
		elseif v.components.burnable ~= nil then
			print("Fire Corrosion")
		    v.components.burnable:Extinguish()
            if v.Transform ~= nil and v:IsValid() and GetPlayer():IsValid() then
	            SpawnPrefab("statue_transition").Transform:SetPosition(v.Transform:GetWorldPosition())
                SpawnPrefab("statue_transition_2").Transform:SetPosition(v.Transform:GetWorldPosition())
            end
		end
    end
	
    if inst.components.stackable ~= nil then
        inst.components.stackable:Get():Remove()
    else
        inst:Remove()
    end
end

local function horrorfuel_fn()
    local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()	

    MakeInventoryPhysics(inst)
    
	inst.AnimState:SetBank("horrorfuel")
	inst.AnimState:SetBuild("horrorfuel")
	inst.AnimState:PlayAnimation("idle_loop", true)
	inst.AnimState:SetMultColour(1, 1, 1, 0.5) 

	inst:AddTag("darkitem")
	inst:AddTag("blackener")
    inst:AddTag("shadowupgrader")
	
    inst:AddComponent("stackable")
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM
 
    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")
	
    inst:AddComponent("fuel")
    inst.components.fuel.fueltype = "NIGHTMARE"
    inst.components.fuel.fuelvalue = TUNING.LARGE_FUEL*2

    inst:AddComponent("repairer")
    inst.components.repairer.repairmaterial = "nightmare"
    inst.components.repairer.finiteusesrepairvalue = TUNING.NIGHTMAREFUEL_FINITEUSESREPAIRVALUE*2

    inst:AddComponent("magicturner")
    inst.components.magicturner.target_require_tags = {"normal_wood"}
	inst.components.magicturner.convert_prefab = "livinglog"
	inst.components.magicturner.doer_require_tags = {"shadow_form_on"}

    inst:AddComponent("machine")	
	inst.components.machine.caninteractfn = function(inst, doer)
	    return doer ~= nil and doer:HasTag("shadow_form_on") and doer.supershadowmanipulation 
	end
    inst.components.machine.cooldowntime = 0.1
	inst.components.machine.CollectSceneActions = function() end  	
	
	if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
		inst:AddComponent("waterproofer")
	    inst.components.waterproofer:SetEffectiveness(0)
	end

	if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
 		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)    
		
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
		
        MakeBlowInHurricane(inst, TUNING.WINDBLOWN_SCALE_MIN.LIGHT, TUNING.WINDBLOWN_SCALE_MAX.LIGHT)

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = TUNING.APPEASEMENT_SMALL*2
	end

	inst:AddComponent("useableitem")
	inst.components.useableitem:SetCanInteractFn(OnlyForShadow)
	inst.components.useableitem:SetOnUseFn(Corrosion)

    inst.core = CreateCore()
    inst.core.entity:SetParent(inst.entity)
	
	return inst
end

local function onequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_cane_flower", "swap_cane")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
	inst.components.perishable:StartPerishing()
	inst.components.caneflowerfxspawner:Enable(true)		
end

local function onunequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
	inst.components.perishable:StopPerishing()
	inst.components.caneflowerfxspawner:Enable(false)		
end

local function onattack(inst, attacker, target)
    if math.random() < 0.25 and target ~= nil and target.Transform ~= nil then
	    if inst:IsOnValidGround() and target:IsOnValidGround() then
	        local x, y, z = target.Transform:GetWorldPosition()
			local prefab_to_spawn = SpawnPrefab(math.random() < 0.001 and "rose_inv" or "petals")
		    if prefab_to_spawn then
		        prefab_to_spawn.Transform:SetPosition(x, y, z)
			    SpawnPrefab("small_puff").Transform:SetPosition(x, y, z)
			    if prefab_to_spawn.GetIsOnWater ~= nil
			        and prefab_to_spawn:GetIsOnWater()
			    then
			        if prefab_to_spawn.components.floatable ~= nil then
				        prefab_to_spawn.components.floatable:OnHitWater(true)
				    elseif prefab_to_spawn.components.inventoryitem ~= nil then
			            prefab_to_spawn.components.inventoryitem:OnHitWater()
			        end			
			    end
		    end			
		end
	end
end

local function caneflower_fn()
	local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("cane")
    inst.AnimState:SetBuild("cane_flower")
    inst.AnimState:PlayAnimation("idle")
	
	inst:AddTag("invflower")
	inst:AddTag("conjurable")
	inst:AddTag("show_spoilage")
	inst:AddTag("flower_cane")
	
    if inst.GetIsOnWater ~= nil then
        MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
	end

    local cane_flower_speed_mult = 1.4
	
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        cane_flower_speed_mult = 0.4
	end
 
    local caneflowerfxspawner_notiles = 
	{
	    GROUND.INVALID,
		GROUND.IMPASSABLE,
		GROUND.ROAD,
		GROUND.ROCKY,
		GROUND.WOODFLOOR,
		GROUND.CARPET,
		GROUND.BRICK,
		GROUND.CHECKER,
		GROUND.UNDERROCK,
		GROUND.BRICK_GLOW,
	}

	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        table.insert(caneflowerfxspawner_notiles, GROUND.VOLCANO_ROCK)
        table.insert(caneflowerfxspawner_notiles, GROUND.VOLCANO)
	    table.insert(caneflowerfxspawner_notiles, GROUND.VOLCANO_LAVA)
        table.insert(caneflowerfxspawner_notiles, GROUND.ASH)
        table.insert(caneflowerfxspawner_notiles, GROUND.SNAKESKIN)
	end
 
	if IsDLCEnabled(PORKLAND_DLC) then
        table.insert(caneflowerfxspawner_notiles, GROUND.FOUNDATION)
        table.insert(caneflowerfxspawner_notiles, GROUND.COBBLEROAD)
        table.insert(caneflowerfxspawner_notiles, GROUND.INTERIOR)        
        table.insert(caneflowerfxspawner_notiles, GROUND.PIGRUINS)
        table.insert(caneflowerfxspawner_notiles, GROUND.PIGRUINS_NOCANOPY)        
        table.insert(caneflowerfxspawner_notiles, GROUND.BEARDRUG)		
	end
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(TUNING.CANE_DAMAGE*1.5)
	inst.components.weapon:SetOnAttack(onattack)
    
    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)   
	
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)
    inst.components.equippable.walkspeedmult = cane_flower_speed_mult
 
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
		inst.components.equippable.dapperfn = function(inst, owner)
		    return owner ~= nil and owner:HasTag("flower_lover") and TUNING.DAPPERNESS_LARGE or 0
		end
	else
	    inst:AddComponent("dapperness")
        inst.components.dapperness.dapperfn = function(inst, owner) 
		    return owner ~= nil and owner:HasTag("flower_lover") and TUNING.DAPPERNESS_LARGE or 0
		end
	end
 
  	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.PERISH_MED)
	inst.components.perishable:SetOnPerishFn(inst.Remove)
	
    inst:AddComponent("repairable")
    inst.components.repairable.repairmaterial = "rose"
	inst.components.repairable.announcecanfix = false

	inst:AddComponent("caneflowerfxspawner")
	inst.components.caneflowerfxspawner:SetPrefab("cane_flower_fx")
	inst.components.caneflowerfxspawner:SetRange(1)
	inst.components.caneflowerfxspawner:SetMax(15)
	inst.components.caneflowerfxspawner:SetRate(0.25)
	inst.components.caneflowerfxspawner.tags = {"cane_flower_fx"}	
	inst.components.caneflowerfxspawner:SetNotTiles(caneflowerfxspawner_notiles)
 
    return inst
end

local function IsPlayerEquipFlowerCane(player)
    if player ~= nil and player.components.inventory ~= nil and player.components.inventory:EquipHasTag("flower_cane") then
	    return true
	end
    return false
end

local function canflowerfx_fn()
    local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()

    inst.animname = math.random(1,4)

	inst:AddTag("FX")
	inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
	inst:AddTag("INTERIOR_LIMBO_IMMUNE")	
	inst:AddTag("cane_flower_fx")

	inst.AnimState:SetBank("wormwood_plant_fx")
	inst.AnimState:SetBuild("wormwood_plant_fx")
	inst.AnimState:PlayAnimation("grow_"..inst.animname)

    inst:ListenForEvent("animover", function(inst, data)
        if inst.ending then
            inst:Remove()
        else
            if not IsPlayerEquipFlowerCane(GetPlayer()) or not inst:IsNear(GetPlayer(), 2) then
                inst.ending = true
                inst.AnimState:PlayAnimation("ungrow_"..inst.animname)
            else
                inst.AnimState:PlayAnimation("idle_"..inst.animname)
            end
        end
    end)

    inst.persists = false

    return inst
end

local function TryToSpawnPurpleGem(inst, chance, target)
    if math.random() < (chance or 0) and target ~= nil and target.Transform ~= nil then
	    if target:IsOnValidGround() then
	        local x, y, z = target.Transform:GetWorldPosition()
			local prefab_to_spawn = SpawnPrefab("purplegem")
		    if prefab_to_spawn then
		        prefab_to_spawn.Transform:SetPosition(x, y, z)
			    SpawnPrefab("small_puff").Transform:SetPosition(x, y, z)
			    if prefab_to_spawn.GetIsOnWater ~= nil
			        and prefab_to_spawn:GetIsOnWater()
			    then
			        if prefab_to_spawn.components.floatable ~= nil then
				        prefab_to_spawn.components.floatable:OnHitWater(true)
				    elseif prefab_to_spawn.components.inventoryitem ~= nil then
			            prefab_to_spawn.components.inventoryitem:OnHitWater()
			        end			
			    end
		    end			
		end
	end
end

local function getrandomposition(inst, target)
    local ground = GetWorld()
    local centers = {}

    if (target.GetIsOnWater and target:GetIsOnWater()) or target:HasTag("aquatic") then
        for i,node in ipairs(ground.topology.nodes) do
            if inst.GetIsOnWater and inst:GetIsOnWater(node.x, 0, node.y) then
                table.insert(centers, {x = node.x, z = node.y})
            end
        end
    else
        for i,node in ipairs(ground.topology.nodes) do
            if IsPassableAtPoint(node.x, 0, node.y) 
			    and (inst.GetIsOnLandOutside == nil or inst:GetIsOnLandOutside(node.x, 0, node.y))
				and (inst.GetIsOnLand == nil or inst:GetIsOnLand(node.x, 0, node.y))
			then
			    table.insert(centers, {x = node.x, z = node.y})
            end
        end
    end

    if #centers > 0 then
		local pos = centers[math.random(#centers)]
		return Point(pos.x, 0, pos.z)
	else
		return GetPlayer():GetPosition()
	end
end

local function teleport_thread(inst, caster, teletarget, loctarget)
    --Super Telestaff can be used in Caves world. And no sanity loss.
    local ground = GetWorld()

    local t_loc = nil
    if loctarget then
        t_loc = loctarget:GetPosition()
    else
        t_loc = getrandomposition(inst, teletarget)
    end

    local teleportee = teletarget
    local pt = teleportee:GetPosition()
    if teleportee.components.locomotor then
        teleportee.components.locomotor:StopMoving()
    end

    inst.components.finiteuses:Use(1)

    if teleportee.components.health then
        teleportee.components.health:SetInvincible(true)
    end

    if teleportee.components.driver and teleportee.components.driver.vehicle and teleportee.components.driver.vehicle.components.boathealth then
        teleportee.components.driver.vehicle.components.boathealth:SetInvincible(true)
    end
    
    GetSeasonManager():DoLightningStrike(pt)
    teleportee:Hide()

    if teleportee == GetPlayer() then
        TheFrontEnd:Fade(false, 2)
        Sleep(3)
    end
    
    if ground.components.seasonmanager then
        ground.components.seasonmanager:ForcePrecip()
    end

    if teleportee.Teleport then
	    teleportee:Teleport(loctarget or t_loc)
	else
        teleportee.Transform:SetPosition(t_loc.x, 0, t_loc.z)
        if teleportee == GetPlayer() then
            TheCamera:Snap()
            TheFrontEnd:DoFadeIn(1)
            Sleep(1)
        end	
	end
 
    TryToSpawnPurpleGem(inst, 0.1, teleportee)
 
    if loctarget and loctarget.onteleto then 
	    loctarget.onteleto(loctarget) 	
	end
	
    GetSeasonManager():DoLightningStrike(t_loc)
    teleportee:Show()
    if teleportee.components.health then
        teleportee.components.health:SetInvincible(false)
    end
    if teleportee.components.driver and teleportee.components.driver.vehicle and teleportee.components.driver.vehicle.components.boathealth then
        teleportee.components.driver.vehicle.components.boathealth:SetInvincible(false)
    end

    if teleportee == GetPlayer() then
        teleportee.sg:GoToState("wakeup")
        teleportee.SoundEmitter:PlaySound("dontstarve/common/staffteleport")
    end
end

local function TeleportFunc(inst, target)
    local mindistance = 1
    local caster = inst.components.inventoryitem.owner
    local tar = target or caster
    local pt = tar:GetPosition()
    local ents = TheSim:FindEntities(pt.x,pt.y,pt.z, 9000, {"telebase"})

    if #ents <= 0 or tar:HasTag("aquatic") or (tar.GetIsOnWater and tar:GetIsOnWater()) then
        --There's no bases, active or inactive. Teleport randomly.
        inst.task = inst:StartThread(function() teleport_thread(inst, caster, tar) end)
        return
    end

    local targets = {}
    for k,v in pairs(ents) do
        local v_pt = v:GetPosition()
        if distsq(pt, v_pt) >= mindistance * mindistance and not (v.GetIsInInterior and v:GetIsInInterior() and tar:HasTag("dontteleporttointerior")) then
            table.insert(targets, {base = v, distance = distsq(pt, v_pt)}) 
        end
    end

    table.sort(targets, function(a,b) return (a.distance) < (b.distance) end)
    for i = 1, #targets do
        local teletarget = targets[i]
        if teletarget.base and teletarget.base.canteleto(teletarget.base) then
            inst.task = inst:StartThread(function() teleport_thread(inst, caster, tar, teletarget.base) end)
            return
        end
    end

    inst.task = inst:StartThread(function() teleport_thread(inst, caster, tar) end)
end

local function CanTeleport(inst, caster, target)
    if target ~= nil then
	    if target:HasTag("canbeteleported") then
		    return true
		elseif target:HasTag("noteleport") then
		    return false
		end	
	    return target.components.locomotor ~= nil
	end
    return true  --True for caster casting in the inventory
end

local function onequip_supertelestaff(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_telestaff_shadowbeam", "swap_purplestaff")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 		
end

local function onunequip_supertelestaff(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 	
end

local function onattack_supertelestaff(inst, attacker, target)
    TryToSpawnPurpleGem(inst, 0.05, target)
end

local function supertelestaff_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("purplestaff")
    inst.AnimState:SetBuild("telestaff_shadowbeam")
    inst.AnimState:PlayAnimation("idle")
	
	inst:AddTag("shadow_item")
    inst:AddTag("shadow")
    inst:AddTag("sharp")
	
    if inst.GetIsOnWater ~= nil then
        MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
	end
	
    inst.fxcolour = {104/255,40/255,121/255}

    inst:AddComponent("inspectable")
	
    inst:AddComponent("tradable")
    
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)   	

    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetOnFinished(inst.Remove)	
    inst.components.finiteuses:SetMaxUses(1000)
    inst.components.finiteuses:SetUses(1000)
	
    inst:AddComponent("spellcaster")
    inst.components.spellcaster.canuseontargets = true	
    inst.components.spellcaster:SetSpellFn(TeleportFunc)
    inst.components.spellcaster:SetSpellTestFn(CanTeleport)

    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip_supertelestaff)
    inst.components.equippable:SetOnUnequip(onunequip_supertelestaff)

    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(68)
	inst.components.weapon:SetOnAttack(onattack_supertelestaff)
	
	inst:ListenForEvent("daytime", function() TryToSpawnPurpleGem(inst, 0.01, inst) end, GetWorld())

    inst:AddComponent("shadowrepairable")
	inst.components.shadowrepairable:SetRepairMaterial("voidcloth")
	inst.components.shadowrepairable:SetOnRepaired(OnRepaired)
	
	return inst
end

local function TryToSpawnShadowFuel(inst, chance, target)
    if math.random() < (chance or 0) and target ~= nil and target.Transform ~= nil then
	    if target:IsOnValidGround() then
	        local x, y, z = target.Transform:GetWorldPosition()
			local prefab_to_spawn = SpawnPrefab(math.random() < 0.25 and "horrorfuel" or "nightmarefuel")
		    if prefab_to_spawn then
		        prefab_to_spawn.Transform:SetPosition(x, y, z)
			    SpawnPrefab("small_puff").Transform:SetPosition(x, y, z)
			    if prefab_to_spawn.GetIsOnWater ~= nil
			        and prefab_to_spawn:GetIsOnWater()
			    then
			        if prefab_to_spawn.components.floatable ~= nil then
				        prefab_to_spawn.components.floatable:OnHitWater(true)
				    elseif prefab_to_spawn.components.inventoryitem ~= nil then
			            prefab_to_spawn.components.inventoryitem:OnHitWater()
			        end			
			    end
		    end			
		end
	end
end

local function UpdateDamage(inst)
    if inst.components.weapon == nil then 
	    return 	
	end
	
    if GetWorld():IsRuins() then
	    inst.components.weapon:SetDamage(306)
	elseif GetWorld():IsCave() then
	    inst.components.weapon:SetDamage(255)
	else
        if GetClock():IsDay() then
            inst.components.weapon:SetDamage(102)
        elseif GetClock():IsNight() then
            inst.components.weapon:SetDamage(204)
        elseif GetClock():IsDusk() then
            inst.components.weapon:SetDamage(153)
		end
    end
end

local function onequip_purenightsword(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_nightsword_lightsbane", "swap_nightmaresword")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 		
end

local function onunequip_purenightsword(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 	
end

local DAMAGE_MUST_TAGS = {"hascombatcomponent"}
local DAMAGE_CANT_TAGS = {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "DECOR", "invisible", "notarget", "boat", "wall", "companion", "bramble", "elephantcactus", "buttercup"}
local function onattack_purenightsword(inst, attacker, target)
    TryToSpawnShadowFuel(inst, 0.1, target)
	if target ~= nil and target.Transform ~= nil then
	    local x, y, z = target.Transform:GetWorldPosition()
	    local ents = TheSim:FindEntities(x, y, z, 5, DAMAGE_MUST_TAGS, DAMAGE_CANT_TAGS)
		for k,v in pairs(ents) do
		    if v ~= inst and v ~= target and v ~= attacker then
			    if v:IsValid() and v.entity:IsVisible() and v.components.combat ~= nil then
				    local damage = inst.components.weapon.damage or 102
				    v.components.combat:GetAttacked(attacker, damage, inst)		
				end
			end
		end	
	end
end

local function purenightsword_fn()
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("nightmaresword")
    inst.AnimState:SetBuild("nightsword_lightsbane")
    inst.AnimState:PlayAnimation("idle")
	
	inst:AddTag("shadow_item")
    inst:AddTag("shadow")
    inst:AddTag("sharp")
	
    if inst.GetIsOnWater ~= nil then
        MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
	end
	
    inst:AddComponent("inspectable")
	
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)   
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(102)
	inst.components.weapon:SetOnAttack(onattack_purenightsword)
	inst.components.weapon:SetRange(2)

    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetOnFinished(inst.Remove)	
    inst.components.finiteuses:SetMaxUses(1000)
    inst.components.finiteuses:SetUses(1000)
	
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip_purenightsword)
    inst.components.equippable:SetOnUnequip(onunequip_purenightsword)	

    inst:DoTaskInTime(0, function() UpdateDamage(inst) end)	
	inst:ListenForEvent("dusktime", function() UpdateDamage(inst) end, GetWorld())
    inst:ListenForEvent("daytime", function() UpdateDamage(inst) end, GetWorld())
    inst:ListenForEvent("nighttime", function() UpdateDamage(inst) end, GetWorld())
	inst:ListenForEvent("arrive", function() UpdateDamage(inst) end, GetWorld())

    inst:AddComponent("shadowrepairable")
	inst.components.shadowrepairable:SetRepairMaterial("voidcloth")
	inst.components.shadowrepairable:SetOnRepaired(OnRepaired)
	
	return inst
end

local function CreateMiasma(inst, initial)
    local x, y, z = inst.Transform:GetWorldPosition()
    if initial then

	    SpawnPrefab("miasma").Transform:SetPosition(x - TILE_SCALE, 0, z - TILE_SCALE)
	    SpawnPrefab("miasma").Transform:SetPosition(x, 0, z - TILE_SCALE)		
	    SpawnPrefab("miasma").Transform:SetPosition(x + TILE_SCALE, 0, z - TILE_SCALE)
	    SpawnPrefab("miasma").Transform:SetPosition(x - TILE_SCALE, 0, z)
	    SpawnPrefab("miasma").Transform:SetPosition(x + TILE_SCALE, 0, z)		
	    SpawnPrefab("miasma").Transform:SetPosition(x - TILE_SCALE, 0, z + TILE_SCALE)
	    SpawnPrefab("miasma").Transform:SetPosition(x, 0, z + TILE_SCALE)
	    SpawnPrefab("miasma").Transform:SetPosition(x + TILE_SCALE, 0, z + TILE_SCALE)	
		
		local pt = inst:GetPosition()
		local offset = FindWalkableOffset(pt, math.random()*2*math.pi, 4+math.random()*8, 20, true, false)
	    if inst.GetIsOnWater ~= nil then
		    offset = FindGroundOffset(pt, math.random()*2*math.pi, 4+math.random()*8, 20, true, false)
		end
        if offset then	    
		    local spawnpt = pt + offset
		    SpawnPrefab("dreadstone_stack").Transform:SetPosition(spawnpt.x, 0, spawnpt.z)
		end	
		
	else
	
        --local theta = math.random() * math.pi * 2
        --local ox, oz = TILE_SCALE * math.cos(theta), TILE_SCALE * math.sin(theta)
		
		--SpawnPrefab("miasma").Transform:SetPosition(x + ox, 0, z + oz)	

		local pt = inst:GetPosition()
		local offset = FindWalkableOffset(pt, math.random()*2*math.pi, 4+math.random()*8, 20, true, false)
	    if inst.GetIsOnWater ~= nil then
		    offset = FindGroundOffset(pt, math.random()*2*math.pi, 4+math.random()*8, 20, true, false)
		end
        if offset then	    
		    local spawnpt = pt + offset
		    SpawnPrefab("dreadstone_stack").Transform:SetPosition(spawnpt.x, 0, spawnpt.z)
		end	

	end
end

local function SetMaxMinimapStatus(inst)
    inst.MiniMapEntity:SetIcon("shadow_rift_max.tex")
end

local function SpawnStageFx(inst)
    local scale = inst._stage * 0.5

    local fx = SpawnPrefab("statue_transition")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
    fx.Transform:SetScale(scale, scale, scale)

    return fx
end

local function CreateParticleFx(inst)
    local fx = SpawnPrefab("charlie_shadow_rift_fx")
    inst:AddChild(fx)

    return fx
end

local function GetStageUpTime(inst)
    return TUNING.TOTAL_DAY_TIME * 4 + TUNING.TOTAL_DAY_TIME * math.random()
end

local function ShakeAllCameras(shakeType, duration, speed, maxShake, inst, maxDist)
    GetPlayer().components.playercontroller:ShakeCamera(inst, shakeType, duration, speed, maxShake, maxDist)
end

local function Initialize(inst)
    inst:SpawnStageFx()

    inst:CreateMiasma(true) -- Initial seed placement.

    inst.components.groundpounder.initialRadius = PHYSICS_SIZE_BY_STAGE[inst._stage]

    inst.components.groundpounder:GroundPound()
    inst.components.groundpounder:GroundPound() -- For pushing items and digging stumps, it's probably not the best thing to do.

    local duration, speed, scale, max_dist = unpack(SHAKE_PARAMS_BY_STAGE[inst._stage])
    ShakeAllCameras("FULL", duration, speed, scale, inst, max_dist)
end

local function LaunchItem(inst, launcher, basespeed)
    if inst ~= nil and inst.Physics ~= nil and inst.Physics:IsActive() and launcher ~= nil then
        local x, y, z = inst.Transform:GetWorldPosition()
        local x1, y1, z1 = launcher.Transform:GetWorldPosition()
        local vx, vz = x - x1, z - z1
        local spd = math.sqrt(vx * vx + vz * vz)
        local angle =
            spd > 0 and
            math.atan2(vz / spd, vx / spd) + (math.random() * 20 - 10) * DEGREES or
            math.random() * math.pi * 2
        spd = (basespeed or 5) + math.random() * 2
        inst.Physics:Teleport(x, .1, z)
        inst.Physics:SetVel(math.cos(angle) * spd, 10, math.sin(angle) * spd)
        if inst.components.inventoryitem and inst.components.inventoryitem.OnStartFalling then
            inst.components.inventoryitem:OnStartFalling()
        end			
    end
end

local BOUNCESTUFF_MUST_TAGS = { "isinventoryitem" }
local BOUNCESTUFF_CANT_TAGS = { "locomotor", "INLIMBO" }
local function TryStageUp(inst)
    if inst._closing then return end

    if inst._stage < 3 then
        local next_stage = inst._stage + 1
        inst._stage = next_stage

        if inst._fx then
            inst._fx:PlayStage(next_stage)
        end

        inst:SpawnStageFx()

        inst.components.groundpounder.initialRadius = PHYSICS_SIZE_BY_STAGE[next_stage]
        inst.components.groundpounder:GroundPound()
	
        inst.Physics:SetCylinder(PHYSICS_SIZE_BY_STAGE[next_stage], 6)  
		inst.Physics:SetActive(true)  --DS need to use this to force to update the physics collision

        local x, y, z = inst.Transform:GetWorldPosition()
        local inv_push_rad = inst.Physics:GetRadius() or 1	
		local inv_ents = TheSim:FindEntities(x, y, z, inv_push_rad+1.5, BOUNCESTUFF_MUST_TAGS, BOUNCESTUFF_CANT_TAGS)
        for k,v in pairs(inv_ents) do
            if v ~= nil
			    and v:IsValid()
				and v.components.inventoryitem ~= nil
				and not v.components.inventoryitem.nobounce
				and (v.GetIsOnWater == nil or not v:GetIsOnWater())
                and v.Physics ~= nil
                and v.Physics:IsActive()
			then
			    LaunchItem(v, inst, 3)
			end
        end		

        local duration, speed, scale, max_dist = unpack(SHAKE_PARAMS_BY_STAGE[next_stage])
        ShakeAllCameras("FULL", duration, speed, scale, inst, max_dist)

        inst.AnimState:PlayAnimation("stage_"..next_stage.."_pre")
        inst.AnimState:PushAnimation("stage_"..next_stage.."_loop", true)

        inst.SoundEmitter:SetParameter(AMBIENT_SOUND_LOOP_NAME, AMBIENT_SOUND_PARAM_NAME, AMBIENT_SOUND_STAGE_TO_INTENSITY[next_stage])
        inst.SoundEmitter:PlaySound("rifts/portal/rift_explode")

        if next_stage < 3 then
            if not inst.components.timer:TimerExists("trynextstage") then
                inst.components.timer:StartTimer("trynextstage", GetStageUpTime())
            end
        else
            inst.components.timer:StopTimer("trynextstage")
        end

        if next_stage == 3 then
            inst:SetMaxMinimapStatus()
            GetWorld():PushEvent("ms_shadowrift_maxsize")
        end
    else
        inst.AnimState:PlayAnimation("stage_"..inst._stage.."_loop", true)
    end
end

local function ClosePortal(inst)
    inst._closing = true

    if inst:IsAsleep() then
        inst:Remove()
    else
        inst.AnimState:PlayAnimation("disappear")
        inst.SoundEmitter:PlaySound("rifts/portal/portal_disappear")

        if inst._fx then
            inst._fx:Disappear()
        end

        inst.components.timer:StartTimer("be_removed", 1)
    end
end

local function OnPortalSleep(inst)
    inst.SoundEmitter:KillSound(AMBIENT_SOUND_LOOP_NAME)

    if inst._fx then
        inst._fx:Remove()
        inst._fx = nil
    end
end

local function OnPortalWake(inst)

    inst.SoundEmitter:PlaySound(AMBIENT_SOUND_PATH, AMBIENT_SOUND_LOOP_NAME)
    inst.SoundEmitter:SetParameter(AMBIENT_SOUND_LOOP_NAME, AMBIENT_SOUND_PARAM_NAME, AMBIENT_SOUND_STAGE_TO_INTENSITY[inst._stage])

    if not inst._fx then
        inst._fx = CreateParticleFx(inst)
        inst._fx:PlayStage(inst._stage, true)
    end
	
end

local function OnTimerDone(inst, data)
    if data.name == "seedmiasma" then
        inst:CreateMiasma()
    elseif data.name == "trynextstage" then
        inst:TryStageUp()
    elseif data.name == "be_removed" then
	    GetWorld():PushEvent("startspawningthronetimer")
        inst:Remove()
    elseif data.name == "close" then
        ClosePortal(inst)
    elseif data.name == "initialize" then
        Initialize(inst)
    end
end  

local function RiftResetInteractive(inst)
	if inst.components.interactivelevel ~= nil then 
	    inst.components.interactivelevel.inactive = true 
	end 
    if inst.resetinteractivetask ~= nil then 
	    inst.resetinteractivetask:Cancel() 
		inst.resetinteractivetask = nil 
	end 
	if inst.resetinteractivetaskinfo ~= nil then 
	    inst.resetinteractivetaskinfo = nil 
	end 
end

local function OnPortalSave(inst, data)
    data.stage = inst._stage

    -- We can't just flag with persists = false, because we need to fire the onremove listener to clean up the area.
    data.finished = inst.components.timer:TimerExists("be_removed")

	if inst.components.timer ~= nil then
	    if inst.components.timer:TimerExists("seedmiasma") then
		    print("Save seedmiasma timer time left")
		    data.timer_seedmiasma = inst.components.timer:GetTimeLeft("seedmiasma")
		end
	    if inst.components.timer:TimerExists("trynextstage") then
		    print("Save trynextstage timer time left")
		    data.timer_trynextstage = inst.components.timer:GetTimeLeft("trynextstage")
		end	
	    if inst.components.timer:TimerExists("close") then
		    print("Save close timer time left")
		    data.timer_close = inst.components.timer:GetTimeLeft("close")
		end
	end 
	
	if inst.resetinteractivetask ~= nil and inst.resetinteractivetaskinfo ~= nil then 
	    data.resetinteractivetimeleft = inst:TimeRemainingInTask(inst.resetinteractivetaskinfo)
	end 

end

local function OnPortalLoad(inst, data)
    if data then
        inst._stage = data.stage or inst._stage

        inst.loaded = true
		inst:DoTaskInTime(0.1, function() inst.loaded = nil end)
		
		local x, y, z = inst.Transform:GetWorldPosition()
		local miasmas = TheSim:FindEntities(x, y, z, 20, {"miasma"})
		
        if data.timer_seedmiasma then
		    if inst.components.timer:TimerExists("seedmiasma") then
		        print("Load seedmiasma timer time")
				inst.components.timer:SetTimeLeft("seedmiasma", data.timer_seedmiasma)			
		    else
			    print("Load missing seedmiasma timer time")
			    inst.components.timer:StartTimer("seedmiasma", data.timer_seedmiasma)
		    end			
		elseif #miasmas <= 0 then
		    inst:CreateMiasma(true)
			inst.components.timer:StartTimer("seedmiasma", TUNING.TOTAL_DAY_TIME * 0.25)		
		end

        if data.timer_trynextstage then
		    if inst.components.timer:TimerExists("trynextstage") then
		        print("Load trynextstage timer time")
				inst.components.timer:SetTimeLeft("trynextstage", data.timer_trynextstage)			
		    else
			    print("Load missing trynextstage timer time")
			    inst.components.timer:StartTimer("trynextstage", data.timer_trynextstage)
		    end		
		elseif inst._stage < 3 then
		    inst.components.timer:StartTimer("trynextstage", GetStageUpTime())
		end
		
        if data.timer_close then
		    if inst.components.timer:TimerExists("close") then
		        print("Load close timer time")
				inst.components.timer:SetTimeLeft("close", data.timer_close)			
		    else
			    print("Load missing close timer time")
			    inst.components.timer:StartTimer("close", data.timer_close)
		    end
        else
            inst.components.timer:StartTimer("close", TUNING.TOTAL_DAY_TIME * 15)		
		end
		
        if inst._stage >= 3 then
            inst.components.timer:StopTimer("trynextstage")
        end
		
        inst.Physics:SetCylinder(PHYSICS_SIZE_BY_STAGE[inst._stage], 6)
		
        inst.components.groundpounder.initialRadius = PHYSICS_SIZE_BY_STAGE[inst._stage]

        if data.finished then
            inst.AnimState:PlayAnimation("disappear")
            inst.components.timer:StartTimer("be_removed", 5)

            if inst._fx then
                inst._fx:Disappear()
            end
        else
            inst.AnimState:PlayAnimation("stage_"..inst._stage.."_loop", true)
            if inst._fx then
                inst._fx:PlayStage(inst._stage, true)
            end
        end 
		
		if data.resetinteractivetimeleft then 
	       	if inst.components.interactivelevel ~= nil then 
	            inst.components.interactivelevel.inactive = false 
	        end 
            if inst.resetinteractivetask then
	            inst.resetinteractivetask:Cancel()
		        inst.resetinteractivetask = nil 
	        end 
	        if inst.resetinteractivetaskinfo then
	            inst.resetinteractivetaskinfo = nil 
          	end
	        inst.resetinteractivetask, inst.resetinteractivetaskinfo = inst:ResumeTask(data.resetinteractivetimeleft, RiftResetInteractive)		
		end 
    end
end

local function OnPortalLongUpdate(inst, dt)

    if inst.resetinteractivetask and inst.resetinteractivetaskinfo then
        local new_time = math.max(inst:TimeRemainingInTask(inst.resetinteractivetaskinfo) - dt, 0)

        inst.resetinteractivetask:Cancel()
        inst.resetinteractivetask = nil 
        inst.resetinteractivetaskinfo = nil

        inst.resetinteractivetask, inst.resetinteractivetaskinfo = inst:ResumeTask(new_time, RiftResetInteractive)
    end

    if inst._stage + 1 >= 3 then
        -- Do nothing, the timer cmp will already do one long update.
        return
    end

    local timer = inst.components.timer

    local trynextstage_timeleft = timer:GetTimeLeft("trynextstage")
    
    if trynextstage_timeleft ~= nil then
        local stageup_time = GetStageUpTime()

        -- Try to skip one stage if possible.
        if dt >= stageup_time then
            inst:TryStageUp()

            -- Increases the remaining time, so that the timer's LongUpdate considers this stage skip time.
            timer:SetTimeLeft("trynextstage", trynextstage_timeleft + stageup_time)
        end
    end
	
end

local function OnPortalLoadPostPass(inst, newents, data)
    -- If we're loading anything, stop our timer.
    inst.components.timer:StopTimer("initialize")
	
    inst.SoundEmitter:KillSound(AMBIENT_SOUND_LOOP_NAME)

    if not inst:IsAsleep() then
        inst.SoundEmitter:PlaySound(AMBIENT_SOUND_PATH, AMBIENT_SOUND_LOOP_NAME)
        inst.SoundEmitter:SetParameter(AMBIENT_SOUND_LOOP_NAME, AMBIENT_SOUND_PARAM_NAME, AMBIENT_SOUND_STAGE_TO_INTENSITY[inst._stage])
    end

    if inst._stage == 3 then
	    inst:SetMaxMinimapStatus()
        GetWorld():PushEvent("ms_shadowrift_maxsize")
    end

end

local function OnPortalRemoved(inst)
    local x, y, z = inst.Transform:GetWorldPosition()
    local miasmas = TheSim:FindEntities(x, y, z, 30, {"miasma"})
    for _,miasma in pairs(miasmas) do
	    miasma:Remove()
	end
end

--[[
local function check_rotation(inst)
    local downvec = TheCamera:GetDownVec()
    local facedown = -(math.atan2(downvec.z, downvec.x) * (180/math.pi))		
	inst.Transform:SetRotation(facedown)
end
]]

local function GushOfFuel(inst)
    if not inst:IsValid() then
	    return
	end
    local x, _, z = inst.Transform:GetWorldPosition()
    local x_dist = 2 + 3 * math.random() + (inst.Physics:GetRadius() or 0)   
	local z_dist = 1 + 4 * math.random() + (inst.Physics:GetRadius() or 0) 
	local subtheta = math.random() * 2 * math.pi
    local spawn_x, spawn_z
    spawn_x, spawn_z = x + math.cos(subtheta) * x_dist, z + math.sin(subtheta) * z_dist	
	local BlockerNearby = TheSim:FindEntities(spawn_x, 0, spawn_z, 1, nil, {"NOCLICK", "FX", "player"})
	if #BlockerNearby <= 0 then					
		inst.components.lootdropper:DropLoot(Vector3(spawn_x, 3, spawn_z))
		SpawnPrefab("small_puff").Transform:SetPosition(spawn_x, 3, spawn_z)
	end
	inst:DoTaskInTime(20+math.random()*10, GushOfFuel)
end

local function rift_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()
	
	MakeObstaclePhysics(inst, PHYSICS_SIZE_BY_STAGE[1])
    inst.Physics:SetCylinder(PHYSICS_SIZE_BY_STAGE[1], 6)
	
	inst.MiniMapEntity:SetIcon("shadow_rift.tex")
	
    inst.AnimState:SetBank("shadowrift_portal")
    inst.AnimState:SetBuild("shadowrift_portal")
    inst.AnimState:PlayAnimation("stage_1_pre") 
    inst.AnimState:PushAnimation("stage_1_loop", true)
    inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
    inst.AnimState:SetSortOrder(3)	
	inst.AnimState:SetLightOverride(0.5)
	
    inst:AddTag("notarget")
    inst:AddTag("scarytoprey")
	inst:AddTag("shadow_destroy_immune")	
	inst:AddTag("pugalisk_avoids")
    inst:AddTag("shadow_rift")	
	
	inst._fx = CreateParticleFx(inst)
	inst._stage = 1
	
	inst:AddComponent("inspectable")

    inst:AddComponent("rotationupdater")

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(20)
    inst.components.combat.playerdamagepercent = 0.5

    inst:AddComponent("timer")

    inst.components.timer:StartTimer("initialize", 0)
	
	inst:DoTaskInTime(0, function()
        if not inst.loaded then
            inst.components.timer:StartTimer("close", TUNING.TOTAL_DAY_TIME * 15)
            inst.components.timer:StartTimer("trynextstage", GetStageUpTime())
            inst.components.timer:StartTimer("seedmiasma", TUNING.TOTAL_DAY_TIME * 0.25)
        end
	end)

    inst:AddComponent("groundpounder")
    inst.components.groundpounder.radiusStepDistance = 1.5
    inst.components.groundpounder.inventoryPushingRings = 2
    inst.components.groundpounder.numRings = 2
    inst.components.groundpounder.destroyer = true

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:AddRandomLoot("nightmarefuel", 99)
	inst.components.lootdropper:AddRandomLoot("horrorfuel", 1)
	inst.components.lootdropper.numrandomloot = 1
	inst.components.lootdropper.speed = 5    --TO DO: MAKE VANILLA AND ROG CHECK THIS

	if GetPlayer() and GetPlayer().prefab == "charlie" then	
        inst:AddComponent("interactivelevel")
        inst.components.interactivelevel.IsShadowRift = true
        inst.components.interactivelevel.OnActivate = function(inst, doer)
            if doer ~= nil and doer:HasTag("shadow_form_on") and doer.supershadowmanipulation then
			    if doer.AnimState ~= nil then 
				    doer.AnimState:PlayAnimation("jump") 
				end
	            if doer.components.health ~= nil then 
		            doer.components.health:SetInvincible(true)
	            end 
				if doer.components.playercontroller ~= nil then 
				    doer.components.playercontroller:Enable(false) 
				end
		        if doer.components.driver and doer.components.driver.vehicle and doer.components.driver.vehicle.components.boathealth then
		            doer.components.driver.vehicle.components.boathealth:SetInvincible(true)
	        	end			
                if doer == GetPlayer() then 
				    doer:DoTaskInTime(1, function() 
		                GetPlayer().HUD:Hide()
		                TheFrontEnd:SetFadeLevel(1)	
					end)
				    doer:DoTaskInTime(4, function() 
			            TheFrontEnd:Fade(true, 2)
			            GetPlayer().HUD:Show() 
		             	if doer.components.health ~= nil then 
				            doer.components.health:SetPercent(1) 
							doer.components.health:DoDelta(0, nil, nil, true) 
			            end 
						if math.random() < 0.1 and doer.components.inventory ~= nil then
						    for i=1, math.random(2) do
							    doer.components.inventory:GiveItem(SpawnPrefab("horrorfuel"), nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition()))) 
							end
						elseif doer.components.inventory ~= nil then
						    for i=1, math.random(4) do
							    doer.components.inventory:GiveItem(SpawnPrefab("nightmarefuel"), nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition()))) 
							end
						end
		            end)		
		            doer:DoTaskInTime(5, function()
                        if doer.components.health ~= nil then 
			                doer.components.health:SetInvincible(false) 
						end
						if doer.components.playercontroller ~= nil then 
			                doer.components.playercontroller:Enable(true) 
						end 
		                if doer.components.driver and doer.components.driver.vehicle and doer.components.driver.vehicle.components.boathealth then
		                    doer.components.driver.vehicle.components.boathealth:SetInvincible(false)
	        	        end								
		            end)		
				elseif doer.components.health ~= nil then  
				    doer.components.health:SetInvincible(false) 
                end				
				if inst.resetinteractivetask == nil then 
				    inst.resetinteractivetask, inst.resetinteractivetaskinfo = inst:ResumeTask(480, RiftResetInteractive)
				end 
			end 
        end 		
	end
	
	inst.GushOfFuel = GushOfFuel
	
	inst:DoTaskInTime(10+math.random()*5, GushOfFuel)
	
    inst:ListenForEvent("timerdone", OnTimerDone)
    inst:ListenForEvent("onremove", OnPortalRemoved)
	
    inst.OnEntitySleep = OnPortalSleep
    inst.OnEntityWake = OnPortalWake
    inst.OnSave = OnPortalSave
    inst.OnLoad = OnPortalLoad
    inst.OnLoadPostPass = OnPortalLoadPostPass
    inst.OnLongUpdate = OnPortalLongUpdate	
    inst.TryStageUp = TryStageUp
    inst.CreateMiasma = CreateMiasma
    inst.SpawnStageFx = SpawnStageFx
    inst.SetMaxMinimapStatus = SetMaxMinimapStatus
	
	return inst
end

local function ParticlePlayStage(inst, stage, load)
    if load then
        inst.AnimState:PlayAnimation("particle_"..stage.."_loop", true)
    else
        inst.AnimState:PlayAnimation("particle_"..stage.."_pre")
        inst.AnimState:PushAnimation("particle_"..stage.."_loop", true)
    end
end

local function ParticleDisappear(inst)
    inst.AnimState:PlayAnimation("particle_disappear")
end

local function rift_fx_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()

    inst.AnimState:SetBank("shadowrift_portal")
    inst.AnimState:SetBuild("shadowrift_portal")
    inst.AnimState:PlayAnimation("particle_1_pre")
    inst.AnimState:PushAnimation("particle_1_loop", true)
    inst.AnimState:SetLightOverride(1)

    inst:AddTag("FX")
    inst:AddTag("NOCLICK")
    inst:AddTag("NOBLOCK")
    inst:AddTag("INTERIOR_LIMBO_IMMUNE")

    inst.persists = false

    inst.PlayStage = ParticlePlayStage
    inst.Disappear = ParticleDisappear

    return inst
end

local function onequip_rainbow(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_spear_terraprisma", "swap_spear")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal")
    if owner.rainbowlight ~= nil and owner.rainbowlight.components.spell ~= nil then
        owner.rainbowlight.components.spell.lifetime = 0
        owner.rainbowlight.components.spell:ResumeSpell()
    else
        local light = SpawnPrefab("rainbowlight")
        light.components.spell:SetTarget(owner)
        if not light.components.spell.target then
            light:Remove()
        end
        light.components.spell:StartSpell()
    end
end

local function onunequip_rainbow(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 	
    if owner.rainbowlight ~= nil and owner.rainbowlight.components.spell ~= nil then
	    owner.rainbowlight.components.spell:OnFinish()
	end
end

local function OnHitWaterRainbow(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
		inst.float_fx.Transform:SetPosition(0, -0.4, 0)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetPosition(0, -0.4, 0)
	end	
end

local function onattack_rainbow(inst, attacker, target)
    if math.random() < 0.01 
	    and target ~= nil
		and target.Transform ~= nil
		and target:IsOnValidGround()
	then
	    local all_kinds_of_gems = {"redgem", "bluegem", "purplegem", "orangegem", "yellowgem", "greengem"}
		local prefab_to_spawn = SpawnPrefab(GetRandomItem(all_kinds_of_gems))
	    local x, y, z = target.Transform:GetWorldPosition()
		if prefab_to_spawn ~= nil then
		    prefab_to_spawn.Transform:SetPosition(x, y, z)
	        SpawnPrefab("small_puff").Transform:SetPosition(x, y, z)
	        if prefab_to_spawn.GetIsOnWater ~= nil
			    and prefab_to_spawn:GetIsOnWater()
			then
			    if prefab_to_spawn.components.floatable ~= nil then
				    prefab_to_spawn.components.floatable:OnHitWater(true)
		        elseif prefab_to_spawn.components.inventoryitem ~= nil then
			        prefab_to_spawn.components.inventoryitem:OnHitWater()
			    end			
		    end		
		end
	end
	
	if attacker ~= nil and attacker.components.health ~= nil then
	    local health_to_gain = 0
		if target ~= nil then
		    if target.components.health ~= nil then
		        health_to_gain = health_to_gain + target.components.health.maxhealth * 0.05
			end
		    if target.components.combat ~= nil then
		        health_to_gain = health_to_gain + target.components.combat.defaultdamage * 0.25
			end			
		end
	    if health_to_gain > 0 then
		    attacker.components.health:DoDelta(health_to_gain, false, inst.prefab)
		end
	end
	
	if attacker ~= nil and attacker.components.sanity ~= nil then
	    local sanity_to_gain = 0
		if target ~= nil then
		    if target.components.health ~= nil then
		        sanity_to_gain = sanity_to_gain + target.components.health.maxhealth * 0.01
			end
		    if target.components.combat ~= nil then
		        sanity_to_gain = sanity_to_gain + target.components.combat.defaultdamage * 0.1
			end			
		end
	    if sanity_to_gain > 0 then
		    attacker.components.sanity:DoDelta(sanity_to_gain)
		end
	end	
	
end

local function rainbowsword_fn()
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
	
    local rainbowsword = TheSim:FindFirstEntityWithTag("rainbowsword")
    if rainbowsword and rainbowsword ~= inst then
	    inst:DoTaskInTime(0, inst.Remove)
	    return inst
	end
	
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	inst.entity:AddMiniMapEntity()
	
    MakeInventoryPhysics(inst)
	
	inst.MiniMapEntity:SetIcon("rainbowsword.tex")
    
    inst.AnimState:SetBank("spear")
    inst.AnimState:SetBuild("spear_terraprisma")
    inst.AnimState:PlayAnimation("idle")
	
    inst:AddTag("sharp")
	inst:AddTag("irreplaceable")
	inst:AddTag("dropontravel")
	inst:AddTag("rainbowsword")
	
    if inst.GetIsOnWater ~= nil then
        MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWaterRainbow)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
	end

    inst:AddComponent("inspectable")
 
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)   	

    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip_rainbow)
    inst.components.equippable:SetOnUnequip(onunequip_rainbow)
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(68)
	inst.components.weapon:SetOnAttack(onattack_rainbow)

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end   
	
	return inst
end

local function swapColor(inst)
    --Rainbow colour: red, orange, yellow, green, blue, cyan, purple
	if inst.is_red then
		inst.is_red = nil
		inst.is_orange = true
		inst.components.lighttweener:StartTween(nil, 5, nil, nil, {255/255, 165/255, 0/255}, 4, swapColor)
	elseif inst.is_orange then
		inst.is_orange = nil
		inst.is_yellow = true
		inst.components.lighttweener:StartTween(nil, 5, nil, nil, {255/255, 255/255, 0/255}, 4, swapColor)
	elseif inst.is_yellow then
	    inst.is_yellow = nil
		inst.is_green = true
		inst.components.lighttweener:StartTween(nil, 5, nil, nil, {100/255, 255/255, 50/255}, 4, swapColor)
	elseif inst.is_green then
	    inst.is_green = nil
		inst.is_blue = true
		inst.components.lighttweener:StartTween(nil, 5, nil, nil, {0/255, 127/255, 255/255}, 4, swapColor)
	elseif inst.is_blue then
	    inst.is_blue = nil
		inst.is_cyan = true
		inst.components.lighttweener:StartTween(nil, 5, nil, nil, {70/255, 190/255, 255/255}, 4, swapColor)
	elseif inst.is_cyan then
	    inst.is_cyan = nil
	    inst.is_purple = true
		inst.components.lighttweener:StartTween(nil, 5, nil, nil, {139/255, 70/255, 255/255}, 4, swapColor)
	else
	    inst.is_purple = nil
		inst.is_red = true
		inst.components.lighttweener:StartTween(nil, 5, nil, nil, {251/255, 30/255, 30/255}, 4, swapColor)
	end
end

local function light_resume(inst, time)
    inst.components.lighttweener:StartTween(nil, 5, 0.8, 0.5, {251/255, 30/255, 30/255}, 0)
	inst.is_red = true
	inst.components.lighttweener:StartTween(nil, nil, nil, nil, {251/255, 30/255, 30/255}, 4, swapColor)
end

local function light_spellfn(inst, target, variables)
    if target then
        inst.Transform:SetPosition(target:GetPosition():Get())
    end
end

local function light_start(inst)
    inst.components.lighttweener:StartTween(nil, 5, 0.8, 0.5, {251/255, 30/255, 30/255}, 0)
	inst.is_red = true
	inst.components.lighttweener:StartTween(nil, nil, nil, nil, {251/255, 30/255, 30/255}, 4, swapColor)
end

local function light_ontarget(inst, target)
    if not target then return end
    target.rainbowlight = inst
    target:AddTag(inst.components.spell.spellname)
    target.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
end

local function light_onfinish(inst)
    if not inst.components.spell.target then
        return
    end
    inst.components.spell.target.rainbowlight = nil
    inst.components.spell.target.AnimState:ClearBloomEffectHandle()
end

local function rainbowlight_fn()
    local inst = CreateEntity()
    
    inst.entity:AddTransform()
    inst.entity:AddLight()

    inst.Light:Enable(true)

    inst:AddTag("FX")
    inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
	--Note: the spell component allows the absent of the tag "INTERIOR_LIMBO_IMMUNE" if target is the player
	--inst:AddTag("INTERIOR_LIMBO_IMMUNE")
	
    inst:AddComponent("lighttweener")
	inst.components.lighttweener.light = inst.Light
	
    inst:AddComponent("spell")
    inst.components.spell.spellname = "rainbowlight"
    inst.components.spell:SetVariables({ radius = 5 })
    inst.components.spell.duration = math.huge
    inst.components.spell.ontargetfn = light_ontarget
    inst.components.spell.onstartfn = light_start
    inst.components.spell.onfinishfn = light_onfinish
    inst.components.spell.fn = light_spellfn
    inst.components.spell.resumefn = light_resume
    inst.components.spell.removeonfinish = true   	
	
	return inst
end

return Prefab("charlie_throne", fn, assets, prefabs), 
    Prefab("charlie_shadow_flower", flower_fn, flower_assets, flower_prefabs),
	Prefab("horrorfuel", horrorfuel_fn, horrorfuel_assets, horrorfuel_prefabs),
	Prefab("cane_flower", caneflower_fn, caneflower_assets, caneflower_prefabs),
	Prefab("cane_flower_fx", canflowerfx_fn, caneflowerfx_assets, caneflowerfx_prefabs),
	Prefab("supertelestaff", supertelestaff_fn, supertelestaff_assets, supertelestaff_prefabs),
	Prefab("purenightsword", purenightsword_fn, purenightsword_assets, purenightsword_prefabs),
	Prefab("charlie_shadow_rift", rift_fn, rift_assets, rift_prefabs),
	Prefab("charlie_shadow_rift_fx", rift_fx_fn, rift_assets, rift_prefabs),
	Prefab("rainbowsword", rainbowsword_fn, rainbowsword_assets, rainbowsword_prefabs),
	Prefab("rainbowlight", rainbowlight_fn, rainbowsword_assets, rainbowsword_prefabs)