require("prefabutil")

local CharlieValentineDayScreen = require("screens/charlievalentinedayscreen")

local heartassets = 
{
    Asset("ANIM", "anim/charlie_heart.zip"),
}

local invheartassets = 
{
    Asset("ANIM", "anim/charlie_heart_inv.zip"),
}

local seedheartassets = 
{
    Asset("ANIM", "anim/charlie_heart_seed.zip"),
    Asset("ANIM", "anim/charlie_heart.zip"),	
}

local candyassets = 
{
    Asset("ANIM", "anim/charlie_candy.zip"),
	Asset("ANIM", "anim/swap_charlie_candy.zip"),
}

local coffeeassets = 
{
    Asset("ANIM", "anim/charlie_coffee.zip"),
}

local heartcoreassets = 
{
    Asset("ANIM", "anim/charlie_heart_core.zip"),
}

local taskroseassets = 
{
    Asset("ANIM", "anim/charlie_rose_2024.zip"),
}

local wineassets = 
{
    Asset("ANIM", "anim/charlie_wine.zip"),
}

local rosegroupassets = 
{
    Asset("ANIM", "anim/charlie_rose_group.zip"),
}

local fishingrodassets =
{
    Asset("ANIM", "anim/valentine_day_fishing_rod.zip"),
	Asset("ANIM", "anim/swap_valentine_day_fishing_rod.zip"),
}

local fxassets = 
{
    Asset("ANIM", "anim/charlie_valentine_day_fx.zip"),
}

local prefabs = 
{

}

local function heartonpickedfn(inst, picker)
    inst.AnimState:PlayAnimation("pick")	
	inst.AnimState:PushAnimation("picked")	
    inst.MiniMapEntity:SetIcon("charlie_heart_picked.tex")
end

local function heartonregenfn(inst)
    inst.AnimState:PlayAnimation("idle")	
    inst.MiniMapEntity:SetIcon("charlie_heart.tex")
end

local function heartmakeemptyfn(inst)
    inst.AnimState:PlayAnimation("picked")	
    inst.MiniMapEntity:SetIcon("charlie_heart_picked.tex")
end

local function OnIgnite(inst)
    --Extinguish itself
	inst:DoTaskInTime(math.random(), function()
	    if inst.components.burnable ~= nil then   
			SpawnPrefab("charlie_valentine_day_fx").Transform:SetPosition(inst.Transform:GetWorldPosition())	
			inst.components.burnable:Extinguish()
		end
	end)
end

local function HeartOnFinished(inst, worker)
    if worker ~= nil and worker:HasTag("sweetheart") then 
        if inst.components.pickable ~= nil and inst.components.lootdropper ~= nil then
            if inst.components.pickable:CanBePicked() then
                inst.components.lootdropper:SpawnLootPrefab("charlie_heart_inv")
            end
            inst.components.lootdropper:SpawnLootPrefab("charlie_heart_seed")
        end
        inst:Remove()
	else
	    inst.components.workable:SetWorkLeft(1)
	end
end

local function CheckHeartType(inst)
    local HeartTypeIsWater = false
	
	local x, y, z = inst.Transform:GetWorldPosition()
	local puddle = TheSim:FindEntities(x, y, z, 2.5, {"puddle"}, {"puddle_disappear"})
    local lilypad = TheSim:FindEntities(x, y, z, 2.5, {"LilyPadForPuddle"})	
    if #puddle > 0 and #lilypad <= 0 then
	    HeartTypeIsWater = true 
	end
	
	if inst.GetIsOnWater ~= nil and inst:GetIsOnWater() then 
	    HeartTypeIsWater = true
	end 
	
    if HeartTypeIsWater then
	    if inst.float_fx == nil then
		    inst.float_fx = SpawnPrefab("float_fx_front")
			inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)
		    inst.float_fx.entity:SetParent(inst.entity) 
    	end
	    if inst.float_fx2 == nil then
		    inst.float_fx2 = SpawnPrefab("float_fx_back")
			inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
	    	inst.float_fx2.entity:SetParent(inst.entity) 
    	end		
	else
	    if inst.float_fx ~= nil then
		    inst.float_fx:Remove()
		    inst.float_fx = nil
    	end
	    if inst.float_fx2 ~= nil then
		    inst.float_fx2:Remove()
	     	inst.float_fx2 = nil
	    end	
    end
end

local function plantheartfn()
    local inst = CreateEntity()
	
	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
    inst.entity:AddMiniMapEntity()
	
	MakeObstaclePhysics(inst, 0.15)
	
	inst:AddTag("shadow_destroy_immune")
	inst:AddTag("heart_plant")
	
	inst.MiniMapEntity:SetIcon("charlie_heart.tex")
	
    inst.AnimState:SetBank("charlie_heart")
    inst.AnimState:SetBuild("charlie_heart")
    inst.AnimState:PlayAnimation("idle")	
	inst.AnimState:SetLightOverride(0.5)
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("lootdropper")
		
    inst:AddComponent("pickable")
    inst.components.pickable:SetUp("charlie_heart_inv", TUNING.TOTAL_DAY_TIME*3) 	
    inst.components.pickable.picksound = "dontstarve/wilson/harvest_berries"
    inst.components.pickable.onpickedfn = heartonpickedfn
    inst.components.pickable.onregenfn = heartonregenfn
	inst.components.pickable.makeemptyfn = heartmakeemptyfn
	
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.DIG)
    inst.components.workable:SetWorkLeft(1)
	inst.components.workable:SetOnFinishCallback(HeartOnFinished)
	
    MakeLargeBurnable(inst, 99)
    MakeMediumPropagator(inst)
    MakeSnowCovered(inst)
	
	inst.components.burnable:SetOnIgniteFn(OnIgnite)
	
	inst:DoTaskInTime(0, CheckHeartType)
	inst.CheckHeartType = CheckHeartType
	
	return inst
end 

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity) 
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity) 
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function inventoryheartfn()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
	
	MakeInventoryPhysics(inst)
	
    inst:AddTag("plant")
	inst:AddTag("shadow_destroy_immune")
	inst:AddTag("valid_for_dragon_weapon")

    inst.AnimState:SetBank("charlie_heart_inv")
    inst.AnimState:SetBuild("charlie_heart_inv")
    inst.AnimState:PlayAnimation("idle")
	inst.AnimState:SetLightOverride(0.25)
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

    if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 8		
    end

    inst:AddComponent("fuel")
    inst.components.fuel.fuelvalue = TUNING.LARGE_FUEL
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = 10
	
	inst:AddComponent("repairer")
	inst.components.repairer.repairmaterial = "charlieheart"
	inst.components.repairer.perishrepairvalue = 0.5	

    MakeSmallBurnable(inst, 99)
    MakeSmallPropagator(inst)
	
	inst.components.burnable:SetOnIgniteFn(OnIgnite)
	
	return inst
end 

local function HeartOnDeploy(inst, pt, deployer) 
    local charlie_heart = SpawnPrefab("charlie_heart")
	charlie_heart.Transform:SetPosition(pt.x, pt.y, pt.z)
	if charlie_heart.components.pickable ~= nil then
		charlie_heart.components.pickable:MakeEmpty()
	end
	if deployer ~= nil then
	    if deployer.SoundEmitter ~= nil then
            deployer.SoundEmitter:PlaySound("dontstarve/common/plant")
		end
	end	
	inst = inst.components.stackable:Get()    
	inst:Remove()
end

local function CanPlantAtPoint(x, y, z)
    local tile = GetWorld().Map:GetTileAtPoint(x, y, z)
    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    if tile == GROUND.ASH or tile == GROUND.VOLCANO 
		    or tile == GROUND.VOLCANO_ROCK or tile == GROUND.BRICK_GLOW
			or tile == GROUND.VOLCANO_LAVA or tile == GROUND.SNAKESKIN
		then
		    return false
		end
	end
	if IsDLCEnabled(PORKLAND_DLC) then
	    if tile == GROUND.FOUNDATION or tile == GROUND.COBBLEROAD
		    or tile == GROUND.GASJUNGLE or tile == GROUND.PIGRUINS
			or tile == GROUND.PIGRUINS_NOCANOPY or tile == GROUND.BEARDRUG or tile == GROUND.INTERIOR
		then                                                              
		    return false
		end
	end
    if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and tile ~= GROUND.ROCKY and tile ~= GROUND.ROAD
		and tile ~= GROUND.UNDERROCK and tile ~= GROUND.WOODFLOOR and tile ~= GROUND.CARPET and tile ~= GROUND.CHECKER
		and tile < GROUND.UNDERGROUND
	then
		return true
	end
	return false
end

local function HeartPlantTest(inst, pt, deployer, rot)
	if CanPlantAtPoint(pt.x, pt.y, pt.z) then
	    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 4, nil, {"NOBLOCK", "player", "FX"})
		local min_spacing = inst.components.deployable.min_spacing or 2
	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt ) < min_spacing*min_spacing then
					return false
				end
			end
		end
		return true
	end
	return false
end

local function seedheartfn()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
	
	MakeInventoryPhysics(inst)

    inst:AddTag("plant")
	inst:AddTag("shadow_destroy_immune")
	inst:AddTag("suitable_for_puddle_all")

    inst.AnimState:SetBank("charlie_heart_seed")
    inst.AnimState:SetBuild("charlie_heart_seed")
    inst.AnimState:PlayAnimation("idle")
	inst.AnimState:SetLightOverride(0.25)
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

    if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		      

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 8
    end
	
    inst:AddComponent("deployable")
    inst.components.deployable.ondeploy = HeartOnDeploy
	inst.components.deployable.test = HeartPlantTest
	inst.components.deployable.placer = "charlie_heart_placer"
	inst.components.deployable.min_spacing = 3
	inst.components.deployable.deploydistance = 0.5

    inst:AddComponent("fuel")
    inst.components.fuel.fuelvalue = TUNING.LARGE_FUEL
	
	inst:AddComponent("repairer")
	inst.components.repairer.repairmaterial = "charlieheart"
	inst.components.repairer.perishrepairvalue = 0.5
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = 20

    MakeSmallBurnable(inst, 99)
    MakeSmallPropagator(inst)
	
	inst.components.burnable:SetOnIgniteFn(OnIgnite)
	
	return inst
end 

local function onequipcandy(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_charlie_candy", "swap_charlie_candy") 
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function onunequipcandy(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

local function CandyOnEaten(inst, eater)
    if eater ~= nil and eater.components.inventory ~= nil and not eater.components.inventory:IsFull() then
	    eater.components.inventory:GiveItem(SpawnPrefab("twigs")) 
	elseif inst.components.inventoryitem ~= nil and inst.components.inventoryitem:GetGrandOwner() then
	    local x, y, z = inst.components.inventoryitem:GetGrandOwner().Transform:GetWorldPosition()
		local twigs = SpawnPrefab("twigs")
		SpawnPrefab("small_puff").Transform:SetPosition(x, y, z)
		twigs.Transform:SetPosition(x, y, z)
		if twigs.GetIsOnWater ~= nil
		    and twigs:GetIsOnWater()
		then
		    if twigs.components.floatable ~= nil then
			    twigs.components.floatable:OnHitWater(true)
			elseif twigs.components.inventoryitem ~= nil then
		        twigs.components.inventoryitem:OnHitWater()
		    end			
		end
	else
	    local instx, insty, instz = inst.Transform:GetWorldPosition()
		local twigs = SpawnPrefab("twigs")
		SpawnPrefab("small_puff").Transform:SetPosition(instx, insty, instz)
		twigs.Transform:SetPosition(instx, insty, instz)	
		if twigs.GetIsOnWater ~= nil
		    and twigs:GetIsOnWater()
		then
		    if twigs.components.floatable ~= nil then
			    twigs.components.floatable:OnHitWater(true)
			elseif twigs.components.inventoryitem ~= nil then
		        twigs.components.inventoryitem:OnHitWater()
		    end			
		end		
	end
end

local function CandyOnAttack(inst, attacker, target)
	if inst.bonus_damage > 0 then
	    if target ~= nil 
	        and target:IsValid() 
	        and target.components.health ~= nil
	    then	
		    target.components.health:SetVal(target.components.health.currenthealth - inst.bonus_damage, inst.prefab)	
		end
	end
    inst.bonus_damage = math.min(inst.bonus_damage + 5, inst.bonus_damage_max)
end

local function HasTargetOrIsTargeted(inst)
    if inst ~= nil then 
	    if inst.components.combat ~= nil and inst.components.combat.target ~= nil then 
	        return true 
	    end 
	    local x, y, z = inst.Transform:GetWorldPosition() 
    	for k,v in pairs(TheSim:FindEntities(x, y, z, 10, {"hascombatcomponent"}, {"FX", "NOCLICK", "INLIMBO", "DECOR", "INTERIOR_LIMBO", "invisible", "notarget"})) do 
	        if v ~= nil 
		        and v:IsValid() 
		    	and v.entity:IsVisible() 
		    	and v.components.combat ~= nil 
		        and v.components.combat.target ~= nil 
		    	and v.components.combat.target == inst 
		    then 
		        return true
	    	end 
    	end 	
	end 
    return false
end 

local function CheckCandyWeaponDamage(inst)
    if inst.bonus_damage > 0 then
	    if inst.components.inventoryitem ~= nil and inst.components.inventoryitem:GetGrandOwner() then
		    if not HasTargetOrIsTargeted(inst.components.inventoryitem:GetGrandOwner()) then
			    inst.bonus_damage = math.max(inst.bonus_damage - 5, 0)
			end 
		else
		    inst.bonus_damage = math.max(inst.bonus_damage - 5, 0)
		end
	end 
end 

local function OnSaveCandy(inst, data)
    data.bonus_damage = inst.bonus_damage
end

local function OnLoadCandy(inst, data)
    if data ~= nil then
	    inst.bonus_damage = data.bonus_damage or 0
	end
end

local function candyfn()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)

    inst:AddTag("sharp")
	inst:AddTag("shadow_destroy_immune")
	
    inst.AnimState:SetBank("charlie_candy")
    inst.AnimState:SetBuild("charlie_candy")
    inst.AnimState:PlayAnimation("idle")	
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

    if inst.GetIsOnWater ~= nil then
	    MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		      

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 8
    end

    inst:AddComponent("repairable")
    inst.components.repairable.repairmaterial = "charlieheart"
	inst.components.repairable.announcecanfix = false
	
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequipcandy)
    inst.components.equippable:SetOnUnequip(onunequipcandy)
	
	inst:AddComponent("bait")
	
	inst:AddComponent("tradable")
	
 	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.TOTAL_DAY_TIME*10)
	inst.components.perishable:StartPerishing()
	inst.components.perishable.onperishreplacement = "twigs"
	
	inst:AddComponent("edible")
	inst.components.edible.foodtype = "VEGGIE"
	inst.components.edible.healthvalue = 0
	inst.components.edible.hungervalue = 30
	inst.components.edible.sanityvalue = 15
	inst.components.edible.oneaten = CandyOnEaten

    MakeSmallBurnable(inst, 99)
    MakeSmallPropagator(inst)
	
	inst.components.burnable:SetOnIgniteFn(OnIgnite)
	
	inst.bonus_damage = 0
	inst.bonus_damage_max = 85
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(17)
	inst.components.weapon:SetOnAttack(CandyOnAttack)
	
	inst:DoPeriodicTask(10, CheckCandyWeaponDamage)
	
	inst.OnSave = OnSaveCandy
	inst.OnLoad = OnLoadCandy
	
	return inst
end 

local function DrinkerCharlieCoffeeEffectDone(inst)
    if inst.UpdateCharlieLocomotor ~= nil then
	    inst:UpdateCharlieLocomotor() 
	else
        if inst.components.locomotor ~= nil then 
            inst.components.locomotor.walkspeed = inst.OriginalWalkSpeed or 4
            inst.components.locomotor.runspeed = inst.OriginalRunSpeed or 6
        end		
	end 
	inst.OriginalWalkSpeed = nil
	inst.OriginalRunSpeed = nil
    if inst.charliecoffeetask then
	    inst.charliecoffeetask:Cancel()
		inst.charliecoffeetask = nil 
	end 
	if inst.charliecoffeeinfo then
	    inst.charliecoffeeinfo = nil 
	end
end 

local function CoffeeOnDrunk(inst, drinker)
    if drinker ~= nil then 
	    if inst.GetIsOnWater == nil and drinker.components.locomotor ~= nil then
		    --Note: need to handle via drinker prefab file in order to save the effect 
			if drinker.charliecoffeetask == nil then 
			    inst.OriginalWalkSpeed = inst.components.locomotor.walkspeed
		    	inst.OriginalRunSpeed = inst.components.locomotor.runspeed
		        drinker.components.locomotor.walkspeed = inst.components.locomotor.walkspeed + 6 
		    	drinker.components.locomotor.runspeed = inst.components.locomotor.runspeed + 6 
			    drinker.charliecoffeetask, drinker.charliecoffeeinfo = drinker:ResumeTask(TUNING.TOTAL_DAY_TIME*2, DrinkerCharlieCoffeeEffectDone)
			end 
		end  
		if not IsDLCEnabled(REIGN_OF_GIANTS) and not IsDLCEnabled(CAPY_DLC) and not IsDLCEnabled(PORKLAND_DLC) then
		    if drinker.components.temperature ~= nil then			
			    drinker.components.temperature:DeltaTemp(inst.components.edible.temperaturedelta + inst.components.edible.temperaturebump)
		    end
		end
		if not IsDLCEnabled(PORKLAND_DLC) then
		    if drinker.components.temperature ~= nil then			
			    drinker.components.temperature:DeltaTemp(inst.components.edible.temperaturebump)
		    end		
		end 
	end 
end

local function coffeefn()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

	MakeInventoryPhysics(inst)
	
	inst:AddTag("preparedfood")
	inst:AddTag("shadow_destroy_immune")

    inst.AnimState:SetBank("charlie_coffee")
    inst.AnimState:SetBuild("charlie_coffee")
    inst.AnimState:PlayAnimation("idle")

    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

    if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		 

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 8		
    end	
	
	inst:AddComponent("edible")
	inst.components.edible.foodtype = "VEGGIE"
	inst.components.edible.foodstate = "PREPARED"
	inst.components.edible.healthvalue = 20
	inst.components.edible.hungervalue = 12.5
	inst.components.edible.sanityvalue = 1
	inst.components.edible.temperaturedelta = 45
	inst.components.edible.temperatureduration = 30
	inst.components.edible.temperaturebump = 20
	inst.components.edible.caffeinedelta = 6
	inst.components.edible.caffeineduration = TUNING.TOTAL_DAY_TIME*2
	inst.components.edible.antihistamine = 375
    inst.components.edible.oneaten = CoffeeOnDrunk

    inst:AddComponent("bait")	
	
	inst:AddComponent("tradable")
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = 40
	
	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.TOTAL_DAY_TIME*10)
	inst.components.perishable:StartPerishing()
	inst.components.perishable.onperishreplacement = "taffy"
	
    MakeSmallBurnable(inst, 99)
    MakeSmallPropagator(inst)
	
	inst.components.burnable:SetOnIgniteFn(OnIgnite)	
	
	return inst
end 

local function coredescription(inst, viwer)
    local desc = ""
    if TUNING.LANGUAGE == true then
		if inst.level == 1 then
            desc = "It's incomplete, I need to find a way to fix it."
        elseif inst.level == 2 then
            desc = "Further repairs would be better."
        elseif inst.level == 3 then
            desc = "Perfect! Now I can acquire Valentine's Day knowledge.\nPressing and holding down ctrl can extract existing ingredients."
        end		
	elseif TUNING.LANGUAGE == false then
		if inst.level == 1 then
            desc = "它不完整，我得想办法修复它。"
        elseif inst.level == 2 then
            desc = "进一步的修复会更好。"
        elseif inst.level == 3 then
            desc = "完美！现在我可以获取情人节知识了。\n按住ctrl可以将已有的原料提取出来。"
        end	
	end
	return desc
end

local function LevelTwoToThree(inst)
    inst.level = 3
    inst.AnimState:PlayAnimation("idle3")
	inst.MiniMapEntity:SetIcon("charlie_heart_core3.tex")
    inst.SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
end

local function LevelOneToTwo(inst)
    inst.level = 2
    inst.AnimState:PlayAnimation("idle2")
	inst.MiniMapEntity:SetIcon("charlie_heart_core2.tex")
    inst.SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
end

local function TryToUpgrade(inst, giver, item)
    inst.TryToUpgrade = true
	inst:DoTaskInTime(0.5, function() inst.TryToUpgrade = nil end)
	if inst.level == 1 then
	    LevelOneToTwo(inst)
        if giver.components.talker ~= nil then
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("Great achievement! But more construction is still needed.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say("大功告成！但还是需要更多的建设。")
		    end
		end	
	elseif inst.level == 2 then
	    LevelTwoToThree(inst)
        if giver.components.talker ~= nil then
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("Complete form! Now I can use the knowledge inside it.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say("完整形态！现在我可以用它里面的知识了。")
		    end
		end	
	end
end

local function CoreOnRefuse(inst, giver, item)
    if giver ~= nil then
        if giver:HasTag("fear_of_darkness") then
            if giver.components.talker ~= nil then
	            if TUNING.LANGUAGE == true then
				    if item ~= nil 
					    and (item.prefab == "rose_inv" or (inst.level == 3 and (item.prefab == "taffy" or item.prefab == "honey"))) 
						and item.components.perishable ~= nil 
						and item.components.perishable:IsSpoiled() then
					    giver.components.talker:Say("The item is not fresh enough.")
				    elseif item ~= nil and inst.rose_inside >= 40 and inst.mosquitosack_inside < 50 and item.prefab == "rose_inv" then
					    giver.components.talker:Say("Blood is needed.")
					elseif item ~= nil and inst.mosquitosack_inside >= 50 and inst.rose_inside < 40 and (item.prefab == "mosquitosack" or item.prefab == "mosquitosack_yellow") then
					    giver.components.talker:Say("Roses are needed.")
					elseif item ~= nil and item.prefab == "taffy" and inst.taffy_inside > 0 then
					    giver.components.talker:Say("The ingredients (taffy) are already filled up.")
					elseif item ~= nil and item.prefab == "honey" and inst.honey_inside >= 3 then
					    giver.components.talker:Say("The ingredients (honey) are already filled up.")
					else
		                giver.components.talker:Say("Blood roses are needed.")
					end
		        elseif TUNING.LANGUAGE == false then
				    if item ~= nil 
					    and (item.prefab == "rose_inv" or (inst.level == 3 and (item.prefab == "taffy" or item.prefab == "honey"))) 
						and item.components.perishable ~= nil 
						and item.components.perishable:IsSpoiled() then
					    giver.components.talker:Say("物品不够新鲜。")
				    elseif item ~= nil and inst.rose_inside >= 40 and inst.mosquitosack_inside < 50 and item.prefab == "rose_inv" then
					    giver.components.talker:Say("需要血。")
					elseif item ~= nil and inst.mosquitosack_inside >= 50 and inst.rose_inside < 40 and (item.prefab == "mosquitosack" or item.prefab == "mosquitosack_yellow") then
					    giver.components.talker:Say("需要玫瑰。")
					elseif item ~= nil and item.prefab == "taffy" and inst.taffy_inside > 0 then
					    giver.components.talker:Say("该原料（太妃糖）已经填充满了。")
					elseif item ~= nil and item.prefab == "honey" and inst.honey_inside >= 3 then
					    giver.components.talker:Say("该原料（蜂蜜）已经填充满了。")
					else
		                giver.components.talker:Say("需要血玫瑰。")
				    end
		        end				
		    end	
		else
            if giver.components.talker ~= nil then
	            if TUNING.LANGUAGE == true then
		            giver.components.talker:Say("It seems to reject me.")
		        elseif TUNING.LANGUAGE == false then
		            giver.components.talker:Say("它似乎排斥我。")
		        end
		    end
		end
	end
end

local function CoreOnAccept(inst, giver, item)
    if giver ~= nil and giver:HasTag("fear_of_darkness") then

        if inst.level == 3 then
            local x, y, z = inst.Transform:GetWorldPosition()
	        local possible_items = 
		    { 
			    "charlie_heart_seed",
			    "charlie_heart_seed",
			    "charlie_coffee",
				"charlie_coffee",
				"charlie_heart_inv",
				"charlie_heart_inv",
				"charlie_heart_inv",
				"charlie_heart_inv",
				"charlie_heart_inv",
				"honey",
				"honey",
				"honey",				
				"honey",
			    "honey",				
				"honeycomb",			
		    }
			if item ~= nil and (item.prefab == "mosquitosack" or item.prefab == "mosquitosack_yellow") then
			    possible_items = 
				{
			        "charlie_heart_seed",
			        "charlie_coffee",
			        "charlie_coffee",
			        "charlie_coffee",
			    	"charlie_coffee",
			    	"honeycomb",
			    	"honeycomb",
			    	"honey",
			    	"honey",
				    "honey",
			    	"taffy",	
			    	"taffy",		
					"charlie_heart_inv",
				    "bandage",		
				    "bee",							
				}
			end
			if item ~= nil  then
			    if item.prefab ~= "taffy" and item.prefab ~= "honey" then
                    local rewarditem = SpawnPrefab(GetRandomItem(possible_items))
	                inst.components.lootdropper:FlingItem(rewarditem, Vector3(x, math.random()*3, z))
					if math.random() < 0.25 then
					    if GetCharlieTaskRose() then
						    GetCharlieTaskRose():SpawnRoseGroup()
						end
					elseif math.random() < 0.25 then
					    if GetCharlieTaskRose() then
						    GetCharlieTaskRose():SpawnWine()
						end					
					end			
					item:Remove()
				elseif item.prefab == "taffy" then
				    inst.taffy_inside = math.min(inst.taffy_inside + 1, 1)
				elseif item.prefab == "honey" then
				    inst.honey_inside = math.min(inst.honey_inside + 1, 3)
				end
			end

		    if not TheSim:FindFirstEntityWithTag("rainbowsword") and math.random() < 0.01 then	
			    if giver.components.inventory ~= nil then
                    giver.components.inventory:GiveItem(SpawnPrefab("rainbowsword"), nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
				else
				    SpawnPrefab("collapse_small").Transform:SetPosition(giver.Transform:GetWorldPosition())
				    SpawnPrefab("rainbowsword").Transform:SetPosition(giver.Transform:GetWorldPosition())
				end
            end			

            if not inst.giverunlockedroserecipe and math.random() < 0.2 then	
                if giver.components.builder ~= nil then
				    giver.components.builder:UnlockRecipe("rose_inv")
				end
				inst.giverunlockedroserecipe = true
            end		

            if not inst.giverunlockedcandyrecipe and math.random() < 0.15 then	
                if giver.components.builder ~= nil then
				    giver.components.builder:UnlockRecipe("charlie_candy")
				end
                inst.giverunlockedcandyrecipe = true
            end	

            if not inst.giverunlockedcastlerecipe and math.random() < 0.05 and IsDLCEnabled(PORKLAND_DLC) then	
                if giver.components.builder ~= nil then
				    giver.components.builder:UnlockRecipe("charlie_castle")
				end
                inst.giverunlockedcastlerecipe = true
            end						
		
		    return
		end
		
		if item ~= nil then
		    if item.prefab == "rose_inv" then
	            inst.rose_inside = math.min(inst.rose_inside + 1, 40)
			elseif item.prefab == "mosquitosack" or item.prefab == "mosquitosack_yellow" then
			    inst.mosquitosack_inside = math.min(inst.mosquitosack_inside + 1, 50)
			end
		end
		
        if not inst.giverunlockedroserecipe and math.random() < 0.02 then	
            if giver.components.builder ~= nil then
			    giver.components.builder:UnlockRecipe("rose_inv")
			end
			inst.giverunlockedroserecipe = true
        end		

        if not inst.giverunlockedcandyrecipe and math.random() < 0.01 then	
            if giver.components.builder ~= nil then
	    	    giver.components.builder:UnlockRecipe("charlie_candy")
			end
            inst.giverunlockedcandyrecipe = true
        end			
		
		if (inst.level == 1 and inst.rose_inside >= 20 and inst.mosquitosack_inside >= 25)
		    or (inst.level == 2 and inst.rose_inside == 40 and inst.mosquitosack_inside == 50)
		then
		    TryToUpgrade(inst, giver, item)
		end	
		
        if (inst.rose_inside < 40 or inst.mosquitosack_inside < 50) and not inst.TryToUpgrade then		    
            if giver.components.talker ~= nil then
	            if TUNING.LANGUAGE == true then
		            giver.components.talker:Say("I need more things to fix it.")
		        elseif TUNING.LANGUAGE == false then
		            giver.components.talker:Say("我需要更多东西来修复它。")
		        end
		    end		
		end
		
	end
	if item ~= nil then
	    item:Remove()
	end
end

local function CoreAcceptTest(inst, item, giver)
    if giver ~= nil and not giver:HasTag("fear_of_darkness") then
	    return false
	end
	if item ~= nil and item.components.perishable ~= nil and item.components.perishable:IsSpoiled() then
	    return false
	end 
	if item ~= nil 
	    and (
		        (inst.rose_inside >= 40 and inst.mosquitosack_inside < 50 and item.prefab == "rose_inv") 
	            or (
				        inst.mosquitosack_inside >= 50 and inst.rose_inside < 40 and (item.prefab == "mosquitosack" or item.prefab == "mosquitosack_yellow")
					)
			)
	then
	    return false
	end 
	if inst.level == 3 
	    and item ~= nil 
		and item.prefab == "taffy"
	    and inst.taffy_inside == 0
	then
	    return true
	end
	if inst.level == 3 
	    and item ~= nil 
		and item.prefab == "honey"
	    and inst.honey_inside < 3
	then
	    return true
	end	
    return item ~= nil 
	    and item.prefab ~= nil 
		and (item.prefab == "rose_inv" 
		        or item.prefab == "mosquitosack" 
				or item.prefab == "mosquitosack_yellow")
end

local function ItemInsideCorePerishRateFn(inst, item)
    if item ~= nil and (item.prefab == "taffy" or item.prefab == "honey") then
	    return 0
	end
    return 1
end

local function CoreOnSave(inst, data)
    data.level = inst.level
	data.rose_inside = inst.rose_inside
	data.mosquitosack_inside = inst.mosquitosack_inside
	data.taffy_inside = inst.taffy_inside
	data.honey_inside = inst.honey_inside
	data.giverunlockedroserecipe = inst.giverunlockedroserecipe
	data.giverunlockedcastlerecipe = inst.giverunlockedcastlerecipe
	data.giverunlockedcandyrecipe = inst.giverunlockedcandyrecipe
end

local function CoreOnLoad(inst, data)
    if data ~= nil then    
        inst.level = data.level or 1
		inst.rose_inside = data.rose_inside or 0
		inst.mosquitosack_inside = data.mosquitosack_inside or 0
		inst.taffy_inside = data.taffy_inside or 0
		inst.honey_inside = data.honey_inside or 0
		inst.giverunlockedroserecipe = data.giverunlockedroserecipe or nil
		inst.giverunlockedcastlerecipe = data.giverunlockedcastlerecipe or nil
		inst.giverunlockedcandyrecipe = data.giverunlockedcandyrecipe or nil
		if inst.level == 2 then
		    inst.AnimState:PlayAnimation("idle2")
			inst.MiniMapEntity:SetIcon("charlie_heart_core2.tex")
		elseif inst.level == 3 then
		    inst.AnimState:PlayAnimation("idle3")
			inst.MiniMapEntity:SetIcon("charlie_heart_core3.tex")
		end
	end
end

local function heartcorefn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddSoundEmitter()
	
    inst:AddTag("charlie_heart_core")
    inst:AddTag("shadow_destroy_immune")
	inst:AddTag("pugalisk_avoids")
	
    MakeObstaclePhysics(inst, 0.25)

    inst.MiniMapEntity:SetIcon("charlie_heart_core1.tex")

    inst.AnimState:SetBank("charlie_heart_core")
    inst.AnimState:SetBuild("charlie_heart_core")
    inst.AnimState:PlayAnimation("idle1")
	inst.AnimState:SetLightOverride(1)	
	
	inst.Transform:SetScale(3, 3, 3)
	
	inst.level = 1
	inst.rose_inside = 0
	inst.mosquitosack_inside = 0
	inst.taffy_inside = 0
	inst.honey_inside = 0
	
    inst:AddComponent("inspectable")
	inst.components.inspectable:SetDescription(coredescription)
	
	inst:AddComponent("inventory")
	
	inst:AddComponent("trader")
	inst.components.trader.acceptnontradable = true
	inst.components.trader.onrefuse = CoreOnRefuse
	inst.components.trader.onaccept = CoreOnAccept
    inst.components.trader:SetAcceptTest(CoreAcceptTest)
	
	inst:AddComponent("lootdropper")
	
	inst:AddComponent("charliecoffeemaker")
	
	inst:AddComponent("preserver")
	inst.components.preserver:SetPerishRateMultiplier(ItemInsideCorePerishRateFn)

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end
	
    MakeSnowCovered(inst)
	
    inst.OnSave = CoreOnSave
    inst.OnLoad = CoreOnLoad	

	return inst
end

local function taskroseonpickedfn(inst, picker)
    inst.IsPicked = true
    inst.AnimState:PlayAnimation("idle_picked")
	inst.MiniMapEntity:SetIcon("charlie_rose_2024_picked.tex")
	if picker ~= nil then
	    if picker.components.sanity ~= nil and not picker:HasTag("plantkin") then
		    if picker:HasTag("flower_lover") then
		        picker.components.sanity:DoDelta(TUNING.SANITY_TINY*3)
			else
			    picker.components.sanity:DoDelta(TUNING.SANITY_TINY)
			end
	    end
        if picker.components.combat ~= nil 
	        and not picker:HasTag("flower_lover") 
	        and not picker:HasTag("bramble_resistant") 
	        --[[and not picker:HasTag("armorcactus")]]
    	then
            picker.components.combat:GetAttacked(inst, TUNING.ROSE_DAMAGE)
            picker:PushEvent("thorns")
        end		
	end
end

local function taskroseonregenfn(inst)
    if inst.AnimState:IsCurrentAnimation("idle_picked") then
	    inst.IsHalfGrow = true
	    inst.AnimState:PlayAnimation("idle_half")
		inst.MiniMapEntity:SetIcon("charlie_rose_2024_half.tex")
        inst.components.pickable:MakeEmpty()
	else
	    inst.IsHalfGrow = nil
	    inst.AnimState:PlayAnimation("idle")
		inst.MiniMapEntity:SetIcon("charlie_rose_2024.tex")
	end
	inst.IsPicked = nil
end

local function TaskRoseOnActivate(inst, doer)
    TheFrontEnd:PushScreen(CharlieValentineDayScreen()) 
	inst.components.interactivelevel.inactive = true
end

local function TaskRoseOnRefuse(inst, giver, item)
    if giver ~= nil then
        if giver:HasTag("charlie") then
            if giver.components.talker ~= nil then
	            if TUNING.LANGUAGE == true then
	                if inst.components.pickable ~= nil and not inst.components.pickable.canbepicked then
					    giver.components.talker:Say("No rose. Need to wait for it to grow.")
					elseif item ~= nil and inst.bubbletaskfinished and item.prefab == "charlie_valentine_day_bubble" then
					    giver.components.talker:Say("The item (bubble) required for this task have been given.")
					elseif item ~= nil and inst.coffeetaskfinished and item.prefab == "charlie_valentine_day_coffee" then
					    giver.components.talker:Say("The item (coffee) required for this task have been given.")
					elseif item ~= nil and inst.envelopetaskfinished and item.prefab == "charlie_valentine_day_envelope" then
					    giver.components.talker:Say("The item (envelope) required for this task have been given.")
					elseif item ~= nil and inst.gifttaskfinished and item.prefab == "charlie_valentine_day_gift" then
					    giver.components.talker:Say("The item (gift) required for this task have been given.")
					else
					    giver.components.talker:Say("Perhaps I need to interact with it to understand what it needs.")
					end
		        elseif TUNING.LANGUAGE == false then
	                if inst.components.pickable ~= nil and not inst.components.pickable.canbepicked then
					    giver.components.talker:Say("没有玫瑰。需要等待它生长。")
					elseif item ~= nil and inst.bubbletaskfinished and item.prefab == "charlie_valentine_day_bubble" then
					    giver.components.talker:Say("这个任务所需要的物品（气泡）已经被给予了。")
					elseif item ~= nil and inst.coffeetaskfinished and item.prefab == "charlie_valentine_day_coffee" then
					    giver.components.talker:Say("这个任务所需要的物品（咖啡）已经被给予了。")
					elseif item ~= nil and inst.envelopetaskfinished and item.prefab == "charlie_valentine_day_envelope" then
					    giver.components.talker:Say("这个任务所需要的物品（信封）已经被给予了。")
					elseif item ~= nil and inst.gifttaskfinished and item.prefab == "charlie_valentine_day_gift" then
					    giver.components.talker:Say("这个任务所需要的物品（礼物）已经被给予了。")
					else
					    giver.components.talker:Say("或许我要跟它互动一下来了解一下它需要什么。")
					end
		        end				
		    end	
		else
            if giver.components.talker ~= nil then
	            if TUNING.LANGUAGE == true then
		            giver.components.talker:Say("It seems to reject me.")
		        elseif TUNING.LANGUAGE == false then
		            giver.components.talker:Say("它似乎排斥我。")
		        end
		    end
		end
	end
end

local function TaskRoseOnAccept(inst, giver, item)
    if giver ~= nil and giver:HasTag("charlie") then
        if item ~= nil then
		    if item.prefab == "charlie_valentine_day_bubble" then
			    inst.bubbletaskfinished = true
				inst.components.lootdropper:SpawnLootPrefab("")
				inst.SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
			elseif item.prefab == "charlie_valentine_day_coffee" then
			    inst.coffeetaskfinished = true
				inst.components.lootdropper:SpawnLootPrefab("")
				inst.SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
			elseif item.prefab == "charlie_valentine_day_envelope" then
			    inst.envelopetaskfinished = true
				inst.components.lootdropper:SpawnLootPrefab("")
				inst.SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
			elseif item.prefab == "charlie_valentine_day_gift" then
			    inst.gifttaskfinished = true
				inst.components.lootdropper:SpawnLootPrefab("")
				inst.SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
			elseif item:HasTag("valid_for_full_task_rose") then
			    if giver.components.talker ~= nil then
				    giver.components.talker:Say(STRINGS.CHARLIE_TRADE_WITH_TASK_ROSE or "")
				end
				if giver.components.inventory ~= nil then
				    giver.components.inventory:GiveItem(SpawnPrefab("shadow_rose_bouquet"), nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
				end
			end
		end
		if item ~= nil 
		    and not item:HasTag("valid_for_full_task_rose") 
			and inst.bubbletaskfinished 
			and inst.coffeetaskfinished 
			and inst.envelopetaskfinished 
			and inst.gifttaskfinished 
		then
		    if inst.SpawnHeartCore ~= nil then
			    inst:SpawnHeartCore()
			end
			if inst.SpawnRoseGroup ~= nil then
			    inst:DoTaskInTime(0.25, function() inst:SpawnRoseGroup() end)
			end
			if inst.SpawnWine ~= nil then
			    inst:DoTaskInTime(0.5, function() inst:SpawnWine() end)
			end
		end
	end
end

local function TaskRoseAcceptTest(inst, item, giver)
    if giver ~= nil and not giver:HasTag("charlie") then
	    return false
	end
	if inst.components.pickable ~= nil and not inst.components.pickable.canbepicked then
	    return false
	end
	if item ~= nil and inst.bubbletaskfinished and item.prefab == "charlie_valentine_day_bubble" then
	    return false
	end 
    if item ~= nil and inst.coffeetaskfinished and item.prefab == "charlie_valentine_day_coffee" then
	    return false
	end
    if item ~= nil and inst.envelopetaskfinished and item.prefab == "charlie_valentine_day_envelope" then
	    return false
	end
	if item ~= nil and inst.gifttaskfinished and item.prefab == "charlie_valentine_day_gift" then
	    return false
	end
	if inst.bubbletaskfinished and inst.coffeetaskfinished and inst.envelopetaskfinished and inst.gifttaskfinished and item ~= nil and item:HasTag("valid_for_full_task_rose") then
	    return true
	end
    return item ~= nil and item:HasTag("charlie_valentineday_task_item")
end

local function TaskRoseGetInspectStatus(inst)
    return (inst.IsHalfGrow and "HALFGROW")
	    or (inst.IsPicked and "PICKED")
	    or "NORMAL"
end

local function SpawnHeartCore(inst)
	if not TheSim:FindFirstEntityWithTag("charlie_heart_core") then
	
	    local centers = {}
	    if GetWorld() and GetWorld().topology and GetWorld().topology.nodes and GetWorld().Map then
            for i, node in pairs(GetWorld().topology.nodes) do
			    if node ~= nil and node.x ~= nil and node.y ~= nil then
		            local tile = GetWorld().Map:GetTileAtPoint(node.x, 0, node.y)
		            local nearby_player = TheSim:FindEntities(node.x, 0, node.y, 200, {"player"}) 
				    local blockers = TheSim:FindEntities(node.x, 0, node.y, 3, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER"})
                    local IsPassableTile = true
			        local NotInInterior = true
                    if tile == GROUND.IMPASSABLE or tile == GROUND.INVALID or tile >= GROUND.UNDERGROUND then
		                IsPassableTile = false
	                end
			        if inst.IsPosSurroundedByLand ~= nil and not inst:IsPosSurroundedByLand(node.x, 0, node.y, 3) then
			            IsPassableTile = false
		    	    end 
			        if IsDLCEnabled(PORKLAND_DLC) then
				        if tile == GROUND.INTERIOR then
				            NotInInterior = false
				        end
		    	    end
                    if #nearby_player == 0 and #blockers == 0 and IsPassableTile and NotInInterior then
                        table.insert(centers, {x = node.x, z = node.y})
                    end
				end
            end
	    end 
        if #centers > 0 then
		    local spawn_position = centers[math.random(#centers)]
            SpawnPrefab("charlie_heart_core").Transform:SetPosition(spawn_position.x, 0, spawn_position.z)
	    end
	
	end
end

local function SpawnRoseGroup(inst)
	if not TheSim:FindFirstEntityWithTag("charlie_rose_group") then
	
	    local centers = {}
	    if GetWorld() and GetWorld().topology and GetWorld().topology.nodes and GetWorld().Map then
            for i, node in pairs(GetWorld().topology.nodes) do
			    if node ~= nil and node.x ~= nil and node.y ~= nil then
		            local tile = GetWorld().Map:GetTileAtPoint(node.x, 0, node.y)
		            local nearby_player = TheSim:FindEntities(node.x, 0, node.y, 200, {"player"}) 
				    local blockers = TheSim:FindEntities(node.x, 0, node.y, 3, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER"})
					local nearby_heart_core = TheSim:FindEntities(node.x, 0, node.y, 100, {"charlie_heart_core"})
					local is_plantable_tile = false
                    if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and tile ~= GROUND.ROCKY and tile ~= GROUND.ROAD
		                and tile ~= GROUND.UNDERROCK and tile ~= GROUND.WOODFLOOR and tile ~= GROUND.CARPET and tile ~= GROUND.CHECKER
		                and tile < GROUND.UNDERGROUND
	                then
		                is_plantable_tile = true
	                end
                    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	                    if tile == GROUND.ASH or tile == GROUND.VOLCANO 
		                    or tile == GROUND.VOLCANO_ROCK or tile == GROUND.BRICK_GLOW
			                or tile == GROUND.VOLCANO_LAVA or tile == GROUND.SNAKESKIN
		                then
		                    is_plantable_tile = false
		                end
	                end					
	                if IsDLCEnabled(PORKLAND_DLC) then
	                    if tile == GROUND.FOUNDATION or tile == GROUND.COBBLEROAD
		                    or tile == GROUND.GASJUNGLE or tile == GROUND.PIGRUINS
			                or tile == GROUND.PIGRUINS_NOCANOPY or tile == GROUND.BEARDRUG or tile == GROUND.INTERIOR
		                then                                                               
		                    is_plantable_tile = false
		                end
	                end						
                    if #nearby_player == 0 and #blockers == 0 and #nearby_heart_core == 0 and is_plantable_tile then
                        table.insert(centers, {x = node.x, z = node.y})
                    end
				end
            end
	    end 
        if #centers > 0 then
		    local spawn_position = centers[math.random(#centers)]
            SpawnPrefab("charlie_rose_group").Transform:SetPosition(spawn_position.x, 0, spawn_position.z)
	    end
	
	end
end

local function SpawnWine(inst)
	if not TheSim:FindFirstEntityWithTag("charlie_wine") then
	
	    local centers = {}
	    if GetWorld() and GetWorld().topology and GetWorld().topology.nodes and GetWorld().Map then
            for i, node in pairs(GetWorld().topology.nodes) do
			    if node ~= nil and node.x ~= nil and node.y ~= nil then
		            local tile = GetWorld().Map:GetTileAtPoint(node.x, 0, node.y)
		            local nearby_player = TheSim:FindEntities(node.x, 0, node.y, 200, {"player"}) 
				    local blockers = TheSim:FindEntities(node.x, 0, node.y, 2, nil, {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "PLAYER"})
					local nearby_heart_core = TheSim:FindEntities(node.x, 0, node.y, 100, {"charlie_heart_core"})
					local nearby_rose_group = TheSim:FindEntities(node.x, 0, node.y, 100, {"charlie_rose_group"})
                    local IsPassableTile = true
			        local NotInInterior = true
                    if tile == GROUND.IMPASSABLE or tile == GROUND.INVALID or tile >= GROUND.UNDERGROUND then
		                IsPassableTile = false
	                end
			        if inst.IsPosSurroundedByLand ~= nil and not inst:IsPosSurroundedByLand(node.x, 0, node.y, 3) then
			            IsPassableTile = false
		    	    end 
			        if IsDLCEnabled(PORKLAND_DLC) then
				        if tile == GROUND.INTERIOR then
				            NotInInterior = false
				        end
		    	    end
                    if #nearby_player == 0 and #blockers == 0 and #nearby_heart_core == 0 and #nearby_rose_group == 0 and IsPassableTile and NotInInterior then
                        table.insert(centers, {x = node.x, z = node.y})
                    end
				end
            end
	    end 
        if #centers > 0 then
		    local spawn_position = centers[math.random(#centers)]
            SpawnPrefab("charlie_wine").Transform:SetPosition(spawn_position.x, 0, spawn_position.z)
	    end
	
	end
end

local function RoseSanityAura(inst, observer)
    if inst.components.pickable ~= nil and not inst.components.pickable.canbepicked then
	    return 0
	end
    if observer ~= nil and observer:HasTag("flower_lover") then
        return TUNING.SANITYAURA_SMALL    
    end
    return 0
end

local function TaskRoseOnSave(inst, data)
    data.IsHalfGrow = inst.IsHalfGrow
	data.IsPicked = inst.IsPicked
	data.bubbletaskfinished = inst.bubbletaskfinished
	data.coffeetaskfinished = inst.coffeetaskfinished
	data.envelopetaskfinished = inst.envelopetaskfinished
	data.gifttaskfinished = inst.gifttaskfinished
end

local function TaskRoseOnLoad(inst, data)
    if data ~= nil then
	    if data.IsHalfGrow then
		    inst.IsHalfGrow = true
			inst.AnimState:PlayAnimation("idle_half")
			inst.MiniMapEntity:SetIcon("charlie_rose_2024_half.tex")
		end
	    if data.IsPicked then
		    inst.IsPicked = true
			inst.AnimState:PlayAnimation("idle_picked")
			inst.MiniMapEntity:SetIcon("charlie_rose_2024_picked.tex")
		end
		if data.bubbletaskfinished then
		    inst.bubbletaskfinished = true
		end
		if data.coffeetaskfinished then
		    inst.coffeetaskfinished = true
		end
		if data.envelopetaskfinished then
		    inst.envelopetaskfinished = true
		end
		if data.gifttaskfinished then
		    inst.gifttaskfinished = true
		end
	end
end

local function taskrosefn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddSoundEmitter()
	
    inst:AddTag("charlie_rose_valentine_day")
    inst:AddTag("shadow_destroy_immune")
	inst:AddTag("pugalisk_avoids")
	
    MakeObstaclePhysics(inst, 0.25)

    inst.MiniMapEntity:SetIcon("charlie_rose_2024.tex")

    inst.AnimState:SetBank("charlie_rose_2024")
    inst.AnimState:SetBuild("charlie_rose_2024")
    inst.AnimState:PlayAnimation("idle")
	inst.AnimState:SetLightOverride(0.5)	
	
    inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = TaskRoseGetInspectStatus
	
	inst:AddComponent("lootdropper")
	
    inst:AddComponent("pickable")
    inst.components.pickable:SetUp("rose_inv", TUNING.TOTAL_DAY_TIME*5)
    inst.components.pickable.picksound = "dontstarve/wilson/harvest_berries"
    inst.components.pickable.onpickedfn = taskroseonpickedfn
	inst.components.pickable.onregenfn = taskroseonregenfn	
	
    inst:AddComponent("interactivelevel")
   	inst.components.interactivelevel.OnActivate = TaskRoseOnActivate	
	inst.components.interactivelevel.IsTaskRose = true

	inst:AddComponent("trader")
	inst.components.trader.acceptnontradable = true
	inst.components.trader.onrefuse = TaskRoseOnRefuse
	inst.components.trader.onaccept = TaskRoseOnAccept
    inst.components.trader:SetAcceptTest(TaskRoseAcceptTest)
	
	inst:AddComponent("sanityaura") 
	inst.components.sanityaura.aurafn = RoseSanityAura
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

    MakeSmallBurnable(inst, 99)
    MakeSmallPropagator(inst)
	
	inst.components.burnable:SetOnIgniteFn(OnIgnite)	
	
	inst.SpawnHeartCore = SpawnHeartCore
	inst.SpawnRoseGroup = SpawnRoseGroup
	inst.SpawnWine = SpawnWine
	inst.OnSave = TaskRoseOnSave
	inst.OnLoad = TaskRoseOnLoad
	
	return inst
end


local function MakeCharlieValentineDayItem(name, custome_fn)
    local commonassets = 
    {
        Asset("ANIM", "anim/charlie_valentine_day_four_items.zip"),
    }

    local commonprefabs = 
    {

    }

    local function commonfn()	
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddAnimState()	
        inst.entity:AddSoundEmitter()
		
		MakeInventoryPhysics(inst)
		
		inst:AddTag("charlie_valentineday_task_item")
		inst:AddTag("shadow_destroy_immune")
		
        inst.AnimState:SetBank("charlie_valentine_day_four_items")
        inst.AnimState:SetBuild("charlie_valentine_day_four_items")		
        inst.AnimState:PlayAnimation(name)

		inst:AddComponent("inspectable")
		
		inst:AddComponent("inventoryitem")
		
        if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
            inst:AddComponent("waterproofer")
            inst.components.waterproofer:SetEffectiveness(0)
        end

        if inst.GetIsOnWater ~= nil then
	        inst:AddComponent("floatable")
		    inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		    inst.components.floatable:SetOnHitLandFn(OnHitLand)
		    inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		      
        end
		
		if custome_fn ~= nil then
		    custome_fn(inst)
		end		

		return inst
	end


    return Prefab("charlie_valentine_day_"..name, commonfn, commonassets, commonprefabs)
end

local function TaskCoffeeOnDrunk(inst, drinker)
    if drinker ~= nil then 
	    if inst.GetIsOnWater == nil and drinker.components.locomotor ~= nil then
		    --Note: need to handle via drinker prefab file in order to save the effect 
			if drinker.charliecoffeetask == nil then 
			    inst.OriginalWalkSpeed = inst.components.locomotor.walkspeed
		    	inst.OriginalRunSpeed = inst.components.locomotor.runspeed
		        drinker.components.locomotor.walkspeed = inst.components.locomotor.walkspeed + 6 
		    	drinker.components.locomotor.runspeed = inst.components.locomotor.runspeed + 6 
			    drinker.charliecoffeetask, drinker.charliecoffeeinfo = drinker:ResumeTask(TUNING.TOTAL_DAY_TIME*4, DrinkerCharlieCoffeeEffectDone)
			end 
		end  
		if not IsDLCEnabled(REIGN_OF_GIANTS) and not IsDLCEnabled(CAPY_DLC) and not IsDLCEnabled(PORKLAND_DLC) then
		    if drinker.components.temperature ~= nil then			
			    drinker.components.temperature:DeltaTemp(inst.components.edible.temperaturedelta + inst.components.edible.temperaturebump)
		    end
		end
		if not IsDLCEnabled(PORKLAND_DLC) then
		    if drinker.components.temperature ~= nil then			
			    drinker.components.temperature:DeltaTemp(inst.components.edible.temperaturebump)
		    end		
		end 
	end 
end

local function TaskCoffeeOnRemove(inst)
    if GetPlayer() and GetPlayer().prefab == "charlie" and GetPlayer().taskcoffeespawntask == nil then
	    GetPlayer().taskcoffeespawntask, GetPlayer().taskcoffeespawntaskinfo = GetPlayer():ResumeTask(TUNING.TOTAL_DAY_TIME*12, GetPlayer().SpawnTaskCoffee)
	end
end

local function TaskGiftDropLoot(inst, pos, doer, items)
    if type(items) == "table" then
        for k,v in pairs(items) do
	        local loot = SpawnPrefab(v)
            if loot ~= nil then
                loot.Transform:SetPosition(pos:Get())
				if loot.components.inventoryitem ~= nil then
                    loot.components.inventoryitem:OnDropped(true, nil, nil, 0.5)
				elseif loot.Physics ~= nil then
				    local vel = Vector3(0, 5, 0)
					local speed = 0.5 + math.random()
                    local angle = math.random()*2*math.pi
                    vel.x = speed*math.cos(angle)
			        vel.y = speed*3
                    vel.z = speed*math.sin(angle)					
				    loot.Physics:SetVel(vel.x, vel.y, vel.z)
				end
            end			
	    end
	end
end

local function TaskGiftOnUnwrapped(inst, pos, doer)
    local possible_loots = 
	{
        "charlie_dragon_feather",
		"rose_inv",
		"rose_inv",
		"rose_inv",
		"rose_inv",
		"rose_inv",
		"cutgrass",
		"twigs",
		"cutreeds",
		"honeycomb",
		"honey",
		"dug_berrybush",
		"dug_berrybush2",
		"dug_grass",
		"dug_sapling",
		"dug_marsh_bush",
		"pinecone",
		"dug_berrybush_juicy",
		"dug_bananabush",
		"dug_monkeytail",		
		"gears",
	    "shadow_rose_bouquet",
		"feather_crow",
		"feather_robin",
		"feather_robin_winter",
		"plantmeat",
		"cutgrass",
		"twigs",
		"lightbulb",
		"lureplantbulb",
	}
	if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    table.insert(possible_loots, "acorn")
		table.insert(possible_loots, "acorn_cooked")
	    table.insert(possible_loots, "coontail")
		table.insert(possible_loots, "goatmilk")
		table.insert(possible_loots, "goose_feather")
	end
	if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)) and GetWorld():HasTag("shipwrecked") then
	    table.insert(possible_loots, "snakeskin")
		table.insert(possible_loots, "dragoonheart")
		table.insert(possible_loots, "shark_fin")
		table.insert(possible_loots, "snakeoil")
		table.insert(possible_loots, "sand")
		table.insert(possible_loots, "dug_bambootree")
		table.insert(possible_loots, "dug_bush_vine")
	    table.insert(possible_loots, "seaweed_stalk")
		table.insert(possible_loots, "dug_coffeebush")
		table.insert(possible_loots, "dug_elephantcactus")
	end
	if IsDLCEnabled(PORKLAND_DLC) and GetWorld():HasTag("porkland") then
	    table.insert(possible_loots, "feather_thunder")
		table.insert(possible_loots, "peagawkfeather")
		table.insert(possible_loots, "vine")
		table.insert(possible_loots, "nectar_pod")
		table.insert(possible_loots, "venus_stalk")
		table.insert(possible_loots, "dug_nettle")
		table.insert(possible_loots, "burr")
		table.insert(possible_loots, "teatree_nut")
		table.insert(possible_loots, "bramble_bulb")
	end
	if math.random() < 0.05 then
	    table.insert(possible_loots, "krampus_sack")
	end
	local item1 = {}
	local item2 = {}
	local item3 = {}
	local item4 = {}
	for k, v in pairs(possible_loots) do
	    local item = GetRandomItem(possible_loots)
		if not table.contains(item2, item) and not table.contains(item3, item) and not table.contains(item4, item) and math.random() < 0.25 then
		    table.insert(item1, item)
		end
		if not table.contains(item1, item) and not table.contains(item3, item) and not table.contains(item4, item) and math.random() < 0.25 then
		    table.insert(item2, item)
		end
		if not table.contains(item1, item) and not table.contains(item2, item) and not table.contains(item4, item) and math.random() < 0.25 then
		    table.insert(item3, item)
		end
		if not table.contains(item1, item) and not table.contains(item2, item) and not table.contains(item3, item) and math.random() < 0.25 then
		    table.insert(item4, item)
		end
		if #item1 > 0 and #item2 > 0 and #item3 > 0 and #item4 > 0 then 
		    break
		end
	end
    if #item1 > 0 then
	    TaskGiftDropLoot(inst, pos, doer, item1)
	end
    if #item2 > 0 then
	    TaskGiftDropLoot(inst, pos, doer, item2)
	end
    if #item3 > 0 then
	    TaskGiftDropLoot(inst, pos, doer, item3)
	end
    if #item4 > 0 then
	    TaskGiftDropLoot(inst, pos, doer, item4)
	end
    if doer ~= nil and doer.SoundEmitter ~= nil then
        doer.SoundEmitter:PlaySound("dontstarve/common/craftable/bundles/packaged")
    end
	
	SpawnPrefab("small_puff").Transform:SetPosition(pos:Get())
	SpawnPrefab("charlie_valentine_day_fx").Transform:SetPosition(pos:Get())

    inst:Remove()
end

local function EnvelopeOnActivate(inst, doer)
    inst.SoundEmitter:PlaySound("dontstarve/common/destroy_stone")
	if doer ~= nil then
	    if doer.components.talker ~= nil then
		    doer.components.talker:Say(STRINGS.VALENTINE_DAY_ENVELOPE_UNKNOWN_WORDS or "")
		end
		if doer:HasTag("charlie") then
		    local x, y, z = inst.Transform:GetWorldPosition()
            local entities = TheSim:FindEntities(x, y, z, 20, nil, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "notarget", "charlie", "companion", "charlie_valentineday_task_item"})
            for k, v in pairs(entities) do
                if v ~= nil and v:IsValid() and v.entity:IsVisible() and v.prefab ~= nil and v.prefab ~= "" then
				    if v.components.workable ~= nil then
						SpawnPrefab("small_puff").Transform:SetPosition(v.Transform:GetWorldPosition())
						SpawnPrefab("charlie_valentine_day_fx").Transform:SetPosition(v.Transform:GetWorldPosition())
					    v.components.workable:Destroy(inst)		
					elseif v.components.health ~= nil then
						SpawnPrefab("small_puff").Transform:SetPosition(v.Transform:GetWorldPosition())
						SpawnPrefab("charlie_valentine_day_fx").Transform:SetPosition(v.Transform:GetWorldPosition())					
					    v.components.health:SetVal(0)
					end
				end
			end
        end
	end 
	inst:Remove()
end

local function BubbleCustomeFn(inst)
    inst:AddTag("task_bubble")
	
	inst:AddComponent("useableitem")
	inst.components.useableitem.caninteractfn = function(inst) return true end
	inst.components.useableitem.onusefn = function(inst) 
	    local TheSeasonManager = GetSeasonManager()
		if TheSeasonManager ~= nil then
		    if TheSeasonManager:IsRaining() then
				if TheSeasonManager.ForceStopPrecip ~= nil then
				    TheSeasonManager:ForceStopPrecip()
				else
				    TheSeasonManager.atmo_moisture = 0
				end
		        TheSeasonManager:StopPrecip()
		    else
			    TheSeasonManager:ForcePrecip()
				TheSeasonManager:StartPrecip()
		    end
		end
		if GetPlayer() then
		    if GetPlayer().components.moisture ~= nil then
			    GetPlayer().components.moisture:SetMoistureLevel(0)
			end
		    if GetPlayer().components.temperature ~= nil then
			    GetPlayer().components.temperature:SetTemperature(35)
			end
		end
		inst.SoundEmitter:PlaySound("dontstarve/common/craftable/rain_meter")
	    inst:Remove()
	end
end

local function TaskCoffeeCustomeFn(inst)
    inst:AddTag("task_coffee")
	inst:AddTag("preparedfood")

	inst:AddComponent("edible")
	inst.components.edible.foodtype = "VEGGIE"
	inst.components.edible.foodstate = "PREPARED"
	inst.components.edible.healthvalue = 25
	inst.components.edible.hungervalue = 15
	inst.components.edible.sanityvalue = 0
	inst.components.edible.temperaturedelta = 45
	inst.components.edible.temperatureduration = 30
	inst.components.edible.temperaturebump = 20
	inst.components.edible.caffeinedelta = 6
	inst.components.edible.caffeineduration = TUNING.TOTAL_DAY_TIME*4
	inst.components.edible.antihistamine = 375
    inst.components.edible.oneaten = TaskCoffeeOnDrunk
	
	inst.OnRemoveEntity = TaskCoffeeOnRemove
end

local function EnvelopeCustomeFn(inst)
    inst:AddTag("task_envelope")
	
    inst:AddComponent("interactivelevel")
  	inst.components.interactivelevel.OnActivate = EnvelopeOnActivate	
end

local function GiftCustomeFn(inst)
    inst:AddTag("task_gift")
	
    inst:AddComponent("unwrappable")
    inst.components.unwrappable:SetOnUnwrappedFn(TaskGiftOnUnwrapped)	
end

local function WineOnDrunk(inst, drinker)
    if drinker ~= nil then 
	    if inst.GetIsOnWater == nil and drinker.components.locomotor ~= nil then
		    --Note: need to handle via drinker prefab file in order to save the effect 
			if drinker.charliecoffeetask == nil then 
			    inst.OriginalWalkSpeed = inst.components.locomotor.walkspeed
		    	inst.OriginalRunSpeed = inst.components.locomotor.runspeed
		        drinker.components.locomotor.walkspeed = inst.components.locomotor.walkspeed + 6 
		    	drinker.components.locomotor.runspeed = inst.components.locomotor.runspeed + 6 
			    drinker.charliecoffeetask, drinker.charliecoffeeinfo = drinker:ResumeTask(TUNING.TOTAL_DAY_TIME*5, DrinkerCharlieCoffeeEffectDone)
			end 
		end  
		if not IsDLCEnabled(REIGN_OF_GIANTS) and not IsDLCEnabled(CAPY_DLC) and not IsDLCEnabled(PORKLAND_DLC) then
		    if drinker.components.temperature ~= nil then			
			    drinker.components.temperature:DeltaTemp(inst.components.edible.temperaturedelta + inst.components.edible.temperaturebump)
		    end
		end
		if not IsDLCEnabled(PORKLAND_DLC) then
		    if drinker.components.temperature ~= nil then			
			    drinker.components.temperature:DeltaTemp(inst.components.edible.temperaturebump)
		    end		
		end 
	end 
end

local function winefn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
    inst:AddTag("charlie_wine")
	inst:AddTag("valid_for_full_task_rose")
	inst:AddTag("preparedfood")
	inst:AddTag("shadow_destroy_immune")	

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("charlie_wine")
    inst.AnimState:SetBuild("charlie_wine")
    inst.AnimState:PlayAnimation("idle")	
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end
	
    if inst.GetIsOnWater ~= nil then
	    MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		      

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 4
    end
	
    inst:AddComponent("bait")	
	
	inst:AddComponent("tradable")
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = 40
	
	inst:AddComponent("edible")
	inst.components.edible.foodtype = "VEGGIE"
	inst.components.edible.foodstate = "PREPARED"
	inst.components.edible.healthvalue = 50
	inst.components.edible.hungervalue = 20
	inst.components.edible.sanityvalue = 0
	inst.components.edible.temperaturedelta = 45
	inst.components.edible.temperatureduration = 30
	inst.components.edible.temperaturebump = 20
	inst.components.edible.caffeinedelta = 6
	inst.components.edible.caffeineduration = TUNING.TOTAL_DAY_TIME*5
	inst.components.edible.antihistamine = 375
    inst.components.edible.oneaten = WineOnDrunk
	
	return inst
end

local function rosegrouponpickedfn(inst, picker)
	if inst.components.burnable ~= nil then
	    inst.components.burnable:Extinguish()
	end
    inst.persists = false
    inst.AnimState:PlayAnimation("pick")

	if picker ~= nil then
	    if picker:HasTag("plantkin") then
            if picker.killedplantfn ~= nil then
                picker:killedplantfn()
			elseif picker.components.sanity ~= nil then
			    picker.components.sanity:DoDelta(-TUNING.SANITY_TINY)
            end		
		elseif picker.components.sanity ~= nil then
		    if picker:HasTag("flower_lover") then
			    picker.components.sanity:DoDelta(TUNING.SANITY_TINY*3)
			else
			    picker.components.sanity:DoDelta(TUNING.SANITY_TINY)
			end
		end
		
        if picker.components.combat ~= nil 
	        and not picker:HasTag("flower_lover") 
	        and not picker:HasTag("bramble_resistant") 
	        --[[and not picker:HasTag("armorcactus")]]
    	then
            picker.components.combat:GetAttacked(inst, TUNING.ROSE_DAMAGE*10)
            picker:PushEvent("thorns")
        end		
	end
	
    inst:ListenForEvent("animover", inst.Remove)
end

local function rosegroupfn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddSoundEmitter()
	
    inst:AddTag("charlie_rose_group")
    inst:AddTag("shadow_destroy_immune")
	inst:AddTag("pugalisk_avoids")
	
    MakeObstaclePhysics(inst, 0.25)

    inst.MiniMapEntity:SetIcon("charlie_rose_group.tex")

    inst.AnimState:SetBank("charlie_rose_group")
    inst.AnimState:SetBuild("charlie_rose_group")
    inst.AnimState:PlayAnimation("idle")
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("lootdropper")

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end
	
    inst:AddComponent("pickable")
    inst.components.pickable:SetUp("rose_inv", TUNING.TOTAL_DAY_TIME*5, 10)
    inst.components.pickable.picksound = "dontstarve/wilson/harvest_berries"
    inst.components.pickable.onpickedfn = rosegrouponpickedfn	
	
	inst:AddComponent("sanityaura") 
	inst.components.sanityaura.aurafn = RoseSanityAura

    MakeSmallBurnable(inst, 99)
    MakeSmallPropagator(inst)
	
	inst.components.burnable:SetOnIgniteFn(OnIgnite)	
	inst.components.burnable:SetOnExtinguishFn(function() end)	
	
	return inst
end

local function fishingrodonequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_valentine_day_fishing_rod", "swap_valentine_day_fishing_rod")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function fishingrodonunequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal")
end

local function IsImpassablePoint(x, y, z)  
    return GetGroundTypeAtPosition(Vector3(x, y, z)) == GROUND.IMPASSABLE
end

local function IsWaterPoint(x, y, z)  
    return GetWorld().Map:IsWater(GetWorld().Map:GetTileAtPoint(x, y, z))
end

local function IsPointNearWater(x, y, z, radius, excludeboat) 
    local radius = radius or 3

    if excludeboat then
	    if GetPlayer().components.driver ~= nil then
		    if GetPlayer().components.driver:GetIsDriving() then
			    return false
			end
		end
	end

    for i = -radius, radius, 1 do
        if IsImpassablePoint(x - radius, y, z + i) or IsImpassablePoint(x + radius, y, z + i) then
            return true
        end	
		if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)	then
            if IsWaterPoint(x - radius, y, z + i) or IsWaterPoint(x + radius, y, z + i) then
                return true
            end
        end			
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if IsImpassablePoint(x + i, y, z - radius) or IsImpassablePoint(x + i, y, z + radius) then
            return true
        end
		if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)	then
		    if IsWaterPoint(x + i, y, z - radius) or IsWaterPoint(x + i, y, z + radius) then
                return true
            end
		end
    end
	
    return false
end

local function FindLandBetweenPoints(p0x, p0y, p1x, p1y)
	local map = GetWorld().Map

	local dx = math.abs(p1x - p0x)
	local dy = math.abs(p1y - p0y)

    local ix = p0x < p1x and TILE_SCALE or -TILE_SCALE
    local iy = p0y < p1y and TILE_SCALE or -TILE_SCALE

    local e = 0;
    for i = 0, dx+dy - 1 do
        local tile_at_point = map:GetTileAtPoint(p0x, 0, p0y)
        if not IsPointNearWater(p0x, 0, p0y) then
			return map:GetTileCenterPoint(p0x, 0, p0y)
		end

        local e1 = e + dy
        local e2 = e - dx
        if math.abs(e1) < math.abs(e2) then
            p0x = p0x + ix
            e = e1
		else
            p0y = p0y + iy
            e = e2
        end
	end

	return nil
end

local function VecUtil_LengthSq(p1_x, p1_z)
    return p1_x * p1_x + p1_z * p1_z
end

local function FindRandomPointOnShoreFromOcean(x, y, z)
	local nodes = {}

    for i, node in ipairs(GetWorld().topology.nodes) do
		if node.type ~= NODE_TYPE.Blank and node.type ~= NODE_TYPE.Blocker then
			table.insert(nodes, {n = node, distsq = VecUtil_LengthSq(x - node.x, z - node.y)})
		end
	end
	table.sort(nodes, function(a, b) return a.distsq < b.distsq end)

	local num_rooms_to_pick = 4

	local closest = {}
	for i = 1, num_rooms_to_pick do
		table.insert(closest, nodes[i])
	end
	shuffleArray(closest)

	local dest_x, dest_y, dest_z
	for _, c in ipairs(closest) do
		dest_x, dest_y, dest_z = FindLandBetweenPoints(x, z, c.n.x, c.n.y)
		if dest_x ~= nil and TheSim:WorldPointInPoly(dest_x, dest_z, c.n.poly) then
			return dest_x, dest_y, dest_z
		end
	end

	for i = num_rooms_to_pick + 1, #nodes do
		local c = nodes[i]
		if c ~= nil then
			dest_x, dest_y, dest_z = FindLandBetweenPoints(x, z, c.n.x, c.n.y)
			if dest_x ~= nil and TheSim:WorldPointInPoly(dest_x, dest_z, c.n.poly) then
				return dest_x, dest_y, dest_z
			end
		end
	end

	return nil
end

local function FixFishingRodPosition(inst)
    local ix, iy, iz = inst.Transform:GetWorldPosition()
    if not inst:IsOnValidGround() then
	    local sx, sy, sz = FindRandomPointOnShoreFromOcean(ix, iy, iz)
		if sx ~= nil and sy ~= nil and sz ~= nil then
            inst.Transform:SetPosition(sx, sy, sz)
		end		
	elseif inst.GetIsOnWater ~= nil and inst:GetIsOnWater() and not inst.SkipWaterCheck then
	    local sx, sy, sz = FindRandomPointOnShoreFromOcean(ix, iy, iz)
		if sx ~= nil and sy ~= nil and sz ~= nil then
            inst.Transform:SetPosition(sx, sy, sz)
		end		
	    inst.SkipWaterCheck = true
	end
end

local function FishingRodOnLoad(inst, data)
    inst.SkipWaterCheck = true
end

local function fishingrodfn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    inst:AddTag("valentine_day_fishing_rod")
    inst:AddTag("shadow_destroy_immune")
	inst:AddTag("allow_action_on_impassable")
    inst:AddTag("irreplaceable")

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("valentine_day_fishing_rod")
    inst.AnimState:SetBuild("valentine_day_fishing_rod")
    inst.AnimState:PlayAnimation("idle")
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end	

    if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		      
    end

    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(fishingrodonequip)
    inst.components.equippable:SetOnUnequip(fishingrodonunequip)
	
	inst:AddComponent("valentinedayfishingrod")
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(TUNING.FISHINGROD_DAMAGE)	

    MakeSmallBurnable(inst, 99)
    MakeSmallPropagator(inst)
	
	inst.components.burnable:SetOnIgniteFn(OnIgnite)	
	
	inst.OnLoad = FishingRodOnLoad
	
	inst:DoTaskInTime(0, FixFishingRodPosition)
	
	return inst
end

local function fxfn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
    inst:AddTag("FX")
	inst:AddTag("NOCLICK")
    inst:AddTag("NOBLOCK")

    inst.AnimState:SetBank("charlie_valentine_day_fx")
    inst.AnimState:SetBuild("charlie_valentine_day_fx")
    inst.AnimState:PlayAnimation("idle")
	
	inst.Transform:SetScale(3, 3, 3)
	
	inst:ListenForEvent("animover", inst.Remove)
	
	inst.persists = false
	
	return inst
end

return Prefab("charlie_heart", plantheartfn, heartassets, prefabs),
    Prefab("charlie_heart_inv", inventoryheartfn, invheartassets, prefabs),
	Prefab("charlie_heart_seed", seedheartfn, seedheartassets, prefabs),
	Prefab("charlie_candy", candyfn, candyassets, prefabs),
	Prefab("charlie_coffee", coffeefn, coffeeassets, prefabs),
	Prefab("charlie_heart_core", heartcorefn, heartcoreassets, prefabs),
	Prefab("charlie_rose_2024", taskrosefn, taskroseassets, prefabs),
	MakeCharlieValentineDayItem("bubble", BubbleCustomeFn),
	MakeCharlieValentineDayItem("coffee", TaskCoffeeCustomeFn),
	MakeCharlieValentineDayItem("envelope", EnvelopeCustomeFn),
	MakeCharlieValentineDayItem("gift", GiftCustomeFn),
	Prefab("charlie_wine", winefn, wineassets, prefabs),
	Prefab("charlie_rose_group", rosegroupfn, rosegroupassets, prefabs),
	Prefab("valentine_day_fishing_rod", fishingrodfn, fishingrodassets, prefabs),
	Prefab("charlie_valentine_day_fx", fxfn, fxassets, prefabs),
    MakePlacer("charlie_heart_placer", "charlie_heart", "charlie_heart", "idle")