require("prefabutil")

local dragonassets = 
{
    Asset("ANIM", "anim/charlie_critter_dragon.zip"),
}

local minidragonassets = 
{
    Asset("ANIM", "anim/charlie_critter_dragon_mini.zip"),
}

local dragonfeatherassets = 
{
    Asset("ANIM", "anim/charlie_dragon_feather.zip"),
}

local dragonswordassets = 
{
    Asset("ANIM", "anim/charlie_dragon_sword.zip"),
    Asset("ANIM", "anim/swap_charlie_dragon_sword.zip"),
	Asset("ANIM", "anim/ui_antlionhat_1x1.zip"),
}

local dragonhouseassets = 
{
    Asset("ANIM", "anim/charlie_dragon_house.zip"),
}

local dragonfireassets = 
{
    Asset("ANIM", "anim/charlie_dragon_fire.zip"),
}

local prefabs = 
{

}

local function ShouldAcceptItem(inst, item, giver)
    if giver ~= nil and giver.components.leader ~= nil and giver.components.leader:IsBeingFollowedBy("charlie_critter_dragon") then
	    return false
	end
    return giver ~= nil and item ~= nil and giver:HasTag("supercritterexpert") and item:HasTag("valid_for_dragon_weapon")
end

local function OnGetItem(inst, giver, item)
    if item ~= nil then
	    item:Remove()
	end
    if inst.components.combat.target and inst.components.combat.target == giver then
        inst.components.combat:SetTarget(nil)
	end
    if giver ~= nil and giver.components.leader and giver:HasTag("supercritterexpert") then
		inst.SoundEmitter:PlaySound("dontstarve/common/makeFriend")
		giver.components.leader:AddFollower(inst)
    end
end

local function OnRefuseItem(inst, giver, item)
    if giver ~= nil and giver.components.talker ~= nil then
	    if TUNING.LANGUAGE == true then
		    giver.components.talker:Say("Wrong item, I have already hired an identical one, or I am not proficient enough.")
        elseif TUNING.LANGUAGE == false then
		    giver.components.talker:Say("错误物品，已经雇佣一只相同的了，或者我不够熟练。")
	    end	
	end
end

local function RetargetFn(inst)
    return FindEntity(inst, 15, function(guy) 
        return guy.components.health ~= nil 
                and not guy.components.health:IsDead()
                and guy.components.combat ~= nil
		        and inst.components.combat:CanTarget(guy)
		        and inst.components.follower.leader ~= nil 
		        and ((inst.components.follower.leader.components.combat ~= nil 
		                and inst.components.follower.leader.components.combat.target ~= nil
				        and inst.components.follower.leader.components.combat.target == guy)
			        or (guy.components.combat.target ~= nil and guy.components.combat.target == inst.components.follower.leader))
	end, nil, {"FX", "NOCLICK", "INLIMBO", "companion", "player", "charliecritter"})
end

local function KeepTargetFn(inst, target)
	return inst.components.combat:CanTarget(target)
end

local function on_remove(inst, data)
    if inst.components.inventory and inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
	    inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS):Remove()
	end
end

local function equip_ranged_weapon(inst)
    if inst.components.inventory and not inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
        local ranged_weapon = CreateEntity()
        ranged_weapon.persists = false

        ranged_weapon.entity:AddTransform()

        ranged_weapon:AddComponent("weapon")
        ranged_weapon.components.weapon:SetDamage(inst.components.combat.defaultdamage)
        ranged_weapon.components.weapon:SetRange(inst.components.combat.attackrange, inst.components.combat.attackrange + 4)
        ranged_weapon.components.weapon:SetProjectile("charlie_dragon_fire")

        ranged_weapon:AddComponent("inventoryitem")
        ranged_weapon.components.inventoryitem:SetOnDroppedFn(ranged_weapon.Remove)

        ranged_weapon:AddComponent("equippable")
        ranged_weapon:AddTag("nosteal")
		ranged_weapon:AddTag("cantdrop")
		ranged_weapon:AddTag("projectile")
        inst.components.inventory:Equip(ranged_weapon)
    end
end

local function DragonCanBeAttackedFn(inst, attacker)
    return attacker ~= nil and not attacker:HasTag("supercritterexpert")
end

local function dragonfn()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
    inst.entity:AddMiniMapEntity()
	inst.entity:AddDynamicShadow()
	
	inst:AddTag("charliecritter")
	inst:AddTag("charliecritterdragon")
    inst:AddTag("companion")
    inst:AddTag("notraptrigger")
    inst:AddTag("flying")
	inst:AddTag("amphibious")
	inst:AddTag("largecreature")
	inst:AddTag("groundpoundimmune")
	inst:AddTag("NOBLOCK")
	
	if not IsDLCEnabled(CAPY_DLC) and not IsDLCEnabled(PORKLAND_DLC) then
	    MakeCharacterPhysics(inst, 500, 0.5)
	else
	    MakeAmphibiousGhostPhysics(inst, 500, 0.5)
	end
	
	inst.DynamicShadow:SetSize(5, 2.5)
	
	inst.Transform:SetFourFaced()
	
    inst.AnimState:SetBank("charlie_critter_dragon")
    inst.AnimState:SetBuild("charlie_critter_dragon")
    inst.AnimState:PlayAnimation("idle")
	inst.AnimState:SetLightOverride(1)
	
	inst.Transform:SetScale(3, 3, 3)
	
	inst.MiniMapEntity:SetIcon("charlie_critter_dragon.tex")
	
	inst:AddComponent("inspectable")
	
    inst:AddComponent("talker")
    inst.components.talker.fontsize = 45
    inst.components.talker.font = TALKINGFONT
    inst.components.talker.colour = Vector3(0.9, 0.29, 0.29)
    inst.components.talker.offset = Vector3(0, -300, 0)	

    inst:AddComponent("locomotor") 
    inst.components.locomotor.runspeed = 7
    inst.components.locomotor.walkspeed = 5

	inst:AddComponent("health")
	inst.components.health:SetMaxHealth(1000)
	inst.components.health:StartRegen(10, 10)
	inst.components.health.fire_damage_scale = 0
	
    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(100)
    inst.components.combat:SetRange(100)
	inst.components.combat:SetAttackPeriod(5)
	inst.components.combat:SetKeepTargetFunction(KeepTargetFn)
	inst.components.combat:SetRetargetFunction(3, RetargetFn)
	inst.components.combat.canbeattackedfn = DragonCanBeAttackedFn
	
    inst:AddComponent("follower")
	
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    inst.components.follower:SetFollowExitDestinations({EXIT_DESTINATION.LAND, EXIT_DESTINATION.WATER})
	end
	
	inst:AddComponent("knownlocations")
	
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"dragonfruit"})
	inst.components.lootdropper:AddChanceLoot("charlie_dragon_feather", 0.25)
	inst.components.lootdropper:AddIfNotChanceLoot("dragonfruit_seeds")
	
	inst:AddComponent("inventory")

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

    inst:AddComponent("trader")
    inst.components.trader:SetAcceptTest(ShouldAcceptItem)
    inst.components.trader.onaccept = OnGetItem
    inst.components.trader.onrefuse = OnRefuseItem
	inst.components.trader.acceptnontradable = true
	
    local _attacked_fn = inst.event_listeners and inst.event_listeners["attacked"] and inst.event_listeners["attacked"][inst][1]
    if _attacked_fn then
        inst.event_listeners["attacked"][inst][1] = function(inst, data)
            if data ~= nil 
			    and data.attacker ~= nil 
				and not data.attacker:HasTag("supercritterexpert") 
				and inst.components.combat ~= nil
			then 
			    inst.components.combat:SetTarget(data.attacker)
			end 
        end
    end	
	
	inst:ListenForEvent("onremove", on_remove)
	
	inst:DoTaskInTime(0, equip_ranged_weapon)
	
	local dragonbrain = require("brains/charliedragonbrain")
	
	inst:SetBrain(dragonbrain)
	inst:SetStateGraph("SGcharliedragon")
	
	return inst
end

local function minidragonfn()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
	inst.entity:AddDynamicShadow()
	
	MakeObstaclePhysics(inst, 0.25)

    inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
    inst:AddTag("NOTARGET")
	inst:AddTag("ABSORB_IMMUNE")
	
	inst.DynamicShadow:SetSize(3, 1.5)
	
    inst.AnimState:SetBank("charlie_critter_dragon_mini")
    inst.AnimState:SetBuild("charlie_critter_dragon_mini")
    inst.AnimState:PlayAnimation("idle")

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

	inst:AddComponent("charlieareafiredamage")
	
	return inst
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity) 
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity) 
	end	
end

local function OnHitWaterSword(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)
		inst.float_fx.entity:SetParent(inst.entity) 
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.entity:SetParent(inst.entity) 
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function dragonfeatherfn()
    local inst = CreateEntity()
	
	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
	
	MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("charlie_dragon_feather")
    inst.AnimState:SetBuild("charlie_dragon_feather")
    inst.AnimState:PlayAnimation("idle")
	inst.AnimState:SetLightOverride(0.5)

    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

    if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		      

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 8
    end

	inst:AddComponent("tradable")
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = 40

	inst:AddComponent("repairer")
	inst.components.repairer.repairmaterial = "charliedragonfeather"
	inst.components.repairer.finiteusesrepairvalue = 100 

	return inst
end

local function TestItemDragonWeapon(inst, item, slot)
	return item:HasTag("valid_for_dragon_weapon")
end

local function DragonWeaponItemGet(inst, data)
    if data ~= nil and data.item ~= nil and data.item:HasTag("valid_for_dragon_weapon") then 
	    inst.charged = true 
	    inst.Light:Enable(true)
	end 
end

local function DragonWeaponItemLose(inst, data)
    inst.charged = nil 
    inst.Light:Enable(false)
end

local slotpos = {}

for y = 0, 0 do
	table.insert(slotpos, Vector3(0, -y*75 -10 ,0))
end

local function DragonWeaponGetHeat(inst, observer)
    if observer ~= nil and observer.components.inventory ~= nil then
	    if observer.components.inventory:EquipHasTag("possess_with_dragon") 
		    or observer.components.inventory:FindItem(function(item) return item.prefab == "charlie_dragon_sword" end) 
		then
		    return 0
		end
	end

    local season_manager = GetSeasonManager()
    if season_manager ~= nil 
	    and season_manager:GetCurrentTemperature() < 0 
		and inst.charged 
    then
	    return 50
	end

    return 0
end

local function DragonWeaponGetEquippedHeat(inst, observer)
    if observer ~= nil and observer.components.inventory ~= nil then
	    if observer.components.inventory:FindItem(function(item) return item.prefab == "charlie_dragon_sword" end) then
		    return 0
		end
	end

    local season_manager = GetSeasonManager()
    if season_manager ~= nil 
	    and season_manager:GetCurrentTemperature() < 0 
		and inst.charged 
    then
	    return 50
	end

    return 0
end

local function DragonWeaponGetCarriedHeat(inst, observer)
    if observer ~= nil and observer.components.inventory ~= nil then
	    if observer.components.inventory:EquipHasTag("possess_with_dragon") then
		    return 0
		end
	end
	
    local season_manager = GetSeasonManager()
    if season_manager ~= nil 
	    and season_manager:GetCurrentTemperature() < 0 
		and inst.charged 
    then
	    return 50
	end	
	
    return 0
end

local function onequipsword(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_charlie_dragon_sword", "swap_charlie_dragon_sword") 
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function onunequipsword(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

local function IsEntValidToIncreaseFirePoint(inst)
    return not (inst:HasTag("veggie") or 
                inst:HasTag("structure") or 
                inst:HasTag("wall") or 
                inst:HasTag("balloon") or 
                inst:HasTag("chess") or 
				inst:HasTag("player") or 
				inst:HasTag("INLIMBO") or
				inst:HasTag("INTERIOR_LIMBO") or
				inst:HasTag("no_durability_loss_on_hit") or
                inst:HasTag("smashable"))
		and ( inst.prefab ~= "pugalisk_body" and inst.prefab ~= "pugalisk_tail" and inst.prefab ~= "pugalisk_segment" )
end

local function SwordOnAttack(inst, attacker, target)
	if target ~= nil then
	    local targetx, targety, targetz = target.Transform:GetWorldPosition()
        local impactfx = SpawnPrefab("explode_small")
		if target:HasTag("flying") and targety == 0 then
		    targety = targety + 0.25
		end 
        if impactfx then
            impactfx.Transform:SetPosition(targetx, targety, targetz)
			if target:HasTag("smallcreature") then
			    impactfx.Transform:SetScale(0.5, 0.5, 0.5)
			elseif target:HasTag("largecreature") then
			    impactfx.Transform:SetScale(2, 2, 2)
			end 
        end		
	    if target.components.health ~= nil then 
		    if target:HasTag("charliecritterdragon") then 
			    target.components.health:DoDelta(inst.charged and 74 or 44)
			else
		        target.components.health:DoFireDamage(inst.charged and 34 or 5, inst, true)
			end
		end
	end		
	inst.SoundEmitter:PlaySound("dontstarve/wilson/fireball_explo")
	if inst.charged and target ~= nil and IsEntValidToIncreaseFirePoint(target) then
	    inst.fire_point = math.min(inst.fire_point + 1, inst.max_fire_point)
	end 
end

local function CreateDragonFire(inst, target, pos)
    if inst.charged and inst.fire_point and inst.fire_point > 24 then
	    local dragonfire1 = SpawnPrefab("charlie_dragon_fire")
		local x, y, z = inst.Transform:GetWorldPosition()
		local LaunchPos = nil
		
		if target ~= nil then
		    x, y, z = target.Transform:GetWorldPosition()
			LaunchPos = target:GetPosition()
		elseif pos ~= nil then
		    x, y, z = pos.x, pos.y, pos.z
			LaunchPos = pos
		end 
	
        local rot = 45 * math.pi / 180
        local sin_rot = math.sin(rot)
        local cos_rot = math.cos(rot)
	    local light_x1 = -1.6
    	local light_z1 = -1.6
	
	    local light_x2 = 2.7
	    local light_z2 = -0.8
	
	    local light_x3 = -0.8
	    local light_z3 = 2.7
		
		local x2 = x + light_x1 * cos_rot - light_z1 * sin_rot
		local z2 = z + light_z1 * cos_rot + light_x1 * sin_rot
		local x3 = x + light_x2 * cos_rot - light_z2 * sin_rot
		local z3 = z + light_z2 * cos_rot + light_x2 * sin_rot
		local x4 = x + light_x3 * cos_rot - light_z3 * sin_rot
		local z4 = z + light_z3 * cos_rot + light_x3 * sin_rot		

		local LaunchPos2 = Point(x2, y, z2)
		local LaunchPos3 = Point(x3, y, z3)
		local LaunchPos4 = Point(x4, y, z4)
	
		dragonfire1.Transform:SetPosition(x, y, z)
		dragonfire1.components.complexprojectile:Launch(LaunchPos, GetPlayer())
		dragonfire1.FromStaff = true
		
		if inst.fire_point > 49 then 
		    local dragonfire2 = SpawnPrefab("charlie_dragon_fire")
		    dragonfire2.Transform:SetPosition(x2, y, z2)	
			dragonfire2.components.complexprojectile:Launch(LaunchPos2, GetPlayer())
			dragonfire2.FromStaff = true
		end
		if inst.fire_point > 74 then 
		    local dragonfire3 = SpawnPrefab("charlie_dragon_fire")
			dragonfire3.Transform:SetPosition(x3, y, z3)
			dragonfire3.components.complexprojectile:Launch(LaunchPos3, GetPlayer())
			dragonfire3.FromStaff = true
		end		

        if inst.fire_point > 99 then 
		    local dragonfire4 = SpawnPrefab("charlie_dragon_fire")
			dragonfire4.Transform:SetPosition(x4, y, z4)
			dragonfire4.components.complexprojectile:Launch(LaunchPos4, GetPlayer())
			dragonfire4.FromStaff = true
		end
		
		inst.components.container:RemoveItemBySlot(1):Remove()
		inst.fire_point = 0
        inst.components.finiteuses:Use(10)
    else
	    if GetPlayer() and GetPlayer().components.talker and GetPlayer().prefab ~= "wes" and GetPlayer().prefab ~= "wilbur" then
            if TUNING.LANGUAGE == true then
                GetPlayer().components.talker:Say("It doesn't have enough fire points to cast special effects.")
	        elseif TUNING.LANGUAGE == false then
	            GetPlayer().components.talker:Say("它没有足够的火焰点数来施放特殊效果。")
	        end
	    end
	end 
end

local function CanCreateDeagonFire(inst, caster, target, pos)
    return inst.charged and inst.fire_point and inst.fire_point > 24
end

local function onfinished(inst)
    inst.components.container:DropEverything()
    inst:Remove()
end

local function OnSaveSword(inst, data)
    data.fire_point = inst.fire_point
end

local function OnLoadSword(inst, data)
    if data ~= nil then
	    inst.fire_point = data.fire_point or 0
	end
end

local function dragonswordfn()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddLight()	
	
	MakeInventoryPhysics(inst)
	
	inst:AddTag("shadow_destroy_immune")	
	inst:AddTag("possess_with_dragon")

    inst.AnimState:SetBank("charlie_dragon_sword")
    inst.AnimState:SetBuild("charlie_dragon_sword")
    inst.AnimState:PlayAnimation("idle")
	inst.AnimState:SetLightOverride(0.25)
	
	inst.Transform:SetScale(0.5, 0.5, 0.5)

    inst.Light:SetRadius(1)
    inst.Light:SetIntensity(0.75)
    inst.Light:SetColour(175/255, 125/255, 175/255)
    inst.Light:SetFalloff(0.25)
    inst.Light:Enable(false)
	
	inst.fire_point = 0
	inst.max_fire_point = 100
    inst.fxcolour = {225/255, 175/255, 50/255}
    inst.castsound = "dontstarve/common/fireOut"
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

    if inst.GetIsOnWater ~= nil then
	    MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWaterSword)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		      

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 8
    end
	
    inst:AddComponent("repairable")
    inst.components.repairable.repairmaterial = "charliedragonfeather"
	inst.components.repairable.announcecanfix = false

    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequipsword)
    inst.components.equippable:SetOnUnequip(onunequipsword)

    inst:AddComponent("finiteuses")	
    inst.components.finiteuses:SetMaxUses(1000)
    inst.components.finiteuses:SetUses(1000)
	inst.components.finiteuses:SetOnFinished(onfinished) 

    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(34)
	inst.components.weapon:SetOnAttack(SwordOnAttack)
	
    inst:AddComponent("container")
    inst.components.container:SetNumSlots(#slotpos)
    inst.components.container.widgetslotpos = slotpos
	inst.components.container.itemtestfn = TestItemDragonWeapon
    inst.components.container.widgetpos = Vector3(-5,-200,0)
	inst.components.container.side_align_tip = 0
	inst.components.container.widgetanimbank = "ui_antlionhat_1x1"
	inst.components.container.widgetanimbuild = "ui_antlionhat_1x1"
    inst.components.container.excludefromcrafting =	true
	inst.components.container.canbeopened = true
	inst.components.container.acceptsstacks = false
	inst.components.container.type = "hand_inv"
	
	inst:AddComponent("heater")
	inst.components.heater.heatfn = DragonWeaponGetHeat
	inst.components.heater.equippedheatfn = DragonWeaponGetEquippedHeat
	inst.components.heater.carriedheatfn = DragonWeaponGetCarriedHeat

    inst:AddComponent("spellcaster")
    inst.components.spellcaster:SetSpellFn(CreateDragonFire)
    inst.components.spellcaster:SetSpellTestFn(CanCreateDeagonFire)
    inst.components.spellcaster.canuseonpoint = true
	inst.components.spellcaster.canuseontargets = true
    inst.components.spellcaster.canusefrominventory = false
	
	inst:ListenForEvent("itemget", DragonWeaponItemGet)
	inst:ListenForEvent("itemlose", DragonWeaponItemLose)
	inst:ListenForEvent("ondropped", function(inst)
	    if not inst.charged then
	        inst.Light:Enable(false)
		end
	end)
	inst:ListenForEvent("onputininventory", function(inst)
	    if inst.charged then
	        inst.Light:Enable(true)
		end
	end)
	
	inst.OnSave = OnSaveSword
	inst.OnLoad = OnLoadSword
	
	return inst
end

local function onhammered(inst, worker)
    if inst.components.burnable ~= nil and inst.components.burnable:IsBurning() then
        inst.components.burnable:Extinguish()
    end
	if inst.components.spawner ~= nil and inst.components.spawner:IsOccupied() then 
	    inst.components.spawner:ReleaseChild() 
	end
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	inst:Remove()
end

local function dragonhousefn()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
    inst.entity:AddMiniMapEntity()
	
	MakeObstaclePhysics(inst, 2)
	
	inst:AddTag("structure")

    inst.AnimState:SetBank("charlie_dragon_house")
    inst.AnimState:SetBuild("charlie_dragon_house")
    inst.AnimState:PlayAnimation("idle")
	inst.AnimState:SetLightOverride(0.25)
	
	inst.Transform:SetScale(3, 3, 3)
	
	inst.MiniMapEntity:SetIcon("charlie_dragon_house.tex")
	
	inst:AddComponent("inspectable")
	
    inst:AddComponent("lootdropper")
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end	
	
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	
	inst:AddComponent("spawner")
    inst.components.spawner:Configure("charlie_critter_dragon", TUNING.TOTAL_DAY_TIME*5)	

	return inst
end

local function onhit(inst, attacker, target)
    local x, y, z = inst.Transform:GetWorldPosition()
	
    local impactfx = SpawnPrefab("explode_small")
    if impactfx then
        impactfx.Transform:SetPosition(x, y, z)
		impactfx.Transform:SetScale(1.5, 1.5, 1.5)
    end

    local entities = TheSim:FindEntities(x, y, z, 5, {"hascombatcomponent"}, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "charliecritter", "companion", "wall", "structure", "notarget"})
    for i, v in pairs(entities) do
        if v ~= nil and v:IsValid() and v.entity:IsVisible() and inst.components.combat:CanTarget(v) then

            if v:HasTag("player") and inst.FromStaff then
                --Do nothing
			elseif attacker ~= nil 
			    and attacker.components.follower ~= nil
				and attacker.components.follower.leader ~= nil 
				and attacker.components.follower.leader == v then
				--Do nothing
			else		
                if attacker ~= nil and v.components.combat ~= nil then
			        v.components.combat:GetAttacked(attacker, 100, inst)
                else
                    inst.components.combat:DoAttack(v)
                end
		        if v.components.health ~= nil then
			        v.components.health:DoFireDamage(25, inst, true)
		     	end					
			end 
		
        end
    end	
	
	inst.SoundEmitter:PlaySound("dontstarve/common/fireOut")	

    inst:Remove()
end

local function onthrown(inst)
    inst.is_throwing = true 
	inst.Light:Enable(true)
end

local function GetHeatFn(inst)
	return inst.is_throwing and 200 or 0
end

local function dragonfirefn()
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	inst.entity:AddLight()
    inst.entity:AddPhysics()	
	
	inst.Light:SetRadius(7.5)
    inst.Light:SetFalloff(0.2)   
    inst.Light:SetIntensity(0.9)   
    inst.Light:SetColour(240/255, 200/255, 125/255)       
    inst.Light:Enable(false)
	
    inst.Physics:SetMass(1)
    inst.Physics:SetFriction(0)
    inst.Physics:SetDamping(0)
    inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
    inst.Physics:ClearCollisionMask()
    inst.Physics:CollidesWith(COLLISION.GROUND)
    inst.Physics:SetCapsule(0.2, 0.2)
	inst.Physics:SetDontRemoveOnSleep(true)
	
    inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
    inst:AddTag("NOTARGET")
	inst:AddTag("ABSORB_IMMUNE")
	
    inst.AnimState:SetBank("charlie_dragon_fire")
    inst.AnimState:SetBuild("charlie_dragon_fire")
    inst.AnimState:PlayAnimation("idle")
	inst.AnimState:SetLightOverride(1)

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end
	
    inst:AddComponent("complexprojectile")
    inst.components.complexprojectile:SetHorizontalSpeed(20)
    inst.components.complexprojectile:SetLaunchOffset(Vector3(0, 15, 0))
	inst.components.complexprojectile:SetOnLaunch(onthrown) 
    inst.components.complexprojectile:SetOnHit(onhit)
	inst.components.complexprojectile.gravity = -5

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(100)
    inst.components.combat:SetRange(100)
    inst.components.combat:SetKeepTargetFunction(function() return false end)	
	
    inst:AddComponent("heater")
    inst.components.heater.heatfn = GetHeatFn

	inst.persists = false
	
	return inst
end

return Prefab("charlie_critter_dragon", dragonfn, dragonassets, prefabs),
    Prefab("charlie_critter_dragon_mini", minidragonfn, minidragonassets, prefabs),
	Prefab("charlie_dragon_feather", dragonfeatherfn, dragonfeatherassets, prefabs),
	Prefab("charlie_dragon_sword", dragonswordfn, dragonswordassets, prefabs),
	Prefab("charlie_dragon_house", dragonhousefn, dragonhouseassets, prefabs),
	Prefab("charlie_dragon_fire", dragonfirefn, dragonfireassets, prefabs),
    MakePlacer("charlie_dragon_house_placer", "charlie_dragon_house", "charlie_dragon_house", "idle")
