local assets = 
{
    Asset("ANIM", "anim/nightmare_crack_upper.zip"),
}

local prefabs = 
{
    "dreadstone",
}

local function GetInspectStatus(inst)
    return inst:HasTag("dreadstone_stack_finished") and "FINISHED"
	    or nil
end

local function Regrow(inst)
    inst:RemoveTag("dreadstone_stack_finished")
	inst.AnimState:PlayAnimation("idle_open_rift", true)
    inst.components.workable:SetWorkLeft(6)
    inst.Physics:SetActive(true)
    if inst.regrowttask then
        inst.regrowttask:Cancel()
        inst.regrowttask = nil
    end
    if inst.regrowttaskinfo then
        inst.regrowttaskinfo = nil
    end	
end

local function OnWorkCallback(inst, worker, workleft)
    local tool = worker ~= nil and worker.components.inventory ~= nil and worker.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) or nil
    if worker ~= nil and ( worker.components.explosive ~= nil or ( tool ~= nil and tool:HasTag("multifunctional_tool") ) ) then
	    if math.random() < 0.1 then
		    inst.components.lootdropper:DropLoot()
		end
	else
	    inst.components.workable:SetWorkLeft(6)
	end
end

local function OnFinishCallback(inst, worker)
    inst.components.lootdropper:DropLoot()
	inst.SoundEmitter:PlaySound("dontstarve/wilson/rock_break")
	local fx = SpawnPrefab("small_puff")
	fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
	fx.Transform:SetScale(1.25, 1.25, 1.25)
    if worker ~= nil and worker.components.explosive ~= nil then
	    inst:Remove()
	else
	    inst:AddTag("dreadstone_stack_finished")
	    inst.AnimState:PlayAnimation("idle_open")
		inst.Physics:SetActive(false)
        if not inst.regrowttask then
            inst.regrowttask, inst.regrowttaskinfo = inst:ResumeTask(TUNING.TOTAL_DAY_TIME*10, Regrow)
        end
	end
end

local function OnLongUpdate(inst, dt)
    if inst.regrowttask then
        local new_time = math.max(inst:TimeRemainingInTask(inst.regrowttaskinfo) - dt, 0)

        inst.regrowttask:Cancel()
        inst.regrowttask = nil 
        inst.regrowttaskinfo = nil

        inst.regrowttask, inst.regrowttaskinfo = inst:ResumeTask(new_time, Regrow)
    end
end

local function OnSave(inst, data)
    data.finished = inst:HasTag("dreadstone_stack_finished") or nil
    if inst.regrowttaskinfo then
        data.regrowttask = inst:TimeRemainingInTask(inst.regrowttaskinfo)
    end
end

local function OnLoad(inst, data)
    if data ~= nil then    
	    if data.finished then
            inst:AddTag("dreadstone_stack_finished")
            inst.AnimState:PlayAnimation("idle_open")
			inst.Physics:SetActive(false)
		end
        if data.regrowttask then
            if inst.regrowttask then 
			    inst.regrowttask:Cancel() 
				inst.regrowttask = nil 		
			end
            inst.regrowttaskinfo = nil
            inst.regrowttask, inst.regrowttaskinfo = inst:ResumeTask(data.regrowttask, Regrow)
        end 		
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	inst.entity:AddMiniMapEntity()

    MakeObstaclePhysics(inst, 1)

    inst.AnimState:SetBuild("nightmare_crack_upper")
    inst.AnimState:SetBank("nightmare_crack_upper")
    inst.AnimState:PlayAnimation("idle_open_rift", true)

	inst.MiniMapEntity:SetIcon("dreadstone_stack.tex")
	
	inst:AddTag("dreadstone_stack")
	inst:AddTag("shadow_destroy_immune")
	inst:AddTag("pugalisk_avoids")

    inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = GetInspectStatus
	
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"dreadstone"})
	
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.MINE)
	inst.components.workable:SetMaxWork(6)
	inst.components.workable:SetWorkLeft(6)
	inst.components.workable:SetOnWorkCallback(OnWorkCallback)
	inst.components.workable:SetOnFinishCallback(OnFinishCallback)
	inst.components.workable.savestate = true

	inst.OnLongUpdate = OnLongUpdate
    inst.OnSave = OnSave
    inst.OnLoad = OnLoad
	
    return inst
end

return Prefab("dreadstone_stack", fn, assets, prefabs)