local assets =
{
	Asset("ANIM", "anim/armor_dreadstone.zip"),
}

local assets_hat =
{
	Asset("ANIM", "anim/hat_dreadstone.zip"),
}

local function OnBlocked(owner)
	owner.SoundEmitter:PlaySound("dontstarve/wilson/hit_marble") --#TODO DREADSTONE SOUND
end

local function GetSetBonusEquip(inst, owner)
	local hat = owner.components.inventory ~= nil and owner.components.inventory:GetEquippedItem(EQUIPSLOTS.HEAD) or nil
	return hat ~= nil and hat.prefab == "dreadstonehat" and hat or nil
end

local function DoRegen(inst, owner)
	if owner.components.sanity ~= nil then
		local setbonus = GetSetBonusEquip(inst, owner) ~= nil and 1.5 or 1
		local rate = 1 / Lerp(1 / (0.01 / 6) / 1.5, 1 / (0.01 / 10) / 1.5, owner.components.sanity:GetPercent())
		inst.components.armor:SetCondition(math.min(inst.components.armor.condition + inst.components.armor.maxcondition * rate * setbonus, inst.components.armor.maxcondition))
	end
	if inst.components.armor.condition >= inst.components.armor.maxcondition then
		inst.regentask:Cancel()
		inst.regentask = nil
	end
end

local function StartRegen(inst, owner)
	if inst.regentask == nil then
		inst.regentask = inst:DoPeriodicTask(1, DoRegen, nil, owner)
	end
end

local function StopRegen(inst)
	if inst.regentask ~= nil then
		inst.regentask:Cancel()
		inst.regentask = nil
	end
end

local function onequip(inst, owner)
	owner.AnimState:OverrideSymbol("swap_body", "armor_dreadstone", "swap_body")
	inst:ListenForEvent("blocked", OnBlocked, owner)

	if owner.components.sanity ~= nil and inst.components.armor.condition < inst.components.armor.maxcondition then
		StartRegen(inst, owner)
	else
		StopRegen(inst)
	end
end

local function onunequip(inst, owner)
	owner.AnimState:ClearOverrideSymbol("swap_body")
	inst:RemoveEventCallback("blocked", OnBlocked, owner)

	StopRegen(inst)
end

local function OnTakeDamage(inst, amount)
	if inst.regentask == nil and inst.components.equippable:IsEquipped() then
		local owner = inst.components.inventoryitem.owner
		if owner ~= nil and owner.components.sanity ~= nil then
			StartRegen(inst, owner)
		end
	end
end

local function CalcDapperness(inst, owner)
    if owner:HasTag("charlie") then
	    return 0
	end
	local insanity = owner.components.sanity ~= nil
	local other = GetSetBonusEquip(inst, owner)
	if other ~= nil then
		return (insanity and (inst.regentask ~= nil or other.regentask ~= nil) and TUNING.CRAZINESS_MED or TUNING.CRAZINESS_SMALL) * 0.5
	end
	return insanity and inst.regentask ~= nil and TUNING.CRAZINESS_MED or TUNING.CRAZINESS_SMALL
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function fn()
	local inst = CreateEntity()

	inst.entity:AddTransform()
	inst.entity:AddAnimState()

	MakeInventoryPhysics(inst)

	inst.AnimState:SetBank("armor_dreadstone")
	inst.AnimState:SetBuild("armor_dreadstone")
	inst.AnimState:PlayAnimation("anim")

	inst:AddTag("dreadstone_armor")
	inst:AddTag("shadow_item")

	inst:AddComponent("inspectable")

	inst:AddComponent("inventoryitem")

	inst:AddComponent("armor")
	inst.components.armor:InitCondition(1800, 0.9) 
	inst.components.armor.ontakedamage = OnTakeDamage

	inst:AddComponent("equippable")
	inst.components.equippable.equipslot = EQUIPSLOTS.BODY
	inst.components.equippable:SetOnEquip(onequip)
	inst.components.equippable:SetOnUnequip(onunequip)
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst.components.equippable.dapperfn = CalcDapperness
	else
	    inst:AddComponent("dapperness")
        inst.components.dapperness.dapperfn = CalcDapperness
	end	

	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		
	end

	return inst
end

local function dreadstone_getsetbonusequip(inst, owner)
	local body = owner.components.inventory ~= nil and owner.components.inventory:GetEquippedItem(EQUIPSLOTS.BODY) or nil
	return body ~= nil and body.prefab == "armordreadstone" and body or nil
end

local function DoRegenHat(inst, owner)
	if owner.components.sanity ~= nil then
		local setbonus = dreadstone_getsetbonusequip(inst, owner) ~= nil and 1.5 or 1
		local rate = 1 / Lerp(1 / (0.01 / 6) / 1.5, 1 / (0.01 / 10) / 1.5, owner.components.sanity:GetPercent())
		inst.components.armor:SetCondition(math.min(inst.components.armor.condition + inst.components.armor.maxcondition * rate * setbonus, inst.components.armor.maxcondition))
	end
	if inst.components.armor.condition >= inst.components.armor.maxcondition then
		inst.regentask:Cancel()
		inst.regentask = nil
	end
end

local function StartRegenHat(inst, owner)
	if inst.regentask == nil then
		inst.regentask = inst:DoPeriodicTask(1, DoRegenHat, nil, owner)
	end
end

local function StopRegenHat(inst)
	if inst.regentask ~= nil then
		inst.regentask:Cancel()
		inst.regentask = nil
	end
end

local function onequiphat(inst, owner)
	owner.AnimState:OverrideSymbol("swap_hat", "hat_dreadstone", "swap_hat")
	owner.AnimState:Show("HAT")
	owner.AnimState:Show("HAIR_HAT")
	owner.AnimState:Hide("HAIR_NOHAT")
	owner.AnimState:Hide("HAIR")
	
	if owner:HasTag("player") then
		owner.AnimState:Hide("HEAD")
		owner.AnimState:Show("HEAD_HAIR")
		owner.AnimState:Hide("HAIRFRONT")
	end	
	
	inst:ListenForEvent("blocked", OnBlocked, owner)

	if owner.components.sanity ~= nil and inst.components.armor.condition < inst.components.armor.maxcondition then
		StartRegenHat(inst, owner)
	else
		StopRegenHat(inst)
	end
end

local function onunequiphat(inst, owner)
	owner.AnimState:Hide("HAT")
	owner.AnimState:Hide("HAIR_HAT")
	owner.AnimState:Show("HAIR_NOHAT")
	owner.AnimState:Show("HAIR")

	if owner:HasTag("player") then
		owner.AnimState:Show("HEAD")
		owner.AnimState:Hide("HEAD_HAIR")
		owner.AnimState:Show("HAIRFRONT")
	end
		
	inst:RemoveEventCallback("blocked", OnBlocked, owner)

	StopRegenHat(inst)
end

local function OnTakeDamageHat(inst, amount)
	if inst.regentask == nil and inst.components.equippable:IsEquipped() then
		local owner = inst.components.inventoryitem.owner
		if owner ~= nil and owner.components.sanity ~= nil then
			StartRegenHat(inst, owner)
		end
	end
end

local function CalcDappernessHat(inst, owner)
    if owner:HasTag("charlie") then
	    return 0
	end
	local insanity = owner.components.sanity ~= nil
	local other = dreadstone_getsetbonusequip(inst, owner)
	if other ~= nil then
		return (insanity and (inst.regentask ~= nil or other.regentask ~= nil) and TUNING.CRAZINESS_MED or TUNING.CRAZINESS_SMALL) * 0.5
	end
	return insanity and inst.regentask ~= nil and TUNING.CRAZINESS_MED or TUNING.CRAZINESS_SMALL
end

local function fn_hat()
	local inst = CreateEntity()

	inst.entity:AddTransform()
	inst.entity:AddAnimState()

	MakeInventoryPhysics(inst)

	inst.AnimState:SetBank("dreadstonehat")
	inst.AnimState:SetBuild("hat_dreadstone")
	inst.AnimState:PlayAnimation("anim")

    inst:AddTag("hat")
	inst:AddTag("dreadstone_hat")
	inst:AddTag("shadow_item")

	inst:AddComponent("inspectable")

	inst:AddComponent("inventoryitem")

	inst:AddComponent("armor")
	inst.components.armor:InitCondition(1800, 0.9) 
	inst.components.armor.ontakedamage = OnTakeDamageHat

	inst:AddComponent("equippable")
	inst.components.equippable.equipslot = EQUIPSLOTS.HEAD
	inst.components.equippable:SetOnEquip(onequiphat)
	inst.components.equippable:SetOnUnequip(onunequiphat)
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst.components.equippable.dapperfn = CalcDappernessHat
		
		inst:AddComponent("waterproofer")
		inst.components.waterproofer:SetEffectiveness(TUNING.WATERPROOFNESS_SMALL)
	else
	    inst:AddComponent("dapperness")
        inst.components.dapperness.dapperfn = CalcDappernessHat
	end	

	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		
	end

	return inst
end

return Prefab("armordreadstone", fn, assets),
    Prefab("dreadstonehat", fn_hat, assets_hat)