require("prefabutil")

local assets =
{
	Asset("ANIM", "anim/flowers.zip"),
}

local rose_assets = 
{
	Asset("ANIM", "anim/flowers.zip"),
	Asset("ANIM", "anim/petals_rose.zip"),
}

local prefabs =
{
    "petals",
	"rose_fx",
}

local names = {"f1","f2","f3","f4","f5","f6","f7","f8","f9","f10"}
local ROSE_NAME = "rose"
local ROSE_CHANCE = 0.01

local BURNROSE_ENG = {"Ah, my beloved rose!", "It's no good doing so!", "It's irrational to put this in the fire!"}
local BURNROSE_CN = {"啊，我心爱的玫瑰！", "这样做是没好处的！", "往火里放这个极不理智！"}
local BURNFLOWER_ENG = {"Ah, my beloved flower!", "It's no good doing so!", "It's irrational to put this in the fire!"}
local BURNFLOWER_CN = {"啊，我心爱的花朵！", "这样做是没好处的！", "往火里放这个极不理智！"}

local function setflowertype(inst, name)
    if inst.animname == nil or (name ~= nil and inst.animname ~= name) then
        if inst.animname == ROSE_NAME then
            inst:RemoveTag("thorny")
        end
        inst.animname = name or (math.random() < ROSE_CHANCE and ROSE_NAME or names[math.random(#names)])
        inst.AnimState:PlayAnimation(inst.animname)
        if inst.animname == ROSE_NAME then
            inst:AddTag("thorny")
			inst:RemoveAllEventCallbacks()
			if inst.components.transformer ~= nil then
			    inst.components.transformer.StartTransform = function() end
                inst.components.transformer.StartRevert = function() end
				inst.components.transformer.Transform = function() end
				inst.components.transformer.Revert = function() end
			end
        end
    end
end

local function ResetFlower(inst)
    if inst.components.interactivelevel ~= nil then
	    inst.components.interactivelevel.inactive = true
	end
    if inst.resettask then
        inst.resettask:Cancel()
        inst.resettask = nil
    end
    if inst.resettaskinfo then
        inst.resettaskinfo = nil
    end	
end

local function onsave(inst, data)
	data.anim = inst.animname
	data.magic_flower = inst:HasTag("magicflower") or nil
	data.no_petals = inst:HasTag("no_petals") or nil
    if inst.resettaskinfo then
        data.resettask = inst:TimeRemainingInTask(inst.resettaskinfo)
    end	
end

local function onload(inst, data)
    setflowertype(inst, data ~= nil and data.anim or nil)
	if data ~= nil and data.magic_flower then
	    inst:AddTag("magicflower")
	end
	if data ~= nil and data.no_petals then
	    inst:AddTag("no_petals")
	    if inst.components.pickable ~= nil then
		    inst.components.pickable:SetUp(nil, nil)
		end
	end		
    if data ~= nil and data.resettask then
        if inst.components.interactivelevel ~= nil then
	        inst.components.interactivelevel.inactive = false
	    end
        if inst.resettask then 
		    inst.resettask:Cancel() 
			inst.resettask = nil 		
    	end
        inst.resettaskinfo = nil
        inst.resettask, inst.resettaskinfo = inst:ResumeTask(data.resettask, ResetFlower)
    end 
end

local function onpickedfn(inst, picker)
    if picker == nil then
	    inst:Remove()
	    return
	end
    if picker:HasTag("plantkin") then
        if picker.killedplantfn ~= nil then
            picker.killedplantfn(picker)
        end
    else
	    if picker.components.sanity ~= nil then
		    if not picker:HasTag("flower_lover") and not inst:HasTag("no_petals") then
		        picker.components.sanity:DoDelta(TUNING.SANITY_TINY)
			else
			    if inst.animname == ROSE_NAME and not inst:HasTag("no_petals") then
				    picker.components.sanity:DoDelta(TUNING.SANITY_TINY*3)
				elseif not inst:HasTag("no_petals") then
				    picker.components.sanity:DoDelta(TUNING.SANITY_TINY)
				end
			end
	    end
    end
	
    if inst.animname == ROSE_NAME 
	    and picker.components.combat ~= nil 
	    and not picker:HasTag("flower_lover") 
	    and not picker:HasTag("bramble_resistant") 
	    --[[and not picker:HasTag("armorcactus")]]
	then
        picker.components.combat:GetAttacked(inst, TUNING.ROSE_DAMAGE)
        picker:PushEvent("thorns")
    end
	
	if inst.animname == ROSE_NAME and picker:HasTag("flower_lover") and picker.components.inventory ~= nil then
	    picker.components.inventory:GiveItem(SpawnPrefab("rose_inv"), nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
	end	
	
	inst:Remove()
end

local function ongustpickfn(inst)
    if inst.components.pickable and inst.components.pickable:CanBePicked() and inst.components.pickable.product ~= nil then
        inst.components.pickable:MakeEmpty()
        local x, y, z = inst.Transform:GetWorldPosition()
        local petals = SpawnPrefab(inst.components.pickable.product)
        petals.Transform:SetPosition(x, y, z)
        inst:Remove()
    end
end

local function SanityAura(inst, observer)

    if observer ~= nil and observer:HasTag("flower_lover") then
        if inst.animname == ROSE_NAME then
            return TUNING.SANITYAURA_SMALL    
        else
            return TUNING.SANITYAURA_SMALL*0.5    
        end
    end

    return 0
end

local function onburnt(inst)
    DefaultBurntFn(inst)
	
    if GetPlayer() and GetPlayer():HasTag("flower_lover") and not GetPlayer():HasTag("sanitylosscooldown") and inst:GetDistanceSqToInst(GetPlayer()) < 8*8 then
	    if inst.animname == ROSE_NAME then
	        GetPlayer().components.sanity:DoDelta(-TUNING.SANITY_MED)
		    if TUNING.LANGUAGE == true then
	            GetPlayer().components.talker:Say(BURNROSE_ENG[math.random(#BURNROSE_ENG)])
	        elseif TUNING.LANGUAGE == false then
	            GetPlayer().components.talker:Say(BURNROSE_CN[math.random(#BURNROSE_CN)])
	        end
		else
	        GetPlayer().components.sanity:DoDelta(-TUNING.SANITY_MED/3)
		    if TUNING.LANGUAGE == true then
	            GetPlayer().components.talker:Say(BURNFLOWER_ENG[math.random(#BURNFLOWER_ENG)])
	        elseif TUNING.LANGUAGE == false then
	            GetPlayer().components.talker:Say(BURNFLOWER_CN[math.random(#BURNFLOWER_CN)])
	        end			
		end
		GetPlayer():AddTag("sanitylosscooldown")
		if not GetPlayer().components.timer then
            GetPlayer():AddComponent("timer")
		end
		GetPlayer().components.timer:StartTimer("sanityloss_cooldown", TUNING.CHARLIE_SANITYLOSS_COOLDOWNTIME)
	end
end

local function GetStatus(inst)
    return inst.animname == ROSE_NAME and "ROSE" or nil
end

local function OnActivate(inst, doer)
    if doer ~= nil and doer:HasTag("flowercommunicator") then
	    local MEET_COMMON_FLOWER_SAY = {"How are you today, little one?", "Don't worry, I will take good care of you.", "I hope you grow up, and become stronger!", "Little flower, could you tell me how to find him?", "I am glad you have a happy day!"}
		local MEET_ROSE_SAY = {"Honey, how are you today?", "One day, I will find him with you...", "They don't understand why you have thorns, but I do.", "Even in this dark forest, you are still my favorite.", "Sweetheart, I'll tell you a secret..."}
		local POSSIBLE_ITEMS_TO_GET = 
		{
		    "cutgrass", 
			"twigs",   
			"cutreeds", 
			"log", 
			"boards", 
			"foliage", 
			"rope", 
			"cutstone", 
			"papyrus", 
			"beeswax", 
		}
		if TUNING.LANGUAGE == false then
		    MEET_COMMON_FLOWER_SAY = {"今天好吗，小家伙？", "别担心，我会好好照料你的。", "希望你长大，变得更强壮！", "小花朵，你能告诉我怎么找到他吗？", "很高兴你拥有快乐的一天！"}
			MEET_ROSE_SAY = {"亲爱的，你今天怎么样了？", "总有一天，我会和你一起找到他的。。。", "他们不理解你为什么带刺，但是我懂。", "即使在这黑暗森林里，你依旧是我的最爱。", "甜心啊，我告诉你一个秘密。。。"}
		end
		if doer.components.talker ~= nil then
		    if inst.animname == ROSE_NAME then
			    table.insert(POSSIBLE_ITEMS_TO_GET, "amulet")
				table.insert(POSSIBLE_ITEMS_TO_GET, "redgem")
		        doer.components.talker:Say(MEET_ROSE_SAY[math.random(#MEET_ROSE_SAY)])	
			else
			    doer.components.talker:Say(MEET_COMMON_FLOWER_SAY[math.random(#MEET_COMMON_FLOWER_SAY)])	
			end
		end
	    if doer:HasTag("flowersupercommunicator") then
		    table.insert(POSSIBLE_ITEMS_TO_GET, "thulecite_pieces")
			table.insert(POSSIBLE_ITEMS_TO_GET, "livinglog")
			if inst.animname == ROSE_NAME and math.random() < 0.25 then
			    if doer.components.inventory ~= nil then
				    doer.components.inventory:GiveItem(SpawnPrefab("shadow_rose_bouquet"), nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
				end
			end
		    if inst.animname ~= ROSE_NAME and math.random() < 0.1 then
			    SpawnPrefab("small_puff").Transform:SetPosition(inst.Transform:GetWorldPosition())
		        setflowertype(inst, ROSE_NAME)
			end						
		end
	    if doer.components.inventory ~= nil then
		    doer.components.inventory:GiveItem(SpawnPrefab(GetRandomItem(POSSIBLE_ITEMS_TO_GET)), nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
		end	
        if not inst.resettask then
            inst.resettask, inst.resettaskinfo = inst:ResumeTask(TUNING.TOTAL_DAY_TIME, ResetFlower)
        end			
	end
end

local function OnLongUpdate(inst, dt)
    if inst.resettask and inst.resettaskinfo then
        local new_time = math.max(inst:TimeRemainingInTask(inst.resettaskinfo) - dt, 0)

        inst.resettask:Cancel()
        inst.resettask = nil 
        inst.resettaskinfo = nil

        inst.resettask, inst.resettaskinfo = inst:ResumeTask(new_time, ResetFlower)
    end
end

local function fn(Sim)
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	
    inst.AnimState:SetBank("flowers")
    inst.AnimState:SetBuild("flowers")
    inst.AnimState:SetRayTestOnBB(true)
    
    inst:AddTag("flower")
    inst:AddTag("cattoy")

    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = GetStatus
	
    inst:AddComponent("pickable")
    inst.components.pickable.picksound = "dontstarve/wilson/pickup_plants"
    inst.components.pickable:SetUp("petals", 10)
	inst.components.pickable.onpickedfn = onpickedfn
    inst.components.pickable.quickpick = true

    MakeSmallBurnable(inst)
    MakeSmallPropagator(inst)
    inst.components.burnable:SetOnBurntFn(onburnt)

--[[
    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("blowinwindgust")
        inst.components.blowinwindgust:SetWindSpeedThreshold(TUNING.FLOWER_WINDBLOWN_SPEED)
        inst.components.blowinwindgust:SetDestroyChance(TUNING.FLOWER_WINDBLOWN_FALL_CHANCE)
        inst.components.blowinwindgust:SetDestroyFn(ongustpickfn)
        inst.components.blowinwindgust:Start()
    end
]]	
	inst:AddComponent("sanityaura") 
	inst.components.sanityaura.aurafn = SanityAura

    setflowertype(inst)

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    if inst.animname ~= nil and inst.animname ~= ROSE_NAME then
            inst:AddComponent("transformer")
            inst.components.transformer:SetTransformEvent("fullmoon")
            inst.components.transformer:SetRevertEvent("daytime")
            inst.components.transformer:SetOnLoadCheck(function() 
                if not GetWorld():IsCave() then
                    return (GetClock():IsNight() and GetClock():GetMoonPhase() == "full")
                end
                return false
            end)         
	    	inst.components.transformer.transformPrefab = "flower_evil"
		end
		
		inst.components.pickable.wildfirestarter = true
		
		inst.components.burnable:MakeDragonflyBait(1)
    end
	
	if GetPlayer() and GetPlayer().prefab == "charlie" then	
        inst:AddComponent("interactivelevel")
        inst.components.interactivelevel.IsFlower = true
        inst.components.interactivelevel.OnActivate = OnActivate	
	end
	
	inst.OnLongUpdate = OnLongUpdate
	
    --------SaveLoad
    inst.OnSave = onsave
    inst.OnLoad = onload 
  
    return inst
end

function rosefn()
    local inst = fn()

    inst:SetPrefabName("flower")
	
	setflowertype(inst, ROSE_NAME)
	
	--[[if inst:HasTag("no_petals") then
	    if inst.components.pickable ~= nil then
		    inst.components.pickable:SetUp(nil, nil)
		end
	end]]
	
    return inst
end

local function SanityAura_inv(inst, observer)
    if inst:IsInLimbo() then
	    return 0
	end
    return observer:HasTag("flower_lover") and TUNING.SANITYAURA_SMALL*0.125 or 0
end

local function LoseSanityForFlowerLover(inst)
    local owner = inst.components.inventoryitem ~= nil and inst.components.inventoryitem:GetGrandOwner() or nil
    if owner ~= nil and owner:HasTag("flower_lover") then
	    if owner.components.sanity ~= nil then
		    owner.components.sanity:DoDelta(-TUNING.SANITY_MED)
		end
	end
end

local function SpawnRoseFX(inst)
    SpawnPrefab("rose_fx").Transform:SetPosition(inst.Transform:GetWorldPosition())
end

local function ondeploy(inst, pt, deployer)
    local rose = SpawnPrefab("flower_rose")
	local PerishablePercent = inst.components.perishable ~= nil and inst.components.perishable:GetPercent() or 0.01
	local RoseIsFresh = inst.components.perishable ~= nil and inst.components.perishable:IsFresh() 
	local RoseIsStale = inst.components.perishable ~= nil and inst.components.perishable:IsStale() 
	local RoseIsSpoiled = inst.components.perishable ~= nil and inst.components.perishable:IsSpoiled() 
	local ConsumeMode = (RoseIsFresh and 1) or (RoseIsStale and 2) or (RoseIsSpoiled and 3) or 1
	local ConsumeBase = (PerishablePercent >= 1 and 0) or (RoseIsFresh and 2.5) or 5
	if PerishablePercent == 0 then
	    PerishablePercent = 0.01
	end
	local hunger_to_consume = math.clamp(ConsumeBase / PerishablePercent * ConsumeMode, 0, 500)
	if rose ~= nil then
	    rose.Transform:SetPosition(pt.x, pt.y, pt.z)
		rose:AddTag("no_petals")
	    if rose.components.pickable ~= nil then
		    rose.components.pickable:SetUp(nil, nil)
		end
	    if rose.components.interactivelevel ~= nil then
		    rose.components.interactivelevel.inactive = false
		end
		if inst.ApplyInheritedMoisture ~= nil then
			inst:ApplyInheritedMoisture(rose)
		end
		if deployer ~= nil and deployer.components.hunger ~= nil and hunger_to_consume > 0 then
		    deployer.components.hunger:DoDelta(-hunger_to_consume)
		end
        if not rose.resettask then
            rose.resettask, rose.resettaskinfo = rose:ResumeTask(TUNING.TOTAL_DAY_TIME, ResetFlower)
        end				
		inst:Remove()
	end
end

local function CanPlantAtPoint(x, y, z)
    local tile = GetWorld().Map:GetTileAtPoint(x, y, z)
    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    if tile == GROUND.ASH or tile == GROUND.VOLCANO 
		    or tile == GROUND.VOLCANO_ROCK or tile == GROUND.BRICK_GLOW
			or tile == GROUND.VOLCANO_LAVA or tile == GROUND.SNAKESKIN
		then
		    return false
		end
	end
	if IsDLCEnabled(PORKLAND_DLC) then
	    if tile == GROUND.FOUNDATION or tile == GROUND.COBBLEROAD
		    or tile == GROUND.GASJUNGLE or tile == GROUND.PIGRUINS
			or tile == GROUND.PIGRUINS_NOCANOPY or tile == GROUND.BEARDRUG --or tile == GROUND.INTERIOR
		then                                                               --Use inst.GetIsOnLandOutside so that GROUND.INTERIOR can be not included
		    return false
		end
	end
    if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and tile ~= GROUND.ROCKY and tile ~= GROUND.ROAD
		and tile ~= GROUND.UNDERROCK and tile ~= GROUND.WOODFLOOR and tile ~= GROUND.CARPET and tile ~= GROUND.CHECKER
		and tile < GROUND.UNDERGROUND
	then
		return true
	end
	return false
end

local function item_deploytest(inst, pt, deployer, rot)
    if deployer ~= nil and deployer.components.hunger ~= nil and deployer.components.hunger:IsStarving() then
	    return false
    end
    if inst.GetIsOnLandOutside ~= nil then
	    if not inst:GetIsOnLandOutside(pt.x, pt.y, pt.z) then
		    return false
		end
	end
	if inst.IsPosSurroundedByLand ~= nil then
	    if not inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 1.5) then
		    return false
		end
	end
	if CanPlantAtPoint(pt.x, pt.y, pt.z) then
	    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 4, nil, {"NOBLOCK", "player", "FX"})
		local min_spacing = inst.components.deployable.min_spacing or 2
	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt ) < min_spacing*min_spacing then
					return false
				end
			end
		end
		return true
	end
	return false
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function rose_inv_fn()
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	
    inst.AnimState:SetBank("petals_rose")
    inst.AnimState:SetBuild("petals_rose")
	inst.AnimState:PlayAnimation("idle", true)	

    MakeInventoryPhysics(inst)
	
	inst:AddTag("invflower")
    inst:AddTag("show_spoilage")
    inst:AddTag("cattoy")
	inst:AddTag("conjurable")

	inst:AddComponent("inventoryitem")

	inst:AddComponent("sanityaura") 
	inst.components.sanityaura.aurafn = SanityAura_inv

    inst:AddComponent("inspectable")
	
	inst:AddComponent("tradable")

    inst:AddComponent("conjurable")
	
 	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.PERISH_MED)
	inst.components.perishable:StartPerishing()
	inst.components.perishable:SetOnPerishFn(LoseSanityForFlowerLover)
	inst.components.perishable.onperishreplacement = "spoiled_food"

    inst:AddComponent("stackable")
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

    if GetPlayer() and GetPlayer():HasTag("flower_lover") then
	    inst:AddComponent("deployable")
	    inst.components.deployable.ondeploy = ondeploy
	    inst.components.deployable.placer = "rose_placer"
	    inst.components.deployable.test = item_deploytest
	    inst.components.deployable.deploydistance = 0.5
	    inst.components.deployable.min_spacing = 0.5
	end

	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
		inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)	
		
		inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = TUNING.APPEASEMENT_SMALL			
	end
	
    MakeSmallBurnable(inst, TUNING.MED_BURNTIME)
    MakeSmallPropagator(inst)

 	inst:AddComponent("fuel")
    inst.components.fuel.fuelvalue = TUNING.MED_FUEL
	
 	inst:AddComponent("repairer")
    inst.components.repairer.repairmaterial = "rose"
    inst.components.repairer.perishrepairvalue = 1
	
	return inst
end

return Prefab( "forest/objects/flower", fn, assets, prefabs),
       Prefab( "forest/objects/flower_rose", rosefn, assets, prefabs),
	   Prefab("rose_inv", rose_inv_fn, rose_assets, prefabs),
	   MakePlacer("rose_placer", "flowers", "flowers", "rose")