
local pizzaassets = 
{
    Asset("ANIM", "anim/charlie_pizza.zip"),
}

local pizzacutterassets = 
{
    Asset("ANIM", "anim/charlie_pizza_cutter.zip"),
    Asset("ANIM", "anim/swap_charlie_pizza_cutter.zip"),
}

local pizzabakerassets = 
{
    Asset("ANIM", "anim/charlie_pizza_baker.zip"),
}

local prefabs = 
{

}

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity) 
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity) 
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function CheckPizzaType(inst)
    if inst.components.edible ~= nil then
	    if inst.IsMeat then
		    inst.components.edible.foodtype = "MEAT"
		elseif inst.IsVegetarian or inst.IsVegan then
		    inst.components.edible.foodtype = "VEGGIE"
		elseif inst.IsUnpalatable then
		    inst.components.edible.foodtype = "HORRIBLE"
		elseif not inst.IsMixed then
		    inst.components.edible.foodtype = "INEDIBLE"
		end
	end
end

local function CommonPizzaOnSave(inst, data)
    if inst.IsMeat then
	    data.IsMeat = true
	elseif inst.IsVegetarian then
	    data.IsVegetarian = true
	elseif inst.IsVegan then
	    data.IsVegan = true
	elseif inst.IsUnpalatable then
	    data.IsUnpalatable = true
	elseif inst.IsMixed then
	    data.IsMixed = true
	end
end

local function CommonPizzaOnLoad(inst, data)
    if data ~= nil then
	    if data.IsMeat then
		    inst.IsMeat = true
		elseif data.IsVegetarian then
		    inst.IsVegetarian = true
		elseif data.IsVegan then
		    inst.IsVegan = true
		elseif data.IsUnpalatable then
		    inst.IsUnpalatable = true
		elseif data.IsMixed then
		    inst.IsMixed = true
		end
	end
end

local function pizzafn1()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
	
	MakeInventoryPhysics(inst)
	
    inst:AddTag("pizza_full")
	inst:AddTag("preparedfood")

    inst.AnimState:SetBank("charlie_pizza")
    inst.AnimState:SetBuild("charlie_pizza")
    inst.AnimState:PlayAnimation("full")
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")

    if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 8		
    end

    inst:AddComponent("fuel")
    inst.components.fuel.fuelvalue = 180
	
    inst:AddComponent("bait")	
	
	inst:AddComponent("tradable")
	inst.components.tradable.goldvalue = 8
	inst.components.tradable.dubloonvalue = 12
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = 40
	
	inst:AddComponent("edible")
	inst.components.edible.foodtype = "GENERIC"
	inst.components.edible.foodstate = "PREPARED"
	inst.components.edible.healthvalue = 45
	inst.components.edible.hungervalue = 125
	inst.components.edible.sanityvalue = 20
    --inst.components.edible.oneaten = PizzaOnEaten
	
	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.TOTAL_DAY_TIME*10)
	inst.components.perishable:StartPerishing()
	inst.components.perishable.onperishreplacement = "spoiled_food"
	
    MakeSmallBurnable(inst, 40)
	MakeSmallPropagator(inst)
	
	inst:AddComponent("pizzacuttable")
    inst.components.pizzacuttable.product = "charlie_pizza_half"
	
	inst:DoTaskInTime(0, CheckPizzaType)
	
	inst.OnSave = CommonPizzaOnSave
	inst.OnLoad = CommonPizzaOnLoad
	
	return inst
end


local function pizzafn2()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
	
	MakeInventoryPhysics(inst)
	
    inst:AddTag("pizza_half")
	inst:AddTag("preparedfood")

    inst.AnimState:SetBank("charlie_pizza")
    inst.AnimState:SetBuild("charlie_pizza")
    inst.AnimState:PlayAnimation("half")
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")

    if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 4		
    end

    inst:AddComponent("fuel")
    inst.components.fuel.fuelvalue = 90
	
    inst:AddComponent("bait")	
	
	inst:AddComponent("tradable")
	inst.components.tradable.goldvalue = 4
	inst.components.tradable.dubloonvalue = 6
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = 40
	
	inst:AddComponent("edible")
	inst.components.edible.foodtype = "GENERIC"
	inst.components.edible.foodstate = "PREPARED"
	inst.components.edible.healthvalue = 22.5
	inst.components.edible.hungervalue = 62.5
	inst.components.edible.sanityvalue = 10
    --inst.components.edible.oneaten = PizzaOnEaten
	
	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.TOTAL_DAY_TIME*9)
	inst.components.perishable:StartPerishing()
	inst.components.perishable.onperishreplacement = "spoiled_food"

	inst:AddComponent("pizzacuttable")
    inst.components.pizzacuttable.product = "charlie_pizza_quarter"
	
    MakeSmallBurnable(inst, 20)
	MakeSmallPropagator(inst)
	
	inst:DoTaskInTime(0, CheckPizzaType)
	
	inst.OnSave = CommonPizzaOnSave
	inst.OnLoad = CommonPizzaOnLoad
	
	return inst
end

local function pizzafn3()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
	
	MakeInventoryPhysics(inst)
	
    inst:AddTag("pizza_quarter")
	inst:AddTag("preparedfood")

    inst.AnimState:SetBank("charlie_pizza")
    inst.AnimState:SetBuild("charlie_pizza")
    inst.AnimState:PlayAnimation("quarter")
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")

    if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 2		
    end

    inst:AddComponent("fuel")
    inst.components.fuel.fuelvalue = 45
	
    inst:AddComponent("bait")	
	
	inst:AddComponent("tradable")
	inst.components.tradable.goldvalue = 2
	inst.components.tradable.dubloonvalue = 3
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = 40
	
	inst:AddComponent("edible")
	inst.components.edible.foodtype = "GENERIC"
	inst.components.edible.foodstate = "PREPARED"
	inst.components.edible.healthvalue = 11.25
	inst.components.edible.hungervalue = 31.25
	inst.components.edible.sanityvalue = 5
    --inst.components.edible.oneaten = PizzaOnEaten

	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.TOTAL_DAY_TIME*8)
	inst.components.perishable:StartPerishing()
	inst.components.perishable.onperishreplacement = "spoiled_food"

	inst:AddComponent("pizzacuttable")
    inst.components.pizzacuttable.product = "charlie_pizza_one_eighth"
	
    MakeSmallBurnable(inst, 10)
	MakeSmallPropagator(inst)
	
	inst:DoTaskInTime(0, CheckPizzaType)
	
	inst.OnSave = CommonPizzaOnSave
	inst.OnLoad = CommonPizzaOnLoad
	
	return inst
end

local function pizzafn4()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
	
	MakeInventoryPhysics(inst)
	
    inst:AddTag("pizza_one_eighth")
	inst:AddTag("preparedfood")

    inst.AnimState:SetBank("charlie_pizza")
    inst.AnimState:SetBuild("charlie_pizza")
    inst.AnimState:PlayAnimation("one_eighth")
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")

    if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)

        inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = 1		
    end

    inst:AddComponent("fuel")
    inst.components.fuel.fuelvalue = 22.5
	
    inst:AddComponent("bait")	
	
	inst:AddComponent("tradable")
	inst.components.tradable.goldvalue = 1
	inst.components.tradable.dubloonvalue = 1
	
	inst:AddComponent("stackable")
	inst.components.stackable.maxsize = 40
	
	inst:AddComponent("edible")
	inst.components.edible.foodtype = "GENERIC"
	inst.components.edible.foodstate = "PREPARED"
	inst.components.edible.healthvalue = 5.625
	inst.components.edible.hungervalue = 15.625
	inst.components.edible.sanityvalue = 2.5
    --inst.components.edible.oneaten = PizzaOnEaten

	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.TOTAL_DAY_TIME*7)
	inst.components.perishable:StartPerishing()
	inst.components.perishable.onperishreplacement = "spoiled_food"
	
    MakeSmallBurnable(inst, 5)
	MakeSmallPropagator(inst)

	inst:DoTaskInTime(0, CheckPizzaType)
	
	inst.OnSave = CommonPizzaOnSave
	inst.OnLoad = CommonPizzaOnLoad
	
	return inst
end

local function onequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_charlie_pizza_cutter", "swap_charlie_pizza_cutter")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function onunequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

local function pizzacutterfn()
    local inst = CreateEntity()

	inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
	
	MakeInventoryPhysics(inst)
	
    inst:AddTag("pizza_cutter")

    inst.AnimState:SetBank("charlie_pizza_cutter")
    inst.AnimState:SetBuild("charlie_pizza_cutter")
    inst.AnimState:PlayAnimation("idle")

    inst:AddComponent("inspectable")
	
	inst:AddComponent("inventoryitem")
	
    if inst.GetIsOnWater ~= nil then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		
    end
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(17)
    
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)
	
	inst:AddComponent("pizzacutter")
	
	inst:AddComponent("finiteuses")
	inst.components.finiteuses:SetMaxUses(100)
	inst.components.finiteuses:SetUses(100)
	inst.components.finiteuses:SetOnFinished(inst.Remove)
	
	return inst
end


local function pizzabakerfn()

end

return Prefab("charlie_pizza_full", pizzafn1, pizzaassets, prefabs),
    Prefab("charlie_pizza_half", pizzafn2, pizzaassets, prefabs),
	Prefab("charlie_pizza_quarter", pizzafn3, pizzaassets, prefabs),
	Prefab("charlie_pizza_one_eighth", pizzafn4, pizzaassets, prefabs),
	Prefab("charlie_pizza_cutter", pizzacutterfn, pizzacutterassets, prefabs),
	Prefab("charlie_pizza_baker", pizzabakerfn, pizzabakerassets, prefabs)