require("prefabutil")

local assets =
{
	Asset("ANIM", "anim/lily_pad.zip"),
}

local prefabs =
{
    "lilypad_seed",
}

local prefabs_seed = 
{
    "lilypad_for_puddle",
}

local function fn()
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    inst.AnimState:SetBuild("lily_pad")
    inst.AnimState:SetBank("lily_pad")
	inst.AnimState:PlayAnimation("small_idle", true)
	inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
	inst.AnimState:SetLayer(LAYER_WORLD_BACKGROUND)
	inst.AnimState:SetSortOrder(3)
	
	inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")	
    inst:AddTag("LilyPadForPuddle")
	
	inst.PlantsWhichCannotBePlantedOnLilypads = { "seaweed", "brainysprout", "mangrovetree", "lotusplant", "watergrass", "seaflora", "seashroom" }

	return inst
end

local function OnHitWater(inst)
    inst.AnimState:OverrideSymbol("ripple2_big", "lily_pad", "ripple2_big")
	inst.AnimState:OverrideSymbol("ripple4_big", "lily_pad", "ripple4_big")
end

local function OnHitLand(inst)
    inst.AnimState:OverrideSymbol("ripple2_big", "lily_pad", "")
	inst.AnimState:OverrideSymbol("ripple4_big", "lily_pad", "")
end

local function BiggerAndBiggerOnSpawn(lilypad, dt)
    if lilypad ~= nil and lilypad.scale ~= nil and dt ~= nil then
	    lilypad.scale = math.min(lilypad.scale + dt, 1)
		lilypad.Transform:SetScale(lilypad.scale, lilypad.scale, lilypad.scale)		
		if lilypad.scale >= 1 then
		    lilypad.BiggerAndBiggerTask:Cancel()
			lilypad.BiggerAndBiggerTask = nil
			lilypad.scale = nil
            lilypad.AnimState:PlayAnimation("small_splash")
	        lilypad.AnimState:PushAnimation("small_idle", true)			
		end
	end
end

local function OnDeploy(inst, pt) 
	local puddles = TheSim:FindEntities(pt.x, pt.y, pt.z, 1, {"puddle"}, {"puddle_disappear"})
    if #puddles > 0 then
        inst = inst.components.stackable:Get()    
        inst:Remove()
		
        local lilypad = SpawnPrefab("lilypad_for_puddle")
        lilypad.Transform:SetPosition(puddles[1].Transform:GetWorldPosition())
		
		lilypad.AnimState:PlayAnimation("small_idle", true)	
		
		lilypad.scale = 0.1
		lilypad.Transform:SetScale(lilypad.scale, lilypad.scale, lilypad.scale)	
		
		lilypad.BiggerAndBiggerTask = lilypad:DoPeriodicTask(1/30, BiggerAndBiggerOnSpawn, nil, 1/30)
	end
end

local function IsPointNearNonLilyPadPuddle(inst, pt)
    local puddle = TheSim:FindEntities(pt.x, pt.y, pt.z, 1, {"puddle"}, {"puddle_disappear"})
    local lilypad = TheSim:FindEntities(pt.x, pt.y, pt.z, 1, {"LilyPadForPuddle"})	
    if #puddle > 0 and #lilypad <= 0 then
	    return true
	else
	    return false
	end
end

local function fn_seed()
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    inst.AnimState:SetBuild("lily_pad")
    inst.AnimState:SetBank("lily_pad")
	inst.AnimState:PlayAnimation("small_idle", true)
	inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
	inst.AnimState:SetLayer(LAYER_WORLD_BACKGROUND)
	inst.AnimState:SetSortOrder(4)
	inst.AnimState:SetMultColour(0.5, 0.5, 0.5, 1)
	
	inst.Transform:SetScale(0.5, 0.5, 0.5)

    MakeInventoryPhysics(inst)
    MakeBlowInHurricane(inst, TUNING.WINDBLOWN_SCALE_MIN.LIGHT, TUNING.WINDBLOWN_SCALE_MAX.LIGHT)
	MakeInventoryFloatable(inst, "small_idle", "small_idle")

	inst.components.floatable:SetOnHitWaterFn(OnHitWater)
	inst.components.floatable:SetOnHitLandFn(OnHitLand)
	
    inst:ListenForEvent("onputininventory", OnHitLand)	
	
    inst:AddTag("plant")
    inst:AddTag("lilypad_seed")
	
	inst:AddComponent("inspectable")

    inst:AddComponent("stackable")
    inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM
	
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.nobounce = true
	
    inst:AddComponent("waterproofer")
    inst.components.waterproofer:SetEffectiveness(0)
			
    inst:AddComponent("deployable")
    inst.components.deployable.ondeploy = OnDeploy
	inst.components.deployable.test = IsPointNearNonLilyPadPuddle
	inst.components.deployable.placer = "lilypad_placer"
	inst.components.deployable.min_spacing = 3
	inst.components.deployable.deploydistance = 2.75

	return inst
end

return Prefab("lilypad_for_puddle", fn, assets, prefabs),
    Prefab("lilypad_seed", fn_seed, assets, prefabs_seed),
    MakePlacer("lilypad_placer", "lily_pad", "lily_pad", "small_idle")
 