require("prefabutil")

local assets =
{
	Asset("ANIM", "anim/magician_chest.zip"),
	Asset("ANIM", "anim/ui_portal_shadow_3x4.zip"),
}

local prefabs =
{
	"collapse_small",
}

local function OnOpen(inst)
	if not inst:HasTag("burnt") then
		inst.AnimState:PlayAnimation("open")
		inst.AnimState:PushAnimation("loop")
	end
end

local function OnClose(inst)
	if not inst:HasTag("burnt") then
		inst.AnimState:PlayAnimation("close")
		inst.AnimState:PushAnimation("closed", false)
	end
end

local function TestItem_MagicChest(inst, item, slot)
	return not item.components.leader and not item:HasTag("magician_hat") and not item:HasTag("irreplaceable")
end

local function OnHammered(inst, worker)
	if inst.components.burnable ~= nil and inst.components.burnable:IsBurning() then
		inst.components.burnable:Extinguish()
	end
	inst.components.lootdropper:DropLoot()
	local fx = SpawnPrefab("collapse_small")
	fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")	
	inst:Remove()
end

local function OnHit(inst, worker)
	if not inst:HasTag("burnt") then
		if inst.components.container ~= nil then 
			inst.components.container:Close()
		end
		inst.AnimState:PlayAnimation("hit")
		inst.AnimState:PushAnimation("closed", false)
	end
end

local function OnBuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("closed", false)
	inst.SoundEmitter:PlaySound("dontstarve/common/chest_craft")
end

local function OnBurnt(inst)
    inst:AddTag("burnt")
    inst.components.burnable.canlight = false
    if inst.AnimState ~= nil then
        inst.AnimState:PlayAnimation("burnt", true)
    end
    inst:PushEvent("burntup")
    if inst.SoundEmitter ~= nil then
        inst.SoundEmitter:KillSound("idlesound")
        inst.SoundEmitter:KillSound("sound")
        inst.SoundEmitter:KillSound("loop")
        inst.SoundEmitter:KillSound("snd")
    end
    if inst.MiniMapEntity ~= nil then
        inst.MiniMapEntity:SetEnabled(false)
    end
    if inst.components.workable ~= nil then
        inst.components.workable:SetWorkLeft(1)
    end
    if inst.components.container ~= nil then
        inst.components.container:Close()
        inst:RemoveComponent("container")
    end
    if inst.components.burnable ~= nil then
        inst:RemoveComponent("burnable")
    end
    inst:RemoveTag("dragonflybait_lowprio")
    inst:RemoveTag("dragonflybait_medprio")
    inst:RemoveTag("dragonflybait_highprio")
end

local function OnSave(inst, data)
	if inst.components.burnable ~= nil and inst.components.burnable:IsBurning() or inst:HasTag("burnt") then
		data.burnt = true
	end
end

local function OnLoad(inst, data)
	if data ~= nil and data.burnt and inst.components.burnable ~= nil then
		inst.components.burnable.onburnt(inst)
	end
end

local magicchest_slotpos = {}

for y = 2.5, -0.5, -1 do
	for x = 0, 2 do
		table.insert(magicchest_slotpos, Vector3(75*x-75*2+75, 75*y-75*2+75, 0))
	end
end

local function fn()
	local inst = CreateEntity()

	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddMiniMapEntity()

	inst:AddTag("spoiler")
	inst:AddTag("main_magic_chest")
	inst:AddTag("magic_container")
	inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")

	inst.AnimState:SetBank("magician_chest")
	inst.AnimState:SetBuild("magician_chest")
	inst.AnimState:PlayAnimation("closed")
	
    inst:AddComponent("container")
	inst.components.container:SetNumSlots(#magicchest_slotpos, true)
	inst.components.container.onopenfn = OnOpen
	inst.components.container.onclosefn = OnClose
	inst.components.container.itemtestfn = TestItem_MagicChest
	inst.components.container.widgetslotpos = magicchest_slotpos
	inst.components.container.widgetpos = Vector3(75, 200, 0)
	inst.components.container.side_align_tip = 160
	inst.components.container.widgetanimbank = "ui_portal_shadow_3x4"
	inst.components.container.widgetanimbuild = "ui_portal_shadow_3x4"	
	
	inst:ListenForEvent("onremove", function() assert(false,"MAIN MAGIC CHEST WAS REMOVED SOMEHOW") end)

	inst:RemoveFromScene()
	
	return inst
end

return Prefab("magic_chest", fn, assets, prefabs)

