--MayBee from Plants vs. Zombies 2.

local maybee_level_data = require("maybee_level_data")

local maybee_assets =
{ --Anim from DST, I cannot make MayBee anim, so use this barnacle plant.
    Asset("ANIM", "anim/barnacle_plant.zip"),
    Asset("ANIM", "anim/barnacle_plant_colour_swaps.zip"),
	Asset("SOUND", "sound/bee.fsb"),
}

local maybee_projectile_assets = 
{ 
	Asset("ANIM", "anim/bee.zip"),
	Asset("ANIM", "anim/bee_build.zip"),
	Asset("ANIM", "anim/bee_angry_build.zip"),
	Asset("SOUND", "sound/bee.fsb"),
}

local prefabs = 
{

}

require("prefabutil")

local brain = require("brains/maybeebrain")
local bee_brain = require("brains/maybeebeebrain")

local function MayBeeOnRefuse(inst, giver, item)
    if giver ~= nil and giver.components.talker ~= nil then
	    if TUNING.LANGUAGE == true then
		    giver.components.talker:Say("It is too busy, or giving wrong item.")
        elseif TUNING.LANGUAGE == false then
		    giver.components.talker:Say("它太忙了，或是给了错误的物品。")
	    end	
	end
end

local function MayBeeOnAccept(inst, giver, item)
    if item ~= nil then
	    item:Remove()
	end
    if inst.components.combat and inst.components.combat.target and inst.components.combat.target == giver then
	    inst.components.combat:SetTarget(nil)
	end
	inst.sg:GoToState("plantfood_attack")
end

local function MayBeeAcceptTest(inst, item, giver)
    if inst.sg ~= nil 
	    and (inst.sg:HasStateTag("busy") or inst.sg:HasStateTag("attack")) 
	then
	    return false
	end
	return item ~= nil and item:HasTag("plantfood")
end

local function SpringMod(amt)
    if not IsDLCEnabled(REIGN_OF_GIANTS) and not IsDLCEnabled(CAPY_DLC) and not IsDLCEnabled(PORKLAND_DLC) then
	    return amt
	end
    if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)) and GetSeasonManager() and GetSeasonManager():IsGreenSeason() then
	    return amt * TUNING.SPRING_COMBAT_MOD
	end
	
    if IsDLCEnabled(PORKLAND_DLC) and GetSeasonManager() and GetSeasonManager():IsLushSeason() then
	    return amt * TUNING.SPRING_COMBAT_MOD
	end
	
    if GetSeasonManager() and GetSeasonManager():IsSpring() then
        return amt * TUNING.SPRING_COMBAT_MOD
    end
	
	return amt
end

local function MayBeeRetargetFn(inst)
    return FindEntity(inst, SpringMod(inst.BEE_TRIGGER_RADIUS or 10), function(guy)
        return inst.components.combat:CanTarget(guy)
		    and (guy.components.follower == nil or (guy.components.follower.leader == nil or guy.components.follower.leader ~= GetPlayer()))
    end, nil, {"NOCLICK", "FX", "INLIMBO", "player", "shadow", "notarget", "maybee", "maybeebee", "companion"}, {"monster", "hostile", "killer", "animal", "smallcreature", "character", "seacreature", "prey"})
end

local function MayBeeBeeRetarget(inst)
    return FindEntity(inst, SpringMod(15), function(guy)
        return inst.persists
		    and inst.components.combat:CanTarget(guy)
		    and (guy.components.follower == nil or (guy.components.follower.leader == nil or guy.components.follower.leader ~= GetPlayer()))
    end, nil, {"NOCLICK", "FX", "INLIMBO", "player", "shadow", "notarget", "maybee", "maybeebee", "companion"}, {"monster", "hostile", "killer", "animal", "smallcreature", "character", "seacreature", "prey"})
end

local function MayBeeKeepTarget(inst, target)
    return inst.persists and inst.components.combat:CanTarget(target)
end

local function on_attacked(inst, data)
    if data ~= nil and data.attacker ~= nil then
	    if inst.sg == nil or not inst.sg:HasStateTag("plantfoodattack") then
            inst.components.combat:SetTarget(data.attacker)
		end
		if data.attacker.components.freezable ~= nil then
		    data.attacker.components.freezable:Freeze(inst.STUN_DURATION or 0.5)
		elseif data.attacker.components.combat ~= nil then
		    data.attacker.components.combat:SetTarget(nil)
		end
	end
end

local function SetPlantFoodDamage(inst, v)
    if v ~= nil then
	    inst.PlantFoodDamage = v
	end
end

local function DoAttack(inst, target)
    target = target or inst.components.combat.target
	local bee = SpawnPrefab(inst.plant_food_attack and "maybee_plantfood_projectile" or "maybee_projectile")
	bee.Transform:SetPosition(inst.Transform:GetWorldPosition())
	bee.components.combat:SetTarget(target)
	if not inst.plant_food_attack then
	    bee.components.combat:SetDefaultDamage(inst.components.combat.defaultdamage)
		bee.stun_chance = inst.BEE_STUN_CHANCE
	elseif inst.PlantFoodDamage ~= nil then
	    bee.components.combat:SetDefaultDamage(inst.PlantFoodDamage)
	end
	bee.Duration = inst.BEE_DURATION or 10
	bee.stun_time = inst.BEE_STUN_DURATION or 1
end

local function LevelDataAdditionalUpdateFn(inst)
    if inst.components.charlierefreshmodlevel.level_data ~= nil then
	    local level_data = inst.components.charlierefreshmodlevel.level_data
		local PREFIX = string.upper(inst.prefab)
		local current_level = inst.components.charlierefreshmodlevel:GetLevel()
		if level_data[PREFIX.."_LEVEL_DATA"] ~= nil then
		    if level_data[PREFIX.."_LEVEL_DATA"]["BEE_TRIGGER_RADIUS"] ~= nil then
			    inst.BEE_TRIGGER_RADIUS = level_data[PREFIX.."_LEVEL_DATA"]["BEE_TRIGGER_RADIUS"][current_level]	
			end	
		    if level_data[PREFIX.."_LEVEL_DATA"]["BEE_DURATION"] ~= nil then
			    inst.BEE_DURATION = level_data[PREFIX.."_LEVEL_DATA"]["BEE_DURATION"][current_level]	
			end				
		    if level_data[PREFIX.."_LEVEL_DATA"]["BEE_COUNT"] ~= nil then
			    inst.BeeCount = level_data[PREFIX.."_LEVEL_DATA"]["BEE_COUNT"][current_level]
			end		
		    if level_data[PREFIX.."_LEVEL_DATA"]["BEE_COUNT_PF"] ~= nil then
			    inst.PlantfoodBeeCount = level_data[PREFIX.."_LEVEL_DATA"]["BEE_COUNT_PF"][current_level]
			end				
		    if level_data[PREFIX.."_LEVEL_DATA"]["BEE_STUN_CHANCE"] ~= nil then
			    inst.BEE_STUN_CHANCE = level_data[PREFIX.."_LEVEL_DATA"]["BEE_STUN_CHANCE"][current_level]
			end	
		    if level_data[PREFIX.."_LEVEL_DATA"]["STUN_DURATION"] ~= nil then
			    inst.STUN_DURATION = level_data[PREFIX.."_LEVEL_DATA"]["STUN_DURATION"][current_level]
			end			
		    if level_data[PREFIX.."_LEVEL_DATA"]["BEE_STUN_DURATION"] ~= nil then
			    inst.BEE_STUN_DURATION = level_data[PREFIX.."_LEVEL_DATA"]["BEE_STUN_DURATION"][current_level]
			end	
		    if level_data[PREFIX.."_LEVEL_DATA"]["ATTACK_RECHARGE_TIME"] ~= nil then
			    inst.components.combat:SetAttackPeriod(level_data[PREFIX.."_LEVEL_DATA"]["ATTACK_RECHARGE_TIME"][current_level])
				inst.components.combat:SetRetargetFunction(level_data[PREFIX.."_LEVEL_DATA"]["ATTACK_RECHARGE_TIME"][current_level], MayBeeRetargetFn)
			end				
		end
	end
end

local function maybee_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	inst.entity:AddMiniMapEntity()
	
	MakeObstaclePhysics(inst, 0.5)

    inst.Transform:SetSixFaced()

    inst:AddTag("maybee")
	inst:AddTag("veggie")
	inst:AddTag("plant_mob")
	inst:AddTag("noauradamage")
	inst:AddTag("groundpoundimmune")

    inst.AnimState:SetBank("barnacle_plant")
    inst.AnimState:SetBuild("barnacle_plant_colour_swaps")
    inst.AnimState:PlayAnimation("idle")
    inst.AnimState:OverrideSymbol("bc_bud", "barnacle_plant_colour_swaps", "white_bc_bud")
    inst.AnimState:OverrideSymbol("bc_face", "barnacle_plant_colour_swaps", "white_bc_face")
    inst.AnimState:OverrideSymbol("bc_flower_petal", "barnacle_plant_colour_swaps", "white_bc_flower_petal")
    inst.AnimState:Hide("bud1")
    inst.AnimState:Hide("bud2")
    inst.AnimState:Hide("bud3")
	
	inst.MiniMapEntity:SetIcon("maybee.tex")

    inst:AddComponent("inspectable")

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(300)

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(20)
    inst.components.combat:SetAttackPeriod(18)
    inst.components.combat:SetRange(10)
    inst.components.combat:SetRetargetFunction(18, MayBeeRetargetFn)
    inst.components.combat:SetKeepTargetFunction(MayBeeKeepTarget)
	
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({})

	inst.SetPlantFoodDamage = SetPlantFoodDamage
	
	inst:AddComponent("trader")
	inst.components.trader.acceptnontradable = true
	inst.components.trader.onrefuse = MayBeeOnRefuse
	inst.components.trader.onaccept = MayBeeOnAccept
    inst.components.trader:SetAcceptTest(MayBeeAcceptTest)	
	
	inst:AddComponent("charlierefreshmodlevel")
	inst.components.charlierefreshmodlevel:SetLevelData(maybee_level_data)
	inst.components.charlierefreshmodlevel:SetAdditionalUpdateFn(LevelDataAdditionalUpdateFn)
	inst.components.charlierefreshmodlevel:EnableMastery(true)
	inst.components.charlierefreshmodlevel:SetRequiredConsumables("seedpacket")	
	
	MakeMediumBurnableCharacter(inst, "body")
	
	inst:ListenForEvent("attacked", on_attacked)
	
	inst.BeeCount = 2
	inst.PlantfoodBeeCount = 4
	inst.BEE_TRIGGER_RADIUS = 10
	inst.BEE_DURATION = 10
	inst.BEE_STUN_CHANCE = 0.25
	inst.STUN_DURATION = 0.5
	inst.BEE_STUN_DURATION = 1
	inst.DoAttack = DoAttack

    inst:SetStateGraph("SGmaybee")
    inst:SetBrain(brain)

    return inst
end

local function maybee_projectile_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	inst.entity:AddDynamicShadow()
	
	inst.DynamicShadow:SetSize(0.8, 0.5)
	
	inst.Transform:SetFourFaced()
	
	MakeInventoryPhysics(inst)
	RemovePhysicsColliders(inst)
	
    inst.AnimState:SetBank("bee")
    inst.AnimState:SetBuild("bee_build")
    inst.AnimState:PlayAnimation("idle", true)

    inst:AddTag("NOCLICK")
    inst:AddTag("notarget")
	inst:AddTag("maybeebee")
	
	inst.Duration = 10

	inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
	inst.components.locomotor:EnableGroundSpeedMultiplier(false)
	inst.components.locomotor:SetTriggersCreep(false)
	
    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(20)
	inst.components.combat:SetAttackPeriod(0.5)
    inst.components.combat:SetRange(0.75)
	inst.components.combat:SetRetargetFunction(0.5, MayBeeBeeRetarget)
    inst.components.combat:SetKeepTargetFunction(MayBeeKeepTarget)	
	
	inst:AddComponent("maybeebeespecific")

	inst:SetStateGraph("SGmaybeebee")
	inst:SetBrain(bee_brain)
	
    return inst	
end

local function maybee_plantfood_projectile_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	inst.entity:AddDynamicShadow()
	
	inst.DynamicShadow:SetSize(0.8, 0.5)
	
	inst.Transform:SetFourFaced()
	
	MakeInventoryPhysics(inst)
	RemovePhysicsColliders(inst)
	
    inst.AnimState:SetBank("bee")
    inst.AnimState:SetBuild("bee_angry_build")
    inst.AnimState:PlayAnimation("idle", true)

    inst:AddTag("NOCLICK")
    inst:AddTag("notarget")
	inst:AddTag("maybeebee")
	
	inst.Duration = 10

	inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
	inst.components.locomotor:EnableGroundSpeedMultiplier(false)
	inst.components.locomotor:SetTriggersCreep(false)
	
    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(60)
	inst.components.combat:SetAttackPeriod(0.5)
    inst.components.combat:SetRange(0.9)
	inst.components.combat:SetRetargetFunction(0.5, MayBeeBeeRetarget)
    inst.components.combat:SetKeepTargetFunction(MayBeeKeepTarget)	
	
	inst:AddComponent("maybeebeespecific")
	
	inst:SetStateGraph("SGmaybeebee")
	inst:SetBrain(bee_brain)
	
    return inst	
end

return Prefab("maybee", maybee_fn, maybee_assets, prefabs),
    Prefab("maybee_projectile", maybee_projectile_fn, maybee_projectile_assets, prefabs),
	Prefab("maybee_plantfood_projectile", maybee_plantfood_projectile_fn, maybee_projectile_assets, prefabs)