local assets =
{
	Asset("ANIM", "anim/multitool_axe_pickaxe_pickaxeaxe.zip"),
	Asset("ANIM", "anim/swap_multitool_axe_pickaxe.zip"),
	Asset("ANIM", "anim/swap_multitool_axe_pickaxe_pickaxeaxe.zip"),
    Asset("INV_IMAGE", "multitool_axe_pickaxe"),
}

local function onequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_multitool_axe_pickaxe_pickaxeaxe", "swap_multitool_axe_pickaxe")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function onunequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

local function CanSpellOn(inst, doer, target, pos) 
    return target ~= nil and (target:HasTag("fallable") or target:HasTag("diggable"))
end

local DIG_MUST_TAGS = {"diggable"}
local DIG_CANT_TAGS = {"fire", "smolder", "invisible", "FX", "NOCLICK", "INLIMBO", "DECOR", "INTERIOR_LIMBO"}

local function OnSpell(inst, target, pos) 
    if target ~= nil then
	    if target:HasTag("fallable") and target.Fall ~= nil then
		    target:Fall(true)
		elseif target:HasTag("diggable") then
		    if target.components.workable ~= nil then
			    local destroyer = inst.components.inventoryitem.owner or GetPlayer()
				local x, y, z = target.Transform:GetWorldPosition()
		        target.components.workable:Destroy(destroyer)
				local ents = TheSim:FindEntities(x, y, z, 5, DIG_MUST_TAGS, DIG_CANT_TAGS)	
				for k,v in pairs(ents) do
				    if v ~= nil and v ~= target and v:IsValid() and v.entity:IsVisible() and v.components.workable ~= nil then
					    local _destroyer = inst.components.inventoryitem.owner or GetPlayer()
					    v.components.workable:Destroy(_destroyer)
					end
				end
		    end
		end
	end
	if inst.components.finiteuses ~= nil then
	    inst.components.finiteuses:Use()
	end
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
    
    anim:SetBank("multitool_axe_pickaxe")
    anim:SetBuild("multitool_axe_pickaxe_pickaxeaxe")
    anim:PlayAnimation("idle")
	
    if inst.GetIsOnWater ~= nil then
        MakeInventoryFloatable(inst, "NewAnimation", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
    end
 
    inst:AddTag("sharp")
    inst:AddTag("multifunctional_tool")
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(42.5)

    inst:AddComponent("tool")
    inst.components.tool:SetAction(ACTIONS.CHOP, 1.33)
    inst.components.tool:SetAction(ACTIONS.MINE, 1.33)
	--inst.components.tool:SetAction(ACTIONS.DIG, 1.33)
	inst.components.tool:SetAction(ACTIONS.HAMMER, 1.5)
	
    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetMaxUses(1600)
    inst.components.finiteuses:SetUses(1600)
    inst.components.finiteuses:SetOnFinished(inst.Remove)
    inst.components.finiteuses:SetConsumption(ACTIONS.CHOP, 1)
    inst.components.finiteuses:SetConsumption(ACTIONS.HAMMER, 2)
    inst.components.finiteuses:SetConsumption(ACTIONS.MINE, 3)
	--inst.components.finiteuses:SetConsumption(ACTIONS.DIG, 4)

    if ACTIONS.HACK ~= nil then
	    inst.components.tool:SetAction(ACTIONS.HACK, 1.33)
		inst.components.finiteuses:SetConsumption(ACTIONS.HACK, 1)
	end

    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
	
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip)   
    inst.components.equippable:SetOnUnequip(onunequip)
 
    inst:AddComponent("spellcaster")
    inst.components.spellcaster.canuseontargets = true
    inst.components.spellcaster.canusefrominventory = false
    inst.components.spellcaster:SetSpellTestFn(CanSpellOn)
    inst.components.spellcaster:SetSpellFn(OnSpell)
 
    inst.fxcolour = {1, 140/255, 0}
 
    return inst
end

return Prefab("multifunctional_tool", fn, assets)