local assets = 
{
	Asset("ANIM", "anim/orangestaff_rodofdiscord.zip"),
	Asset("ANIM", "anim/swap_orangestaff_rodofdiscord.zip"),
	Asset("ANIM", "anim/swap_staffs.zip"),
}

local prefabs = 
{
    "sand_puff_large_front",
    "sand_puff_large_back",
	"horrorfuel",
}

local function OnFinished(inst)
    inst.SoundEmitter:PlaySound("dontstarve/common/gem_shatter")
    inst:Remove()
end

local STOP_MOVING_MUST_TAGS = {"locomotor"}
local STOP_MOVING_CANT_TAGS = {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "DECOR", "invisible", "notarget", "boat", "wall", "player"}
local function OnBlink(inst, pt, caster)
    inst.bonus_damage = math.min(inst.bonus_damage + 5, inst.bonus_damage_max)
	if pt ~= nil then
        local ents_to_stop = TheSim:FindEntities(pt.x, 0, pt.z, 5, STOP_MOVING_MUST_TAGS, STOP_MOVING_CANT_TAGS)
		for i,ent in pairs(ents_to_stop) do
		    if ent ~= inst and ent ~= caster then
			    if ent:IsValid() and ent.entity:IsVisible() and ent.components.locomotor ~= nil then	
				    ent.components.locomotor:Stop()
                    inst.components.blinkstaff:SpawnEffect(ent)	
				end
			end
		end					
	end
	
	if inst.bonus_damage >= inst.bonus_damage_max and math.random() < 0.5 then
	    if caster ~= nil and caster:IsOnValidGround() then
	        local x, y, z = caster.Transform:GetWorldPosition()
			local prefab_to_spawn = SpawnPrefab("horrorfuel")
		    if prefab_to_spawn then
		        prefab_to_spawn.Transform:SetPosition(x, y, z)
			    SpawnPrefab("small_puff").Transform:SetPosition(x, y, z)
			    if prefab_to_spawn.GetIsOnWater ~= nil
			        and prefab_to_spawn:GetIsOnWater()
			    then
			        if prefab_to_spawn.components.floatable ~= nil then
				        prefab_to_spawn.components.floatable:OnHitWater(true)
				    elseif prefab_to_spawn.components.inventoryitem ~= nil then
			            prefab_to_spawn.components.inventoryitem:OnHitWater()
			        end			
			    end
		    end			
		end
	end

end

local function OnEquipShadowOrangeStaff(inst, owner)
    owner.AnimState:OverrideSymbol("swap_object", "swap_orangestaff_rodofdiscord", "swap_orangestaff")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function OnUnequipShadowOrangeStaff(inst, owner)
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal")
    inst.bonus_damage = 0	
end

local function ShadowOrangeStaffOnAttack(inst, attacker, target)
    local bonus_damage = inst.bonus_damage or 0
	local shadow_mult = 2
	if GetWorld():IsRuins() then
	    shadow_mult = 4
	elseif GetWorld():IsCave() then
	    shadow_mult = 3
	end
	if attacker ~= nil and attacker:HasTag("shadow_form_on") then
	    bonus_damage = bonus_damage * shadow_mult
	end
	
	if bonus_damage > 0 then

	    if target ~= nil 
	        and target:IsValid() 
	        and target.components.health ~= nil
	    then	
		    target.components.health:SetVal(target.components.health.currenthealth - bonus_damage, inst.prefab)	
		end
	
	end

end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function OnRepaired(inst, doer, repair_item)
	if doer ~= nil and doer.SoundEmitter ~= nil then
	    doer.SoundEmitter:PlaySound("dontstarve/HUD/repair_clothing")
	end
end

local function OnSave(inst, data)
    data.bonus_damage = inst.bonus_damage
end

local function OnLoad(inst, data)
    if data ~= nil then
	    inst.bonus_damage = data.bonus_damage or 0
	end
end

local function fn()
    local inst = CreateEntity()

    local SHADOWORANGESTAFF_SPEED_MULT = 1.5

	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        SHADOWORANGESTAFF_SPEED_MULT = 0.5
	end
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("orangestaff")
    inst.AnimState:SetBuild("orangestaff_rodofdiscord")
    inst.AnimState:PlayAnimation("idle")
	
	inst:AddTag("shadow_item")
    inst:AddTag("shadow")
    inst:AddTag("sharp")
	
    inst.fxcolour = {1, 145/255, 0}
	inst.castsound = "dontstarve/common/staffteleport"
	inst.bonus_damage = 0
	inst.bonus_damage_max = 432

    inst:AddComponent("inspectable")
	
    inst:AddComponent("tradable")
    
    inst:AddComponent("inventoryitem")

    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetOnFinished(OnFinished)	
    inst.components.finiteuses:SetMaxUses(1000)
    inst.components.finiteuses:SetUses(1000)
	
    inst:AddComponent("blinkstaff")
	inst.components.blinkstaff.onblinkfn = OnBlink

    inst:AddComponent("reticule")
    inst.components.reticule.targetfn = function() return inst.components.blinkstaff:GetBlinkPoint() end
    inst.components.reticule.ease = true
	
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(OnEquipShadowOrangeStaff)
    inst.components.equippable:SetOnUnequip(OnUnequipShadowOrangeStaff)
	inst.components.equippable.walkspeedmult = SHADOWORANGESTAFF_SPEED_MULT

    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(68)
	inst.components.weapon:SetOnAttack(ShadowOrangeStaffOnAttack)

    if inst.GetIsOnWater ~= nil then
        MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)   	
	end

    inst:AddComponent("shadowrepairable")
	inst.components.shadowrepairable:SetRepairMaterial("voidcloth")
	inst.components.shadowrepairable:SetOnRepaired(OnRepaired)	
	
	inst.OnSave = OnSave
	inst.OnLoad = OnLoad
	
	return inst
end

return Prefab("orangestaff_shadow", fn, assets, prefabs)