require("prefabutil")

local function CanPlantAtPoint(x, y, z)
    local tile = GetWorld().Map:GetTileAtPoint(x, y, z)
    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    if tile == GROUND.ASH or tile == GROUND.VOLCANO 
		    or tile == GROUND.VOLCANO_ROCK or tile == GROUND.BRICK_GLOW
			or tile == GROUND.VOLCANO_LAVA or tile == GROUND.SNAKESKIN
		then
		    return false
		end
	end
	if IsDLCEnabled(PORKLAND_DLC) then
	    if tile == GROUND.FOUNDATION or tile == GROUND.COBBLEROAD
		    or tile == GROUND.GASJUNGLE or tile == GROUND.PIGRUINS
			or tile == GROUND.PIGRUINS_NOCANOPY or tile == GROUND.BEARDRUG --or tile == GROUND.INTERIOR
		then                                                               --Use inst.GetIsOnLandOutside so that GROUND.INTERIOR can be not included
		    return false
		end
	end
    if tile ~= GROUND.IMPASSABLE and tile ~= GROUND.INVALID and tile ~= GROUND.ROCKY and tile ~= GROUND.ROAD
		and tile ~= GROUND.UNDERROCK and tile ~= GROUND.WOODFLOOR and tile ~= GROUND.CARPET and tile ~= GROUND.CHECKER
		and tile < GROUND.UNDERGROUND
	then
		return true
	end
	return false
end

local function deploytest_normal(inst, pt)
    if inst.GetIsOnLandOutside ~= nil then
	    if not inst:GetIsOnLandOutside(pt.x, pt.y, pt.z) then
		    return false
		end
	end
	if inst.IsPosSurroundedByLand ~= nil then
	    if not inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 2) then
		    return false
		end
	end
	if CanPlantAtPoint(pt.x, pt.y, pt.z) then
	    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 4, nil, {"NOBLOCK", "player", "FX"})
		local min_spacing = inst.components.deployable.min_spacing or 2
	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt ) < min_spacing*min_spacing then
					return false
				end
			end
		end
		return true
	end
	return false
end

local function deploytest_nobeach(inst, pt)
    if inst.GetIsOnLandOutside ~= nil then
	    if not inst:GetIsOnLandOutside(pt.x, pt.y, pt.z) then
		    return false
		end
	end
	if inst.IsPosSurroundedByLand ~= nil then
	    if not inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 2) then
		    return false
		end
	end
	local tile = GetWorld().Map:GetTileAtPoint(pt.x, pt.y, pt.z)
	if CanPlantAtPoint(pt.x, pt.y, pt.z) and (GROUND.BEACH == nil or tile ~= GROUND.BEACH) then
	    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 4, nil, {"NOBLOCK", "player", "FX"})
		local min_spacing = inst.components.deployable.min_spacing or 2
	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt ) < min_spacing*min_spacing then
					return false
				end
			end
		end
		return true
	end
	return false
end


local function make_plantable(data)
    local bank = data.bank or data.name
    local assets =
    {
        Asset("ANIM", "anim/"..bank..".zip"),
    }

    if data.build ~= nil then
        table.insert(assets, Asset("ANIM", "anim/"..data.build..".zip"))
    end
	
    local function OnHitWater(inst, size, vert_offset, x_scale, y_scale, z_scale)
	    if inst.float_fx == nil then
		    inst.float_fx = SpawnPrefab("float_fx_front")
		    inst.float_fx.entity:SetParent(inst.entity)
		    if size ~= nil then
		        inst.float_fx.AnimState:PlayAnimation("idle_front_"..size, true)
		    end
		    if vert_offset ~= nil then
		        inst.float_fx.Transform:SetPosition(0, vert_offset, 0)
		    end
		    if x_scale ~= nil then
                inst.float_fx.Transform:SetScale(x_scale, y_scale or x_scale, z_scale or x_scale)
		    end			
	    end
	    if inst.float_fx2 == nil then
		    inst.float_fx2 = SpawnPrefab("float_fx_back")
		    inst.float_fx2.entity:SetParent(inst.entity)
		    if size ~= nil then
		        inst.float_fx2.AnimState:PlayAnimation("idle_back_"..size, true)
		    end
		    if vert_offset ~= nil then
		        inst.float_fx2.Transform:SetPosition(0, vert_offset, 0)
		    end
            if x_scale ~= nil then
		        inst.float_fx2.Transform:SetScale(x_scale, y_scale or x_scale, z_scale or x_scale)
		    end				
	    end	
    end

    local function OnHitLand(inst)
	    if inst.float_fx ~= nil then
		    inst.float_fx:Remove()
		     inst.float_fx = nil
	    end
	    if inst.float_fx2 ~= nil then
		    inst.float_fx2:Remove()
		    inst.float_fx2 = nil
	    end	
    end

    local function ondeploy(inst, pt, deployer)
        local tree = SpawnPrefab(data.name)
        if tree ~= nil then
            tree.Transform:SetPosition(pt:Get())
            inst.components.stackable:Get():Remove()
            if tree.components.pickable ~= nil then
                tree.components.pickable:OnTransplant()
            end
            if deployer ~= nil and deployer.SoundEmitter ~= nil then
                deployer.SoundEmitter:PlaySound("dontstarve/common/plant")
            end
        end
    end

    local function fn()
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddAnimState()
        inst.entity:AddSoundEmitter()

        MakeInventoryPhysics(inst)

        inst:AddTag("plant")

        inst.AnimState:SetBank(data.bank or data.name)
        inst.AnimState:SetBuild(data.build or data.name)
        inst.AnimState:PlayAnimation("dropped")

        inst:AddComponent("stackable")
        inst.components.stackable.maxsize = TUNING.STACK_SIZE_LARGEITEM

        inst:AddComponent("inspectable")
        inst.components.inspectable.nameoverride = data.inspectoverride or ("dug_"..data.name)
		
        inst:AddComponent("inventoryitem")
		
        inst:AddComponent("fuel")
        inst.components.fuel.fuelvalue = TUNING.LARGE_FUEL

        MakeMediumBurnable(inst, TUNING.LARGE_BURNTIME)
        MakeSmallPropagator(inst)
		
		if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
		    inst.components.burnable:MakeDragonflyBait(3)
        end
		
        inst:AddComponent("deployable")
        inst.components.deployable.ondeploy = ondeploy
		inst.components.deployable.min_spacing = data.minspace or 2
		inst.components.deployable.product = data.name

        if data.name ~= "berrybush_juicy" then
		    inst.components.deployable.test = deploytest_normal
		else
		    inst.components.deployable.test = deploytest_nobeach
		end

	    inst:AddComponent("edible")
	    inst.components.edible.foodtype = "WOOD"
	    inst.components.edible.woodiness = 10
		
	    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	        inst:AddComponent("floatable")
		    inst.components.floatable:SetOnHitWaterFn(function(inst)
			    if data.floater ~= nil then
			        OnHitWater(inst, data.floater[1], data.floater[2], data.floater[3])
				elseif data.name == "berrybush_juicy" then
				    OnHitWater(inst, "large", 0.025, 0.65, 0.5, 0.65)
				else
				    OnHitWater(inst)
				end
			end)
		    inst.components.floatable:SetOnHitLandFn(OnHitLand)
            inst:ListenForEvent("onputininventory", OnHitLand)	
			inst:AddComponent("appeasement")
    	    inst.components.appeasement.appeasementvalue = TUNING.WRATH_SMALL
	    end
        ---------------------
        return inst
    end

    return Prefab("dug_"..data.name, fn, assets)
end

local additional_plantables =
{
    {
        name = "berrybush_juicy",
        anim = "dead",
        inspectoverride = "dug_berrybush",
    },
	{
	    name = "bananabush", 
		anim = "idle_big", --idle_dead
		minspace = 1,
		floater = {"med", 0.2, 0.95},
	},
	{
	    name = "monkeytail", 
		bank = "grass", 
		build = "reeds_monkeytails",
		minspace = 1,
		floater = {"med", 0.1, 0.92},  
	},
}

local prefabs = {}
for k,v in pairs(additional_plantables) do
	table.insert(prefabs, make_plantable(v))
	table.insert(prefabs, MakePlacer( "dug_"..v.name.."_placer", v.bank or v.name, v.build or v.name, v.anim or "idle" ))
end

return unpack(prefabs) 