local PopupDialogScreen = require "screens/popupdialog"

local assets =
{
    Asset("ANIM", "anim/save_reseter.zip"),
}

local assets_save_saver =
{
    Asset("ANIM", "anim/save_saver.zip"),
}

local function CanInteract(inst)
    return not (inst.components.useableitem and inst.components.useableitem.inuse)
end

local function save_reseter_on_use(inst)
    SetPause(true)

    local function cancel()
        TheFrontEnd:PopScreen()
        SetPause(false)
		
		inst.components.useableitem.inuse = false
    end

    local function RollbackToLastSave()
        SetPause(false)
        TheFrontEnd:PopScreen()

        TheFrontEnd:HideConsoleLog()
        TheSim:SetDebugRenderEnabled(false)

        if not GetPlayer() then
            TheFrontEnd:Fade(false, 1, function()
                EnableAllDLC()
                StartNextInstance()
            end)
        end

        GetPlayer().HUD:Hide()

        TheFrontEnd:Fade(false, 1, function()
        StartNextInstance(
                {
                    reset_action = RESET_ACTION.LOAD_SLOT,
                    save_slot = SaveGameIndex:GetCurrentSaveSlot()
                },
            true
            )
        end)
		
		inst.components.useableitem.inuse = false
    end

    TheFrontEnd:PushScreen(PopupDialogScreen(
        STRINGS.RESET_SAVE_TITLE, STRINGS.RESET_SAVE_BODY,
        {
            {text = STRINGS.RESET_SAVE_YES, cb = RollbackToLastSave},
            {text = STRINGS.RESET_SAVE_NO, cb = cancel}
        }
    ))
end

local function save_saver_on_use(inst)
    SetPause(true)

    local function cancel()
        TheFrontEnd:PopScreen()
        SetPause(false)
		
		inst.components.useableitem.inuse = false
    end

    local function SaveTheSave()
        SetPause(false)
        TheFrontEnd:PopScreen()

        if GetPlayer() 
		    and GetPlayer().components.autosaver
		then
            GetPlayer().components.autosaver:DoSave()
		end
		
		inst.components.useableitem.inuse = false
    end

    TheFrontEnd:PushScreen(PopupDialogScreen(
        STRINGS.SAVE_SAVE_TITLE, STRINGS.SAVE_SAVE_BODY,
        {
            {text = STRINGS.RESET_SAVE_YES, cb = SaveTheSave},
            {text = STRINGS.RESET_SAVE_NO, cb = cancel}
        }
    ))

end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx.Transform:SetScale(1.25, 1.25, 1.25)
		inst.float_fx.Transform:SetPosition(0, -0.5, 0)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetScale(1.25, 1.25, 1.25)
		inst.float_fx2.Transform:SetPosition(0, -0.5, 0) 
	end	
end

local function OnHitWaterSaveReseter(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)
		inst.float_fx.Transform:SetPosition(0, -0.3, 0)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)
		inst.float_fx2.Transform:SetPosition(0, -0.3, 0)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function fn(Sim)
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddMiniMapEntity()

    inst.MiniMapEntity:SetIcon("save_reseter.tex")

    inst:AddTag("nosteal")

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("save_reseter")
    inst.AnimState:SetBuild("save_reseter")
    inst.AnimState:PlayAnimation("idle")

    inst:AddComponent("inspectable")

    inst:AddComponent("inventoryitem")

	inst:AddComponent("useableitem")
	inst.components.useableitem:SetCanInteractFn(CanInteract)
	inst.components.useableitem:SetOnUseFn(save_reseter_on_use)
	
    if inst.GetIsOnWater ~= nil then
        inst:AddComponent("floatable")
	    inst.components.floatable:SetOnHitWaterFn(OnHitWaterSaveReseter)
    	inst.components.floatable:SetOnHitLandFn(OnHitLand)
	    inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
    end	

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end 

    return inst
end

local function fn_save_saver(Sim)
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddMiniMapEntity()

    inst.MiniMapEntity:SetIcon("save_saver.tex")

    inst:AddTag("nosteal")

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("save_saver")
    inst.AnimState:SetBuild("save_saver")
    inst.AnimState:PlayAnimation("idle")
	
	inst.Transform:SetScale(0.6, 0.6, 0.6)

    inst:AddComponent("inspectable")

    inst:AddComponent("inventoryitem")
	
	inst:AddComponent("useableitem")
	inst.components.useableitem:SetCanInteractFn(CanInteract)
	inst.components.useableitem:SetOnUseFn(save_saver_on_use)

    if inst.GetIsOnWater ~= nil then
        inst:AddComponent("floatable")
	    inst.components.floatable:SetOnHitWaterFn(OnHitWater)
    	inst.components.floatable:SetOnHitLandFn(OnHitLand)
	    inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
    end	

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end 

    return inst
end

return Prefab("save_reseter", fn, assets),
    Prefab("save_saver", fn_save_saver, assets_save_saver)