--SeaFlora from Plants vs. Zombies 2.

local seaflora_level_data = require("seaflora_level_data")

local seaflora_assets =
{ --Anim from DST, I cannot make SeaFlora anim, so use this barnacle plant.
    Asset("ANIM", "anim/barnacle_plant.zip"),
    Asset("ANIM", "anim/barnacle_plant_colour_swaps.zip"),
}

local seaflora_projectile_assets = 
{ 
	Asset("ANIM", "anim/hermit_pearl.zip"),
}

local prefabs = 
{

}

require("prefabutil")

local brain = require("brains/seaflorabrain")

local function SeaFloraOnRefuse(inst, giver, item)
    if giver ~= nil and giver.components.talker ~= nil then
	    if TUNING.LANGUAGE == true then
		    giver.components.talker:Say("It is too busy, or giving wrong item.")
        elseif TUNING.LANGUAGE == false then
		    giver.components.talker:Say("它太忙了，或是给了错误的物品。")
	    end	
	end
end

local function SeaFloraOnAccept(inst, giver, item)
    if item ~= nil then
	    item:Remove()
	end
    if inst.components.combat and inst.components.combat.target and inst.components.combat.target == giver then
	    inst.components.combat:SetTarget(nil)
	end
	inst.sg:GoToState("plantfood_attack")
end

local function SeaFloraAcceptTest(inst, item, giver)
    if inst.sg ~= nil 
	    and (inst.sg:HasStateTag("busy") or inst.sg:HasStateTag("attack")) 
	then
	    return false
	end
	return item ~= nil and item:HasTag("plantfood")
end

local function SpringMod(amt)
    if not IsDLCEnabled(REIGN_OF_GIANTS) and not IsDLCEnabled(CAPY_DLC) and not IsDLCEnabled(PORKLAND_DLC) then
	    return amt
	end
    if (IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)) and GetSeasonManager() and GetSeasonManager():IsGreenSeason() then
	    return amt * TUNING.SPRING_COMBAT_MOD
	end
	
    if IsDLCEnabled(PORKLAND_DLC) and GetSeasonManager() and GetSeasonManager():IsLushSeason() then
	    return amt * TUNING.SPRING_COMBAT_MOD
	end
	
    if GetSeasonManager() and GetSeasonManager():IsSpring() then
        return amt * TUNING.SPRING_COMBAT_MOD
    end
	
	return amt
end

local function SeaFloraRetargetFn(inst)
    return FindEntity(inst, SpringMod(20), function(guy)
        return inst.components.combat:CanTarget(guy)
    end, nil, {"NOCLICK", "FX", "INLIMBO", "player", "shadow", "notarget", "seaflora"}, {"monster", "hostile", "killer"})
end

local function SeaFloraKeepTarget(inst, target)
    return inst.components.combat:CanTarget(target)
end

local function SeaFloraWaterChange(inst, onwater)
    if onwater then
	    inst.IsOnWater = true
	
	    inst.components.combat:SetDefaultDamage(65)
 
        if not inst:HasChildPrefab("float_fx_front") and not inst:HasChildPrefab("float_fx_back") then
            local ripple_front = SpawnPrefab("float_fx_front")
	        local ripple_back = SpawnPrefab("float_fx_back")
    
            inst:AddChild(ripple_front)
	        ripple_front.AnimState:PlayAnimation("idle_front_med", true)
			ripple_front.Transform:SetPosition(0, 0.1, 0)
			ripple_front.Transform:SetScale(1.1, 0.9, 1.1)
	
	        inst:AddChild(ripple_back)
	        ripple_back.AnimState:PlayAnimation("idle_back_med", true)
			ripple_back.Transform:SetPosition(0, 0.1, 0)
			ripple_back.Transform:SetScale(1.1, 0.9, 1.1)
        end

    else
	    inst.IsOnWater = false
		
	    inst.components.combat:SetDefaultDamage(50)
		
        if inst.children then
            for k,child in pairs(inst.children) do
                if k.prefab == "float_fx_front" or k.prefab == "float_fx_back" then
                    k:Remove()
                end
            end
        end	
		
	end
end

local function on_attacked(inst, data)
    if data ~= nil 
	    and data.attacker ~= nil 
		and (inst.sg == nil or not inst.sg:HasStateTag("plantfoodattack"))
	then
        inst.components.combat:SetTarget(data.attacker)
	end
end

local function SetNormalDamage(inst, v)
    if v ~= nil then
	    inst.NormalDamage = v
	end
end

local function SetNormalWaterDamage(inst, v)
    if v ~= nil then
	    inst.NormalWaterDamage = v
	end
end

local function SetPlantFoodDamage(inst, v)
    if v ~= nil then
	    inst.PlantFoodDamage = v
	end
end

local function SetPlantFoodWaterDamage(inst, v)
    if v ~= nil then
	    inst.PlantFoodWaterDamage = v
	end
end

local function SetPlantFoodExplosionDamage(inst, v)
    if v ~= nil then
	    inst.PlantFoodExplosionDamage = v
	end
end

local function SetPlantFoodWaterExplosionDamage(inst, v)
    if v ~= nil then
	    inst.PlantFoodWaterExplosionDamage = v
	end
end

local function on_remove(inst, data)
    if inst.components.inventory and inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
	    inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS):Remove()
	end
end

local function equip_ranged_weapon(inst)
    if inst.components.inventory and not inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
        local ranged_weapon = CreateEntity()
        ranged_weapon.persists = false

        ranged_weapon.entity:AddTransform()

        ranged_weapon:AddComponent("weapon")
        ranged_weapon.components.weapon:SetDamage(inst.components.combat.defaultdamage)
        ranged_weapon.components.weapon:SetRange(inst.components.combat.attackrange, inst.components.combat.attackrange + 4)
        ranged_weapon.components.weapon:SetProjectile("seaflora_projectile")

        ranged_weapon:AddComponent("inventoryitem")
        ranged_weapon.components.inventoryitem:SetOnDroppedFn(ranged_weapon.Remove)

        ranged_weapon:AddComponent("equippable")
        ranged_weapon:AddTag("nosteal")
		ranged_weapon:AddTag("cantdrop")
		ranged_weapon:AddTag("projectile")
        inst.components.inventory:Equip(ranged_weapon)
    end
end

local function OnEntityWake(inst)
	inst.components.tiletracker:Start()
end

local function OnEntitySleep(inst)
	inst.components.tiletracker:Stop()
end

local function LevelDataAdditionalUpdateFn(inst)
    if inst.components.charlierefreshmodlevel.level_data ~= nil then
	    local level_data = inst.components.charlierefreshmodlevel.level_data
		local PREFIX = string.upper(inst.prefab)
		local current_level = inst.components.charlierefreshmodlevel:GetLevel()
		if level_data[PREFIX.."_LEVEL_DATA"] ~= nil then
		    if level_data[PREFIX.."_LEVEL_DATA"]["PIERCING_VALUE"] ~= nil then
			    inst.PiercingValue = level_data[PREFIX.."_LEVEL_DATA"]["PIERCING_VALUE"][current_level]	
			end	
		    if level_data[PREFIX.."_LEVEL_DATA"]["BUBBLE_TRAVEL_DISTANCE"] ~= nil then
			    inst.BUBBLE_TRAVEL_DISTANCE = level_data[PREFIX.."_LEVEL_DATA"]["BUBBLE_TRAVEL_DISTANCE"][current_level]	
			end				
		    if level_data[PREFIX.."_LEVEL_DATA"]["DAMAGE_WATER"] ~= nil then
			    inst:SetNormalWaterDamage(level_data[PREFIX.."_LEVEL_DATA"]["DAMAGE_WATER"][current_level]) 
			end		
		    if level_data[PREFIX.."_LEVEL_DATA"]["DAMAGE_WATER_PF"] ~= nil then
			    inst:SetPlantFoodWaterDamage(level_data[PREFIX.."_LEVEL_DATA"]["DAMAGE_WATER_PF"][current_level])
			end				
		    if level_data[PREFIX.."_LEVEL_DATA"]["AREA_EFFECT_DAMAGE_PF"] ~= nil then
			    inst:SetPlantFoodExplosionDamage(level_data[PREFIX.."_LEVEL_DATA"]["AREA_EFFECT_DAMAGE_PF"][current_level])
			end	
		    if level_data[PREFIX.."_LEVEL_DATA"]["AREA_EFFECT_DAMAGE_PF_WATER"] ~= nil then
			    inst:SetPlantFoodWaterExplosionDamage(level_data[PREFIX.."_LEVEL_DATA"]["AREA_EFFECT_DAMAGE_PF_WATER"][current_level])
			end				
		end
	end
end

local function seaflora_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	inst.entity:AddMiniMapEntity()
	
	MakeObstaclePhysics(inst, 0.5)

    inst.Transform:SetSixFaced()

    inst:AddTag("seaflora")
	inst:AddTag("veggie")
	inst:AddTag("plant_mob")
	inst:AddTag("noauradamage")
	inst:AddTag("groundpoundimmune")

    inst.AnimState:SetBank("barnacle_plant")
    inst.AnimState:SetBuild("barnacle_plant_colour_swaps")
    inst.AnimState:PlayAnimation("idle")
    inst.AnimState:Hide("bud1")
    inst.AnimState:Hide("bud2")
    inst.AnimState:Hide("bud3")
	
	inst.MiniMapEntity:SetIcon("seaflora.tex")

    inst:AddComponent("inspectable")

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(325)

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(50)
    inst.components.combat:SetAttackPeriod(4)
    inst.components.combat:SetRange(20)
    inst.components.combat:SetRetargetFunction(4, SeaFloraRetargetFn)
    inst.components.combat:SetKeepTargetFunction(SeaFloraKeepTarget)
	
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({})
	
	inst:AddComponent("inventory")

	inst.SetNormalDamage = SetNormalDamage
	inst.SetNormalWaterDamage = SetNormalWaterDamage	
	inst.SetPlantFoodDamage = SetPlantFoodDamage
	inst.SetPlantFoodWaterDamage = SetPlantFoodWaterDamage
	inst.SetPlantFoodExplosionDamage = SetPlantFoodExplosionDamage
	inst.SetPlantFoodWaterExplosionDamage = SetPlantFoodWaterExplosionDamage
	
	inst.PiercingValue = 5
	inst.BUBBLE_TRAVEL_DISTANCE = 1
	
	inst:AddComponent("charlierefreshmodlevel")
	inst.components.charlierefreshmodlevel:SetLevelData(seaflora_level_data)
	inst.components.charlierefreshmodlevel:SetAdditionalUpdateFn(LevelDataAdditionalUpdateFn)
	inst.components.charlierefreshmodlevel:EnableMastery(true)
	inst.components.charlierefreshmodlevel:SetRequiredConsumables("seedpacket")
	
	inst:AddComponent("trader")
	inst.components.trader.acceptnontradable = true
	inst.components.trader.onrefuse = SeaFloraOnRefuse
	inst.components.trader.onaccept = SeaFloraOnAccept
    inst.components.trader:SetAcceptTest(SeaFloraAcceptTest)	
	
	MakeMediumBurnableCharacter(inst, "body")
	
	inst:ListenForEvent("attacked", on_attacked)
	inst:ListenForEvent("onremove", on_remove)
	
	inst:DoTaskInTime(math.random()*2*FRAMES, equip_ranged_weapon)
	
    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("tiletracker")
	    inst.components.tiletracker:SetOnWaterChangeFn(SeaFloraWaterChange)
		
	    inst.OnEntityWake = OnEntityWake
	    inst.OnEntitySleep = OnEntitySleep
    end
	
    inst:SetStateGraph("SGseaflora")
    inst:SetBrain(brain)

    return inst
end

local function projectile_keeptarget(inst)
    return false
end

local function OverrideHit(self, target)
    local attacker = self.owner
    local weapon = self.inst
    if not attacker.components.combat and attacker.components.weapon and attacker.components.inventoryitem then
        weapon = attacker
        attacker = weapon.components.inventoryitem.owner
    end

    if attacker 
	    and attacker.components.combat 
		and target ~= nil
	    and self.inst.AlreadyHitEnts ~= nil
		and not self.inst.AlreadyHitEnts[target.GUID]
	then
        attacker.components.combat:DoAttack(target, weapon, self.inst)
		self.inst.AlreadyHitEnts[target.GUID] = true
    end	
	
    if self.onhit then
        self.onhit(self.inst, attacker, target, weapon)
    end
end

local function OnHit(inst, attacker, target)

	if target and not table.contains(inst.AlreadyHitEntsNum, target.GUID) then
        table.insert(inst.AlreadyHitEntsNum, target.GUID)
	end
	inst.components.projectile:SetSpeed(1)
	
	local x1, y1, z1 = inst.Transform:GetWorldPosition()
	local entities = TheSim:FindEntities(x1, 0, z1, 10, {"hascombatcomponent"}, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "shadow", "notarget", "seaflora"})
    for i, v in pairs(entities) do
        if v ~= nil and v ~= target and v:IsValid() and v.entity:IsVisible() and inst.components.combat:CanTarget(v) and v.components.combat ~= nil then
			if v:HasTag("player") then
				if attacker ~= nil 
					and attacker.components.combat ~= nil 
				    and attacker.components.combat.target ~= nil
					and attacker.components.combat.target == v 	
				then
				    inst.components.projectile.target = v
					break
				end
			else
			    inst.components.projectile.target = v 
				break
			end
        end
    end	

	if #inst.AlreadyHitEntsNum > ((attacker.PiercingValue or 5) - 1) then
        if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
		    local x2, y2, z2 = inst.Transform:GetWorldPosition()
            local fx = SpawnPrefab("splash_water")
            fx.Transform:SetPosition(x2, 0, z2)
            if inst.SoundEmitter then
                inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/item_float")
            end			
        end
	    inst:Remove()
	end
end

local function OnHitPF(inst, attacker, target)

	local x1, y1, z1 = inst.Transform:GetWorldPosition()
	local entities = TheSim:FindEntities(x1, 0, z1, 10, {"hascombatcomponent"}, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "shadow", "notarget", "seaflora"})
    for i, v in pairs(entities) do
        if v ~= nil and v ~= target and v:IsValid() and v.entity:IsVisible() and inst.components.combat:CanTarget(v) and v.components.combat ~= nil then
			if v:HasTag("player") then
				if attacker ~= nil 
					and attacker.components.combat ~= nil 
				    and attacker.components.combat.target ~= nil
					and attacker.components.combat.target == v 	
				then
				    inst.components.projectile.target = v
					break
				end
			else
			    inst.components.projectile.target = v 
				break
			end
        end
    end	
		
	if target then
        if target.components.freezable ~= nil then
	        target.components.freezable:Freeze(1)
	   	end
        if target.Transform ~= nil then
		    local _x, _y, _z = inst.Transform:GetWorldPosition()
            target.Transform:SetPosition(_x, 0, _z)
	   	end
	end
	
    inst:DoTaskInTime(attacker ~= nil and attacker.BUBBLE_TRAVEL_DISTANCE ~= nil and attacker.BUBBLE_TRAVEL_DISTANCE or 1, function(inst)
	    local x2, y2, z2 = inst.Transform:GetWorldPosition()
        if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
            local fx = SpawnPrefab("splash_water_big")
            fx.Transform:SetPosition(x2, 0, z2)
            if inst.SoundEmitter then
                inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/item_sink")
            end			
        end					
		local entities2 = TheSim:FindEntities(x2, 0, z2, 5, {"hascombatcomponent"}, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "shadow", "notarget", "seaflora"})
	    for i, v2 in pairs(entities2) do
           if v2 ~= nil and v2:IsValid() and v2.entity:IsVisible() and inst.components.combat:CanTarget(v2) and v2.components.combat ~= nil then
				if v2:HasTag("player") then
				    if attacker ~= nil 
					    and attacker.components.combat ~= nil 
						and attacker.components.combat.target ~= nil
						and attacker.components.combat.target == v2 	
					then
					    if attacker.PlantFoodExplosionDamage ~= nil and attacker.PlantFoodWaterExplosionDamage ~= nil then
					        v2.components.combat:GetAttacked(attacker, (attacker.IsOnWater and (attacker.PlantFoodWaterExplosionDamage or 135)) or (attacker.PlantFoodExplosionDamage or 125), inst)
						else 
						    v2.components.combat:GetAttacked(attacker, attacker.IsOnWater and 135 or 125, inst)
						end
					end
				else
				    if attacker ~= nil then
                        if attacker.PlantFoodExplosionDamage ~= nil and attacker.PlantFoodWaterExplosionDamage ~= nil then
                            v2.components.combat:GetAttacked(attacker, (attacker.IsOnWater and (attacker.PlantFoodWaterExplosionDamage or 135)) or (attacker.PlantFoodExplosionDamage or 125), inst)
				        else 
                            v2.components.combat:GetAttacked(attacker, attacker.IsOnWater and 135 or 125, inst)
						end
                    else
                        v2.components.combat:GetAttacked(inst, inst.GetIsOnWater and inst:GetIsOnWater() and 135 or 125)
                    end	
    			end
            end
        end	    
	    inst:Remove()
	end)	

end

local function OnMiss(inst, owner, target)
    inst:Remove()
end

local function seaflora_projectile_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)
	inst.Physics:ClearCollisionMask()
	
    inst.AnimState:SetBank("hermit_pearl")
    inst.AnimState:SetBuild("hermit_pearl")
    inst.AnimState:PlayAnimation("idle")

    inst:AddTag("NOCLICK")
    inst:AddTag("notarget")
  
    inst.AlreadyHitEnts = {}
    inst.AlreadyHitEntsNum = {}
	
	inst:AddComponent("projectile")
	inst.components.projectile:SetSpeed(10)
	inst.components.projectile:SetOnHitFn(OnHit)
	inst.components.projectile:SetRange(40)
	inst.components.projectile:SetHitDist(2)
	inst.components.projectile:SetHoming(false)
	inst.components.projectile:SetLaunchOffset(Vector3(1, 2, 0))
	inst.components.projectile:SetOnMissFn(OnMiss)
	inst.components.projectile.Hit = OverrideHit 

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(50)
    inst.components.combat:SetRange(40)
    inst.components.combat:SetKeepTargetFunction(projectile_keeptarget)	

	inst.persists = false
	
    return inst	
end

local function seaflora_plantfood_projectile_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)
	inst.Physics:ClearCollisionMask()
	
    inst.AnimState:SetBank("hermit_pearl")
    inst.AnimState:SetBuild("hermit_pearl")
    inst.AnimState:PlayAnimation("idle")
	
	inst.Transform:SetScale(3, 3, 3)

    inst:AddTag("NOCLICK")
    inst:AddTag("notarget")
	
	inst.AlreadyHitEnts = {}
 
	inst:AddComponent("projectile")
	inst.components.projectile:SetSpeed(5)
	inst.components.projectile:SetOnHitFn(OnHitPF)
	inst.components.projectile:SetRange(100)
	inst.components.projectile:SetHitDist(2)
	inst.components.projectile:SetHoming(false)
	inst.components.projectile:SetLaunchOffset(Vector3(1, 2, 0))
	inst.components.projectile:SetOnMissFn(OnMiss)
	inst.components.projectile.Hit = OverrideHit 

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(75)
    inst.components.combat:SetRange(100)
    inst.components.combat:SetKeepTargetFunction(projectile_keeptarget)	

	inst.persists = false
	
    return inst	
end

return Prefab("seaflora", seaflora_fn, seaflora_assets, prefabs),
    Prefab("seaflora_projectile", seaflora_projectile_fn, seaflora_projectile_assets, prefabs),
	Prefab("seaflora_plantfood_projectile", seaflora_plantfood_projectile_fn, seaflora_projectile_assets, prefabs)