
local assets_basic =
{
    Asset("ANIM", "anim/shadow_acid_bottle.zip"),
    Asset("ANIM", "anim/swap_shadow_acid_bottle.zip"),
}

local assets_advanced =
{
    Asset("ANIM", "anim/shadow_acid_bottle_advanced.zip"),
    Asset("ANIM", "anim/swap_shadow_acid_bottle_advanced.zip"),
}

local prefabs = 
{

} 

local function CheckIfOwnerIsInShadowForm(inst, owner)
    inst:DoTaskInTime(0, function()
	    if owner ~= nil and not owner:HasTag("shadow_form_on") and owner.components.inventory ~= nil then
		    owner.components.inventory:DropItem(inst, true, true)
		end 
	end)
end

local function OnEquipBasic(inst, owner)
    owner.AnimState:OverrideSymbol("swap_object", "swap_shadow_acid_bottle", "swap_shadow_acid_bottle")
    owner.AnimState:Show("ARM_carry")
    owner.AnimState:Hide("ARM_normal")
	CheckIfOwnerIsInShadowForm(inst, owner)
end

local function OnUnequip(inst, owner)
    owner.AnimState:Hide("ARM_carry")
    owner.AnimState:Show("ARM_normal")
end

local function OnEquipAdvanced(inst, owner)
    owner.AnimState:OverrideSymbol("swap_object", "swap_shadow_acid_bottle_advanced", "swap_shadow_acid_bottle_advanced")
    owner.AnimState:Show("ARM_carry")
    owner.AnimState:Hide("ARM_normal")
	CheckIfOwnerIsInShadowForm(inst, owner)
end

local function on_inventory_thrown(inst)
    inst:AddTag("NOCLICK")
    inst.persists = false

    inst.AnimState:PlayAnimation("spin_loop", true)
end

local function on_inventory_hit(inst, attacker, target)
    local x, y, z = inst.Transform:GetWorldPosition()
    local impactfx = SpawnPrefab("impact")
    if impactfx ~= nil then
        impactfx.Transform:SetPosition(x, y, z)
		impactfx.Transform:SetScale(2, 2, 2)
		if attacker ~= nil then
            impactfx:FacePoint(attacker.Transform:GetWorldPosition())
		end
    end 
	if inst.SoundEmitter ~= nil then 
	    inst.SoundEmitter:PlaySound("dontstarve/common/destroy_magic")
	end 
	local shadow_effect_area = 4
	if inst:HasTag("advancedshadoweffect") then 
	    shadow_effect_area = 6
	end
    local entities = TheSim:FindEntities(x, y, z, shadow_effect_area, {"hascombatcomponent"}, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "notarget", "player", "companion", "wall", "structure"})
    for k, v in pairs(entities) do
        if v ~= nil and v:IsValid() and v.entity:IsVisible() then
            if v:HasTag("shadow") then
			    if v.components.health ~= nil then 
				    v.components.health:DoDelta(inst:HasTag("advancedshadoweffect") and 1000 or 100)
				end 
				if math.random() < 0.25 and v.components.lootdropper ~= nil then 
				    if math.random() < 0.25 and inst:HasTag("advancedshadoweffect") then 
					    v.components.lootdropper:SpawnLootPrefab("horrorfuel")
					else
					    v.components.lootdropper:SpawnLootPrefab("nightmarefuel")
					end 
				end 
			elseif v.components.combat ~= nil then 
			    v.components.combat:GetAttacked(inst, inst:HasTag("advancedshadoweffect") and 1000 or 100) 
			end 	
        end
    end	
	
    local shadowacidpuddles = TheSim:FindEntities(x, y, z, 2, {"shadowacidpuddle"})
	local puddles = TheSim:FindEntities(x, y, z, 3, {"puddle"})
	if #shadowacidpuddles <= 0 and #puddles <= 0 and inst:IsOnValidGround() then
		if inst.IsPosSurroundedByLand == nil or inst:IsPosSurroundedByLand(x, y, z, 3) then
			local shadowacidpuddle = SpawnPrefab("shadowacidpuddle")
			shadowacidpuddle.Transform:SetPosition(x, y, z)
		    shadowacidpuddle:SetLifeTime(inst:HasTag("advancedshadoweffect") and 2400 or 960)
			if inst:HasTag("advancedshadoweffect") then 
			    shadowacidpuddle.superacid = true 
			end 
			shadowacidpuddle.AnimState:PlayAnimation("dry_pst")
			shadowacidpuddle.AnimState:PushAnimation("idle", true)
		end		
	end		
	
    inst:Remove()
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity) 
		inst.float_fx.Transform:SetScale(1.2, 1.2, 1.2)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity) 
		inst.float_fx2.Transform:SetScale(1.2, 1.2, 1.2)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function OnHitWaterAdvanced(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity) 
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx.Transform:SetScale(0.9, 0.9, 0.9)		
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx2.Transform:SetScale(0.9, 0.9, 0.9)		
	end	
end

local function basicfn()
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState() 
	inst.entity:AddSoundEmitter() 

    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("shadow_acid_bottle")
    inst.AnimState:SetBuild("shadow_acid_bottle")
    inst.AnimState:PlayAnimation("idle") 
	
	inst:AddTag("limited_to_shadow_form")
	inst:AddTag("rangedweapon")
	inst:AddTag("projectile")
	
    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem") 
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

    if inst.GetIsOnWater ~= nil then
	    MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)			
    end	
	
	inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(OnEquipBasic)
    inst.components.equippable:SetOnUnequip(OnUnequip) 
	
    inst:AddComponent("complexprojectile")
    inst.components.complexprojectile:SetHorizontalSpeed(15)
    inst.components.complexprojectile:SetGravity(-10)
    inst.components.complexprojectile:SetLaunchOffset(Vector3(0.25, 1, 0))
    inst.components.complexprojectile:SetOnLaunch(on_inventory_thrown)
    inst.components.complexprojectile:SetOnHit(on_inventory_hit)
	inst.components.complexprojectile.CollectEquippedActions = function(self, doer, target, actions, right)
	    if self ~= nil 
		    and self.inst ~= nil 
			and doer ~= nil 
			and right 
			and doer:HasTag("shadow_form_on") 
		then
		    table.insert(actions, ACTIONS.SHADOW_TOSS)
		end 
	end
	inst.components.complexprojectile.CollectPointActions = function(self, doer, pos, actions, right)
	    if self ~= nil 
		    and self.inst ~= nil 
			and doer ~= nil 
			and right 
			and doer:HasTag("shadow_form_on") 
		then
		    table.insert(actions, ACTIONS.SHADOW_TOSS)
		end 
	end
	
    inst:AddComponent("weapon")
	inst.components.weapon.projectilelaunchsymbol = "swap_object"
	inst.components.weapon.CollectEquippedActions = function() end 
	inst.components.weapon.CollectUseActions = function() end 
    inst.components.weapon:SetDamage(100)
    inst.components.weapon:SetRange(8, 10)

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(100)
    inst.components.combat:SetRange(3)
    inst.components.combat:SetKeepTargetFunction(function() return false end)	
	
	return inst 
end


local function advancedfn() 
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState() 
	inst.entity:AddSoundEmitter() 

    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("shadow_acid_bottle_advanced")
    inst.AnimState:SetBuild("shadow_acid_bottle_advanced")
    inst.AnimState:PlayAnimation("idle") 
	
	inst:AddTag("limited_to_shadow_form")
	inst:AddTag("rangedweapon") 
	inst:AddTag("projectile")
	inst:AddTag("advancedshadoweffect")
	
    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem") 	
	
    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end

    if inst.GetIsOnWater ~= nil then
	    MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWaterAdvanced)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		      
    end	

	inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(OnEquipAdvanced)
    inst.components.equippable:SetOnUnequip(OnUnequip)

    inst:AddComponent("complexprojectile")
    inst.components.complexprojectile:SetHorizontalSpeed(18)
    inst.components.complexprojectile:SetGravity(-10)
    inst.components.complexprojectile:SetLaunchOffset(Vector3(0.25, 1, 0))
    inst.components.complexprojectile:SetOnLaunch(on_inventory_thrown)
    inst.components.complexprojectile:SetOnHit(on_inventory_hit)
	inst.components.complexprojectile.CollectEquippedActions = function(self, doer, target, actions, right)
	    if self ~= nil 
		    and self.inst ~= nil 
			and doer ~= nil 
			and right 
			and doer:HasTag("shadow_form_on") 
		then
		    table.insert(actions, ACTIONS.SHADOW_TOSS)
		end 	
	end
	inst.components.complexprojectile.CollectPointActions = function(self, doer, pos, actions, right)
	    if self ~= nil 
		    and self.inst ~= nil 
			and doer ~= nil 
			and right 
			and doer:HasTag("shadow_form_on") 
		then
		    table.insert(actions, ACTIONS.SHADOW_TOSS)
		end 	
	end

    inst:AddComponent("weapon")
	inst.components.weapon.projectilelaunchsymbol = "swap_object" 
	inst.components.weapon.CollectEquippedActions = function() end 
	inst.components.weapon.CollectUseActions = function() end 
    inst.components.weapon:SetDamage(1000)
    inst.components.weapon:SetRange(12, 14)

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(1000)
    inst.components.combat:SetRange(5)
    inst.components.combat:SetKeepTargetFunction(function() return false end)	
	
	return inst 
end 

return Prefab("shadow_acid_bottle", basicfn, assets_basic, prefabs),
    Prefab("shadow_acid_bottle_advanced", advancedfn, assets_advanced, prefabs)