local assets =
{
    Asset("ANIM", "anim/shadow_forge.zip"),
}

local prefabs =
{

}

local BLOCKER_EXCLUDE_TAGS = {"FX", "INLIMBO"}
local function IsPointNearBlocker(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, BLOCKER_EXCLUDE_TAGS) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearPlayer(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, {"player"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearSomething(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"charlie_stage", "charlie_throne", "ruins_exit", "pugalisk_fountain"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointNearWallOrStructure(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, nil, nil, {"wall", "structure"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function FindSafeLocationToMove(inst)
	local centers = {}
    for i, node in ipairs( GetWorld().topology.nodes ) do
        if node.type ~= NODE_TYPE.Blank and node.type ~= NODE_TYPE.Blocker then
            local x = node.x
            local z = node.y
            local pos = Vector3(x, 0, z)
            local tilepos = Vector3( GetWorld().Map:GetTileCenterPoint(pos.x, pos.y, pos.z) )
            if IsPassableAtPoint(tilepos.x, tilepos.y, tilepos.z) 
			    and ( GetPlayer().IsPosSurroundedByLand == nil or GetPlayer():IsPosSurroundedByLand(tilepos.x, tilepos.y, tilepos.z, 3) )
				and not IsPointNearPlayer(tilepos, 150) 		
			    and not IsPointNearSomething(tilepos, 30) 	
				and not IsPointNearBlocker(tilepos, 5) 		
				and not IsPointNearWallOrStructure(tilepos, 10) 			
			then                                            
                table.insert(centers, {x = node.x, z = node.y})
            end
        end		
	end
    if #centers > 0 then
        local pos = centers[math.random(#centers)]
        return pos.x, 0, pos.z
    else
        return inst.Transform:GetWorldPosition()
    end
end

local function SpawnShadowFX(inst)
    SpawnPrefab("statue_transition").Transform:SetPosition(inst.Transform:GetWorldPosition())
    SpawnPrefab("statue_transition_2").Transform:SetPosition(inst.Transform:GetWorldPosition())
end

local function LevelTwoToThree(inst)
    inst.level = 3
    if inst.AnimState:IsCurrentAnimation("proximity_loop") or inst.AnimState:IsCurrentAnimation("use") then
        --In case other animations were still in queue
        inst.AnimState:PlayAnimation("proximity_loop", true)
    else
        if inst.AnimState:IsCurrentAnimation("place") then
            inst.AnimState:PushAnimation("proximity_pre")
        else
            inst.AnimState:PlayAnimation("proximity_pre")
        end
        inst.AnimState:PushAnimation("proximity_loop", true)
    end
    if not inst.SoundEmitter:PlayingSound("loopsound") then
        inst.SoundEmitter:PlaySound("rifts2/shadow_forge/proximity_lp", "loopsound")
    end
end

local function LevelOneToTwo(inst)
    inst.level = 2
    inst.AnimState:PlayAnimation("place")
    inst.AnimState:PushAnimation("idle", false)
    inst.SoundEmitter:PlaySound("rifts2/shadow_forge/place")
end

local function OnRefuse(inst, giver, item)
    if giver ~= nil then
        if giver:HasTag("strengthened_by_darkness") then
            if giver.components.talker ~= nil then
	            if TUNING.LANGUAGE == true then
		            giver.components.talker:Say("Well, it's not this thing. I think it's a darker thing.")
		        elseif TUNING.LANGUAGE == false then
		            giver.components.talker:Say("额，不是这个东西。我认为是一种更黑暗的东西。")
		        end				
		    end	
		else
            if giver.components.talker ~= nil then
	            if TUNING.LANGUAGE == true then
		            giver.components.talker:Say("It seems to reject me.")
		        elseif TUNING.LANGUAGE == false then
		            giver.components.talker:Say("它似乎排斥我。")
		        end
		    end
		end
	end
end

local function TryToUpgrade(inst, giver, item)
    inst.TryToUpgrade = true
	inst:DoTaskInTime(0.5, function() inst.TryToUpgrade = nil end)
	if inst.level == 1 then
	    LevelOneToTwo(inst)
        if giver.components.talker ~= nil then
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("Great achievement! But more construction is still needed.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say("大功告成！但还是需要更多的建设。")
		    end
		end	
	elseif inst.level == 2 then
	    LevelTwoToThree(inst)
        if giver.components.talker ~= nil then
	        if TUNING.LANGUAGE == true then
		        giver.components.talker:Say("Complete form! Now I can use the knowledge inside it.")
		    elseif TUNING.LANGUAGE == false then
		        giver.components.talker:Say("完整形态！现在我可以用它里面的知识了。")
		    end
		end	
	end
end

local function OnAccept(inst, giver, item)
    if giver ~= nil and giver:HasTag("strengthened_by_darkness") then

        if inst.level == 3 then
            local x, y, z = inst.Transform:GetWorldPosition()
	        local possible_items = 
		    { 
			    "cutstone",
			    "marble",
			    "dreadstone",
			    "voidcloth",
		    }
            local item = SpawnPrefab(GetRandomItem(possible_items))
	        inst.components.lootdropper:FlingItem(item, Vector3(x, math.random()*3, z))	    
            if giver.components.talker ~= nil then
	            if TUNING.LANGUAGE == true then
		            giver.components.talker:Say("Don't need more fuel, but it seems to have given me something.")
		        elseif TUNING.LANGUAGE == false then
		            giver.components.talker:Say("不需要更多燃料了，但它好像给了我某些东西。")
		        end
		    end					
		    return
		end

	    inst.fuel_inside = math.min(inst.fuel_inside + 1, inst.max_fuel_to_save)
		
		if inst.fuel_inside == 5 or inst.fuel_inside == 15 then
		    TryToUpgrade(inst, giver, item)
		end	
		
        if inst.fuel_inside < inst.max_fuel_to_save and not inst.TryToUpgrade then		    
            if giver.components.talker ~= nil then
	            if TUNING.LANGUAGE == true then
		            giver.components.talker:Say("Eh, I need more fuel to study it.")
		        elseif TUNING.LANGUAGE == false then
		            giver.components.talker:Say("嗯，我需要更多燃料来研究它。")
		        end
		    end		
		end
	end
end

local function AcceptTest(inst, item, giver)
    if giver ~= nil and not giver:HasTag("strengthened_by_darkness") then
	    return false
	end
    return item ~= nil and item.prefab ~= nil and item.prefab == "horrorfuel"
end

local function UpdateUsing(inst)
    if inst.level == 3 then
	    if inst.components.playerprox:IsPlayerClose() then
		    if not inst.on_used then
		        inst.on_used = true
                inst.AnimState:PlayAnimation("use", true)
                inst.SoundEmitter:PlaySound("rifts2/shadow_forge/use")
            end				
		else
		    if inst.on_used then
		        inst.on_used = nil
			    inst.AnimState:PlayAnimation("proximity_loop", true)
			end
		end	
	end
end

local function description(inst, viwer)
    local desc = ""
    if TUNING.LANGUAGE == true then
		if inst.level == 1 then
            desc = "It's incomplete, I need to find a way to fix it."
        elseif inst.level == 2 then
            desc = "Further repairs would be better."
        elseif inst.level == 3 then
            desc = "Perfect! Now I can acquire shadow knowledge."
        end		
	elseif TUNING.LANGUAGE == false then
		if inst.level == 1 then
            desc = "它不完整，我得想办法修复它。"
        elseif inst.level == 2 then
            desc = "进一步的修复会更好。"
        elseif inst.level == 3 then
            desc = "完美！现在我可以获取暗影知识了。"
        end	
	end
	return desc
end


local function OnSave(inst, data)
    data.level = inst.level
	data.fuel_inside = inst.fuel_inside
end

local function OnLoad(inst, data)
    if data ~= nil then    
        inst.level = data.level or 1
		inst.fuel_inside = data.fuel_inside or 0
		if inst.level == 1 then
	        inst.AnimState:PlayAnimation("kit")
		elseif inst.level == 2 then
		    inst.AnimState:PlayAnimation("idle")
		elseif inst.level == 3 then
		    inst.AnimState:PlayAnimation("proximity_loop", true)
		end
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddSoundEmitter()

    MakeObstaclePhysics(inst, 0.4)

    inst.MiniMapEntity:SetIcon("shadow_forge.tex")

    inst.AnimState:SetBank("shadow_forge")
    inst.AnimState:SetBuild("shadow_forge")
    inst.AnimState:PlayAnimation("kit")

    inst.level = 1
	inst.fuel_inside = 0
	inst.max_fuel_to_save = 20

    inst:AddTag("shadow_forge")
    inst:AddTag("shadow_destroy_immune")
	inst:AddTag("pugalisk_avoids")

    inst:AddComponent("inspectable")
	inst.components.inspectable:SetDescription(description)

    inst:AddComponent("lootdropper")

    inst:AddComponent("playerprox")
    inst.components.playerprox:SetDist(2, 3)
    inst.components.playerprox:SetOnPlayerNear(UpdateUsing)
    inst.components.playerprox:SetOnPlayerFar(UpdateUsing)

	inst:AddComponent("trader")
	inst.components.trader.acceptnontradable = true
	inst.components.trader.onrefuse = OnRefuse
	inst.components.trader.onaccept = OnAccept
    inst.components.trader:SetAcceptTest(AcceptTest)
	
	inst:AddComponent("shadowbookforger")
	
	inst.FindSafeLocationToMove = FindSafeLocationToMove
	inst.SpawnShadowFX = SpawnShadowFX

    MakeSnowCovered(inst)

    inst.OnSave = OnSave
    inst.OnLoad = OnLoad

    return inst
end

return Prefab("shadow_forge", fn, assets, prefabs)
