local assets = 
{
	Asset("ANIM", "anim/skeletonhat_shadowhelmet.zip"),
}

local prefabs = 
{

}

local function OnRepaired(inst, doer, repair_item)
	if doer ~= nil and doer.SoundEmitter ~= nil then
	    doer.SoundEmitter:PlaySound("dontstarve/HUD/repair_clothing")
	end
end

local function OnBlocked(owner) 
    if owner.SoundEmitter ~= nil then
        owner.SoundEmitter:PlaySound("dontstarve/wilson/hit_nightarmour") 
	end
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function OnEquip(inst, owner)
	owner.AnimState:OverrideSymbol("swap_hat", "skeletonhat_shadowhelmet", "swap_hat")
    owner.AnimState:Show("HAT")
    owner.AnimState:Show("HAIR_HAT")
    owner.AnimState:Hide("HAIR_NOHAT")
    owner.AnimState:Hide("HAIR")
    if owner:HasTag("player") then
		owner.AnimState:Hide("HEAD")
    	owner.AnimState:Show("HEAD_HAIR")
		owner.AnimState:Hide("HAIRFRONT")
	end	
	inst:ListenForEvent("blocked", OnBlocked, owner)
end

local function OnUnequip(inst, owner)
    owner.AnimState:Hide("HAT")
    owner.AnimState:Hide("HAIR_HAT")
    owner.AnimState:Show("HAIR_NOHAT")
    owner.AnimState:Show("HAIR")
	if owner:HasTag("player") then
    	owner.AnimState:Show("HEAD")
		owner.AnimState:Hide("HEAD_HAIR")
		owner.AnimState:Show("HAIRFRONT")
	end
	inst:RemoveEventCallback("blocked", OnBlocked, owner)
end

local function OnTakeDamage(inst, damage_amount, absorbed, leftover)
	local owner = inst.components.inventoryitem.owner
	if owner then
		local sanity = owner.components.sanity
		if sanity then
			local unsaneness = damage_amount * TUNING.ARMOR_SANITY_DMG_AS_SANITY
			sanity:DoDelta(-unsaneness, false)
		end
	end
end

local function fn()
	local inst = CreateEntity()
	local player = GetPlayer()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()

	MakeInventoryPhysics(inst)

	inst.AnimState:SetBank("footballhat")
	inst.AnimState:SetBuild("skeletonhat_shadowhelmet")
	inst.AnimState:PlayAnimation("anim")	

    inst:AddTag("hat")
    inst:AddTag("shadow_helmet")

	inst:AddComponent("inspectable")

	inst:AddComponent("inventoryitem")

	inst:AddComponent("armor")
	inst.components.armor:InitCondition(750, 0.95)

	inst:AddComponent("equippable")
	inst.components.equippable.equipslot = EQUIPSLOTS.HEAD
	inst.components.equippable:SetOnEquip(OnEquip)
	inst.components.equippable:SetOnUnequip(OnUnequip)
	
	inst:AddComponent("tradable")		
	
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    MakeInventoryFloatable(inst, "idle_water", "anim")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)		
	end	
	
	if player ~= nil and not player:HasTag("fear_of_darkness") then
	    inst.components.armor.ontakedamage = OnTakeDamage
		if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	        inst.components.equippable.dapperness = TUNING.CRAZINESS_SMALL
		else
            inst:AddComponent("dapperness")
            inst.components.dapperness.dapperness = TUNING.CRAZINESS_SMALL		
		end	
	end
	
    inst:AddComponent("shadowrepairable")
	inst.components.shadowrepairable:SetRepairMaterial("voidcloth")
	inst.components.shadowrepairable:SetOnRepaired(OnRepaired)
	
	return inst
end

return Prefab("shadow_helmet", fn, assets, prefabs)