local assets =
{
    Asset("ANIM", "anim/charlies_lovely_gift.zip"),
    Asset("ANIM", "anim/swap_charlies_lovely_gift.zip"), 
}

local prefabs =
{
    "rose_fx",
	"rose_inv",
}

local function TryToSpawn(inst) 
    if GetClock() 
	    and GetClock():IsDay() 
		and inst:IsOnValidGround() 
	    and GetPlayer()
		and GetPlayer():HasTag("flower_lover")
		and GetPlayer().entity:IsVisible()
	then
	    local prefab_to_spawn = SpawnPrefab(math.random() < 0.01 and "rose_inv" or "petals")
		if prefab_to_spawn then
		    local x, y, z = inst.Transform:GetWorldPosition()
		    prefab_to_spawn.Transform:SetPosition(x, y, z)
			SpawnPrefab("small_puff").Transform:SetPosition(x, y, z)
			if prefab_to_spawn.GetIsOnWater ~= nil
			    and prefab_to_spawn:GetIsOnWater()
			then
			    if prefab_to_spawn.components.floatable ~= nil then
				    prefab_to_spawn.components.floatable:OnHitWater(true)
				elseif prefab_to_spawn.components.inventoryitem ~= nil then
			        prefab_to_spawn.components.inventoryitem:OnHitWater()
			    end			
			end
		end
	end
end

local function UpdateDamage(inst)
    if inst.components.weapon == nil then return end
    if GetWorld():IsCave() then
	    inst.components.weapon:SetDamage(TUNING.BOUQUET_DAMAGE_CAVE)
	else
        if GetClock():IsDay() then
            inst.components.weapon:SetDamage(TUNING.BOUQUET_BASE_DAMAGE)
        elseif GetClock():IsNight() then
            inst.components.weapon:SetDamage(TUNING.BOUQUET_DAMAGE_NIGHT)
        elseif GetClock():IsDusk() then
            inst.components.weapon:SetDamage(TUNING.BOUQUET_DAMAGE_DUSK)
		end
    end
end

local function onequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_charlies_lovely_gift", "swap_charlies_lovely_gift") 
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function onunequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

local function SanityAura(inst, observer)

    if inst.components.inventoryitem ~= nil and inst.components.inventoryitem.owner == nil then
        if observer:HasTag("flower_lover") then
            return TUNING.SANITYAURA_SMALL*0.125
        end
    end
	
    return 0
end

local DAMAGE_MUST_TAGS = {"hascombatcomponent"}
local DAMAGE_CANT_TAGS = {"FX", "NOCLICK", "INLIMBO", "INTERIOR_LIMBO", "DECOR", "invisible", "notarget", "boat", "wall", "companion", "bouquet_immune", "buttercup"}
local function onattack(inst, attacker, target)
	if inst.components.inventoryitem ~= nil 
	    and inst.components.inventoryitem.owner ~= nil
		and inst.components.inventoryitem.owner.components.combat ~= nil 
		and inst.components.inventoryitem.owner.components.combat.target ~= nil 
	then
		local owner = inst.components.inventoryitem.owner
		local target = owner.components.combat.target
		
		local target_x, target_y, target_z = nil, nil, nil
		if target.Transform ~= nil then
		    target_x, target_y, target_z = target.Transform:GetWorldPosition()
		end
		local owner_x, owner_y, owner_z = owner.Transform:GetWorldPosition()
		local fx_x, fx_y, fx_z = nil, nil, nil
		
		if target_x > owner_x then
			fx_x = owner_x - (owner_x - target_x)/2
		elseif target_x < owner_x then
			fx_x = target_x - (target_x - owner_x)/2
		else
			fx_x = owner_x
		end
		
		if target_y > owner_y then
			fx_y = owner_y - (owner_y - target_y)/2
		elseif target_y < owner_y then
			fx_y = target_y - (target_y - owner_y)/2
		else
			fx_y = owner_y
		end
		
		if target_z > owner_z then
			fx_z = owner_z - (owner_z - target_z)/2
		elseif target_z < owner_z then
			fx_z = target_z - (target_z - owner_z)/2
		else
			fx_z = owner_z
		end
		
		if target.Transform ~= nil and fx_x ~= nil and fx_y ~= nil and fx_z ~= nil then
			SpawnPrefab("rose_fx").Transform:SetPosition(fx_x, fx_y, fx_z)
		else
			SpawnPrefab("rose_fx").Transform:SetPosition(owner.Transform:GetWorldPosition())
		end
	else
		SpawnPrefab("rose_fx").Transform:SetPosition(inst.Transform:GetWorldPosition())
	end	
	inst.SoundEmitter:PlaySound("dontstarve/wilson/pickup_reeds")
	if target ~= nil and target.Transform ~= nil then
	    local x, y, z = target.Transform:GetWorldPosition()
	    local ents = TheSim:FindEntities(x, y, z, TUNING.BOUQUET_DAMAGE_RADIUS, DAMAGE_MUST_TAGS, DAMAGE_CANT_TAGS)
		for k,v in pairs(ents) do
		    if v ~= inst and v ~= target and v ~= attacker then
			    if v:IsValid() and v.entity:IsVisible() and v.components.combat ~= nil then
				    v.components.combat:GetAttacked(attacker, TUNING.BOUQUET_BASE_DAMAGE, inst)
					if v.Transform ~= nil then
					    SpawnPrefab("rose_fx").Transform:SetPosition(v.Transform:GetWorldPosition())
				    end
	                if v.components.pollinator ~= nil and math.random() < 0.25 then
	                    v.components.combat:SetTarget(nil)
	                end					
				end
			end
		end
	end
	
	if target ~= nil 
	    and not target:HasTag("buttercup")
	    and target.components.pollinator ~= nil 
		and target.components.combat ~= nil 
		and math.random() < 0.25 
	then
	    target.components.combat:SetTarget(nil)
	end
	
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.Transform:SetPosition(0, 0.1, 0)
		inst.float_fx.Transform:SetScale(1.2, 0.65, 1.2)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.Transform:SetPosition(0, 0.1, 0)
		inst.float_fx2.Transform:SetScale(1.2, 0.65, 1.2)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function onrepaired(inst, doer, repair_item)
    if inst.UpdateDamage ~= nil then
	    inst:UpdateDamage()
	end
	inst.SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
end

local function fn()
    local inst = CreateEntity()
    
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("charlies_lovely_gift")
    inst.AnimState:SetBuild("charlies_lovely_gift")
    inst.AnimState:PlayAnimation("idle")

	inst:AddTag("shadow_item")
    inst:AddTag("sharp")
	inst:AddTag("invflower")
    inst:AddTag("show_spoilage")
    inst:AddTag("cattoy")
	inst:AddTag("conjurable")
	
	inst:AddComponent("inventoryitem")

	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	    inst:AddComponent("floatable")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
		
		inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = TUNING.APPEASEMENT_SMALL
	end
	
    inst:AddComponent("inspectable")
	
	inst:AddComponent("tradable")

    inst:AddComponent("conjurable")
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(TUNING.BOUQUET_BASE_DAMAGE)
	inst.components.weapon:SetOnAttack(onattack)
	inst.components.weapon:SetRange(1.5)

    inst:DoTaskInTime(0, function() UpdateDamage(inst) end)	
	inst:ListenForEvent("dusktime", function() UpdateDamage(inst) end, GetWorld())
    inst:ListenForEvent("daytime", function() UpdateDamage(inst) end, GetWorld())
    inst:ListenForEvent("nighttime", function() UpdateDamage(inst) end, GetWorld())
	inst:ListenForEvent("arrive", function() UpdateDamage(inst) end, GetWorld())
	
	inst.UpdateDamage = UpdateDamage
	
 	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.PERISH_FAST)
	inst.components.perishable:StartPerishing()
	inst.components.perishable:SetOnPerishFn(inst.Remove)

	inst:AddComponent("sanityaura") 
	inst.components.sanityaura.aurafn = SanityAura
    
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
		inst.components.equippable.dapperfn = function(inst, owner)
		    return owner ~= nil and owner:HasTag("flower_lover") and TUNING.DAPPERNESS_LARGE*2 or TUNING.CRAZINESS_MED
		end
	else
	    inst:AddComponent("dapperness")
        inst.components.dapperness.dapperfn = function(inst, owner) 
		    return owner ~= nil and owner:HasTag("flower_lover") and TUNING.DAPPERNESS_LARGE*2 or TUNING.CRAZINESS_MED
		end
	end
	
    inst:AddComponent("repairable")
    inst.components.repairable.repairmaterial = "rose"
	inst.components.repairable.onrepaired = onrepaired
	inst.components.repairable.announcecanfix = false

    inst:ListenForEvent("daytime", function() TryToSpawn(inst) end, GetWorld())
	
	inst:ListenForEvent("locomote", function(player, data) 
	    if player ~= nil 
		    and player.sg ~= nil 
			and player.sg:HasStateTag("moving")
		    and inst.components.equippable ~= nil 
			and inst.components.equippable.equipper ~= nil
			and inst.components.equippable.equipper == player
			and not inst.IsInRoseFxCooldown
		then
		    SpawnPrefab("rose_fx").Transform:SetPosition(player.Transform:GetWorldPosition())
			inst.IsInRoseFxCooldown = true
			inst:DoTaskInTime(0.2, function() inst.IsInRoseFxCooldown = nil end)
		end
	end, GetPlayer())
	
	inst:ListenForEvent("newstate", function(player, data) 
	    if player ~= nil 
            and player.sg ~= nil 
			and not player.sg:HasStateTag("moving")
			and player.sg:HasStateTag("doing")
		    and inst.components.equippable ~= nil 
			and inst.components.equippable.equipper ~= nil
			and inst.components.equippable.equipper == player
		then
		    SpawnPrefab("rose_fx").Transform:SetPosition(player.Transform:GetWorldPosition())
		end
	end, GetPlayer())
	
    return inst	
end

return Prefab("shadow_rose_bouquet", fn, assets, prefabs)
