require("prefabutil")

local assets =
{
    Asset("ANIM", "anim/treasure_chest.zip"),
    Asset("ANIM", "anim/treasurechest_terrarium.zip"),
	Asset("ANIM", "anim/quagmire_ui_chest_3x3.zip"),
}

local fx_assets =
{
    Asset("ANIM", "anim/terrariumchest_fx.zip"),
}

local prefabs =
{
    "collapse_small",
	"nightmarefuel",
}

local fx_prefabs =
{

}

local function OnWorkFinished(inst, worker)
    if inst.components.container ~= nil then
        inst.components.container:DropEverything()
    end
    inst.components.lootdropper:DropLoot()
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
    fx.SoundEmitter:PlaySound("dontstarve/common/destroy_magic")
    inst:Remove()
end

local function OnWorked(inst, worker)
    if inst.components.container ~= nil then
        inst.components.container:Close()
    end
    inst.AnimState:PlayAnimation("hit")
    inst.AnimState:PushAnimation("closed", false)
end

local function onopen(inst)
	if inst.fx ~= nil then
		inst.fx:Remove()
		inst.fx = nil
	end
    inst.AnimState:PlayAnimation("open")
	inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_open")
end 

local function onclose(inst) 
	if inst.fx == nil then
		inst.fx = SpawnPrefab("shadowcorrosionmachine_fx")
		inst.fx.entity:SetParent(inst.entity)
	end
    inst.AnimState:PlayAnimation("close")
    inst.AnimState:PushAnimation("closed", false)
    inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_close")
end

local function onbuilt(inst)
    inst.AnimState:PlayAnimation("place")
    inst.AnimState:PushAnimation("closed", false)
    inst.SoundEmitter:PlaySound("dontstarve/common/chest_craft")
end

local function reset(inst)
    inst:RemoveTag("shadowcorrosionmachine_in_cooldown")
    if inst.resettask then
        inst.resettask:Cancel()
        inst.resettask = nil
    end
    if inst.resettaskinfo then
        inst.resettaskinfo = nil
    end	
end

local function OnLongUpdate(inst, dt)
    if inst.resettask and inst.resettaskinfo then
        local new_time = math.max(inst:TimeRemainingInTask(inst.resettaskinfo) - dt, 0)

        inst.resettask:Cancel()
        inst.resettask = nil 
        inst.resettaskinfo = nil

        inst.resettask, inst.resettaskinfo = inst:ResumeTask(new_time, reset)
    end
end

local slotpos_3x3 = {}

for y = 2, 0, -1 do
	for x = 0, 2 do
		table.insert(slotpos_3x3, Vector3(80*x-80*2+80, 80*y-80*2+80, 0))
	end
end

local function convertintonightmarefuel(inst)
    for i = 1, inst.components.container:GetNumSlots() do
	    local item = inst.components.container:GetItemInSlot(i)
	     if item then 
	        local replacement = nil 
            if not item:HasTag("irreplaceable") 
			    and item.prefab ~= "nightmarefuel" 
			    and item.prefab ~= "horrorfuel"
			then
                replacement = "nightmarefuel"
            end
			if inst:HasTag("shadowcorrosionmachine_in_cooldown") 
			    and not item:HasTag("irreplaceable")
			then
			    replacement = ""  --Empty string is considered as true in lua
			end
		    if replacement then 
	     	    local stacksize = 1 
	     	    if item.components.stackable then 
	     		    stacksize = math.max(item.components.stackable:StackSize() - math.random(40), 1)
	     	    end 
	     	    inst.components.container:RemoveItemBySlot(i)
	     	    item:Remove()					
				if replacement ~= "" then
	     	   	    local newprefab = SpawnPrefab(replacement)
	     	        if newprefab.components.stackable then 
	     		        newprefab.components.stackable:SetStackSize(stacksize)
	     	        end 
					inst.components.container:GiveItem(newprefab, i)
				end
                if not inst.resettask then
				    inst:AddTag("shadowcorrosionmachine_in_cooldown")
                    inst.resettask, inst.resettaskinfo = inst:ResumeTask(TUNING.TOTAL_DAY_TIME*10, reset)
                end				
	     	end 
		end 
	end
    inst.components.container:DropEverythingWithTag("irreplaceable")
end

local widgetbuttoninfo_eng = 
{
    text = "CONVERT",
    position = Vector3(0, -145, 0),
    fn = function(inst)
        convertintonightmarefuel(inst)
    end,
    validfn = function(inst)
        return inst.components.container and not inst.components.container:IsEmpty()
    end,	
}

local widgetbuttoninfo_cn = 
{
    text = "转换",
    position = Vector3(0, -145, 0),
    fn = function(inst)
        convertintonightmarefuel(inst)
    end,
    validfn = function(inst)
        return inst.components.container and not inst.components.container:IsEmpty()
    end,	
}

local function RestoreShadowItems(inst)
    local slots = inst.components.container and inst.components.container.slots or {}
    for k,v in pairs(slots) do
        if v ~= nil and v:HasTag("shadow_item") and v.components.finiteuses ~= nil then
            local percent = v.components.finiteuses:GetPercent()
            if percent < 1 and math.random() < 0.5 then
                v.components.finiteuses:SetPercent(math.min(1, percent + 0.01))
            end
        end
    end
end

local function OnSave(inst, data)
    data.InCooldown = inst:HasTag("shadowcorrosionmachine_in_cooldown") or nil
    if inst.resettaskinfo then
        data.resettask = inst:TimeRemainingInTask(inst.resettaskinfo)
    end
end

local function OnLoad(inst, data)
    if data ~= nil then    
	    if data.InCooldown then
            inst:AddTag("shadowcorrosionmachine_in_cooldown")
		end
        if data.resettask then
            if inst.resettask then 
			    inst.resettask:Cancel() 
				inst.resettask = nil 		
			end
            inst.resettaskinfo = nil
            inst.resettask, inst.resettaskinfo = inst:ResumeTask(data.resettask, reset)
        end 		
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()

    MakeObstaclePhysics(inst, 0.25)

    inst.MiniMapEntity:SetIcon("shadowcorrosionmachine.tex")

    inst.AnimState:SetBank("chest")
    inst.AnimState:SetBuild("treasurechest_terrarium")
    inst.AnimState:PlayAnimation("closed")

    inst:AddTag("structure")
    inst:AddTag("fireimmune")
    inst:AddTag("shadowcorrosionmachine")
	inst:AddTag("pogproof")
	inst:AddTag("shadow_destroy_immune")

    inst:AddComponent("inspectable")

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
    inst.components.workable:SetOnFinishCallback(OnWorkFinished)
    inst.components.workable:SetOnWorkCallback(OnWorked)
	
    inst:AddComponent("lootdropper")

    inst:AddComponent("container")
    inst.components.container:SetNumSlots(#slotpos_3x3, true)
    inst.components.container.onopenfn = onopen
    inst.components.container.onclosefn = onclose	
    inst.components.container.widgetslotpos = slotpos_3x3
    inst.components.container.widgetanimbank = "quagmire_ui_chest_3x3"
    inst.components.container.widgetanimbuild = "quagmire_ui_chest_3x3"
    inst.components.container.widgetpos = Vector3(0, 200, 0)
    inst.components.container.side_align_tip = 160
	
	if TUNING.LANGUAGE == true then
        inst.components.container.widgetbuttoninfo = widgetbuttoninfo_eng
	elseif TUNING.LANGUAGE == false then
	    inst.components.container.widgetbuttoninfo = widgetbuttoninfo_cn
	end

    inst:ListenForEvent("onbuilt", onbuilt)

	inst:DoTaskInTime(0, function()
	    if inst.fx == nil then
			inst.fx = SpawnPrefab("shadowcorrosionmachine_fx")
			inst.fx.entity:SetParent(inst.entity)
		end
	end)
	
	inst.RestoreTask = inst:DoPeriodicTask(60, RestoreShadowItems)
	
	MakeSnowCovered(inst)

	inst.OnLongUpdate = OnLongUpdate
    inst.OnSave = OnSave
    inst.OnLoad = OnLoad

    return inst
end

local function fx_fn()
    local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()

	inst:AddTag("FX")
    inst:AddTag("INTERIOR_LIMBO_IMMUNE")
	inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")

	inst.AnimState:SetBuild("terrariumchest_fx")
	inst.AnimState:SetBank("terrariumchest_fx")
	inst.AnimState:PlayAnimation("idle_back", true)
	inst.AnimState:SetFinalOffset(1)
	
	inst.persists = false
	
    return inst
end

return Prefab("shadowcorrosionmachine", fn, assets, prefabs),
    Prefab("shadowcorrosionmachine_fx", fx_fn, fx_assets, fx_prefabs),
    MakePlacer("shadowcorrosionmachine_placer", "chest", "treasurechest_terrarium", "closed")