local assets = 
{
	Asset("ANIM", "anim/starstaff_portal.zip"),
	Asset("ANIM", "anim/swap_starstaff_portal.zip"),
	Asset("ANIM", "anim/swap_staffs.zip"),
}

local prefabs = 
{

}

local function OnFinished(inst)
    inst.SoundEmitter:PlaySound("dontstarve/common/gem_shatter")
    inst:Remove()
end

local function CanCreateLight(inst, caster, target, pos)
    if GetWorld() and pos then
        local tile = GetWorld().Map:GetTileAtPoint(pos.x, pos.y, pos.z)
        return tile ~= GROUND.IMPASSABLE and tile < GROUND.UNDERGROUND
    end
    return false
end

local function CreateLight(inst, target, pos)

    SpawnPrefab("stafflight").Transform:SetPosition(pos.x, 0, pos.z)

    local rot = 45 * math.pi / 180
    local sin_rot = math.sin(rot)
    local cos_rot = math.cos(rot)
	local light_x1 = -1.6
	local light_z1 = -1.6
	
	local light_x2 = 2.7
	local light_z2 = -0.8
	
	local light_x3 = -0.8
	local light_z3 = 2.7
	
	SpawnPrefab("stafflight").Transform:SetPosition(pos.x + light_x1 * cos_rot - light_z1 * sin_rot, 0, pos.z + light_z1 * cos_rot + light_x1 * sin_rot)
	SpawnPrefab("stafflight").Transform:SetPosition(pos.x + light_x2 * cos_rot - light_z2 * sin_rot, 0, pos.z + light_z2 * cos_rot + light_x2 * sin_rot)
	SpawnPrefab("stafflight").Transform:SetPosition(pos.x + light_x3 * cos_rot - light_z3 * sin_rot, 0, pos.z + light_z3 * cos_rot + light_x3 * sin_rot)

    inst.components.finiteuses:Use(10)

end

local function OnEquipShadowStarStaff(inst, owner)
    owner.AnimState:OverrideSymbol("swap_object", "swap_starstaff_portal", "swap_yellowstaff")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function OnUnequipShadowStarStaff(inst, owner)
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal")	
end

local function ShadowStarStaffOnAttack(inst, attacker, target)
    if attacker ~= nil and attacker.SoundEmitter ~= nil then
        attacker.SoundEmitter:PlaySound("dontstarve/wilson/fireball_explo")
	end

    if target ~= nil and target.components.burnable ~= nil and target.components.burnable:IsBurning() then
	    target.components.burnable:Extinguish()
	    return
	end
	
    if target ~= nil 
	    and target.components.burnable ~= nil 
		and target.components.burnable.IsSmoldering ~= nil
		and target.components.burnable:IsSmoldering()
	then
	    if target.components.burnable.SmotherSmolder ~= nil then
	        target.components.burnable:SmotherSmolder()
		end
	    return
	end	

    local x, y, z = inst.Transform:GetWorldPosition()
	if target ~= nil then
	    x, y, z = target.Transform:GetWorldPosition()
	end
	local not_find_any_fire = true
	local no_campfire_target = target == nil or not target:HasTag("campfire")
	local ents = TheSim:FindEntities(x, 0, z, 5, {"fire"})
	for k,v in pairs(ents) do
	    if v.prefab == "houndfire" then
		    not_find_any_fire = false
		    break
		end
	end
	
	if (inst.IsPosSurroundedByLand == nil or inst:IsPosSurroundedByLand(x, y, z, 3)) and not_find_any_fire and no_campfire_target then

        SpawnPrefab("houndfire").Transform:SetPosition(x, 0, z)

        local rot = 36 * math.pi / 180
        local sin_rot = math.sin(rot)
        local cos_rot = math.cos(rot)
	    local light_x1 = 0.8
	    local light_z1 = -2.7
		
	    local light_x2 = 2.7
	    local light_z2 = -0.8
		
	    local light_x3 = -0.8
	    local light_z3 = 2.7	
		
	    local light_x4 = -2.7
	    local light_z4 = 0.8
		
	    SpawnPrefab("houndfire").Transform:SetPosition(x + light_x1 * cos_rot - light_z1 * sin_rot, 0, z + light_z1 * cos_rot + light_x1 * sin_rot)
	    SpawnPrefab("houndfire").Transform:SetPosition(x + light_x2 * cos_rot - light_z2 * sin_rot, 0, z + light_z2 * cos_rot + light_x2 * sin_rot)
	    SpawnPrefab("houndfire").Transform:SetPosition(x + light_x3 * cos_rot - light_z3 * sin_rot, 0, z + light_z3 * cos_rot + light_x3 * sin_rot)
	    SpawnPrefab("houndfire").Transform:SetPosition(x + light_x4 * cos_rot - light_z4 * sin_rot, 0, z + light_z4 * cos_rot + light_x4 * sin_rot)
		
	end

    if target == nil or not target:IsValid() then
        return
    elseif target.components.burnable ~= nil and not target.components.burnable:IsBurning() then
        if target.components.freezable ~= nil and target.components.freezable:IsFrozen() then           
            target.components.freezable:Unfreeze()            
		elseif target.components.fueled ~= nil and target:HasTag("campfire") and target:HasTag("structure") then
		    local fuel = SpawnPrefab("boards")
			if fuel ~= nil then
			    target.components.fueled:TakeFuelItem(fuel)
			end
        elseif target.components.health == nil and target.components.combat == nil then
            target.components.burnable:Ignite(true)
        end   
    end

    if target:HasTag("aquatic") and not target.components.burnable then 
        local pt = target:GetPosition()
        local smoke = SpawnPrefab("smoke_out")
        smoke.Transform:SetPosition(pt:Get())

        if target.SoundEmitter ~= nil then 
            target.SoundEmitter:PlaySound("dontstarve_DLC002/common/fire_weapon_out") 
        end 
    end 	
	
    if target.components.sleeper ~= nil and target.components.sleeper:IsAsleep() then
        target.components.sleeper:WakeUp()
    end	
	
end

local function GetHeat(inst, observer)
    if observer ~= nil and observer.components.inventory ~= nil then
	    if observer.components.inventory:EquipHasTag("starstaff_shadow") 
		    or observer.components.inventory:FindItem(function(item) return item.prefab == "starstaff_shadow" end) 
		then
		    return 0
		end
	end

    local season_manager = GetSeasonManager()
    if season_manager ~= nil 
	    and season_manager:GetCurrentTemperature() < 0 
    then
	    return 60
	end

    return 0
end

local function GetEquippedHeat(inst, observer)
    if observer ~= nil and observer.components.inventory ~= nil then
	    if observer.components.inventory:FindItem(function(item) return item.prefab == "starstaff_shadow" end) then
		    return 0
		end
	end

    local season_manager = GetSeasonManager()
    if season_manager ~= nil 
	    and season_manager:GetCurrentTemperature() < 0 
    then
	    return 60
	end

    return 0
end

local function GetCarriedHeat(inst, observer)
    if observer ~= nil and observer.components.inventory ~= nil then
	    if observer.components.inventory:EquipHasTag("starstaff_shadow") then
		    return 0
		end
	end
	
    local season_manager = GetSeasonManager()
    if season_manager ~= nil 
	    and season_manager:GetCurrentTemperature() < 0 
    then
	    return 60
	end	
	
    return 0
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)	
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function OnRepaired(inst, doer, repair_item)
	if doer ~= nil and doer.SoundEmitter ~= nil then
	    doer.SoundEmitter:PlaySound("dontstarve/HUD/repair_clothing")
	end
end

local function fn()
    local inst = CreateEntity()
	
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("yellowstaff")
    inst.AnimState:SetBuild("starstaff_portal")
    inst.AnimState:PlayAnimation("idle")
	
	inst:AddTag("shadow_item")
    inst:AddTag("shadow")
    inst:AddTag("sharp")
	inst:AddTag("projectile")
	inst:AddTag("starstaff_shadow")
	inst:AddTag("rangedlighter")     --Not used in vanilla (without any DLCs)
	inst:AddTag("extinguisher")      --Not used in vanilla (without any DLCs)
	
    inst.fxcolour = {223/255, 208/255, 69/255}
    inst.castsound = "dontstarve/common/staffteleport"

    inst:AddComponent("inspectable")
	
    inst:AddComponent("tradable")
    
    inst:AddComponent("inventoryitem")

    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetOnFinished(OnFinished)	
    inst.components.finiteuses:SetMaxUses(1000)
    inst.components.finiteuses:SetUses(1000)

    inst:AddComponent("spellcaster")
    inst.components.spellcaster:SetSpellFn(CreateLight)
    inst.components.spellcaster:SetSpellTestFn(CanCreateLight)
    inst.components.spellcaster.canuseonpoint = true
    inst.components.spellcaster.canusefrominventory = false

    inst:AddComponent("reticule")
    inst.components.reticule.targetfn = function() return Vector3(GetPlayer().entity:LocalToWorldSpace(5, 0, 0)) end
    inst.components.reticule.ease = true
	
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(OnEquipShadowStarStaff)
    inst.components.equippable:SetOnUnequip(OnUnequipShadowStarStaff)

    inst:AddComponent("weapon")
    inst.components.weapon.projectilelaunchsymbol = "swap_object"
    inst.components.weapon:SetDamage(50)
    inst.components.weapon:SetRange(10, 12)
    inst.components.weapon:SetProjectile("fire_projectile")
	inst.components.weapon:SetOnAttack(ShadowStarStaffOnAttack)
	
	inst:AddComponent("heater")
	inst.components.heater.heatfn = GetHeat
	inst.components.heater.equippedheatfn = GetEquippedHeat
	inst.components.heater.carriedheatfn = GetCarriedHeat

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(0)
    end 

    if inst.GetIsOnWater ~= nil then
        MakeInventoryFloatable(inst, "idle_water", "idle")
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)   	
	end

    inst:AddComponent("shadowrepairable")
	inst.components.shadowrepairable:SetRepairMaterial("voidcloth")
	inst.components.shadowrepairable:SetOnRepaired(OnRepaired)	
	
	return inst
end

return Prefab("starstaff_shadow", fn, assets, prefabs)