require("prefabutil")

local assets = 
{
	Asset("ANIM", "anim/statue_maxwell.zip"),
	Asset("ANIM", "anim/statue_maxwell_build.zip"),
	Asset("ANIM", "anim/statue_maxwell_vine_build.zip"),
	Asset("MINIMAP_IMAGE", "statue"),
}

local prefabs =
{
	"marble",
	"rose_inv",
	"small_puff",
}

local function OnWorked(inst, worker, workleft, dontplayhitanim)
    local pt = Point(inst.Transform:GetWorldPosition())
    if workleft <= 0 then
		inst.SoundEmitter:PlaySound("dontstarve/wilson/rock_break")
	    inst.components.lootdropper:DropLoot(pt)
	    inst:Remove()
	else	            
	    if workleft < TUNING.MARBLEPILLAR_MINE*(1/3) then
		    if not dontplayhitanim then
	            inst.AnimState:PlayAnimation("hit_low")
	            inst.AnimState:PushAnimation("idle_low")
			else
			    inst.AnimState:PlayAnimation("idle_low")
			end
	    elseif workleft < TUNING.MARBLEPILLAR_MINE*(2/3) then
		    if not dontplayhitanim then
	            inst.AnimState:PlayAnimation("hit_med")
	            inst.AnimState:PushAnimation("idle_med")
			else
			    inst.AnimState:PlayAnimation("idle_med")
			end
	    else
		    if not dontplayhitanim then
	            inst.AnimState:PlayAnimation("hit_full")
	            inst.AnimState:PushAnimation("idle_full")
			else
			    inst.AnimState:PlayAnimation("idle_full")
			end
	    end
	end
end

local function OnWorkLoad(inst)
    OnWorked(inst, nil, inst.components.workable.workleft, true)
end

local function onpickedfn(inst, picker)
    inst.AnimState:ClearOverrideBuild("statue_maxwell_vine_build")
end

local function onregenfn(inst)
    inst.AnimState:AddOverrideBuild("statue_maxwell_vine_build")
end

local function OnLoad(inst, data)
    if inst.components.pickable ~= nil and inst.components.pickable.hasbeenpicked then
	    inst.AnimState:ClearOverrideBuild("statue_maxwell_vine_build")
	end
end

local function OnRemoveEntity(inst)
    if inst.components.pickable ~= nil and inst.components.pickable.canbepicked then
	    inst.components.lootdropper:SpawnLootPrefab("rose_inv")
	end
end

local function fn()
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddMiniMapEntity()

    inst:AddTag("maxwell")

	MakeObstaclePhysics(inst, 0.66)

	inst:AddComponent("lootdropper")

	inst.AnimState:SetBank("statue_maxwell")
	inst.AnimState:SetBuild("statue_maxwell_build")
	inst.AnimState:PlayAnimation("idle_full")

    inst.AnimState:AddOverrideBuild("statue_maxwell_vine_build")

	inst.MiniMapEntity:SetIcon("statue.png")

	inst:AddComponent("inspectable")
	inst:AddComponent("workable")
	inst.components.workable.savestate = true
	inst.components.workable:SetWorkAction(ACTIONS.MINE)
	inst.components.workable:SetWorkLeft(TUNING.MARBLEPILLAR_MINE)
	inst.components.workable:SetOnWorkCallback(OnWorked)
	inst.components.workable:SetOnLoadFn(OnWorkLoad)
	
	inst:AddComponent("pickable")
	inst.components.pickable:SetUp("rose_inv", TUNING.TOTAL_DAY_TIME*3)
	inst.components.pickable.picksound = "dontstarve/wilson/pickup_reeds"
    inst.components.pickable.onpickedfn = onpickedfn
    inst.components.pickable.onregenfn = onregenfn		

    inst.OnLoad = OnLoad
	inst.OnRemoveEntity = OnRemoveEntity
	
	return inst
end

return Prefab("statuemaxwell_rose", fn, assets, prefabs),
    MakePlacer("statuemaxwell_rose_placer", "statue_maxwell", "statue_maxwell_build", "idle_full")  --Additional changes are in other places (modmain_additional.lua).
