require("prefabutil")

local function ResetActivating(inst)
    if inst.components.activatable ~= nil then
	    inst.components.activatable.inactive = true
	end
end

local function OnActivate(inst, doer)
    if not inst.off then
	    inst.off = true
	    inst.AnimState:PlayAnimation("turn_off", false)
	    inst.AnimState:PushAnimation("off", true)
    else
        inst.off = nil
	    inst.AnimState:PlayAnimation("turn_on", false)
	    inst.AnimState:PushAnimation("loop", true)		
	end
    inst:DoTaskInTime(0.5, ResetActivating)
end

local function onhammered(inst, worker)
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
    fx.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")

    inst.components.lootdropper:DropLoot()
    inst:Remove()
end

local function OnPick(inst, picker)
	inst.AnimState:OverrideSymbol("eggride_seatsway", inst.build, "")
	inst.AnimState:OverrideSymbol("ride_1eggride_seatsway", inst.build, "")
    inst.AnimState:OverrideSymbol("egg", inst.build, "")	
end

local function OnRegen(inst)
	inst.AnimState:OverrideSymbol("eggride_seatsway", inst.build, "eggride_seatsway")
	inst.AnimState:OverrideSymbol("ride_1eggride_seatsway", inst.build, "ride_1eggride_seatsway")
    inst.AnimState:OverrideSymbol("egg", inst.build, "egg")
end

local function OnBuilt(inst)
	inst.AnimState:PlayAnimation("place", false)
	inst.AnimState:PushAnimation("loop", true)
end

local function MakeSummerEventStructure(name, bank, build, custome_fn)
	local assets =
	{
		Asset("ANIM", "anim/"..build..".zip"),
	}
	
    local prefabs = 
    {
        "collapse_small",
    }
	
	local function normal_fn()	
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddAnimState()
        inst.entity:AddSoundEmitter()
		
		MakeObstaclePhysics(inst, 0.25)

        inst.AnimState:SetBank(bank)
        inst.AnimState:SetBuild(build)
        inst.AnimState:PlayAnimation("loop", true)

   	    inst.build = build

	    inst:AddTag("summerevent_structure")
        inst:AddTag("structure")
        inst:AddTag("cattoyairborne")
		
        inst:AddComponent("pickable")
	    inst.components.pickable:SetUp("tallbirdegg", TUNING.TOTAL_DAY_TIME*10) 
        inst.components.pickable.picksound = "dontstarve/wilson/pickup_reeds" 
        inst.components.pickable.onpickedfn = OnPick
        inst.components.pickable.onregenfn = OnRegen

        inst:AddComponent("inspectable")

	    inst:AddComponent("lootdropper")
		
	    inst:AddComponent("workable")
	    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	    inst.components.workable:SetWorkLeft(1)
	    inst.components.workable:SetOnFinishCallback(onhammered)

     	inst:AddComponent("activatable")
	    inst.components.activatable.OnActivate = OnActivate
        inst.components.activatable.inactive = true
	    inst.components.activatable.standingaction = true

	    inst:ListenForEvent("onbuilt", OnBuilt)

	    MakeMediumBurnable(inst)
	    MakeMediumPropagator(inst)

        inst.OnSave = function(inst, data) data.off = inst.off or nil end
        inst.OnLoad = function(inst, data)
            if data ~= nil and data.off then
	            inst.off = true
		        inst.AnimState:PlayAnimation("off", true)
	        end
            if inst.components.pickable ~= nil and inst.components.pickable.hasbeenpicked then
                inst.AnimState:OverrideSymbol("eggride_seatsway", build, "")
				inst.AnimState:OverrideSymbol("ride_1eggride_seatsway", build, "")
				inst.AnimState:OverrideSymbol("egg", build, "")
	        end
        end 		

		if custome_fn then
		    custome_fn(inst)
		end
		
	    return inst
	end
	
	return Prefab("summerevent_"..name, normal_fn, assets, prefabs)

end

local function deployablekititem_ondeploy(inst, pt, deployer, rot)
    local structure = SpawnPrefab(inst._prefab_to_deploy)
    if structure ~= nil then
        structure.Transform:SetPosition(pt:Get())
		structure:PushEvent("onbuilt", { builder = deployer, pos = pt, rot = rot, deployable = inst })
        inst:Remove()
    end
end

local function IsPointOutsideInteriorWall(pt, radius)
    if TheCamera.interior then
        local interiorSpawner = GetWorld().components.interiorspawner 
        if GetWorld() and interiorSpawner and interiorSpawner.current_interior then
            local originpt = interiorSpawner:getSpawnOrigin()
            local width = interiorSpawner.current_interior.width
            local depth = interiorSpawner.current_interior.depth        
            local dMax = originpt.x + depth/2
            local dMin = originpt.x - depth/2
            local wMax = originpt.z + width/2
            local wMin = originpt.z - width/2        
            local dist = radius or 0.5
            if pt.x < dMin+dist or pt.x > dMax-dist or pt.z < wMin+dist or pt.z > wMax-dist then
                return true
            end
        end
    end
    return false
end

local function IsPassableAtPointAndIsDeployPointClear(inst, pt, atdeploy)
	if inst.IsPosSurroundedByLand ~= nil and not inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 2) then
	    return false
	end
    if not IsPointOutsideInteriorWall(pt, 1) and IsPassableAtPoint(pt.x, pt.y, pt.z) then
	    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 2, nil, {"NOBLOCK", "FX", "player"})
	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt) < 1*1 then
					return false
				end
			end
		end
		return true
	end
	return false
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)		
		inst.float_fx.Transform:SetScale(1.1, 1.1, 1.1)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.Transform:SetScale(1.1, 1.1, 1.1)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function MakeDeployableKitItem(kit_name, kit_bank, kit_build, prefab_to_deploy, kit_custome_fn)
	local kit_assets =
	{
		Asset("ANIM", "anim/"..kit_build..".zip"),
	}

	local function kit_normal_fn()	
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddAnimState()
		
		MakeInventoryPhysics(inst)

        inst.AnimState:SetBank(kit_bank)
        inst.AnimState:SetBuild(kit_build)
        inst.AnimState:PlayAnimation("kit_item")

   	    inst._prefab_to_deploy = prefab_to_deploy

	    inst:AddTag("deploykititem")
		inst:AddTag("eyeturret")  --For action string

        inst:AddComponent("inspectable")
		
		inst:AddComponent("inventoryitem")

		inst:AddComponent("stackable")
		inst.components.stackable.maxsize = TUNING.STACK_SIZE_LARGEITEM

		inst:AddComponent("deployable")
		inst.components.deployable.ondeploy = deployablekititem_ondeploy
	    inst.components.deployable.test = IsPassableAtPointAndIsDeployPointClear
	    inst.components.deployable.placer = prefab_to_deploy.."_placer"
    	inst.components.deployable.deploydistance = 0.5
	
	    MakeSmallBurnable(inst)
	    MakeSmallPropagator(inst)

		inst:AddComponent("fuel")
		inst.components.fuel.fuelvalue = TUNING.SMALL_FUEL

        if inst.GetIsOnWater ~= nil then
            inst:AddComponent("floatable")
	        inst.components.floatable:SetOnHitWaterFn(OnHitWater)
    	    inst.components.floatable:SetOnHitLandFn(OnHitLand)
	        inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
        end		

		if kit_custome_fn then
		    kit_custome_fn(inst)
		end
		
	    return inst
	end

    return Prefab("summerevent_"..kit_name.."_kit", kit_normal_fn, kit_assets, {prefab_to_deploy})
end

return MakeSummerEventStructure("eggride1", "carnivaldecor_eggride1", "carnivaldecor_eggride1"),
    MakeSummerEventStructure("eggride2", "carnivaldecor_eggride2", "carnivaldecor_eggride2"),
	MakeSummerEventStructure("eggride3", "carnivaldecor_eggride3", "carnivaldecor_eggride3"),
	MakeSummerEventStructure("eggride4", "carnivaldecor_eggride4", "carnivaldecor_eggride4"),
	MakeDeployableKitItem("eggride1", "carnivaldecor_eggride1", "carnivaldecor_eggride1", "summerevent_eggride1"),
	MakeDeployableKitItem("eggride2", "carnivaldecor_eggride2", "carnivaldecor_eggride2", "summerevent_eggride2"),
	MakeDeployableKitItem("eggride3", "carnivaldecor_eggride3", "carnivaldecor_eggride3", "summerevent_eggride3"),
	MakeDeployableKitItem("eggride4", "carnivaldecor_eggride4", "carnivaldecor_eggride4", "summerevent_eggride4"),
	MakePlacer("summerevent_eggride1_placer", "carnivaldecor_eggride1", "carnivaldecor_eggride1", "loop"),
	MakePlacer("summerevent_eggride2_placer", "carnivaldecor_eggride2", "carnivaldecor_eggride2", "loop"),
	MakePlacer("summerevent_eggride3_placer", "carnivaldecor_eggride3", "carnivaldecor_eggride3", "loop"),
	MakePlacer("summerevent_eggride4_placer", "carnivaldecor_eggride4", "carnivaldecor_eggride4", "loop")