local assets =
{
	Asset("ANIM", "anim/torch_shadow.zip"),
	Asset("ANIM", "anim/swap_torch_shadow.zip"),	
}

local prefabs =
{
	"nightlight_flame",
}    

local function updatedamage(inst)
    if GetWorld():IsCave() then
	    inst.components.weapon:SetDamage(TUNING.TORCH_DAMAGE*4)
	else
        if GetClock():IsDay() then
            inst.components.weapon:SetDamage(TUNING.TORCH_DAMAGE)
        elseif GetClock():IsNight() then
            inst.components.weapon:SetDamage(TUNING.TORCH_DAMAGE*3)
        elseif GetClock():IsDusk() then
            inst.components.weapon:SetDamage(TUNING.TORCH_DAMAGE*2)
		end
    end
end

local function UpdateFireFX(inst)
    if inst.components.burnable ~= nil and inst.components.burnable.fxchildren ~= nil then
	    local owner = inst.components.equippable ~= nil and inst.components.equippable.owner or nil
        for i,fx in ipairs(inst.components.burnable.fxchildren) do
	        if fx.Light ~= nil then
		        fx.Light:Enable(false)
		    end
			if fx.Follower ~= nil then 
                if owner ~= nil then
			        fx.Follower:FollowSymbol(owner.GUID, "swap_object", 0, -110, 1)
			    else
				    fx.Follower:FollowSymbol(inst.GUID, "art", 1, -110, 0)
				end
			end
			fx:AddTag("INTERIOR_LIMBO_IMMUNE")		
	    end
    end
end

local function onequip(inst, owner, force, ignoreanim) 
	local HasSWorHAM = IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC)
    if not (HasSWorHAM and owner.sg:HasStateTag("rowing")) or force then 

		if not inst.components.fueled:IsEmpty() then
		    inst.components.fueled:StartConsuming()
		    inst.components.burnable:Ignite()
			inst.Light:Enable(true)
		end
		
		if owner ~= nil and owner.AnimState ~= nil and not ignoreanim then
            owner.AnimState:OverrideSymbol("swap_object", "swap_torch_shadow", "swap_torch")
            owner.AnimState:Show("ARM_carry") 
            owner.AnimState:Hide("ARM_normal") 
		end
        
		inst:DoTaskInTime(0, UpdateFireFX)
		
        inst.SoundEmitter:PlaySound("dontstarve/wilson/torch_LP", "torch")
        inst.SoundEmitter:PlaySound("dontstarve/wilson/torch_swing")
        inst.SoundEmitter:SetParameter( "torch", "intensity", 0.5 )

        --take a percent of fuel next frame instead of this one, so we can remove the torch properly if it runs out at that point
    	inst:DoTaskInTime(0, function()
     	    if inst.components.fueled.currentfuel < inst.components.fueled.maxfuel and not (owner ~= nil and owner:HasTag("shadow_form_on")) then
    		    inst.components.fueled:DoDelta(-inst.components.fueled.maxfuel*0.01)
    	    end
    	end)
    end 
end

local function onunequip(inst, owner, ignoreanim) 
    inst.Light:Enable(false)
    inst.components.burnable:Extinguish()
	inst.components.fueled:StopConsuming()
	
	if owner ~= nil and owner.AnimState ~= nil and not ignoreanim then
        owner.AnimState:Hide("ARM_carry") 
        owner.AnimState:Show("ARM_normal")
	end
    inst.SoundEmitter:KillSound("torch")
    inst.SoundEmitter:PlaySound("dontstarve/common/fireOut")        
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx.Transform:SetScale(0.68, 0.68, 0.68)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetScale(0.68, 0.68, 0.68)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function SanityAura(inst, observer)
    if observer:HasTag("shadow_form_on") then
        return 0
    end
	
    if inst.components.inventoryitem ~= nil and inst.components.inventoryitem.owner == nil then
        return -TUNING.SANITYAURA_SMALL*0.125
    end
	
    return 0
end

local function ChangeIntoShadowFire(inst)
    if inst.components.burnable.fxchildren ~= nil then
        for i,fx in ipairs(inst.components.burnable.fxchildren) do
	        if fx.AnimState ~= nil then
			    fx.AnimState:SetMultColour(0, 0, 0, 0.6)
			end
	    end	
	end 
end

local function OnBurntChanceToDropExp(inst)
    local chance = 0.1
	if GetPlayer().superemberguru and GetPlayer().components.inventory ~= nil then
        local shadowboxes = GetPlayer().components.inventory:FindItems(function(item) 
		    if item ~= nil and item.prefab == "charlie_shadowbox" then
			    return true
			end
		    return false
		end) 
		if #shadowboxes > 0 then
		    for i=1, #shadowboxes do
			    chance = chance + 0.05 
				if chance >= 1 then
				    break
				end
			end
		end		    
	end 
	if math.random() < chance then  
        local x, y, z = inst.Transform:GetWorldPosition()
		if inst.components.lootdropper ~= nil then
		    local charlie_exp = SpawnPrefab("charlie_exp")
            inst.components.lootdropper:FlingItem(charlie_exp, Vector3(x, math.random()*4, z))		
			charlie_exp.AnimState:PlayAnimation("idle_pre")
			charlie_exp.AnimState:PushAnimation("idle_loop", true)	
		else
			local charlie_exp = SpawnPrefab("charlie_exp")
			charlie_exp.Transform:SetPosition(x, math.random()*4, z)
			charlie_exp.Physics:SetVel(math.random()*math.cos(math.random()*2*math.pi), GetRandomWithVariance(8, 4), math.random()*math.sin(math.random()*2*math.pi))
			charlie_exp.AnimState:PlayAnimation("idle_pre")
			charlie_exp.AnimState:PushAnimation("idle_loop", true)	
		end			
    end	
end

local function OnShadowTorchLight(inst, target)
    if target ~= nil and target.components.burnable ~= nil then
	    target:DoTaskInTime(0, ChangeIntoShadowFire)
	    local _onburnt = target.components.burnable.onburnt
		if _onburnt ~= nil then
		    target.components.burnable.onburnt = function(target)
			    _onburnt(target)
			    OnBurntChanceToDropExp(target)
		    end
		else
		    target.components.burnable.onburnt = function(target)
				OnBurntChanceToDropExp(target)
			end
		end
	end
end

local function on_inventory_thrown(inst)
    inst:AddTag("NOCLICK")
	if inst.components.inventoryitem ~= nil then 
	    inst.components.inventoryitem.canbepickedup = false 
	end 
    inst.AnimState:OverrideSymbol("art", "swap_torch_shadow", "swap_torch")
	inst.components.burnable:Ignite()
	inst.Light:Enable(true)
	inst:DoTaskInTime(0, UpdateFireFX)
end

local function on_inventory_hit(inst, attacker, target)
    local x, y, z = inst.Transform:GetWorldPosition()
    local impactfx = SpawnPrefab("impact")
    if impactfx ~= nil then
        impactfx.Transform:SetPosition(x, y, z)
		impactfx.Transform:SetScale(2, 2, 2)
		if attacker ~= nil then
            impactfx:FacePoint(attacker.Transform:GetWorldPosition())
		end
    end 
	if inst.SoundEmitter ~= nil then 
	    inst.SoundEmitter:PlaySound("dontstarve/common/destroy_magic")
	end 

    local shadow_acid_smokes = TheSim:FindEntities(x, y, z, 2, {"shadow_acid_smoke"})
	local puddles = TheSim:FindEntities(x, y, z, 3, {"puddle"})
	if #shadow_acid_smokes <= 0 and #puddles <= 0 and inst:IsOnValidGround() then
		if inst.IsPosSurroundedByLand == nil or inst:IsPosSurroundedByLand(x, y, z, 3) then
			local shadow_acid_smoke = SpawnPrefab("shadow_acid_smoke")
			shadow_acid_smoke.Transform:SetPosition(x, y, z)
		    shadow_acid_smoke:SetLifeTime(120)
		end		
	end		
	
	if inst.components.fueled ~= nil then
        inst.components.fueled:DoDelta(-inst.components.fueled.maxfuel*0.01)
    end	
	if inst.components.inventoryitem ~= nil then 
	    inst.components.inventoryitem.canbepickedup = true  
	end 	
    inst:RemoveTag("NOCLICK")
	inst.Light:Enable(false)
	inst.components.burnable:Extinguish()
    inst.AnimState:OverrideSymbol("art", "torch_shadow", "art")
	inst:DoTaskInTime(0, function()
        if inst.components.floatable ~= nil then 
		    inst.components.floatable:SetAnimationFromPosition() 
		end 	
	end)
end

local function returntointeriorscene(inst)
	if inst.components.equippable and inst.components.equippable:IsEquipped() then
		inst.Light:Enable(true)
	else
		inst.Light:Enable(false)
	end
end

local function fn(Sim)
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	inst.entity:AddLight()
	
	inst.AnimState:SetBank("torch")
    inst.AnimState:SetBuild("torch_shadow")
    inst.AnimState:PlayAnimation("idle")
	
    MakeInventoryPhysics(inst)
	
	inst:AddTag("shadow")
	inst:AddTag("shadow_item")
	inst:AddTag("shadow_torch")
  
    inst.Light:Enable(false)
    inst.Light:SetIntensity(0.6)
    inst.Light:SetColour(128/255, 0/255, 128/255)
    inst.Light:SetFalloff(0.5)
    inst.Light:SetRadius(6)
  
    inst:AddComponent("inventoryitem")
    
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("floatable")		
		inst.components.floatable:SetOnHitWaterFn(OnHitWater)
		inst.components.floatable:SetOnHitLandFn(OnHitLand)
		inst:ListenForEvent("onputininventory", OnHitLand)
        inst:ListenForEvent("startrowing", function(inst,data) 
            onunequip(inst, data.owner)
        end, inst)  

        inst:ListenForEvent("stoprowing", function(inst, data) 
            onequip(inst, data.owner, true)
        end, inst)

		inst:AddComponent("appeasement")
        inst.components.appeasement.appeasementvalue = TUNING.APPEASEMENT_SMALL
    end
    
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(TUNING.TORCH_DAMAGE)
    inst.components.weapon:SetAttackCallback(
        function(inst, attacker, target)

            if inst.components.fueled and inst.components.fueled:IsEmpty() then
	            inst:Remove()
	            return
	        end
			
		    if target ~= nil and target.Transform ~= nil and target:IsValid() then
	            SpawnPrefab("statue_transition").Transform:SetPosition(target.Transform:GetWorldPosition())
                SpawnPrefab("statue_transition_2").Transform:SetPosition(target.Transform:GetWorldPosition())
				if target.components.combat ~= nil and not target:HasTag("buttercup") then
				    target.components.combat:SetTarget(nil)
					if target:HasTag("shadow") then
     	                if inst.components.fueled ~= nil 
			                and inst.components.fueled.currentfuel < inst.components.fueled.maxfuel then
    		                inst.components.fueled:DoDelta(inst.components.fueled.maxfuel*0.01)
    	                end					
					end
				end
            end	
			
        end
    )
	inst.components.weapon:SetRange(2)
    
	inst:ListenForEvent("dusktime", function() updatedamage(inst) end, GetWorld())
    inst:ListenForEvent("daytime", function() updatedamage(inst) end, GetWorld())
    inst:ListenForEvent("nighttime", function() updatedamage(inst) end, GetWorld())
	inst:ListenForEvent("arrive", function() updatedamage(inst) end, GetWorld())
    updatedamage(inst)

    inst:AddComponent("equippable")
    inst.components.equippable:SetOnPocket(function(owner) 
	    inst.components.burnable:Extinguish()
		inst.Light:Enable(false) 
		inst.components.fueled:StopConsuming()
	end)    
	
    inst.components.equippable:SetOnEquip(onequip)    
    inst.components.equippable:SetOnUnequip(onunequip)

    if IsDLCEnabled(REIGN_OF_GIANTS) or IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
        inst:AddComponent("waterproofer")
	    inst.components.waterproofer:SetEffectiveness(TUNING.WATERPROOFNESS_SMALL)
		inst.components.equippable.dapperfn = function(inst, owner)
		    return owner ~= nil and owner:HasTag("charlie") and 0 or TUNING.CRAZINESS_SMALL/2
		end
	else
	    inst:AddComponent("dapperness")
        inst.components.dapperness.dapperfn = function(inst, owner) 
		    return owner ~= nil and owner:HasTag("charlie") and 0 or TUNING.CRAZINESS_SMALL/2
		end
	end
    
    inst:AddComponent("inspectable")
	
	inst:AddComponent("lighter")
	inst.components.lighter:SetOnLightFn(OnShadowTorchLight)
	inst.components.lighter.CollectUseActions = function() end 
	
	inst:AddComponent("shadowtorchspecific")
    inst.components.shadowtorchspecific:SetHorizontalSpeed(16)
    inst.components.shadowtorchspecific:SetGravity(-10)
    inst.components.shadowtorchspecific:SetLaunchOffset(Vector3(0.25, 1, 0))
    inst.components.shadowtorchspecific:SetOnLaunch(on_inventory_thrown)
    inst.components.shadowtorchspecific:SetOnHit(on_inventory_hit)
	
    inst:AddComponent("burnable")
    inst.components.burnable.canlight = false
    inst.components.burnable.fxprefab = nil
	inst.components.burnable:AddBurnFX("nightlight_flame", Vector3(0, 0, 0), "swap_torch_shadow")
 
    inst:DoTaskInTime(0, function()
        if inst:HasTag("lightsource") then       
            inst:RemoveTag("lightsource")    
        end
    end)

	inst:ListenForEvent("ondropped", function(inst)
	    inst.Light:Enable(false)
	end)
 
    inst:AddComponent("fueled")
    inst.components.fueled.accepting = true
    inst.components.fueled.fueltype = "NIGHTMARE"   
    inst.components.fueled:SetUpdateFn(function()
	    local owner = inst.components.equippable ~= nil and inst.components.equippable.owner or nil
        local rate = owner ~= nil and owner:HasTag("shadow_form_on") and -0.25 or 1
        inst.components.fueled.rate = rate
    end)

    inst.components.fueled:SetSectionCallback(
        function(section)
            if section == 0 then
			    if inst.components.equippable ~= nil 
		            and inst.components.equippable:IsEquipped() then
			        onunequip(inst, inst.components.equippable.owner, true)						
				end
	            inst:RemoveTag("shadow")
	            inst:RemoveTag("shadow_item")
				inst:RemoveTag("shadow_torch")
                if inst.components.inventoryitem and inst.components.inventoryitem:IsHeld() then
                    local owner = inst.components.inventoryitem and inst.components.inventoryitem.owner or nil                 
                    if owner then
                        owner:PushEvent("torchranout", {torch = inst})
                    end
                end
			else
	            inst:AddTag("shadow")
	            inst:AddTag("shadow_item")
				inst:AddTag("shadow_torch")
			    if inst.components.equippable ~= nil 
		            and inst.components.equippable:IsEquipped() then
			        onequip(inst, inst.components.equippable.owner, nil, true)						
				end
            end
        end)
    inst.components.fueled:InitializeFuelLevel(TUNING.TORCH_FUEL*3)
    inst.components.fueled:SetDepletedFn(function(inst)  
		inst.components.burnable:Extinguish()
        inst.Light:Enable(false)			
	end)  
	
    inst.returntointeriorscene = returntointeriorscene
	
	inst:AddComponent("sanityaura") 
	inst.components.sanityaura.aurafn = SanityAura

    inst:AddComponent("fuel")
    inst.components.fuel.fueltype = "NIGHTMARE"
    inst.components.fuel.fuelvalue = TUNING.LARGE_FUEL
	
    return inst   
end   

return Prefab("torch_shadow", fn, assets, prefabs)