require("prefabutil")

local assets =
{
    Asset("ANIM", "anim/townportal.zip"),
}

local fx_assets =
{
    Asset("ANIM", "anim/teleport_sand_fx.zip"),
    Asset("ANIM", "anim/sand_splash_fx.zip"),
}

local prefabs =
{
    "collapse_small",
    "townportalsandcoffin_fx",
}

local function OnEntityWake(inst)
    if inst.playingsound and not (inst:IsAsleep() or inst.SoundEmitter:PlayingSound("active")) then
        --inst.SoundEmitter:PlaySound("dontstarve/common/together/town_portal/talisman_active", "active")
    end
end

local function OnEntitySleep(inst)
    inst.SoundEmitter:KillSound("active")
end

local function StartSoundLoop(inst)
    if not inst.playingsound then
        inst.playingsound = true
        OnEntityWake(inst)
    end
end

local function StopSoundLoop(inst)
    if inst.playingsound then
        inst.playingsound = nil
        inst.SoundEmitter:KillSound("active")
    end
end

local function teleport_thread(inst, doer, target)
    SpawnPrefab("townportalsandcoffin_fx").Transform:SetPosition(doer.Transform:GetWorldPosition())
    if doer.components.health ~= nil then
        doer.components.health:SetInvincible(true)
    end
	doer:Hide()
    if doer == GetPlayer() then
        TheFrontEnd:Fade(false, 2)
        Sleep(3)
    end	
    if doer.components.sanity ~= nil then
        doer.components.sanity:DoDelta(-TUNING.SANITY_HUGE)
    end	
	if doer.Teleport ~= nil then
	    doer:Teleport(target)
	else
	    local x, y, z = target.Transform:GetWorldPosition()
        doer.Transform:SetPosition(x, 0, z)
        if doer == GetPlayer() then
            TheCamera:Snap()
            TheFrontEnd:DoFadeIn(1)
            Sleep(1)
        end	
	end
	SpawnPrefab("townportalsandcoffin_fx").Transform:SetPosition(doer.Transform:GetWorldPosition())
	doer:Show()
    if doer.components.health ~= nil then
        doer.components.health:SetInvincible(false)
    end
    if doer == GetPlayer() then
	    if doer.sg ~= nil then
            doer.sg:GoToState("wakeup")
		end
		if doer.SoundEmitter ~= nil then
            doer.SoundEmitter:PlaySound("dontstarve/common/staffteleport")
		end
    end
	if target:HasTag("linked_townportal") then 
	    target:RemoveTag("linked_townportal")
		if target.MiniMapEntity ~= nil then
		    target.MiniMapEntity:SetIcon("townportal.tex")
		end
		if target.AnimState ~= nil then
		    target.AnimState:PlayAnimation("turn_off")
            target.AnimState:PushAnimation("idle_off")
		end
	end
end

local function OnActivate(inst, doer)
    if not inst:HasTag("linked_townportal") then 
        inst:AddTag("linked_townportal")
	    inst.components.activatable.inactive = false
		inst:DoTaskInTime(1, function() inst.components.activatable.inactive = true end)
        inst.AnimState:PlayAnimation("turn_on")
        inst.AnimState:PushAnimation("idle_on_loop")	
	    inst.MiniMapEntity:SetIcon("townportal_activated.tex")
	elseif doer ~= nil then
	    inst:RemoveTag("linked_townportal")
	    inst.components.activatable.inactive = false
		inst:DoTaskInTime(1, function() inst.components.activatable.inactive = true end)
        inst.AnimState:PlayAnimation("turn_off")
        inst.AnimState:PushAnimation("idle_off")		
		inst.MiniMapEntity:SetIcon("townportal.tex")
	    local x, y, z = doer.Transform:GetWorldPosition()
		local ents = TheSim:FindEntities(x, y, z, 9000, {"linked_townportal"})
		local target = nil
		for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() then
				target = v
			end
		end
		if target ~= nil then
		    inst.task = inst:StartThread(function() teleport_thread(inst, doer, target) end)
		else
		
		end
	end
end

local function onhammered(inst)
    inst.components.lootdropper:DropLoot()
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
	fx.SoundEmitter:PlaySound("dontstarve/common/destroy_metal")
    inst:Remove()
end

local function onhit(inst)
    if inst:HasTag("linked_townportal") then
        inst:RemoveTag("linked_townportal")
		inst.MiniMapEntity:SetIcon("townportal.tex")
		inst.components.activatable.inactive = true
        inst.AnimState:PlayAnimation("hit_on")
    else
        inst.AnimState:PlayAnimation("hit_off")
    end
    inst.AnimState:PushAnimation("idle_off")
end

local function onbuilt(inst)
    --inst.SoundEmitter:PlaySound("dontstarve/common/together/town_portal/craft")
    inst.AnimState:PlayAnimation("place")
    inst.AnimState:PushAnimation("idle_off")
end

local function GetStatus(inst)
    return inst:HasTag("linked_townportal")
        and "ACTIVE"
        or nil
end

local function onsave(inst, data)
    data.linked_townportal = inst:HasTag("linked_townportal") or nil
end

local function onload(inst, data)
    if data ~= nil then
	    if data.linked_townportal then
            inst:AddTag("linked_townportal")
	        inst.components.activatable.inactive = true
            inst.AnimState:PlayAnimation("idle_on_loop", true)	
	        inst.MiniMapEntity:SetIcon("townportal_activated.tex")			
	    end
    end	
end

local function OnHitWater(inst)
	if inst.float_fx == nil then
		inst.float_fx = SpawnPrefab("float_fx_front")
		inst.float_fx.entity:SetParent(inst.entity)
		inst.float_fx.AnimState:PlayAnimation("idle_front_med", true)		
		inst.float_fx.Transform:SetScale(0.72, 0.72, 0.72)
	end
	if inst.float_fx2 == nil then
		inst.float_fx2 = SpawnPrefab("float_fx_back")
		inst.float_fx2.entity:SetParent(inst.entity)
		inst.float_fx2.AnimState:PlayAnimation("idle_back_med", true)
		inst.float_fx2.Transform:SetScale(0.72, 0.72, 0.72)
	end	
end

local function OnHitLand(inst)
	if inst.float_fx ~= nil then
		inst.float_fx:Remove()
		inst.float_fx = nil
	end
	if inst.float_fx2 ~= nil then
		inst.float_fx2:Remove()
		inst.float_fx2 = nil
	end	
end

local function ondeploy(inst, pt, deployer)
    local townportal = SpawnPrefab("townportal") 
    if townportal then 
        pt = Vector3(pt.x, 0, pt.z)
        townportal.Physics:SetCollides(false)
        townportal.Physics:Teleport(pt.x, pt.y, pt.z) 
        townportal.Physics:SetCollides(true)
        townportal.AnimState:PlayAnimation("place")
        townportal.AnimState:PushAnimation("idle_off")
        townportal.SoundEmitter:PlaySound("dontstarve/common/place_structure_stone")
        inst:Remove()
    end 
end

local function IsPointNearTownPortal(pt, range)
    for i, v in ipairs( TheSim:FindEntities(pt.x, 0, pt.z, range, {"townportal"}) ) do
        if v:GetDistanceSqToPoint(pt) < range * range then
            return true
        end
    end
    return false
end

local function IsPointOutsideInteriorWall(pt, radius)
    if TheCamera.interior then
        local interiorSpawner = GetWorld().components.interiorspawner 
        if GetWorld() and interiorSpawner and interiorSpawner.current_interior then
            local originpt = interiorSpawner:getSpawnOrigin()
            local width = interiorSpawner.current_interior.width
            local depth = interiorSpawner.current_interior.depth        
            local dMax = originpt.x + depth/2
            local dMin = originpt.x - depth/2
            local wMax = originpt.z + width/2
            local wMin = originpt.z - width/2        
            local dist = radius or 0.5
            if pt.x < dMin+dist or pt.x > dMax-dist or pt.z < wMin+dist or pt.z > wMax-dist then
                return true
            end
        end
    end
    return false
end

local function deploytest(inst, pt)
	if inst.IsPosSurroundedByLand ~= nil then
	    if not inst:IsPosSurroundedByLand(pt.x, pt.y, pt.z, 3) then
		    return false
		end
	end
	local radius = 2	
    for i = -radius, radius, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x - radius, pt.y, pt.z + i) == GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(pt.x + radius, pt.y, pt.z + i) == GROUND.IMPASSABLE then
            return false
        end
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z - radius) == GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(pt.x + i, pt.y, pt.z + radius) == GROUND.IMPASSABLE then
            return false
        end
    end			
	
	if not IsPointNearTownPortal(pt, 3) and not IsPointOutsideInteriorWall(pt, 1) then
	    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 6, nil, {"NOBLOCK", "player", "FX"})
		local min_spacing = inst.components.deployable.min_spacing or 2
	    for k, v in pairs(ents) do
			if v ~= inst and v:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt ) < min_spacing*min_spacing then
					return false
				end
			end
		end
		return true
	end
	return false
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()

    inst.MiniMapEntity:SetIcon("townportal.tex")

    MakeObstaclePhysics(inst, .1)

    inst.AnimState:SetBank("townportal")
    inst.AnimState:SetBuild("townportal")
    inst.AnimState:PlayAnimation("idle_off", true)
    inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")

    inst:AddTag("structure")
    inst:AddTag("townportal")

    -----------------------
    MakeSnowCovered(inst)

    -------------------------
    inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"cutstone","cutstone"})
	
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
    inst.components.workable:SetOnFinishCallback(onhammered)
    inst.components.workable:SetOnWorkCallback(onhit)

	inst:AddComponent("activatable")
    inst.components.activatable.OnActivate = OnActivate
    inst.components.activatable.inactive = true
	inst.components.activatable.standingaction = true

    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = GetStatus

    -----------------------------
    --inst:ListenForEvent("onbuilt", onbuilt)

    inst.OnSave = onsave
    inst.OnLoad = onload 

    return inst
end

local function KillFX(inst)
    if inst.killtask ~= nil then
        inst.killtask:Cancel()
        inst.killtask = nil
        inst.Physics:SetActive(false)
        --inst.SoundEmitter:PlaySound("dontstarve/common/together/teleport_sand/out")
        inst.AnimState:PlayAnimation("portal_out")
        inst:DoTaskInTime(inst.AnimState:GetCurrentAnimationLength() + .5, inst.Remove)
    end
end

local function fx_fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    MakeObstaclePhysics(inst, .5)

    inst.AnimState:SetBank("teleport_sand_fx")
    inst.AnimState:SetBuild("teleport_sand_fx")
    inst.AnimState:OverrideSymbol("sand_splash", "sand_splash_fx", "sand_splash")
    inst.AnimState:PlayAnimation("portal_in")
	inst.AnimState:SetFinalOffset(7)

    inst:AddTag("NOCLICK")
	inst:AddTag("NOBLOCK")
    inst:AddTag("FX")

    --inst.SoundEmitter:PlaySound("dontstarve/common/together/teleport_sand/in")

    inst.persists = false
    inst.KillFX = KillFX
    inst.killtask = inst:DoTaskInTime(35 * FRAMES, KillFX)

    return inst
end

local function fn_item()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("townportal")
    inst.AnimState:SetBuild("townportal")
    inst.AnimState:PlayAnimation("idle")
    inst.AnimState:SetScale(0.75, 0.75, 0.75)

    inst:AddTag("eyeturret")  --For action string

    inst:AddComponent("inspectable")

    inst:AddComponent("inventoryitem")

    if inst.GetIsOnWater ~= nil then
        inst:AddComponent("floatable")
	    inst.components.floatable:SetOnHitWaterFn(OnHitWater)
    	inst.components.floatable:SetOnHitLandFn(OnHitLand)
	    inst.components.inventoryitem:SetOnPutInInventoryFn(OnHitLand)
    end	

    inst:AddComponent("deployable")
    inst.components.deployable.ondeploy = ondeploy
	inst.components.deployable.test = deploytest
    inst.components.deployable.placer = "townportal_placer"
    inst.components.deployable.deploydistance = 1.5
    inst.components.deployable.min_spacing = 2

    return inst
end



return Prefab("townportal", fn, assets, prefabs),
    MakePlacer("townportal_placer", "townportal", "townportal", "idle"),
    Prefab("townportalsandcoffin_fx", fx_fn, fx_assets),
    Prefab("townportal_item", fn_item, assets, prefabs) --All ancient tabs things keep items.
