--Year of the Pig Food--

local assets =
{
    Asset("ANIM", "anim/yotp_food.zip"),
    Asset("ANIM", "anim/yotp_food_water.zip"),
}

local foodinfo =
{
    {
        name = "tribute roast",
	    foodtype = "MEAT",
		atlasname = "images/inventoryimages/bananabush_dug.xml",
		imagename = "bananabush_dug",
		hunger = TUNING.CALORIES_SUPERHUGE,			
		health = TUNING.HEALING_SMALL*4,	
		sanity = TUNING.SANITY_TINY,		
		perishtime = TUNING.PERISH_SLOW,    
		floater = {"med", 0.9, false}
	},     -- tribute roast (same tuning as bonestew)
    {
        name = "mud pie",
	    foodtype = "HORRIBLE",	
		atlasname = "images/inventoryimages/bananabush_dug.xml",
		imagename = "bananabush_dug",
		hunger = TUNING.CALORIES_SUPERHUGE,			
		health = 0,							
		sanity = 0
	},     -- mud pie
    {
        name = "fish head skewers",
	    foodtype = "MEAT",		
		atlasname = "images/inventoryimages/bananabush_dug.xml",
		imagename = "bananabush_dug",
		hunger = TUNING.CALORIES_HUGE,				
		health = TUNING.HEALING_SMALL*2,	
		sanity = TUNING.SANITY_SUPERTINY,	
		perishtime = TUNING.PERISH_SLOW,    
		floater = {"med", {0.8, 0.5, 0.8}, true}
	},    -- fish head skewers
}

local function MakeFood(num)
	local data = foodinfo[num]

    local function OnHitWater(inst, size, vert_offset, x_scale, y_scale, z_scale)
	    if data.name == "fish head skewers" then
		    inst.AnimState:PlayAnimation("food3_float")
		end
	    if inst.float_fx == nil then
		    inst.float_fx = SpawnPrefab("float_fx_front")
		    inst.float_fx.entity:SetParent(inst.entity)
		    if size ~= nil then
		        inst.float_fx.AnimState:PlayAnimation("idle_front_"..size, true)
		    end
		    if vert_offset ~= nil then
		        inst.float_fx.Transform:SetPosition(0, vert_offset, 0)
		    end
		    if x_scale ~= nil then
                inst.float_fx.Transform:SetScale(x_scale, y_scale or x_scale, z_scale or x_scale)
		    end			
	    end
	    if inst.float_fx2 == nil then
		    inst.float_fx2 = SpawnPrefab("float_fx_back")
		    inst.float_fx2.entity:SetParent(inst.entity)
		    if size ~= nil then
		        inst.float_fx2.AnimState:PlayAnimation("idle_back_"..size, true)
		    end
		    if vert_offset ~= nil then
		        inst.float_fx2.Transform:SetPosition(0, vert_offset, 0)
		    end
            if x_scale ~= nil then
		        inst.float_fx2.Transform:SetScale(x_scale, y_scale or x_scale, z_scale or x_scale)
		    end				
	    end	
    end

    local function OnHitLand(inst)
	    if data.name == "fish head skewers" then
		    inst.AnimState:PlayAnimation("food3")
		end
	    if inst.float_fx ~= nil then
		    inst.float_fx:Remove()
		     inst.float_fx = nil
	    end
	    if inst.float_fx2 ~= nil then
		    inst.float_fx2:Remove()
		    inst.float_fx2 = nil
	    end	
    end

    local function fn()
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddAnimState()
        inst.entity:AddSoundEmitter()

        MakeInventoryPhysics(inst)

        inst.AnimState:SetBank("yotp_food")
        inst.AnimState:SetBuild("yotp_food")

        local anim_name = "food"..tostring(num)
        inst.AnimState:PlayAnimation(anim_name)

        --[[if data.tags ~= nil then
            for _,v in ipairs(data.tags) do
                inst:AddTag(v)
            end
        end]]
		
        inst:AddTag("pre-preparedfood")

        inst:AddComponent("edible")
        inst.components.edible.foodtype = data.foodtype
        inst.components.edible.hungervalue = data.hunger
        inst.components.edible.healthvalue = data.health
        inst.components.edible.sanityvalue = data.sanity
        --inst.components.edible.temperaturedelta = data.temperature or 0
        --inst.components.edible.temperatureduration = data.temperatureduration or 0

		if data.perishtime ~= nil and data.perishtime > 0 then
			inst:AddComponent("perishable")
			inst.components.perishable:SetPerishTime(data.perishtime)
			inst.components.perishable:StartPerishing()
			inst.components.perishable.onperishreplacement = "spoiled_food"
		end

        inst:AddComponent("stackable")
        inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

        inst:AddComponent("tradable")
        inst:AddComponent("inspectable")
        inst:AddComponent("inventoryitem")
	    inst.components.inventoryitem.atlasname = data.atlasname
        inst.components.inventoryitem.imagename = data.imagename

        if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
	        inst:AddComponent("floatable")
		    inst.components.floatable:SetOnHitWaterFn(function(inst)
			    if data.name == "tribute roast" then
			        OnHitWater(inst, "med", 0.9)
				elseif data.name == "fish head skewers" then
				    OnHitWater(inst, "med", 0.8, 0.5, 0.8)
				else
				    OnHitWater(inst)
				end
			end)
		    inst.components.floatable:SetOnHitLandFn(OnHitLand)
            inst:ListenForEvent("onputininventory", OnHitLand)
        end

        return inst
    end

    return Prefab("yotp_food"..num, fn, assets)
end

local ret = {}
for k = 1, #foodinfo do
    table.insert(ret, MakeFood(k))
end

return unpack(ret)
