local Screen = require "widgets/screen"
local Button = require "widgets/button"
local AnimButton = require "widgets/animbutton"
local ImageButton = require "widgets/imagebutton"
local Menu = require "widgets/menu"
local Text = require "widgets/text"
local Image = require "widgets/image"
local UIAnim = require "widgets/uianim"
local Widget = require "widgets/widget"
local PopupDialogScreen = require "screens/popupdialog"

--The c_countprefabs function does not support for stackable items count...
function CharlieRefreshModCountPrefabs(prefab, printonconsole)
    local count = 0
    local asleep = 0
    for k,v in pairs(Ents) do
        if v.prefab == prefab then
		    if v.components.stackable ~= nil then
			    count = count + v.components.stackable:StackSize()
                if v:IsAsleep() then
                    asleep = asleep + v.components.stackable:StackSize()
                end					
			else
                count = count + 1
                if v:IsAsleep() then
                    asleep = asleep + 1
                end				
			end
        end
    end
	if printonconsole then
        print(string.format("There are %d %s's in the world (%d asleep).", count, prefab, asleep))
	end
    return count
end

function CharlieRefreshModRemovePrefabs(prefab, number, printonconsole)
    local num = number or 1
	local count = 0
	local remain = 1
	for _, ent in pairs(Ents) do
        if ent.prefab == prefab then
			if count < num then
			    if ent.components.stackable ~= nil then
				    remain = num - count
					if remain < ent.components.stackable:StackSize() then
			            ent.components.stackable:Get(remain):Remove()
						count = count + remain
					else
					    ent:Remove()
						count = count + ent.components.stackable:StackSize() 
					end			    
	            else
			        ent:Remove()
				    count = count + 1	
				end
			else
                break
            end
         end										    
	end	
	if printonconsole then
	    if count <= 0 then
		    print("No entities removed.")
		elseif count == 1 then
		    print(count, " entity removed.")
		else
	        print(count, " entities removed.")
		end
	end
end

local CharlieSkillTreeScreen = Class(Screen, function(self)
	Screen._ctor(self, "CharlieSkillTreeScreen")
	
	self.active = true
	
	--There is a problem that the SetPause function cannot be used, as it will make shadow and lunar anims not to be played correctly.
	--SetPause(true)
	
	self.CurrentPage = 1
	self.MaxPage = 3
	
    self.black = self:AddChild(Image("images/global.xml", "square.tex"))
    self.black:SetVRegPoint(ANCHOR_MIDDLE)
    self.black:SetHRegPoint(ANCHOR_MIDDLE)
    self.black:SetVAnchor(ANCHOR_MIDDLE)
    self.black:SetHAnchor(ANCHOR_MIDDLE)
    self.black:SetScaleMode(SCALEMODE_FILLSCREEN)
	self.black:SetTint(0.5, 0.5, 0.5, 0.5)	

	self.proot = self:AddChild(Widget("ROOT"))
    self.proot:SetVAnchor(ANCHOR_MIDDLE)
    self.proot:SetHAnchor(ANCHOR_MIDDLE)
    self.proot:SetPosition(0, 0, 0)
    self.proot:SetScaleMode(SCALEMODE_PROPORTIONAL)

    self.bg = self.proot:AddChild(Image("images/globalpanels.xml", "panel_customization.tex"))
    self.bg:SetVRegPoint(ANCHOR_MIDDLE)
    self.bg:SetHRegPoint(ANCHOR_MIDDLE)
	self.bg:SetScale(0.8, 0.8, 0.8)
	self.bg:SetTint(1, 1, 1, 1)
	
    self.title = self.proot:AddChild(Text(TITLEFONT, 50))
    self.title:SetPosition(0, 225, 0)
    self.title:SetString(STRINGS.UI.CHARLIESKILLTREE.TITLE1)

    self.text1 = self.proot:AddChild(Text(BODYTEXTFONT, 30))
    self.text1:SetVAlign(ANCHOR_TOP)
    self.text1:SetPosition(-125, 0, 0)
    self.text1:SetString("")
    self.text1:EnableWordWrap(true)
    self.text1:SetRegionSize(750, 400)

    self.text2 = self.proot:AddChild(Text(BODYTEXTFONT, 30))
    self.text2:SetVAlign(ANCHOR_TOP)
    self.text2:SetPosition(125, 0, 0)
    self.text2:SetString("")
    self.text2:EnableWordWrap(true)
    self.text2:SetRegionSize(750, 400)
	
    self.text3 = self.proot:AddChild(Text(BODYTEXTFONT, 30))
    self.text3:SetVAlign(ANCHOR_TOP)
    self.text3:SetPosition(-125, -220, 0)
    self.text3:SetString("")
    self.text3:EnableWordWrap(true)
    self.text3:SetRegionSize(750, 400)
	
    self.text4 = self.proot:AddChild(Text(BODYTEXTFONT, 30))
    self.text4:SetVAlign(ANCHOR_TOP)
    self.text4:SetPosition(125, -220, 0)
    self.text4:SetString("")
    self.text4:EnableWordWrap(true)
    self.text4:SetRegionSize(750, 400)
	
    self.shadowtext1 = self.proot:AddChild(Text(BODYTEXTFONT, 30))
    self.shadowtext1:SetVAlign(ANCHOR_TOP)
    self.shadowtext1:SetPosition(-125, 0, 0)
    self.shadowtext1:SetString("")
    self.shadowtext1:EnableWordWrap(true)
    self.shadowtext1:SetRegionSize(750, 400)
	
    self.shadowtext2 = self.proot:AddChild(Text(BODYTEXTFONT, 30))
    self.shadowtext2:SetVAlign(ANCHOR_TOP)
    self.shadowtext2:SetPosition(125, 0, 0)
    self.shadowtext2:SetString("")
    self.shadowtext2:EnableWordWrap(true)
    self.shadowtext2:SetRegionSize(750, 400)

    self.lunartext1 = self.proot:AddChild(Text(BODYTEXTFONT, 30))
    self.lunartext1:SetVAlign(ANCHOR_TOP)
    self.lunartext1:SetPosition(-125, 0, 0)
    self.lunartext1:SetString("")
    self.lunartext1:EnableWordWrap(true)
    self.lunartext1:SetRegionSize(750, 400)

    self.lunartext2 = self.proot:AddChild(Text(BODYTEXTFONT, 30))
    self.lunartext2:SetVAlign(ANCHOR_TOP)
    self.lunartext2:SetPosition(125, 0, 0)
    self.lunartext2:SetString("")
    self.lunartext2:EnableWordWrap(true)
    self.lunartext2:SetRegionSize(750, 400)	

	self.dividertext = self.proot:AddChild(Text(BODYTEXTFONT, 60))
    self.dividertext:SetVAlign(ANCHOR_TOP)
    self.dividertext:SetPosition(5, -155, 0)
    self.dividertext:SetString("")
    self.dividertext:EnableWordWrap(true)
    self.dividertext:SetRegionSize(750, 400)
	self.dividertext:SetColour(0, 0, 0, 1)
	
	self.flowerimage1 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.flowerimage1:SetPosition(-175, 50, 0)   
	self.flowerimage1:SetScale(0.5, 0.5, 0.5)  
	
	self.flowerimage2 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.flowerimage2:SetPosition(-75, 50, 0)  
    self.flowerimage2:SetScale(0.5, 0.5, 0.5)	

	self.flowerimage3 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.flowerimage3:SetPosition(-175, 125, 0)   
	self.flowerimage3:SetScale(0.5, 0.5, 0.5)

	self.flowerimage4 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.flowerimage4:SetPosition(-75, 125, 0)   
	self.flowerimage4:SetScale(0.5, 0.5, 0.5)

	self.flowericonimage1 = self.proot:AddChild(Image("images/charlie_flowerskill1.xml", "charlie_flowerskill1.tex"))
	self.flowericonimage1:SetPosition(-175, 125, 0)  
	self.flowericonimage1:SetScale(0.5, 0.5, 0.5)

	self.flowericonimage2 = self.proot:AddChild(Image("images/charlie_flowerskill2.xml", "charlie_flowerskill2.tex"))
	self.flowericonimage2:SetPosition(-75, 125, 0)  
	self.flowericonimage2:SetScale(0.5, 0.5, 0.5)
	
	self.flowericonimage3 = self.proot:AddChild(Image("images/charlie_flowerskill3.xml", "charlie_flowerskill3.tex"))
	self.flowericonimage3:SetPosition(-175, 50, 0)
    self.flowericonimage3:SetScale(0.5, 0.5, 0.5)	
	
	self.flowericonimage4 = self.proot:AddChild(Image("images/charlie_flowerskill4.xml", "charlie_flowerskill4.tex"))
	self.flowericonimage4:SetPosition(-75, 50, 0)  
	self.flowericonimage4:SetScale(0.5, 0.5, 0.5)
	
	self.critterimage1 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.critterimage1:SetPosition(175, 50, 0)   
	self.critterimage1:SetScale(0.5, 0.5, 0.5)  
	
	self.critterimage2 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.critterimage2:SetPosition(75, 50, 0)  
    self.critterimage2:SetScale(0.5, 0.5, 0.5)	

	self.critterimage3 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.critterimage3:SetPosition(175, 125, 0)   
	self.critterimage3:SetScale(0.5, 0.5, 0.5)

	self.critterimage4 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.critterimage4:SetPosition(75, 125, 0)   
	self.critterimage4:SetScale(0.5, 0.5, 0.5)	
	
	self.crittericonimage1 = self.proot:AddChild(Image("images/charlie_critterskill1.xml", "charlie_critterskill1.tex"))
	self.crittericonimage1:SetPosition(75, 125, 0)  
	self.crittericonimage1:SetScale(0.5, 0.5, 0.5)

	self.crittericonimage2 = self.proot:AddChild(Image("images/charlie_critterskill2.xml", "charlie_critterskill2.tex"))
	self.crittericonimage2:SetPosition(175, 125, 0)  
	self.crittericonimage2:SetScale(0.5, 0.5, 0.5)
	
	self.crittericonimage3 = self.proot:AddChild(Image("images/charlie_critterskill3.xml", "charlie_critterskill3.tex"))
	self.crittericonimage3:SetPosition(75, 50, 0)
    self.crittericonimage3:SetScale(0.5, 0.5, 0.5)	
	
	self.crittericonimage4 = self.proot:AddChild(Image("images/charlie_critterskill4.xml", "charlie_critterskill4.tex"))
	self.crittericonimage4:SetPosition(175, 50, 0)  
	self.crittericonimage4:SetScale(0.5, 0.5, 0.5)	
	
	self.combatimage1 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.combatimage1:SetPosition(-175, -100, 0)   
	self.combatimage1:SetScale(0.5, 0.5, 0.5)  
	
	self.combatimage2 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.combatimage2:SetPosition(-75, -100, 0)  
    self.combatimage2:SetScale(0.5, 0.5, 0.5)	

	self.combatimage3 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.combatimage3:SetPosition(-175, -175, 0)   
	self.combatimage3:SetScale(0.5, 0.5, 0.5)

	self.combatimage4 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.combatimage4:SetPosition(-75, -175, 0)   
	self.combatimage4:SetScale(0.5, 0.5, 0.5)

	self.combaticonimage1 = self.proot:AddChild(Image("images/charlie_combatskill1.xml", "charlie_combatskill1.tex"))
	self.combaticonimage1:SetPosition(-175, -175, 0)  
	self.combaticonimage1:SetScale(0.5, 0.5, 0.5)

	self.combaticonimage2 = self.proot:AddChild(Image("images/charlie_combatskill2.xml", "charlie_combatskill2.tex"))
	self.combaticonimage2:SetPosition(-75, -175, 0)  
	self.combaticonimage2:SetScale(0.5, 0.5, 0.5)
	
	self.combaticonimage3 = self.proot:AddChild(Image("images/charlie_combatskill3.xml", "charlie_combatskill3.tex"))
	self.combaticonimage3:SetPosition(-175, -100, 0)
    self.combaticonimage3:SetScale(0.5, 0.5, 0.5)	
	
	self.combaticonimage4 = self.proot:AddChild(Image("images/charlie_combatskill4.xml", "charlie_combatskill4.tex"))
	self.combaticonimage4:SetPosition(-75, -100, 0)  
	self.combaticonimage4:SetScale(0.5, 0.5, 0.5)

	self.controlimage1 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.controlimage1:SetPosition(175, -100, 0)   
	self.controlimage1:SetScale(0.5, 0.5, 0.5)  
	
	self.controlimage2 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.controlimage2:SetPosition(75, -100, 0)  
    self.controlimage2:SetScale(0.5, 0.5, 0.5)	

	self.controlimage3 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.controlimage3:SetPosition(175, -175, 0)   
	self.controlimage3:SetScale(0.5, 0.5, 0.5)

	self.controlimage4 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.controlimage4:SetPosition(75, -175, 0)   
	self.controlimage4:SetScale(0.5, 0.5, 0.5)	
	
	self.controliconimage1 = self.proot:AddChild(Image("images/charlie_controlskill1.xml", "charlie_controlskill1.tex"))
	self.controliconimage1:SetPosition(75, -175, 0)  
	self.controliconimage1:SetScale(0.5, 0.5, 0.5)

	self.controliconimage2 = self.proot:AddChild(Image("images/charlie_controlskill2.xml", "charlie_controlskill2.tex"))
	self.controliconimage2:SetPosition(175, -175, 0)  
	self.controliconimage2:SetScale(0.5, 0.5, 0.5)
	
	self.controliconimage3 = self.proot:AddChild(Image("images/charlie_controlskill3.xml", "charlie_controlskill3.tex"))
	self.controliconimage3:SetPosition(75, -100, 0)
    self.controliconimage3:SetScale(0.5, 0.5, 0.5)	
	
	self.controliconimage4 = self.proot:AddChild(Image("images/charlie_controlskill4.xml", "charlie_controlskill4.tex"))
	self.controliconimage4:SetPosition(175, -100, 0)  
	self.controliconimage4:SetScale(0.5, 0.5, 0.5)	
	
	self.specialimage1 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.specialimage1:SetPosition(-175, 25, 0)   
	self.specialimage1:SetScale(0.5, 0.5, 0.5)  
	
	self.specialimage2 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.specialimage2:SetPosition(-75, 25, 0)  
    self.specialimage2:SetScale(0.5, 0.5, 0.5)	

	self.specialimage3 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.specialimage3:SetPosition(-175, 100, 0)   
	self.specialimage3:SetScale(0.5, 0.5, 0.5)

	self.specialimage4 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.specialimage4:SetPosition(-75, 100, 0)   
	self.specialimage4:SetScale(0.5, 0.5, 0.5)
	
	self.specialimage5 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.specialimage5:SetPosition(175, 25, 0)   
	self.specialimage5:SetScale(0.5, 0.5, 0.5)  
	
	self.specialimage6 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.specialimage6:SetPosition(75, 25, 0)  
    self.specialimage6:SetScale(0.5, 0.5, 0.5)	

	self.specialimage7 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.specialimage7:SetPosition(175, 100, 0)   
	self.specialimage7:SetScale(0.5, 0.5, 0.5)

	self.specialimage8 = self.proot:AddChild(Image(HUD_ATLAS, "inv_slot.tex"))
	self.specialimage8:SetPosition(75, 100, 0)   
	self.specialimage8:SetScale(0.5, 0.5, 0.5)
	
	self.shadowskillimage1 = self.proot:AddChild(Image("images/charlie_shadowrestoreskill1.xml", "charlie_shadowrestoreskill1.tex"))
	self.shadowskillimage1:SetPosition(-175, 100, 0)  
	self.shadowskillimage1:SetScale(0.5, 0.5, 0.5)

	self.shadowskillimage2 = self.proot:AddChild(Image("images/charlie_shadowrestoreskill2.xml", "charlie_shadowrestoreskill2.tex"))
	self.shadowskillimage2:SetPosition(-75, 100, 0)  
	self.shadowskillimage2:SetScale(0.5, 0.5, 0.5)
	
	self.shadowskillimage3 = self.proot:AddChild(Image("images/charlie_shadowrestoreskill3.xml", "charlie_shadowrestoreskill3.tex"))
	self.shadowskillimage3:SetPosition(-175, 25, 0)
    self.shadowskillimage3:SetScale(0.5, 0.5, 0.5)	
	
	self.shadowskillimage4 = self.proot:AddChild(Image("images/charlie_shadowrestoreskill4.xml", "charlie_shadowrestoreskill4.tex"))
	self.shadowskillimage4:SetPosition(-75, 25, 0)  
	self.shadowskillimage4:SetScale(0.5, 0.5, 0.5)		

	self.shadowskillimage5 = self.proot:AddChild(Image("images/charlie_shadowcorrosionskill1.xml", "charlie_shadowcorrosionskill1.tex"))
	self.shadowskillimage5:SetPosition(75, 100, 0)  
	self.shadowskillimage5:SetScale(0.5, 0.5, 0.5)

	self.shadowskillimage6 = self.proot:AddChild(Image("images/charlie_shadowcorrosionskill2.xml", "charlie_shadowcorrosionskill2.tex"))
	self.shadowskillimage6:SetPosition(175, 100, 0)  
	self.shadowskillimage6:SetScale(0.5, 0.5, 0.5)
	
	self.shadowskillimage7 = self.proot:AddChild(Image("images/charlie_shadowcorrosionskill3.xml", "charlie_shadowcorrosionskill3.tex"))
	self.shadowskillimage7:SetPosition(75, 25, 0)
    self.shadowskillimage7:SetScale(0.5, 0.5, 0.5)	
	
	self.shadowskillimage8 = self.proot:AddChild(Image("images/charlie_shadowcorrosionskill4.xml", "charlie_shadowcorrosionskill4.tex"))
	self.shadowskillimage8:SetPosition(175, 25, 0)  
	self.shadowskillimage8:SetScale(0.5, 0.5, 0.5)		
	
	self.lunarskillimage1 = self.proot:AddChild(Image("images/charlie_lunarprophetskill1.xml", "charlie_lunarprophetskill1.tex"))
	self.lunarskillimage1:SetPosition(-175, 100, 0)  
	self.lunarskillimage1:SetScale(0.5, 0.5, 0.5)

	self.lunarskillimage2 = self.proot:AddChild(Image("images/charlie_lunarprophetskill2.xml", "charlie_lunarprophetskill2.tex"))
	self.lunarskillimage2:SetPosition(-75, 100, 0)  
	self.lunarskillimage2:SetScale(0.5, 0.5, 0.5)
	
	self.lunarskillimage3 = self.proot:AddChild(Image("images/charlie_lunarprophetskill3.xml", "charlie_lunarprophetskill3.tex"))
	self.lunarskillimage3:SetPosition(-175, 25, 0)
    self.lunarskillimage3:SetScale(0.5, 0.5, 0.5)	
	
	self.lunarskillimage4 = self.proot:AddChild(Image("images/charlie_lunarprophetskill4.xml", "charlie_lunarprophetskill4.tex"))
	self.lunarskillimage4:SetPosition(-75, 25, 0)  
	self.lunarskillimage4:SetScale(0.5, 0.5, 0.5)		

	self.lunarskillimage5 = self.proot:AddChild(Image("images/charlie_lunarprovidenceskill1.xml", "charlie_lunarprovidenceskill1.tex"))
	self.lunarskillimage5:SetPosition(75, 100, 0)  
	self.lunarskillimage5:SetScale(0.5, 0.5, 0.5)

	self.lunarskillimage6 = self.proot:AddChild(Image("images/charlie_lunarprovidenceskill2.xml", "charlie_lunarprovidenceskill2.tex"))
	self.lunarskillimage6:SetPosition(175, 100, 0)  
	self.lunarskillimage6:SetScale(0.5, 0.5, 0.5)
	
	self.lunarskillimage7 = self.proot:AddChild(Image("images/charlie_lunarprovidenceskill3.xml", "charlie_lunarprovidenceskill3.tex"))
	self.lunarskillimage7:SetPosition(75, 25, 0)
    self.lunarskillimage7:SetScale(0.5, 0.5, 0.5)	
	
	self.lunarskillimage8 = self.proot:AddChild(Image("images/charlie_lunarprovidenceskill4.xml", "charlie_lunarprovidenceskill4.tex"))
	self.lunarskillimage8:SetPosition(175, 25, 0)  
	self.lunarskillimage8:SetScale(0.5, 0.5, 0.5)	
	
	self.flowerbutton1 = self.proot:AddChild(ImageButton())
	self.flowerbutton1:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.flowerbutton1:SetPosition(-210, 95, 0)   
	self.flowerbutton1:SetScale(0.4, 0.4, 0.4)
    self.flowerbutton1:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local FLOWERSKILL1 = STRINGS.UI.CHARLIESKILLTREE.FLOWERSKILL1..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1
			if GetPlayer().SanityGainFromPlantingFlower ~= nil and GetPlayer().SanityGainFromPlantingFlower > 10 then
			    FLOWERSKILL1 = FLOWERSKILL1..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(FLOWERSKILL1, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end
    end)	
	
	self.flowerbutton2 = self.proot:AddChild(ImageButton())
	self.flowerbutton2:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.flowerbutton2:SetPosition(-150, 95, 0)   
	self.flowerbutton2:SetScale(0.4, 0.4, 0.4)
    self.flowerbutton2:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1 then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1)
					GetPlayer().SanityGainFromPlantingFlower = 20
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end		
    end)	

	self.flowerbutton3 = self.proot:AddChild(ImageButton())
	self.flowerbutton3:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.flowerbutton3:SetPosition(-90, 95, 0)   
	self.flowerbutton3:SetScale(0.4, 0.4, 0.4)
    self.flowerbutton3:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local FLOWERSKILL2 = STRINGS.UI.CHARLIESKILLTREE.FLOWERSKILL2..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2
			if GetPlayer().SanityGainFromPlantingFlower ~= nil and GetPlayer().SanityGainFromPlantingFlower > 20 then
			    FLOWERSKILL2 = FLOWERSKILL2..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(FLOWERSKILL2, 7.5)
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end       
    end)	

	self.flowerbutton4 = self.proot:AddChild(ImageButton())
	self.flowerbutton4:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.flowerbutton4:SetPosition(-35, 95, 0)   
	self.flowerbutton4:SetScale(0.4, 0.4, 0.4)
    self.flowerbutton4:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if (CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2) 
				    and GetPlayer().SanityGainFromPlantingFlower ~= nil and GetPlayer().SanityGainFromPlantingFlower > 10
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2)
					GetPlayer().SanityGainFromPlantingFlower = 30
				elseif GetPlayer().SanityGainFromPlantingFlower ~= nil and GetPlayer().SanityGainFromPlantingFlower <= 10 then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	        
    end)	

	self.flowerbutton5 = self.proot:AddChild(ImageButton())
	self.flowerbutton5:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.flowerbutton5:SetPosition(-210, 20, 0)   
	self.flowerbutton5:SetScale(0.4, 0.4, 0.4)
    self.flowerbutton5:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local FLOWERSKILL3 = STRINGS.UI.CHARLIESKILLTREE.FLOWERSKILL3..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3
			if GetPlayer():HasTag("flowercommunicator") then
			    FLOWERSKILL3 = FLOWERSKILL3..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(FLOWERSKILL3, 7.5)
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end         
    end)	

	self.flowerbutton6 = self.proot:AddChild(ImageButton())
	self.flowerbutton6:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.flowerbutton6:SetPosition(-150, 20, 0)   
	self.flowerbutton6:SetScale(0.4, 0.4, 0.4)
    self.flowerbutton6:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3 then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3)
					GetPlayer():AddTag("flowercommunicator")
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	               
    end)	

	self.flowerbutton7 = self.proot:AddChild(ImageButton())
	self.flowerbutton7:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.flowerbutton7:SetPosition(-90, 20, 0)   
	self.flowerbutton7:SetScale(0.4, 0.4, 0.4)
    self.flowerbutton7:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local FLOWERSKILL4 = STRINGS.UI.CHARLIESKILLTREE.FLOWERSKILL4..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4
			if GetPlayer():HasTag("flowersupercommunicator") then
			    FLOWERSKILL4 = FLOWERSKILL4..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(FLOWERSKILL4, 7.5)
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end             
    end)	

	self.flowerbutton8 = self.proot:AddChild(ImageButton())
	self.flowerbutton8:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.flowerbutton8:SetPosition(-35, 20, 0)   
	self.flowerbutton8:SetScale(0.4, 0.4, 0.4)
    self.flowerbutton8:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if (CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4)
				    and GetPlayer():HasTag("flowercommunicator")
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4)
					GetPlayer():AddTag("flowersupercommunicator")
				elseif not GetPlayer():HasTag("flowercommunicator") then
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	                      
    end)		
	
	self.critterbutton1 = self.proot:AddChild(ImageButton())
	self.critterbutton1:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.critterbutton1:SetPosition(210, 95, 0)   
	self.critterbutton1:SetScale(0.4, 0.4, 0.4)
    self.critterbutton1:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if (CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2)
				    and GetPlayer():HasTag("bonusrabbitdoer") 
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2)
					GetPlayer():AddTag("superbonusrabbitdoer")
				elseif not GetPlayer():HasTag("bonusrabbitdoer") then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	       
    end)	
	
	self.critterbutton2 = self.proot:AddChild(ImageButton())
	self.critterbutton2:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.critterbutton2:SetPosition(150, 95, 0)   
	self.critterbutton2:SetScale(0.4, 0.4, 0.4)
    self.critterbutton2:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local CRITTERSKILL2 = STRINGS.UI.CHARLIESKILLTREE.CRITTERSKILL2..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2
			if GetPlayer():HasTag("superbonusrabbitdoer") then
			    CRITTERSKILL2 = CRITTERSKILL2..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(CRITTERSKILL2, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end  	
    end)	

	self.critterbutton3 = self.proot:AddChild(ImageButton())
	self.critterbutton3:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.critterbutton3:SetPosition(90, 95, 0)   
	self.critterbutton3:SetScale(0.4, 0.4, 0.4)
    self.critterbutton3:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1 then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1)
					GetPlayer():AddTag("bonusrabbitdoer") 
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	       
    end)	

	self.critterbutton4 = self.proot:AddChild(ImageButton())
	self.critterbutton4:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.critterbutton4:SetPosition(35, 95, 0)   
	self.critterbutton4:SetScale(0.4, 0.4, 0.4)
    self.critterbutton4:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local CRITTERSKILL1 = STRINGS.UI.CHARLIESKILLTREE.CRITTERSKILL1..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1
			if GetPlayer():HasTag("bonusrabbitdoer") then
			    CRITTERSKILL1 = CRITTERSKILL1..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(CRITTERSKILL1, 7.5)
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end          
    end)	

	self.critterbutton5 = self.proot:AddChild(ImageButton())
	self.critterbutton5:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.critterbutton5:SetPosition(210, 20, 0)   
	self.critterbutton5:SetScale(0.4, 0.4, 0.4)
    self.critterbutton5:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if (CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4)
				    and GetPlayer():HasTag("critterexpert") 
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4)
					GetPlayer():AddTag("supercritterexpert") 
					GetPlayer():RemoveTag("scarytoprey") 
					if GetPlayer().components.builder ~= nil then
						GetPlayer().components.builder:UnlockRecipe("magic_smallmeat")
						GetPlayer().components.builder:UnlockRecipe("magic_honey")
						GetPlayer().components.builder:UnlockRecipe("magic_rabbithole")
						GetPlayer().components.builder:UnlockRecipe("magic_honeycomb")
						GetPlayer().components.builder:UnlockRecipe("magic_tentaclespike")
						GetPlayer().components.builder:UnlockRecipe("magic_butter")
						GetPlayer().components.builder:UnlockRecipe("charlie_dragon_house")
					end
				elseif not GetPlayer():HasTag("critterexpert") then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)								
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	       
    end)	

	self.critterbutton6 = self.proot:AddChild(ImageButton())
	self.critterbutton6:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.critterbutton6:SetPosition(150, 20, 0)   
	self.critterbutton6:SetScale(0.4, 0.4, 0.4)
    self.critterbutton6:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local CRITTERSKILL4 = STRINGS.UI.CHARLIESKILLTREE.CRITTERSKILL4..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4
			if GetPlayer():HasTag("supercritterexpert") then
			    CRITTERSKILL4 = CRITTERSKILL4..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(CRITTERSKILL4, 7.5)
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end           
    end)	

	self.critterbutton7 = self.proot:AddChild(ImageButton())
	self.critterbutton7:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.critterbutton7:SetPosition(90, 20, 0)   
	self.critterbutton7:SetScale(0.4, 0.4, 0.4)
    self.critterbutton7:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3 then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3)
					GetPlayer():AddTag("critterexpert") 
					if GetPlayer().components.builder ~= nil then
						GetPlayer().components.builder:UnlockRecipe("magic_carrot")
					end
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	        
    end)	

	self.critterbutton8 = self.proot:AddChild(ImageButton())
	self.critterbutton8:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.critterbutton8:SetPosition(35, 20, 0)   
	self.critterbutton8:SetScale(0.4, 0.4, 0.4)
    self.critterbutton8:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local CRITTERSKILL3 = STRINGS.UI.CHARLIESKILLTREE.CRITTERSKILL3..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3
			if GetPlayer():HasTag("critterexpert") then
			    CRITTERSKILL3 = CRITTERSKILL3..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(CRITTERSKILL3, 7.5)
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end          
    end)
	
	self.combatbutton1 = self.proot:AddChild(ImageButton())
	self.combatbutton1:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.combatbutton1:SetPosition(-210, -130, 0)   
	self.combatbutton1:SetScale(0.4, 0.4, 0.4)
    self.combatbutton1:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local COMBATSKILL1 = STRINGS.UI.CHARLIESKILLTREE.COMBATSKILL1..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1
			if GetPlayer().swiftwarrior then
			    COMBATSKILL1 = COMBATSKILL1..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(COMBATSKILL1, 7.5)
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end         
    end)	
	
	self.combatbutton2 = self.proot:AddChild(ImageButton())
	self.combatbutton2:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.combatbutton2:SetPosition(-150, -130, 0)   
	self.combatbutton2:SetScale(0.4, 0.4, 0.4)
    self.combatbutton2:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1 then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1)
					GetPlayer().swiftwarrior = true
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	       
    end)	

	self.combatbutton3 = self.proot:AddChild(ImageButton())
	self.combatbutton3:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.combatbutton3:SetPosition(-90, -130, 0)   
	self.combatbutton3:SetScale(0.4, 0.4, 0.4)
    self.combatbutton3:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local COMBATSKILL2 = STRINGS.UI.CHARLIESKILLTREE.COMBATSKILL2..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2
			if GetPlayer().superswiftwarrior then
			    COMBATSKILL2 = COMBATSKILL2..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(COMBATSKILL2, 7.5)
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end         
    end)	

	self.combatbutton4 = self.proot:AddChild(ImageButton())
	self.combatbutton4:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.combatbutton4:SetPosition(-35, -130, 0)   
	self.combatbutton4:SetScale(0.4, 0.4, 0.4)
    self.combatbutton4:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if (CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2)
				    and GetPlayer().swiftwarrior 
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2)
					GetPlayer().superswiftwarrior = true
				elseif not GetPlayer().swiftwarrior then
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)		
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	        
    end)	

	self.combatbutton5 = self.proot:AddChild(ImageButton())
	self.combatbutton5:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.combatbutton5:SetPosition(-210, -205, 0)   
	self.combatbutton5:SetScale(0.4, 0.4, 0.4)
    self.combatbutton5:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local COMBATSKILL3 = STRINGS.UI.CHARLIESKILLTREE.COMBATSKILL3..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3
			if GetPlayer().dodgemaster then
			    COMBATSKILL3 = COMBATSKILL3..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(COMBATSKILL3, 7.5)
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end           
    end)	

	self.combatbutton6 = self.proot:AddChild(ImageButton())
	self.combatbutton6:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.combatbutton6:SetPosition(-150, -205, 0)   
	self.combatbutton6:SetScale(0.4, 0.4, 0.4)
    self.combatbutton6:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3 then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3)
					GetPlayer().dodgemaster = true
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	          
    end)	

	self.combatbutton7 = self.proot:AddChild(ImageButton())
	self.combatbutton7:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.combatbutton7:SetPosition(-90, -205, 0)   
	self.combatbutton7:SetScale(0.4, 0.4, 0.4)
    self.combatbutton7:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local COMBATSKILL4 = STRINGS.UI.CHARLIESKILLTREE.COMBATSKILL4..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4
			if GetPlayer().superdodgemaster then
			    COMBATSKILL4 = COMBATSKILL4..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(COMBATSKILL4, 7.5)
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end         
    end)	

	self.combatbutton8 = self.proot:AddChild(ImageButton())
	self.combatbutton8:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.combatbutton8:SetPosition(-35, -205, 0)   
	self.combatbutton8:SetScale(0.4, 0.4, 0.4)
    self.combatbutton8:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if (CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4)
				    and GetPlayer().dodgemaster 
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4)
					GetPlayer().superdodgemaster = true
				elseif not GetPlayer().dodgemaster then
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)		
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	                 
    end)
	
	self.controlbutton1 = self.proot:AddChild(ImageButton())
	self.controlbutton1:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.controlbutton1:SetPosition(210, -130, 0)   
	self.controlbutton1:SetScale(0.4, 0.4, 0.4)
    self.controlbutton1:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if (CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2)
				    and GetPlayer().thermologist 
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2)
					GetPlayer().superthermologist = true 
					if GetPlayer().components.builder ~= nil then 
					    GetPlayer().components.builder:UnlockRecipe("charlie_dragon_sword")
					end 
				elseif not GetPlayer().thermologist then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	               
    end)	
	
	self.controlbutton2 = self.proot:AddChild(ImageButton())
	self.controlbutton2:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.controlbutton2:SetPosition(150, -130, 0)   
	self.controlbutton2:SetScale(0.4, 0.4, 0.4)
    self.controlbutton2:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local CONTROLSKILL2 = STRINGS.UI.CHARLIESKILLTREE.CONTROLSKILL2..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED2
			if GetPlayer().superthermologist then
			    CONTROLSKILL2 = CONTROLSKILL2..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(CONTROLSKILL2, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end  	       
    end)	

	self.controlbutton3 = self.proot:AddChild(ImageButton())
	self.controlbutton3:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.controlbutton3:SetPosition(90, -130, 0)   
	self.controlbutton3:SetScale(0.4, 0.4, 0.4)
    self.controlbutton3:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1 then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1)
					GetPlayer().thermologist = true 
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	             
    end)	

	self.controlbutton4 = self.proot:AddChild(ImageButton())
	self.controlbutton4:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.controlbutton4:SetPosition(35, -130, 0)   
	self.controlbutton4:SetScale(0.4, 0.4, 0.4)
    self.controlbutton4:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local CONTROLSKILL1 = STRINGS.UI.CHARLIESKILLTREE.CONTROLSKILL1..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED1
			if GetPlayer().thermologist then
			    CONTROLSKILL1 = CONTROLSKILL1..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(CONTROLSKILL1, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end  	              
    end)	

	self.controlbutton5 = self.proot:AddChild(ImageButton())
	self.controlbutton5:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.controlbutton5:SetPosition(210, -205, 0)   
	self.controlbutton5:SetScale(0.4, 0.4, 0.4)
    self.controlbutton5:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if (CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4)
				    and GetPlayer().emberguru  
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4)
					GetPlayer().superemberguru = true 
					for _,ent in pairs(Ents) do
					    if ent.prefab == "charlie_shadowbox" and ent.UpdateContainer then
						    ent:UpdateContainer()
						end
					end
				elseif not GetPlayer().emberguru then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	             
    end)	

	self.controlbutton6 = self.proot:AddChild(ImageButton())
	self.controlbutton6:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.controlbutton6:SetPosition(150, -205, 0)   
	self.controlbutton6:SetScale(0.4, 0.4, 0.4)
    self.controlbutton6:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local CONTROLSKILL4 = STRINGS.UI.CHARLIESKILLTREE.CONTROLSKILL4..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED4
			if GetPlayer().superemberguru then
			    CONTROLSKILL4 = CONTROLSKILL4..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(CONTROLSKILL4, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end          
    end)	

	self.controlbutton7 = self.proot:AddChild(ImageButton())
	self.controlbutton7:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.controlbutton7:SetPosition(90, -205, 0)   
	self.controlbutton7:SetScale(0.4, 0.4, 0.4)
    self.controlbutton7:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3 then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3)
					GetPlayer().emberguru = true 
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	              
    end)	

	self.controlbutton8 = self.proot:AddChild(ImageButton())
	self.controlbutton8:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.controlbutton8:SetPosition(35, -205, 0)   
	self.controlbutton8:SetScale(0.4, 0.4, 0.4)
    self.controlbutton8:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local CONTROLSKILL3 = STRINGS.UI.CHARLIESKILLTREE.CONTROLSKILL3..STRINGS.UI.CHARLIESKILLTREE.SKILL_EXP_REQUIRED3
			if GetPlayer().emberguru then
			    CONTROLSKILL3 = CONTROLSKILL3..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(CONTROLSKILL3, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end                 
    end)	
	
	self.shadowrestorebutton1 = self.proot:AddChild(ImageButton())
	self.shadowrestorebutton1:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.shadowrestorebutton1:SetPosition(-210, 70, 0)   
	self.shadowrestorebutton1:SetScale(0.4, 0.4, 0.4)
    self.shadowrestorebutton1:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local SHADOWRESTORESKILL1 = STRINGS.UI.CHARLIESKILLTREE.SHADOWRESTORESKILL1..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED1..STRINGS.UI.CHARLIESKILLTREE.UNLOCK_NEED_NIGHTMAREFUEL..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED1
			if GetPlayer().ShadowHealthGain ~= nil and GetPlayer().ShadowHealthGain > 12 then
			    SHADOWRESTORESKILL1 = SHADOWRESTORESKILL1..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(SHADOWRESTORESKILL1, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end       
    end)	
	
	self.shadowrestorebutton2 = self.proot:AddChild(ImageButton())
	self.shadowrestorebutton2:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.shadowrestorebutton2:SetPosition(-150, 70, 0)   
	self.shadowrestorebutton2:SetScale(0.4, 0.4, 0.4)
    self.shadowrestorebutton2:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED1 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED1 
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED1) 
					CharlieRefreshModRemovePrefabs("nightmarefuel", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED1) 
					GetPlayer().ShadowHealthGain = 15 
					GetPlayer().ShadowHealthTime = 15 
					GetPlayer().syntheticobserver = true 
				elseif CharlieRefreshModCountPrefabs("charlie_exp") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED1 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED1 then 
					GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP.."\n"..STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 		
				elseif CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED1 then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 					
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end		        
    end)	

	self.shadowrestorebutton3 = self.proot:AddChild(ImageButton())
	self.shadowrestorebutton3:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.shadowrestorebutton3:SetPosition(-90, 70, 0)   
	self.shadowrestorebutton3:SetScale(0.4, 0.4, 0.4)
    self.shadowrestorebutton3:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local SHADOWRESTORESKILL2 = STRINGS.UI.CHARLIESKILLTREE.SHADOWRESTORESKILL2..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED2..STRINGS.UI.CHARLIESKILLTREE.UNLOCK_NEED_NIGHTMAREFUEL..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED2
			if GetPlayer().ShadowHealthGain ~= nil and GetPlayer().ShadowHealthGain > 15 then
			    SHADOWRESTORESKILL2 = SHADOWRESTORESKILL2..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(SHADOWRESTORESKILL2, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end             
    end)	

	self.shadowrestorebutton4 = self.proot:AddChild(ImageButton())
	self.shadowrestorebutton4:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.shadowrestorebutton4:SetPosition(-35, 70, 0)   
	self.shadowrestorebutton4:SetScale(0.4, 0.4, 0.4)
    self.shadowrestorebutton4:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED2 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED2 
					and GetPlayer().ShadowHealthGain ~= nil and GetPlayer().ShadowHealthGain > 12 
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED2) 
					CharlieRefreshModRemovePrefabs("nightmarefuel", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED2) 
					GetPlayer().ShadowHealthGain = 18 
					GetPlayer().ShadowHealthTime = 10  
					GetPlayer().supersyntheticobserver = true 
				elseif GetPlayer().ShadowHealthGain == nil or GetPlayer().ShadowHealthGain <= 12 then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)		
				elseif CharlieRefreshModCountPrefabs("charlie_exp") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED2 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED2 then 
					GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP.."\n"..STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 		
				elseif CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED2 then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 					
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end		           
    end)	

	self.shadowrestorebutton5 = self.proot:AddChild(ImageButton())
	self.shadowrestorebutton5:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.shadowrestorebutton5:SetPosition(-210, -5, 0)   
	self.shadowrestorebutton5:SetScale(0.4, 0.4, 0.4)
    self.shadowrestorebutton5:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local SHADOWRESTORESKILL3 = STRINGS.UI.CHARLIESKILLTREE.SHADOWRESTORESKILL3..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED3..STRINGS.UI.CHARLIESKILLTREE.UNLOCK_NEED_NIGHTMAREFUEL..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED3
			if GetPlayer().shadowmanipulation then
			    SHADOWRESTORESKILL3 = SHADOWRESTORESKILL3..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(SHADOWRESTORESKILL3, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end            
    end)	

	self.shadowrestorebutton6 = self.proot:AddChild(ImageButton())
	self.shadowrestorebutton6:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.shadowrestorebutton6:SetPosition(-150, -5, 0)   
	self.shadowrestorebutton6:SetScale(0.4, 0.4, 0.4)
    self.shadowrestorebutton6:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED3  
				    and CharlieRefreshModCountPrefabs("nightmarefuel") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED3  
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED3) 
					CharlieRefreshModRemovePrefabs("nightmarefuel", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED3) 
					GetPlayer().shadowmanipulation = true 
					if GetPlayer().components.autoshadowabsorber ~= nil then 
					    GetPlayer().components.autoshadowabsorber:CheckIfCanEnable() 
					end 
				elseif CharlieRefreshModCountPrefabs("charlie_exp") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED3 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED3 then 
					GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP.."\n"..STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 		
				elseif CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED3 then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 					
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end		            
    end)	

	self.shadowrestorebutton7 = self.proot:AddChild(ImageButton())
	self.shadowrestorebutton7:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.shadowrestorebutton7:SetPosition(-90, -5, 0)   
	self.shadowrestorebutton7:SetScale(0.4, 0.4, 0.4)
    self.shadowrestorebutton7:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local SHADOWRESTORESKILL4 = STRINGS.UI.CHARLIESKILLTREE.SHADOWRESTORESKILL4..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED4..STRINGS.UI.CHARLIESKILLTREE.UNLOCK_NEED_NIGHTMAREFUEL..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED4
			if GetPlayer().supershadowmanipulation then
			    SHADOWRESTORESKILL4 = SHADOWRESTORESKILL4..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(SHADOWRESTORESKILL4, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end             
    end)	

	self.shadowrestorebutton8 = self.proot:AddChild(ImageButton())
	self.shadowrestorebutton8:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.shadowrestorebutton8:SetPosition(-35, -5, 0)   
	self.shadowrestorebutton8:SetScale(0.4, 0.4, 0.4)
    self.shadowrestorebutton8:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED4  
				    and CharlieRefreshModCountPrefabs("nightmarefuel") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED4  
					and GetPlayer().shadowmanipulation 
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED4) 
					CharlieRefreshModRemovePrefabs("nightmarefuel", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED4) 
					GetPlayer().supershadowmanipulation = true 
                    if GetPlayer().CheckPeriodicShadowPuddleSpawn ~= nil then 
					    GetPlayer():CheckPeriodicShadowPuddleSpawn() 
					end 
				elseif not GetPlayer().shadowmanipulation then
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL) 
				elseif CharlieRefreshModCountPrefabs("charlie_exp") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED4 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED4 then 
					GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP.."\n"..STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 		
				elseif CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED4 then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 					
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end		        
    end)		
	
	self.shadowcorrosionbutton1 = self.proot:AddChild(ImageButton())
	self.shadowcorrosionbutton1:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.shadowcorrosionbutton1:SetPosition(210, 70, 0)   
	self.shadowcorrosionbutton1:SetScale(0.4, 0.4, 0.4)
    self.shadowcorrosionbutton1:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED2 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED2 
					and GetPlayer().shadowacider
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED2) 
					CharlieRefreshModRemovePrefabs("nightmarefuel", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED2) 
					GetPlayer().supershadowacider = true 
					if GetPlayer().components.builder ~= nil then
					    GetPlayer().components.builder:UnlockRecipe("shadow_acid_bottle_advanced")
					end
				elseif not GetPlayer().shadowacider then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)		
				elseif CharlieRefreshModCountPrefabs("charlie_exp") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED2 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED2 then 
					GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP.."\n"..STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 		
				elseif CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED2 then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 					
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	       
    end)	
	
	self.shadowcorrosionbutton2 = self.proot:AddChild(ImageButton())
	self.shadowcorrosionbutton2:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.shadowcorrosionbutton2:SetPosition(150, 70, 0)   
	self.shadowcorrosionbutton2:SetScale(0.4, 0.4, 0.4)
    self.shadowcorrosionbutton2:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local SHADOWCORROSIONSKILL2 = STRINGS.UI.CHARLIESKILLTREE.SHADOWCORROSIONSKILL2..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED2..STRINGS.UI.CHARLIESKILLTREE.UNLOCK_NEED_NIGHTMAREFUEL..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED2
			if GetPlayer().supershadowacider then
			    SHADOWCORROSIONSKILL2 = SHADOWCORROSIONSKILL2..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(SHADOWCORROSIONSKILL2, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end   

    end)	

	self.shadowcorrosionbutton3 = self.proot:AddChild(ImageButton())
	self.shadowcorrosionbutton3:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.shadowcorrosionbutton3:SetPosition(90, 70, 0)   
	self.shadowcorrosionbutton3:SetScale(0.4, 0.4, 0.4)
    self.shadowcorrosionbutton3:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED1 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED1 
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED1) 
					CharlieRefreshModRemovePrefabs("nightmarefuel", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED1) 
					GetPlayer().shadowacider = true 
					if GetPlayer().components.builder ~= nil then
					    GetPlayer().components.builder:UnlockRecipe("shadow_acid_bottle")
					end	
				elseif CharlieRefreshModCountPrefabs("charlie_exp") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED1 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED1 then 
					GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP.."\n"..STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 		
				elseif CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED1 then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 					
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	           
    end)	

	self.shadowcorrosionbutton4 = self.proot:AddChild(ImageButton())
	self.shadowcorrosionbutton4:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.shadowcorrosionbutton4:SetPosition(35, 70, 0)   
	self.shadowcorrosionbutton4:SetScale(0.4, 0.4, 0.4)
    self.shadowcorrosionbutton4:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local SHADOWCORROSIONSKILL1 = STRINGS.UI.CHARLIESKILLTREE.SHADOWCORROSIONSKILL1..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED1..STRINGS.UI.CHARLIESKILLTREE.UNLOCK_NEED_NIGHTMAREFUEL..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED1
			if GetPlayer().shadowacider then
			    SHADOWCORROSIONSKILL1 = SHADOWCORROSIONSKILL1..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(SHADOWCORROSIONSKILL1, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end         
    end)	

	self.shadowcorrosionbutton5 = self.proot:AddChild(ImageButton())
	self.shadowcorrosionbutton5:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.shadowcorrosionbutton5:SetPosition(210, -5, 0)   
	self.shadowcorrosionbutton5:SetScale(0.4, 0.4, 0.4)
    self.shadowcorrosionbutton5:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED4 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED4 
					and GetPlayer().acidshadowsmoker
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED4) 
					CharlieRefreshModRemovePrefabs("nightmarefuel", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED4) 
					GetPlayer().superacidshadowsmoker = true 
				elseif not GetPlayer().acidshadowsmoker then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_REQUIRE_PRE_SKILL)		
				elseif CharlieRefreshModCountPrefabs("charlie_exp") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED4 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED4 then 
					GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP.."\n"..STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 		
				elseif CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED4 then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 					
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	        
    end)	

	self.shadowcorrosionbutton6 = self.proot:AddChild(ImageButton())
	self.shadowcorrosionbutton6:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.shadowcorrosionbutton6:SetPosition(150, -5, 0)   
	self.shadowcorrosionbutton6:SetScale(0.4, 0.4, 0.4)
    self.shadowcorrosionbutton6:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local SHADOWCORROSIONSKILL4 = STRINGS.UI.CHARLIESKILLTREE.SHADOWCORROSIONSKILL4..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED4..STRINGS.UI.CHARLIESKILLTREE.UNLOCK_NEED_NIGHTMAREFUEL..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED4
			if GetPlayer().superacidshadowsmoker then
			    SHADOWCORROSIONSKILL4 = SHADOWCORROSIONSKILL4..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(SHADOWCORROSIONSKILL4, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end           
    end)	

	self.shadowcorrosionbutton7 = self.proot:AddChild(ImageButton())
	self.shadowcorrosionbutton7:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.shadowcorrosionbutton7:SetPosition(90, -5, 0)   
	self.shadowcorrosionbutton7:SetScale(0.4, 0.4, 0.4)
    self.shadowcorrosionbutton7:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    if GetPlayer().components.talker ~= nil then
			    if CharlieRefreshModCountPrefabs("charlie_exp") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED3 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") >= STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED3 
				then
			        GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.UNLOCK_SUCCESS)
					if GetPlayer().SoundEmitter ~= nil then
					    GetPlayer().SoundEmitter:PlaySound("dontstarve/HUD/get_gold")
					end
				    CharlieRefreshModRemovePrefabs("charlie_exp", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED3) 
					CharlieRefreshModRemovePrefabs("nightmarefuel", STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED3) 
					GetPlayer().acidshadowsmoker = true 
				elseif CharlieRefreshModCountPrefabs("charlie_exp") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED3 
				    and CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED3 then 
					GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP.."\n"..STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 		
				elseif CharlieRefreshModCountPrefabs("nightmarefuel") < STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED3 then 
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_NIGHTMAREFUEL) 					
				else
				    GetPlayer().components.talker:Say(STRINGS.UI.CHARLIESKILLTREE.NOT_ENOUGH_EXP)
				end
			end	
            if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end			
		end	            
    end)	

	self.shadowcorrosionbutton8 = self.proot:AddChild(ImageButton())
	self.shadowcorrosionbutton8:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.shadowcorrosionbutton8:SetPosition(35, -5, 0)   
	self.shadowcorrosionbutton8:SetScale(0.4, 0.4, 0.4)
    self.shadowcorrosionbutton8:SetOnClick(function()
        TheFrontEnd:PopScreen(self)
		if GetPlayer() then
		    local SHADOWCORROSIONSKILL3 = STRINGS.UI.CHARLIESKILLTREE.SHADOWCORROSIONSKILL3..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_EXP_REQUIRED3..STRINGS.UI.CHARLIESKILLTREE.UNLOCK_NEED_NIGHTMAREFUEL..STRINGS.UI.CHARLIESKILLTREE.SHADOW_SKILL_NIGHTMAREFUEL_REQUIRED3
			if GetPlayer().acidshadowsmoker then
			    SHADOWCORROSIONSKILL3 = SHADOWCORROSIONSKILL3..STRINGS.UI.CHARLIESKILLTREE.ALREADYUNLOCK
			end
		    if GetPlayer().components.talker ~= nil then
			    GetPlayer().components.talker:Say(SHADOWCORROSIONSKILL3, 7.5)
			end	
	        if GetPlayer().shadowbox ~= nil then
			    if GetPlayer().shadowbox.AnimState ~= nil then
                    GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
				end
				GetPlayer().shadowbox = nil
            end		
		end           
    end)

	self.lunarprophetbutton1 = self.proot:AddChild(ImageButton())
	self.lunarprophetbutton1:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.lunarprophetbutton1:SetPosition(-210, 70, 0)   
	self.lunarprophetbutton1:SetScale(0.4, 0.4, 0.4)
    self.lunarprophetbutton1:SetOnClick(function()
        
    end)	
	
	self.lunarprophetbutton2 = self.proot:AddChild(ImageButton())
	self.lunarprophetbutton2:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.lunarprophetbutton2:SetPosition(-150, 70, 0)   
	self.lunarprophetbutton2:SetScale(0.4, 0.4, 0.4)
    self.lunarprophetbutton2:SetOnClick(function()
        
    end)	

	self.lunarprophetbutton3 = self.proot:AddChild(ImageButton())
	self.lunarprophetbutton3:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.lunarprophetbutton3:SetPosition(-90, 70, 0)   
	self.lunarprophetbutton3:SetScale(0.4, 0.4, 0.4)
    self.lunarprophetbutton3:SetOnClick(function()
        
    end)	

	self.lunarprophetbutton4 = self.proot:AddChild(ImageButton())
	self.lunarprophetbutton4:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.lunarprophetbutton4:SetPosition(-35, 70, 0)   
	self.lunarprophetbutton4:SetScale(0.4, 0.4, 0.4)
    self.lunarprophetbutton4:SetOnClick(function()
        
    end)	

	self.lunarprophetbutton5 = self.proot:AddChild(ImageButton())
	self.lunarprophetbutton5:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.lunarprophetbutton5:SetPosition(-210, -5, 0)   
	self.lunarprophetbutton5:SetScale(0.4, 0.4, 0.4)
    self.lunarprophetbutton5:SetOnClick(function()
        
    end)	

	self.lunarprophetbutton6 = self.proot:AddChild(ImageButton())
	self.lunarprophetbutton6:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.lunarprophetbutton6:SetPosition(-150, -5, 0)   
	self.lunarprophetbutton6:SetScale(0.4, 0.4, 0.4)
    self.lunarprophetbutton6:SetOnClick(function()
        
    end)	

	self.lunarprophetbutton7 = self.proot:AddChild(ImageButton())
	self.lunarprophetbutton7:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.lunarprophetbutton7:SetPosition(-90, -5, 0)   
	self.lunarprophetbutton7:SetScale(0.4, 0.4, 0.4)
    self.lunarprophetbutton7:SetOnClick(function()
        
    end)	

	self.lunarprophetbutton8 = self.proot:AddChild(ImageButton())
	self.lunarprophetbutton8:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.lunarprophetbutton8:SetPosition(-35, -5, 0)   
	self.lunarprophetbutton8:SetScale(0.4, 0.4, 0.4)
    self.lunarprophetbutton8:SetOnClick(function()
        
    end)		
	
	self.lunarprovidencebutton1 = self.proot:AddChild(ImageButton())
	self.lunarprovidencebutton1:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.lunarprovidencebutton1:SetPosition(210, 70, 0)   
	self.lunarprovidencebutton1:SetScale(0.4, 0.4, 0.4)
    self.lunarprovidencebutton1:SetOnClick(function()
        
    end)	
	
	self.lunarprovidencebutton2 = self.proot:AddChild(ImageButton())
	self.lunarprovidencebutton2:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.lunarprovidencebutton2:SetPosition(150, 70, 0)   
	self.lunarprovidencebutton2:SetScale(0.4, 0.4, 0.4)
    self.lunarprovidencebutton2:SetOnClick(function()
        
    end)	

	self.lunarprovidencebutton3 = self.proot:AddChild(ImageButton())
	self.lunarprovidencebutton3:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.lunarprovidencebutton3:SetPosition(90, 70, 0)   
	self.lunarprovidencebutton3:SetScale(0.4, 0.4, 0.4)
    self.lunarprovidencebutton3:SetOnClick(function()
        
    end)	

	self.lunarprovidencebutton4 = self.proot:AddChild(ImageButton())
	self.lunarprovidencebutton4:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.lunarprovidencebutton4:SetPosition(35, 70, 0)   
	self.lunarprovidencebutton4:SetScale(0.4, 0.4, 0.4)
    self.lunarprovidencebutton4:SetOnClick(function()
        
    end)	

	self.lunarprovidencebutton5 = self.proot:AddChild(ImageButton())
	self.lunarprovidencebutton5:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.lunarprovidencebutton5:SetPosition(210, -5, 0)   
	self.lunarprovidencebutton5:SetScale(0.4, 0.4, 0.4)
    self.lunarprovidencebutton5:SetOnClick(function()
        
    end)	

	self.lunarprovidencebutton6 = self.proot:AddChild(ImageButton())
	self.lunarprovidencebutton6:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.lunarprovidencebutton6:SetPosition(150, -5, 0)   
	self.lunarprovidencebutton6:SetScale(0.4, 0.4, 0.4)
    self.lunarprovidencebutton6:SetOnClick(function()
        
    end)	

	self.lunarprovidencebutton7 = self.proot:AddChild(ImageButton())
	self.lunarprovidencebutton7:SetText(STRINGS.UI.CHARLIESKILLTREE.UNLOCK)
	self.lunarprovidencebutton7:SetPosition(90, -5, 0)   
	self.lunarprovidencebutton7:SetScale(0.4, 0.4, 0.4)
    self.lunarprovidencebutton7:SetOnClick(function()
        
    end)	

	self.lunarprovidencebutton8 = self.proot:AddChild(ImageButton())
	self.lunarprovidencebutton8:SetText(STRINGS.UI.CHARLIESKILLTREE.INFO)
	self.lunarprovidencebutton8:SetPosition(35, -5, 0)   
	self.lunarprovidencebutton8:SetScale(0.4, 0.4, 0.4)
    self.lunarprovidencebutton8:SetOnClick(function()
        
    end)	
	
	self.leftbutton = self.proot:AddChild(ImageButton("images/ui.xml", "scroll_arrow.tex", "scroll_arrow_over.tex", "scroll_arrow_disabled.tex"))
    self.leftbutton:SetPosition(-300, 0, 0)   
	self.leftbutton:SetRotation(-180)
	self.leftbutton:SetScale(0.75, 0.75, 0.75)
    self.leftbutton:SetOnClick(function()
        self.CurrentPage = self.CurrentPage - 1
        if self.CurrentPage < 1 then
            self.CurrentPage = self.MaxPage
        end
        self:UpdateText()
    end)

	self.rightbutton = self.proot:AddChild(ImageButton("images/ui.xml", "scroll_arrow.tex", "scroll_arrow_over.tex", "scroll_arrow_disabled.tex"))
    self.rightbutton:SetPosition(300, 0, 0)  
	self.rightbutton:SetRotation(0)
	self.rightbutton:SetScale(0.75, 0.75, 0.75)
    self.rightbutton:SetOnClick(function()
        self.CurrentPage = self.CurrentPage + 1
        if self.CurrentPage > self.MaxPage then
            self.CurrentPage = 1
        end
        self:UpdateText()
    end)

	self.shadow_anim = self.proot:AddChild(UIAnim())
	self.shadow_anim:GetAnimState():SetBank("skills_shadow")
	self.shadow_anim:GetAnimState():SetBuild("skills_shadow")
	self.shadow_anim:GetAnimState():PlayAnimation("idle", true)
	self.shadow_anim:SetPosition(0, 0, 0)
	self.shadow_anim:Hide()
	self.shadow_anim:SetClickable(false)
    self.shadow_anim.inst:ListenForEvent("animover", function()
        if math.random() < 0.3 then
            self.shadow_anim:GetAnimState():PlayAnimation("twitch")
            self.shadow_anim:GetAnimState():PushAnimation("idle", true)
        else
            self.shadow_anim:GetAnimState():PlayAnimation("idle", true)
        end
    end)	

	self.lunar_anim = self.proot:AddChild(UIAnim())
	self.lunar_anim:GetAnimState():SetBank("skills_lunar")
	self.lunar_anim:GetAnimState():SetBuild("skills_lunar")
	self.lunar_anim:GetAnimState():PlayAnimation("idle", true)
	self.lunar_anim:GetAnimState():SetMultColour(0.7, 0.7, 0.7, 0.7)	
	self.lunar_anim:SetPosition(0, -10, 0)
	self.lunar_anim:Hide()
	self.lunar_anim:SetClickable(false)
    self.lunar_anim.inst:ListenForEvent("animover", function()
        if math.random() < 0.05 then  
            self.lunar_anim:GetAnimState():PlayAnimation("twitch")
            self.lunar_anim:GetAnimState():PushAnimation("idle", true)
        else
            self.lunar_anim:GetAnimState():PlayAnimation("idle", true)
        end
    end)
	
	local button_w = 160
	
	local buttons = {}
	table.insert(buttons, {text = STRINGS.UI.CHARLIESKILLTREE.EXIT, cb = function() 
	    TheFrontEnd:PopScreen(self) 	        
		if GetPlayer() and GetPlayer().shadowbox ~= nil then
	        if GetPlayer().shadowbox.AnimState ~= nil then
                GetPlayer().shadowbox.AnimState:PlayAnimation("idle_off")
			end
			GetPlayer().shadowbox = nil
        end	
	end})

	self.menu = self.proot:AddChild(Menu(buttons, button_w, true))
	self.menu:SetPosition(0, -275, 0)

	TheInputProxy:SetCursorVisible(true)
	self.default_focus = self.menu
	self:UpdateText()
end)

function CharlieSkillTreeScreen:OnControl(control, down)
	if CharlieSkillTreeScreen._base.OnControl(self, control, down) then 
	    return true 
	end

	if not down then
		if control == CONTROL_PAUSE or control == CONTROL_CANCEL then	
			self.active = false
			TheFrontEnd:PopScreen() 
		elseif control == CONTROL_PAGELEFT then
            if self.leftbutton.shown then
				self.CurrentPage = self.CurrentPage - 1
				if self.CurrentPage < 1 then
					self.CurrentPage = self.MaxPage
				end
				self:UpdateText()
            end
        elseif control == CONTROL_PAGERIGHT then
            if self.rightbutton.shown then
				self.CurrentPage = self.CurrentPage + 1
				if self.CurrentPage > self.MaxPage then
					self.CurrentPage = 1
				end
				self:UpdateText()
            end
		else
			return false
		end
		return true
	end
end

function CharlieSkillTreeScreen:UpdateText()
    if self.CurrentPage == 1 then
		self.title:SetString(STRINGS.UI.CHARLIESKILLTREE.TITLE1)
        self.text1:SetString(STRINGS.UI.CHARLIESKILLTREE.FLOWER)
		self.text2:SetString(STRINGS.UI.CHARLIESKILLTREE.CRITTER)
		self.text3:SetString(STRINGS.UI.CHARLIESKILLTREE.COMBAT)
		self.text4:SetString(STRINGS.UI.CHARLIESKILLTREE.CONTROL)
		self.dividertext:SetString("___________________________")		
		self.shadowtext1:SetString("")
		self.shadowtext2:SetString("")
		self.lunartext1:SetString("")
		self.lunartext2:SetString("")
		self.shadow_anim:Hide()
		self.lunar_anim:Hide()
        self.flowerimage1:Show()  
        self.flowerimage2:Show()  
        self.flowerimage3:Show()  
        self.flowerimage4:Show()  
		self.flowericonimage1:Show()
		self.flowericonimage2:Show()
		self.flowericonimage3:Show()
		self.flowericonimage4:Show()
        self.critterimage1:Show()	
        self.critterimage2:Show()
        self.critterimage3:Show()
        self.critterimage4:Show()
		self.crittericonimage1:Show()
		self.crittericonimage2:Show()
		self.crittericonimage3:Show()
		self.crittericonimage4:Show()		
        self.combatimage1:Show()	
        self.combatimage2:Show()
        self.combatimage3:Show()
        self.combatimage4:Show()
		self.combaticonimage1:Show() 
		self.combaticonimage2:Show() 
		self.combaticonimage3:Show() 
		self.combaticonimage4:Show() 
        self.controlimage1:Show()	
        self.controlimage2:Show()
        self.controlimage3:Show()
        self.controlimage4:Show()	
		self.controliconimage1:Show() 
		self.controliconimage2:Show() 
		self.controliconimage3:Show() 
		self.controliconimage4:Show() 
        self.specialimage1:Hide()
        self.specialimage2:Hide()
        self.specialimage3:Hide()
        self.specialimage4:Hide()
        self.specialimage5:Hide()
        self.specialimage6:Hide()
        self.specialimage7:Hide()
        self.specialimage8:Hide()
        self.shadowskillimage1:Hide()		
        self.shadowskillimage2:Hide()
        self.shadowskillimage3:Hide()
        self.shadowskillimage4:Hide()
        self.shadowskillimage5:Hide()
        self.shadowskillimage6:Hide()
        self.shadowskillimage7:Hide()
        self.shadowskillimage8:Hide()
		self.lunarskillimage1:Hide() 
		self.lunarskillimage2:Hide() 
		self.lunarskillimage3:Hide() 
		self.lunarskillimage4:Hide() 
		self.lunarskillimage5:Hide() 
		self.lunarskillimage6:Hide() 
		self.lunarskillimage7:Hide() 
		self.lunarskillimage8:Hide() 
        self.shadowrestorebutton1:Hide()
        self.shadowrestorebutton2:Hide()
        self.shadowrestorebutton3:Hide()
        self.shadowrestorebutton4:Hide()
        self.shadowrestorebutton5:Hide()
        self.shadowrestorebutton6:Hide()
        self.shadowrestorebutton7:Hide()
        self.shadowrestorebutton8:Hide()
        self.shadowcorrosionbutton1:Hide()
        self.shadowcorrosionbutton2:Hide()
        self.shadowcorrosionbutton3:Hide()
        self.shadowcorrosionbutton4:Hide()
        self.shadowcorrosionbutton5:Hide()
        self.shadowcorrosionbutton6:Hide()
        self.shadowcorrosionbutton7:Hide()
        self.shadowcorrosionbutton8:Hide()
        self.lunarprophetbutton1:Hide()
        self.lunarprophetbutton2:Hide()
        self.lunarprophetbutton3:Hide()
        self.lunarprophetbutton4:Hide()
        self.lunarprophetbutton5:Hide()
        self.lunarprophetbutton6:Hide()
        self.lunarprophetbutton7:Hide()
        self.lunarprophetbutton8:Hide()
        self.lunarprovidencebutton1:Hide()
        self.lunarprovidencebutton2:Hide()
        self.lunarprovidencebutton3:Hide()
        self.lunarprovidencebutton4:Hide()
        self.lunarprovidencebutton5:Hide()
        self.lunarprovidencebutton6:Hide()
        self.lunarprovidencebutton7:Hide()
        self.lunarprovidencebutton8:Hide()			
        self.flowerbutton1:Show()	
        self.flowerbutton2:Show()
        self.flowerbutton3:Show()
        self.flowerbutton4:Show()	
        self.flowerbutton5:Show()	
        self.flowerbutton6:Show()
        self.flowerbutton7:Show()
        self.flowerbutton8:Show()		
        self.critterbutton1:Show()	
        self.critterbutton2:Show()
        self.critterbutton3:Show()
        self.critterbutton4:Show()	
        self.critterbutton5:Show()	
        self.critterbutton6:Show()
        self.critterbutton7:Show()
        self.critterbutton8:Show()
        self.combatbutton1:Show()	
        self.combatbutton2:Show()
        self.combatbutton3:Show()
        self.combatbutton4:Show()	
        self.combatbutton5:Show()	
        self.combatbutton6:Show()
        self.combatbutton7:Show()
        self.combatbutton8:Show()
        self.controlbutton1:Show()	
        self.controlbutton2:Show()
        self.controlbutton3:Show()
        self.controlbutton4:Show()	
        self.controlbutton5:Show()	
        self.controlbutton6:Show()
        self.controlbutton7:Show()
        self.controlbutton8:Show()		
    elseif self.CurrentPage == 2 then
		self.title:SetString(STRINGS.UI.CHARLIESKILLTREE.TITLE2)
		self.shadowtext1:SetString(STRINGS.UI.CHARLIESKILLTREE.SHADOW_RESTORE)
		self.shadowtext2:SetString(STRINGS.UI.CHARLIESKILLTREE.SHADOW_CORROSION) 
        self.text1:SetString("")
		self.text2:SetString("")
		self.text3:SetString("")
		self.text4:SetString("")     
		self.lunartext1:SetString("")
		self.lunartext2:SetString("")	
        self.dividertext:SetString("")			
		self.lunar_anim:Hide()
        self.shadow_anim:Show()
        self.shadow_anim:GetAnimState():PlayAnimation("pre")
        self.shadow_anim:GetAnimState():PushAnimation("idle", true)	
        self.flowerimage1:Hide()	
        self.flowerimage2:Hide()
        self.flowerimage3:Hide()
        self.flowerimage4:Hide()	
		self.flowericonimage1:Hide()
		self.flowericonimage2:Hide()
		self.flowericonimage3:Hide()
		self.flowericonimage4:Hide()		
        self.critterimage1:Hide()	
        self.critterimage2:Hide()
        self.critterimage3:Hide()
        self.critterimage4:Hide()
		self.crittericonimage1:Hide()
		self.crittericonimage2:Hide()
		self.crittericonimage3:Hide()
		self.crittericonimage4:Hide()		
        self.combatimage1:Hide()	
        self.combatimage2:Hide()
        self.combatimage3:Hide()
        self.combatimage4:Hide()
		self.combaticonimage1:Hide() 
		self.combaticonimage2:Hide() 
		self.combaticonimage3:Hide() 
		self.combaticonimage4:Hide() 
        self.controlimage1:Hide()	
        self.controlimage2:Hide()
        self.controlimage3:Hide()
        self.controlimage4:Hide()
		self.controliconimage1:Hide() 
		self.controliconimage2:Hide() 
		self.controliconimage3:Hide() 
		self.controliconimage4:Hide() 		
        self.specialimage1:Show()
        self.specialimage2:Show()
        self.specialimage3:Show()
        self.specialimage4:Show()
        self.specialimage5:Show()
        self.specialimage6:Show()
        self.specialimage7:Show()
        self.specialimage8:Show()
        self.shadowskillimage1:Show()		
        self.shadowskillimage2:Show()
        self.shadowskillimage3:Show()
        self.shadowskillimage4:Show()
        self.shadowskillimage5:Show()
        self.shadowskillimage6:Show()
        self.shadowskillimage7:Show()
        self.shadowskillimage8:Show()
		self.lunarskillimage1:Hide() 
		self.lunarskillimage2:Hide() 
		self.lunarskillimage3:Hide() 
		self.lunarskillimage4:Hide() 
		self.lunarskillimage5:Hide() 
		self.lunarskillimage6:Hide() 
		self.lunarskillimage7:Hide() 
		self.lunarskillimage8:Hide() 		
        self.shadowrestorebutton1:Show()
        self.shadowrestorebutton2:Show()
        self.shadowrestorebutton3:Show()
        self.shadowrestorebutton4:Show()
        self.shadowrestorebutton5:Show()
        self.shadowrestorebutton6:Show()
        self.shadowrestorebutton7:Show()
        self.shadowrestorebutton8:Show()
        self.shadowcorrosionbutton1:Show()
        self.shadowcorrosionbutton2:Show()
        self.shadowcorrosionbutton3:Show()
        self.shadowcorrosionbutton4:Show()
        self.shadowcorrosionbutton5:Show()
        self.shadowcorrosionbutton6:Show()
        self.shadowcorrosionbutton7:Show()
        self.shadowcorrosionbutton8:Show()
        self.lunarprophetbutton1:Hide()
        self.lunarprophetbutton2:Hide()
        self.lunarprophetbutton3:Hide()
        self.lunarprophetbutton4:Hide()
        self.lunarprophetbutton5:Hide()
        self.lunarprophetbutton6:Hide()
        self.lunarprophetbutton7:Hide()
        self.lunarprophetbutton8:Hide()
        self.lunarprovidencebutton1:Hide()
        self.lunarprovidencebutton2:Hide()
        self.lunarprovidencebutton3:Hide()
        self.lunarprovidencebutton4:Hide()
        self.lunarprovidencebutton5:Hide()
        self.lunarprovidencebutton6:Hide()
        self.lunarprovidencebutton7:Hide()
        self.lunarprovidencebutton8:Hide()		
        self.flowerbutton1:Hide()	
        self.flowerbutton2:Hide()
        self.flowerbutton3:Hide()
        self.flowerbutton4:Hide()	
        self.flowerbutton5:Hide()	
        self.flowerbutton6:Hide()
        self.flowerbutton7:Hide()
        self.flowerbutton8:Hide()	
        self.critterbutton1:Hide()	
        self.critterbutton2:Hide()
        self.critterbutton3:Hide()
        self.critterbutton4:Hide()	
        self.critterbutton5:Hide()	
        self.critterbutton6:Hide()
        self.critterbutton7:Hide()
        self.critterbutton8:Hide()
        self.combatbutton1:Hide()	
        self.combatbutton2:Hide()
        self.combatbutton3:Hide()
        self.combatbutton4:Hide()	
        self.combatbutton5:Hide()	
        self.combatbutton6:Hide()
        self.combatbutton7:Hide()
        self.combatbutton8:Hide()
        self.controlbutton1:Hide()	
        self.controlbutton2:Hide()
        self.controlbutton3:Hide()
        self.controlbutton4:Hide()	
        self.controlbutton5:Hide()	
        self.controlbutton6:Hide()
        self.controlbutton7:Hide()
        self.controlbutton8:Hide()		
    elseif self.CurrentPage == 3 then
		self.title:SetString(STRINGS.UI.CHARLIESKILLTREE.TITLE3)
		self.lunartext1:SetString(STRINGS.UI.CHARLIESKILLTREE.LUNAR_PROPHET)
		self.lunartext2:SetString(STRINGS.UI.CHARLIESKILLTREE.LUNAR_PROVIDENCE)
        self.text1:SetString("")
		self.text2:SetString("")
		self.text3:SetString("")
		self.text4:SetString("")  
		self.shadowtext1:SetString("")
		self.shadowtext2:SetString("")		
        self.dividertext:SetString("")	
		self.shadow_anim:Hide()
		self.lunar_anim:Show()
        self.flowerimage1:Hide()	
        self.flowerimage2:Hide()
        self.flowerimage3:Hide()
        self.flowerimage4:Hide()
		self.flowericonimage1:Hide()
		self.flowericonimage2:Hide()
		self.flowericonimage3:Hide()
		self.flowericonimage4:Hide()		
        self.critterimage1:Hide()	
        self.critterimage2:Hide()
        self.critterimage3:Hide()
        self.critterimage4:Hide()
		self.crittericonimage1:Hide()
		self.crittericonimage2:Hide()
		self.crittericonimage3:Hide()
		self.crittericonimage4:Hide()		
        self.combatimage1:Hide()	
        self.combatimage2:Hide()
        self.combatimage3:Hide()
        self.combatimage4:Hide()
		self.combaticonimage1:Hide() 
		self.combaticonimage2:Hide() 
		self.combaticonimage3:Hide() 
		self.combaticonimage4:Hide() 		
        self.controlimage1:Hide()	
        self.controlimage2:Hide()
        self.controlimage3:Hide()
        self.controlimage4:Hide()
		self.controliconimage1:Hide() 
		self.controliconimage2:Hide() 
		self.controliconimage3:Hide() 
		self.controliconimage4:Hide() 				
        self.specialimage1:Show()
        self.specialimage2:Show()
        self.specialimage3:Show()
        self.specialimage4:Show()
        self.specialimage5:Show()
        self.specialimage6:Show()
        self.specialimage7:Show()
        self.specialimage8:Show()
        self.shadowskillimage1:Hide()		
        self.shadowskillimage2:Hide()
        self.shadowskillimage3:Hide()
        self.shadowskillimage4:Hide()
        self.shadowskillimage5:Hide()
        self.shadowskillimage6:Hide()
        self.shadowskillimage7:Hide()
        self.shadowskillimage8:Hide()
		self.lunarskillimage1:Show() 
		self.lunarskillimage2:Show() 
		self.lunarskillimage3:Show() 
		self.lunarskillimage4:Show() 
		self.lunarskillimage5:Show() 
		self.lunarskillimage6:Show() 
		self.lunarskillimage7:Show() 
		self.lunarskillimage8:Show() 				
        self.shadowrestorebutton1:Hide()
        self.shadowrestorebutton2:Hide()
        self.shadowrestorebutton3:Hide()
        self.shadowrestorebutton4:Hide()
        self.shadowrestorebutton5:Hide()
        self.shadowrestorebutton6:Hide()
        self.shadowrestorebutton7:Hide()
        self.shadowrestorebutton8:Hide()
        self.shadowcorrosionbutton1:Hide()
        self.shadowcorrosionbutton2:Hide()
        self.shadowcorrosionbutton3:Hide()
        self.shadowcorrosionbutton4:Hide()
        self.shadowcorrosionbutton5:Hide()
        self.shadowcorrosionbutton6:Hide()
        self.shadowcorrosionbutton7:Hide()
        self.shadowcorrosionbutton8:Hide()
        self.lunarprophetbutton1:Show()
        self.lunarprophetbutton2:Show()
        self.lunarprophetbutton3:Show()
        self.lunarprophetbutton4:Show()
        self.lunarprophetbutton5:Show()
        self.lunarprophetbutton6:Show()
        self.lunarprophetbutton7:Show()
        self.lunarprophetbutton8:Show()
        self.lunarprovidencebutton1:Show()
        self.lunarprovidencebutton2:Show()
        self.lunarprovidencebutton3:Show()
        self.lunarprovidencebutton4:Show()
        self.lunarprovidencebutton5:Show()
        self.lunarprovidencebutton6:Show()
        self.lunarprovidencebutton7:Show()
        self.lunarprovidencebutton8:Show()		
        self.flowerbutton1:Hide()	
        self.flowerbutton2:Hide()
        self.flowerbutton3:Hide()
        self.flowerbutton4:Hide()	
        self.flowerbutton5:Hide()	
        self.flowerbutton6:Hide()
        self.flowerbutton7:Hide()
        self.flowerbutton8:Hide()	
        self.critterbutton1:Hide()	
        self.critterbutton2:Hide()
        self.critterbutton3:Hide()
        self.critterbutton4:Hide()	
        self.critterbutton5:Hide()	
        self.critterbutton6:Hide()
        self.critterbutton7:Hide()
        self.critterbutton8:Hide()
        self.combatbutton1:Hide()	
        self.combatbutton2:Hide()
        self.combatbutton3:Hide()
        self.combatbutton4:Hide()	
        self.combatbutton5:Hide()	
        self.combatbutton6:Hide()
        self.combatbutton7:Hide()
        self.combatbutton8:Hide()
        self.controlbutton1:Hide()	
        self.controlbutton2:Hide()
        self.controlbutton3:Hide()
        self.controlbutton4:Hide()	
        self.controlbutton5:Hide()	
        self.controlbutton6:Hide()
        self.controlbutton7:Hide()
        self.controlbutton8:Hide()		
        self.lunar_anim:GetAnimState():PlayAnimation("pre")
        self.lunar_anim:GetAnimState():PushAnimation("idle", true)	
    end
end

function CharlieSkillTreeScreen:OnUpdate(dt)
    if GetPlayer() then
	    if GetPlayer().SanityGainFromPlantingFlower ~= nil then
		    if GetPlayer().SanityGainFromPlantingFlower > 10 and GetPlayer().SanityGainFromPlantingFlower < 30 then
		        self.flowerbutton2:Disable()
		    elseif GetPlayer().SanityGainFromPlantingFlower >= 30 then
			    self.flowerbutton2:Disable()
				self.flowerbutton4:Disable()
		    end
	    end
		if GetPlayer():HasTag("flowercommunicator") then
		    self.flowerbutton6:Disable()
		end
		if GetPlayer():HasTag("flowersupercommunicator") then
			self.flowerbutton8:Disable()
		end 
		if GetPlayer():HasTag("bonusrabbitdoer") then
		    self.critterbutton3:Disable()
		end
		if GetPlayer():HasTag("superbonusrabbitdoer") then
			self.critterbutton1:Disable()
		end	
		if GetPlayer():HasTag("critterexpert") then
			self.critterbutton7:Disable()
		end	
		if GetPlayer():HasTag("supercritterexpert") then
			self.critterbutton5:Disable()
		end			
		if GetPlayer().swiftwarrior then
		    self.combatbutton2:Disable()
		end
		if GetPlayer().superswiftwarrior then
			self.combatbutton4:Disable()
		end 
		if GetPlayer().dodgemaster then
		    self.combatbutton6:Disable()
		end
		if GetPlayer().superdodgemaster then
			self.combatbutton8:Disable()
		end 		
		if GetPlayer().thermologist then
		    self.controlbutton3:Disable()
		end
		if GetPlayer().superthermologist then
			self.controlbutton1:Disable()
		end 
		if GetPlayer().emberguru then
		    self.controlbutton7:Disable()
		end
		if GetPlayer().superemberguru then
			self.controlbutton5:Disable()
		end 			
	    if GetPlayer().ShadowHealthGain ~= nil then
		    if GetPlayer().ShadowHealthGain > 12 and GetPlayer().ShadowHealthGain < 18 then
		        self.shadowrestorebutton2:Disable()
		    elseif GetPlayer().ShadowHealthGain >= 18 then
			    self.shadowrestorebutton2:Disable()
				self.shadowrestorebutton4:Disable()
		    end
	    end		
		if GetPlayer().shadowmanipulation then
		    self.shadowrestorebutton6:Disable()
		end
		if GetPlayer().ssuperhadowmanipulation then
			self.shadowrestorebutton8:Disable()
		end 			
		if GetPlayer().shadowacider then
		    self.shadowcorrosionbutton3:Disable()
		end
		if GetPlayer().supershadowacider then
		    self.shadowcorrosionbutton1:Disable()
		end
		if GetPlayer().acidshadowsmoker then
		    self.shadowcorrosionbutton7:Disable()
		end
		if GetPlayer().superacidshadowsmoker then
		    self.shadowcorrosionbutton5:Disable()
		end		
	end
end

function CharlieSkillTreeScreen:OnBecomeActive()
	CharlieSkillTreeScreen._base.OnBecomeActive(self)
	TheFrontEnd:HideTopFade()
end

return CharlieSkillTreeScreen
