require("stategraphs/commonstates")

local events =
{
    CommonHandlers.OnAttack(),
    CommonHandlers.OnDeath(),
}

local function return_to_idle(inst)
    inst.sg:GoToState("idle")
end

local states =
{
    State{
        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("idle")
        end,

        timeline=
        {

        },

        events =
        {
            EventHandler("animover", function(inst)
			    inst.sg:GoToState(math.random() < 0.5 and "taunt" or "idle")
			end),
        },
    },

    State{
        name = "death",
        tags = {"busy"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("death")
			inst.SoundEmitter:PlaySound("dontstarve/common/butterfly_trap")	
        end,

        timeline =
        {
            TimeEvent(17*FRAMES, function(inst)
                local pos = inst:GetPosition()
                inst.components.lootdropper:DropLoot(pos)
            end),
        },

        events =
        {
            EventHandler("animover", function(inst)
                inst.SoundEmitter:PlaySound("dontstarve/common/butterfly_trap")
            end),
        }
    },

    State{
        name = "attack",
        tags = {"attack", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("attack")
            inst.components.combat:StartAttack()
			inst.SoundEmitter:PlaySound("dontstarve/common/butterfly_trap")
			inst.plant_food_attack = nil
        end,

        timeline =
        {
            TimeEvent(10*FRAMES, function(inst)
                inst.components.combat:DoAttack()
                inst.SoundEmitter:PlaySound("dontstarve/common/butterfly_trap")				
            end),
        },

        events =
        {
            EventHandler("animover", return_to_idle),
        },
    },
	
    State{
        name = "plantfood_attack",
        tags = {"attack", "plantfoodattack", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("attack")
            inst.components.combat:StartAttack()
			inst.components.health:SetPercent(1)
			inst.components.health:SetInvincible(true)
            inst.plant_food_attack = true		
            inst.SoundEmitter:PlaySound("dontstarve/common/butterfly_trap")			
        end,

        timeline =
        {
            TimeEvent(10*FRAMES, function(inst)
			    local x, y, z = inst.Transform:GetWorldPosition()
			    local entities = TheSim:FindEntities(x, y, z, 10, {"hascombatcomponent"}, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "flying", "ghost", "shadow", "buttercup", "notarget", "plant_mob", "player"})		
			    for i, v in pairs(entities) do 
				    if v ~= nil and v:IsValid() and v.entity:IsVisible() and inst.components.combat:CanTarget(v) then
                        inst.components.combat:DoAttack(v)
					end
				end								
				inst.SoundEmitter:PlaySound("dontstarve/common/butterfly_trap")
            end),
        },

        events =
        {
            EventHandler("animover", function(inst)
			    inst:DoTaskInTime(0.25, function() inst.plant_food_attack = nil end) --Delay this as the butter projectile needs some time to hit ground
			    inst.components.health:SetInvincible(false)
				inst.sg:GoToState("idle")
			end),
        },
    },	

    State{
        name = "hit",
        tags = {"busy"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("hit")
        end,

        events =
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end),
        },
    },

    State{
        name = "taunt",
        tags = {"canrotate"}, --"busy"

        onenter = function(inst)
            inst.AnimState:PlayAnimation("taunt")
        end,

        timeline =
        {
		
        },

        events =
        {
            EventHandler("animover", return_to_idle),
        },
    },

}

return StateGraph("buttercup", states, events, "idle")
