require("stategraphs/commonstates")

local events =
{
    CommonHandlers.OnAttack(),
    CommonHandlers.OnDeath(),
}

local function return_to_idle(inst)
    inst.sg:GoToState("idle")
end

local states =
{
    State{
        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("idle")
        end,

        timeline=
        {

        },

        events =
        {
            EventHandler("animover", function(inst)
			    inst.sg:GoToState(math.random() < 0.5 and "taunt" or "idle")
			end),
        },
    },

    State{
        name = "death",
        tags = {"busy"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("death")
		    inst.SoundEmitter:PlaySound("dontstarve/bee/bee_death")	
        end,

        timeline =
        {
            TimeEvent(17*FRAMES, function(inst)
                local pos = inst:GetPosition()
                inst.components.lootdropper:DropLoot(pos)
            end),
        },

        events =
        {
            EventHandler("animover", function(inst)
			    inst.SoundEmitter:PlaySound("dontstarve/bee/bee_death")	
            end),
        }
    },

    State{
        name = "attack",
        tags = {"attack", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("attack")
            inst.components.combat:StartAttack()
		    inst.SoundEmitter:PlaySound("dontstarve/bee/bee_attack")	
			inst.plant_food_attack = nil
        end,

        timeline =
        {
            TimeEvent(10*FRAMES, function(inst)
			    inst.SoundEmitter:PlaySound("dontstarve/bee/bee_attack")
				local BeeCount = inst.BeeCount or 1
				for i=1, BeeCount do
                    inst:DoAttack()	
                end					
            end),		
        },

        events =
        {
            EventHandler("animover", return_to_idle),
        },
    },
	
    State{
        name = "plantfood_attack",
        tags = {"attack", "plantfoodattack", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("attack")
            inst.components.combat:StartAttack()
			inst.components.health:SetPercent(1)
			inst.components.health:SetInvincible(true)
            inst.plant_food_attack = true		
		    inst.SoundEmitter:PlaySound("dontstarve/bee/killerbee_attack")	
        end,

        timeline =
        {
            TimeEvent(10*FRAMES, function(inst)
			    inst.SoundEmitter:PlaySound("dontstarve/bee/killerbee_attack")
				local PlantfoodBeeCount = inst.PlantfoodBeeCount or 2
                for i=1, PlantfoodBeeCount do
				    local PlantFoodTarget = FindEntity(inst, 15, function(guy) return inst.components.combat:CanTarget(guy) and (guy.components.follower == nil or (guy.components.follower.leader == nil or guy.components.follower.leader ~= GetPlayer())) end, nil, {"NOCLICK", "FX", "INLIMBO", "player", "shadow", "notarget", "maybee", "maybeebee", "companion"}, {"monster", "hostile", "killer", "animal", "smallcreature", "character", "seacreature", "prey"})
	                inst:DoAttack(inst.components.combat.target or PlantFoodTarget)
				end
            end),		
        },

        events =
        {
            EventHandler("animover", function(inst)
			    inst.plant_food_attack = nil
			    inst.components.health:SetInvincible(false)
				inst.sg:GoToState("idle")
			end),
        },
    },	

    State{
        name = "hit",
        tags = {"busy"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("hit")
        end,

        events =
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end),
        },
    },

    State{
        name = "taunt",
        tags = {"canrotate"}, --"busy"

        onenter = function(inst)
            inst.AnimState:PlayAnimation("taunt")
        end,

        timeline =
        {
		
        },

        events =
        {
            EventHandler("animover", return_to_idle),
        },
    },

}

return StateGraph("maybee", states, events, "idle")
