require("stategraphs/commonstates")

local events =
{
    EventHandler("doattack", function(inst) inst.sg:GoToState("attack") end), 
    EventHandler("locomote", function(inst) 
        if not inst.sg:HasStateTag("busy") then
			local is_moving = inst.sg:HasStateTag("moving")
			local wants_to_move = inst.components.locomotor:WantsToMoveForward()
			if not inst.sg:HasStateTag("attack") and is_moving ~= wants_to_move then
				if wants_to_move then
					inst.sg:GoToState("premoving")
				else
					inst.sg:GoToState("idle")
				end
			end
        end
    end), 
}

local states =
{
    State{
        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("idle", true)
        end,
    },

    State{
        name = "premoving",
        tags = {"moving", "canrotate"},
        
        onenter = function(inst)
			inst.components.locomotor:WalkForward()
            inst.AnimState:PlayAnimation("walk_pre")
        end,
        
        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("moving") end),
        },
    },
    
    State{
        name = "moving",
        tags = {"moving", "canrotate"},
        
        onenter = function(inst)
            inst.components.locomotor:WalkForward()
            inst.AnimState:PushAnimation("walk_loop", true)
            inst.sg:SetTimeout(2.5+math.random())
        end,
        
        ontimeout = function(inst)
            inst.sg:GoToState("moving")
        end,
    }, 

    State{
        name = "attack",
        tags = {"attack"},
        
        onenter = function(inst, cb)
            inst.Physics:Stop()
            inst.components.combat:StartAttack()
            inst.AnimState:PlayAnimation("atk")
        end,
        
        timeline=
        {
            TimeEvent(10*FRAMES, function(inst) inst.SoundEmitter:PlaySound(inst.prefab == "maybee_plantfood_projectile" and "dontstarve/bee/killerbee_attack" or "dontstarve/bee/bee_attack") end),
            TimeEvent(15*FRAMES, function(inst) 
			    inst.components.combat:DoAttack() 
			    if math.random() < (inst.prefab == "maybee_plantfood_projectile" and 1 or (inst.stun_chance or 0.25))
				    and inst.components.combat
					and inst.components.combat.target
					and inst.components.combat.target.components.freezable ~= nil 
				then
				    inst.components.combat.target.components.freezable:Freeze(inst.stun_time or 1)
				end
			end),
        },
        
        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },
	
}

return StateGraph("maybeebee", states, events, "idle")
