require("stategraphs/commonstates")

local events =
{
    CommonHandlers.OnAttack(),
    CommonHandlers.OnDeath(),
}

local function return_to_idle(inst)
    inst.sg:GoToState("idle")
end

local states =
{
    State{
        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("idle")
        end,

        timeline=
        {

        },

        events =
        {
            EventHandler("animover", function(inst)
			    inst.sg:GoToState(math.random() < 0.5 and "taunt" or "idle")
			end),
        },
    },

    State{
        name = "death",
        tags = {"busy"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("death")
		    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
			    inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/item_sink")	
			end
        end,

        timeline =
        {
            TimeEvent(17*FRAMES, function(inst)
                local pos = inst:GetPosition()
                inst.components.lootdropper:DropLoot(pos)
            end),
        },

        events =
        {
            EventHandler("animover", function(inst)
       		    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
			        inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/item_sink")	
		    	end
            end),
        }
    },

    State{
        name = "attack",
        tags = {"attack", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("attack")
            inst.components.combat:StartAttack()
		    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
			    inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/item_float")	
			end
            local ranged_weapon = inst.components.inventory ~= nil and inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) or nil
	    	if ranged_weapon and ranged_weapon.components.weapon ~= nil then
			    ranged_weapon.components.weapon:SetProjectile("seaflora_projectile")
				ranged_weapon.components.weapon:SetDamage(inst.NormalDamage or 50)
			end
			inst.plant_food_attack = nil
        end,

        timeline =
        {
            TimeEvent(10*FRAMES, function(inst)
                local ranged_weapon = inst.components.inventory ~= nil and inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) or nil
				if ranged_weapon then
				    if ranged_weapon.components.weapon ~= nil then
					    ranged_weapon.components.weapon:SetDamage((inst.IsOnWater and (inst.NormalWaterDamage or 65)) or (inst.NormalDamage or 50))
					end
				end
                inst.components.combat:DoAttack()
		        if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
			        inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/item_float")	
			    end				
            end),
            TimeEvent(14*FRAMES, function(inst)
				if inst.IsOnWater then
                    inst.components.combat:DoAttack()
				end
            end),		
	        TimeEvent(18*FRAMES, function(inst)
				if inst.IsOnWater then
                    inst.components.combat:DoAttack()
				end
            end),		
        },

        events =
        {
            EventHandler("animover", return_to_idle),
        },
    },
	
    State{
        name = "plantfood_attack",
        tags = {"attack", "plantfoodattack", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("attack")
            inst.components.combat:StartAttack()
			inst.components.health:SetPercent(1)
			inst.components.health:SetInvincible(true)
            inst.plant_food_attack = true		
		    if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
			    inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/item_float")	
			end			
            local ranged_weapon = inst.components.inventory ~= nil and inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) or nil
			if ranged_weapon and ranged_weapon.components.weapon ~= nil then
			    ranged_weapon.components.weapon:SetProjectile("seaflora_plantfood_projectile")
				ranged_weapon.components.weapon:SetDamage(inst.PlantFoodDamage or 75)
			end
        end,

        timeline =
        {
            TimeEvent(10*FRAMES, function(inst)
			    local ranged_weapon = inst.components.inventory ~= nil and inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) or nil
				if ranged_weapon then
				    if ranged_weapon.components.weapon ~= nil then
					    ranged_weapon.components.weapon:SetDamage((inst.IsOnWater and (inst.PlantFoodWaterDamage or 90)) or (inst.PlantFoodDamage or 75))
					end
				end
				
			    local x, y, z = inst.Transform:GetWorldPosition()
			    local entities = TheSim:FindEntities(x, y, z, 20, {"hascombatcomponent"}, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "shadow", "notarget", "seaflora"})		
			    for i, v in pairs(entities) do 
				    if v ~= nil and v:IsValid() and v.entity:IsVisible() and inst.components.combat:CanTarget(v) then
					    if v:HasTag("player") then
					        if inst.components.combat ~= nil 
							    and inst.components.combat.target ~= nil
							    and inst.components.combat.target == v 	
						    then
                                inst.components.combat:DoAttack(v)
						    end						
						else
                            inst.components.combat:DoAttack(v)
						end
					end
				end								
		        if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then
			        inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/item_float")	
			    end		
            end),
            TimeEvent(14*FRAMES, function(inst)
				if inst.IsOnWater then
			        local x, y, z = inst.Transform:GetWorldPosition()
			        local entities = TheSim:FindEntities(x, y, z, 20, {"hascombatcomponent"}, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "shadow", "notarget", "seaflora"})		
			        for i, v in pairs(entities) do 
				        if v ~= nil and v:IsValid() and v.entity:IsVisible() and inst.components.combat:CanTarget(v) then
					        if v:HasTag("player") then
					            if inst.components.combat ~= nil 
							        and inst.components.combat.target ~= nil
							        and inst.components.combat.target == v 	
						        then
                                    inst.components.combat:DoAttack(v)
						        end						
						    else
                                inst.components.combat:DoAttack(v)
						    end
					    end
			    	end
				end
            end),		
	        TimeEvent(18*FRAMES, function(inst)
				if inst.IsOnWater then
			        local x, y, z = inst.Transform:GetWorldPosition()
			        local entities = TheSim:FindEntities(x, y, z, 20, {"hascombatcomponent"}, {"NOCLICK", "FX", "INLIMBO", "INTERIOR_LIMBO", "shadow", "notarget", "seaflora"})		
			        for i, v in pairs(entities) do 
				        if v ~= nil and v:IsValid() and v.entity:IsVisible() and inst.components.combat:CanTarget(v) then
					        if v:HasTag("player") then
					            if inst.components.combat ~= nil 
							        and inst.components.combat.target ~= nil
							        and inst.components.combat.target == v 	
						        then
                                    inst.components.combat:DoAttack(v)
						        end						
						    else
                                inst.components.combat:DoAttack(v)
						    end
					    end
			    	end
				end
            end),			
        },

        events =
        {
            EventHandler("animover", function(inst)
			    inst.plant_food_attack = nil
			    inst.components.health:SetInvincible(false)
				inst.sg:GoToState("idle")
			end),
        },
    },	

    State{
        name = "hit",
        tags = {"busy"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("hit")
        end,

        events =
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end),
        },
    },

    State{
        name = "taunt",
        tags = {"canrotate"}, --"busy"

        onenter = function(inst)
            inst.AnimState:PlayAnimation("taunt")
        end,

        timeline =
        {
		
        },

        events =
        {
            EventHandler("animover", return_to_idle),
        },
    },

}

return StateGraph("seaflora", states, events, "idle")
