local Screen = require "widgets/screen"
local Button = require "widgets/button"
local AnimButton = require "widgets/animbutton"
local ImageButton = require "widgets/imagebutton"
local Image = require "widgets/image"
local Widget = require "widgets/widget"
require "util"

local TOOHARDTOCHANGE_ENG = {"It's not the right time.", "It's too hard to change clothes here.", "Find somewhere else."}
local TOOHARDTOCHANGE_CN = {"现在不是时候。", "在这太难换装了。", "找找别的地方。"}
local SHADOWCHANGE_ENG = {"I don't like changing too much.", "The power of Shadow won't let me do this!", "You will love this one..."}
local SHADOWCHANGE_CN = {"我还不是太喜欢换呢。", "暗影之力不让我这么做！", "你会喜欢这一套的……"}
local MOVECHANGE_ENG = {"It's too difficult to do that while walking.", "I need to stop first!", "This can not achieve the desired effect."}
local MOVECHANGE_CN = {"走路的时候太难做到了。", "我要停下来先！", "这样无法达到预期效果。"}

local function IsOnEdge(inst)
    local x, y, z = inst.Transform:GetWorldPosition()
    local radius = 1

    if GetWorld().Map:GetTileAtPoint(x, y, z) ~= GROUND.IMPASSABLE then
        return false
    end
	
    for i = -radius, radius, 1 do
        if GetWorld().Map:GetTileAtPoint(x - radius, y, z + i) ~= GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(x + radius, y, z + i) ~= GROUND.IMPASSABLE then
            return true
        end
    end
	
    for i = -(radius - 1), radius - 1, 1 do
        if GetWorld().Map:GetTileAtPoint(x + i, y, z - radius) ~= GROUND.IMPASSABLE or GetWorld().Map:GetTileAtPoint(x + i, y, z + radius) ~= GROUND.IMPASSABLE then
            return true
        end
    end
	
    return false

end

local function ChangeSkin(inst, anim)
	if inst.AnimState ~= nil and (
		inst.AnimState:IsCurrentAnimation("jumpboat") or
        inst.AnimState:IsCurrentAnimation("land") or
		inst.AnimState:IsCurrentAnimation("landboat") 
      	) then
	    return  --So the player will not drown when jumping form a boat to another
	end
	
    if (not inst:IsOnValidGround() and not IsOnEdge(inst)) 
		or (inst.components.rider ~= nil and inst.components.rider:IsRiding()) then
		if inst.components.talker ~= nil then
            if TUNING.LANGUAGE == true then
		        inst.components.talker:Say(TOOHARDTOCHANGE_ENG[math.random(#TOOHARDTOCHANGE_ENG)])
            elseif TUNING.LANGUAGE == false then
		        inst.components.talker:Say(TOOHARDTOCHANGE_CN[math.random(#TOOHARDTOCHANGE_CN)])
	        end
		end
    elseif inst.sg ~= nil and inst.sg:HasStateTag("moving") then
	    if inst.components.talker ~= nil then
            if TUNING.LANGUAGE == true then
		       inst.components.talker:Say(MOVECHANGE_ENG[math.random(#MOVECHANGE_ENG)])
            elseif TUNING.LANGUAGE == false then
	           inst.components.talker:Say(MOVECHANGE_CN[math.random(#MOVECHANGE_CN)])
		    end
		end
    elseif (inst.components.transformable ~= nil and inst.components.transformable.shadowform) or (inst.shadowform ~= nil) then
	    if inst.components.talker ~= nil then
            if TUNING.LANGUAGE == true then
                inst.components.talker:Say(SHADOWCHANGE_ENG[math.random(#SHADOWCHANGE_ENG)])
            elseif TUNING.LANGUAGE == false then
		        inst.components.talker:Say(SHADOWCHANGE_CN[math.random(#SHADOWCHANGE_CN)])
		    end
		end
	else
	    if inst.sg ~= nil then
		    inst.sg:GoToState("change_skin", anim)
		end
	end
end

local skin_charlie = 
{
	"charlie",
    "charlie_rose",
	"charlie_goh",   --Guest of Honor
	"charlie_queen",
	"charlie_survivor",
	"charlie_gladiator",
	"charlie_hightide",
}

ChangeSkinScreen = Class(Widget, function(self)
	Widget._ctor(self, "ChangeSkinScreen")
	self.IsUIShow = false
    SetPause(true)
    self.root = self:AddChild(Widget("ROOT"))
    self.root:SetVAnchor(ANCHOR_MIDDLE)
    self.root:SetHAnchor(ANCHOR_MIDDLE)
    self.root:SetPosition(0, 0, 0)
    self.root:SetScaleMode(SCALEMODE_PROPORTIONAL)
	self.skins = {}
	if GetPlayer().prefab == "charlie" then 
	    for i,v in ipairs(skin_charlie) do
		    local a = (i - 1)*360/8 + 360/8
		    local num = math.ceil((i-8)/8)
		    local x, y, z = (200+num*100)*math.cos(2*math.pi/360*a), (200+num*100)*math.sin(2*math.pi/360*a), 0
		    local INCO = "images/"..v..".xml"
		    self.skins[v] = self.root:AddChild(ImageButton(INCO , v .. ".tex", v .. ".tex", v .. ".tex"))
		    self.skins[v]:SetPosition(x, y, z)
            if v == "charlie_gladiator" or v == "charlie_hightide" then
			    self.skins[v]:SetScale(0.25, 0.25, 0.25)
		    else
			    self.skins[v]:SetScale(0.5, 0.5, 0.5)
			end
		    self.skins[v]:SetOnClick(function() 
			    ChangeSkin(GetPlayer(), v) 	
				self:Close() 
			end)	
	    end
	end
end)

function ChangeSkinScreen:Close()
	SetPause(false, "console")
	self:Hide()
	self.IsUIShow = false
end

function ChangeSkinScreen:OnControl(control, down)
	if ChangeSkinScreen._base.OnControl(self,control, down) then
		return true
	end
	if not down then
		if control == CONTROL_PAUSE or control == CONTROL_CANCEL then
			self:Close()
		    return true
		end
	end
	return false
end

return ChangeSkinScreen